/*
!C*****************************************************************************
!File: lbin_lst.c

!Description: Grid Library Functions - library routines to 
 perform mapping to and from the Integerized Sinusoidal and the 
 Sinusoidal grids.  These routines perform the
 mapping from geographic coordinates (longitude/latitude)
 to the grid coordinates (row, column).  In addition
 the library supports nested grids and subsetting (tiling) of the grids.

!Input Parameters:

!Output Parameters:

!Revision History: (for C Version)

 Revision V2.1.1  2002/10/12
 Zhengming Wan
 - Use lbin_lst.h (V2.1.1) modified by Robert Wolfe and Zhengming Wan
 - Use set->row[row].ref_lon & set->row[irow].dlon_grid in Lbin_FwdW_sin()
   and Lbin_Inv_sin()
 - Do not check set->proj in Lbin_FwdW and Lbin_Inv at the pixel level

 Revision V2.1  2002/09/12
 Robert Wolfe
 - Added sinusoidal grid (changed names of common functions).
 - Reworked error handling.
 - Extracted from mod11_lst2A1.c into seperate source module.
 - Deleted unused functions and code.

 Revision V1.1  1996/6/6
 Zhengming Wan
 - modified for LST processing at 1km resolution Integerized Sinusoidal grids.

 Revision 02.33  1996/6/6
 Tung Lau and Robert Wolfe
 - Put in goode homolosine (_gh) functions.
 - Moved NROW_1KM from pntr.c to Lbin_Init
 - Change Lbin_Init parameter list.  Now only one call to it will initialize
   the grid.  This avoids repeated calls to Lbin_SelTile.
 - Modified the HDF read and write routines to read and write the new 
   metadata.

 Revision 02.32  1995/11/24
 Robert Wolfe and Paul Fisher
 1. Modified to fix memory leak in 'Lbin_HDFreadAttr'.
 2. Fixed prologs and SMF error handling.

 Revision 02.31  1995/11/08
 Robert Wolfe and Paul Fisher
 1. Modified to add SMF error handling.

 Revision 02.30  1995/08/04
 Robert E. Wolfe/MODLAND Support
 1. Added functions to read and write tile information to HDF file.
 2. Added start pixel and number of pixels calculations and functions.

 Revision 02.20  1995/07/22
 Robert E. Wolfe/MODLAND Support
 1. Complete rewrite of C version.  Changed routine prefixes from 'bin' to
    'Lbin'.
 2. Changed calling sequence so fractional rows and columns are explicitly
    referenced.
 3. Changed the row direction so the Northern most row is zero and the
    row number increases as it moves Southward.
 4. Incorporated the tile concept directly into grid calculations. 

 Revision 02.10  1995/06/21
 Robert E. Wolfe/MODLAND Support
 1. Removed radius of earth from most calculations.  Improved speed by
    a small amount.
 2. Added routine to return number of bins in each row.
 3. Fixed small bug in which occurred when 'STAND_ALONE' wasn't defined.

 Revision 02.00  1995/05/31
 Robert E. Wolfe/MODLAND Support
 Hand converted from RADFOR to C.
 1. The arguments are now all passed in radians (my preference) instead 
    of degrees.
 2. The bin numbers and row and column numbers are all zero relative 
    instead of  one relative (C standard).
 3. Unsigned integers are used for the bin numbers so that the 1 km grid 
    can be supported.  500 and 250 m grids are still out of reach with 
    32 bit integers.
 4. A nested grid capability was added.
 5. Memory is dynamically allocated for the grid indexes.
 6. The inverse mapping uses a direct solution (using asin) instead of a 
    binary search.  Actually, the binary search seems to be faster for 
    coarser resolution grids.

 Revision 02.01  1995/09/11
 Robert E. Wolfe/MODLAND Support
 Fixed bug in reuse of old set.

!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team Support Group for the National Aeronautics and 
    Space Administration, Goddard Space Flight Center, under 
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space 
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

  References and Credits:
  
    MODIS Science Team Member:
      Christopher O. Justice
      MODIS Land Science Team           University of Maryland
      justice@kratmos.gsfc.nasa.gov     NASA/GSFC Code 923
      phone: 301-286-7372               Greenbelt, MD 20771

    Developers:
      Robert E. Wolfe
      MODIS Land Team Support Group     Raytheon ITSS
      robert.wolfe@gsfc.nasa.gov        7701 Greenbelt Road, #400
      phone: 301-441-4585               Greenbelt, MD 20770  

      Paul S. Fisher
      MODIS Science Data Support Team   Research and Data Systems Corporation
      fisher@modis-xl.gsfc.nasa.gov     SAIC/GSC MODIS Support Office
      phone: 301-352-2129               7501 Forbes Blvd
                                        Seabrook, MD 20706  
      Yun-Tung (Tung) Lau
      MODIS Land Team                   University of Maryland
      YTLau@ltpmail.gsfc.nasa.gov       NASA/GSFC Code 920.2
      phone: 301-286-0869               Greenbelt, MD 20771

 ! Usage Notes:
   1. The following functions are available externally:  

     ...... Grid dependent subroutines ......

        Use '_isin' suffix for Integerized Sinusoidal (ISIN) grid
	  or '_sin' suffix for Sinusoidal (SIN) grid.

	Lbin_Init_(xx) - Initialize bin transformations by calculating
			 resolution dependent constants and a short cut
		 	 lookup table for the grid
	Lbin_FwdW_(xx) - Forward mapping from geo coordinates ('lon', 'lat')
			 to grid coordinates ('irow_o', 'icol_o',
			 'irow'.'rrr', 'icol'.'ccc')
	Lbin_Inv_(xx)  - Inverse mapping from
			 grid coordinates ('irow'.'rrr', 'icol'.'ccc')
		  	 to geographic coordinates ('lon', 'lat')

     ...... Grid independent subroutines ......
	Lbin_Free     - Deallocate the set data structure and array memory


   2. The following functions are only available internally:  

     ...... Grid dependent subroutines ......
	Lbin_SelTile_(xx) - Select a specific tile in the grid

     ...... Grid independent subroutines ......
	Lbin_Tile     - Initialize the tiling of the grid
	Lbin_Error    - Handles any fatal errors
			
   3. The fractional row and column ('rrr' and 'ccc') are:

        cell quadrant              ('rrr', 'ccc')

        upper left (north west)    (-0.5, -0.5)
        upper right (north east)   (-0.5, +0.5)
        center                     ( 0.0,  0.0)
        lower left (south west)    (+0.5, -0.5)
        lower right (south east)   (+0.5, +0.5)

      Note that for the ISIN grid, points that are near the edges of the 
      rows in the original (non-nested) resolution are ill-defined.

   4. The rows are counted from north to south and the columns from 
      west to east.  They are both zero relative.  The tiles follow the same 
      numbering scheme.  Pixels are counted from left to right (west to east) 
      and are also zero relative.

   5. There is a single tile (the whole grid) if there is no 'Lbin_Tile' call.
      The default tile is (0, 0) if no tile is selected.

   6. When the grid is tiled, all of the mappings are
      relative to the tile.  If both full grid and tile relative mappings are
      needed then use two grid sets, one for the full grid and a second for a
      specific tile.

   7. Only one grid can be active at a time.

   8. Latitudes are expected to in the range (-'HALFPI' to 'HALFPI'].

   9. Longitudes are expected to be in the range [-'PI' to 'PI').
   
   10. The external 'set' can be used to access indivdual items in the 
       'set' data structure.

   11. The externals 'irow_lower', 'irow_uppr', 'icol_left' and 
      'icol_right' must be initized by the caller and are used by 
       the 'Lbin_FwdW_(xx)' to determine if points outside of the tile 
       are near the tile edge.
       
   12. When a fatal error occurs, the function 'free_lst_space' is called 
       to free any space allocated by the 'MOD_PR11A1.exe' program.  The 'smf'
       error handling is used to handle fatal errors.

   13. Column numbers are relative to the first valid column in the tile.  To 
       get the pixel number within the tile, the external 
       'set->row[irow]->ipix_start', where 'irow' is the current row number,
       must be added to the column number.

!END***************************************************************************
*/

/*  RCF_STUFF ...
* Revision History for Original RATFOR Version:
*
* $Id: bin9kmf.rat,v 1.10 1995/03/16 19:52:23 jim Exp $
*
* $Log: bin9kmf.rat,v $
* Revision 1.10  1995/03/16  19:52:23  jim
* Add debugging code to check for NaN's (Not-a-Number).
* Disable NaN debugging code.
*
* Revision 1.9  1994/05/18  17:51:59  sue
* Make sure input parameters are valid; abort if calculate bad bin number.
*
* Revision 1.8  1994/02/16  02:49:05  jim
* Add reference longitude to bin9kminit routine.  This is the longitude of the
* center of the binned image.  E.g., ref=0 means left edge is -180 and right
* edge is +180; ref=+90 means left edge is -90 and right edge is 270 (-90);
* ref=-180 (or +180) means left edge is 0 and right edge is 360 (0).
*
* Revision 1.7  1993/08/09  15:38:19  jim
* Fix new mistake in fwd2dn.
*
* Revision 1.6  1993/08/09  15:37:37  jim
* Force fwd2dn row number to be reasonable.
*
* Revision 1.5  1993/08/09  15:18:16  sue
* Force type conversion; rename variable IN to IIN for debugging; fix chopping
* of image due to a pole.
*
* Revision 1.4  1993/05/01  19:04:01  jim
* Add multiple simultaneous bin sets (two for now) to bin transform routines
* (about 10 percent speed hit on SGI).  Maximum bin grid resolution is still
* 4 Km (internal table sizes).
*
* Revision 1.3  1993/05/01  16:24:48  jim
* Add new routine rev2dn.
* Add brief comments to describe function of all subroutines more accurately.
*
* Revision 1.2  1992/07/13  21:15:49  jim
* New algorithm that works over the poles and across the dateline.
* Could potentially save 50% of memory usage with a heuristic ....
*
* Revision 1.1  1992/05/05  03:29:52  jim
* Initial revision.
*
*/

/*
* Original Copyright from RATFOR version:
*
* Copyright 1988-1992 by Rosenstiel School of Marine and Atmospheric Science,
* University of Miami, Miami, Florida.
*
*                       All Rights Reserved
*
* Permission to use, copy, modify, and distribute this software and its
* documentation for non-commercial purposes and without fee is hereby granted,
* provided that the above copyright notice appear in all copies and that both
* that copyright notice and this permission notice appear in supporting
* documentation, and that the names of University of Miami and/or RSMAS not be
* used in advertising or publicity pertaining to distribution of the software
* without specific, written prior permission. 
*
* UNIVERSITY OF MIAMI DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
* INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT
* SHALL UNIVERSITY OF MIAMI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
* WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
* OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
*/

#include <stdlib.h>
#include <math.h>
#include "lbin_lst.h"
#include "PGS_SMF.h"
#include "PGS_MODIS_37111.h"

/* Externals */

Lbin_set_t *set = NULL;

extern int32	i_tile_in_proc;
extern int32	irow_lower, irow_uppr;
extern int32	icol_left, icol_right;

void free_lst_space(void);

/* Constants */

#ifndef M_PI
#define M_PI 3.1415926535897931
#endif
#define PI M_PI		 /* numeric constant: 'pi' */
#define TWOPI (2.0 * PI) /* numeric constant: two * 'pi' */
#define HALFPI  (PI / 2.0)

#define MIN2(x, y)  ((x) < (y) ? (x) : (y))
#define MAX2(x, y)  ((x) > (y) ? (x) : (y))
					   
#define EPS_COS (1.0e-30)

/* Types */

typedef struct {
  int16 num;             /* Error number */
  char *str;           /* Error message */
} Lbin_error_t;

/* Error Messages */

static Lbin_error_t LBIN_NOINIT		= { -5, "Initialization not called"};
static Lbin_error_t LBIN_NOMEMORY	= { -6, "Insufficient Memory"};
static Lbin_error_t LBIN_BADPARM	= { -7, "Invalid parameter"};
static Lbin_error_t LBIN_INVPROJ	= { -9, "Invalid projection"};

/* Private Prototypes */

void Lbin_Error(const Lbin_error_t *err, const char *routine);
void Lbin_Tile(int16 ntile_horiz, int16 ntile_vert);
void Lbin_SelTile_isin(int16 itile_horiz, int16 itile_vert);
void Lbin_SelTile_sin(int16 itile_horiz, int16 itile_vert);

/* 'Lbin' Functions */

void Lbin_Free(void) 
/*
!C*****************************************************************************
!Description: Lbin_Free (set free) deallocates the set data structure and
 array memory.
 
!Input Parameters:
 (none)

!Output Parameters:
 (none)

!Revision History:

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_(xx)' must have been previously called.
   2. This is a grid independent routine.

!END***************************************************************************
*/
{

  if (set == (Lbin_set_t *)NULL)
    Lbin_Error(&LBIN_NOINIT,"Free");

  /* Free the memory */

  free(set->row);
  free(set);
  
  set = (Lbin_set_t *)NULL;

  return;
}

void Lbin_Error(const Lbin_error_t *err, const char *routine) 
/*
!C*****************************************************************************

!Description: Private function to handle errors

!Input Parameters:
 err		Error structure
 routine	Sting containing name of routine where error occurred

!Output Parameters:
 (returns)	status:
		  LBIN_ERROR - normal return

!Revision History:

  * revised 05/20/1999 17:43	Z. Wan

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

 ! Usage Notes:
   1. This is a grid independent routine.

!END***************************************************************************
*/
{
  char msg[PGS_SMF_MAX_MSG_SIZE];
  char smf_message[PGS_SMF_MAX_MSGBUF_SIZE];
  char HaltMsg[2048];

  sprintf(msg, "%s Lbin_%s", err->str, routine);

  PGS_SMF_GetMsgByCode(MODIS_E_LBIN_ERROR, smf_message);
  sprintf(HaltMsg, "%s[%d]: %s %s", __FILE__, __LINE__, smf_message, msg);

  PGS_SMF_GenerateStatusReport(HaltMsg);
  
  /* printf("%s\n", HaltMsg); */

  free_lst_space(); 

  exit(1);
}

void Lbin_Tile(int16 ntile_horiz, int16 ntile_vert)
/*
!C*****************************************************************************
!Description: Lbin_Tile (tile grid) initializes the tiling of the grid.

!Input Parameters:
 ntile_horiz	number of horizontal tiles
 ntile_vert	number of vertical tiles

!Output Parameters:
 (none)

!Revision History:

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_(xx)' must have been previously called.
   2. 'ntile_vert' must be either 1 or some multiple of 2 and no more than
       the number of rows in the grid.
   3. 'ntile_horiz' must be either 1 or two times 'ntile_vert'.
   4. This is a grid independent routine.

!END***************************************************************************
*/
{
  /* Check the input parameters */
  if (set == (Lbin_set_t *)NULL)
    Lbin_Error(&LBIN_NOINIT, "Tile");

  if (ntile_vert == 1) {
    if (ntile_horiz != 1  &&  ntile_horiz != 2) 
      Lbin_Error(&LBIN_BADPARM, "Tile");
  } else if (ntile_vert < 1  ||  ntile_vert > set->nrow)
    Lbin_Error(&LBIN_BADPARM, "Tile");
  else if ((ntile_vert % 2) != 0)
    Lbin_Error(&LBIN_BADPARM, "Tile");
  else if (ntile_horiz != 2 * ntile_vert)
    Lbin_Error(&LBIN_BADPARM, "Tile");

  set->ntile_horiz = ntile_horiz;
  set->ntile_vert = ntile_vert;

  /* Select tile (0, 0) (only for the global grid) */ 

  if (ntile_horiz == 1  &&  ntile_vert == 1) {
    if (set->proj == LBIN_ISIN)
      Lbin_SelTile_isin(0, 0);
    else if (set->proj == LBIN_SIN)
      Lbin_SelTile_sin(0, 0); 
  }

  return;
}

/* ============ Subroutines for Integerized Sinusoidal Grid ================= */

#define NROW_1KM 180*60*2 /* number of rows in the 1km sinusoidal grid; 
  			   * The value (180*60*2) will generate a 30 arcsecond 
			   * (.93 km) grid */

void Lbin_Init_isin(int32 nrow, double ref_lon, int16 nest_lev,
    int16 ntile_horiz, int16 ntile_vert, int16 itile_horiz, int16 itile_vert)
/*
!C****************************************************************************
!Description: Lbin_Init_isin (initialize, integerized sinudoidal) initializes 
 the bin transformations by calculating resolution dependent constants and 
 short cut lookup tables.

!Input Parameters:
 nrow		number of rows in the grid
 ref_lon	reference longitude (radians)
 nest_lev       nesting level
 ntile_horiz    number of horizontal tiles
 ntile_vert     number of vertical tiles
 itile_horiz    horizontal tile number
 itile_vert     vertical tile number

!Output Parameters:
 (none)

!Revision History:
   1. Modified by Zhengming Wan, 1996/09/21, for nest_lev 1 only.
   2. Name changed to Lbin_Init_isin by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'nrow' must must 2 or larger and multiple of 2.  If 'nrow' is zero, the 
      value of (NROW_1KM * nest_lev) is used.
   2. 'nest_lev' must be 1 or larger and some integer fraction of 'nrow'.
      nrow/nest_lev gives the number of base grid cells.  nest_lev is the
      number of cells within a base cell.
   3. If 'Lbin_SelSet_isin' was not called then then space for set is 
      allocated.
   4. If this routine is called for a set which has already been initialized
      the set is reinitialized.
   5. The number of tiles (horizontally and vertically) is set to one.
   6. 'ref_lon' is the longitude corresponding to the left-most
       (western-most) grid location (left-most edge of the cell).

!END***************************************************************************
*/
{

  if (set == (Lbin_set_t *)NULL) {
    if ((set = (Lbin_set_t *)malloc(sizeof(Lbin_set_t))) == NULL)
      Lbin_Error(&LBIN_NOMEMORY,"Init_isin");
  }
  set->proj = LBIN_ISIN;

  /* set nrow if it is zero */
  if (nrow==0) nrow = NROW_1KM * nest_lev;

  /* Nesting Level */
  set->nest_lev = nest_lev;
  if (nest_lev < 1  || (nrow % nest_lev) != 0  ||
      nrow < 2  ||  (nrow % 2) != 0)
    Lbin_Error(&LBIN_BADPARM, "Init_isin");

  /* Zero (left-most) longitude for grid */
  set->ref_lon = ref_lon;
  if (set->ref_lon < -PI) set->ref_lon += TWOPI;
  else if (set->ref_lon > PI) set->ref_lon -= TWOPI;

  /* some constants */

  set->radius = LBIN_RE_ISIN / 1000.;  /* convert to km */
  set->ang_size = PI / nrow;
  set->grid_size = set->ang_size * set->radius;

  /* Whole bands pole-to-pole */ 
  set->nrow = (PI / set->ang_size) + 0.5;

  /* Set up tiles
  set->valid_tile = FALSE;
  set->row = NULL; */
  Lbin_Tile(ntile_horiz, ntile_vert);

  /* Select tile */ 
  Lbin_SelTile_isin(itile_horiz, itile_vert);

  return;
}

void Lbin_SelTile_isin(int16 itile_horiz, int16 itile_vert)
/*
!C*****************************************************************************
!Description: Lbin_SelTile_isin (select tile, integerized sinusoidal) selects 
 and sets up a specific tile in the grid.

!Input Parameters:
 itile_horiz	horizontal tile number
 itile_vert	vertical tile number

!Output Parameters:
 (none)

!Revision History:
   1. Name changed to Lbin_SelTile_isin by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_isin' must have been previously called.
   2. 'itile_vert' must be in the range [0, 'Set->ntile_vert' - 1].
   3. 'itile_horiz' must be in the range [0, 'Set->ntile_horiz' - 1].
   4. If 'set->row' was already allocated, the memory is released and 
      reallocated.

!END***************************************************************************
*/
{

  int16 ntile_vert_half; /* number of tiles to equator */
  int16 ntile_horiz_half; /* number of tiles to center */
  int32 nrow_half;      /* number of rows to equator */
  int32 nrow_tile_full; /* number of rows in a full tile */
  int32 nrow_tile_end;  /* number of rows in an end tile */
  int32 ncol_grid_left; /* number of columns on left in grid */
  int32 ncol_grid_right; /* number of columns on right in grid */
  int32 ncol_tile_full; /* number of columns in a full tile */
  int32 irow;           /* row number in tile */
  int32 ncol_left;      /* number of columns on the left of center */
  int32 ncol_right;     /* number of columns on the right of center */
  int32 jrow;           /* row number in grid */
  int32 jrow_base;      /* row in base grid */
  int32 ncol_base;      /* number of columns in base grid in row */
  int32 ncol;	       /* number of columns in row */
  double blat, clat;   /* base and center latitude of row */
  double ang_size_base;  /* angular size of char. bin in base grid */
  double blat_base, clat_base;/* base and center latitude in base grid of row */
  int32  i1, i2;

  if (set->proj != LBIN_ISIN) Lbin_Error(&LBIN_INVPROJ, "SelTile_isin");

  /* Check Inputs */
  if (set == (Lbin_set_t *)NULL)
    Lbin_Error(&LBIN_NOINIT, "SelTile_isin");
  if (set->proj != LBIN_ISIN) 
    Lbin_Error(&LBIN_INVPROJ, "SelTile_isin");
  if (itile_vert < 0  ||  itile_vert >= set->ntile_vert) 
    Lbin_Error(&LBIN_BADPARM, "SelTile_isin");
  if (itile_horiz < 0  ||  itile_horiz >= set->ntile_horiz) 
    Lbin_Error(&LBIN_BADPARM, "SelTile_isin");

  set->itile_horiz = itile_horiz;
  set->itile_vert = itile_vert;
  
  /* Vertical tile dependent calculations */
  nrow_half = set->nrow / 2;
  ntile_vert_half = set->ntile_vert / 2;

  if (set->ntile_vert == 1) {
    nrow_tile_full = set->nrow;
    nrow_tile_end = set->nrow;
  } else {
    nrow_tile_full = (nrow_half - 1) / ntile_vert_half + 1;
    nrow_tile_end = nrow_half - nrow_tile_full * (ntile_vert_half - 1);
    /* nrow_tile_end > 0  is guaranteed if nrow_tile_full >> ntile_vert_half */
  }

  if (itile_vert == 0 || itile_vert == (set->ntile_vert - 1))
    set->nrow_tile = nrow_tile_end;
  else
    set->nrow_tile = nrow_tile_full;

  if (itile_vert == 0)
    set->irow_start = 0;
  else
    set->irow_start = nrow_tile_end + nrow_tile_full * (itile_vert - 1);

  /* If already allocated free memory of old tile information */

  /* if (set->row != NULL) free(set->row); */

  /* Allocate arrays based on number of rows in tile */
 if ( i_tile_in_proc == 0) {
  if((set->row=(Lbin_row_t *)malloc((set->nrow_tile+1)*sizeof(Lbin_row_t))) == 
      (Lbin_row_t *)NULL)
    Lbin_Error(&LBIN_NOMEMORY, "SelTile_init");
 }

  /* Calculate for each row: Number of Columns */
  for (irow = 0; irow <= set->nrow_tile; irow++) {

    /* Actual row in grid */
    if (irow < set->nrow_tile)
      jrow = irow + set->irow_start;
    else
      jrow = nrow_half - 1;  /* last entry is for center row (above equator) */

    /* Compute base latitude */
    blat = HALFPI - (jrow + 1) * set->ang_size;

    /* Center latitude of row */
    clat = blat + (set->ang_size / 2.0);

    if (set->nest_lev == 1)
      /* Number of columns */
      ncol = (TWOPI / set->ang_size) * cos(clat) + 0.5;
    else {
      /* Compute base grid base latitude */
      jrow_base = jrow / set->nest_lev;
      ang_size_base = set->ang_size * set->nest_lev;
      blat_base = HALFPI - (jrow_base + 1) * ang_size_base;

      /* Base grid center latitude of row */
      clat_base = blat_base + (ang_size_base / 2.0);

      /* Base grid number of columns */
      ncol_base = (TWOPI / ang_size_base) * cos(clat_base) + 0.5;

      /* Number of columns */
      ncol = set->nest_lev * ncol_base;
    }
    if (irow < set->nrow_tile)
      set->row[irow].ncol = ncol;
    else
      set->ncol_max = ncol;

    /* the following three lines originally in InitBin after
    	Lbin_Init (0, ref_lon, nest, 1, 1, 0, 0) */

    set->row[irow].dlon_grid = set->ang_size / cos(clat);
    set->row[irow].scale[0] = ncol / TWOPI;
    set->row[irow].scale[1] = NROW_1KM * set->nest_lev / PI;
    set->row[irow].ref_lon = set->ref_lon;

  } /* for (irow ... */

  /* Horizontal tile dependencies */
  ncol_grid_left = (set->ncol_max + 1) / 2;  /* number of columns on the left (at equator) */
  ncol_grid_right = set->ncol_max - ncol_grid_left;  /* number of columns on the right (at equator) */
  if (set->ntile_horiz > 1) {
    ntile_horiz_half = set->ntile_horiz / 2;  /* number of tiles to center */
    ncol_tile_full = (ncol_grid_left - 1) / ntile_horiz_half + 1;  /* number of columns in a full tile */
  } 

  /* Number of pixels in rectangular tile */
  if (set->ntile_horiz == 1)
    set->npix = set->ncol_max;
  else {
    if (itile_horiz < ntile_horiz_half) {
      i2 = ncol_grid_left - ncol_tile_full * (ntile_horiz_half - itile_horiz - 1);
      set->npix = MIN2(ncol_tile_full, i2);
    }
    else {
      i1 = ncol_grid_right - ncol_tile_full * (itile_horiz - ntile_horiz_half);
      i2 = MIN2(ncol_tile_full, i1);
      set->npix = MAX2(0, i2);
    }
  }
    
  /* For each row in the tile */
  for (irow = 0; irow < set->nrow_tile; irow++) {

    ncol_left = (set->row[irow].ncol + 1) / 2;
    ncol_right = set->row[irow].ncol - ncol_left;

    if (set->ntile_horiz == 1) {

      set->row[irow].icol_start = 0;
      set->row[irow].ncol_tile = set->row[irow].ncol;
      set->row[irow].ipix_start = ncol_grid_left - ncol_left;

    } else {

      if (itile_horiz < ntile_horiz_half) {
        i1 = ncol_left - ncol_tile_full * (ntile_horiz_half - itile_horiz);
        set->row[irow].icol_start = MAX2(0, i1);
        i1 = ncol_left - ncol_tile_full * (ntile_horiz_half - itile_horiz - 1);
	i2 = MIN2(ncol_tile_full, i1);
        set->row[irow].ncol_tile = MAX2(0, i2);
        set->row[irow].ipix_start = set->npix - set->row[irow].ncol_tile;
      } else {
        i1 = ncol_left + ncol_tile_full * (itile_horiz - ntile_horiz_half);
        set->row[irow].icol_start = MIN2(set->row[irow].ncol, i1);
        i1 = ncol_right - ncol_tile_full * (itile_horiz - ntile_horiz_half);
	i2 = MIN2(ncol_tile_full, i1);
        set->row[irow].ncol_tile = MAX2(0, i2);
        set->row[irow].ipix_start = 0;
      }
    }
  } /* end for (irow... */
  
  /* Calculate the start pixel for the tile in the entire grid */
  if (set->ntile_horiz == 1)
    set->ipix_start = 0;
  else {
    i1 = ncol_grid_left - ncol_tile_full * (ntile_horiz_half - itile_horiz);
    set->ipix_start = MAX2(0, i1);
  }

  /* Calculate for each row: Base bin number */
  set->row[set->nrow_tile].ncol_tile = 0; 

  return;
}

int32 Lbin_FwdW_isin(double lon, double lat, int32 *irow_o, int32 *icol_o,
             int32 *irow, int32 *icol, float32 *rrr, float32 *ccc)
/*
!C*****************************************************************************
!Description: Lbin_FwdW_isin (forward mapping, integerized sinusoidal) maps 
        from geographic coordinates ('lon', 'lat') to the tile 
	coordinates ('irow_o', 'icol_o', 'irow'.'rrr', 'icol'.'ccc').

!Input Parameters:
 lon		longitude (radians)
 lat		latitude (radians)

!Output Parameters:
 irow_o		row number before shift
 icol_o		column number before shift
 irow		row number which may be shifted
 icol		column number which may be shifted
 rrr		fractional row number (relative to 'irow')
 ccc		fractional column number (relative to 'icol')
 (returns)      status:
		  LBIN_NORMAL - normal return
                  LBIN_ERANGE - point not in tile

!Revision History:

   1. Revision 2.0  1997/10/06  00:34     Z. Wan
      for dealing with pixels by the edge.
   2. Name changed to Lbin_FwdW_isin by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_isin' must have been previously called.
   2. The latitude is truncated to the range [-'PI' to 'PI'].
   3. The row and column numbers are relative to the currently selected tile.
   4. If the latitude is not in the tile, the column number is not
      calculated and the status returned is 'LBIN_ERANGE'.
   5. If the pixel is near the edge of the tile, LBIN_NORMAL is still returned.
      The original location is returned as 'irow_o' and 'icol_o' and the 
      within tile location is returned as 'irow' and 'icol'.
      The values 'rrr' and 'ccc' are always relative to 'irow' and 'icol'.  
      This means that they may have a magnitude greater than 0.5.
   6. The externals 'irow_lower', 'irow_uppr', 'icol_left' and 
      'icol_right' must be initized by the caller and are used to 
      determine if points outside of the tile are near the tile edge.

!END***************************************************************************
*/
{
  int32 jrow, jcol;
  double row, col;
  double flon;

  /*if (set == (Lbin_set_t *)NULL) Lbin_Error(&LBIN_NOINIT, "FwdW_isin");
  if (set->proj != LBIN_ISIN) Lbin_Error(&LBIN_INVPROJ, "FwdW_isin");*/
 
  /* row */
  row = (HALFPI - lat) / set->ang_size;

  jrow = row; 
  *irow_o = jrow - set->irow_start;
  *irow = *irow_o;
  *rrr = row - 0.5 - jrow;

 if ( *irow < (set->nrow_tile + irow_uppr) ) {
  while ( *irow > (set->nrow_tile - 1)) {
    (*irow)--; *rrr += 1.0;
  }
 }
 else if ( *irow > irow_lower ) {
  while ( *irow < 0 ) {
    (*irow)++; *rrr -= 1.0;
  }
 }

  if (*irow < 0  ||  *irow > (set->nrow_tile - 1)) {
    *icol = 0;
    *ccc = 0.0;
    return LBIN_ERANGE;
  }

  /* column */
  flon = (lon - set->row[*irow].ref_lon) / TWOPI;
  if (flon < 0.0) flon += 1 - (int32) flon;
  if (flon >= 1.0) flon -= (int32) flon;

  col = set->row[*irow].ncol * flon;
  jcol = col;  
  if (col < 0.0) jcol--;
  *icol_o = jcol - set->row[*irow].icol_start;
  *icol = *icol_o;
  *ccc = col - 0.5 - jcol;

 if ( *icol < (set->row[*irow].ncol_tile + icol_right) ) {
  while ( *icol > (set->row[*irow].ncol_tile - 1)) {
    (*icol)--; *ccc += 1.0;
  }
 }
 else if ( *icol > icol_left ) {
  while ( *icol < 0 ) {
    (*icol)++; *ccc -= 1.0;
  }
 }

  if (*icol < 0  ||  *icol > (set->row[*irow].ncol_tile - 1))
    return LBIN_ERANGE;

  return LBIN_NORMAL;
}

int32 Lbin_Inv_isin(int32 irow, int32 icol, float rrr, float ccc,
	     double *lon, double *lat)
/*
!C*****************************************************************************
!Description: Lbin_Inv_isin (inverse mapping, integerized sinusoidal) maps 
 from grid coordinates ('irow'.'rrr', 'icol'.'ccc') to geographic 
 coordinates ('lon', 'lat').

!Input Parameters:
 irow		row number
 icol		column number
 rrr		fractional row number
 ccc		fractional column number

!Output Parameters:
 lon		longitude (radians)
 lat		latitude (radians)
 (returns)      status:
		  LBIN_NORMAL - normal return
                  LBIN_ERANGE - point not in tile

!Revision History:
   1. Name changed to Lbin_Inv_isin by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_isin' must have been previously called.
   2. The longitude returned is in the range [-'PI' to 'PI'].
   3. The row and column numbers are relative to the currently selected tile.
   4. If the row is not in the tile, the longitude is not
      calculated and the status returned is 'LBIN_ERANGE'.

!END***************************************************************************
*/
{

  int32 jrow, jcol;
  int32 stat;

  /*if (set == (Lbin_set_t *)NULL) Lbin_Error(&LBIN_NOINIT, "Inv_isin");
  if (set->proj != LBIN_ISIN) Lbin_Error(&LBIN_INVPROJ, "Inv_isin");*/

  stat = LBIN_NORMAL;

  /* latitude */
  jrow = set->irow_start + irow;
  if (jrow < 0) {jrow = 0;  stat = LBIN_ERANGE;}
  else if (jrow >= set->nrow) {jrow = set->nrow;  return LBIN_ERANGE;}

  *lat = HALFPI - (jrow + (double) rrr + 0.5) * set->ang_size;

  if (irow < 0  ||  irow >= set->nrow_tile) stat = LBIN_ERANGE;
  if (stat != LBIN_NORMAL) {*lon = 0.0;  return stat;}

  /* longitude */
  jcol = set->row[irow].icol_start + icol;
  if (jcol < 0) {jcol = 0;  stat = LBIN_ERANGE;}
  else if (jcol >= set->row[irow].ncol)
    {jcol = set->row[irow].ncol - 1;  stat = LBIN_ERANGE;}

  *lon = set->row[irow].ref_lon + TWOPI *
	((jcol + (double) ccc + 0.5) / set->row[irow].ncol);

  if (*lon < -PI) *lon += TWOPI;
  else if (*lon >= PI) *lon -= TWOPI;

  if (icol < 0  ||  icol >= set->row[irow].ncol_tile) stat = LBIN_ERANGE;

  return stat;
}


/* ================== Subroutines for Sinusoidal Grid ======================= */

#define NROW_1KM 180*60*2 /* number of rows in the 1km sinusoidal grid; 
  			   * The value (180*60*2) will generate a 30 arcsecond 
			   * (.93 km) grid */

void Lbin_Init_sin(int32 nrow, double ref_lon, int16 nest_lev,
    int16 ntile_horiz, int16 ntile_vert, int16 itile_horiz, int16 itile_vert)
/*
!C****************************************************************************
!Description: Lbin_Init_isin (initialize, sinudoidal) initializes 
 the bin transformations by calculating resolution dependent constants and 
 short cut lookup tables.

!Input Parameters:
 nrow		number of rows in the grid
 ref_lon	reference longitude (radians)
 nest_lev       nesting level
 ntile_horiz    number of horizontal tiles
 ntile_vert     number of vertical tiles
 itile_horiz    horizontal tile number
 itile_vert     vertical tile number

!Output Parameters:
 (none)

!Revision History:
  Derived from Lbin_Init by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'nrow' must must 2 or larger and multiple of 2.  If 'nrow' is zero, the 
      value of (NROW_1KM * nest_lev) is used.
   2. 'nest_lev' must be 1 or larger and some integer fraction of 'nrow'.
      nrow/nest_lev gives the number of base grid cells.  nest_lev is the
      number of cells within a base cell.
   3. If 'Lbin_SelSet_sin' is not called then the set is initilized.
   4. If this routine is called for a set which has already been initialized
      the set is reinitialized.
   5. The number of tiles (horizontally and vertically) is set to one.
   6. 'ref_lon' is the longitude corresponding to the left-most
       (western-most) grid location (left-most edge of the cell).
   7. Modified by Zhengming Wan, 1996/09/21, for nest_lev 1 only

!END***************************************************************************
*/
{

  if (set == (Lbin_set_t *)NULL) {
    if ((set = (Lbin_set_t *)malloc(sizeof(Lbin_set_t))) == NULL)
      Lbin_Error(&LBIN_NOMEMORY,"Init_sin");
  }
  set->proj = LBIN_SIN;

  /* set nrow if it is zero */
  if (nrow==0) nrow = NROW_1KM * nest_lev;

  /* Nesting Level */
  set->nest_lev = nest_lev;
  if (nest_lev < 1  || (nrow % nest_lev) != 0  ||
      nrow < 2  ||  (nrow % 2) != 0)
    Lbin_Error(&LBIN_BADPARM, "Init_isin");

  /* Zero longitude for grid */
  set->ref_lon = ref_lon;
  if (set->ref_lon < -PI) set->ref_lon += TWOPI;
  else if (set->ref_lon > PI) set->ref_lon -= TWOPI;

  /* some constants */

  set->radius = LBIN_RE_SIN / 1000.;  /* convert to km */
  set->ang_size = PI / nrow;
  set->grid_size = set->ang_size * set->radius;

  /* Whole bands pole-to-pole */ 
  set->nrow = (PI / set->ang_size) + 0.5;

  /* Set up tiles
  set->valid_tile = FALSE;
  set->row = NULL; */
  Lbin_Tile(ntile_horiz, ntile_vert);

  /* Select tile */ 
  Lbin_SelTile_sin(itile_horiz, itile_vert);

  return;
}

void Lbin_SelTile_sin(int16 itile_horiz, int16 itile_vert)
/*
!C*****************************************************************************
!Description: Lbin_SelTile_sin (select tile, sinusoidal) selects 
 a specific tile in the grid.

!Input Parameters:
 itile_horiz	horizontal tile number
 itile_vert	vertical tile number

!Output Parameters:
 (none)

!Revision History:
   1. Derived from Lbin_SelTile by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_sin' must have been previously called.
   2. 'itile_vert' must be in the range [0, 'Set->ntile_vert' - 1].
   3. 'itile_horiz' must be in the range [0, 'Set->ntile_horiz' - 1].

!END***************************************************************************
*/
{

  int16 ntile_vert_half; /* number of tiles to equator */
  int16 ntile_horiz_half; /* number of tiles to center */
  int32 nrow_half;      /* number of rows to equator */
  int32 nrow_tile_full; /* number of rows in a full tile */
  int32 nrow_tile_end;  /* number of rows in an end tile */
  int32 ncol_grid_left; /* number of columns on left in grid */
  int32 ncol_grid_right; /* number of columns on right in grid */
  int32 ncol_tile_full; /* number of columns in a full tile */
  int32 irow;           /* row number in tile */
  int32 ncol_left;      /* number of columns on the left of center */
  int32 ncol_right;     /* number of columns on the right of center */
  int32 jrow;           /* row number in grid */
  int32 jrow_base;      /* row in base grid */
  int32 ncol_base;      /* number of columns in base grid in row */
  int32 ncol;	        /* number of columns in row */
  double blat, clat;    /* base and narrowest latitude of row */
  double ang_size_base; /* angular size of bins in base grid */
  int32 nrow_base;      /* number of rows in the base grid */
  double blat_base, clat_base;
                        /* base and narrowest latitude in base grid of row */
  int32  i1, i2;

  if (set->proj != LBIN_SIN) Lbin_Error(&LBIN_INVPROJ, "SelTile_sin");

  /* Check Inputs */
  if (set == (Lbin_set_t *)NULL)
    Lbin_Error(&LBIN_NOINIT, "SelTile_sin");
  if (set->proj != LBIN_SIN) 
    Lbin_Error(&LBIN_INVPROJ, "SelTile_sin");
  if (itile_vert < 0  ||  itile_vert >= set->ntile_vert) 
    Lbin_Error(&LBIN_BADPARM, "SelTile_sin");
  if (itile_horiz < 0  ||  itile_horiz >= set->ntile_horiz) 
    Lbin_Error(&LBIN_BADPARM, "SelTile_sin");

  set->itile_horiz = itile_horiz;
  set->itile_vert = itile_vert;
  
  /* Vertical tile dependent calculations */
  nrow_half = set->nrow / 2;
  ntile_vert_half = set->ntile_vert / 2;

  if (set->ntile_vert == 1) {
    nrow_tile_full = set->nrow;
    nrow_tile_end = set->nrow;
  } else {
    nrow_tile_full = (nrow_half - 1) / ntile_vert_half + 1;
    nrow_tile_end = nrow_half - nrow_tile_full * (ntile_vert_half - 1);
    /* nrow_tile_end > 0  is guaranteed if nrow_tile_full >> ntile_vert_half */
  }

  if (itile_vert == 0 || itile_vert == (set->ntile_vert - 1))
    set->nrow_tile = nrow_tile_end;
  else
    set->nrow_tile = nrow_tile_full;

  if (itile_vert == 0)
    set->irow_start = 0;
  else
    set->irow_start = nrow_tile_end + nrow_tile_full * (itile_vert - 1);

  /* If already allocated free memory of old tile information */

  /* if (set->row != NULL) free(set->row); */

  /* Allocate arrays based on number of rows in tile */
 if ( i_tile_in_proc == 0) {
  if((set->row=(Lbin_row_t *)malloc((set->nrow_tile+1)*sizeof(Lbin_row_t))) == 
      (Lbin_row_t *)NULL)
    Lbin_Error(&LBIN_NOMEMORY, "SelTile_init");
 }

  ang_size_base = set->ang_size * set->nest_lev;
  nrow_base = set->nrow / set->nest_lev;

  /* Calculate for each row: Number of Columns */
  for (irow = 0; irow <= set->nrow_tile; irow++) {

    /* Actual row in grid */

    if (irow < set->nrow_tile)
      jrow = irow + set->irow_start;
    else
      jrow = nrow_half - 1;  /* last entry is for center row (above equator) */

    if (set->nest_lev == 1) {

      /* Compute base latitude */
      clat = blat = HALFPI - (jrow + 1) * set->ang_size;

      /* clat += ( set->ang_size / 2.0);	not the row's center */

      /* Get the latitude at the narrowest part of the row:
           top of the row in the northern hemisphere; 
           bottom of the row in the southern hemisphere */

      if (jrow < nrow_half)
        clat += set->ang_size;

      /* Number of columns */
      ncol = 2 * ((int)(set->nrow * cos(clat)));
    
    } else {

      /* Compute base grid base latitude */

      jrow_base = jrow / set->nest_lev;

      blat_base = HALFPI - (jrow_base + 1) * ang_size_base;

      if (jrow < nrow_half)
        clat_base = blat_base + ang_size_base;

      /* Base grid number of columns */
      ncol_base = 2 * ((int)(nrow_base * cos(clat_base)));

      /* Number of columns */
      ncol = set->nest_lev * ncol_base;
 
    }

    if (irow < set->nrow_tile)
      set->row[irow].ncol = ncol;

    /* the following three lines originally in InitBin after
    	Lbin_Init (0, ref_lon, nest, 1, 1, 0, 0) */

    set->row[irow].dlon_grid = set->ang_size / cos(clat);
    set->row[irow].scale[0] = ncol / TWOPI;
    set->row[irow].scale[1] = NROW_1KM * set->nest_lev / PI;
    set->row[irow].ref_lon = - 0.5 * set->ang_size * ncol / cos(clat);
/*
    set->row[irow].scale[0] = 1.0;
    set->row[irow].scale[1] = 1.0;
*/
  } /* for (irow ... */

  /* Total columns in grid */

  set->ncol_max = 2 * set->nrow;
  
  /* Horizontal tile dependencies */

  /* Number of columns on the left and right (at equator) */

  ncol_grid_left = ncol_grid_right = set->ncol_max / 2;  

  if (set->ntile_horiz > 1) {
    /* Number of tiles to center */
    ntile_horiz_half = set->ntile_horiz / 2;
    /* Number of columns in a full tile */
    ncol_tile_full = (ncol_grid_left - 1) / ntile_horiz_half + 1;  
  } 

  /* Number of pixels in rectangular tile */
  if (set->ntile_horiz == 1)
    set->npix = set->ncol_max;
  else {
    if (itile_horiz < ntile_horiz_half) {
      i2 = ncol_grid_left - ncol_tile_full * 
           (ntile_horiz_half - itile_horiz - 1);
      set->npix = MIN2(ncol_tile_full, i2);
    }
    else {
      i1 = ncol_grid_right - ncol_tile_full * 
           (itile_horiz - ntile_horiz_half);
      i2 = MIN2(ncol_tile_full, i1);
      set->npix = MAX2(0, i2);
    }
  }

  /* For each row in the tile */
  for (irow = 0; irow < set->nrow_tile; irow++) {

    ncol_left = ncol_right = set->row[irow].ncol / 2;

    if (set->ntile_horiz == 1) {

      set->row[irow].icol_start = 0;
      set->row[irow].ncol_tile = set->row[irow].ncol;
      set->row[irow].ipix_start = ncol_grid_left - ncol_left;

    } else {

      if (itile_horiz < ntile_horiz_half) {
        i1 = ncol_left - ncol_tile_full * (ntile_horiz_half - itile_horiz);
        set->row[irow].icol_start = MAX2(0, i1);
        i1 = ncol_left - ncol_tile_full * (ntile_horiz_half - itile_horiz - 1);
	i2 = MIN2(ncol_tile_full, i1);
        set->row[irow].ncol_tile = MAX2(0, i2);
        set->row[irow].ipix_start = set->npix - set->row[irow].ncol_tile;
      } else {
        i1 = ncol_left + ncol_tile_full * (itile_horiz - ntile_horiz_half);
        set->row[irow].icol_start = MIN2(set->row[irow].ncol, i1);
        i1 = ncol_right - ncol_tile_full * (itile_horiz - ntile_horiz_half);
	i2 = MIN2(ncol_tile_full, i1);
        set->row[irow].ncol_tile = MAX2(0, i2);
        set->row[irow].ipix_start = 0;
      }

    }
  } /* end for (irow... */
  
  /* Calculate the start pixel for the tile in the entire grid */
  if (set->ntile_horiz == 1)
    set->ipix_start = 0;
  else {
    i1 = ncol_grid_left - ncol_tile_full * (ntile_horiz_half - itile_horiz);
    set->ipix_start = MAX2(0, i1);
  }

  /* Calculate for each row: Base bin number */
  set->row[set->nrow_tile].ncol_tile = 0; 

  return;
}

int32 Lbin_FwdW_sin(double lon, double lat, int32 *irow_o, int32 *icol_o,
             int32 *irow, int32 *icol, float32 *rrr, float32 *ccc)
/*
!C*****************************************************************************
!Description: Lbin_FwdW_sin (forward mapping, sinusoidal) maps 
        from geographic coordinates ('lon', 'lat') to the tile 
	coordinates ('irow_o', 'icol_o', 'irow'.'rrr', 'icol'.'ccc').

!Input Parameters:
 lon		longitude (radians)
 lat		latitude (radians)

!Output Parameters:
 irow_o		row number before shift
 icol_o		column number before shift
 irow		row number which may be shifted
 icol		column number which may be shifted
 rrr		fractional row number (relative to 'irow')
 ccc		fractional column number (relative to 'icol')
 (returns)      status:
		  LBIN_NORMAL - normal return
                  LBIN_ERANGE - point not in tile

!Revision History:

   1. Revision 2.0  1997/10/06  00:34     Z. Wan
      for dealing with pixels by the edge.
   2. Derived from Lbin_FwdW by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init_sin' must have been previously called.
   2. The latitude is truncated to the range [-'PI' to 'PI'].
   3. The row and column numbers are relative to the currently selected tile.
   4. If the latitude is not in the tile, the column number is not
      calculated and the status returned is 'LBIN_ERANGE'.
   5. If the pixel is near the edge of the tile, LBIN_NORMAL is still returned.
      The original location is returned as 'irow_o' and 'icol_o' and the 
      within tile location is returned as 'irow' and 'icol'.
      The values 'rrr' and 'ccc' are always relative to 'irow' and 'icol'.  
      This means that they may have a magnitude greater than 0.5.
   6. The externals 'irow_lower', 'irow_uppr', 'icol_left' and 
      'icol_right' must be initized by the caller and are used to 
      determine if points outside of the tile are near the tile edge.

!END***************************************************************************
*/
{
  int32 jrow, jcol;
  double row, col;
  double flon;

  /*if (set == (Lbin_set_t *)NULL) Lbin_Error(&LBIN_NOINIT, "FwdW_sin");
  if (set->proj != LBIN_SIN) Lbin_Error(&LBIN_INVPROJ, "FwdW_sin");*/
 
  /* row */

  row = (HALFPI - lat) / set->ang_size;

  jrow = row; 
  *irow_o = jrow - set->irow_start;
  *irow = *irow_o;
  *rrr = row - 0.5 - jrow;

  if ( *irow < (set->nrow_tile + irow_uppr) ) {
    while ( *irow > (set->nrow_tile - 1)) {
      (*irow)--; *rrr += 1.0;
    }
  } else if ( *irow > irow_lower ) {
    while ( *irow < 0 ) {
      (*irow)++; *rrr -= 1.0;
    }
  }

  if (*irow < 0  ||  *irow > (set->nrow_tile - 1)) {
    *icol = 0;
    *ccc = 0.0;
    return LBIN_ERANGE;
  }

  /* column */

  /* flon = (lon - set->row[*irow].ref_lon) / TWOPI;

  if (flon < 0.0) flon += 1 - (int32)flon;
  if (flon >= 1.0) flon -= (int32)flon;

  col = set->nrow * (1.0 + ((2.0 * flon) - 1.0) * cos(lat)); */

  col = (lon - set->row[*irow].ref_lon) / set->row[*irow].dlon_grid;

  jcol = col;  
  if (col < 0.0) jcol--;
  *icol_o = jcol - set->row[*irow].icol_start;

  /* *icol_o = jcol - (set->row[*irow].ipix_start + set->ipix_start); */

  *icol = *icol_o;
  *ccc = col - 0.5 - jcol;

  if ( *icol < (set->row[*irow].ncol_tile + icol_right) ) {
    while ( *icol > (set->row[*irow].ncol_tile - 1)) {
      (*icol)--; *ccc += 1.0;
    }
  } else if ( *icol > icol_left ) {
     while ( *icol < 0 ) {
      (*icol)++; *ccc -= 1.0;
    }
  }

  if (*icol < 0  ||  *icol > (set->row[*irow].ncol_tile - 1))
    return LBIN_ERANGE;

  return LBIN_NORMAL;
}

int32 Lbin_Inv_sin(int32 irow, int32 icol, float rrr, float ccc,
	     double *lon, double *lat)
/*
!C*****************************************************************************
!Description: Lbin_Inv_sin (inverse mapping, sinusoidal) maps 
 from grid coordinates ('irow'.'rrr', 'icol'.'ccc') to geographic 
 coordinates ('lon', 'lat').

!Input Parameters:
 irow		row number
 icol		column number
 rrr		fractional row number
 ccc		fractional column number

!Output Parameters:
 lon		longitude (radians)
 lat		latitude (radians)
 (returns)      status:
		  LBIN_NORMAL - normal return
                  LBIN_ERANGE - point not in tile

!Revision History:
   1. Derived from Lbin_Inv_isin by Robert Wolfe, 2002/09/12.

!Team-unique Header:

 ! Usage Notes:
   1. 'Lbin_Init' must have been previously called.
   2. The longitude returned is in the range [-'PI' to 'PI'].
   3. The row and column numbers are relative to the currently selected tile.
   4. If the row is not in the tile, the longitude is not
      calculated and the status returned is 'LBIN_ERANGE'.

!END***************************************************************************
*/
{

  double cos_lat;
  int32 jrow, jcol;
  int32 stat;

  /*if (set == (Lbin_set_t *)NULL) Lbin_Error(&LBIN_NOINIT, "Inv_sin");
  if (set->proj != LBIN_SIN) Lbin_Error(&LBIN_INVPROJ, "Inv_sin");*/

  stat = LBIN_NORMAL;

  /* latitude */
  jrow = set->irow_start + irow;
  if (jrow < 0.0) {jrow = 0.0;  stat = LBIN_ERANGE;}
  else if (jrow >= (double)set->nrow) 
    {jrow = set->nrow; return LBIN_ERANGE;}

  *lat = HALFPI - (jrow + (double) rrr + 0.5 ) * set->ang_size;

  if (irow < 0  ||  irow >= set->nrow_tile) stat = LBIN_ERANGE;

  if (stat != LBIN_NORMAL) {*lon = 0.0;  return stat;}

  /* longitude */

  if (icol < 0  ||  icol >= set->row[irow].ncol_tile) stat = LBIN_ERANGE;

  jcol = set->row[irow].icol_start + icol;

  /* jcol = set->ipix_start + set->row[irow].ipix_start + icol; */

  if ( jcol < 0 ) {jcol = 0; stat = LBIN_ERANGE; }
  else if ( jcol >= set->row[irow].ncol ) {
	    jcol = set->row[irow].ncol - 1; stat = LBIN_ERANGE; }

  /* cos_lat = cos(*lat);

  if (fabs(cos_lat) > EPS_COS) {
    *lon = PI * (1.0 + (((jcol + (double)ccc + 0.5) / set->nrow) - 1.0) / cos_lat);
  } else
    *lon = 0.0;		*/

  *lon = set->row[irow].ref_lon + jcol * set->row[irow].dlon_grid;

  if (*lon < -PI) *lon = -PI;
  else if (*lon >= PI) *lon = PI;

  return stat;
}
