/******************************************************************************
!C-INC

File: lbin_lst.h

!Description: 
  ISCCP Grid Library Header - constants and prototypes
  for grid library functions.

!Input Parameters: N/A

!Output Parameters: N/A

!Revision History:

  Revision V2.1.1  2002/10/12
  Zhengming Wan
  - replace int32 ncol_grid with float32 dlon_grid in Lbin_row_t
  - add float32 ref_lon in Lbin_row_t

  Revision V2.1  2002/09/12
  Robert Wolfe
  - Added sinusoidal grid (changed names of common functions).
  - Reworked error handling.
  - Extracted from mod11_lst2A1.c into seperate source module.

  Revision 02.32  1996/6/6
  Tung Lau
  Add _gh functions for goode homolosine grid and function pointers.

  Revision 02.31  1995/11/08
  Robert Wolfe and Paul Fisher
  1. Modified to add SMF error handling.
  
  Revision 02.30  1995/08/04
  Robert E. Wolfe/MODLAND Support
  (added HDF; see lbin.c)

  Revision 02.20  1995/07/22
  Robert E. Wolfe/MODLAND Support
  (complete rewrite of C version; see lbin.c)

  Revision 02.10  1995/06/21
  Robert E. Wolfe/MODLAND Support
  (small changes; see lbin.c)

  Revision 02.00  1995/05/31
  Robert E. Wolfe/MODLAND Support
  (hand converted from RADFOR to C; see lbin.c)

!Team-unique Header:
  This software was developed by:
    MODIS Land Science Team for the National Aeronautics and 
    Space Administration, Goddard Space Flight Center, under 
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and 
    Space Administration, Goddard Space Flight Center, under 
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space 
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

!References and Credits:
  
    MODIS Science Team Member:
      Christopher O. Justice
      MODIS Land Science Team           University of Maryland
      justice@kratmos.gsfc.nasa.gov     NASA/GSFC Code 923
      phone: 301-286-7372               Greenbelt, MD 20771

    Developers:
      Robert E. Wolfe
      MODIS Land Team Support Group     Hughes STX
      robert.wolfe@gsfc.nasa.gov        7701 Greenbelt Road, #400
      phone: 301-441-4585               Greenbelt, MD 20770  

      Paul S. Fisher
      MODIS Science Data Support Team   Research and Data Systems Corporation
      fisher@modis-xl.gsfc.nasa.gov     SAIC/GSC MODIS Support Office
      phone: 301-352-2129               7501 Forbes Blvd
                                        Seabrook, MD 20706  
      Yun-Tung (Tung) Lau
      MODIS Land Team                   University of Maryland
      YTLau@ltpmail.gsfc.nasa.gov       NASA/GSFC Code 920.2
      phone: 301-286-0869               Greenbelt, MD 20771

!Design Notes:

!END
*******************************************************************************/

/* Return Codes */

#include "hdf.h"               /* HDF header files */
#include "mfhdf.h"

#define LBIN_NORMAL 0     /* Normal return */
#define LBIN_ERANGE -2	  /* Input varible out of range */
#define	NOT_VALID -1

#define LBIN_RE_ISIN (6371007.181)      /* Earth radius (m), sphere with
					   the best surface area fit of
					   WGS84 ellipsoid. Spherecode -1,
					   to be consistent with grid.h in
					   PGE12/MOD_PRMGPNTR */
#define LBIN_RE_SIN (LBIN_RE_ISIN)      /* Earth radius for sinusoidal is 
					   the same as the radius
					   for the integerized 
					   sinusoidal */
					   

typedef enum {LBIN_ISIN, LBIN_SIN} Lbin_proj_t;

/* Inititialize bin transformations by calculating resolution dependent
 * constants and a short cut lookup table */

/* Deallocate the set data structure and array memory */

void Lbin_Free(void);

/* Initialize the grid */

void Lbin_Init_isin(int32 nrow, double ref_lon, int16 nest_lev,
    int16 ntile_horiz, int16 ntile_vert, int16 itile_horiz, int16 itile_vert);

void Lbin_Init_sin(int32 nrow, double ref_lon, int16 nest_lev,
    int16 ntile_horiz, int16 ntile_vert, int16 itile_horiz, int16 itile_vert);

typedef void (*PLbin_Init)(int32 nrow, double ref_lon, int16 nest_lev,
    int16 ntile_horiz, int16 ntile_vert, int16 itile_horiz, int16 itile_vert);

/* Forward mapping; map from geographic coordinates ('lon', 'lat') to
 * grid coordinates ('irow'.'rrr', 'icol'.'ccc') and ('irow_o' and 'icol_o') */

int32 Lbin_FwdW_isin(double, double, int32 *, int32 *, int32 *, int32 *, 
                  float32 *, float32 *);

int32 Lbin_FwdW_sin(double, double, int32 *, int32 *, int32 *, int32 *, 
                  float32 *, float32 *);

typedef int32 (*PLbin_FwdW)(double, double, int32 *, int32 *, int32 *, int32 *, 
                  float32 *, float32 *);

/* Inverse mapping; map from grid coordinates ('irow'.'rrr', 'icol'.'ccc')
 * to geographic coordinates ('lon', 'lat') */

int32 Lbin_Inv_isin(int32 irow, int32 icol, float rrr, float ccc,
             double *lon, double *lat);

int32 Lbin_Inv_sin(int32 irow, int32 icol, float rrr, float ccc,
             double *lon, double *lat);

typedef int32 (*PLbin_Inv)(int32 irow, int32 icol, float rrr, float ccc,
             double *lon, double *lat);

/* Structures */

/* Row Structure */

typedef struct {
  int32 ncol;		  /* Number of columns in entire grid */
  int32 icol_start;	  /* number of start column in the entire grid */
  int32 icol_end;	  /* number of end column in the entire grid */
  int32 ncol_tile;	  /* Number of columns with grids in a tile */
  int32 ipix_start;	  /* Start pixel of first column in rectangular tile */
  float32 dlon_grid;	  /* dlon of a grid */
  float32 ref_lon;	  /* reference longitude (rad) for each row */
  float32 scale[2];	  /* vertical and horizontal scale factor */
} Lbin_row_t;

/* Lbin set Structure */

typedef struct {
                      /* values are assigned in Lbin_Init_(xx) */
  Lbin_proj_t proj;       /* Projection */
  double ref_lon;	  /* Zero reference longitude (rad) */
  double ang_size;	  /* Grid angular resolution (rad) */
  int32 nrow;		  /* Total rows in full grid (latitude) */
  int16 nest_lev;	  /* Nesting Level */
  int16 valid_tile;	  /* Valid tile; TRUE if tile initialization has been
  			   * called (Lbin_SelTile_(xx)) */ 

                      /* values assiagned in Lbin_Tile */
  int16 ntile_horiz;	  /* Number of horizontal tiles */
  int16 ntile_vert;	  /* Number of vertical tiles */

                      /* values assiagned in Lbin_SelTile_(xx) */
  int16 itile_horiz;	  /* Horizontal tile number */
  int16 itile_vert;	  /* Vertical tile number */
  int32 irow_start;	  /* Start row of tile in entire grid */
  int32 ipix_start;	  /* Start pixel of tile in entire grid */
  int32 nrow_tile;	  /* Number of rows in tile */
  int32 npix;		  /* Number of pixels per row in rectangular tile */
  int32 ncol_max;	  /* Maximum number of columns/row in entire grid */
  double grid_size;	  /* Grid resolution (km) */
  double radius;	  /* Sphere radius (km) */
  Lbin_row_t *row;	  /* Pointer to information about each row */
} Lbin_set_t;
