/*
!C-INC**************************************************************************

  ... 3/08/96 - R. Wolfe - modification to read new L1b format ...
  ... 3/15/96 - R. Wolfe - modification to read new cloud mask format ...

!File: lhdf.h  (Land HDF functions)

!Description: Land HDF functions header - structures, types and 
 prototypes for the land HDF functions.

!Input Parameters: N/A

!Output Parameters: N/A

!Revision History:
 Revision 00.01  1995/08/15
 Robert E. Wolfe/MODIS Land Team Support
 Original version.

 Revision 00.02 1995/09/10
 Robert E. Wolfe/MODIS Land Team Support
 See 'lhdf.c'.

 Revision 00.03 1995/09/10
 Robert E. Wolfe/MODIS Land Team Support and Paul Fisher/SDST
 Added new file types and fixed prolog.  See 'lhdf.c'. 

 Revision 00.04 1996/04/12
 Paul Fiher/SDST
 Added MODLAND V1 metadata functionality.

!Team-unique Header:
  This software was developed by the MODIS Land Science Team Support 
  Group for the National Aeronautics and Space Administration, Goddard 
  Space Flight Center, under NASA Task 92-012-05.

!References and Credits:
    Written by Robert E. Wolfe
               Hughes STX
               robert.wolfe@gsfc.nasa.gov
               phone: 301-441-4585
               
               Hughes STX
               7701 Greenbelt Road, #400
               Greenbelt, MD 20770  

!Design Notes:

!END*****************************************************************************/

#include "hdf.h"               /* HDF header files */
#include "mfhdf.h"

/* File type ids */

typedef enum {LHDF_RAD, LHDF_GEO,  LHDF_CLOUD, 
	      LHDF_REF, LHDF_FIRE, LHDF_LST, LHDF_SNOW, LHDF_ICE, 
	      LHDF_MAX_TYPE} Lhdf_type_t;

/* File access flags */

typedef enum {LHDF_RDONLY, LHDF_RDWR} Lhdf_access_t;

/* Parameter ids */

typedef enum {
  LHDF_RAD_B1, LHDF_RAD_B2, LHDF_RAD_B3, LHDF_RAD_B4, 
  LHDF_RAD_B5, LHDF_RAD_B6, LHDF_RAD_B7,
  LHDF_RAD_B8, 
  LHDF_RAD_B20, LHDF_RAD_B21, LHDF_RAD_B31,  

  LHDF_LAT, LHDF_LON, LHDF_SA, LHDF_SZ, LHDF_VA, LHDF_VZ, 
  LHDF_RNG, 

  LHDF_CMASK1, LHDF_CMASK2, LHDF_CMASK3, LHDF_CMASK4, 
  LHDF_CMASK5, LHDF_CMASK6, 

  LHDF_REF_B1, LHDF_REF_B2, LHDF_REF_B3, LHDF_REF_B4, 
  LHDF_REF_B5, LHDF_REF_B6, LHDF_REF_B7, 
  LHDF_MVI, LHDF_NDVI, LHDF_QC_250M, LHDF_QC_500M,

  LHDF_FIRE_MASK, LHDF_ENERGY, LHDF_SMOLD, LHDF_FIRE_QC, 

  LHDF_LST_VAL, LHDF_LST_QC, LHDF_LST_ERR, LHDF_EMIS_31, 
  LHDF_EMIS_32, LHDF_EMIS_2X,
  
  LHDF_SNOW_VAL, 

  LHDF_ICE_VAL,

  LHDF_RAD_CAL, 
  LHDF_MAX_PARAM1
} Lhdf_param_t;

#define LHDF_MAX_PARAM (LHDF_MAX_PARAM1 - 1)

/* Return Status */

#define LHDF_SUCCESS 0
#define LHDF_ERROR -1

/* Line Buffer Structure (for the 'RAD' and 'CLD' file types) */

typedef struct {
  int il_prev;		/* Previous line read or written */
  int changed;		/* Buffer modified flag; TRUE = changed */
  size_t sample_size;	/* Size of a sample (bytes) */
  unsigned char *buf;	/* Buffer for all bands */
} Lhdf_line_t; 

/* File Structure */

typedef struct {
  long key;		/* Unique key for the file definition structure */
  char *name;		/* Name of the file */
  Lhdf_type_t file_type; /* File type */
  Lhdf_access_t access;	/* File access mode */
  int sd_fid;		/* SD id */
  long nl, ns;		/* Number of 1km lines and 1km samples per line */
  struct {		/* Parameter information structure */
    int32 id, index;	/*   Parameter HDF id and index */
    int exist, is_select; /*   Parameter exists and selected flags; 
                           *     exist = TRUE = parameter exists in HDF file
                           *     is_select = TRUE = parameter has been 
                           *       selected */
  } param[LHDF_MAX_PARAM1];
  Lhdf_line_t line[4];	/* Line buffers for 'RAD' and 'CLD' file types */
} Lhdf_file_t;

/* Function Prototypes */

/* Create an HDF file */
int Lhdf_Create(const char *filename);

/* Open an existing HDF file for read only access. */
Lhdf_file_t *Lhdf_OpenRead(const char *filename, Lhdf_type_t type,
                          long *nline_1km, long *nsample_1km);
                          
/* Open an existing HDF file for read/write access. */
Lhdf_file_t *Lhdf_OpenWrite(const char *filename, Lhdf_type_t type,
                          long nline_1km, long nsample_1km);

/* Read the data for one line or more lines of an parameter stored in 
 * an HDF file. */
int Lhdf_Get(Lhdf_file_t *file, Lhdf_param_t ip, 
             long iline_1km, long nline_1km, void *ptr);

/* Write the data for one line or more lines of an parameter to an HDF file. */ 
int Lhdf_Put(Lhdf_file_t *file, Lhdf_param_t ip, 
             long iline_1km, long nline_1km, void *ptr);

/* Return the SDS id of a parameter stored in an HDF file. */
int Lhdf_GetSDSid(Lhdf_file_t *file, Lhdf_param_t ip, int32 *id);

/* Get calibration scale and offset for a parameter stored in an HDF file */
int Lhdf_GetCal(Lhdf_file_t *file, Lhdf_param_t ip, 
                double *scale, double *offset);

/* Write calibration scale and offset for a parameter stored to an HDF file */
int Lhdf_PutCal(Lhdf_file_t *file, Lhdf_param_t ip, 
                double scale, double offset);

/* Close an HDF file and free's file data structure */
int Lhdf_Close(Lhdf_file_t *file);

/* V1 Metadata definitions */
/* calibrated_nt value for MOD09 L2 */
#define REFCALNT 5

/* Fill value for MOD09_L2 Ref SDSs */
#define REFFV -100

/* Fill Value for MOD09_L2 QC SDSs */
#define QCFV "NA"

/* Fill Value for MOD09_L2 VI SDSs */
#define VIFV -2000

/* Valid Range for MOD09_L2 REF SDSs */
#define REFVR1 0
#define REFVR2 16100

/* Valid Range for MOD09_L2 QC 250 SDSs */
#define QC250VR1 0
#define QC250VR2 255

/* Valid Range for MOD09_L2 QC 500 SDSs */
#define QC500VR1 0
#define QC500VR2 65535

/* Valid Range for MOD09_l2 VI SDSs */
#define VIVR1 -1000
#define VIVR2 1000

/* nadir data resolution for MOD09_L2 bands 1 and 2 and V1 */
#define NADIR250 "250m"

/* nadir data resolution for MOD09_L2 bands 3-7 and 500m QC */
#define NADIR500 "500m"

/* nadir data resolution for MOD09_L2 250m QC */
#define NADIR250Q "mixed"

/* units for MOD09_L2 reflectance bands */
#define REFUNITS "percent reflectance"

/* units for MOD09_L2 QC SDSs */
#define QCUNITS "bit field"

/* units for VI SDSs for MOD09_L2 */
#define VIUNITS "NDVI"

/* _FillValue for MOD09 L2 500m QC */
#define QC500FV 19580

/* _FillValue for MOD09 L2 250m QC */
#define QC250FV 255

/* possible ranges for data types */
#define INT32H 2147483648
#define INT32L -2147483648
#define INT16H 32767
#define INT16L -32767
#define UINT16H 65535
#define UINT16L 0
#define UINT8H 255
#define UINT8L 0

