/*
!C*****************************************************************************

!Description:
   Main program of MODIS Land-Surface Temperature (LST) Version 5.2.1 for
   removing cloud-contaminated LSTs in MOD11A1 and MYD11A1 products.

!Input Parameters:
   SDS in MOD11A1 or MYD11A1 product.

!Output Parameters:
   SDS in the modified MOD11A1 or MYD11A1 product.

!Revision History:

   $Log: mod11a1_cldscreen.c,v $
   * Version 5.2.2 2005/12/01  22:12     Z. Wan
   *    modify values in lst_lcv_tdt and use f_t_d & f_t_n
   *	set "SCF V5.2.2AS"
   * Version 5.2.1 (initial) 2005/09/20  22:45     Z. Wan
   *    just follow the version # in mod11_lst2A1.c and appended with S
   $Id: mod11a1_cldscreen.c, v5.2.1  2005/09/20  22:45     Z. Wan Exp wan $

!Team-unique Header:
   
   This software was developed by:
   Zhengming Wan for the National Aeronautics and Space Administration,
   Goddard Space Flight Center, under NASA contract NAS5-31370.

   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541
   Programmer/Analyst, Xiaoning Duan
   	duan@icess.ucsb.edu
   	805-893-7353
   ICESS
   University of California
   Santa Barbara, CA 93106-3060

!References and Credits 

This code contains calls to software of the Herarchical Data Format library, 
developed and distributed by, and used with the permission of, the 
National Center for Supercomputing Applications, University of Illinois at
Urbana-Champaign, Champaign, Illinois

!Design Notes

   Toolkits sdptk5.2.4, HDF4.1r1, and MAPI2.3.1 are used.

!END**************************************************************************
*/

#define SMF_SOURCE	"main mod11a1_cldscreen.c"
#define USE_ECS		/* ECS meta data handling */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#include "mapi.h"	/* M-API 2.3.1 */
#include "PGS_SMF.h"
#include "PGS_MODIS_37111.h"

#include "HdfEosDef.h"
#include "PGS_MEM.h"
#include "hntdefs.h"
#include "smfio.h"
#include "bit_flag.h"	   /* bit flags for QC */

#include "lst_1dy_data.h"

/* G. Ye added function DoMetaData that was modified by R. Cember. and Z. Wan */

void DoMetaData( MODFILE *);

/* functions for space allocation and allocated variables */
/* declared as global variables so that they can be freed */
/* both in normal and anormal exits */

char 	   **alloc_2d (int32, int32, size_t);
char 	   ***alloc_3d (int32, int32, int32, size_t);

/* for the 1km LCV */

uint8	**lcv_1km = NULL;

/* for the daily 1km LST */

uint16  ***lst_dt = NULL;
uint16  ***lst_nt = NULL;
uint8   ***QC_dt = NULL;
uint8	***v_time_dt = NULL;
uint8	***v_angl_dt = NULL;
uint8   ***QC_nt = NULL;
uint8	***v_time_nt = NULL;
uint8	***v_angl_nt = NULL;
uint8	***em31_t = NULL;
uint8	***em32_t = NULL;
uint16	***clr_cov_dt = NULL;
uint16	***clr_cov_nt = NULL;

uint16  **lst_d = NULL;
uint16	*lst_d_p;
uint8   **QC_d = NULL;
uint8   *QC_d_p = NULL;
uint8   **v_time_d = NULL;
uint8   *v_time_d_p;
uint8   **v_angl_d = NULL;
uint8   *v_angl_d_p = NULL;
uint16  **lst_n = NULL;
uint16  *lst_n_p;
uint8   **QC_n = NULL;
uint8   *QC_n_p = NULL;
uint8   **v_time_n = NULL;
uint8   *v_time_n_p;
uint8   **v_angl_n = NULL;
uint8   *v_angl_n_p = NULL;
uint8   **em31 = NULL;
uint8   **em32 = NULL;
uint16   **clr_cov_d = NULL;
uint16   **clr_cov_n = NULL;

char msgString[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for use with modsmf */
char LogMsg[PGS_SMF_MAX_MSG_SIZE]={NULL};    /* string for smf status report */
char systemTag[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for systemTag */

ECSattr_names_for_all_handles   HDFattrNames;   /* Meta data attributes */
long int                        NumHandles = 2; /* two master metadata groups*/

#include "PGS_PC.h"

#define PCLogL3_LC		212550  /* MOD12 L3 Lancover input */
#define PCLogL3_1km             211031  /* MOD11 L3 LST 1km input/output */
#define MCF_FILE_L3_1KM         211253  /* MCF for the L3 1km LST file */

#define BOTH "Both"

#ifdef USE_ECS          /* ECS meta data handling (Paul Fisher/YTLau) */

#include "PGS_MET.h"
PGSt_MET_all_handles            mdhandles;      /* Meta data handles */

/* ECS metadata string and variable names */
#define CORES		"CoreMetadata.0"
#define ARCHIVES_IN	"ArchiveMetadata.0"
#define ARCHIVES_OUT	"ArchiveMetadata.0"

/* the following three defines must be defined in relation to one another */
#define METAHANDLES mdhandles
#define COREHANDLES mdhandles[1]
#define ARCHIVEHANDLES mdhandles[2]

/* default value for QAPERCENTINTERPOLATEDDATA */
#define QAINTDATA 0
/* default value for QAPERCENTMISSINGDATA */
#define QAMISDATA 0
/* default value for QAPERCENTOUTOFBOUNDSDATA */
#define QAOBDATA 0

#endif  /* end USE_ECS */

void Error(const char *, const char *); 

void	free_lst_space(void);
void	zeroout_general(void *, int32, int32);

int32	total_space = 0;

#define ReprocessingPlanned 800503       /* its common LUN number */
#define ReprocessingActual  800504       /* its common LUN number */
#define ScienceQualityFlagExplanation  800506   /* it's a LUN number */
#define ProcessingEnvironment  800550    /* it's a common LUN number */
#define PGE16_Version      800500       /* run-time defined PGE16 Version  */

char	pgeversion[64];
 
char	rep_planned[PGSd_PC_VALUE_LENGTH_MAX];
char	rep_actual[PGSd_PC_VALUE_LENGTH_MAX];
char	sci_qua_exp[PGSd_PC_VALUE_LENGTH_MAX];
char    process_env[PGSd_PC_VALUE_LENGTH_MAX];

/*Next variable added for use in setting PGEVersion*/
char    pge_version[PGSd_PC_VALUE_LENGTH_MAX];

char    *ShortNameVal;
char	localgranuleId_vers_num[PGSd_PC_VALUE_LENGTH_MAX];
int32	vers_id_num = 5;

/* variables for reading input file */

char	   modstring[MAX_NC_NAME] = {0};	/* used in complex messages */
char	   data_type[DATATYPELENMAX];
long int sta_lc[2] = { 0, 0 };
long int sta_1km[2] = { 0, 0 };
long int dims_1km[2] = { 1200, 1200 };
int32	   i_esdt, n_esdt = NSD_LST_1KM_L3;
int32    sd_id, attr_index;

#define modsmf_id_vers(i1, i2)	{ \
	sprintf (modstring, "Logical Id %d Version %d", i1, i2); \
	modsmf (MODIS_F_GETREF_FAIL, modstring, "main, mod11a1_cldscreen.c"); \
	free_lst_space();	exit (1); }

  /* used for getting information from Core and Archive Attributes of
     the daily CMG LST product file.
     We define strings with enough sizes */

char	HaltMsg[2048];
char	smf_message[PGS_SMF_MAX_MSGBUF_SIZE]={0};

#define HALTsmf(mnemon, a)      { \
	PGS_SMF_GetMsgByCode(mnemon, smf_message); \
	sprintf(HaltMsg,"%s[%d]: %s %s", __FILE__, __LINE__, smf_message, a);\
	PGS_SMF_GenerateStatusReport(HaltMsg); \
	free_lst_space(); exit(1);   }

#define	   SATELLITE			"A"
#define	   VERSION			"005"

#define    N_BYTES_CORE_L3		128000
#define    N_BYTES_ARCH_L3		256000
char	   core_L3[N_BYTES_CORE_L3];
char	   arch_L3[N_BYTES_ARCH_L3];

int32 get_meta_attr(char *s, char *p, int32 type, int32 num, char *d);

#define rm_path(s, p)	\
  { p = ( s[0] == '/' || s[0] == '.') ? strrchr ( s, '/' ) + 1 : s; } 

void SMFLOG(char *a);

/* R. Cember: for calls to MAPI function getMODISfileinfo() */
char sd_attr_type[MAX_NC_ATTRS];
char sd_attr_name[MAX_NC_NAME];
long nelements;

/* R. Cember: add declaration. */
char trash_char[5];

char                  HaltMsg[2048];
#define HALT(a)       { sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, a);\
			PGS_SMF_GenerateStatusReport(HaltMsg);\
			free_lst_space(); exit(1); }

PGSt_SMF_status      current_time_a(char *);

void          *local = NULL;

#define MAXINPUTFILES	32

MODFILE *lst_1d_inf[MAXINPUTFILES];
MODFILE *lst_inf = NULL;
MODFILE *lcv_inf = NULL;

int32	days_sequence[MAXINPUTFILES];
int32	i_day_1st = 1;
char	fname_1d[MAX_NC_NAME] = {NULL};

PGSt_integer    Vers = 1, Vers_tmp = 1;

int32	total_l3_1km[4] = { 0, 0, 0, 0 };

char			DayNightFlag[32];
enum AutoQualFlag	AutomaticQualityFlag = PASSED;
MET_counts		met_cnt_1km;
MET_counts		*met_cnt;

char	CharVals[10][PGSd_PC_VALUE_LENGTH_MAX];

PGSt_integer numfiles_in;
int32	ndays;

char	*CharStg2[] = {
	 "RANGEBEGINNINGDATE",
	 "RANGEBEGINNINGTIME",
	 "RANGEENDINGDATE",
	 "RANGEENDINGTIME",
	 "EXCLUSIONGRINGFLAG.1",
	 "EQUATORCROSSINGDATE.1",
	 "EQUATORCROSSINGTIME.1",
	 "OPERATIONMODE",
	 "PGEVERSION",
	 "ADDITIONALATTRIBUTENAME.1" };

char	date_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char	date_end[2][PGSd_PC_VALUE_LENGTH_MAX];
char	time_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char	time_end[2][PGSd_PC_VALUE_LENGTH_MAX];
char	ch_tileid[128] = {NULL};

int32	i_date_begin, i_date_end;

char	MoreCodeStgVals[4][PGSd_PC_VALUE_LENGTH_MAX];

char	cyear[5] = "2000\0";
char	cdoy[4] = "001\0";

char	esdt_1km_L3[8] = "MOD11A1";
char	*esdt_tmp;

char  a_ih_1km[PGSd_PC_VALUE_LENGTH_MAX];
char  a_iv_1km[PGSd_PC_VALUE_LENGTH_MAX];
int32 ih_1km, iv_1km;
int32 ih_1km_n, iv_1km_n;

float64 ul[2], lr[2];

main(int32 argc, char **argv)
{
   char		fname[128];
   char		f_name[128], cmd_line[128];
   char		*local2;
   size_t	siz = 0;

   char	 fname_part2[32];
   char	 cld_screened[8];
   char	 fname_lcv[128];
   int32 i_doy_in, i_doy_1st = 366;
   int32 t_nrow_1km = 1200;
   int32 t_ncol_1km = 1200;
   int32 t_nrow = 1200;
   int32 t_ncol = 1200;
   int32 i_row, j_col;
   int32 i = 0, j = 0;
   int32 i_screened[32];
   int32 n_not_screened = 0;

   int32 i_insert, i_day;
   int32 irow, returnStatus;
   int32 ii, k = 0;

   uint8	u8_0 = 0, u8_1 = 1, u8_2 = 2, u8_3 = 3;
   uint8	u8_0cons = 0, u8_1cons = 1, u8_2cons = 2, u8_3cons = 3;

   int32        nn, n;
   uint16	u16_dlst, u16_nlst;
   uint8	u8_em31, u8_em32;
   uint8	u8_qa_d, u8_vtime_d, u8_vangl_d;
   uint8	u8_qa_n, u8_vtime_n, u8_vangl_n;
   uint8	u8_qa_man_d, u8_qa_dat_d, u8_qa_lst_d, u8_qa_em_d;
   uint8	u8_qa_man_n, u8_qa_dat_n, u8_qa_lst_n, u8_qa_em_n;
   uint8	k_qa_man_d, k_qa_dat_d, k_qa_lst_d, k_qa_em_d;
   uint8	k_qa_man_n, k_qa_dat_n, k_qa_lst_n, k_qa_em_n;
   uint8	u8_qa_man, u8_qa_lst;

   uint8	lcv, u8_lcv = 037;
   uint8	lcv_snow = 15, is_snow;
   float32	lst_lcv_tdt[18] = {	/* in unit of 0.02K */
		 150,	/* water 0 */
		 380,	/* evergreen needleleaf forest 1 */
		 360,	/* evergreen broadleaf forest 2 */
		 360,	/* (green & senescent) deciduous needleleaf forest 3 */
		 350,	/* (green & senescent) deciduous broadleaf forest 4 */
		 350,	/* (green & senescent) mixed forest 5 */
		 400,	/* (green & senescent) closed shrublands 6 */
		 450,	/* (green & senescent) open shrublands 7 */
		 420,	/* (green & senescent) woody savannas 8 */
		 450,	/* (green & senescent) savannas 9 */
		 500,	/* (green & senescent) grasslands 10 */
		 250,	/* permanent wetlands 11 */
		 400,	/* (green & senescent) croplands 12 */
		 500,	/* (green & senescent) urban and built-up 13 */
		 400,	/* (green & senescent) cropland and mosaics 14 */
		 200,	/* snow and ice 15 */
		 500,	/* bare soil and rocks 16 */
		 500 	/* unclassified 17 */	};
   float64	lcv_dlst_avg[19][4], lcv_nlst_avg[19][4], lcv_dlst_max[19], lcv_nlst_max[19];
   float64	lcv_dlst_num[19][5], lcv_nlst_num[19][5];
   float64	lcv_dlst_std[19][4], lcv_nlst_std[19][4];
   float64	lcv_dlst_f_t[19][4], lcv_nlst_f_t[19][4];
   float64	lcv_dlst_f_c[19][4], lcv_nlst_f_c[19][4];
   float64	lcv_dlst_16d_f_t[19][4], lcv_nlst_16d_f_t[19][4];
   int32	iday_dk[MAXINPUTFILES];		/* dlst should be kept */
   int32	iday_nk[MAXINPUTFILES];		/* nlst should be kept */
   int32	i_yr[MAXINPUTFILES];
   int32	j_day[MAXINPUTFILES], c_day[MAXINPUTFILES];
   int32	n8_d[4], n8_n[4];
   float32	dlst, nlst;
   float32	dt_tempo, dt4_tempo, dt3_tempo, dt2_tempo;
   float32	tmd_max, t16d_max[2], t8d_max[4], t8d_avg[4];
   float32	tmn_max, t16n_max[2], t8n_max[4], t8n_avg[4];
   int32	dlst_min_1[32], dlst_max_1[32], n_dlst_1[32];
   int32	nlst_min_1[32], nlst_max_1[32], n_nlst_1[32];
   int32	dlst_min_2[32], dlst_max_2[32], n_dlst_2[32];
   int32	nlst_min_2[32], nlst_max_2[32], n_nlst_2[32];
   float64	dlst_avg_1[32], nlst_avg_1[32], dlst_avg_2[32], nlst_avg_2[32];
   float64	pct_grids_tile, tmp_d, f_t_d, f_t_n, f_c_d, f_c_n;
   char		granule_id[64];
   int32	id_vers, i_yr1, i_yr2, n8, n16;

   /* to hold the PROCESSINGDATETIME metadata field */
   char protime[PGSd_PC_VALUE_LENGTH_MAX]="Error in PROCESSINGDATETIME";

   pct_grids_tile = 100.0 / (1200 * 1200);

   for ( i = 0; i < 32; i++ ) {
	dlst_max_1[i] = dlst_avg_1[i] = n_dlst_1[i] = 0;  dlst_min_1[i] = 100000000;
	nlst_max_1[i] = nlst_avg_1[i] = n_nlst_1[i] = 0;  nlst_min_1[i] = 100000000;
	dlst_max_2[i] = dlst_avg_2[i] = n_dlst_2[i] = 0;  dlst_min_2[i] = 100000000;
	nlst_max_2[i] = nlst_avg_2[i] = n_nlst_2[i] = 0;  nlst_min_2[i] = 100000000;
   }

   /* set callerID first to get rid of the error message to LogStatus */
   PGS_SMF_SetCallerID(PGSd_CALLERID_SMF);

   /* get PROCESSINGDATETIME */
   if(current_time_a(protime) != MODIS_S_SUCCESS) {
	sprintf(modstring,
		"the PROCESSINGDATETIME metadata field will not be written correctly.");
	HALT(modstring);
   }

   if (PGS_PC_GetConfigData(PGE16_Version, pgeversion) != PGS_S_SUCCESS) {
	sprintf(modstring,"no PGE16_Version (LUN 800500) in PCF");
	HALT(modstring);
   }

   if (PGS_PC_GetConfigData(ReprocessingPlanned, rep_planned) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingPlanned info from pcf");

   if (PGS_PC_GetConfigData(ReprocessingActual, rep_actual) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingActual info from pcf");

   if (PGS_PC_GetConfigData(ScienceQualityFlagExplanation, sci_qua_exp) != PGS_S_SUCCESS)
	SMFLOG("Error getting ScienceQualityFlagExplanation info from pcf");

   if (PGS_PC_GetConfigData(ProcessingEnvironment, process_env) != PGS_S_SUCCESS)
	SMFLOG("Error getting ProcessingEnvironment info from pcf");

   siz = sizeof (uint8);
   lcv_1km = (uint8 **) alloc_2d (t_nrow_1km, t_ncol_1km, siz);

   Vers = 1;
   Vers_tmp = Vers;
   if (PGS_PC_GetReference(PCLogL3_LC, &Vers_tmp, fname_lcv) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL3_LC, Vers);

   if ( (lcv_inf = openMODISfile (fname_lcv, "r")) == NULL ) {
        sprintf(modstring, "cannot open %s\n", fname_lcv);
        HALT(modstring);
   }

   if ( getMODISarray(lcv_inf, land_cv[0].name, land_cv[0].gname,
		sta_lc, land_cv[0].dims, &lcv_1km[0][0]) == -1)
	 HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray Land_Cover_Type_1");
   closeMODISfile (&lcv_inf);

   for ( i = 0; i < t_nrow; i++ ) {
	for ( j = 0; j < t_ncol; j++ ) {
		lcv = lcv_1km[i][j];
		if ( lcv >= 254 )	lcv_1km[i][j] = 17;
	}
   }

   if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLogL3_1km, &numfiles_in)) !=  PGS_S_SUCCESS )
	SMFLOG("Error getting number of MOD11A1 files");

   if ( numfiles_in >  MAXINPUTFILES )
	SMFLOG("STOP--too many MOD11A1 inputs");

/*fprintf(stderr, "numfiles_in %d\n", numfiles_in); */

   for ( ii = 0; ii < numfiles_in; ii++ ) {

    Vers = ii + 1;
    Vers_tmp = Vers;
    if (PGS_PC_GetReference(PCLogL3_1km, &Vers_tmp, fname_1d) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL3_1km, Vers);

    /* open the input daily L3 files */
    if((lst_inf = openMODISfile (fname_1d, "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_1d);

    sprintf(sd_attr_name, CORES);
    sprintf(sd_attr_type, "char *");
    nelements = (long) N_BYTES_CORE_L3;
    if(getMODISfileinfo(lst_inf,sd_attr_name,sd_attr_type,&nelements,
		core_L3) != MAPIOK)
	 Error("Re: lst_1km_l3_outf: failure on call to ", "getMODISfileinfo");

    granule_id[0] = '\0';
    local2 = granule_id;
    if (get_meta_attr(core_L3, "LOCALGRANULEID", 1, 1, (char *) local2 ) == FAIL )
	HALT("get_meta_attr for LOCALGRANULEID fails on core_L3");
    sscanf(&granule_id[9], "%4d", &i_yr[ii]);
    sscanf(&granule_id[13], "%3d", &j_day[ii]);
    sscanf(&granule_id[26], "%1d", &id_vers);
/*fprintf(stderr, "%s %4d %3d id_vers %d\n", granule_id, i_yr[ii], j_day[ii], id_vers);*/

    a_ih_1km[0] = '\0';
    local2 = a_ih_1km;
    if (get_meta_attr(core_L3, "PARAMETERVALUE.6", 1, 1, (char *) local2 ) == FAIL )
	HALT("get_meta_attr for HORIZONTALTILENUMBER fails on core_L3");
    if ( ii == 0 )	ih_1km = (uint16) atoi (a_ih_1km);
    else		ih_1km_n = (uint16) atoi (a_ih_1km);

    a_iv_1km[0] = '\0';
    local2 = a_iv_1km;
    if (get_meta_attr(core_L3, "PARAMETERVALUE.7", 1, 1, (char *) local2 ) == FAIL )
	HALT("get_meta_attr for VERTICALTILENUMBER fails on core_L3");
    if ( ii == 0 )	iv_1km = (uint16) atoi (a_iv_1km);
    else		iv_1km_n = (uint16) atoi (a_iv_1km);

    if ( ii && ( ih_1km_n != ih_1km || iv_1km_n != iv_1km ) )
	SMFLOG("STOP--different ih iv numbers found in the input files");

  if ( id_vers == 5 ) {
    cld_screened[0] = '\0';
    if(get_meta_attr(core_L3, "PARAMETERVALUE.13", 1, 1,(char *) &cld_screened[0] ) == FAIL )
	HALT("get_meta_attr for CLOUD_CONTAMINATED_LST_SCREENED fails on core_L3");

    if ( !strcmp(cld_screened, "NO") ) {
	i_screened[ii] = 0;
	n_not_screened += 1;
    }
    else
	i_screened[ii] = 1;
  }
  else {
	i_screened[ii] = 0;
	n_not_screened += 1;
  }

/*fprintf(stderr, "%s %s %s %s %d\n", fname_1d, a_ih_1km, a_iv_1km, cld_screened, i_screened[ii]); */

    lst_1d_inf[ii] = lst_inf;

   }

   if ( !n_not_screened )
	SMFLOG("STOP--all MOD11A1 inputs already screened");

   ndays = numfiles_in;

   i_yr1 = i_yr[0];	i_yr2 = 0;
   for ( k = 1; k < ndays; k++ )
	if ( i_yr[k] < i_yr1 )	i_yr1 = i_yr[k];
   for ( k = 0; k < ndays; k++ ) {
	c_day[k] = j_day[k];
	if ( i_yr[k] != i_yr1 )	i_yr2 = i_yr[k];
   }
   if ( i_yr2 && i_yr2 != i_yr1 ) {
	if ( i_yr1 % 4 == 0 && i_yr1 % 100 != 0 || i_yr1 % 400 == 0 )	i_doy_1st = 366;
	else	i_doy_1st = 365;
	for ( k = 0; k < ndays; k++ )
		if ( i_yr[k] == i_yr2 )		c_day[k] += i_doy_1st;
   }
   i_doy_1st = 366;
   for ( k = 0; k < ndays; k++ )
	if ( c_day[k] < i_doy_1st )	i_doy_1st = c_day[k];
   for ( k = 0; k < ndays; k++ )
	days_sequence[k] = c_day[k] - i_doy_1st;

   if ( days_sequence[ndays-1] >= MAXINPUTFILES )
	SMFLOG("STOP--the input data days are out of the period of 32 days");

/*
   for ( k = 0; k < ndays; k++ )
	fprintf(stderr, "days_sequence[%02d] %2d\n", k, days_sequence[k]);
fprintf(stderr, "total %d input files\n", numfiles_in); fflush(stderr);
*/

   /* for daily 1km LST */
   siz = sizeof (uint16);
   lst_dt = (uint16 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   lst_nt = (uint16 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   clr_cov_dt = (uint16 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   clr_cov_nt = (uint16 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   siz = sizeof (uint8);
   QC_dt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   v_time_dt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   v_angl_dt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   QC_nt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   v_time_nt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   v_angl_nt = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   em31_t = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);
   em32_t = (uint8 ***) alloc_3d(ndays,t_nrow,t_ncol, siz);

/*fprintf(stderr, "total_space %d\n", total_space); */

   /* input the daily LST data */
   for ( ii = 0; ii < numfiles_in; ii++ ) {

/*fprintf(stderr,"reading from lst_1d_inf[%d] %d\n", ii, lst_1d_inf[ii]); */

     lst_d = lst_dt[ii];
     lst_n = lst_nt[ii];
     QC_d = QC_dt[ii];
     v_time_d = v_time_dt[ii];
     v_angl_d = v_angl_dt[ii];
     QC_n = QC_nt[ii];
     v_time_n = v_time_nt[ii];
     v_angl_n = v_angl_nt[ii];
     em31 = em31_t[ii];
     em32 = em32_t[ii];
     clr_cov_d = clr_cov_dt[ii];
     clr_cov_n = clr_cov_nt[ii];

     getMODISarray(lst_1d_inf[ii], lst_1km_l3[0].name,
	lst_1km_l3[0].gname, sta_1km, dims_1km, &lst_d[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[4].name,
	lst_1km_l3[4].gname, sta_1km, dims_1km, &lst_n[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[1].name,
	lst_1km_l3[1].gname, sta_1km, dims_1km, &QC_d[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[2].name,
	lst_1km_l3[2].gname, sta_1km, dims_1km, &v_time_d[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[3].name,
	lst_1km_l3[3].gname, sta_1km, dims_1km, &v_angl_d[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[5].name,
	lst_1km_l3[5].gname, sta_1km, dims_1km, &QC_n[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[6].name,
	lst_1km_l3[6].gname, sta_1km, dims_1km, &v_time_n[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[7].name,
	lst_1km_l3[7].gname, sta_1km, dims_1km, &v_angl_n[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[8].name,
	lst_1km_l3[8].gname, sta_1km, dims_1km, &em31[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[9].name,
	lst_1km_l3[9].gname, sta_1km, dims_1km, &em32[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[10].name,
	lst_1km_l3[10].gname, sta_1km, dims_1km, &clr_cov_d[0][0]);
     getMODISarray(lst_1d_inf[ii], lst_1km_l3[11].name,
	lst_1km_l3[11].gname, sta_1km, dims_1km, &clr_cov_n[0][0]);

    }

    for ( i = 0; i <= 18; i++ ) {
	lcv_dlst_avg[i][0] = lcv_dlst_avg[i][1] = 0.0;
	lcv_dlst_avg[i][2] = lcv_dlst_avg[i][3] = 0.0;
	lcv_dlst_std[i][0] = lcv_dlst_std[i][1] = 0.0;
	lcv_dlst_std[i][2] = lcv_dlst_std[i][3] = 0.0;
	lcv_dlst_num[i][0] = lcv_dlst_num[i][1] = 0.0;
	lcv_dlst_num[i][2] = lcv_dlst_num[i][3] = lcv_dlst_num[i][4] =0.0;
	lcv_dlst_f_t[i][0] = lcv_dlst_f_t[i][1] = 0.0;
	lcv_dlst_f_t[i][2] = lcv_dlst_f_t[i][3] = 0.0;
	lcv_dlst_16d_f_t[i][0] = lcv_dlst_16d_f_t[i][1] = 0.0;
	lcv_dlst_16d_f_t[i][2] = lcv_dlst_16d_f_t[i][3] = 0.0;
	lcv_dlst_f_c[i][0] = lcv_dlst_f_c[i][1] = 0.0;
	lcv_dlst_f_c[i][2] = lcv_dlst_f_c[i][3] = 0.0;
	lcv_nlst_avg[i][0] = lcv_nlst_avg[i][1] = 0.0;
	lcv_nlst_avg[i][2] = lcv_nlst_avg[i][3] = 0.0;
	lcv_nlst_std[i][0] = lcv_nlst_std[i][1] = 0.0;
	lcv_nlst_std[i][2] = lcv_nlst_std[i][3] = 0.0;
	lcv_nlst_num[i][0] = lcv_nlst_num[i][1] = 0.0;
	lcv_nlst_num[i][2] = lcv_nlst_num[i][3] = lcv_nlst_num[i][4] = 0.0;
	lcv_nlst_f_t[i][0] = lcv_nlst_f_t[i][1] = 0.0;
	lcv_nlst_f_t[i][2] = lcv_nlst_f_t[i][3] = 0.0;
	lcv_nlst_16d_f_t[i][0] = lcv_nlst_16d_f_t[i][1] = 0.0;
	lcv_nlst_16d_f_t[i][2] = lcv_nlst_16d_f_t[i][3] = 0.0;
	lcv_nlst_f_c[i][0] = lcv_nlst_f_c[i][1] = 0.0;
	lcv_nlst_f_c[i][2] = lcv_nlst_f_c[i][3] = 0.0;
	lcv_dlst_max[i] = lcv_nlst_max[i] = 0.0;
    }

    for ( i_row = 0; i_row < t_nrow; i_row++ ) {
     for ( j = 0; j < t_ncol; j++ ) {

       lcv = lcv_1km[i_row][j] & u8_lcv;

	for ( k = 0; k < ndays; k++ ) {
        	lst_d = lst_dt[k];
        	lst_n = lst_nt[k];
        	dlst = lst_d[i_row][j];
        	nlst = lst_n[i_row][j];
		if ( dlst > 0.0 && dlst < 17200. ) {
			n = days_sequence[k] / 8;
			lcv_dlst_avg[lcv][n] += dlst;
			lcv_dlst_num[lcv][n] += 1;
			lcv_dlst_num[lcv][4] += 1;
			lcv_dlst_avg[18][n] += dlst;
			lcv_dlst_num[18][n] += 1;
			lcv_dlst_num[18][4] += 1;
		}
		if ( nlst > 0.0 && nlst < 17000. ) {
			n = days_sequence[k] / 8;
			lcv_nlst_avg[lcv][n] += nlst;
			lcv_nlst_num[lcv][n] += 1;
			lcv_nlst_num[lcv][4] += 1;
			lcv_nlst_avg[18][n] += nlst;
			lcv_nlst_num[18][n] += 1;
			lcv_nlst_num[18][4] += 1;
		}
	}
     }
    }
    for ( i = 0; i <= 18; i++ ) {
      for ( j = 0; j < 4; j++ ) {
	if ( lcv_dlst_num[i][j] > 0.0 ) {
		lcv_dlst_avg[i][j] /= lcv_dlst_num[i][j];
		if ( lcv_dlst_avg[i][j] > lcv_dlst_max[i] )
			lcv_dlst_max[i] = lcv_dlst_avg[i][j];
	}
	if ( lcv_nlst_num[i][j] > 0.0 ) {
		lcv_nlst_avg[i][j] /= lcv_nlst_num[i][j];
		if ( lcv_nlst_avg[i][j] > lcv_nlst_max[i] )
			lcv_nlst_max[i] = lcv_nlst_avg[i][j];
	}
      }
    }

    for ( i_row = 0; i_row < t_nrow; i_row++ ) {
     for ( j = 0; j < t_ncol; j++ ) {

       lcv = lcv_1km[i_row][j] & u8_lcv;

	for ( k = 0; k < ndays; k++ ) {
        	lst_d = lst_dt[k];
        	lst_n = lst_nt[k];
        	dlst = lst_d[i_row][j];
        	nlst = lst_n[i_row][j];
		if ( dlst > 0.0 && dlst < 17200. ) {
			n = days_sequence[k] / 8;
			tmp_d = dlst - lcv_dlst_avg[lcv][n];
			tmp_d *= tmp_d;
			lcv_dlst_std[lcv][n] += tmp_d;
			lcv_dlst_std[18][n] += tmp_d;
		}
		if ( nlst > 0.0 && nlst < 17000. ) {
			n = days_sequence[k] / 8;
			tmp_d = nlst - lcv_nlst_avg[lcv][n];
			tmp_d *= tmp_d;
			lcv_nlst_std[lcv][n] += tmp_d;
			lcv_nlst_std[18][n] += tmp_d;
		}
	}
     }
    }
    for ( i = 0; i <= 18; i++ ) {
      for ( j = 0; j < 4; j++ ) {
        if ( lcv_dlst_num[i][j] > 0.0 ) {
		tmp_d = lcv_dlst_std[i][j] / lcv_dlst_num[i][j];
		lcv_dlst_std[i][j] = sqrt(tmp_d);
		lcv_dlst_f_t[i][j] = lcv_dlst_max[i] - lcv_dlst_avg[i][j];
		if ( lcv_dlst_avg[i][j] < 13600. )
		   lcv_dlst_f_c[i][j] = (13600.0 - lcv_dlst_avg[i][j])/5.0;
		if ( lcv_dlst_f_c[i][j] > 1000.0 )	lcv_dlst_f_c[i][j] = 1000.0;
	}

        if ( lcv_nlst_num[i][j] > 0.0 ) {
		tmp_d = lcv_nlst_std[i][j] / lcv_nlst_num[i][j];
		lcv_nlst_std[i][j] = sqrt(tmp_d);
		lcv_nlst_f_t[i][j] = lcv_nlst_max[i] - lcv_nlst_avg[i][j];
                if ( lcv_nlst_avg[i][j] < 13600. )
                   lcv_nlst_f_c[i][j] = (13600.0 - lcv_nlst_avg[i][j])/5.0;
                if ( lcv_nlst_f_c[i][j] > 1000.0 )	lcv_nlst_f_c[i][j] = 1000.0;
        }
      }
    }
    for ( i = 0; i <= 18; i++ )
	lcv_dlst_max[i] = lcv_nlst_max[i] = 0.0;
    for ( i = 0; i <= 18; i++ ) {
	for ( j = 0; j < 2; j++ )
		if ( lcv_dlst_avg[i][j] > lcv_dlst_max[i] )	lcv_dlst_max[i] = lcv_dlst_avg[i][j];
	for ( j = 0; j < 2; j++ )
	    if ( lcv_dlst_num[i][j] )
		lcv_dlst_16d_f_t[i][j] = lcv_dlst_max[i] - lcv_dlst_avg[i][j];
	for ( j = 0; j < 2; j++ )
		if ( lcv_nlst_avg[i][j] > lcv_nlst_max[i] )	lcv_nlst_max[i] = lcv_nlst_avg[i][j];
	for ( j = 0; j < 2; j++ )
	    if ( lcv_nlst_num[i][j] )
		lcv_nlst_16d_f_t[i][j] = lcv_nlst_max[i] - lcv_nlst_avg[i][j];
    }
    for ( i = 0; i <= 18; i++ )
	lcv_dlst_max[i] = lcv_nlst_max[i] = 0.0;
    for ( i = 0; i <= 18; i++ ) {
	for ( j = 2; j < 4; j++ )
		if ( lcv_dlst_avg[i][j] > lcv_dlst_max[i] )	lcv_dlst_max[i] = lcv_dlst_avg[i][j];
	for ( j = 2; j < 4; j++ )
	    if ( lcv_dlst_num[i][j] )
		lcv_dlst_16d_f_t[i][j] = lcv_dlst_max[i] - lcv_dlst_avg[i][j];
	for ( j = 2; j < 4; j++ )
		if ( lcv_nlst_avg[i][j] > lcv_nlst_max[i] )	lcv_nlst_max[i] = lcv_nlst_avg[i][j];
	for ( j = 2; j < 4; j++ )
	    if ( lcv_nlst_num[i][j] )
		lcv_nlst_16d_f_t[i][j] = lcv_nlst_max[i] - lcv_nlst_avg[i][j];
    }
    for ( i = 0; i < 18; i++ ) {
	if ( lcv_dlst_num[i][4] > 0. && lcv_dlst_num[i][4] < 1000. ) {
	   for ( j = 0; j < 4; j++ ) {
		lcv_dlst_f_t[i][j] = lcv_dlst_f_t[18][j];
		lcv_dlst_16d_f_t[i][j] = lcv_dlst_16d_f_t[18][j];
		lcv_dlst_f_c[i][j] = lcv_dlst_f_c[18][j];
	   }
	}
	if ( lcv_nlst_num[i][4] > 0. && lcv_nlst_num[i][4] < 1000. ) {
	   for ( j = 0; j < 4; j++ ) {
		lcv_nlst_f_t[i][j] = lcv_nlst_f_t[18][j];
		lcv_nlst_16d_f_t[i][j] = lcv_nlst_16d_f_t[18][j];
		lcv_nlst_f_c[i][j] = lcv_nlst_f_c[18][j];
	   }
	}
    }
/*
    for ( i = 0; i <= 18; i++ ) {

printf("%2d %5.1f %5.1f %5.1f %5.1f %7.0f %7.0f %7.0f %7.0f %8.0f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f\n", i,
	0.02 * lcv_dlst_avg[i][0], 0.02 * lcv_dlst_avg[i][1],
	0.02 * lcv_dlst_avg[i][2], 0.02 * lcv_dlst_avg[i][3],
	lcv_dlst_num[i][0], lcv_dlst_num[i][1],
	lcv_dlst_num[i][2], lcv_dlst_num[i][3], lcv_dlst_num[i][4],
	0.02 * lcv_dlst_f_t[i][0], 0.02 * lcv_dlst_f_t[i][1],
	0.02 * lcv_dlst_f_t[i][2], 0.02 * lcv_dlst_f_t[i][3],
	0.02 * lcv_dlst_16d_f_t[i][0], 0.02 * lcv_dlst_16d_f_t[i][1],
	0.02 * lcv_dlst_16d_f_t[i][2], 0.02 * lcv_dlst_16d_f_t[i][3],
	0.02 * lcv_dlst_f_c[i][0], 0.02 * lcv_dlst_f_c[i][1],
	0.02 * lcv_dlst_f_c[i][2], 0.02 * lcv_dlst_f_c[i][3]);
printf("   %5.1f %5.1f %5.1f %5.1f %7.0f %7.0f %7.0f %7.0f %8.0f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f %4.1f\n",
	0.02 * lcv_nlst_avg[i][0], 0.02 * lcv_nlst_avg[i][1],
	0.02 * lcv_nlst_avg[i][2], 0.02 * lcv_nlst_avg[i][3],
	lcv_nlst_num[i][0], lcv_nlst_num[i][1],
	lcv_nlst_num[i][2], lcv_nlst_num[i][3], lcv_nlst_num[i][4],
	0.02 * lcv_nlst_f_t[i][0], 0.02 * lcv_nlst_f_t[i][1],
	0.02 * lcv_nlst_f_t[i][2], 0.02 * lcv_nlst_f_t[i][3],
	0.02 * lcv_nlst_16d_f_t[i][0], 0.02 * lcv_nlst_16d_f_t[i][1],
	0.02 * lcv_nlst_16d_f_t[i][2], 0.02 * lcv_nlst_16d_f_t[i][3],
	0.02 * lcv_nlst_f_c[i][0], 0.02 * lcv_nlst_f_c[i][1],
	0.02 * lcv_nlst_f_c[i][2], 0.02 * lcv_nlst_f_c[i][3]);
    }
*/

    for ( i_row = 0; i_row < t_nrow; i_row++ ) {

	    for ( j = 0; j < t_ncol; j++ ) {

       lcv = lcv_1km[i_row][j] & u8_lcv;

       dt_tempo = lst_lcv_tdt[lcv];

       is_snow = (lcv == lcv_snow) ? 1 : 0;

       dt2_tempo = 2.0 * dt_tempo;
       dt3_tempo = 3.0 * dt_tempo;
       dt4_tempo = 4.0 * dt_tempo;

       tmd_max = 0.0;	t16d_max[0] = t16d_max[1] = 0.0;
       t8d_max[0] = t8d_max[1] = 0.0;	t8d_max[2] = t8d_max[3] = 0.0;
       t8d_avg[0] = t8d_avg[1] = 0.0;	t8d_avg[2] = t8d_avg[3] = 0.0;
       n8_d[0] = n8_d[1] = 0.0;	n8_d[2] = n8_d[3] = 0.0;
       tmn_max = 0.0;	t16n_max[0] = t16n_max[1] = 0.0;
       t8n_max[0] = t8n_max[1] = 0.0;	t8n_max[2] = t8n_max[3] = 0.0;
       t8n_avg[0] = t8n_avg[1] = 0.0;	t8n_avg[2] = t8n_avg[3] = 0.0;
       n8_n[0] = n8_n[1] = 0.0;	n8_n[2] = n8_n[3] = 0.0;

       for ( k = 0; k < ndays; k++ ) {

	iday_dk[k] = 0;
	iday_nk[k] = 0;
	lst_d = lst_dt[k];
	lst_n = lst_nt[k];
	dlst = lst_d[i_row][j];
	nlst = lst_n[i_row][j];

	n8 = days_sequence[k] / 8;
	n16 = days_sequence[k] / 16;

	if ( dlst > 0.0 ) {

	   if ( dlst > dlst_max_1[k] )  dlst_max_1[k] = dlst;
	   if ( dlst < dlst_min_1[k] )  dlst_min_1[k] = dlst;
	   dlst_avg_1[k] += dlst;
	   n_dlst_1[k] += 1;

	   iday_dk[k] = days_sequence[k] + 1;
	   if ( dlst > 17200. )		iday_dk[k] = 0;
	   else {
	   	if ( dlst > tmd_max )		tmd_max = dlst;
	   	if ( dlst > t16d_max[n16] )	t16d_max[n16] = dlst;
	   	if ( dlst > t8d_max[n8] )	t8d_max[n8] = dlst;
	   }
        }
        if ( nlst > 0.0 ) {

	   if ( nlst > nlst_max_1[k] )  nlst_max_1[k] = nlst;
	   if ( nlst < nlst_min_1[k] )  nlst_min_1[k] = nlst;
	   nlst_avg_1[k] += nlst;
	   n_nlst_1[k] += 1;

	   iday_nk[k] = days_sequence[k] + 1;
	   if ( nlst > 17000. )		iday_nk[k] = 0;
	   else {
	   	if ( nlst > tmn_max )		tmn_max = nlst;
	   	if ( nlst > t16n_max[n16] )	t16n_max[n16] = nlst;
	   	if ( nlst > t8n_max[n8] )	t8n_max[n8] = nlst;
	   }
	}
       }

       for ( k = 0; k < ndays; k++ ) {

	lst_d = lst_dt[k];
	lst_n = lst_nt[k];
	dlst = lst_d[i_row][j];
	nlst = lst_n[i_row][j];
	n8 = days_sequence[k] / 8;
	n16 = days_sequence[k] / 16;

	if ( dlst > 0.0 && iday_dk[k] ) {
	 if ( i_screened[k] ) {
		t8d_avg[n8] += dlst;
		n8_d[n8] += 1;
	 }
	 else {
	   f_c_d = 0.0;
	   if (tmd_max < 13600.0) {
		if ( is_snow )
			f_c_d = lcv_dlst_f_c[lcv][n8] + 2.0 * lcv_dlst_std[lcv][n8];
		else
			f_c_d = lcv_dlst_f_c[lcv][n8];
	   }
	   f_t_d = lcv_dlst_f_t[lcv][n8] + f_c_d;
	   if ( (tmd_max - dlst) > (f_t_d + dt4_tempo) )	iday_dk[k] = 0;
	   else {
		f_t_d = lcv_dlst_16d_f_t[lcv][n8] + f_c_d;
	   	if ( (t16d_max[n16] - dlst) > (f_t_d + dt3_tempo) )	iday_dk[k] = 0;
		else {
	   		if ( (t8d_max[n8] - dlst) > (f_c_d + dt2_tempo) )	iday_dk[k] = 0;
	   		if ( iday_dk[k] ) {
				t8d_avg[n8] += dlst;
				n8_d[n8] += 1;
			}
		}
	   }
	 }
	}
	if ( nlst > 0.0 && iday_nk[k] ) {
	 if ( i_screened[k] ) {
		t8n_avg[n8] += nlst;
		n8_n[n8] += 1;
	 }
	 else {
	   f_c_n = 0.0;
	   if (tmn_max < 13600.0) {
		if ( is_snow )
			f_c_n = lcv_nlst_f_c[lcv][n8] + 2.0 * lcv_nlst_std[lcv][n8];
		else
			f_c_n = lcv_nlst_f_c[lcv][n8];
	   }
	   f_t_n = lcv_nlst_f_t[lcv][n8] + f_c_n;
	   if ( (tmn_max - nlst) > (f_t_n + dt4_tempo) )	iday_nk[k] = 0;
	   else {
		f_t_n = lcv_nlst_16d_f_t[lcv][n8] + f_c_n;
	   	if ( (t16n_max[n16] - nlst) > (f_t_n + dt3_tempo) )	iday_nk[k] = 0;
		else {
	   		if ( (t8n_max[n8] - nlst) > (f_c_n + dt2_tempo) )	iday_nk[k] = 0;
	   		if ( iday_nk[k] ) {
				t8n_avg[n8] += nlst;
				n8_n[n8] += 1;
	   		}
	   	}
	   }
	 }
	}
       }
       for ( k = 0; k < 4 ; k++ ) {
		if ( n8_d[k] )	t8d_avg[k] /= n8_d[k];
		if ( n8_n[k] )	t8n_avg[k] /= n8_n[k];
       }
       for ( k = 0; k < ndays; k++ ) {

	if ( i_screened[k] )	continue;

	lst_d = lst_dt[k];
	lst_n = lst_nt[k];
	dlst = lst_d[i_row][j];
	nlst = lst_n[i_row][j];
	n = days_sequence[k] / 8;

	f_c_d = f_c_n =  0.0;
	if (tmd_max < 13600.0) {
		if ( is_snow )
			f_c_d = lcv_dlst_f_c[lcv][n] + 2.0 * lcv_dlst_std[lcv][n];
		else
			f_c_d = lcv_dlst_f_c[lcv][n];
	}
	if (tmn_max < 13600.0) {
		if ( is_snow )
			f_c_n = lcv_nlst_f_c[lcv][n] + 2.0 * lcv_nlst_std[lcv][n];
		else
			f_c_n = lcv_nlst_f_c[lcv][n];
	}

	if ( is_snow ) {
	  if ( dlst > 0.0 && iday_dk[k] ) {
	      if ( fabs ((double) (t8d_avg[n] - dlst)) > (f_c_d + dt_tempo) )	iday_dk[k] = 0;
	  }
	  if ( nlst > 0.0 && iday_nk[k] ) {
	      if ( fabs ((double) (t8n_avg[n] - nlst)) > (f_c_n + dt_tempo) )	iday_nk[k] = 0;
	  }
	}
	else {
	  if ( dlst > 0.0 && iday_dk[k] ) {
	   if ( (t8d_avg[n] - dlst) > (f_c_d + dt_tempo) )	iday_dk[k] = 0;
	  }
	  if ( nlst > 0.0 && iday_nk[k] ) {
	   if ( (t8n_avg[n] - nlst) > (f_c_n + dt_tempo) )	iday_nk[k] = 0;
	  }
	}
       }

       for ( k = 0; k < ndays; k++ ) {

	if ( i_screened[k] )	continue;

	lst_d = lst_dt[k];
	QC_d = QC_dt[k];
	v_time_d = v_time_dt[k];
	v_angl_d = v_angl_dt[k];
	lst_n = lst_nt[k];
	QC_n = QC_nt[k];
	v_time_n = v_time_nt[k];
	v_angl_n = v_angl_nt[k];
	em31 = em31_t[k];
	em32 = em32_t[k];
	clr_cov_d = clr_cov_dt[k];
	clr_cov_n = clr_cov_nt[k];

	u16_dlst = lst_d[i_row][j];
	u16_nlst = lst_n[i_row][j];
	u8_em31 = em31[i_row][j];

	if ( u16_dlst && !iday_dk[k] ) {
			lst_d[i_row][j] = 0;
			QC_d[i_row][j] = u8_2cons;
			v_time_d[i_row][j] = 255;
			v_angl_d[i_row][j] = 255;
			clr_cov_d[i_row][j] = 0;
			em31[i_row][j] = 0;
			em32[i_row][j] = 0;
	}
	if ( u16_nlst && !iday_nk[k] ) {
			lst_n[i_row][j] = 0;
			QC_n[i_row][j] = u8_2cons;
			v_time_n[i_row][j] = 255;
			v_angl_n[i_row][j] = 255;
			clr_cov_n[i_row][j] = 0;
			if ( !iday_dk[k] && u8_em31 ) {
				em31[i_row][j] = 0;
				em32[i_row][j] = 0;
			}
	}
      }
     } /* j */
   } /* i_row */

   /* output the modified daily MOD11A1 data */
   for ( ii = 0; ii < numfiles_in; ii++ ) {

     if ( i_screened[ii] )	continue;

     lst_d = lst_dt[ii];
     lst_n = lst_nt[ii];
     QC_d = QC_dt[ii];
     v_time_d = v_time_dt[ii];
     v_angl_d = v_angl_dt[ii];
     QC_n = QC_nt[ii];
     v_time_n = v_time_nt[ii];
     v_angl_n = v_angl_nt[ii];
     em31 = em31_t[ii];
     em32 = em32_t[ii];
     clr_cov_d = clr_cov_dt[ii];
     clr_cov_n = clr_cov_nt[ii];

     for ( i = 0; i < t_nrow; i++ ) {
      for ( j = 0; j < t_ncol; j++ ) {
	dlst = lst_d[i][j];
	nlst = lst_n[i][j];
	if ( dlst > 0.0 ) {
		if ( dlst > dlst_max_2[ii] )    dlst_max_2[ii] = dlst;
		if ( dlst < dlst_min_2[ii] )    dlst_min_2[ii] = dlst;
		dlst_avg_2[ii] += dlst;
		n_dlst_2[ii] += 1;
	}
	if ( nlst > 0.0 ) {
		if ( nlst > nlst_max_2[ii] )    nlst_max_2[ii] = nlst;
		if ( nlst < nlst_min_2[ii] )    nlst_min_2[ii] = nlst;
		nlst_avg_2[ii] += nlst;
		n_nlst_2[ii] += 1;
	}
      }
     }
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[0].name,
	lst_1km_l3[0].gname, sta_1km, dims_1km, &lst_d[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[4].name,
	lst_1km_l3[4].gname, sta_1km, dims_1km, &lst_n[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[1].name,
	lst_1km_l3[1].gname, sta_1km, dims_1km, &QC_d[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[2].name,
	lst_1km_l3[2].gname, sta_1km, dims_1km, &v_time_d[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[3].name,
	lst_1km_l3[3].gname, sta_1km, dims_1km, &v_angl_d[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[5].name,
	lst_1km_l3[5].gname, sta_1km, dims_1km, &QC_n[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[6].name,
	lst_1km_l3[6].gname, sta_1km, dims_1km, &v_time_n[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[7].name,
	lst_1km_l3[7].gname, sta_1km, dims_1km, &v_angl_n[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[8].name,
	lst_1km_l3[8].gname, sta_1km, dims_1km, &em31[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[9].name,
	lst_1km_l3[9].gname, sta_1km, dims_1km, &em32[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[10].name,
	lst_1km_l3[10].gname, sta_1km, dims_1km, &clr_cov_d[0][0]);
     putMODISarray(lst_1d_inf[ii], lst_1km_l3[11].name,
	lst_1km_l3[11].gname, sta_1km, dims_1km, &clr_cov_n[0][0]);

     for (i = 0; i < 4; i++)
	total_l3_1km[i] = 0;

     for ( i_row = 0; i_row < t_nrow ; i_row++ ) {
	for ( j = 0; j < t_ncol; j++ ) {

		u8_1 = QC_d[i_row][j] & u8_3cons;
		total_l3_1km[u8_1] += 1;
		u8_2 = QC_n[i_row][j] & u8_3cons;
		total_l3_1km[u8_2] += 1;

	}
     }

     memset( &met_cnt_1km.FirstEntryInStruct, 0, sizeof( MET_counts ) );
     met_cnt_1km.AllPixelsProcessed =  2 * 1200 * 1200;     /* day and night */

     met_cnt_1km.QA_GoodQuality = total_l3_1km[0];
     met_cnt_1km.QA_OtherQuality = total_l3_1km[1];
     met_cnt_1km.QA_NotProducedCloud = total_l3_1km[2];
     met_cnt_1km.QA_NotProducedOther = total_l3_1km[3];
     met_cnt_1km.QA_NotProducedOther = (met_cnt_1km.AllPixelsProcessed -
	total_l3_1km[0] - total_l3_1km[1] - total_l3_1km[2]);

     if (met_cnt_1km.QA_NotProducedOther < 0) met_cnt_1km.QA_NotProducedOther = 0;

     DoMetaData ( lst_1d_inf[ii] );

     if ( closeMODISfile(&lst_1d_inf[ii]) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "1km L3 output file failed");

   }
/*
   for ( k = 0; k < ndays; k++ ) {
	if ( n_dlst_1[k] )      dlst_avg_1[k] /= n_dlst_1[k];
	if ( n_nlst_1[k] )      nlst_avg_1[k] /= n_nlst_1[k];
        if ( n_dlst_2[k] )      dlst_avg_2[k] /= n_dlst_2[k];
        if ( n_nlst_2[k] )      nlst_avg_2[k] /= n_nlst_2[k];
printf("%2d %3d %5.1f %5.1f %5.1f %4.1f  %5.1f %5.1f %5.1f %4.1f   %5.1f %5.1f %5.1f %4.1f  %5.1f %5.1f %5.1f %4.1f\n", k+1, c_day[k],
 (dlst_min_1[k] > 1.8e6) ? 0.0 : 0.02*dlst_min_1[k], 0.02*dlst_max_1[k], 0.02*dlst_avg_1[k], n_dlst_1[k] * pct_grids_tile,
 (dlst_min_2[k] > 1.8e6) ? 0.0 : 0.02*dlst_min_2[k], 0.02*dlst_max_2[k], 0.02*dlst_avg_2[k], n_dlst_2[k] * pct_grids_tile,
 (nlst_min_1[k] > 1.8e6) ? 0.0 : 0.02*nlst_min_1[k], 0.02*nlst_max_1[k], 0.02*nlst_avg_1[k], n_nlst_1[k] * pct_grids_tile,
 (nlst_min_2[k] > 1.8e6) ? 0.0 : 0.02*nlst_min_2[k], 0.02*nlst_max_2[k], 0.02*nlst_avg_2[k], n_nlst_2[k] * pct_grids_tile);
   }
*/

  /* free all allocated spaces */

  free_lst_space();

  sprintf (LogMsg, "Completed MOD_PR11A1S successfully\n"); 
  SMFLOG(LogMsg);

} /* end main function */

char **
alloc_2d (int32 rows, int32 cols, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 2-d array.

!Input Parameters:	rows 	the first dimension
			cols 	the second dimension
			elsize 	element size in bytes
		     
!Output Parameters:	xh 	pointer to the array

!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

        Zhengming Wan
        UCSB/ICESS
        wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **xh = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	sp = sizeof(char *);
  int32	j = 0;
  size_t  siz;
  
  cols *= elsize; 	/*  cols is now row-size in bytes	*/
  siz = rows * cols;
  
  /*  buf is pointer to data area itself	*/
  /*  xh is pointer to row addresses	*/
  if ((buf = (char *) malloc (siz)) == NULL) {
    sprintf (modstring, "alloc_2d: can't allocate buffer of size %d", siz);
    modsmf(MODIS_E_CALLOC, modstring, "main, mod11a1_cldscreen.c");
    HALTsmf(MODIS_E_CALLOC, modstring);
  }
total_space += siz;

  /*  make xh for pointer vector */
  if ((xh = (char **) malloc(rows * sp)) == NULL) {
    free (buf);
    HALTsmf(MODIS_E_CALLOC, "alloc_2d: can't allocate pointer vector");
  }
total_space += rows * sp;

  /* set pointers to the beginning of each row */
  for (j = 0; j < rows; j++)
    xh[j] = buf + j*cols;
  
  return(xh);
}

char ***
alloc_3d (int32 dim1, int32 dim2, int32 dim3, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 3-d array.

!Input Parameters:	dim1, dim2, dim3 	dimensions
			elsize 			element size in bytes
		     
!Output Parameters:	x1d 	pointer to the array


!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.


!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **x2d = NULL;
  char  ***x1d = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	j=0;
  int32	sp = sizeof(char *);
  size_t	siz;
  
  dim3 *= elsize; 	/*  dim3 is now row-size in bytes */

  siz = dim1 * dim2 * dim3;

  /*  buf is pointer to data area itself	*/
  if ((buf = (char *) malloc(siz)) == NULL) {
     sprintf (modstring, "alloc_3d: can't allocate buffer of size %lu", siz);
     HALTsmf(MODIS_E_CALLOC, modstring);
  }
total_space += siz;

  /*  make x2d for pointer vector for the 2nd dimension */
  if ((x2d = (char **) malloc(dim1 * dim2 * sp)) == NULL) {
     free(buf);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 2nd dim");
  }
total_space += dim1 * dim2 * sp;
	
  /*  make x1d for pointer vector for the 1st dimemsion */
  if ((x1d = (char ***) malloc(dim1 * sp)) == NULL) {
     free(buf);	free(x2d[0]);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 1st dim");
  }
total_space += dim1 * sp;

  /* set the 2nd dimension pointers */
  for (j = 0; j < dim1 * dim2; j++)
    x2d[j] = buf + j*dim3;

  /* set the 1st dimension pointers */
  for (j = 0; j < dim1; j++)
    x1d[j] = x2d + j * dim2;
  
  return(x1d);
}

void free_lst_space(void) 
{
/*
!C***************************************************************************

!Description:
This function frees all allocated dynamic memories normally or abnormally 

!Input Parameters:	none
		     
!Output Parameters:	none

!Revision History:

   * Version 4.0.0 2002/12/16	Z. Wan
   * Initial version

Returns:                none

References and Credits:

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  if (lcv_1km) { free (lcv_1km[0]); free (lcv_1km); }
  if (lst_dt) { free(lst_dt[0][0]);free(lst_dt[0]); free(lst_dt); }
  if (QC_dt) { free(QC_dt[0][0]);free(QC_dt[0]);free(QC_dt); }
  if (v_time_dt) { free(v_time_dt[0][0]);free(v_time_dt[0]);free(v_time_dt); }
  if (v_angl_dt) { free(v_angl_dt[0][0]);free(v_angl_dt[0]);free(v_angl_dt); }
  if (lst_nt) { free(lst_nt[0][0]);free(lst_nt[0]); free(lst_nt); }
  if (QC_nt) { free(QC_nt[0][0]);free(QC_nt[0]); free(QC_nt); }
  if (v_time_nt) { free(v_time_nt[0][0]);free(v_time_nt[0]); free(v_time_nt); }
  if (v_angl_nt) { free(v_angl_nt[0][0]);free(v_angl_nt[0]); free(v_angl_nt); }
  if (em31_t) { free(em31_t[0][0]);free(em31_t[0]);free(em31_t); }
  if (em32_t) { free(em32_t[0][0]);free(em32_t[0]);free(em32_t); }
  if (clr_cov_dt) { free(clr_cov_dt[0][0]);free(clr_cov_dt[0]);free(clr_cov_dt); }
  if (clr_cov_nt) { free(clr_cov_nt[0][0]);free(clr_cov_nt[0]);free(clr_cov_nt); }

}

void Error(const char *message, const char *module)
/******************************************************************************
!C

!Description: 

  'Error' handles the error status for the 'pntr' functions.  If
  USE_SMF is not defined, an error message is written to 'stderr' and
  the program exits with an error status (1).  If USE_SMF is defined
  the error messages are written to LogStatus using SDP toolkit SMF
  functionality, and execution terminates through the modsmf function.
 
!Input Parameters:
  message	error message string
  module	module in which the error occurred

!Output Parameters: (none)

!Revision History:

!Team-unique Header:

  Usage Notes: (none)

!END
*******************************************************************************/
{

  char msg[PGS_SMF_MAX_MSG_SIZE];

  sprintf(msg, "%s in %s", message, module);
  HALTsmf(MODIS_E_BUF_ERROR, msg);

}

void
zeroout_general(void *buf, int32 nelems, int32 type_size)
/*
!C*****************************************************************************

!Description:
   This function sets contiguous memory (4 x nelems) locations to zero.

!Input Parameters:	buf	the starting address of the memory location
			nelems	number of elements (4 bytes each)
		     
!Output Parameters:	none

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/
{  /* begin function */

uint8 *buf8;
uint16 *buf16;
uint32 *buf32;

if (type_size == 1)  {
  buf8 = buf;
  do {
	*buf8 = 0;
	buf8++;
  }
  while (--nelems);
}
else if (type_size == 2) {
  buf16 = buf;
  do {
	*buf16 = 0;
	buf16++;
  }
  while (--nelems);
}
else if (type_size == 4) {
  buf32 = buf;
  do {
	*buf32 = 0;
	buf32++;
  }
  while (--nelems);
}
else {
  HALT("STOP -- wrong data type in zeroout_general -- use 1, 2 or 4.\n");
  exit (1);
}

}  /* end function */

void SMFLOG(char *a)
/*
!C*****************************************************************************

!Description:

This function, SMFLOG, is called by the DoMetaData function.  It replaces 
a macro named SMFLOG, the behavior of which depends on syntactical subtleties
that seem to be compiler-dependent.

!Input Parameters:  Message string

!Output Parameters: None		

!Revision History:
	Created by Richard P. Cember, MODIS SDST, from macro SMFLOG in
	original V1 code delivery. 3 January 1997.

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
        
!Design Notes:
  
!END*************************************************************************/
{

  char	systemTag[PGSd_SMF_TAG_LENGTH_MAX];
  char	LogMsg1[1024];
  char	a1[64] = { NULL };

  strcpy (&a1[0], a);
  a1[34] = '\0';

  sprintf(systemTag, "MOD11A1S LST Version %d", vers_id_num);
  sprintf(LogMsg1,"%s: %s", systemTag, a);
  PGS_SMF_GenerateStatusReport(LogMsg1);
  if (strcmp(a1, "Completed MOD_PR11A1S successfully") != 0)
	exit(1);
  else
  	exit(0);

}

uint8 u8getbits (uint8 x, int32 p, int32 n)
/*
!C***************************************************************************

!Description:
    This function gets the (right adjusted) n-bit field of x that begins at
    position p.  The bit position 0 is at the right end.

!Input Parameters:	x	uint8
			p	the beginning bit posision
			n	number of bits to be checked
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	uint8	x1;

	x1 = x >> (p+1-n) & ~(~0 << n);

	return (x1);
}

void u8setbits (uint8 *x, int32 p, int32 n, uint8 y)
/*
!C***************************************************************************

!Description:
    This function sets value y to the (right adjusted) n-bit field of *x that
    begins at position p.  The bit position 0 is at the right end.

!Input Parameters:	x	pointer to uint8
			p	the beginning bit posision
			n	number of bits to be set
			y	the value to be set
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	int32	pn;
	uint8	y1, z;

	pn = p + 1 - n;
	y1 = (y & ~(~0 << n)) << pn;
	z = ~((~0 >> pn) & ~(~0 << n) << pn);
	*x &= z;
	*x |= y1;
}

/*
!C*****************************************************************************

!Description:

This function, DoMetaData, is called by the main function for writing L2
Metadata, and by function Granule_tile for writing 1km & 1km L3 MetaData.
It is responsible for writing all "core" and "archive" meta data on the
L2 and 1km & 1km L3 LST HDF files.
The meta data that it processes comes from the L1A and/or L1B input files, 
the MOD12 Landcover input files, the PCF file, the MCF file, and some 
generated internally by this function itself.

This void function has no return code.  The program aborts if there are errors.

!Input Parameters:

	MODFILE         	Handle of the output file

!Output Parameters:		None

!Revision History:

  $Log: DoMetaData() $
 * Revision 2.2.4 2005/09/21  22:29     Z. Wan
 *	simplified with only one input parameter
 * Revision 2.2.3 2000/03/07  21:00     Z. Wan
 *	comment out the change made on 1999/12/29 so that MCF files will
 *	not modified during run time.
 * Revision 2.2.1 1999/12/29  11:04     Z. Wan
 *	to set a true value to INPUTPOINTER's NUM_VAL
 * Revision 2.03	 1998/10-05   ff       Z. Wan
 * add MOD11UPD file
 * Revision 2.02	 1998/01-03   ff       R. Cember
 * also modified MCF files
 * Revision 2.01	 1997/11/20  23:55     Z. Wan
 * working with two seperate MCF files
 *
 * DoMetaData()
 * Revision 2.00	 1997/09/30  23:55     Z. Wan
 * for L2, 1km & 1km L3 output files

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
   Originally developed by 
        Gang Ye,
	MODIS/SDST, SAIC/GSC
	301-352-2109
	gang.ye@gsfc.nasa.gov

   Major modifications made by
	Richard Cember
	MODIS/SDST, SAIC/GSC
	301-352-2148
	rcember@ltpmail.gsfc.nasa.gov

!Design Notes:

  Richard Cember made the following modifications:
  1. used a separate MCF for each output file.
  2. checked metadata in each MCF file.
  3. made N_BYTES_CORE_L3 and N_BYTES_ARCH_L3 big enough.
  4. all calls to PGS_PC_GetReference() have been changed to calls to
     PGS_PC_GetUniversalRef().

!END***************************************************************************
*/

#define MAX_CODE_STG_VALS	600

char	         *FloatStg = "SIZEMBECSDATAGRANULE";
PGSt_double      FloatVal;

char	         *DoubleStg1[] = {  "NORTHBOUNDINGCOORDINATE", 
				    "SOUTHBOUNDINGCOORDINATE",
				    "EASTBOUNDINGCOORDINATE",  
				    "WESTBOUNDINGCOORDINATE" };

char	CodeStgVals[MAX_CODE_STG_VALS][PGSd_PC_VALUE_LENGTH_MAX]; 

char	*MoreCodeStg[4] = {
			   "SHORTNAME",
		 	   "HORIZONTALTILENUMBER",
			   "VERTICALTILENUMBER",
			   "GRIDTYPE" };

char	MoreCodeStgVals[4][PGSd_PC_VALUE_LENGTH_MAX];

char	*DoubleStg2[] = {
	 "GRINGPOINTLATITUDE.1",
	 "GRINGPOINTLONGITUDE.1",
	 "EQUATORCROSSINGLONGITUDE.1" };
/* R. Cember: in the current ECS metadata model, EquatorCrossingLongitude is
   a float, not a double, but I see no reason to change the code, since the
   conversion should be OK except for a loss of unnecessary precision. */

char	*IntegerStg2[] = {
	 "GRINGPOINTSEQUENCENO.1",
	 "ORBITNUMBER.1" };

void DoMetaData( MODFILE *outfile )
{

  PGSt_integer	j, version;
  char	*local2;
  char	s_tmp[128] = {NULL};
  char	time2[128] = {NULL};
  char	timeb[128] = {NULL};
  char	LocalGranuleID[128] = {NULL};
  char	augmented_string[MAX_NC_NAME] = {NULL};
  char	str_num[32] = {NULL};
  char  CharSt[256] = {NULL};
  char  GranuleNumber[32] = {NULL};
  int32 i, ii = 0, jj = 2;
  int32	n_gran_pnts_old = 0, n_gran_pnts = 0;
  int32 n_val, sd_id;
  PGSt_integer	i4[4];
  PGSt_double   d4[4];
  PGSt_double	DoubleBounds[4] = {2.F};

  /* R. Cember: for LocalGranuleID proc date & time upgrade. */
  char proc_year[PGSd_PC_VALUE_LENGTH_MAX], proc_day[PGSd_PC_VALUE_LENGTH_MAX];
  char proc_hour[PGSd_PC_VALUE_LENGTH_MAX], proc_minute[PGSd_PC_VALUE_LENGTH_MAX];  
  char proc_second[PGSd_PC_VALUE_LENGTH_MAX];
  char pdt_string[PGSd_PC_VALUE_LENGTH_MAX], *pdt_string_ptr = pdt_string;

/* R. Cember: for InputPointer metadatum upgrade. */
#define MAX_INPUT_PTRS 2000
  char InputPointer[MAX_INPUT_PTRS][PGSd_PC_VALUE_LENGTH_MAX];
  char *InputPointerPtr[MAX_INPUT_PTRS];
  int32 NextInputPointer = 0;
  int32 accum_input_ptrs (char *, const char *); 
/* Z. Wan comments the following line out
  char    mcf_name[128], mcf_kp_name[128], shcmd[256];	*/
  
  /* get core and archiv metadata from the existing L3 output file */

      sprintf(sd_attr_name, ARCHIVES_IN);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_ARCH_L3;
      if(getMODISfileinfo(outfile,sd_attr_name,sd_attr_type,&nelements,
		arch_L3) != MAPIOK)
	Error("Re: lst_1km_l3_outf: failure on call to ", "getMODISfileinfo");

      sprintf(sd_attr_name, CORES);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_CORE_L3;
      if(getMODISfileinfo(outfile,sd_attr_name,sd_attr_type,&nelements,
		core_L3) != MAPIOK)
	 Error("Re: lst_1km_l3_outf: failure on call to ", "getMODISfileinfo");

    date_begin[1][0] = '\0';
    local2 = date_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[0], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_begin from core_L3");
    time_begin[1][0] = '\0';
    local2 = time_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[1], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_begin from core_L3");
    date_end[1][0] = '\0';
    local2 = date_end[1];
    if ( get_meta_attr(core_L3, CharStg2[2], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_end from core_L3");
    time_end[1][0] = '\0';
    local2 = time_end[1];
    if ( get_meta_attr(core_L3, CharStg2[3], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_end from core_L3");

  /* get INPUTGRANULEPOINTERs from the existing output file */

    local = s_tmp;
    if (get_meta_attr(core_L3,"PARAMETERVALUE.5", 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_L3");
    sscanf (local, "%d", &n_gran_pnts_old);

    local = ch_tileid;
    if (get_meta_attr(core_L3,"PARAMETERVALUE.8", 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_L3");

    for (j = 1; j <= n_gran_pnts_old; j++) {

	local = CodeStgVals[jj];
	CodeStgVals[jj][0] = '\0';

	strcpy(augmented_string, "INPUTGRANULEPOINTER");  
	sprintf(str_num, ".%d", j);
	strcat(augmented_string, str_num);

	/* R. Cember: move InputGranulePointer from core to archive */
	if (get_meta_attr(arch_L3,augmented_string,1, 1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3"); 
 
	jj++;

    }

  n_gran_pnts = jj - 2;

  if ( PGS_MET_Init(MCF_FILE_L3_1KM, mdhandles) != PGS_S_SUCCESS )
	HALT("Failure in PGS_MET_Init");

  /*** get VersionID from MCF file**04/06/01**/

  if(PGS_MET_GetSetAttr(mdhandles[1],"VersionID",&vers_id_num) != PGS_S_SUCCESS)
	SMFLOG("Failure in PGS_MET_GetSetAttr for VersionID");

  met_cnt = &met_cnt_1km;

  /* Process Char meta data. */
  for ( j = 0; j < 9; j++ ) {

    /* R. Cember: there is no more OperationMode metadatum, which is j=7, the
    way this code is set.  Skip this case. */
    if (j==7) continue;

    /* Comment added by R. Cember: This next "if" bypasses the 
    EquatorCrossingDate and EquatorCrossingTime metadata for the L3 products */
    if ( j > 4 && j < 7 ) continue;

    CharVals[j][0] = '\0';
    local2 = CharVals[j];

    if ( get_meta_attr(core_L3, CharStg2[j], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails on core_L3");

    if ( j == 8 )
	strcpy ( &local2[0], pgeversion );

    local = local2; 
 
     if(PGS_MET_SetAttr(mdhandles[1], CharStg2[j], &local ) != PGS_S_SUCCESS ) 
      HALT("FAILURE putting a CORE string value");

   }

   /* Process DOUBLE meta data */

   for ( j = 0; j < 3; j++ ) {

    if ( j == 2 )	break;

    n_val = 4;

    if(get_meta_attr(core_L3,DoubleStg2[j],3,n_val,(char *) &d4[0])==FAIL)
		HALT("get_meta_attr fails on core_L3");

    if(PGS_MET_SetAttr(mdhandles[1],DoubleStg2[j], d4) != PGS_S_SUCCESS )
      HALT("FAILURE putting d4");
   }

   /* Process INTEGER meta data */
   for ( j = 0; j < 2; j++ ) {

    if ( j == 1 )	break;
    n_val = ( j ) ? 1 : 4;

    /* set GRINGPOINTSEQUENCENO */
    i4[0] = 1; i4[1] = 2; i4[2] = 3; i4[3] = 4;

    if(PGS_MET_SetAttr(mdhandles[1], IntegerStg2[j], &i4 ) != PGS_S_SUCCESS )
      HALT("FAILURE putting a CORE integer value");
   }

   /* R. Cember: moved ProcessingDateTime up here so we can use its info to
   construct the date/time suffix for LocalGranuleID. */
   {
    /* Construct ProcessingDateTime from UNIX time. */
    char	s[256], *p = s;
    time_t	clock;
    struct tm	*gm_p;

    clock = time(NULL);
    gm_p = gmtime( &clock );

    sprintf(pdt_string, "%i-%02i-%02iT%02i:%02i:%02i.000Z",
	    gm_p->tm_year+1900, gm_p->tm_mon+1, gm_p->tm_mday,
	    gm_p->tm_hour, gm_p->tm_min, gm_p->tm_sec );

    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingDateTime",&pdt_string_ptr) == FAIL )
      HALT("FAILURE putting ProcessingDateTime");

    strcpy( s, process_env );
    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingEnvironment",&p ) == FAIL )
	HALT("FAILURE putting ProcessingEnvironment");

    /* R. Cember: for LocalGranuleID. */
    sprintf( proc_year, "%.4d", 1900+(gm_p->tm_year) );
    sprintf( proc_day, "%.3d", (gm_p->tm_yday)+1 );
    sprintf( proc_hour, "%.2d", gm_p->tm_hour );
    sprintf( proc_minute, "%.2d", gm_p->tm_min );
    sprintf( proc_second, "%.2d", gm_p->tm_sec );
  }

    /* Make the LocalGranuleID from RangeBeginningDateTime */
    sprintf( time2, "%sT%sZ", CharVals[0], CharVals[1] );

    if ( PGS_TD_ASCIItime_AtoB( time2, timeb ) != PGS_S_SUCCESS )
	HALT("PGS_TD_ASCIItime_AtoB failed");
    timeb[4] = timeb[8] = timeb[11] = timeb[14] = '\0';

    /* get correct ESDT via SHORTNAME - added by G. Fireman 2005-11-07 */
    if ( get_meta_attr(core_L3, "SHORTNAME", 1, 1, (char *) esdt_1km_L3) == FAIL)
      HALT("get_meta_attr fails to get SHORTNAME from core_L3");
  
    sprintf( LocalGranuleID, "%s.%s%s%s.h%.2dv%.2d.%03d.%s%s%s%s%s.hdf", 
	esdt_1km_L3, SATELLITE, timeb,
	&timeb[5], ih_1km, iv_1km, vers_id_num,
	proc_year, proc_day, proc_hour, proc_minute, proc_second);

  local = LocalGranuleID;
  if (PGS_MET_SetAttr(mdhandles[1],"LocalGranuleID", &local ) != PGS_S_SUCCESS)
	HALT("FAILURE putting LocalGranuleID in CORE");

  /* Process BoundingCoordinate double meta data */
  for ( j = 0; j < 4; j++ ) {

    if(get_meta_attr(arch_L3,DoubleStg1[j],3,1,(char *)&DoubleBounds[j])==FAIL)
	HALT("get_meta_attr failure getting a ARCHIVE double value");

    if(PGS_MET_SetAttr(mdhandles[2], DoubleStg1[j], 
		&DoubleBounds[j] ) != PGS_S_SUCCESS )
      HALT("FAILURE putting an GEO input ARCHIVE double value");
  }

    for (j = 2; j < jj; j++) {

      /* R. Cember: move InputGranulePointer to archive from core metadata */
      sprintf (&s_tmp[0], "INPUTGRANULEPOINTER.%d", j - 1); 
      rm_path(CodeStgVals[j], local);
      if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
	   SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");

      /* R. Cember: to collect all the input pointers, in this case the
         InputGranulePointers.*/
      NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                CodeStgVals[j]);

    }  /* end for-loop on j */

{  /* Comment added by R. Cember: Begin local block. */

  char		s[256], *p = s, CharString[256], *p1;
  int32		x;
  void		*loc2 = CharString, *loc3 = GranuleNumber;
  char		s_name[64], s_val[64], s_msg[128];	/* added by Z. Wan */

  strcpy( s, rep_actual );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingActual",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingActual");

  strcpy( s, rep_planned );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingPlanned",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingPlanned");

  strcpy( s, BOTH );
  if(PGS_MET_SetAttr(mdhandles[1],"DayNightFlag",&p ) == FAIL )
    HALT("FAILURE putting DayNightFlag");

  strcpy( s, "SCF V5.2.2AS" );
  if(PGS_MET_SetAttr(mdhandles[1],"LocalVersionID",&p ) == FAIL )
    HALT("FAILURE putting LocalVersionID");

  strcpy( s, "Land-cover based band emissivity database" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.1",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.1");
  strcpy( s, "band_emis.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.1",&p ) == FAIL )
    HALT("FAILURE putting band_emis.h in LookupTablePointer");

  strcpy( s, "the generalized split-window LST coefficients" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.2",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.2");
  strcpy( s, "lst_coef.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.2",&p ) == FAIL )
    HALT("FAILURE putting lst_coef.h in LookupTablePointer");

/* R. Cember: End conversion of AncillaryInputType and AncillaryInputPointer
to LookupTableType and LookupTable Pointer. */

  /* Set up for placement in the metadata as a string array. */
  for ( i = 0; i < NextInputPointer; i++ )
	InputPointerPtr[i] = InputPointer[i];

  /* Close out the InputPointer list. */

  InputPointerPtr[NextInputPointer] = NULL;

  /* Reset the InputPointer count for next call to DoMetaData. */
  s[0] = '\0';
  NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], s);

  /* Set the InputPointer metadatum. */
  if(PGS_MET_SetAttr(mdhandles[1],"InputPointer", &InputPointerPtr ) == FAIL )
    HALT("FAILURE setting InputPointer");

  strcpy( s, "MOD 1KM L3 LST" );
  if(PGS_MET_SetAttr(mdhandles[1],"ParameterName.1",&p ) == FAIL )
    HALT("FAILURE putting ParameterName");

  switch(AutomaticQualityFlag) {
  case PASSED:      strcpy( s, "Passed" );      break;
  case SUSPECT:     strcpy( s, "Suspect");      break;
  case FAILED:      strcpy( s, "Failed" );
  }
  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlag.1",&p) == FAIL)
    HALT("FAILURE putting AutomaticQualityFlag");

  strcpy(s,"Passed if algorithm ran within bounds of execution constraints.  ");
  strcat(s,"Suspect if bounds of execution constraints violated.  ");
  strcat(s,"Failed if PGE failed.");

  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlagExplanation.1",&p ) == FAIL )
    HALT("FAILURE putting AutomaticQualityFlagExplanation");

  strcpy( s, sci_qua_exp );
  if(PGS_MET_SetAttr(mdhandles[1],"ScienceQualityFlagExplanation.1",&p ) == FAIL)
    HALT("FAILURE putting ScienceQualityFlagExplanation");

  x = 0;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentInterpolatedData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentInterpolatedData");

  x =  met_cnt->QA_MissingData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],
		     "QAPercentMissingData.1", &x ) == FAIL )
    HALT("FAILURE putting QAPercentMissingData");

  x =  met_cnt->QA_OutOfBoundsData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentOutOfBoundsData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentOutOfBoundsData");

  x = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentCloudCover.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentCloudCover");

/* Do the AdditionalAttributes.n. The ParamterValue.n must be a char string. */

  FloatVal = met_cnt->QA_GoodQuality * 100.0 / met_cnt->AllPixelsProcessed +0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTGOODQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.1", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.1");
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.1", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.1");

  FloatVal = met_cnt->QA_OtherQuality * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTOTHERQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.2", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.2");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.2", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.2");

  FloatVal = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDCLOUD");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.3", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.3");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.3", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.3");

  FloatVal = met_cnt->QA_NotProducedOther * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDOTHER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.4", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.4");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.4", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.4");

	strcpy (s, "N_GRAN_POINTERS");
	sprintf (s_tmp, "%d", n_gran_pnts);
	local2 = s_tmp;

  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.5", &p )
		== FAIL )
	HALT("FAILURE putting AdditionalAttributeName.5");

  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.5", &local2 ) == FAIL )
	HALT("FAILURE putting ParameterValue.5");

/* R. Cember: Re: DDTS 728.  Adding two more PSAs here. */

  strcpy(p, "HORIZONTALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.6", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.6");
  sprintf(p, "%.2d", ih_1km);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.6", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.6");

  strcpy(p, "VERTICALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.7", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.7");
  sprintf(p, "%.2d", iv_1km);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.7", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.7");

/* C. Collins, 11/4/02, change TILEID to TileID */
  strcpy(p, "TileID");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.8", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.8");
  /*sprintf(p, "11%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);*/
  sprintf(p, "%s", ch_tileid);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.8", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.8");

  FloatVal = met_cnt->QA_GoodQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONGOODQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", 9);
  sprintf(s_val, "ParameterValue.%d", 9);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_OtherQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONOTHERQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", 10);
  sprintf(s_val, "ParameterValue.%d", 10);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedCloud * 1.0  / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDCLOUD");
  sprintf(s_name, "AdditionalAttributeName.%d", 11);
  sprintf(s_val, "ParameterValue.%d", 11);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedOther * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDOTHER");
  sprintf(s_name, "AdditionalAttributeName.%d", 12);
  sprintf(s_val, "ParameterValue.%d", 12);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  sprintf(CharString, "%s", "YES");
  strcpy (s, "CLOUD_CONTAMINATED_LST_SCREENED");
  sprintf(s_name, "AdditionalAttributeName.%d", 13);
  sprintf(s_val, "ParameterValue.%d", 13);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  strcpy( s, "102004" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageAcceptanceDate",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageAcceptanceDate");
  strcpy( s, "Normal" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageMaturityCode",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageMaturityCode");
  strcpy( s, "MOD_PR11B" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageName",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageName");
/*next lines edited by C. Collins on 11/11/04 so AlgorithmPackageVersion would
be string*/
  strcpy (s, "5");
/*  x = 5; - line commented out by ccollins on 11/11/04*/
/*x changed to p in next line by ccollins on 11/11/04*/
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageVersion",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageVersion");

  strcpy( s, "GSFC" );
  if(PGS_MET_SetAttr(mdhandles[2],"ProcessingCenter",&p ) == FAIL )
    HALT("FAILURE putting ProcessingCenter");

  strcpy( s, "2484 and 3323" );
  if(PGS_MET_SetAttr(mdhandles[2],"SPSOParameters",&p ) == FAIL )
    HALT("FAILURE putting SPSOParameters");

  local = MoreCodeStgVals[0];
  MoreCodeStgVals[0][0] = '\0';
  if (get_meta_attr(arch_L3,"LOCALINPUTGRANULEID",1,1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3");

  if(PGS_MET_SetAttr( mdhandles[2], "LocalInputGranuleID", &local ) == FAIL )
    HALT("FAILURE putting LocalInputGranuleID");

  /* set tile information */

  sprintf( MoreCodeStgVals[1], "%.2d", ih_1km);
  sprintf( MoreCodeStgVals[2], "%.2d", iv_1km);
  strcpy( MoreCodeStgVals[3], "INTEGERIZED SINUSOIDAL");

  for ( j = 1; j < 3; j++ ) {

    local = MoreCodeStgVals[j];

    if(PGS_MET_SetAttr(mdhandles[2], MoreCodeStg[j], &local)!=PGS_S_SUCCESS)
	SMFLOG("PGS_MET_SetAttr -- Error adding arch meta data");
  } 
}

 sd_id = outfile->sd_id;

 if(PGS_MET_Write(mdhandles[1], CORES, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing CORE");

/*
 if(PGS_MET_Write(mdhandles[2], ARCHIVES_OUT, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing ARCHIVES");
*/

 PGS_MET_Remove();

}

int32 accum_input_ptrs (char *InputPtrElem, const char *InputString)
/*
!C*****************************************************************************

!Description:

This is a service function for DoMetaData.  It accumulates the 
InputGranulePointers, InputtilePointers and LookupTablePointers into one
array so they can be conveniently put into the InputPointer metadatum.

!Input Parameters:
  char *InputPtrArray   
  char *InputPtrElement

!Output Parameters:  
  None

!Returns:
  The number of elements so far accumulated.

!Revision History:

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

   This function written by Richard P. Cember, SDST, 29 January 1998

!References and Credits:

!Design Notes:

This is basically a kludge to adapt PGE16/MOD_PR11 to ECS metadata requirements
which were not well known when the code was written.

It is expected that the last function call within each call to DoMetaData
will contain a NULL InputPtrElement per TK User's Guide p. 6-60.  The 
current number of elements, including the final null, is returned on each call.
When the final null is passed in, the PtrIndex is reset for the next time 
DoMetaData itself is called.

!END***************************************************************************
*/
{

  static int32	PtrIndex = 0;
  int32		NumPtrs;
  char		test_string[1] = {'\0'};

  strcpy(InputPtrElem, InputString);

  PtrIndex++;
  NumPtrs = PtrIndex; 

  /* Get set for the next pass through DoMetaData. */
  if ( strcmp(InputString, test_string) == 0) PtrIndex = 0;

  if ( NumPtrs == MAX_INPUT_PTRS ) {
    free_lst_space();
    SMFLOG("STOP--NumPtrs reachs the limit");
  }

  return (NumPtrs);

}

/*
!C*****************************************************************************

!Description:

This function, get_meta_attr, is used to get an attribute of metadata
which is stored in main memory.  It is more efficient than getting a
metadata attribute from the HDF file.

!Input Parameters:

        s			pointer to a metadata string
        p			pointer to the attribute name string
        type			type of the attribute (1 for char,
				2 for int32, 3 for float64)
	n			number of the values of the attribute

!Output Parameters:

	d			pointer to the returned attribute values

!Revision History:

  $Log: get_meta_attr() $
 * Revision 2.00	 1997/11/20  21:55     Z. Wan
 *

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:

!Design Notes:

!END***************************************************************************
*/

#define OK	0

int32 get_meta_attr (char *s, char *p, int32 type, int32 n, char *d)
{
	char	*s1, *s2, *s3;
	char	pt[64];
	char	*s_o;
	int32	class, i_class;
	int32	i_c_match;
	int32	i, j, k;

	int32	*i_o;
	double	*d_o;

	k = strlen (p);

	i_class = 0;

	for (i = 0; i < k; i++) {
		if (p[i] == '.') {
			sscanf (&p[i+1], "%d", &i_class);
			if ( i_class == 0 )	return ( FAIL );
			break;
		}
		pt[i] = p[i];
	}
	pt[i] = '\0';

	if ( (s1 = strstr (s, pt)) == NULL)
		return ( FAIL );

	i_c_match = 1;

	if ( i_class ) {
		i_c_match = 0;
		for ( i = 0; i < i_class; i++) {
			s2 = strstr (s1, "CLASS");
			s3 = strstr (s2, "= ");
			sscanf (s3+3, "%d", &class);
			if ( i_class == class ) {
				i_c_match = 1;
				s1 = s3;
				break;
			}
			else {
				s1 = s3+3;
				s2 = strstr (s1, "END_OBJECT");
				s3 = strstr (s2, pt);
				s1 = s3 + 1;
				if ( (s2 = strstr (s1, pt)) == NULL)
					return ( FAIL );
				s1 = s2;
			}
		}
	}

	if ( ! i_c_match ) return ( FAIL );

	s2 = strstr (s1, "VALUE");
	s3 = strstr (s2, "= ");

   if ( type == 1 ) {

	s_o = (char *) &d[0];
	s3 += 3;
	for ( i = 0; i < n; i++ ) {
		for ( j = 0; ; j++) {
			if (s3[j] == '"') {
				s_o[j] = '\0';
				break;
			}
			s_o[j] = s3[j];
		}
	}
   }
   else if ( type == 2 ) {
	  
	  i_o = (int32 *) &d[0];
	  s3++;

	  if ( n == 1 ) {
		sscanf (s3, "%d", &i_o[0]);
	  }
	  else {
	    for ( i = 0; i < n; i++ ) {

		if ( i )
			s2 = strstr (s3, ",");
		else
			s2 = strstr (s3, "(");
		s3 = s2;

		sscanf (s3+1, "%d", &i_o[i]);
		s3++;
	    }
	  }
	}
	else if ( type == 3 ) {

		d_o = (double *) &d[0];
		s3++;

		if ( n == 1 ) {
			sscanf (s3, "%le", &d_o[0]);
		}
		else {
		   for ( i = 0; i < n; i++ ) {
			if ( i )
				s2 = strstr (s3, ",");
			else
				s2 = strstr (s3, "(");
			s3 = s2;

			sscanf (s3+1, "%le", &d_o[i]);
			s3++;
		   }
		}
	     }
	     else	return ( FAIL );

	return ( OK );
}

/* a short program to setup __ctype_b, __ctype_toupper and __ctype_tolower
   written by Scott Burns, Mirrabooka Systems.
   It is downloaded from http://www.dbforums.com/t964847.html
   by
   Zhengming Wan, ICESS, University of California, Santa Barbara, CA 93106
   It is used only #ifdef _LINUX_ct

*/

#ifdef _LINUX_ct

#include <ctype.h>

__const unsigned short int *__ctype_b;
__const __int32_t *__ctype_tolower;
__const __int32_t *__ctype_toupper;

void ctSetup()
{
	__ctype_b = *(__ctype_b_loc());
	__ctype_toupper = *(__ctype_toupper_loc());
	__ctype_tolower = *(__ctype_tolower_loc());
}

#endif
