/* start of update_for_lst.h */

#ifndef update_for_lst_h
#define update_for_lst_h

/*****************************************************************************
!C-INC

!Description:
  Header file for the data structure used in the intermediate file for storage
  of the most up date daytime and nighttime data.  An intermediate file
  corresponds to each 1-km tile.  The stored intermediate data will be used
  in the day/night LST algorithm for simultaneously retrieving surface
  temperatures and band emissivities at 6km resolution.

!Input Parameters: from
		  MOD021KM, MOD03, MOD07, MOD10, MOD12Q, MOD35,
		  and MOD43B1

!Output Parameters: condensed information of the above parameters.

$Log: updata_for_lst.h, v $
 *
 * Revision V5.0.4, 2005/04/04 11:34     wan
 *	move uint8 cwv_day and cwv_nit to the place after int16 sf_elev
 *
 * Revision V5.0.2, 2005/01/21 16:27     wan
 *	use N2_VIEW_ANGS = 2 * N_VIEW_ANGS = 16
 *	reduce _UPD_FOR_LST_SV to 64 bytes by replacing float32 with uint16
 *      for tb_day[7] & tb_nit[7], float32 with uint8 for cwv_day & cwv_nit
 *
 * Revision V5.0.0, 2004/06/11 11:07     wan
 *	increase N_VIEW_ANGS to 6 from 5
 *	update _upd_ang_range_u2[N_VIEW_ANGS]
 *	add to_suppl_upd for use in east-west slope dependent grids
 *
 * Revision V4.2.0, 2002/10/12 13:50     wan
 *	keep near_cld and far_cld info in nit_status and day_status
 *
 * Revision V4.0.0, 2002/04/24 11:50     wan
 *	change uint16  tb_band26 to b7_alpha
 *	increase N_VIEW_ANGS to 5 from 4
 *
 * Revision V3.1, 2001/04/19 11:10     wan
 *	to save information at up to four viewing angles
 *
 * Revision V2.1, 1999/12/30 01:46     wan
 *
 * Revision V2.0, 1997/11/10 17:50     wan
 * Add state_bits and micros for bit operations
 *
 * Revision V1.1, 1996/06/21 23:02:00  wan
 * Initial revision
 *
$Id: updata_for_lst.h, v V2.0 1997/11/10 27:50 wan Exp wan $

!Revision History:


!Team-unique Header:
  This software has been created by MODIS Science Team member Zhengming Wan
  for the National Aeronautics and Space Administration, Goddard Space
  Flight Center, under contract NAS5-31370.

!References and Credits:

	Written by Zhengming Wan
	ICESS
	UCSB
	Santa Barbara, CA 93106
	wan@icess.ucsb.edu

!Design Notes:

!END
*****************************************************************************/

#define	N_VIEW_ANGS 8
#define	N2_VIEW_ANGS 16

#define	BITS_DAY 0777
#define	BITS_HRS 0177

#define CB_DAY_HRS(id, ih) ( (uint16) ((id & BITS_DAY) | (ih & BITS_HRS) << 9))
#define GET_DAY(x)	   ( x & BITS_DAY )
#define GET_HRS(x)	   ( ( x >> 9) & BITS_HRS )

/* bits in day_status & nit_status */

#define	EXIST_BIT		01
#define	NEW_BIT			02
#define	NEW_EXIST_BITS		03
#define	SCREEN_OFF		020
#define	NEAR_CLD		0100
#define FAR_CLD			0200

#define CHK_TWO_BITS(x, bit1, bit2) ( (x & bit1) && (x & bit2) )

uint8   _upd_ang_range_u2[N_VIEW_ANGS] = { 20, 40, 60, 78, 94, 108, 120, 130 };

typedef struct _upd_for_lst {	/* structure for working intermediate data
				   used in the day/night LST algorithm */

	float32	tb_day[8];		/* brightness temperatures in bands
					   20, 22, 23, 29, 31-33
					   plus split-window day LST */

	float32	tb_nit[8];		/* brightness temperatures in bands
					   20, 22, 23, 29, 31-33
					   plus split-window night LST */

	float32	cwv_day;		/* column water vapor */
	float32	cwv_nit;

	uint16	day_date_and_view_time;	/* bit 00-08 for date
					   bit 09-15 for time (0.2hr) */

	uint16	nit_date_and_view_time; /* bit 00-08 for date
					   bit 09-15 for time (0.2hr) */

	uint16	b7_alpha;		/* in unit of 0.0002 so that max 13.1 */

	int16	sf_elev;		/* averaged surface elevation in m */

	uint8	i_data_yr_dn;		/* data year -  2000, higher 4 bits
					   for day and 4 lower bits for nit */

	uint8	ta_day;			/* atmospheric temperature at surface
					   level in unit of 0.6K above 185K */

	uint8	ta_nit;			/* atmospheric temperature at surface
					   level in unit of 0.6K above 185K */

	uint8	view_zenith_day;	/* in unit of 0.5 deg */
	uint8	sola_zenith;		/*            0.5 deg */
	uint8	view_zenith_nit;	/* in unit of 0.5 deg */

	uint8	landcov;		/* 6km land-cover type
					   bit 0100=1 for lake grid
					   bit 0200=1 for sunglint lake grid */

	uint8	snow_bits;		/* each bit for one of six 1km cells
				     	   in one 1km row, for current snow */

	uint8	lakeice_bits;		/* each bit for one of six 1km cells in
				     	   one 1km row, for current lakeice */

	uint8	day_status;		/* 01 for data availability */

	uint8	nit_status;		/* 01 for data availability */

	uint8   QA_bits;		/* QA_day in the higher 4 bits and
					   QA_nit in the lower 4 bits */

	uint8	upd_cov_d, upd_cov_n;	/* coverages */

	uint8	reserved[2];

}	_UPD_FOR_LST;			/* its size is 96 bytes */

typedef struct _upd_ang_lst {

	_UPD_FOR_LST _upd_for_lst[N2_VIEW_ANGS];

}	_UPD_ANG_LST;

#define	SZ_UPD_FOR_LST	sizeof (_UPD_FOR_LST)

#define	SZ_UPD_ANG_LST	sizeof (_UPD_ANG_LST)


typedef struct _upd_for_lst_sv {  /* structure for storing intermediate data
				   used in the day/night LST algorithm */

	uint16	tb_day[8];		/* brightness temperatures in bands
					   20, 22, 23, 29, 31-33 (unit 0.02K)
					   plus split-window day LST */

	uint16	tb_nit[8];		/* brightness temperatures in bands
					   20, 22, 23, 29, 31-33 (unit 0.02K)
					   plus split-window night LST */

	uint16	day_date_and_view_time;	/* bit 00-08 for date
					   bit 09-15 for time (0.2hr) */

	uint16	nit_date_and_view_time; /* bit 00-08 for date
					   bit 09-15 for time (0.2hr) */

	uint16	b7_alpha;		/* in unit of 0.0002 so that max 13.1 */

	int16	sf_elev;		/* average surface elevation (m) */

	uint8	cwv_day;		/* column water vapor (unit 0.05cm) */
	uint8	cwv_nit;

	uint8	i_data_yr_dn;		/* data year - 2000, use the higher
				     4 bits for day and lower 4 bits for nit */

	uint8	ta_day;			/* atmospheric temperature at surface
					   level in unit of 0.6K above 185K */

	uint8	ta_nit;			/* atmospheric temperature at surface
					   level in unit of 0.6K above 185K */

	uint8	view_zenith_day;	/* in unit of 0.5 deg */
	uint8	sola_zenith;		/*            0.5 deg */
	uint8	view_zenith_nit;	/* in unit of 0.5 deg */

	uint8	landcov;		/* 6km land-cover type
					   bit 0100=1 for lake grid
					   bit 0200=1 for sunglint lake grid */

	uint8	snow_bits;		/* each bit for one of six 1km cells
				     	   in one 1km row, for current snow */

	uint8	lakeice_bits;		/* each bit for one of six 1km cells in
				     	   one 1km row, for current lakeice */

	uint8	day_status;		/* 01 for data availability */

	uint8	nit_status;		/* 01 for data availability */

	uint8   QA_bits;                /* QA_day in the higher 4 bits and
					   QA_nit in the lower 4 bits */

	uint8	upd_cov_d, upd_cov_n;	/* coverages */

	uint8	n_retrieve_times;	/* averaged_em by times */

	uint8	averaged_em[7];		/* averaged emis in 6 bands */

}	_UPD_FOR_LST_SV;		/* its size is 64 bytes */

typedef struct _upd_ang_lst_sv {

	_UPD_FOR_LST_SV _upd_for_lst_sv[N2_VIEW_ANGS];

}	_UPD_ANG_LST_SV;

#define	SZ_UPD_FOR_LST_SV	sizeof (_UPD_FOR_LST_SV)

#define	SZ_UPD_ANG_LST_SV	sizeof (_UPD_ANG_LST_SV)

#endif

/* end of update_for_lst.h */
