/*
!C*****************************************************************************

!Description:
   Main program of MODIS Land-Surface Temperature (LST) Version 5.2.1, part 1.

   It allocates memory space for the L1A and L1B data arrays, the look-up
   tables to be used in the MODIS LST algorithms.
   It reads in parts of the MODIS L1A, L1B and Cloudmask simulated data,
   determines how many tiles to be needed to cover the whole granule and
   how many scans to be needed to process for each of the selected tiles.
   It calls function Graule_tile once for each of the tiles.
   It writes metadata for the L2 output file and free all allocated space
   before completion of the PGE.

!Input Parameters:
   PCLog1A & fname_L1a		for L1A input file from MOD_PR11.pcf
   PCLog1B & fname_L1b		for L1B input file from MOD_PR11.pcf
   PCLogCmask & fname_Cld	for Cloudmask input file from MOD_PR11.pcf
   PCLogProfile & fname_Prof	for Aprofile input file from MOD_PR11.pcf
   PCLogSnow & fname_Snow	for Snow-cover input file from MOD_PR11.pcf

   parameters and the whole	from "land_tile_tbl.h"
   land tile system

!Output Parameters:
   n_tiles			the total number of tiles to cover the whole
				input granule

   tile_pntr			pointers to the information for the selected
				tiles

!Revision History:

 * Revision 5.2.2, 2005/11/23           Z. Wan
 *	extend "95-99% clear over land" to "66-99% clear over land" for pixels above 2000m
 *	Set "SCF V5.2.2A"
 * Revision 5.2.1, 2005/09/15           G. Fireman
 *	Read Latitude & Longitude from MOD021KM and write to swath as GeoFields 
 *	 so MOD11_L2 file can be read by subsetting tool
 *	Z. Wan made some minor changes for writing correct QAPERCENT values to
 *	 metadata of MOD11A1 and MOD11B1 files (2005/09/23)
 *	Set "SCF V5.2.1A"
 * Revision 5.2.0, 2005/06/19 22:40     Z. Wan
 *	Using the lowest non-fillied value of the temperature profile as surface air temperature
 *	 starting from MOD07 PGEVERSION 5.2.0
 *	Set "SCF V5.2.0A"
 * Revision 5.1.0, 2005/06/01 10:21     Z. Wan
 *	Tested with sample data of v5.1.0 MOD07
 *	Update the name of "Snow_Cover" or "Snow Cover" using MOD_Swath_Snow
 *	Set "SCF V5.1.0A"
 * Revision 5.0.4, 2005/04/04 11:35     Z. Wan
 *	Use V5.0.4 update_for_lst.h
 *	Expand the definition of cloud-free pixels to 95% clear over land
 *	Make minor changes to ensure all LSTs from instant observations
 *	Set "SCF V5.0.4A"
 * Revision 5.0.2, 2005/01/21 17:15     Z. Wan
 *	Use V5.0.2 lst_1dy_data.h and update_for_lst.h (N2_VIEW_ANGS = 16,
 *	 i.e., the whole swath is separated into 16 sub-ranges)
 *	Use the procedure in V3.0.0 to interpolate the surface air temperature
 *	 from the temperature profile because the Surface_Temperature SDS in
 *	 the new MOD07 product is changed to the regression-based retrieval
 *	 of skin temperature from the GDAS surface temperature in the original
 *	 MOD07 code (Suzanne W. Seemann confirmed in her e-mail of 1/11/05)
 *	Set "SCF V5.0.2A"
 * Revision 5.0.1, 2004/11/11 		C. Collins
 *	change snow_cv to snow_cv1 and add snow_cv2 so that that code could
 *	work with old or new MOD10_L2
 * Revision 5.0.0, 2004/06/11 12:57	Z. Wan
 *	Use V5.0.0 lst_1dy_data.h and update_for_lst.h
 *	Set "SCF V5.0.0A"
 * Revision 4.2.0 2002/12/31  10:31     Z. Wan
 *	Use lst_1dy_data.h and update_for_lst.h (V4.2.0)
 *	Set "SCF V4.2.0A"
 * Revision 4.1.7 2002/12/24  15:58     Z. Wan
 *	Update the 0100 and 0200 bits in cldmask for near_cld and far_cld
 *	Do not save the radiances of extreme cold (below 200K) pixels to
 *	 UPD files due to the large quantization error in band 20
 *	Set "SCF V4.1.7A"
 * Revision 4.1.5 2002/12/16  22:11     Z. Wan
 *	Update bit 00-01 value in QC_out (MOD11_L2) and qc_1km_l3 (MOD11A1)
 *	Set "SCF V4.1.5A"
 * Revision 4.1.3 2002/11/28  11:48     Z. Wan
 *	Use lst_1dy_data.h updated according to MOD11_L2.fs, MOD11A1.fs, and
 *	 MOD11B1.fs (v4.1.3)
 *	Set QC_Day and QC_Night in MOD11B1 for the grids with values of
 *	 LST_Day_5km_Aggregated_from_1km & LST_Night_5km_Aggregated_from_1km
 *	Set "SCF V4.1.3A"
 * Revision 4.1.1 2002/11/10  23:41     Z. Wan
 *	Change Day_view_angl and Night_view_angl in MOD11A1 and MOD11B1
 *	 and their file specs so that they are negative if viewing from east
 *	Set "SCF V4.1.1A"
 * Revision 4.1.0 2002/10/12  13:56     Z. Wan
 *	Use lst_1dy_data.h and update_for_lst.h (V4.1.0)
 *	Add a new QC_Emis to MOD11B1
 *	Use lbin_lst.c (V2.1.1) modified by Robert Wolfe and Zhengming Wan
 *	 for sin and isin projections
 *	Use lbin_lst.h (V2.1.1) modified by Robert Wolfe and Zhengming Wan
 *	Use the GenObs routine for Integerized Sinusoidal and Sinusoidal grids
 *	The first 10 pixels in each scan line are processed and assume that
 *	 the effect of using the nearest cwv_prof value is similar to near_cld
 *	#include "mod.fm1.rad7.h" for processing Aqua MODIS data
 *	Delete old lbin functions and unused variables
 *	Set "SCF V4.1.0A"
 * Revision 4.0.1 2002/06/08  00:56     Z. Wan
 *	Use band 7 BRDF info in MOD43B1C (6km-resolution BRDF)
 *	Set "SCF V4.0.1A"
 * Revision 4.0.0 2002/04/29  21:18     Z. Wan
 *	Use band 7 BRDF info in MOD43B1
 *	Use modified update_for_lst.h and lst_1dy_data.h (V4.0.0)
 *	Delete the option of selecting land-tiles with reg_id number(s) given
 *	 in argv[2] and beyond.
 *	Set "SCF V4.0.0A"
 * Revision 3.1.3 2002/04/27  17:56     Z. Wan
 *	A fix to a bug in v3.1.1 mod11_lst2A1.c
 *	The first 10 pixels will not be processed in response to the new
 *	 change in MOD07_L2 v3.1.0
 *	Set "SCF V3.1.3A"
 * Revision 3.1.1 2002/03/28  11:21     Z. Wan
 *	A fix to the bad LocalGranuleId problem in geolocation files
 *	 is to use only the last four chars in the LocalGranuleId (the
 *	 beginning time of the granule, saved in a new variable granule_hrs)
 *	 in the match loop for MOD03 files
 *	Set "SCF V3.1.1A"
 * Revision 3.1.0 2002/01/26  11:15     Z. Wan
 *	Set "SCF V3.1.0A"
 *	A patch was made to prevent PGE16 from skipping some grids in tiles
 *	 h09v02, h06v03, h00v10, and h14v17.
 * Revision 3.0.3 2001/08/31  16:30     Z. Wan
 *	Set "SCF V3.0.3A"
 *	cakecut.c was also included in this code.
 *	A fix was made to prevent PGE16 from failure on pixels in strange
 *	 shapes.
 * Revision 3.0.2 2001/08/06  16:30     Z. Wan
 *	To skip pixels with longitude values out of the normal sequence
 *	 in order to avoid the PGE failure at a small number of pixels with
 *	 abnormal longitude values.
 *	Set QA_NotProducedOther to 0 if it has a negative value.
 * Revision 3.0.1 2001/06/01  18:24     Z. Wan
 *	Set the Data Quality flag bits for MOD11_L2 and MOD11A1 products
 * Revision 3.0.0 2001/05/03  23:43     Z. Wan
 *	The original mod11_lst2.c was separated into two parts.  This is the
 *	 first part for the production of MOD11_L2 and MOD11A1 products.
 *	 The second part was named as mod11_lstB1.c to generate the
 *	 MOD11B1 product with the day/night LST algorithm.
 *	Read ReprocessingPlanned & ReprocessingActual information from the PC
 *	Fetch the LocalGranuleID value from the mcf file
 *	Set i_calib_correction to 0 by default
 *	The is_a_side_data value will be determined by the data day, it may
 *	 be overridden in PCF by setting the parameter IS_A_SIDE_DATA
 * Revision 2.5.1 2001/04/29  21:30     Z. Wan
 *	To change the scale from 0.0004 to 0.0005 in cal_1km_l3[10] & [11],
 *	 for Clear_day_cov & Clear_nit_cov in the 1km L3 LST product.
 *	LUT mod_rad7_lut.h was replaced by mod.pfm-vC.rad7.h with a wider
 *	 temperature range from 180K to 400K.
 *	LUT day_nit_lut.h was replaced by day_nit_lut21x15.h for wider
 *	 ranges of atmospheric temperature and water vapor conditions.
 *	LUT f_stat_lut.h was removed because a modified TISI (temperature
 *	 independent spectral index method was used to replace the
 *	 statistical regression method for providing initial guess for
 *	 day/night LST algorithm.
 * Revision 2.4.1 2001/03/24  10:44     Z. Wan
 *	To make this code Terra and Aqua compatible.  It is assumed that
 *	 the names of all input and output files are started with "MOD" in
 *	 the processing of Terra data and with "MYD" in the Aqua processing.
 *	 "PM-1" is used for Aqua's ASSOCIATEDPLATFORMSHORTNAME.
 * Revision 2.3.6 2001/03/13  16:29     Z. Wan
 *	To change the scale from 0.0001 to 0.0004 in cal_1km_l3[10] & [11],
 *	 for Clear_day_cov & Clear_night_cov in the 1km L3 LST product.
 *	To do u8setbits for an initial qc_1km_l3 even if cell_cov <= 0.66.
 * Revision 2.3.4 2001/01/08  15:39     Z. Wan
 *	To update the option to correct the effect of calibration errors in
 *	 bands 31 and 32 (viewing angle dependent dt_avg_b31_b32 and
 *	 dt_dif_b31_b32 based on V2.5.4 L1B data and the vicarious calibration
 *	 field campaign over Lake Titicaca, June 2000) on the MOD11_L2 product.
 *	 This option can be turned off by setting a run-time parameter in PCF
 *	 as 211315|TURN_OFF_CALIBRATION_CORRECTION|1.
 * Revision 2.3.2 2000/12/09  09:57     Z. Wan
 *	To fix the Anomolously high LST value problem posted in JB_MOD11_00340
 *	 on the LDOPE website (this problem would be avoid if turning
 *	 IGNORE_UNCERT off with a line 211314|IGNORE_UNCERT|0 in PCF, or
 *	 if SDS EV_1KM_Emissive values are given as _FillValue 65535 or -1
 *	 rather than 32765 in the valid range in
 *	 MOD021KM.A2000252.1540.002.2000259114748.hdf, not a normal L1B file,
 *	 in which all "%Valid EV Observations" are zero), ignore granules with
 *	 "%Valid EV Observations" < 0.0001 in bands 31 and 32, and print a
 *	 message to LogReport file.
 *	To change the lower boundary of the valid range to 1 from 0 for
 *	 Columns_per_Global_Grid_Row in MOD11A1 and MOD11B1 (in response to
 *	 JB_MOD11_00335).
 * Revision 2.3.1 2000/12/02  17:50     Z. Wan
 *	To make the code default for B-side data, add runtime parameter line
 *	 211316|IS A-SIDE DATA|1 in PCF for processing A-side data.
 *	To fix the Buffer error problem when working on partly bad granules or
 *	 titled granules with Sector Rotation EV Data (such as in A2000291.1500,
 *	 A2000291.1640, and A2000291.1645), IJMAX was increased to 192 from 24
 *	 in cakecut.h and MAX_NOBS was increased to 1020 from 255 in
 *	 obsgrid_lst.h.
 * Revision 2.2.21 2000/09/30  13:19     Z. Wan
 *	To add an option to correct the effect of calibration errors in bands
 *	 31 and 32 (b31_dt = 0.18, b32_dt = -0.15 as results of the vicarious
 *	 calibration field campaign over Lake Titicaca, June 2000) on the
 *	 MOD11_L2 product.  This option can be turned off by setting a run-time
 *	 parameter in PCF as 211315|TURN_OFF_CALIBRATION_CORRECTION|1.
 * Revision 2.2.20 2000/09/14  22:44     Z. Wan
 *	To add two SDSs to the MOD11B1 output (see MOD11B1.fs and lst_1dy_data.h),
 *	 one is LST_Day_5km_Aggregated_from_1km, another is
 *	 LST_Night_5km_Aggregated_from_1km.  They are based on MOD11A1 and
 *	 can be used in the global LST browse image processing.
 *	To replace dimensions 10*nscans, Max_EV_frames, 2*nscans, and
 *	 Max_EV_frames/5 with Along_swath_lines_1km, Cross_swath_pixels_1km,
 *	 Coarse_swath_lines_6km, and Coarse_swath_pixels_6km, respectively.
 * Revision 2.2.16 2000/08/10  10:01     Z. Wan
 *	To fix the "Fatal error in ... getMODISarray from upd_ang_lst" problem
 * Revision 2.2.14 2000/07/18  23:11     Z. Wan
 *	To fix the MOD11_L2 QAPERCENTMISSINGDATA problem per Mike Linda's
 *	e-mail and LDOPE case number SD_MOD11_060100A
 * Revision 2.2.11 2000/05/20  2:10     Z. Wan
 *	To rectify the OperationalQualityFlag problem per David Roy and Mike
 *	Linda's e-mails (refer DR_MOD11_051800B)
 *	The format of input granule IDs in archive metadata LOCALINPUTGRANULEID
 *	was simplified to avoid failure in the high latitude zones.
 * Revision 2.2.9 2000/04/28  12:17     Z. Wan
 *	To fix the GRING problem (refer DR_MOD11_040500)
 *	To add the new PSA QAFRACTION's after the original ones
 *	 (refer DR_MOD11_033000)
 *	To reduce the stripping caused by the cloudyness due to the noisy 4th
 *	channel of band 22 (refer DR_MOD11_022800)
 *	To set bits 04-05 of QC in MOD11_L2 appropriately (CC_MOD11_032700)
 *	The micro ADJ() was modified
 *	clarification of most possibly confusing points:
 *	a) The value of AllPixelsProcessed is the fixed maximum number of
 *	  pixels or grids possibly in a granule or tile.
 *	b) In MOD11_L2, the value of QAPERCENTCLOUDCOVER is defined by the
 *	  number of all pixels not VeryHighConfidentClear in MOD35, but in
 *	  MOD11A1 and MOD11B1, the value of QAPERCENTCLOUDCOVER is defined
 *	  by the number of land grids not covering any clear-sky LST pixels.
 *	c) The value of QAPERCENTNOTPRODUCEDOTHER is defined by the number
 *	  of pixels or grids skipped by all reasons but clouds.  Ocean pixels
 *	  are skipped before skipping cloudy land pixels.
 *	d) The value of QAPERCENTNOTPRODUCEDCLOUD is defined by the number
 *	  of land pixels or grids skipped due to clouds.
 *	e) The value of QAPERCENTGOODQUALITY is defined by the number of
 *	  pixels or grids with good quality LST.
 *	f) The value of QAPERCENTOTHERQUALITY is defined by the number of
 *	  pixels or grids with LST not in good quality.
 * Revision 2.2.8 2000/04/11  10:29     Z. Wan
 *	four QAFRACTION* of floating point numbers are moved to core metadata
 *	from archive metadata
 *	MOD11_L2.mcf, MOD11A1.mcf, and MOD11B1.mcf modified accordingly
 *	skip those pixels with Fill value for latitude or longitude.
 * Revision 2.2.7 2000/04/07  00:31     Z. Wan
 *	DoMetaData( n_6km_l3_old_file, ...) after DoMetaData( n_old_1km, ...)
 *	to fix the RANGEBEGINNINGTIME or/and RANGEENDINGTIME problem informed
 *	by Nazmi Z El Saleous on March 29, 2000
 * Revision 2.2.5 2000/03/18  15:47     Z. Wan
 *	mod11_lst2.c and lst_1dy_data.h modified to use the SDS Water_Vapor
 *	in MOD07
 * Revision 2.2.4 2000/03/13  11:20     Z. Wan
 *	remove the QC=3 seam in L2 LST output by the edge of tiles (to correct
 *	the problem in QA Case # DR_MOD11_030900)
 * Revision 2.2.3 2000/03/07  21:00     Z. Wan
 *	Per SDST's suggestion in Mike Linda's e-mail, to make things easy
 *	INPUTPOINTER's NUM_VAL will be kept to a fixed value rather than
 *	the true value of the number of inputpointers used in the metadata
 *	so that the MCF files will not be modified during run time.
 * Revision 2.2.2 2000/02/20  22:34     Z. Wan
 *	change the QAPERCENT* values to integers,
 *	add four QAFRACTION* of floating point numbers in archive metadata.
 * Revision 2.2.1 1999/12/29  11:04     Z. Wan
 *	change "SCF V2.1.9" to "SCF V2.2.1"
 *	land_tile_tbl.h updated for the use of the UM at-launch land-cover
 *	 product (version 3).  From the total 328 tiles, only 317 tiles were
 *	 selected as input to the PGE16 processing.  There are land and/or
 *	 coastline grids in these 317 tiles.
 *	band_emis.h was modified correspondingly
 *	lst_1dy_data.h updated for adding SDS Land_Cover_Type_QC
 *	move the calculation of total_l2_pix[] to the location just after
 *	 the Granule_tile loop
 *	DoMetaData() was modified to set a true value to INPUTPOINTER's NUM_VAL
 *	values of N_BYTES_CORE_L3, N_BYTES_ARCH_L3, and MAX_CODE_STG_VALS
 *	 were increased to solve the problem of processing in zones N1 & S1
 *	 (latitude ranges from 60 deg. to 90 deg. and from -90 deg. to -60 deg.)
 *	take DoMetaData() out from the i_scan_loop to avoid the duplicate
 *	 input pointers in metadata
 * Revision 2.1.11 1999/10/28  10:25     Z. Wan
 *      print more message to Logreport
 *        in case where the input granule is an ocean granule
 * Revision 2.1.10 1999/09/25  21:40     Z. Wan
 *	print more message to LogReport
 *	EV_1KM_Emissive_Uncert_Indexes will be ignored if a run-time parameter
 *	  is defined in PCF with 211314|IGNORE_UNCERT|1
 *	change the type of QA_GoodQuality, QA_OtherQuality,
 *	  QA_NotProducedCloud, and QA_NotProducedOther back to FloatVal
 *	  in order to provide useful information for ocean granules that
 *	  may cover only a few land pixels over a small island.
 * Revision 2.1.9 1999/09/17  09:18     Z. Wan
 *	use Day, Night or Both in DayNightFlag instead DAY, NIGHT or BOTH
 *      read PGEVersion (PGE16_Version with LUN 800500) from PCF
 *	add begin_seconds & end_seconds into struct _tile_pointer in
 *	  land_tile_tbl.h
 *	change the type of QA_GoodQuality, QA_OtherQuality,
 *	  QA_NotProducedCloud, and QA_NotProducedOther to Integer from FloatVal
 * Revision 2.1.8 1999/08/11  20:42     Z. Wan
 *	fixed a bug in lines 1391 & 1392 ("core_1B" was changed to "core_Snow")
 * Revision 2.1.7 1999/07/11  13:15     Z. Wan
 *     type of tile_begin_seconds, tile_end_seconds, and seconds_tmp was
 *	changed to float64 from float32.
 *     time format "%02i:%02i:%f" was changed to "%02i:%02i:%09.6f".
 * Revision 2.1.6 1999/05/20  17:50     Z. Wan
 *     use micro HALTsmf() to write error message to LogReport
 *     set VERSIONID to 1 for the at-launch version of code in MCF files
 *     use L1B's PGEVERSION to determine its format
 * Revision 2.1.5 1999/05/17  20:20     Z. Wan
 *     some modifications made in Granule_tile()
 * Revision 2.1.4 1999/05/07  16:10     Z. Wan
 *     It is able to input new and old format L1B data.
 * Revision 2.1.1 1998/04/02   ff.      Z. Wan
 * Revision 2.0  1998/01/24   ff.      R. Cember, SDST
 * Revision 2.03 1998/01/03  02:00     Z. Wan
 * Revision 2.02 1997/12/01  01:56     Z. Wan
 * Revision 2.01 1997/09/15  23:55     Z. Wan
 * Revision 1.3  1997/09/08  23:05     Z. Wan
 * Revision 1.2  1997/08/25  13:13     R. Cember and G. Ye, SDST
 * Revision 1.1  1996/11/20  08:56:56  Wan
 * Revision 1.0  1996/09/25  23:18:29  Wan
 *
 * 1. Output L2 & L3
 * 2. Changed *modoutf1 to *lst_l2_outf
 * 3. Added intermediate binary data file update_for_lst
 * 4. Added day/night LST algorithm
 *
 * Revision B3.2  1995/09/11  11:26:20  wan
 *
 * Zhengming Wan: use generic variable types such as uint16, int32, float32,
 * float64, and etc. except long int required by M-API2.2.1
 * All allocated spaces are freed in normal and abnormal exits.
 * Write SD attributes according to the MOD11 product HDF formats.
 *
 * Revision B3.1  1995/09/01  11:07:01  wan
 * Initial version
 *
   $Id: mod11_lst2A1.c, v 3.0.0 2001/05/03 23:43 wan Exp wan $

 *====
 *
 * Changes made by Robert Wolfe, 2002/09/12
 *
 * 1. Added support for the Sinusoidal grid.
 *    a. Moved the Lbin routines to a seperate file and added the
 *       Sinusoidal grid routines..
 *    b. Added the GenObs_sin routine that is specific to the
 *       Sinusoidal grid.
 *    c. Change the name of Integerized Sinusoidal GenObs routine to
 *       GenObs_isin.
 *    d. Changed the calls to the grid dependent subroutines so that the
 *       function names are pointers.
 * 2. Removed references to more than one grid resolution.
 *
 *
 *====

!Team-unique Header:
   
   This software was developed by:
   Zhengming Wan for the National Aeronautics and Space Administration,
   Goddard Space Flight Center, under NASA contract NAS5-31370.

   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541
   Programmer/Analyst, Xiaoning Duan
   	duan@icess.ucsb.edu
   	805-893-7353
   ICESS
   University of California
   Santa Barbara, CA 93106-3060

   HDF Portions developed at the National Center for Supercomputing
   Applications at the Univ. of Illinois at Urbana-Champaign.

!References and Credits:

   Zhengming Wan and Jeff Dozier, "A generalized split-window algorithm for
     retrieving land-surface temperature from space", IEEE Trans. Geoscience
     and Remote Sensing, vol. 34, no. 4, pp. 892-905, 1996.

   Zhengming Wan and Zhao-Liang Li, "A Physics-based algorithm for retrieving
     land-surface emissivity and temperature from EOS/MODIS data", IEEE
     Trans. Geoscience and Remote Sensing, vol. 35, no. 4, pp. 980-996, 1997.

   Zhengming Wan, MODIS LST ATBD (ATBD-MOD-11), v3.3, April, 1999.

   Zhengming Wan developed this MODIS LST code.  Assistances with integrating
   PGS, M-API toolkits, and HDF was provided by Xiaoning Duan under helps of
   Gang Ye, and Paul Fisher of SDST/STIG.

   The code of writing metadata was implemented by Gang Ye of SDST in V1 and
   was modified by Zhengming Wan for Version 2.  

   R. Cember modified the code to solve the memory leak problem, to replace
   ECS-prohibited functions (such as lseek, read, and write with fseek, fread,
   and fwrite, respectively, on 1/29/97), to fuly prototype all functions, and
   to reduce the level of nesting.

   The adapted subset of the L2G library functions developed by Robert E.
   Wolfe, MODIS Land Team Support Group in Christopher O. Justice's MODIS
   Land Science Team.

!Design Notes:

   This is the phase 1 of the daily LST V2.1 code. MODIS product snow-cover and
   Atmospheric profiles have been used as additional inputs to the MOD11 code.

   Because the snow-cover product is only available for daytime granules and
   day/night pairs on the same day will be used as the first choice for the
   day/night LST method, granules need to be processed in sequence of files
   with DAYNIGHTFLAG as "DAY" in the first place, then "BOTH", and finally
   "NIGHT".

   The cumulative comments have been cleared for clarity.  To see old versions
   for the detailed modifications in different stages.

   1. The version number of the input granule file is given in argv[1] that
      is assigned by the script file for MODIS PGE16.
   2. The reg_id number(s) of selected land-tiles are given in argv[2].  These
      numbers are separated by comma (,) without blank space between them.
      If no such id # is given, all land-tiles are selected.
      See land_tile_tbl.h for the reg_id # of each land tiles.
   3. In order to provide LST values twice a day for all land grids in the
      1km L3 product, daytime is defined as local standard time 6:00 to 18:00.

This source file includes calls to functions of the HDF (Hierarchical Data
Format) applications programming interface libraries, developed and distributed
by the National Center for Supercomputing Applications (NCSA), University of 
Illinois, Urbana-Champaign.

!END**************************************************************************
*/

/* Special compile time flags because of inconsistant support accross
 * computers for the 'const' qualifier */

#define SMF_SOURCE	"main mod11_lst2A1.c"

#define USE_ECS		/* ECS meta data handling */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#include "mapi.h"	/* M-API 2.3.1 */

#include "PGS_SMF.h"
#include "PGS_MODIS_37111.h"

#include "smfio.h"

#include "mod.fm1.rad7.h"     /* radiance look-up table for bands 20, 22, 23,
			         29, and 31-33 of Aqua MODIS */
#include "mod.pfm-vC.rad7.h"  /* radiance look-up table for bands 20, 22, 23,
			         29, and 31-33 */
#include "lst_coef.h"	   /* coefficients of the generalized split-window
			      LST algorithm */
#include "band_emis.h"	   /* prototype band emissivity knowledge base */
#include "bit_flag.h"	   /* bit flags for QC */
#include "lhdf.h"
#include "HdfEosDef.h"
#include "lbin_lst.h"
#include "pntr_lst.h"
#include "obsgrid_lst.h"
#include "cakecut.h"
#include "lst_1dy_data.h"
#include "land_tile_tbl.h"

/* for writing MOD_Swath_LST */
#include "PGS_MEM.h"
#include "hntdefs.h"
int32   SWfidout, SWidLST;              /* Swath handles */
int32   hdfID;
int32   LstHDFhandle;

#define	SMALL_VALUE	1.0e-07

_GSW_LST_COEF_TBL  *gsw_lst_coef_p;   /* pointer to generalized
					 split-window LST
					 coefficeints table */

int32	   multi_write_out = 1;	      /* write output file by segments or not*/

/* functions for space allocation and allocated variables */
/* declared as global variables so that they can be freed */
/* both in normal and anormal exits */

char 	   **alloc_2d (int32, int32, size_t);
char       ***alloc_3d (int32, int32, int32, size_t);
int32	   z_siz;
int32	   siz1 = 0, siz2 = 0, siz3 = 0;

float32	   scales[16];
float32	   offsets[16];

uint16     ***data = NULL;            /* allocated space for L1B data input */
uint8      ***emissive_uncert_indexes = NULL;

uint8      **l1b_qa = NULL;

uint16     **data_p = NULL;
uint8      **uncert_p = NULL;

float32	   radiance[8];
float64	   btemp[7];
float64	   **coef_mesh = NULL;
float64    *coeff_p = NULL;

uint16	   **lst_out = NULL;	       /* data array for output L2 LST */
uint16     *lst_o_p = NULL;
uint16	   **QC_out = NULL;	       /* data array for output QC */
uint16     *QC_o_p = NULL;
uint8	   **err_lst_out = NULL;       /* data array for output Error_LST */
uint8      *err_lst_p = NULL;
uint8	   **emis31_out = NULL;	       /* data array for output Emis_31 */
uint8      *e31_o_p = NULL;
uint8	   **emis32_out = NULL;	       /* data array for output Emis_32 */
uint8      *e32_o_p = NULL;
uint8	   **view_angl_out = NULL;     /* data array for output View_angle */
uint8	   *vangl_o_p = NULL;
uint8	   **view_time_out = NULL;     /* data array for output View_time */
uint8	   *vtime_o_p = NULL;
float32    **lat_5pix = NULL;          /* data array for latitude from MOD02 */
float32    **lon_5pix = NULL;          /* data array for longitude from MOD02 */

/* for daily 1km L3 LST */

uint16	**lst_1km_Sv = NULL;        /* for lst_day_1km_Sv or lst_nit_1km_Sv */
uint8	**QC_1km_l3 = NULL;         /* for QC_Day or QC_Night */
uint8	**v_time_1km_l3_Sv = NULL;
uint8	**v_angl_1km_l3_Sv = NULL;
uint8	**em31_1km_l3_Sv = NULL;
uint8	**em32_1km_l3_Sv = NULL;
uint16	**clear_day_cov = NULL;
uint16	**clear_nit_cov = NULL;

uint16	*lst_1km_Sp = NULL;
uint8	*QC_1km_l3_p = NULL;
uint8	*v_time_1km_l3_Sp = NULL;
uint8	*v_angl_1km_l3_Sp = NULL;
uint8	*em31_1km_l3_Sp = NULL;
uint8	*em32_1km_l3_Sp = NULL;
uint16	*clear_day_cov_p = NULL;
uint16	*clear_nit_cov_p = NULL;

/* for daily 6km L3 LST */

uint16	**lst_6km_S_from_1km = NULL;
uint16	*lst_6km_Sp_from_1km = NULL;
uint16	**lst_6km_Sd = NULL;
uint8	**QC_6km_day = NULL;
uint8	**v_time_6km_l3_Sd = NULL;
int16	**v_angl_6km_l3_Sd = NULL;
uint16	**lst_6km_Sn = NULL;
uint8	**QC_6km_nit = NULL;
uint8	**v_time_6km_l3_Sn = NULL;
int16	**v_angl_6km_l3_Sn = NULL;
uint8	**em20_6km_l3_Sv = NULL;
uint8	**em22_6km_l3_Sv = NULL;
uint8	**em23_6km_l3_Sv = NULL;
uint8	**em29_6km_l3_Sv = NULL;
uint8	**em31_6km_l3_Sv = NULL;
uint8	**em32_6km_l3_Sv = NULL;
uint8	**QC_emis = NULL;
uint8	**pct_land = NULL;
uint8	**QC_6km;
float32	**em31_6km_d = NULL;
float32	**em31_6km_n = NULL;

int16   **d_azm_6km = NULL;

uint8	**v_time_6km_l3_Sv = NULL;	/* internal variable */
uint8	**v_angl_6km_l3_Sv = NULL;	/* internal variable */
float32	  **sf_elev_6km_d = NULL;	/* internal variable */
float32	  **sf_elev_6km_n = NULL;	/* internal variable */
float32	  *sf_elev_6km_d_p = NULL;	/* internal variable */
float32	  *sf_elev_6km_n_p = NULL;

float32   brdf_6km[4];

/* working space for lst_1km_L3 */

float32	   **cov_1km_day_S = NULL;
float32	   **cov_1km_nit_S = NULL;
float32	   **cov_1km_Sp = NULL;
float32	   **lst_1km_S = NULL;		/* for sum of radiance in b31 */
uint8	   **qc_1km_l3d = NULL;
uint8	   **qc_1km_l3n = NULL;
uint8	   **qc_1km_l3 = NULL;
float32	   **vtime_1km_l3_S = NULL;
float32	   **vangl_1km_l3_S = NULL;
float32	   **em31_1km_l3_S = NULL;
float32	   **em32_1km_l3_S = NULL;
float32    view_sign = 1.0;
int32	   view_ang_sign = 0;

/*next line LUN changed from 211006 to 211106 by c. collins on 10/21/04 so
 unique LUN would be in use -- 211006 is being used by another PGE for
 different product*/
#define PCLogslopelut	211106	/* the hdf file for slope_landonly_6km */
char	slope_fname[MAX_NC_NAME] = {NULL};
uint8	***slope_6km = NULL;
MODFILE	*slope_handle=NULL;

/* for the intermediate update file */

#include "update_for_lst.h"

int32	n_old_update = 0;

int32	nrow_6km = TILE_YDIM_6KM;
int32	ncol_6km = TILE_XDIM_6KM;

_UPD_ANG_LST	**upd_ang_lst_W = NULL;	   /* for working space (8 angles) */
_UPD_ANG_LST	*upd_a_lst_w_p = NULL;		/* pointer to upd_ang_lst_W */
_UPD_ANG_LST	*upd_a_lst_w_p_snow = NULL;	/* pointer to upd_ang_lst_W */
_UPD_ANG_LST	*upd_a_lst_w_p_ice = NULL;	/* pointer to upd_ang_lst_W */
_UPD_ANG_LST	*upd_a_lst_w_p1 = NULL;	   /* pointer to upd_ang_lst_W */
_UPD_FOR_LST	*upd_lst_w_p = NULL;	   /* pointer to upd_for_lst in W */
_UPD_FOR_LST	*upd_lst_w_p1 = NULL;	   /* pointer to upd_for_lst in W */
_UPD_FOR_LST	*upd_lst_w_p_snow = NULL;  /* pointer to upd_for_lst in W */
_UPD_FOR_LST	*upd_lst_w_p_ice = NULL;   /* pointer to upd_for_lst in W */
_UPD_FOR_LST	*upd_lst_w_p1_snow = NULL; /* pointer to upd_for_lst in W */
_UPD_FOR_LST	*upd_lst_w_p1_ice = NULL;  /* pointer to upd_for_lst in W */
_UPD_ANG_LST_SV	**upd_ang_lst_Sv = NULL; 	/* previously saved */
_UPD_ANG_LST_SV	*upd_a_lst_S_svp = NULL;	/* pointer to upd_ang_lst_Sv */
_UPD_FOR_LST_SV	*upd_lst_S_svp = NULL;		/* pointer to upd_for_lst_Sv */
_UPD_FOR_LST_SV	*upd_lst_S_svp_snow = NULL;	/* pointer to upd_for_lst_Sv */
_UPD_FOR_LST_SV	*upd_lst_S_svp_ice = NULL;	/* pointer to upd_for_lst_Sv */

float32		***upd_cov_d = NULL;
float32		***upd_cov_n = NULL;
float32		*upd_cov_d_p = NULL;
float32		*upd_cov_n_p = NULL;
float32		cov_v = 0.0;
float32		rad_v = 0.0;
uint8		cov_s_u8 = 0, cov_l_u8 = 0;
uint8		u8_1r = 0, u8_2r = 0;
float32		cov_previous = 0.0, cov_current = 0.0;

float32         ***cov_prof_d = NULL;
float32         ***cov_prof_n = NULL;
float32		*cov_prof_p = NULL;
float32         cov_v_prof = 0.0, cov_v_prof1 = 0.0;

/* data array for Cloudmask */

uint8	***cmask = NULL;
uint8	**cmask1 = NULL;
uint8	**cldmask = NULL;

uint8	near_cld = 0, far_cld = 0;

/* data array for Atmospheric profile */

float32	pres_levels[14] = {	/* at the lower 14 output levels */
	1000., 950., 920., 850., 780., 700., 620., 500., 400., 300.,
	 250., 200., 150., 100. };

uint8	**cmask_prof = NULL;
uint8	***qa_prof = NULL;
int16	**ts_prof = NULL;
int16	**ps_prof = NULL;
int16   ***t_prof = NULL;
int16	**cwv_prof_raw = NULL;
int16	**cwv_prof = NULL;
float32 ***t_prof_sum_d[N2_VIEW_ANGS];
float32 ***t_prof_sum_n[N2_VIEW_ANGS];
int32	n_along_prof, n_cross_prof;
int32	n_zones, i_zone;
int32	n_ts_in_zone[24], n_cwv_in_zone[24];
float32	tair_zones[24], cwv_zones[24];

_tile_pointer tile_pntr[N_TILE_PNTRS];

/* data array for landcover */

uint8	**land_type = NULL;
uint8	***landcov_qc = NULL;
uint8	**lc_qc = NULL;

/* data array for BRDF MOD43B1 */
int16   ****brdf_p, ***brdf_3p;
int32   ***brdf_q;
long int sta_brdf_p[4] = { 0, 0, 6, 0 };
long int sta_brdf_q[3] = { 0, 0, 1 };

/* data array for 6km BRDF MOD43B1C */
int16   ****brdf_p_6km = NULL, ***brdf_3p_6km = NULL;
int16	*brdf_p_p;
int32   ***brdf_q_6km;
int32   brdf_param_6km_type = DFNT_INT16;
long int sta_brdf_p_6km[4] = { 0, 0, 6, 0 };
long int sta_brdf_q_6km[3] = { 0, 0, 1 };

/* data array for geoloc L1A */

int16	***dat1_1a = NULL;
int16	**sf_height = NULL;
int16	**sensr_zen = NULL;
int16	**solar_zen = NULL;
int16   **sensr_azm = NULL;
int16   **solar_azm = NULL;
float32	**lat = NULL;
float32	**lon = NULL;
uint8	**land_sea_mask_1a = NULL;
float64	*ev_start_time = NULL;

/* data array for snow cover */

uint8	**snowcv = NULL;
uint8	**snowcv_old = NULL;

uint8	**lakeice = NULL;
uint8	**lakeice_old = NULL;

int32	ii_scan_b;
int32   i_cloudy_pix = 0;
int32   i_cloudy_pix_u = 0;
int32   i_cloudy_pix_l = 0;
float32	**p_time;
float32	pix_time;
float32	continu_time;
float32	continu_time_sv;
float32	pix_zero_time;
float32	pix_zen_deg;
int32	pix_sol_deg;

float64 lat_pix, lon_pix;		/* variables used in Granule_tile */
float64 lat_pix_inv, lon_pix_inv;	/* inversed values by Lbin_Inv */
int32	irow_pix_o, icol_pix_o;		/* variables used in Lbin_FwdW */
int32	irow_pix, icol_pix;		/* variables used in Lbin_FwdW */
float32	rrr_pix, ccc_pix;		/* variables used in Lbin_FwdW */
int32	near_tile_pix = 0;	/* center of the pixel near the tile edge */
int32	in_tile_pix = 0;	/* center of the pixel within the tile */
int32	n_pix_in_tile = 0;
int32	n_pix_near_tile = 0;
int32	n_pix_lst_done = 0;
int32	nobs_1km_tot = 0;

long int	sta_geo1[1] = { 0 };
long int	sta_geo2[2] = { 0, 0 };
long int	sta_lc[2] = { 0, 0 };
long int	sta_snow[2] = { 0, 0 };
long int	sta_prof2[2] = { 0, 0 };
long int	sta_prof[3] = { 6, 0, 0 };
long int	sta_qa_prof[3] = { 0, 0, 0 };

char msgString[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for use with modsmf */
char LogMsg[PGS_SMF_MAX_MSG_SIZE]={NULL};    /* string for smf status report */
char systemTag[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for systemTag */

ECSattr_names_for_all_handles   HDFattrNames;   /* Meta data attributes */
long int                        NumHandles = 2; /* two master metadata groups*/

#include "PGS_PC.h"
#define PCLog1B         700002   /* L1B input */
#define PCLog1A         600000   /* L1A input */

int32	PCLogupdate[MAX_N_DAYS_PER_CYCLE] = { 211000, 211001 };
char	id_char_update[8] = "A";
int32	n_days_per_cycle_update_pcf;
int32	i_days_in_cycle_update_pcf;
int32	n_days_per_cycle_update = 2;
int32	i_days_in_cycle_update = 0;
int32	i_do_multi_days = 1;
PGSt_integer	numfiles_update[MAX_N_DAYS_PER_CYCLE] = { 0, 0 };

#define PCLogL2         211020   /* MOD11 L2 LST output */
#define PCLogL3_1km     211031   /* MOD11 L3 LST 1km input/output */
#define PCLogL3_6km     211035   /* MOD11 L3 LST 6km input/output */
#define MCF_FILE_L2     211252   /* MCF for the L2 LST file */

/* R. Cember: a separate MCF for each output file. */
#define MCF_FILE_L3_1KM     211253       /* MCF for the L3 1km LST file */
#define MCF_FILE_L3_6KM     211254       /* MCF for the L3 6km LST file */
#define MCF_FILE_L3_UPD     211259       /* MCF for the update_for_lst file */

/* R. Cember: setting the DAYNIGHTFLAG valids through macros. */
#define DAY "Day"
#define NIGHT "Night"
#define BOTH "Both"

/* R. Cember: for use in metadata setting. */
char UR_L1b[PGSd_PC_VALUE_LENGTH_MAX];

/* Z. Wan: add it for Geolocation in metadata setting. */
char UR_geo[PGSd_PC_VALUE_LENGTH_MAX];

#define PCLogLC         212550   /* MOD12 L3 Landcover 96 day input */
#define PCLogBRDF   	243260	 /* MOD43B1C (6km) 16-day */
/*next line added by C. collins on 10/20/04 because same LUN cannot be used
 for both MOD43B1 and MOD43B1C*/
#define PCLogBRDF1km   	243200	 /* MOD43B1 (1km) 16-day */
#define PCLogSnow       210100   /* MOD10 L2 Snow input */
#define PCLogCmask      422500   /* MOD35 L2 Cloud Mask input */  
#define PCLogProfile    420000   /* MOD07 L2 Atmo. Profile input */

/* the following 122 lines come from pntr.c only with minor changes */

#ifdef USE_ECS          /* ECS meta data handling (Paul Fisher/YTLau) */

#include "PGS_MET.h"
PGSt_MET_all_handles            mdhandles;      /* Meta data handles */

/* ECS metadata string and variable names */
#define CORES		"CoreMetadata.0"
#define ARCHIVES_IN	"ArchiveMetadata.0"
#define ARCHIVES_OUT	"ArchiveMetadata.0"

/* the following three defines must be defined in relation to one another */
#define METAHANDLES mdhandles
#define COREHANDLES mdhandles[1]
#define ARCHIVEHANDLES mdhandles[2]

/* default value for QAPERCENTINTERPOLATEDDATA */
#define QAINTDATA 0
/* default value for QAPERCENTMISSINGDATA */
#define QAMISDATA 0
/* default value for QAPERCENTOUTOFBOUNDSDATA */
#define QAOBDATA 0

#endif  /* end USE_ECS */

/* Fixed parameters */

#define AQFLAG	"passed"

#define SIZE 0.5	/* half the size of the cell; This parameter can 
			 * increased to increase the size of the cell 
			 * without increasing the spacing between the 
			 * cell centers */

#define NTILE_HORIZ 36	/* number of horizontal tiles in the grid; nominally 
			 * this value (36) will generate 10 degree (1100 km)
			 * tiles; this should always be either 1 or two 
			 * times 'NTILE_VERT' */
#define NTILE_VERT 18	/* number of vertical tiles in the grid; nominally 
			 * this value (18) will generate 10 degree (1100 km)
			 * tiles */

#ifndef M_PI
#define M_PI 3.1415926535897931
#endif
#ifndef PI
#define PI M_PI		 /* numeric constant: 'pi' */
#endif
#define TWOPI (2.0 * PI) /* numeric constant: two * 'pi' */
#define PIO2 (PI*0.5)
#define DEG (180.0 / PI) /* factor for converting radians to degrees */
#define DTR(x)	((x) * 0.0174532925199433) /* degree to radians by * PI/180. */
#define RTD(x)	((x) * 57.295779513082323) /* radians to degree by * 180./PI */
#define DLAT_6KM_ROW 0.05	        /* the latitude increament (deg) for
					   one 6km row */

/* micro for water vapor conversion from dew point temperature Td to mass
   density (GM CM-2 KM-1): wv = WV_EXP(x) * T0/T where x = T0/Td */
#define WV_EXP(x) ( 0.1 * expf(18.9766 - 14.9595 * x - 2.43882 * x * x) )

#define IO_ERROR -1     /* i/o area flag */

/* Macros for testing and setting bits */

#define BIT_TEST(x, b) (((x) & BIT[(b)]) != 0)
#define BIT_SET(x, b) x = x | BIT[(b)]

/* User entered parameters data structure */

typedef struct {
  
  int16 itile_horiz, itile_vert; /* vertical and horizontal tile numbers */
  float32 coverage_min;	/* minimum coverage */
  char gridtype[8];    /* grid type string */
  int32 itile_size;	/* tile size */
  int32 ngran;          /* number of granules */
} uparam_t;

/* Tile and binning data structure */

typedef struct {
  int16 itile_horiz, itile_vert; /* selected horizontal and vertical tile */
  int32 nrow, npix;		/* number of rows and pixels per row */
  int32 irow_start, irow_end; /* start and end row of the tile in entire grid */
  int32 itile_size;
} tile_t;

/* Function Prototypes */

void InitBin(tile_t *tile, int32 itile_size);
void Error(const char *, const char *); 
void poly_intp_vector (float64 *, int32, float64, int32, float64 **, float64 *);

void GenObs(const double cen[2], const dgeo_t *cor, int32 *nobs, obs_t *obs);
typedef void (*PGenObs)(const double cen[2], const dgeo_t *cor, int32 *nobs, obs_t *obs);
PGenObs pGenObs;

PLbin_Init pLbin_Init;
PLbin_FwdW pLbin_FwdW;
PLbin_Inv pLbin_Inv;

/* Global (within file) variables */

static float32 coverage_min; /* minimum coverage ratio */
static char gridtype[80];    /* gridtype is set by InitBin */

#define MIN2(x, y)  ((x) < (y) ? (x) : (y))
#define MAX2(x, y)  ((x) > (y) ? (x) : (y))

/* external link to variables in lbin_isin.c and lbin_sin.c */

extern Lbin_set_t *set;
extern void Lbin_Free();

uint16		time_1b, kn = 0;
char		time_1b_a[8];
uint16    	**gran_order = NULL;
PGSt_integer    Vers1A = 1;
PGSt_integer    Vers1B = 1;
PGSt_integer    VersCmask = 1;
PGSt_integer    VersAprof = 1;
PGSt_integer    VersSnow = 1;

/* G. Ye added function DoMetaData that was modified by R. Cember. and Z. Wan */

void DoMetaData( int32 , int32 , PGSt_PC_Logical , PGSt_integer ,
	PGSt_integer , MODFILE *);

/* mod11 specific variables */

int32	nit_mode;
int32	n_stat = 0;

/* Start Lbin_FwdW_(xx) externals
   These values are used by the Lbin_FwdW_(xx) functions to determine if the
   points outside the tile are near the tile edge */

const int32 irow_lower = -2, irow_uppr = 2;
const int32 icol_left = -5, icol_right = 4;

/* End Lbin_FwdW_(xx) externals */

int16	*i_day_scan, *i_nit_scan, *i_scan_mode;
int16	*i_cld_scan;
int32	i_scan_line;
int32	n_scan_pixels;
int32	i_scan_prof, j_scan_prof;

Obs_gd_t *gd;

/* mod11 specific functions */

void	free_lst_space(void);
void 	SDsetdim_LST (int32, sds_t, int32 *);
void 	SDsetattr_LST (int32, char *, int32, int32, VOIDP);
void 	SDsetattr_LST_V (int32, char *, int32, int32, float32 *);

/* R. Cember: add function "zeroout_general". */
void zeroout_general(void *buf, int32 nelems, int32 type_size);

/* R. Cember: bring run-time parameters into line with process-specified
   LUN ranges. */

#define N_READ_LINES	   211312  	/* run-time defined n_read_lines   */
#define SUBSET_1B	   211313  	/* run-time defined i_subset_1b    */
#define IGNORE_UNCERT	   211314  	/* run-time defined i_ignore_uncert */
#define TURN_OFF_CALIBRATION_CORRECTION 211315 /* run-time defined to set
					   i_calib_correctin to 0 */
#define IS_A_SIDE_DATA	   211316  	/* run-time defined is_a_side_data */
#define GRID_TYPE	   211317  	/* run-time defined gridtype */
#define PGE16_Version	   800500  	/* run-time defined PGE16 Version  */

#define ReprocessingPlanned 800503	/* it's a common LUN number */
#define ReprocessingActual  800504	/* it's a common LUN number */
#define ScienceQualityFlagExplanation  800506	/* it's a LUN number */
#define ProcessingEnvironment  800550	/* it's a common LUN number */

char    rep_planned[PGSd_PC_VALUE_LENGTH_MAX];
char    rep_actual[PGSd_PC_VALUE_LENGTH_MAX];
char    sci_qua_exp[PGSd_PC_VALUE_LENGTH_MAX];
char    process_env[PGSd_PC_VALUE_LENGTH_MAX];
char    localgranuleId_vers_num[PGSd_PC_VALUE_LENGTH_MAX];
char    data_yr[16];
char    data_day[16];
char    granule_hrs[16];
int32   vers_id_num = 5;
int32	i_data_day;
uint8	i_data_yr;
uint8	i_data_yr_d;
uint8	i_data_yr_n;
 
char	pgeversion[64];

char	pval[PGSd_PC_VALUE_LENGTH_MAX];
int32	i_tile_in_proc = 0;
int32	n_lines_per_scub = 10;

/* R. Cember: Add declaration of file handles for conversion from
   ECS-prohibited C functions to equivalent SDP toolkit calls. */ 

  PGSt_integer Vers;

/* R. Cember: making this declaration global. */
  PGSt_SMF_status returnStatus = MODIS_S_SUCCESS;  /* Default for success */

  MODFILE	  *modinf1b = NULL;		  /* input file L1B */
  MODFILE	  *modinf1a = NULL;		  /* input file GEO */
  MODFILE	  *modinfbrdf = NULL;	/* input file MOD43B1 or MOD43B1 */
  MODFILE	  *modinfcld = NULL;		  /* input file Cloudmask */
  MODFILE	  *modinfprof = NULL;		  /* input file Aprofile */
  MODFILE	  *modinfsnow = NULL;		  /* input file Snow-cover */
  MODFILE	  *modinflc = NULL;		  /* input file landcov */
  MODFILE	  *update_handle = NULL;	/* input file update_for_lst */
  PGSt_integer    Vers_lc = 1;
  PGSt_integer    Vers_tmp = 1;
  PGSt_integer    n_tiles = 0;

  /* variables for reading input file */

  char	   modstring[MAX_NC_NAME] = {0};	/* used in complex messages */
  char	   fname_L1a[MAX_NC_NAME] = {NULL};	/* MAX_NC_NAME = 256 shown */
  char	   fname_L1b[MAX_NC_NAME] = {NULL}; 	/* in HDF4.1r1/include */
  char     fname_brdf[MAX_NC_NAME] = {NULL};	/* for MOD43B1 or MOD43B1C */
  char	   fname_Cld[MAX_NC_NAME] = {NULL}; 
  char	   fname_Aprof[MAX_NC_NAME] = {NULL}; 
  char	   fname_Snow[MAX_NC_NAME] = {NULL}; 
  char	   fname_LC[MAX_NC_NAME] = {NULL}; 
  char     upd_fname[MAX_NC_NAME] = {NULL};
  char	   data_type[DATATYPELENMAX];
  char	   scan_type[MAX_NC_ATTRS];
  int32	   n_day_pixels = 0, n_nit_pixels = 0;
  int32	   ntot_day_pixels = 0, ntot_nit_pixels = 0;
  int32	   n_cld_pixels = 0, tot_cld_pixels = 0;

char                  HaltMsg[2048];
char smf_message[PGS_SMF_MAX_MSGBUF_SIZE]={0};
char add_message[PGS_SMF_MAX_MSGBUF_SIZE]={0};

#define HALT(a)       { sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, a);\
			PGS_SMF_GenerateStatusReport(HaltMsg); \
			free_lst_space(); exit(1);   }

#define WARN(a)       { sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, a);\
			PGS_SMF_GenerateStatusReport(HaltMsg); }

#define HALTsmf(mnemon, a)      { \
	PGS_SMF_GetMsgByCode(mnemon, smf_message); \
	sprintf(HaltMsg,"%s[%d]: %s %s", __FILE__, __LINE__, smf_message, a);\
	PGS_SMF_GenerateStatusReport(HaltMsg); \
	free_lst_space(); exit(1);   }

#define modsmf_id_vers(i1, i2)	{ \
	sprintf (modstring, "Logical Id %d Version %d", i1, i2); \
	sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, modstring);\
	PGS_SMF_GenerateStatusReport(HaltMsg); \
	free_lst_space();	exit (1); }

  /* used for getting information from Core Attributes, its current size
     is 9089 bytes for L1A, 17366 bytes for L1B, 25032 bytes for Cmask,
     and 14994 bytes for snow.
     We define strings with sizes of about 1.25 time the current sizes */

#define    N_BYTES_CORE_1A			31288
#define    N_BYTES_CORE_1B			21704
#define    N_BYTES_CORE_PROF			21704
#define    N_BYTES_CORE_SNOW			18736
#define    N_BYTES_ARCH_1A			14960
char	   core_1A[N_BYTES_CORE_1A];		/* also for Cmask and Prof */
char	   core_1B[N_BYTES_CORE_1B];
char	   core_PROF[N_BYTES_CORE_PROF];
char	   core_Snow[N_BYTES_CORE_SNOW];
char	   arch_1A[N_BYTES_ARCH_1A];

int32 get_meta_attr(char *s, char *p, int32 type, int32 num, char *d);

#define rm_path(s, p) { p = ( s[0] == '/' || s[0] == '.') ? strrchr ( s, '/' ) + 1 : s; }

int32      iday_of_year(int32, int32, int32);

char	   datetime[MAX_NC_NAME];
char	   *pattern_date = "RANGEBEGINNINGDATE";
char	   *pattern_time = "RANGEBEGINNINGTIME";
char	   *s_start;
int32	   iyr, imonth, iday;
int32	   ihrs, iminutes;
float32  seconds;
float64  seconds_start;
int32	   i_day_of_yr_d = 0;
int32	   i_day_of_yr_n = 0;
static int32	   i_day_of_yr_curr = 0;
int32	   i_data_date_d = 0;
int32	   i_data_date_n = 0;
int32	   i_data_date_curr = 0;
int32	   i_time_pix = 0;

uparam_t uparam;      		/* user parameters data structure */
uparam_t *uparam_p = &uparam;

int32    n_read_times = 0;
int32    n_read_lines = 10;           /* read 10 lines each time */
int32	 i_subset_1b = 0;             /* input full 1B as default */

char     dtype[] = "uint16";          /* Array type for EV_1KM_Emissive */
char     arrnm[] = "EV_1KM_Emissive";         /* Array name */
char     arrnm_unc[] = "EV_1KM_Emissive_Uncert_Indexes";
char     arrnm_subset[] = "EV_1KM_SUBSET";    /* Array name in subset 1B */
char     arrnm_subset_unc[] = "EV_1KM_SUBSET_Uncert_Indexes";
char     grpnm[] = "\0";                      /* Group name */
/*arrnm_snow changed from Snow Cover to Snow_Cover by C. Collins on 11/3/04*/
/*
char	 arrnm_snow1[] = "Snow Cover";
char	 arrnm_snow2[] = "Snow_Cover";
*/

long int dims[3] = {1, 100, 1354};    /* Array dimensions for input L1B */
long int dims_o[2] = {100, 1354};     /* Array dimensions for L2 */
long int sta[3] = {0, 0, 0};          /* Array start indices */
long int sta_o[2] = {0, 0};
long int sta_o_lat[2] = {0, 0};
long int rank = MAX_VAR_DIMS;
long int dim_prof[MAX_VAR_DIMS];
long int dim_l_sizes[MAX_VAR_DIMS];
long int rank_brdf = MAX_VAR_DIMS;
long int dim_brdf_sizes[MAX_VAR_DIMS];
/*added send rank_snow and dim_snow_sizes on 11/9/04 C. Collins*/
/*
long int rank_snow1 = MAX_VAR_DIMS;
long int dim_snow_sizes1[MAX_VAR_DIMS];
long int rank_snow2 = MAX_VAR_DIMS;
long int dim_snow_sizes2[MAX_VAR_DIMS];
*/
int32    sd_id, attr_index;
int32    kk = 0;
int32    dim_sizes[MAX_VAR_DIMS];

/* arrary dim. for update_for_lst_sv's I/O */
long int upd_dims[3] = {TILE_YDIM_6KM, TILE_XDIM_6KM, TILE_ZDIM_UPD};
long int upd_sta[3] = { 0, 0, 0};

int32	n_update_visit = 0;
int32	nn_lines = 0;
int32	n_pix_per_line = 0;
tile_t  tile;		/* tile structure */

int32   igran;		/* granule index */

void Granule_tile(int32);
void SMFLOG(char *a);

/* R. Cember: for calls to MAPI function getMODISfileinfo() */
char sd_attr_type[MAX_NC_ATTRS]; 
char sd_attr_name[MAX_NC_NAME];
long nelements;

/* R. Cember: add declaration. */
char trash_char[5];

PGSt_SMF_status      current_time_a(char *);

void          *local = NULL;

PGSt_double	tile_bound[4] = {2.F};

PGSt_double	tile_LatLons[2][4];
PGSt_double	tile_LatLons_edge[2][4];

PGSt_double	lat_6km_1row = 0.0;

gran_link_t	gran_link;

PGSt_integer   	rel_vers_gran = 1;
PGSt_integer   	vers_gran = 1;
PGSt_integer   	Vers2 = 1;
char     fname_L2[MAX_NC_NAME] = {NULL};
MODFILE  *lst_l2_outf = NULL;
int32    n_l2_old_file;

MODFILE	 *lst_1km_l3_outf = NULL;
MODFILE	 *lst_6km_l3_outf = NULL;

char     fname_1km_L3[MAX_NC_NAME] = {NULL};
char     fname_6km_L3[MAX_NC_NAME] = {NULL};

/* processed good and other quality pixels,
   not processed due to cloud and other reasons, respectively */

int32	total_l2_pix[4] = { 0, 0, 0, 0 };
int32	total_l3_1km[4] = { 0, 0, 0, 0 };
int32	total_l3_6km_dn[4] = { 0, 0, 0, 0 };

int32	n_regions;
int32	i_regions[13];

char			DayNightFlag[32];
char			PGEVERSION_L1B[32];
char			PGEVERSION_MOD07[32];
enum AutoQualFlag	AutomaticQualityFlag = PASSED;
MET_counts		met_cnt_L2;
MET_counts		met_cnt_1km;
MET_counts		met_cnt_6km;
MET_counts		*met_cnt;

char    CharVals[10][PGSd_PC_VALUE_LENGTH_MAX];

char	*CharStg2[] = {
	 "RANGEBEGINNINGDATE",
	 "RANGEBEGINNINGTIME",
	 "RANGEENDINGDATE",
	 "RANGEENDINGTIME",
	 "EXCLUSIONGRINGFLAG.1",
	 "EQUATORCROSSINGDATE.1",
	 "EQUATORCROSSINGTIME.1",
	 "OPERATIONMODE",
	 "PGEVERSION",
	 "ADDITIONALATTRIBUTENAME.1" };

char    date_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char    time_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char    date_end[2][PGSd_PC_VALUE_LENGTH_MAX];
char    time_end[2][PGSd_PC_VALUE_LENGTH_MAX];

int32	tile_hours, tile_minutes;
float64 tile_begin_seconds;	
float64 tile_end_seconds, seconds_tmp;
int32	i_new_l1b = 1;
int32	i_new_mod07 = 1;

void sec_to_hrs (float64 *, int32 *, int32 *);

/* R. Cember: making this main void to quiet the fullwarn flag since exit
   from main is through SMFLOG(). */

uint8 u8_0cons = 0, u8_1cons = 1, u8_2cons = 2, u8_3cons = 3;
uint8 u8_4cons = 4, u8_5cons = 5, u8_6cons = 6, u8_7cons = 7;
uint16 u16_0cons = 0, u16_1cons = 1, u16_2cons = 2, u16_3cons = 3;

int32   npx_bad_split_window = 0, npx_bad_day_nit = 0;
int32   max_EV_1KM_Emissive_Uncert_Indexes = 0;
int32   i_ignore_uncert = 0;
int32	is_a_side_data = 0;
int32	i_calib_correction = 0;
int32	j_scan_start_pix = 10;	/* in response to the new change in MOD07_L2
				   v3.1.0: the first 10 pixels are omitted */

/* updated by Z. Wan 2001/01/08 */
float32	dt_avg_b31_b32_a = 0.16445, dt_avg_b31_b32_b = -0.05799;
float32	dt_dif_b31_b32_a = 0.02467, dt_dif_b31_b32_b =  0.26541;

char    esdt_L1B[16];
char    esdt_L2[16] = "MOD11_L2";
char    esdt_1km_L3[8] = "MOD11A1";
char    esdt_6km_L3[8] = "MOD11B1";
char    esdt_lst_upd[16] = "MOD11UPD";
char    *esdt_tmp;
int32	i_fm1 = 1;

char	ih_lc_a[16], iv_lc_a[16];
int16	ih_lc, iv_lc;
int32	*ver_landcv_to_lut;
int32	id1, id2;
int32	i_stop, n_stops[11];
int32	i_have_brdf = 0;
int32	i_brdf = 0;
int32	n_b7_alpha = 0;
int32	n_6km_water = 0;
int32	n_6km_water_sunglint = 0;
PGSt_integer numfiles_brdf = 0;
/*next defines of variable added by C. Collins on 10/20/04 for MOD43B1 products*/
PGSt_integer numfiles_brdf1km = 0;
PGSt_integer brdf_num_files =0;

int32	i_lake_pix = 0;
int32	i_hiland = 0;
int32   total_space = 0;
uint16	u16_qa_l2_man, u16_qa_l2_dat, u16_qa_l2_cld, u16_qa_l2_emer;

/*int32	snow_cv_use = 0;*/
int32	brdf_lun = 0;
int32	swath_snow_size;

main(int32 argc, char **argv) 
{

   PGSt_integer numfiles_l1b;
   PGSt_integer numfiles_geoloc;
   PGSt_integer numfiles_cmask;
   PGSt_integer numfiles_aprof;
   PGSt_integer numfiles_snow;
   PGSt_integer numfiles_mod11A1;
   PGSt_integer numfiles_landcov;
   _LAND_TILE_TBL      *tile_tbl_p;

   size_t     siz = 0;

   char  *InputStg[4][2] = {
	"ORBITNUMBER.1", "ORBITNUMBER",
	"LOCALGRANULEID", "LOCALGRANULEID",
	"PARAMETERVALUE.6", "PARAMETERVALUE.6", /* tile horizontal num */
	"PARAMETERVALUE.7", "PARAMETERVALUE.7"  /* tile vertical num */
	};
   char  CharV[2][PGSd_PC_VALUE_LENGTH_MAX];
   char  *pattern_id = ".A";

   /* R. Cember: use a character string rather than a pointor in order to avoid
   memory leaks by stepping on memory locations just a little higher than the
   location of void pointer "local". */
   char  str_start[MAX_NC_NAME];

   char  *str_sub1, *str_sub2;
   int32 i_match = 0;

   int32 returnStatus;

   int32 sl, i_ch3;

   int32 ii, jj;

   int32 ier = 0;
   int32 i1 = 0;
   int32 i = 0;
   int32 j = 0;
/*next line added by C. Collins
   int32 get_arrnm_snow1;
   int32 get_arrnm_snow2;
*/
   int32   i_insert, i_rel_vers_1b;
   int32   i_prof, j_prof, ii_prof, jj_prof;
   int32   i_skip_prof, kk_prof, i_clear_1km, n_clear_1km = 0;
   int32   k1_prof, i1_prof, i2_prof;
   float32 t_p_prof[14];
   float32 r_prof, tc_prof, dlp_prof, a_prof;
   float32 x_prof, pres_previous;
   uint8   *cm_prof_p, qa1;
   int16   *t_prof_p, *ps_prof_p;

#ifdef _LINUX_ct
   void ctSetup();
#endif

   /* to hold the PROCESSINGDATETIME metadata field */
   char protime[PGSd_PC_VALUE_LENGTH_MAX]="Error in PROCESSINGDATETIME";

   for ( i_stop = 0; i_stop < 11; i_stop++ )
	n_stops[i_stop] = 0;

   if (argc < 2) {
     sprintf (modstring, "no granule version provided");
     HALT(modstring);
   }

   /* set callerID first to get rid of the error message to LogStatus */
   PGS_SMF_SetCallerID(PGSd_CALLERID_SMF);

   /* Initialize the granule ids */

   uparam.ngran = 1;
   uparam.coverage_min = 0.02;
   uparam.itile_size = 1;
   strcpy(uparam_p->gridtype, "sin");

   /* it can be overridden in PCF */
   PGS_PC_GetConfigData(GRID_TYPE, uparam_p->gridtype);

   if (PGS_PC_GetConfigData(N_READ_LINES, pval) == PGS_S_SUCCESS)
  	n_read_lines = atoi (pval);

   if (PGS_PC_GetConfigData(SUBSET_1B, pval) == PGS_S_SUCCESS)
  	i_subset_1b = atoi (pval);

   if (PGS_PC_GetConfigData(IGNORE_UNCERT, pval) == PGS_S_SUCCESS)
  	i_ignore_uncert = atoi (pval);

   if (PGS_PC_GetConfigData(TURN_OFF_CALIBRATION_CORRECTION, pval) ==
		PGS_S_SUCCESS) {
	if ( atoi(pval) )
  		i_calib_correction = 0;
   }

   if (PGS_PC_GetConfigData(PGE16_Version, pgeversion) != PGS_S_SUCCESS) {
  	sprintf(modstring,"no PGE16_Version (LUN 800500) in PCF");
	HALT(modstring);
   }

   if (PGS_PC_GetConfigData(ReprocessingPlanned, rep_planned) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingPlanned info from pcf");

   if (PGS_PC_GetConfigData(ReprocessingActual, rep_actual) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingActual info from pcf");

   if (PGS_PC_GetConfigData(ScienceQualityFlagExplanation, sci_qua_exp) != PGS_S_SUCCESS)
	SMFLOG("Error getting ScienceQualityFlagExplanation info from pcf");

   if(PGS_PC_GetConfigData(ProcessingEnvironment, process_env) != PGS_S_SUCCESS)
	SMFLOG("Error getting ProcessingEnvironment info from pcf");

   if ( (n_read_lines % n_lines_per_scub) ) {
	sprintf(modstring,"n_read_lines (not multiple of 10)");
	HALT(modstring);
   }

   rel_vers_gran = atoi (argv[1]);

   if ( argc > 2 ) {
      if ( argc < 5 ) {
	 sprintf(modstring,"usage: %s vers_gran or %s vers_gran A 2 1 (or 2)",
		argv[0], argv[0] );
	 HALT(modstring);
      }
      id_char_update[0] = argv[2][0];
      n_days_per_cycle_update = atoi (argv[3]);
      i_days_in_cycle_update = atoi (argv[4]);
      i_do_multi_days = 1;
   }

   if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLog1B, &numfiles_l1b)) !=
                PGS_S_SUCCESS )
	SMFLOG("Error getting number of L1B files");

   if ( numfiles_l1b > MAXGRANULES )
        SMFLOG("STOP--too many L1B inputs");

   if ( numfiles_l1b == 0 )
        SMFLOG("STOP--there is no L1B granule input");

   if ( rel_vers_gran > numfiles_l1b ) {
	sprintf (LogMsg,
	 "Completed MOD_PR11 successfully (%d -all granules already processed)",
	  rel_vers_gran );
	SMFLOG(LogMsg);
   }

   /* place the granules into the time sequence order */

   siz = sizeof (uint16);
   gran_order = (uint16 **) alloc_2d ( numfiles_l1b, 2, siz );

   for ( i = 0; i < numfiles_l1b; i++ )
	gran_order[i][1] = 0;

   sprintf(sd_attr_name, CORES);
   sprintf(sd_attr_type, "char *");

   i_rel_vers_1b = 0;

   for (i = 0; i < numfiles_l1b; i++) {
      Vers1B = i + 1;
      Vers_tmp = Vers1B;

      if (PGS_PC_GetReference(PCLog1B, &Vers_tmp, fname_L1b) != PGS_S_SUCCESS)
		modsmf_id_vers(PCLog1B, Vers1B);

      rm_path(fname_L1b, esdt_tmp);
      sscanf ( &esdt_tmp[18], "%4s", time_1b_a);
      time_1b = (uint16) atoi (time_1b_a);

      i_insert = 0;
      for ( j = 0; j < i_rel_vers_1b; j++ ) {
	if ( time_1b < gran_order[j][0] ) {
		for ( kn = i_rel_vers_1b; kn > j; kn-- ) {
			gran_order[kn][0] = gran_order[kn-1][0];
			gran_order[kn][1] = gran_order[kn-1][1];
		}
		gran_order[j][0] = time_1b;
		gran_order[j][1] = Vers1B;
		i_insert = 1;
		break;
	}
      }
      if ( !i_insert ) {
		gran_order[i_rel_vers_1b][0] = time_1b;
		gran_order[i_rel_vers_1b][1] = Vers1B;
      }
      i_rel_vers_1b++;
   }

   if ( i_rel_vers_1b != numfiles_l1b )
        SMFLOG("STOP--error in ordering L1B input files");

   /* assign the real version number of the input L1B granule */

   vers_gran = gran_order[rel_vers_gran - 1][1];

   /* check if L2 file is already exists */

   Vers2 = vers_gran;	Vers_tmp = Vers2;
   if (PGS_PC_GetReference(PCLogL2, &Vers_tmp, fname_L2) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL2, Vers2);

   if ( Hishdf (fname_L2) == TRUE ) {

     SMFLOG ("STOP - L2 file already exists");
   }

   rm_path(fname_L2, esdt_tmp);
   sscanf ( esdt_tmp, "%8s", &esdt_L2[0] );

   gran_link.gran1B_ver_n = vers_gran;
   gran_link.geo1A_ver_n = 0;
   gran_link.cmask_ver_n = 0;
   gran_link.snow_ver_n = 0;

  /* open L1B input file */
  Vers1B = vers_gran;
  Vers_tmp = Vers1B;
  if (PGS_PC_GetReference(PCLog1B, &Vers_tmp, fname_L1b) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLog1B, vers_gran);

  rm_path(fname_L1b, esdt_tmp);
  sscanf ( esdt_tmp, "%8s", &esdt_L1B[0] );
  if ( strcmp(esdt_L1B, "MOD021KM" ) == 0 )	i_fm1 = 0;
  else						i_fm1 = 1;

  /* R. Cember: getting UR of L1B for later use in setting metadata. */
  Vers_tmp = Vers1B;
  if (PGS_PC_GetUniversalRef(PCLog1B, &Vers_tmp, UR_L1b) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLog1B, vers_gran);

  if ((modinf1b = openMODISfile (fname_L1b, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_L1b);
	HALT(modstring);
  }

  sprintf(sd_attr_name, CORES);
  sprintf(sd_attr_type, "char *");
  nelements = (long) N_BYTES_CORE_1B;
  if (getMODISfileinfo(modinf1b, sd_attr_name, sd_attr_type, &nelements,
		   core_1B) != MAPIOK)
	Error("Re: Level 1B: failure on call to ", "getMODISfileinfo");

  /* ## get the PGEVERSION */
  local = &PGEVERSION_L1B[0];
  if ( get_meta_attr (core_1B, "PGEVERSION", 1, 1, (char *) local ) == FAIL)
	HALT("get_meta_attr fails on core_1B");

  /* get the DayNightFlag mode */
  local = &DayNightFlag[0];
  if ( get_meta_attr (core_1B, "DAYNIGHTFLAG", 1, 1, (char *) local )
		== FAIL)
	HALT("get_meta_attr fails on core_1B");

  if ( get_meta_attr (core_1B, InputStg[0][0], 2, 1, (char *) &id1) == FAIL)
	HALT("get_meta_attr fails on core_1B");
  gran_link.orbit_n = id1;

  local = CharV[0];
  if ( get_meta_attr (core_1B, InputStg[1][0], 1, 1, (char *) local ) == FAIL)
	HALT("get_meta_attr fails on core_1B");

  strcpy(str_start, strstr (local, pattern_id) );

  sscanf (str_start+1, "%13s", gran_link.local_gran_id);
  sscanf (str_start+1, "%8s", data_day);
  sscanf (str_start+2, "%4s", data_yr);

  i_data_yr = atoi (data_yr) - 2000;
  i_data_day = atoi (&data_day[5]);

  if ( !i_days_in_cycle_update ) {
	i_days_in_cycle_update = i_data_day % 2;
	if ( !i_days_in_cycle_update ) i_days_in_cycle_update = 2;
  }

  if ( i_do_multi_days ) {
	if ( ( i_days_in_cycle_update == 1 &&
		( (i_data_day % n_days_per_cycle_update ) != 1 ) ) ||
	     ( i_days_in_cycle_update == 2 &&
		( (i_data_day % n_days_per_cycle_update ) != 0 ) ) ) {

		sprintf (msgString, "wrong UPD file used");
		HALT(msgString);
	}
  }

  sscanf (str_start+10, "%4s", granule_hrs);

  /* determine is_a_side_data (old) by the data day */
  if ( strcmp(data_day, "A2000306") < 0 )
	is_a_side_data = 1;

  /* it can be overridden in PCF */
  if (PGS_PC_GetConfigData(IS_A_SIDE_DATA, pval) == PGS_S_SUCCESS)
	is_a_side_data = atoi (pval);

  /* read in latitude & longitude from MOD02 */
     /*     added by G. Fireman 2005-09-15 */
  {
    int32        dim_d1, dim_d2;
    if (getMODISardims(modinf1b, geo_5pix[0].name, geo_5pix[0].gname, data_type,
		       &geo_5pix[0].rank, geo_5pix[0].dims) == FAIL) {
      HALTsmf(MODIS_F_MAPI_FAIL, "getMODISardims, GEO_SDS");
    }
    siz = sizeof (float32);
    dim_d1 = geo_5pix[0].dims[0];
    dim_d2 = geo_5pix[0].dims[1];
    geo_5pix[1].dims[0] = dim_d1;
    geo_5pix[1].dims[1] = dim_d2;
    lst_l2[7].dims[0] = lst_l2[8].dims[0] = dim_d1;
    lst_l2[7].dims[1] = lst_l2[8].dims[1] = dim_d2;
    lat_5pix = (float32 **) alloc_2d (dim_d1, dim_d2, siz);
    lon_5pix = (float32 **) alloc_2d (dim_d1, dim_d2, siz);
    sta_geo2[0] = 0;
    if ((ier = getMODISarray(modinf1b, geo_5pix[0].name, geo_5pix[0].gname,
			     sta_geo2, &geo_5pix[0].dims[0], &lat_5pix[0][0])) == -1) {
      HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray MOD35 Latitude");
    }
    if ((ier = getMODISarray(modinf1b, geo_5pix[1].name, geo_5pix[1].gname,
			     sta_geo2, &geo_5pix[1].dims[0], &lon_5pix[0][0])) == -1) {
      HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray MOD35 Longitude");
    }
  }
  
  /* close the L1B file in MAPI function */
  returnStatus = closeMODISfile (&modinf1b);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "L1B file");

  /* Get "%Valid EV Observations" */
  {
    int32     sd_id_1km, data_type_1km, ncount = 38;
    float32   ValidEVobservations[38];

    if( (sd_id_1km = SDstart( fname_L1b, DFACC_READ )) == FAIL )
	HALT("Can't open the L1B input file.");

    if( (attr_index = SDfindattr(sd_id_1km, "%Valid EV Observations")) == FAIL)
	HALT("Can't get attr_index for '%Valid EV Observations' in L1B file");
/*
    if( SDattrinfo(sd_id_1km,attr_index,"%Valid EV Observations",
			&data_type_1km, &ncount) == FAIL)
	HALT("Can't get attrinfo for '%Valid EV Observations' in L1B file");

    if( (data_type_1km != DFNT_FLOAT32) || (ncount != 38) )
	HALT("sds_id_1km '%Valid EV Observations' has bad data_type and/or count.");
*/

    if( SDreadattr( sd_id_1km, attr_index, ValidEVobservations ) == FAIL )
	HALT("Can't read attribute for '%Valid EV Observations' in L1B file");

    if( SDend( sd_id_1km ) == FAIL )
	HALT("Failure with SDend for sd_id_1km for L1B");

    if ( ValidEVobservations[32] < 0.0001 && ValidEVobservations[33] < 0.0001 ) {

      free_lst_space ();
      sprintf(LogMsg, "Completed %s (%d for %s ignored due to no valid EV observations in bands 31 and 32)",
	esdt_L2, rel_vers_gran, gran_link.local_gran_id );
      SMFLOG(LogMsg);
    }
  }

  /* Get L1B swath handle, Band_1KM_Emssive scales and offsets. */
  {

#define MAX_SAMPS	1354

    int32     SWfidin, SWidin;
    char      *band1kmname_old = "Band_1KM_Emissive:MODIS_SWATH_Type_L1B";
    char      *band1kmname_new = "EV_1KM_Emissive";
    char      *band1kmname;
    int32     sd_id_1km, sds_id_1km, sds_index_1km, rank_1km, data_type_1km;
    int32     dimsizes_1km[MAX_VAR_DIMS], nattrs_1km;
    int32     NoScans, NoSamps, ncount = 16;
    int32     No_band_1km_emiss;
    int32     i_scale;

/* ## determine i_new_l1b */
    i_new_l1b = (strcmp (PGEVERSION_L1B, "2.1.1") >= 0) ? 1 : 0;

    if ( i_new_l1b )	band1kmname = band1kmname_new;
    else		band1kmname = band1kmname_old;

    if( (SWfidin = SWopen( fname_L1b, DFACC_READ )) == FAIL )
      HALT("SWopen failed for L1B file");

    if( (SWidin = SWattach( SWfidin, "MODIS_SWATH_Type_L1B" )) == FAIL )
      HALT("SWattach failed for L1B file");

    if( (No_band_1km_emiss = SWdiminfo(SWidin, "Band_1KM_Emissive")) != 16 )
      HALT("Swath dimension for Band_1KM_Emissive != 16");

    if( (NoScans =  SWdiminfo(SWidin, "10*nscans")) == -1 )
      HALT("Swath dimension for 10*nscans failed.");

    if( (NoSamps =  SWdiminfo(SWidin, "Max_EV_frames")) == -1 )
      HALT("Swath dimension for Max_EV_frames failed.");

    if( NoSamps > MAX_SAMPS )
	HALT("Max_EV_Frames is greater than MAX_SAMPS");

    if( (sd_id_1km = SDstart( fname_L1b, DFACC_READ )) == FAIL )
	HALT("Can't open the L1B input file.");

    if( (sds_index_1km = SDnametoindex( sd_id_1km, band1kmname )) == FAIL )
	HALT("sds_index failed in L1B file");

    if( (sds_id_1km = SDselect( sd_id_1km, sds_index_1km ) ) == FAIL )
	HALT("sds_id_1km failed in L1B file");
/*
    if(SDgetinfo( sds_id_1km, band1kmname, &rank_1km,
		    dimsizes_1km, &data_type_1km, &nattrs_1km) == FAIL )
	HALT("SDgetinfo failed for band1kmname in L1B file");
*/

    /* Get scales */
    if( (attr_index = SDfindattr( sds_id_1km, "radiance_scales" )) == FAIL)
	HALT("Can't get attr_index for 'radiance_scales' in L1B file");

/*
    if( SDattrinfo(sds_id_1km,attr_index,"radiance_scales",
		     &data_type_1km, &ncount) == FAIL)
	HALT("Can't get attrinfo for 'radiance_scales' in L1B file");

    if( (data_type_1km != DFNT_FLOAT32) || (ncount != 16) )
	HALT("sds_id_1km radiance_scales has bad data_type and/or count.");
*/

    if( SDreadattr( sds_id_1km, attr_index, scales ) == FAIL )
	HALT("Can't read attribute for 'radiance_scales' in L1B file");

    /* Get offsets */
    if( (attr_index = SDfindattr( sds_id_1km, "radiance_offsets" )) == FAIL)
	HALT("Can't get attr_index for 'radiance_offsets' in L1B file");
/*
    if( SDattrinfo(sds_id_1km,attr_index,"radiance_offsets",
		     &data_type_1km, &ncount) == FAIL)
	HALT("Can't get attrinfo for 'radiance_offsets' in L1B file");

    if( (data_type_1km != DFNT_FLOAT32) || (ncount != 16) )
	HALT("sds_id_1km radiance_offsets has bad data_type and/or count.");
*/

    if( SDreadattr( sds_id_1km, attr_index, offsets ) == FAIL )
	HALT("Can't read attribute for 'radiance_offsets' in L1B file");

    if( SDendaccess( sds_id_1km ) == FAIL )
	HALT("Cannot SDendaccess for sds_id_1km");
  
    /* close the L1B file */
    if( SWdetach( SWidin ) == FAIL )
	  HALT("Failure with SWdetach for L1B");

    if( SWclose( SWfidin ) == FAIL )
	  HALT("Failure with SWclose for L1B");

    if( SDend( sd_id_1km ) == FAIL )
	  HALT("Failure with SDend for sd_id for L1B");
  }

   memset( &met_cnt_L2.FirstEntryInStruct, 0, sizeof( MET_counts ) );

   n_regions = 0;

  if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLog1A, &numfiles_geoloc) ) !=
		PGS_S_SUCCESS )
	SMFLOG("Error getting number of files on geolocation");

  if ( numfiles_geoloc < numfiles_l1b ) {
	sprintf (LogMsg, "STOP-- %d geoloc file(s) short in PCF",
		numfiles_l1b - numfiles_geoloc );
	SMFLOG(LogMsg);
  }

  if ( numfiles_geoloc > MAXGRANULES )
	SMFLOG("STOP--too many geoloc inputs");

  i_match = 0;
  sprintf(sd_attr_name, CORES);
  sprintf(sd_attr_type, "char *");
  for (i = 0; i < numfiles_geoloc; i++) {
     Vers1A = i + 1;
     Vers_tmp = Vers1A;

     if (PGS_PC_GetReference(PCLog1A, &Vers_tmp, fname_L1a) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLog1A, Vers1A);

     if ((modinf1a = openMODISfile (fname_L1a, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_L1a);
	HALT(modstring);
     }
     nelements = (long) N_BYTES_CORE_1A;
     memset ( &core_1A[0], '\0', nelements );
     if (getMODISfileinfo(modinf1a, sd_attr_name, sd_attr_type, &nelements,
		core_1A) != MAPIOK)
	Error("Re: Geo 1A: failure on call to ", "getMODISfileinfo");
     closeMODISfile (&modinf1a);

     if ( get_meta_attr(core_1A, InputStg[0][0], 2, 1, (char *) &id1) == FAIL)
	HALT("get_meta_attr fails on core_1A");

     local = CharV[0];
     if ( get_meta_attr(core_1A, InputStg[1][0], 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_1A");

     strcpy(str_start, strstr (local, pattern_id) );

/* do not use the whole LocalGranuleId because the data day may be wrong

     sscanf (str_start+1, "%13s", CharV[1]);

   only use the beginning time of the granule, Z. Wan, 3/28/2002 */

     sscanf (str_start+10, "%4s", CharV[1]);

/*
     if (gran_link.orbit_n == id1 &&
	 ! strcmp(gran_link.local_gran_id, CharV[1]) )
*/
/* modify on 3/28/2002
     if (
	 ! strcmp(gran_link.local_gran_id, CharV[1]) )
*/
     if (
	 ! strcmp(granule_hrs, CharV[1]) ) {

	i_match = 1;
	gran_link.geo1A_ver_n = Vers1A;
	break;
     }
  }
  if ( !i_match ) {
	sprintf (modstring, "no geo file matches the version %d L1B file",
		 Vers1B);
	HALT(modstring);
  }

  if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLogCmask, &numfiles_cmask) )
		!= PGS_S_SUCCESS )
	SMFLOG("Error getting number of files on Cmask");
  if ( numfiles_cmask > MAXGRANULES )
	SMFLOG("STOP--too many Cmask input files");
  if ( numfiles_cmask < numfiles_l1b ) {
	sprintf (LogMsg, "STOP-- %d Cmask file(s) short in PCF",
		numfiles_l1b - numfiles_cmask );
	SMFLOG(LogMsg);
  }

  i_match = 0;
  for (i = 0; i < numfiles_cmask; i++) {
     VersCmask = i + 1;
     Vers_tmp = VersCmask;

     if (PGS_PC_GetReference(PCLogCmask, &Vers_tmp, fname_Cld)
		!= PGS_S_SUCCESS)
	modsmf_id_vers(PCLogCmask, VersCmask);

     if ((modinfcld = openMODISfile (fname_Cld, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_Cld);
	HALT(modstring);
     }
     nelements = (long) N_BYTES_CORE_1A;
     memset ( &core_1A[0], '\0', nelements );
     if (getMODISfileinfo(modinfcld, sd_attr_name, sd_attr_type, &nelements,
		core_1A) != MAPIOK)
	Error("Re: Cmask: failure on call to ", "getMODISfileinfo");
     closeMODISfile (&modinfcld);
     if ( get_meta_attr(core_1A, InputStg[0][0], 2, 1, (char *) &id1) == FAIL)
	HALT("get_meta_attr fails on core_Cmask");

     local = CharV[0];
     if ( get_meta_attr(core_1A, InputStg[1][0], 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_Cmask");

     strcpy(str_start, strstr (local, pattern_id) );

     sscanf (str_start+1, "%13s", CharV[1]);

     if (gran_link.orbit_n == id1 &&
		!strcmp(gran_link.local_gran_id, CharV[1]) ) {

	i_match = 1;
	gran_link.cmask_ver_n = VersCmask;
	break;
     }
  }
  if ( !i_match ) {
	sprintf (modstring, "No Cmask matches the version %d L1B file", Vers1B);
	HALT(modstring);
  }

  if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLogProfile, &numfiles_aprof) )
		!= PGS_S_SUCCESS )
	SMFLOG("Error getting number of files on Aprof");
  if ( numfiles_aprof > MAXGRANULES )
	SMFLOG("STOP--too many Aprof input files");
  if ( numfiles_aprof < numfiles_l1b ) {
	sprintf (LogMsg, "STOP-- %d Aprof file(s) short in PCF",
		numfiles_l1b - numfiles_aprof );
	SMFLOG(LogMsg);
  }

  i_match = 0;
  for (i = 0; i < numfiles_aprof; i++) {
     VersAprof = i + 1;
     Vers_tmp = VersAprof;

     if (PGS_PC_GetReference(PCLogProfile, &Vers_tmp, fname_Aprof)
		!= PGS_S_SUCCESS)
	modsmf_id_vers(PCLogProfile, VersAprof);

     if ((modinfprof = openMODISfile (fname_Aprof, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_Aprof);
	HALT(modstring);
     }
     nelements = (long) N_BYTES_CORE_1A;
     memset ( &core_1A[0], '\0', nelements );
     if (getMODISfileinfo(modinfprof, sd_attr_name, sd_attr_type, &nelements,
		core_1A) != MAPIOK)
	Error("Re: Cmask: failure on call to ", "getMODISfileinfo");
     closeMODISfile (&modinfprof);
     if ( get_meta_attr(core_1A, InputStg[0][0], 2, 1, (char *) &id1) == FAIL)
	HALT("get_meta_attr fails on core_Aprof");

     local = CharV[0];
     if ( get_meta_attr(core_1A, InputStg[1][0], 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_Aprof");

     strcpy(str_start, strstr (local, pattern_id) );

     sscanf (str_start+1, "%13s", CharV[1]);

     if (gran_link.orbit_n == id1 &&
		!strcmp(gran_link.local_gran_id, CharV[1]) ) {

	i_match = 1;
	gran_link.aprof_ver_n = VersAprof;
	break;
     }
  }
  if ( !i_match ) {
	sprintf (modstring, "No Aprof matches the version %d L1B file", Vers1B);
	HALTsmf(MODIS_F_FAIL_MATCH, modstring);
  }

  returnStatus = PGS_PC_GetNumberOfFiles(PCLogSnow, &numfiles_snow);
  if ( numfiles_snow > MAXGRANULES )
	SMFLOG("STOP--too many snow input files");

  i_match = 0;
  for (i = 0; i < numfiles_snow; i++) {
     VersSnow = i + 1;
     Vers_tmp = VersSnow;
     if (PGS_PC_GetReference(PCLogSnow, &Vers_tmp, fname_Snow) !=
		PGS_S_SUCCESS)
	modsmf_id_vers(PCLogSnow, VersSnow);

     if ((modinfsnow = openMODISfile (fname_Snow, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_Snow);
	HALT(modstring);
     }
     sprintf(sd_attr_name, CORES);
     sprintf(sd_attr_type, "char *");
     nelements = (long) N_BYTES_CORE_SNOW;
     if (getMODISfileinfo(modinfsnow, sd_attr_name, sd_attr_type, &nelements,
		core_Snow) != MAPIOK)
	Error("Re: Snow: failure on call to ", "getMODISfileinfo");
     closeMODISfile (&modinfsnow);

     if ( get_meta_attr(core_Snow, InputStg[0][0], 2, 1, (char *) &id1) == FAIL)
	HALT("get_meta_attr fails on core_Snow");
     local = CharV[0];
     if ( get_meta_attr(core_Snow, InputStg[1][0], 1, 1, (char *)local) == FAIL)
	HALT("get_meta_attr fails on core_Snow");

     strcpy(str_start, strstr (local, pattern_id) ); 

     sscanf (str_start+1, "%13s", CharV[1]);

     if ( gran_link.orbit_n == id1 &&
		!strcmp(gran_link.local_gran_id, CharV[1]) ) {

	i_match = 1;
	gran_link.snow_ver_n = VersSnow;
	break;
     }
  }

  returnStatus = PGS_PC_GetNumberOfFiles(PCLogL3_1km, &numfiles_mod11A1);
  if ( numfiles_mod11A1 > N_LAND_TILE )
	SMFLOG("STOP--too many output MOD11A1 files");

  returnStatus = PGS_PC_GetNumberOfFiles(PCLogLC, &numfiles_landcov);
  if ( numfiles_landcov < numfiles_mod11A1 )
	SMFLOG("STOP--not enough input MOD12Q1 files");

  returnStatus = PGS_PC_GetNumberOfFiles(PCLogupdate[i_days_in_cycle_update-1],
			&numfiles_update[i_days_in_cycle_update-1]);
  if ( numfiles_update[i_days_in_cycle_update-1] != N_LAND_TILE )
	SMFLOG("STOP--not enough PCLogupdate");

  returnStatus = PGS_PC_GetNumberOfFiles(PCLogBRDF, &numfiles_brdf);
  /*next line added by C.collins on 10/20/04 for MOD43B1 products */
  returnStatus = PGS_PC_GetNumberOfFiles(PCLogBRDF1km, &numfiles_brdf1km);

  ver_landcv_to_lut = (int32 *) malloc (numfiles_landcov * sizeof (int32));

  for (i = 0; i < numfiles_landcov; i++)
	ver_landcv_to_lut[i] = 0;

  for (i = 0; i < numfiles_landcov; i++) {
     Vers_lc = i + 1;
     Vers_tmp = Vers_lc;
     if (PGS_PC_GetReference(PCLogLC, &Vers_tmp, fname_LC) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogLC, Vers_lc);

     rm_path(fname_LC, esdt_tmp);
     sscanf ( &esdt_tmp[18], "%2s%c%2s", ih_lc_a, &trash_char[0], iv_lc_a );
     ih_lc = atoi (ih_lc_a);
     iv_lc = atoi (iv_lc_a);

     for ( id2 = 0; id2 < N_LAND_TILE; id2++ ) {
	tile_tbl_p = &land_tile_tbl[id2];
	if ( ih_lc == tile_tbl_p->i_horiz && iv_lc == tile_tbl_p->i_vert ) {
		ver_landcv_to_lut[id2] = Vers_lc;
		break;
	}
     }
  }
  for ( i = 0; i < N_LAND_TILE; i++ ) {
     if ( ver_landcv_to_lut[i] <= 0 ) {
	sprintf(modstring,"no MOD12Q1 file matches the version %d MOD11A1 file",
		i + 1);
	HALT(modstring);
     }
  }

  /* get PROCESSINGDATETIME */
  if(current_time_a(protime) != MODIS_S_SUCCESS) {
    sprintf(modstring,
	"the PROCESSINGDATETIME metadata field will not be written correctly.");
    HALT(modstring);
  }

  /* read in L1B data dimension */

  if ((modinf1b = openMODISfile (fname_L1b, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_L1b);
	HALT(modstring);
  }

  if ( i_subset_1b == 0 ) {
   if (getMODISardims(modinf1b, arrnm, grpnm, data_type, &rank, dim_l_sizes)
		== FAIL) {
	sprintf (modstring, "getMODISardims of EV_1KM_Emissive failed");
	HALT(modstring);
   }
  }
  else {
   if (getMODISardims(modinf1b,arrnm_subset,grpnm,data_type,&rank,dim_l_sizes)
		== FAIL) {
	sprintf (modstring, "getMODISardims of EV_1KM_SUBSET failed");
	HALT(modstring);
   }
  }

  for (j = 0; j < rank; j++)
    dim_sizes[j] = dim_l_sizes[j];

  n_pix_per_line = dim_sizes[2];

  n_read_times = dim_sizes[1] / n_read_lines;

  if ( n_read_times > 1 )
	multi_write_out = 1;

  /* read in snow-cover data dimension */

  VersSnow = gran_link.snow_ver_n;
  Vers_tmp = VersSnow;
  modinfsnow = NULL;
  if ( VersSnow ) {
    if (PGS_PC_GetReference(PCLogSnow, &Vers_tmp,fname_Snow) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogSnow, VersSnow);

{	/* update the name of SDS "Snow_Cover" or "Snow Cover" */
  int32	SWfidin, SWidin;
  char	snow_fldlist[128];
  int32	sw_rank[8];
  int32	sw_ntypes[8];
  int32	nflds, ic_snow;

  SWfidin = SWopen(fname_Snow, DFACC_READ );
  SWidin = SWattach( SWfidin, "MOD_Swath_Snow" );
  nflds = SWinqdatafields(SWidin, snow_fldlist, &sw_rank[0], &sw_ntypes[0] );
  /*sscanf (&snow_fldlist[0], "%10s", &arrnm_snow[0] ); it may not work */
  for ( ic_snow = 0; ic_snow < 10; ic_snow++ )
	arrnm_snow[ic_snow] = snow_fldlist[ic_snow];
  arrnm_snow[10] = '\0';
  /*fprintf(stderr, "arrnm_snow %s\n", arrnm_snow); fflush(stderr);*/
  SWdetach( SWidin );
  SWclose( SWfidin );
}

    if ((modinfsnow = openMODISfile (fname_Snow, "r")) == NULL) {
	sprintf (modstring, "failed to open %s", fname_Snow);
	HALT(modstring);
    }

/*next block modified by c. Collins so code will look for either Snow Cover or
Snow_Cover and be able to process old or new MOD10_L2 products*/
/* the next block modified in order to avoid the error messages in LogStarus, Z. Wan, 5/11/05
    if ((get_arrnm_snow2 = getMODISardims(modinfsnow,arrnm_snow2,grpnm,dtype_snow,&rank_snow2, dim_snow_sizes2))== FAIL) {
            if ((get_arrnm_snow1 = getMODISardims(modinfsnow,arrnm_snow1,grpnm,dtype_snow,&rank_snow1, dim_snow_sizes1)) == FAIL) {
		sprintf (modstring, "SDS for snow product not found.");
		HALT(modstring);
	}
	snow_cv_use = 1;
    }
    else
	snow_cv_use = 2;

    if ( snow_cv_use == 1 ) {
      if ( dim_snow_sizes1[0] != ( 2 * dim_l_sizes[1] ) ||
	   dim_snow_sizes1[1] != ( 2 * dim_l_sizes[2] ) ) {

	sprintf(LogMsg, "check dimension of SDS %s in %s", arrnm_snow1,fname_Snow);
	HALT(LogMsg);

	gran_link.snow_ver_n = 0;
	closeMODISfile (&modinfsnow);
	modinfsnow = NULL;
    }
    snow_cv1[0].dims[0] = 2 * n_read_lines;
    snow_cv1[0].dims[1] = 2 * n_pix_per_line;
  }

    if ( snow_cv_use == 2 ) {
      if ( dim_snow_sizes2[0] != ( 2 * dim_l_sizes[1] ) ||
	   dim_snow_sizes2[1] != ( 2 * dim_l_sizes[2] ) ) {

	sprintf(LogMsg, "check dimension of SDS %s in %s", arrnm_snow2,fname_Snow);
	HALT(LogMsg);

	gran_link.snow_ver_n = 0;
	closeMODISfile (&modinfsnow);
	modinfsnow = NULL;
    }
    snow_cv2[0].dims[0] = 2 * n_read_lines;
    snow_cv2[0].dims[1] = 2 * n_pix_per_line;
   }
*/
    if ( getMODISardims(modinfsnow,arrnm_snow,grpnm,dtype_snow,&snow_cv[0].rank,snow_cv[0].dims)== FAIL) {
	sprintf (modstring, "getMODISardims of Snow Cover failed");
	HALT(modstring);
    }
    else {
      if ( snow_cv[0].dims[0] != ( 2 * dim_l_sizes[1] ) ||
	   snow_cv[0].dims[1] != ( 2 * dim_l_sizes[2] ) ) {

	sprintf(LogMsg, "check dimension of SDS %s in %s", snow_cv[0].name,fname_Snow);
	HALT(LogMsg);

	gran_link.snow_ver_n = 0;
	closeMODISfile (&modinfsnow);
	modinfsnow = NULL;
    }
    snow_cv[0].dims[0] = 2 * n_read_lines;
    snow_cv[0].dims[1] = 2 * n_pix_per_line;
   }
  }

  met_cnt_L2.AllPixelsProcessed = dim_sizes[1] * n_pix_per_line;

  /* open geoloc 1A file */
  Vers1A = gran_link.geo1A_ver_n;
  Vers_tmp = Vers1A;
  if (PGS_PC_GetReference(PCLog1A, &Vers_tmp, fname_L1a) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLog1A, Vers1A);

  /* Z. Wan: getting UR of L1A for later use in setting metadata. */
  Vers_tmp = Vers1A;
  if (PGS_PC_GetUniversalRef(PCLog1A, &Vers_tmp, UR_geo) != PGS_S_SUCCESS)
	  modsmf_id_vers(PCLog1A, Vers1A);

  if ((modinf1a = openMODISfile (fname_L1a, "r")) == NULL) {
    sprintf (modstring, "failed to open %s", fname_L1a);
    HALT(modstring);
  }

  /* R. Cember: somehow the native HDF calls are stepping on the MAPI pointers;
  get the attribute value with MAPI instead. */

  sprintf(sd_attr_name, CORES);
  sprintf(sd_attr_type, "char *");
  nelements = (long) N_BYTES_CORE_1A;  
  memset ( &core_1A[0], '\0', nelements );
  if (getMODISfileinfo(modinf1a, sd_attr_name, sd_attr_type, &nelements, 
	core_1A) != MAPIOK)
      Error("Re: Level 1a: failure on call to get core", "getMODISfileinfo");

  sprintf(sd_attr_name, ARCHIVES_IN);
  sprintf(sd_attr_type, "char *");
  nelements = (long) N_BYTES_ARCH_1A;  
  memset ( &arch_1A[0], '\0', nelements );
  if (getMODISfileinfo(modinf1a, sd_attr_name, sd_attr_type, &nelements, 
	arch_1A) != MAPIOK)
      Error("Re: Level 1a: failure on call to get arch", "getMODISfileinfo");

  s_start = strstr (core_1A, pattern_date);
  if (s_start == NULL){
	Error ("String search error", "main, mod11_lst2.c"); }

  sscanf(s_start+80, "%10s", datetime); /* ...27 characters in Time A format */ 

  sscanf (datetime, "%4d%c%2d%c%2d",
	&iyr, &trash_char[0], &imonth, &trash_char[1], &iday);

  s_start = strstr (core_1A, pattern_time);
  if (s_start == NULL){
	Error ("String search error", "main, mod11_lst2.c"); }

  sscanf(s_start+80, "%15s", datetime); /* ...27 characters in Time A format */ 

  sscanf (datetime, "%2d%c%2d%c%9f",
	&ihrs, &trash_char[3], &iminutes, &trash_char[4], &seconds);

  i_day_of_yr_curr = iday_of_year (iyr, imonth, iday);

  seconds_start = ihrs * 3600. + 60. * iminutes + seconds;

  for (i = 0; i < NSD_MOD03_GEO; i++) {

   if (getMODISardims(modinf1a, geo_1a[i].name, geo_1a[i].gname, data_type,
  	&geo_1a[i].rank, geo_1a[i].dims) == FAIL) {

       sprintf (modstring, "getMODISardims of GEO_SDS failed");
       HALT(modstring);
   }

  }  /* end for loop on i */

/* input n_read_lines of Height, SensorZenith & SolarZenith in L1A each time */
  for (i = 0; i < 3; i++)
	geo_1a[i].dims[0] = n_read_lines;

/* input n_read_lines of SensorAzimuth & SolarAzimuth in L1A each time */
  for (i = 7; i < 9; i++)
	geo_1a[i].dims[0] = n_read_lines;

  /* allocate space for data arrays in GEO 1A */

  siz = sizeof (float64);
  ev_start_time = (float64 *) malloc (geo_1a[6].dims[0] * siz);
  siz = sizeof (int16);
  dat1_1a = (int16 ***) alloc_3d (5, geo_1a[0].dims[0],n_pix_per_line,siz);
  sf_height = (int16 **) dat1_1a[0];
  sensr_zen = (int16 **) dat1_1a[1];
  solar_zen = (int16 **) dat1_1a[2];
  sensr_azm = (int16 **) dat1_1a[3];
  solar_azm = (int16 **) dat1_1a[4];
  i_day_scan = (int16 *) malloc (n_read_times * siz);
  i_nit_scan = (int16 *) malloc (n_read_times * siz);
  i_scan_mode = (int16 *) malloc (n_read_times * siz);
  siz = sizeof (float32);
  lat = (float32 **) alloc_2d (geo_1a[3].dims[0], n_pix_per_line, siz);
  lon = (float32 **) alloc_2d (geo_1a[4].dims[0], n_pix_per_line, siz);
  p_time = (float32 **) alloc_2d (geo_1a[4].dims[0] / n_lines_per_scub,
				  n_pix_per_line, siz);
  siz = sizeof (uint8);
  land_sea_mask_1a = (uint8 **) alloc_2d(geo_1a[5].dims[0],n_pix_per_line, siz);

  /* allocate data arrays for Cloudmask */

  siz = sizeof (uint8);
  cmask = (uint8 ***) alloc_3d (1, dim_sizes[1], n_pix_per_line, siz);
  cmask1 = cmask[0];
  cldmask = (uint8 **) alloc_2d (dim_sizes[1], n_pix_per_line, siz);
  siz = sizeof (int16);
  i_cld_scan = (int16 *) malloc (n_read_times * siz);

  /* allocate data arrays for Aprof */

  dims_prof[1] = dim_sizes[1] / 5;
  dims_prof[2] = n_pix_per_line / 5;
  n_along_prof = dims_prof[1];
  n_cross_prof = dims_prof[2];
  dims_qa_prof[0] = n_along_prof;
  dims_qa_prof[1] = n_cross_prof;

  for ( i = 0; i < NSD_MOD07_PROF; i++ ) {
	mod07_prof[i].dims[0] = n_along_prof;
	mod07_prof[i].dims[1] = n_cross_prof;
  }

  siz = sizeof (uint8);
  cmask_prof = (uint8 **) alloc_2d ( n_along_prof, n_cross_prof, siz);
  qa_prof = (uint8 ***) alloc_3d ( n_along_prof, n_cross_prof, 1, siz);
  siz = sizeof (int16);
  t_prof = (int16 ***) alloc_3d( dims_prof[0], n_along_prof, n_cross_prof, siz);
  cwv_prof_raw = (int16 **) alloc_2d( n_along_prof, n_cross_prof, siz);
  cwv_prof = (int16 **) alloc_2d( n_along_prof, n_cross_prof, siz);
  ts_prof = (int16 **) alloc_2d( n_along_prof, n_cross_prof, siz);
  ps_prof = (int16 **) alloc_2d( n_along_prof, n_cross_prof, siz);
  siz = sizeof (float32);
  for ( i = 0; i < N2_VIEW_ANGS; i++ ) {
    t_prof_sum_d[i] = NULL;	t_prof_sum_n[i] = NULL;
    t_prof_sum_d[i] = (float32 ***) alloc_3d ( 200, 200, 1, siz );
    t_prof_sum_n[i] = (float32 ***) alloc_3d ( 200, 200, 1, siz );
  }

  /* allocate data array for snow cover */

  siz = sizeof (uint8);
  snowcv = (uint8 **) alloc_2d (2 * n_read_lines, 2 * n_pix_per_line, siz); 
  lakeice = (uint8 **) alloc_2d (2 * n_read_lines, 2 * n_pix_per_line, siz); 

  z_siz = 2 * n_read_lines * 2 * n_pix_per_line; 

  zeroout_general(&snowcv[0][0], z_siz, 1);
  zeroout_general(&lakeice[0][0], z_siz, 1);

  if ((ier = getMODISarray(modinf1a, geo_1a[3].name, geo_1a[3].gname, sta_geo2,
  		&geo_1a[3].dims[0], &lat[0][0])) == -1) {
	sprintf ( modstring, "getMODISarray of Latitude failed");
	HALT(modstring);
  }
  if ((ier = getMODISarray(modinf1a, geo_1a[4].name, geo_1a[4].gname, sta_geo2,
  		&geo_1a[4].dims[0], &lon[0][0])) == -1) {
	sprintf ( modstring, "getMODISarray of Longitude failed");
	HALT(modstring);
  }

  if ((ier = getMODISarray(modinf1a, geo_1a[5].name, geo_1a[5].gname, sta_geo2,
  		&geo_1a[5].dims[0], &land_sea_mask_1a[0][0])) == -1) {
	sprintf ( modstring, "failed on getMODISarray Land/SeaMask");
	HALT(modstring);
  }

  if ((ier = getMODISarray(modinf1a, geo_1a[6].name, geo_1a[6].gname, sta_geo1,
  		&geo_1a[6].dims[0], &ev_start_time[0])) == -1) {
	sprintf ( modstring, "failed on getMODISarray EV start time");
	HALT(modstring);
  }

  /* open the input Cmask file in MAPI function */
  VersCmask = gran_link.cmask_ver_n;
  Vers_tmp = VersCmask;
  if (PGS_PC_GetReference(PCLogCmask, &Vers_tmp, fname_Cld) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogCmask, VersCmask);

  if ((modinfcld = openMODISfile (fname_Cld, "r")) == NULL) {
    sprintf (modstring, "failed to open %s", fname_Cld);
    HALT(modstring);
  }

  if (getMODISardims(modinfcld,cld_arrnm,cld_grpnm,cld_type,&rank,cld_dims) == FAIL) {
    sprintf (modstring, "getMODISardims of Cloud_Mask failed");
    HALT(modstring);
  }

  if ( cld_dims[1] != dim_l_sizes[1] ) {
    sprintf(LogMsg,"scan lines %d of Cloud_Mask in %s not equal to %d in MOD02",	cld_dims[1], fname_Cld, dim_l_sizes[1]);
    HALT(LogMsg);
  }

  /* read Cloudmask */

  sta[0] = 0;
  sta[1] = 0;
  cld_dims[0] = 1;
  cld_dims[1] = dim_sizes[1];
  ier = getMODISarray (modinfcld, cld_arrnm, cld_grpnm, sta, cld_dims,
		&cmask[0][0][0]);
  if (ier == -1) {
    sprintf (modstring, "failed on getMODISarray Cloud_Mask");
    HALT(modstring);
  }

  /* calculate i_cloud_pix infor and save it in cldmask */

  for (kk = 0; kk < n_read_times; kk++) {

    sta[1] = kk * n_read_lines;
    sta_geo2[0] = kk * n_read_lines;

    if((ier = getMODISarray(modinf1a,geo_1a[0].name,geo_1a[0].gname,sta_geo2,
      geo_1a[0].dims, &dat1_1a[0][0][0])) == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray height");

    for (i = 0; i < n_read_lines; i++ ) {

      i_scan_line = i + sta[1];
      i_ch3 = 0;
      if ( (i_scan_line % 10) == 3 )      i_ch3 = 1;

      for (j = 0; j < n_pix_per_line; j++) {

	cldmask[i_scan_line][j] = 0;
	i_lake_pix = 0;
	u8_1r = land_sea_mask_1a[i_scan_line][j];
	if ( u8_1r > u8_1cons && u8_1r < u8_6cons && sf_height[i][j] > 5 )
		i_lake_pix = 1;
	i_hiland = (sf_height[i][j] > 2000) ? 1 : 0;

	i_cloudy_pix = 0;   u8_1r = (cmask1[i_scan_line][j] & 07);

	/* It is a cloudy pixel if it is not 95-99% clear over land <= 2000m or
	   not 66-99% clear over land > 2000m or not 66-99% clear over lake and rivers */

	if ( !i_lake_pix && (!i_hiland && u8_1r < u8_5cons || i_hiland && u8_1r < u8_3cons)
	     || i_lake_pix && u8_1r < u8_3cons )
		i_cloudy_pix = 1;

	if ( is_a_side_data && i_cloudy_pix && i_ch3 ) {
	    i_cloudy_pix_u = i_cloudy_pix_l = 0;
	    u8_1r = (cmask1[i_scan_line-1][j] & 07);
	    if ( !i_lake_pix && (!i_hiland && u8_1r < u8_5cons || i_hiland && u8_1r < u8_3cons)
	       || i_lake_pix && u8_1r < u8_3cons )
			i_cloudy_pix_u = 1;
	    u8_1r = (cmask1[i_scan_line+1][j] & 07);
	    if ( !i_lake_pix && (!i_hiland && u8_1r < u8_5cons || i_hiland && u8_1r < u8_3cons)
	       || i_lake_pix && u8_1r < u8_3cons )
			i_cloudy_pix_l = 1;
	    if ( ! i_cloudy_pix_u && ! i_cloudy_pix_l )
			i_cloudy_pix = 0;
	}
	if ( i_cloudy_pix )
		cldmask[i_scan_line][j] = u8_1cons;
      }
    }
  }

  /* calculate cloud pixel neighboring information */

  {
    int32 t_lines, t_lines_1;
    int32 t_pixes, t_pixes_1;
    int32 ii5, jj5;
    int32 i_u_5p, i_d_5p;
    int32 j_l_5p, j_r_5p;
    int32 n_pix_2p, n_pix_5p;

    t_lines = dim_sizes[1];
    t_lines_1 = t_lines - 1;
    t_pixes = n_pix_per_line;
    t_pixes_1 = t_pixes - 1;

    /* keep the 020 bit of cmask1 in cldmask */
    for ( i = 0; i < t_lines; i++ ) {
	for (j = 0; j < t_pixes; j++ ) {
		if ( cmask1[i][j] & 020 )
			cldmask[i][j] |= 020;
	}
    }

    /* set NEAR_CLD 0100 and FAR_CLD 0200 bits for near_cld and far_cld */
    for ( i = 0; i < t_lines; i++ ) {

	i_u_5p = i - 5;
	if ( i_u_5p < 0 )	i_u_5p = 0;
	i_d_5p = i + 5;
	if ( i_d_5p > t_lines_1 )	i_d_5p = t_lines_1;

	for (j = 0; j < t_pixes; j++ ) {

	    j_l_5p = j - 5;
	    if ( j_l_5p < 0 )	j_l_5p = 0;
	    j_r_5p = j + 5;
	    if ( j_r_5p > t_pixes_1 )	j_r_5p = t_pixes_1;

	    n_pix_2p = 0;	n_pix_5p = 0;

	    for ( ii5 = i_u_5p; ii5 < i_d_5p; ii5 ++ ) {
		for ( jj5 = j_l_5p; jj5 < j_r_5p; jj5 ++ ) {
			if ( ii5 == i && jj5 == j )	continue;
			if ( abs(ii5 - i) <= 2 && abs(jj5 - j) <= 2 ) {
				if ( cldmask[ii5][jj5] & 07 )
					n_pix_2p += 1;
			}
			else {
				if ( cldmask[ii5][jj5] & 07 )
					n_pix_5p += 1;
			}
		}
	    }
	    if ( n_pix_2p >= 2 )	cldmask[i][j] |= NEAR_CLD;
	    if ( n_pix_5p >= 4 )	cldmask[i][j] |= FAR_CLD;
	}
    }
  }

  /* read in latitude & longitude from MOD35 */
     /*     lat & lon now read from MOD02 */
     /*     commented out by G. Fireman 2005-09-15 */
/*   { */

/*    int32        dim_d1, dim_d2; */

/*    if (getMODISardims(modinfcld, geo_5pix[0].name, geo_5pix[0].gname, data_type, */
/* 	&geo_5pix[0].rank, geo_5pix[0].dims) == FAIL) { */

/* 	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISardims, GEO_SDS"); */
/*    } */
/*    siz = sizeof (float32); */
/*    dim_d1 = geo_5pix[0].dims[0]; */
/*    dim_d2 = geo_5pix[0].dims[1]; */
/*    geo_5pix[1].dims[0] = dim_d1; */
/*    geo_5pix[1].dims[1] = dim_d2; */
/*    lst_l2[7].dims[0] = lst_l2[8].dims[0] = dim_d1; */
/*    lst_l2[7].dims[1] = lst_l2[8].dims[1] = dim_d2; */
/*    lat_5pix = (float32 **) alloc_2d (dim_d1, dim_d2, siz); */
/*    lon_5pix = (float32 **) alloc_2d (dim_d1, dim_d2, siz); */
/*    sta_geo2[0] = 0; */
/*    if ((ier = getMODISarray(modinfcld, geo_5pix[0].name, geo_5pix[0].gname, */
/* 	sta_geo2, &geo_5pix[0].dims[0], &lat_5pix[0][0])) == -1) { */
/* 	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray MOD35 Latitude"); */
/*    } */
/*    if ((ier = getMODISarray(modinfcld, geo_5pix[1].name, geo_5pix[1].gname, */
/* 	sta_geo2, &geo_5pix[1].dims[0], &lon_5pix[0][0])) == -1) { */
/* 	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray MOD35 Longitude"); */
/*    } */
/*   } */

  /* Close the input Cldoumask file */
  returnStatus = closeMODISfile (&modinfcld);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "fail to close Cldmask file");

  /* the system time (in seconds) of the 1st pix */
  pix_zero_time = ev_start_time[0];

  if ( geo_1a[3].dims[0] != dim_sizes[1] ) {
     sprintf (modstring, "%d scan lines in %s MOD021KM but %d scan lines in %s",
	dim_sizes[1], gran_link.local_gran_id, geo_1a[3].dims[0], fname_L1a );
     WARN(modstring);
  }

  dims[1] = n_read_lines;

  n_scan_pixels = n_read_lines * n_pix_per_line;

 /* to determine which land tiles cover with the input granule */
 {
  int32       i_dif, i_tile, i_vert, i_hori;
  int32       ii_p, ii_v, ii_vp1, i_seq;
  float32     grd_lat, grd_lon;
  float32     grd_rat;
  float32     left_lon, rigt_lon;
  _tile_edge  *t_edge_p;

  tile_pntr[0].k_scan_start = -1;
  tile_pntr[0].k_scan_end = -1;

  for (i = 0; i < N_TILE_PNTRS; i++)
	tile_pntr[i].s_num = -1;

  for (kk = 0; kk < n_read_times; kk++) {

    sta[1] = kk * n_read_lines;
    i_day_scan[kk] = 0;
    i_nit_scan[kk] = 0;
    i_cld_scan[kk] = 0;

    n_day_pixels = 0;
    n_nit_pixels = 0;
    n_cld_pixels = 0;

    sta_o[0] = kk * n_read_lines;

    /* input L1A data */

    sta_geo2[0] = kk * n_read_lines;

    if((ier = getMODISarray(modinf1a,geo_1a[1].name,geo_1a[1].gname,sta_geo2,
  		geo_1a[1].dims, &dat1_1a[1][0][0])) == -1) {
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray sensr_zen");
    }

    /* determine i_day_scan & i_nit_scan */ 

    for (i = 0; i < n_read_lines; i++) {

	i_scan_line = i + sta[1];

	i_ch3 = 0;
	if ( (i_scan_line % 10) == 3 )      i_ch3 = 1;

	for (j = 0; j < n_pix_per_line; j++) {
	    if ( sensr_zen[i][j] > 7500 )	continue;
	    grd_lat = lat[i_scan_line][j];
	    if ( grd_lat < -90.0 )	continue;
	    grd_lon = lon[i_scan_line][j];
	    if ( grd_lon < -180.0 )	continue;
	    i_vert = (90. - grd_lat) / 10.;
	    i_hori = -1;
	    ii_scan_b = (i_scan_line) / 10;
	    seconds_tmp = seconds_start + EV_TIME( ev_start_time[ii_scan_b], j)
			- pix_zero_time;
	    pix_time = seconds_tmp / 3600.;
	    pix_time += grd_lon / 15.0;
	    if ( pix_time > 24. )        {
		pix_time -= 24.0;
	    }
	    if ( pix_time < 0. )         {
		pix_time += 24.0;
	    }
	    if (pix_time >= 6. && pix_time < 18.0)	n_day_pixels++;
	    else					n_nit_pixels++;
	    p_time[ii_scan_b][j] = pix_time;

	    i_cloudy_pix = cldmask[i_scan_line][j] & 01;

	    if ( i_cloudy_pix )	n_cld_pixels++;

	    for (jj = 0; jj < N_HORI_TILE; jj++) {
		ii_p =  _land_tile_id[i_vert][jj];
		if ( ii_p == -1 ) continue;           /* in sea */
		tile_tbl_p = &land_tile_tbl[ii_p];
		ii_v = -1;
		for (ii = 0; ii < N_EDGE_PNTS; ii++) {
		      t_edge_p = &tile_tbl_p->edg[ii];
		      if (grd_lat > t_edge_p->lat)    ii_v = ii;
		}
		if ( ii_v == -1 ) continue;

		if (ii_v > (N_EDGE_PNTS - 2)) ii_v = N_EDGE_PNTS - 2;

		t_edge_p = &tile_tbl_p->edg[ii_v];
		ii_vp1 = ii_v + 1;

		grd_rat = (grd_lat - t_edge_p->lat) /
		      (tile_tbl_p->edg[ii_vp1].lat - t_edge_p->lat);
		left_lon = t_edge_p->lon_left + grd_rat *
		      (tile_tbl_p->edg[ii_vp1].lon_left - t_edge_p->lon_left);
		rigt_lon = t_edge_p->lon_right + grd_rat *
		      (tile_tbl_p->edg[ii_vp1].lon_right - t_edge_p->lon_right);
		if (grd_lon >= left_lon && grd_lon < rigt_lon)
		      i_hori = tile_tbl_p->i_horiz;
	    }
	    if ( i_hori == -1)      continue;
	    i_dif = 1;
	    i_seq = _land_tile_id[i_vert][i_hori];

	    i_match = ( !n_regions ) ? 1 : 0;

	    for (i1 = 0; i1 < n_regions; i1++) {
		if ( i_regions[i1] == land_tile_tbl[i_seq].reg_id ) {
			i_match = 1;
			break;
		}
	    }

	    if ( !i_match )	continue;

	    for (i_tile = 0; i_tile < n_tiles; i_tile++) {
		if (tile_pntr[i_tile].s_num == i_seq) {
			if ( kk > tile_pntr[i_tile].k_scan_end )
				tile_pntr[i_tile].k_scan_end = kk;
			if ( seconds_tmp > tile_pntr[i_tile].end_seconds )
				tile_pntr[i_tile].end_seconds = seconds_tmp;
			i_dif = 0;
			break;
		}
	    }
	    if ( i_dif ) {
		tile_pntr[n_tiles].iv = i_vert;
		tile_pntr[n_tiles].ih = i_hori;
		tile_pntr[n_tiles].s_num = i_seq;
		if ( tile_pntr[n_tiles].k_scan_start == -1 )
			tile_pntr[n_tiles].k_scan_start = kk;
		if ( kk > tile_pntr[n_tiles].k_scan_end )
			tile_pntr[n_tiles].k_scan_end = kk;

		tile_pntr[n_tiles].begin_seconds = seconds_tmp;
		tile_pntr[n_tiles].end_seconds = seconds_tmp;

		n_tiles++;

if ( n_tiles == N_TILE_PNTRS ) {	/* added by Z. Wan on 25 May 1998 */
   free_lst_space();
   sprintf (LogMsg,
     "n_tiles %d exceeds its maximum, increase N_TILE_PNTRS in land_tile_tbl.h",
	n_tiles);
   SMFLOG(LogMsg);
}
		tile_pntr[n_tiles].k_scan_start = -1;
		tile_pntr[n_tiles].k_scan_end = -1;
	    }
	}
    }

    if ( n_day_pixels )	i_day_scan[kk] = 1;
    if ( n_nit_pixels )	i_nit_scan[kk] = 1;
    if ( i_nit_scan[kk] && !i_day_scan[kk] )	i_scan_mode[kk] = 0;
    if ( !i_nit_scan[kk] && i_day_scan[kk] )	i_scan_mode[kk] = 1;
    if ( i_nit_scan[kk] && i_day_scan[kk] )	i_scan_mode[kk] = 2;

    if ( n_cld_pixels == n_scan_pixels )	i_cld_scan[kk] = 1;
    tot_cld_pixels += n_cld_pixels;
    ntot_day_pixels += n_day_pixels;
    ntot_nit_pixels += n_nit_pixels;

  }	/* end of the kk loop */

  if ( n_tiles == 0 ) {         /* nothing to be processed */

	closeMODISfile (&modinf1a);
	closeMODISfile (&modinf1b);
	if (modinfsnow) closeMODISfile (&modinfsnow);
	free_lst_space();
sprintf(LogMsg, "Completed MOD_PR11 (%d for %s n_tiles=0), Note: no output for this ocean granule.",
		rel_vers_gran, gran_link.local_gran_id );
	SMFLOG(LogMsg);
  }

  /* open the input atmospheric profile file in MAPI function */
  VersAprof = gran_link.aprof_ver_n;
  Vers_tmp = VersAprof;
  if (PGS_PC_GetReference(PCLogProfile,&Vers_tmp, fname_Aprof) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogProfile, VersAprof);

  if ((modinfprof = openMODISfile (fname_Aprof, "r")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_Aprof);

/* ## get PGEVERSION_MOD07 */
  sprintf(sd_attr_name, CORES);
  sprintf(sd_attr_type, "char *");
  nelements = (long) N_BYTES_CORE_PROF;
  if (getMODISfileinfo(modinfprof, sd_attr_name, sd_attr_type, &nelements, core_PROF) != MAPIOK)
	Error("Re: MOD07_L2: failure on call to ", "getMODISfileinfo");

  local = &PGEVERSION_MOD07[0];
  if ( get_meta_attr (core_PROF, "PGEVERSION", 1, 1, (char *) local ) == FAIL)
	HALT("get_meta_attr fails on core_PROF");
  i_new_mod07 = (strcmp (PGEVERSION_MOD07, "5.2.0") >= 0) ? 1 : 0;
/*fprintf(stderr, "PGEVERSION_MOD07 %s i_new_mod07 %d\n", local, i_new_mod07);*/

  if (getMODISardims(modinfprof,mod07_prof[3].name, mod07_prof[3].gname,
      dtype_mod07, &mod07_prof[3].rank, dim_prof) == FAIL) {
	sprintf (modstring, "getMODISardims of Water_Vapor failed");
	HALT(modstring);
  }
  mod07_prof[3].dims[0] = dim_prof[0];
  mod07_prof[3].dims[1] = dim_prof[1];

  if ( (5*mod07_prof[3].dims[0]) != dim_l_sizes[1] ) {
	sprintf(LogMsg, "scan lines 5x%d in %s not equal to %d in MOD02",
		mod07_prof[1].dims[0], fname_Aprof, dim_l_sizes[1]);
	HALT(LogMsg);
  }

  /* read atmospheric profile */

  ier = getMODISarray (modinfprof, mod07_prof[0].name, mod07_prof[0].gname,
	sta_prof2, mod07_prof[0].dims, &cmask_prof[0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray cmask_prof");

  ier = getMODISarray (modinfprof, mod07_prof[2].name, mod07_prof[2].gname,
	sta_prof2, mod07_prof[2].dims, &ps_prof[0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray ps_prof");

  ier = getMODISarray (modinfprof, mod07_prof[3].name, mod07_prof[3].gname,
	sta_prof2, mod07_prof[3].dims, &cwv_prof_raw[0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray cwv_prof");

  ier = getMODISarray (modinfprof, sd_t_prof, prof_grpnm, sta_prof, dims_prof,
			&t_prof[0][0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray t_prof");

  sta_qa_prof[2] = 0;
  ier = getMODISarray (modinfprof, sd_qa_prof, prof_grpnm, sta_qa_prof,
		dims_qa_prof, &qa_prof[0][0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray qa_prof");

  /* calculate air temperature at surface */

  if ( i_new_mod07 ) {

   for ( i_prof = 0; i_prof < n_along_prof; i_prof++ ) {

    cm_prof_p = cmask_prof[i_prof];
    ps_prof_p = ps_prof[i_prof];

    for ( j_prof = 0; j_prof < n_cross_prof; j_prof++ ) {

	i_skip_prof = 0;

	/* skip cells that are less than 95-99% clear */
	if ( !( cm_prof_p[j_prof] & 04 ) )
		i_skip_prof = 1;

	qa1 = qa_prof[i_prof][j_prof][0];

	/* if t_prof not useful */
	if ( !(qa1 & 01) ) i_skip_prof = 1;

	/* if w_prof not useful */
	if ( !(qa1 & 020) ) i_skip_prof = 1;

	ts_prof[i_prof][j_prof] = -32768;
	if ( i_skip_prof )	continue;

	for ( k1_prof = 13; k1_prof >= 0; k1_prof-- ) {
	    if ( t_prof[k1_prof][i_prof][j_prof] != -32768 ) {
		ts_prof[i_prof][j_prof] = t_prof[k1_prof][i_prof][j_prof];
		break;
	    }
	}
    }
   }
  }
  else {

   for ( i_prof = 0; i_prof < n_along_prof; i_prof++ ) {

    cm_prof_p = cmask_prof[i_prof];
    ps_prof_p = ps_prof[i_prof];

    for ( j_prof = 0; j_prof < n_cross_prof; j_prof++ ) {

	i_skip_prof = 0;

	/* the following block is not necessary unless to check out n_clear_1km

	i_clear_1km = 0;
	for ( j = 5 * j_prof, jj_prof = 0; jj_prof < 5; j++, jj_prof++ ) {
	    for ( i = 0; i < 5; i++ ) {
		i_scan_line = 5 * i_prof + i;
		if ( cmask1[i_scan_line][j] & 04)
			i_clear_1km++;
	    }
	}
	if ( i_clear_1km )	n_clear_1km++;			*/

	/* skip cells that are less than 95-99% clear */
	if ( !( cm_prof_p[j_prof] & 04 ) )
		i_skip_prof = 1;

	/* skip cells with fill values */
	if ( t_prof[9][i_prof][j_prof] == -32768 )
	     	i_skip_prof = 1;

	qa1 = qa_prof[i_prof][j_prof][0];

	/* if t_prof not useful */
	if ( !(qa1 & 01) ) i_skip_prof = 1;

	/* if w_prof not useful */
	if ( !(qa1 & 020) ) i_skip_prof = 1;

	if ( i_skip_prof ) {
		ts_prof[i_prof][j_prof] = -32768;
		continue;
	}

	if ( ps_prof_p[j_prof] == -32768 )
	     ps_prof_p[j_prof] = pres_levels[0];

	ii_prof = 0;
	for ( i = 0; i < 14; i++ ) {
		if ( pres_levels[i] <= (0.1 * ps_prof_p[j_prof]) ) {
			ii_prof = i;
			break;
		}
	}
	kk_prof = 13 - ii_prof;
	k1_prof = kk_prof + 1;
	i2_prof = ii_prof;	i1_prof = ii_prof - 1;
	if ( ii_prof == 0 ) {
		kk_prof = 12;	k1_prof = 13;
		i2_prof = 1;	i1_prof = 0;
	}
	t_prof_p = t_prof[kk_prof][i_prof];

	x_prof = (0.1 * ps_prof[i_prof][j_prof] - pres_levels[i2_prof]) /
			(pres_levels[i1_prof] - pres_levels[i2_prof]);
	t_p_prof[i1_prof] = t_prof_p[j_prof] +
			x_prof * (t_prof[k1_prof][i_prof][j_prof] -
					t_prof_p[j_prof]);
	ts_prof[i_prof][j_prof] = t_p_prof[i1_prof];
    }
   }
  }

  /* free unwanted space */
  free (cmask[0][0]); free (cmask[0]); free (cmask); cmask = NULL;
  free (qa_prof[0][0]); free (qa_prof[0]); free (qa_prof); qa_prof = NULL;
  free (t_prof[0][0]);  free (t_prof[0]);  free (t_prof);  t_prof = NULL;
  free ( ps_prof[0] );  free ( ps_prof );  ps_prof = NULL;

  {
    int32	iz, jz;
    int32	iiz, jjz;
    int32	t_cwv = 0, t_weight = 0;
    int16	i_1_j_1, i_1_j, i_1_j_p1;
    int16	i_j_1, i_j, i_j_p1;
    int16	i_p1_j_1, i_p1_j, i_p1_j_p1, ij;

    /* replace the fillvalue in the first two columns in V4.1.2 MOD07 */
    for ( iz = 0; iz < n_along_prof; iz++ ) {
	for ( jz = 0; jz < 2; jz++ ) {
	    if ( cwv_prof_raw[iz][jz] > 0 ) continue;	/* not necessary */
	    iiz = 0;					/* in new versions */
	    for ( jjz = 2; jjz < n_cross_prof && iiz < 10 ; jjz++, iiz++ ) {
		if ( cwv_prof_raw[iz][jjz] > 0 ) {
			cwv_prof_raw[iz][jz] = cwv_prof_raw[iz][jjz];
			break;

		}
	    }
	}
    }

    iiz = n_along_prof - 1;
    jjz = n_cross_prof - 1;

    for ( iz = 0; iz < n_along_prof; iz++ ) {
	for ( jz = 0; jz < n_cross_prof; jz++ )
		cwv_prof[iz][jz] = 0;
    }
    for ( jz = 0; jz < n_cross_prof; jz++ ) {
		cwv_prof[0][jz] = cwv_prof_raw[0][jz];
		cwv_prof[iiz][jz] = cwv_prof_raw[iiz][jz];
    }
    for ( iz = 0; iz < n_along_prof; iz++ ) {
		cwv_prof[iz][0] = cwv_prof_raw[iz][0];
		cwv_prof[iz][jjz] = cwv_prof_raw[iz][jjz];
    }

    for ( iz = 1; iz < iiz; iz++ ) {
	for ( jz = 1; jz < jjz; jz++ ) {
	    /*if ( cwv_prof_raw[iz][jz] <= 0 )	continue;*/
	    t_cwv = 0;
	    t_weight = 0;
	    i_1_j_1 = cwv_prof_raw[iz-1][jz-1];
	    i_1_j = cwv_prof_raw[iz-1][jz];
	    i_1_j_p1 = cwv_prof_raw[iz-1][jz+1];
	    i_j_1 = cwv_prof_raw[iz][jz-1];
	    i_j = cwv_prof_raw[iz][jz];
	    i_j_p1 = cwv_prof_raw[iz][jz+1];
	    i_p1_j_1 = cwv_prof_raw[iz+1][jz-1];
	    i_p1_j = cwv_prof_raw[iz+1][jz];
	    i_p1_j_p1 = cwv_prof_raw[iz+1][jz+1];
	    if ( i_1_j_1 > 0 ) {
		t_cwv += i_1_j_1;
		t_weight += 1.0;
	    }
	    if ( i_1_j > 0 ) {
		t_cwv += i_1_j;
		t_weight += 1.0;
	    }
	    if ( i_1_j_p1 > 0 ) {
		t_cwv += i_1_j_p1;
		t_weight += 1.0;
	    }
	    if ( i_j_1 > 0 ) {
		t_cwv += i_j_1;
		t_weight += 1.0;
	    }
	    if ( i_j > 0 ) {
		t_cwv += i_j;
		t_weight += 1.0;
	    }
	    if ( i_j_p1 > 0 ) {
		t_cwv += i_j_p1;
		t_weight += 1.0;
	    }
	    if ( i_p1_j_1 > 0 ) {
		t_cwv += i_p1_j_1;
		t_weight += 1.0;
	    }
	    if ( i_p1_j > 0 ) {
		t_cwv += i_p1_j;
		t_weight += 1.0;
	    }
	    if ( i_p1_j_p1 > 0 ) {
		t_cwv += i_p1_j_p1;
		t_weight += 1.0;
	    }

	    if ( t_weight > 0 )
		cwv_prof[iz][jz] = (double) t_cwv / t_weight + 0.5;
	}
    }
    free (cwv_prof_raw[0]);	free(cwv_prof_raw);	cwv_prof_raw = NULL;
  }

  n_zones = n_along_prof / 20;

  for ( i_zone = 0; i_zone < n_zones; i_zone++ ) {
    tair_zones[i_zone] = 0.0;
    cwv_zones[i_zone] = 0.0;
    n_ts_in_zone[i_zone] = 0;
    n_cwv_in_zone[i_zone] = 0;
    for ( i_prof = 0; i_prof < 20; i_prof++ ) {
	i = 20 * i_zone + i_prof;
	for ( j_prof = 0; j_prof < n_cross_prof; j_prof++ ) {
	  if ( !( cmask_prof[i][j_prof] & 02 || cmask_prof[i][j_prof] & 04) )
		continue;
	  if (ts_prof[i][j_prof] >= 0 && ts_prof[i][j_prof] <= 20000) {
		tair_zones[i_zone] += 0.01 * ( ts_prof[i][j_prof] + 15000);
		n_ts_in_zone[i_zone] += 1;
	  }
	  if (cwv_prof[i][j_prof] > 0 ) {
		cwv_zones[i_zone] += 0.001 * cwv_prof[i][j_prof];
		n_cwv_in_zone[i_zone] += 1;
	  }
	}
    }
    if ( n_ts_in_zone[i_zone] )	tair_zones[i_zone] /= n_ts_in_zone[i_zone];
    if ( n_cwv_in_zone[i_zone] ) cwv_zones[i_zone] /= n_cwv_in_zone[i_zone];
  }

  tair_zones[n_zones] = 0.0;
  cwv_zones[n_zones] = 0.0;
  n_ts_in_zone[n_zones] = 0;
  n_cwv_in_zone[n_zones] = 0;

  for ( i_zone = 0; i_zone < n_zones; i_zone++ ) {
	if ( tair_zones[i_zone] > 0.01 ) {
		tair_zones[n_zones] += tair_zones[i_zone];
		n_ts_in_zone[n_zones] += 1;
	}
	if ( cwv_zones[i_zone] > 0.001 ) {
		cwv_zones[n_zones] += cwv_zones[i_zone];
		n_cwv_in_zone[n_zones] += 1;
	}
  }
  if ( n_ts_in_zone[n_zones] ) tair_zones[n_zones] /= n_ts_in_zone[n_zones];
  if ( n_cwv_in_zone[n_zones] ) cwv_zones[n_zones] /= n_cwv_in_zone[n_zones];

  returnStatus = closeMODISfile (&modinfprof);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "fail to close profile file");

  geo_1a[0].dims[0] = n_read_lines;

  gd = NULL;

  /* determine the scan mode */
  for (i_tile = 0; i_tile < n_tiles; i_tile++) {

    i_seq = tile_pntr[i_tile].s_num;
    tile_pntr[i_tile].scan_mode = -1;

    for ( i = tile_pntr[i_tile].k_scan_start;
			i <= tile_pntr[i_tile].k_scan_end; i++) {

	if ( tile_pntr[i_tile].scan_mode == -1 ) {
		if ( !i_day_scan[i] && i_nit_scan[i] )
			tile_pntr[i_tile].scan_mode = 0;
		if ( i_day_scan[i] && !i_nit_scan[i] )
			tile_pntr[i_tile].scan_mode = 1;
		if ( i_day_scan[i] && i_nit_scan[i] )
			tile_pntr[i_tile].scan_mode = 2;
	}
	else {
		if ( tile_pntr[i_tile].scan_mode == 0 && i_day_scan[i] )
			tile_pntr[i_tile].scan_mode = 2;
		if ( tile_pntr[i_tile].scan_mode == 1 && i_nit_scan[i] )
			tile_pntr[i_tile].scan_mode = 2;
	}
    }
  }
 } /* end of the block for determining land tiles */

  /* convert unit from "degree" to "radians" */
  ii = geo_1a[3].dims[0];
  jj = geo_1a[3].dims[1];
  for ( i = 0; i < ii; i++)
      for ( j = 0; j < jj; j++) {
	lat[i][j] = DTR(lat[i][j]);
	lon[i][j] = DTR(lon[i][j]);
      }

  /* allocate space for L1B input */

  siz = sizeof (uint16);
  data = (uint16 ***) alloc_3d (7, n_read_lines, n_pix_per_line, siz);
  siz = sizeof (uint8);
  emissive_uncert_indexes = (uint8 ***) alloc_3d (7, n_read_lines, n_pix_per_line, siz);
  l1b_qa = (uint8 **) alloc_2d (dim_sizes[1], n_pix_per_line, siz);
  siz = sizeof (float64);
  coef_mesh = (float64 **) alloc_2d (8, 9, siz);
  coeff_p = (float64 *) malloc (8 * siz);

  /* allocate data arrays for L2 LST output */

  nn_lines = multi_write_out ? n_read_lines : dim_sizes[1];
  siz = sizeof (uint16);
  lst_out = (uint16 **) alloc_2d (nn_lines, n_pix_per_line, siz);
  QC_out = (uint16 **) alloc_2d (nn_lines, n_pix_per_line, siz);

  siz = sizeof (uint8);
  err_lst_out = (uint8 **) alloc_2d (nn_lines, n_pix_per_line, siz);
  emis31_out = (uint8 **) alloc_2d (nn_lines, n_pix_per_line, siz);
  emis32_out = (uint8 **) alloc_2d (nn_lines, n_pix_per_line, siz);
  view_angl_out = (uint8 **) alloc_2d (nn_lines, n_pix_per_line, siz);
  view_time_out = (uint8 **) alloc_2d (nn_lines, n_pix_per_line, siz);

  /* initialize l1b_qa */

  for ( i = 0; i < dim_sizes[1]; i++ )
	for ( j = 0; j < n_pix_per_line; j++ )
		l1b_qa[i][j] = u8_0cons;

  /*** open the input L1B file in MAPI function ***/

  if ((modinf1b = openMODISfile (fname_L1b, "r")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_L1b);

  coverage_min = uparam.coverage_min;
  strcpy(gridtype, uparam_p->gridtype);

  /* the loop for calling Granule_tile once for each of land tiles */
  for ( i_tile_in_proc = 0; i_tile_in_proc < n_tiles; i_tile_in_proc++ ) {

    uparam.itile_horiz = tile_pntr[i_tile_in_proc].ih;
    uparam.itile_vert = tile_pntr[i_tile_in_proc].iv;

    memset( &met_cnt_1km.FirstEntryInStruct, 0, sizeof( MET_counts ) );
    memset( &met_cnt_6km.FirstEntryInStruct, 0, sizeof( MET_counts ) );
    met_cnt_1km.AllPixelsProcessed = 2 * TILE_YDIM * TILE_XDIM;		/* day and night */
    met_cnt_6km.AllPixelsProcessed = 2 * TILE_YDIM_6KM * TILE_XDIM_6KM;	/* day and night */

    for (i = 0; i < 4; i++) {
	total_l3_1km[i] = 0;
	total_l3_6km_dn[i] = 0;
    }

    tile_begin_seconds = tile_pntr[i_tile_in_proc].begin_seconds;
    tile_end_seconds = tile_pntr[i_tile_in_proc].end_seconds; 

    Granule_tile ( i_tile_in_proc );

n_stops[0] += 1;

  } /* end of the i_tile_in_proc loop */

  /* open the L2 file for reading and writing metadata */
  if ((lst_l2_outf = openMODISfile (fname_L2, "a")) == NULL)
        HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_L2);

  dims_o[0] = n_read_lines;

  for (kk = 0; kk < n_read_times; kk++) {

    sta_o[0] = kk * n_read_lines;
    ier = getMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		sta_o,dims_o, &QC_out[0][0]);
    if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray QC from L2");

    for (i = 0; i < n_read_lines; i++) {

	QC_o_p = QC_out[i];

	for (j = 0; j < n_pix_per_line; j++) {

		u16_qa_l2_man = u16getbits(QC_o_p[j], 1, 2);
		total_l2_pix[u16_qa_l2_man] += 1;
	}
    }
    ier = putMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		    sta_o,dims_o, &QC_out[0][0]);
    if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray QC to L2");
  }

  for ( i = 0; i < dim_sizes[1]; i++ ) {
      for ( j = 0; j < n_pix_per_line; j++ ) {
		if ( l1b_qa[i][j] == u8_1cons )
			met_cnt_L2.QA_MissingData++;
		if ( l1b_qa[i][j] == u8_2cons )
			met_cnt_L2.QA_OutOfBoundsData++;
      }
  }

  met_cnt_L2.NumberOfPixels_LST_Produced = total_l2_pix[0] + total_l2_pix[1];
  met_cnt_L2.QA_GoodQuality = total_l2_pix[0];
  met_cnt_L2.QA_OtherQuality = total_l2_pix[1];
  met_cnt_L2.QA_NotProducedCloud = total_l2_pix[2];
  met_cnt_L2.QA_NotProducedOther = total_l2_pix[3];
/* It is necessary to use the following formula because the input granule
   may NOT COMPLETELY be processed in the regional processing */
  met_cnt_L2.QA_NotProducedOther = (met_cnt_L2.AllPixelsProcessed -
      met_cnt_L2.QA_GoodQuality - total_l2_pix[2] - met_cnt_L2.QA_OtherQuality);
  if ( met_cnt_L2.QA_NotProducedOther < 0 )
	met_cnt_L2.QA_NotProducedOther = 0;

  DoMetaData( (int32) 0, (int32) 2, (PGSt_PC_Logical) PCLogL2,
	  (PGSt_integer) n_tiles, (PGSt_integer) 1, lst_l2_outf );

  if (closeMODISfile(&lst_l2_outf) == -1)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "L2 output file failed");

  /* ---- Set attributes for LST_L2 SDSs ----- */

  {
    char	str_tmp[64];
    int32	f_id, sds_id, istat;
    int32	i_offset, n_chars, x = 5;
    double	dummy;

    /* Open the file and initiate the SD interface */

    f_id = SDstart(fname_L2, DFACC_WRITE);
    if ( f_id == FAIL )
	HALTsmf(MODIS_E_HDF_FAIL, "SDstart(fname_L2, DFACC_WRITE)");

    for ( i = 0; i < NSD_LST_L2; i++) {

      SDsetdim_LST (f_id, lst_l2[i], &sds_id);

      if (strcmp (lparam_lst_l2[i].units, "none"))
	      istat = SDsetdatastrs (sds_id, lparam_lst_l2[i].long_name,
				(VOIDP) lparam_lst_l2[i].units, NULL, NULL);
      else
	      istat = SDsetdatastrs (sds_id, lparam_lst_l2[i].long_name,
	      			NULL, NULL, NULL);

      SDsetattr_LST (sds_id, "Number Type", DFNT_CHAR8, 
		     strlen(lst_l2[i].type), (VOIDP) lst_l2[i].type);

      SDsetattr_LST_V (sds_id, "valid_range", lparam_lst_l2[i].type, 2, 
		     lparam_lst_l2[i].vrange); 

      if ( lparam_lst_l2[i].fillv > -1.0 || lparam_lst_l2[i].fillv < -2.0 )
         SDsetattr_LST_V (sds_id, "_FillValue", lparam_lst_l2[i].type, 1, 
		     &lparam_lst_l2[i].fillv);

      if ( istat == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDsetdatastrs, LST_L2");

      if ( cal_lst_l2[i].valid )
      {
        i_offset = ( fabs (cal_lst_l2[i].offset) > SMALL_VALUE ) ? 1 : 0;
        n_chars = strlen (dataconvt_lst_l2[2*i+1]);
        if ( ! i_offset ) n_chars -= 13;

	SDsetattr_LST (sds_id, dataconvt_lst_l2[2*i], DFNT_CHAR8, 
		       n_chars, (VOIDP) dataconvt_lst_l2[2*i+1] );
  
       if ( cal_lst_l2[i].sd_type == 1 ) {

	SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_lst_l2[i].scale);
	SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_lst_l2[i].offset);
       }
       else if ( cal_lst_l2[i].sd_type == 2 ) {

	SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
			 (VOIDP) &cal_lst_l2[i].scale);
	SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1, 
			 (VOIDP) &cal_lst_l2[i].offset);
       }
       if ( cal_lst_l2[i].sd_type == 1  ||  cal_lst_l2[i].sd_type == 2 ) {
	dummy = 0.;
	SDsetattr_LST (sds_id,"scale_factor_err",DFNT_FLOAT64,1,(VOIDP) &dummy);
	SDsetattr_LST (sds_id,"add_offset_err", DFNT_FLOAT64,1, (VOIDP) &dummy);
	SDsetattr_LST (sds_id, "calibrated_nt", DFNT_INT32,1, (VOIDP) &x);
       }
      }

      if ( i >= (NSD_LST_L2 - 2) ) {
	  memset( &str_tmp[0], '\0', 64);
	  sprintf ( &str_tmp[0], "3, %d, 5", lst_l2[0].dims[0] - 2);
	  SDsetattr_LST (sds_id, "Cell_Along_Swath_Sampling", DFNT_CHAR8,
			strlen(str_tmp), (VOIDP) str_tmp);
	  memset( &str_tmp[0], '\0', 64);
	  if ( lst_l2[7].dims[1] == 271 )
	  	sprintf ( &str_tmp[0], "3, %d, 5", lst_l2[0].dims[1] - 1);
	  else
	  	sprintf ( &str_tmp[0], "3, 8, ...");
	  SDsetattr_LST (sds_id, "Cell_Across_Swath_Sampling", DFNT_CHAR8,
			strlen(str_tmp), (VOIDP) str_tmp);
      }

      /* Terminate access the the array */

      if ( SDendaccess (sds_id) == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDendaccess(sds_id)");
    }

    /* Terminate access to the SD interface and close the file */

    istat = SDend (f_id);

    if ( istat == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDend (f_id)");
  }

  /* Close the input L1A file */
  returnStatus = closeMODISfile (&modinf1a);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "L1A file failed");

  /* Close the input L1B file */
  returnStatus = closeMODISfile (&modinf1b);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "L1B file failed");

  /* Close the input snow-cover file */
  if ( modinfsnow ) {
    returnStatus = closeMODISfile (&modinfsnow);
    if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "snow-cover file failed");
  }

  /* free all allocated spaces */

  free_lst_space ();

  {
    float32	pct1, pct2, pct3, pct4;

    pct1 = 100. * met_cnt_L2.QA_GoodQuality/met_cnt_L2.AllPixelsProcessed;
    pct2 = 100. * met_cnt_L2.QA_OtherQuality/met_cnt_L2.AllPixelsProcessed;
    pct3 = 100. * met_cnt_L2.QA_NotProducedCloud/met_cnt_L2.AllPixelsProcessed;
    pct4 = 100. * met_cnt_L2.QA_NotProducedOther/met_cnt_L2.AllPixelsProcessed;

    if ( (pct1 + pct2) > 0 )
sprintf(LogMsg,
    "Completed %s successfully (%d for %s) QA Pct's: %.5f %.5f %.5f %.5f",
		esdt_L2, rel_vers_gran, gran_link.local_gran_id,
		pct1, pct2, pct3, pct4 );
    else {
	if ( npx_bad_split_window )
sprintf(LogMsg,"Completed %s (%d for %s EV_1KM_Emissive_Uncert_Indexes upto %d in L1B) QA Pct's: %.5f %.5f %.5f %.5f",
		esdt_L2, rel_vers_gran, gran_link.local_gran_id,
		max_EV_1KM_Emissive_Uncert_Indexes,
		pct1, pct2, pct3, pct4 );
	else
sprintf(LogMsg,"Completed %s (%d for %s) QA Pct's: %.5f %.5f %.5f %.5f",
		esdt_L2, rel_vers_gran, gran_link.local_gran_id,
		pct1, pct2, pct3, pct4 );
    }
  }

  /* R. Cember: exit thru SMFLOG() */

  SMFLOG(LogMsg);

} /* end main function */

/******************************************************************************
!C

!Description:
   Granule_tile, the major function called by the main function, performs
   the processing on one of the tiles covered by the input granule.

   Granule_tile reads in MODIS L1B and Geolocation simulated data, retrieves
   LST by the use of the generalized split-window LST algorithm, and writes
   1km level-2 and level-3 LST products.  The day/night LST algorithm is used
   to retrieve surface emissivities and temperatures at 5-km resolution grids.
   A sub set of the L2G library functions has been adapted with some
   modifications for calculation of the coverages of each pixel on 1km
   resolution grids in the sinusoidal tile system.

   After multiple runs of this function, it completes the L2 processing for
   the whole granule.

!Input Parameters:
   i_tile_proc			the id number for the tile to be processed

   EV_1KM_Emissive              1KM resolution Earth view data in
				MODIS L1B simulated data granule file modinf1b
   scales and offsets           slope and constant for conversion to radiance

   Scan number                  in Geo_1A data (MOD03)
   EV frames                    ...
   EV start time                ...
   Height                       ...
   Latitude                     ...
   Longitude                    ...
   SensorZenith                 ...
   SolarZenith                  ...

!Output Parameters:
  for the L2 LST product:
    lst_l2_outf                 output MODIS LST L2 HDF file
    dim_sizes[2]                pixels per scan line
    dim_sizes[0]                number of scan lines
    lst_out                     output SDS 1            uint16
    QC_out                      output SDS 2            uint16
    err_lst_out                 output SDS 3            uint8
    emis31_out                  output SDS 4            uint8
    emis32_out                  output SDS 5            uint8
    view_angle                  output SDS 6            uint8
    view_time                   output SDS 7            uint8

  for the daily 1km L3 LST product:
    lst_1km_l3_outf	        output MODIS 1km LST L3 HDF file
    lst_day_1km_Sv		output SDS 1		uint16
    QC_day_1km			output SDS 2		uint8
    v_time_1km_day_Sv		output SDS 3		uint8
    v_angl_1km_day_Sv		output SDS 4		uint8
    lst_nit_1km_Sv		output SDS 5		uint16
    QC_nit_1km			output SDS 6		uint8
    v_time_1km_nit_Sv		output SDS 7		uint8
    v_angl_1km_nit_Sv		output SDS 8		uint8
    em31_1km_l3_Sv	        output SDS 9		uint8
    em32_1km_l3_Sv	        output SDS 10		uint8
    clear_day_cov	        output SDS 11		uint16
    clear_nit_cov	        output SDS 12		uint16

  for the daily 6km L3 LST product:
    lst_6km_l3_outf	        output MODIS 6km LST L3 HDF file
    lst_6km_Sd			output SDS 1		uint16
    QC_6km_day			output SDS 2		uint16
    v_time_6km_day_Sv		output SDS 3		uint8
    v_angl_6km_day_Sv		output SDS 4		uint8
    lst_6km_Sn			output SDS 5		uint16
    QC_6km_nit			output SDS 6		uint8
    v_time_6km_nit_Sv		output SDS 7		uint8
    v_angl_6km_nit_Sv		output SDS 8		uint8
    em20_6km_l3_Sv	        output SDS 9		uint8
    em22_6km_l3_Sv	        output SDS 10		uint8
    em23_6km_l3_Sv	        output SDS 11		uint8
    em29_6km_l3_Sv	        output SDS 12		uint8
    em31_6km_l3_Sv	        output SDS 13		uint8
    em32_6km_l3_Sv	        output SDS 14		uint8
    lst_6km_D_from_1km	        output SDS 15		uint16
    lst_6km_N_from_1km	        output SDS 16		uint16
    QC_emis 	        	output SDS 17		uint8
    pct_land 	        	output SDS 18		uint8

!Revision History:

   $Log: mod11_lst2.c,v $
 * Revision 4.2.0  2002/10/08  22:40     Z. Wan
 *  delete columns_per_global_grid_row since it is not used in the 1km and 6km L3 products
 * Revision 2.1.5  1999/05/17  20:20     Z. Wan
 * 1. write fill values for new update_handle
 * 2. close update_handle by the end of i_scan_loop
 * 3. if (update_handle) { } changed to if ( Hishdf ( upd_fname ) == TRUE ) { }
 * 4. DoMetaData( n_old_update, ...) moved into the above if condition.
 * Revision 2.1.4  1999/05/07  16:10     Z. Wan
 * Revision 2.1  1998/05/02  01:52     Z. Wan
 * Revision 2.0  1997/09/15  23:55     Z. Wan
 * Revision 1.3  1997/09/08  23:05     Z. Wan
 * Revision 1.2  1997/08/25  13:13     R. Cember and G. Ye, SDST
 * Revision 1.1  1996/11/20  08:56:56  Wan
 * Revision 1.0  1996/09/25  23:18:29  Wan
 * 1. Output L2 & L3
 * 2. Changed *modoutf1 to *lst_l2_outf
 * 3. Added intermediate binary data file update_for_lst
 * 4. Added day/night LST algorithm
 *
 * Revision B3.2  1995/09/11  11:26:20  wan
 *
 * Zhengming Wan: use generic variable types such as uint16, int32, float32,
 * float64, and etc. except long int required by M-API2.0
 * All allocated spaces are freed in normal and abnormal exits.
 * Write SD attributes according to the MOD11 product HDF formats.
 *
 * Revision B3.1  1995/09/01  11:07:01  wan
 * Initial version
 *
   $Id: mod11_lst2.c, v 2.1 1998/05/02 01:52 wan Exp wan $

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541
   Programmer/Analyst, Xiaoning Duan
   	duan@icess.ucsb.edu
   	805-893-7353
   ICESS
   University of California
   Santa Barbara, CA 93106-3060

   HDF Portions developed at the National Center for Supercomputing
   Applications at the Univ. of Illinois at Urbana-Champaign.

!References and Credits 

   Zhengming Wan and Jeff Dozier, "A generalized split-window algorithm for
     retrieving land-surface temperature from space", IEEE Trans. Geoscience
     and Remote Sensing, vol. 34, no. 4, pp. 892-905, 1996.

   Zhengming Wan and Zhao-Liang Li, "A Physics-based algorithm for retrieving
     land-surface emissivity and temperature from EOS/MODIS data", IEEE
     Trans. Geoscience and Remote Sensing, IEEE Trans. Geoscience and Remote
     Sensing, vol. 35, no. 4, pp. 980-996, 1997.

   Zhengming Wan, MODIS LST ATBD (ATBD-MOD-11), v3.3, April, 1999.

   Zhengming Wan developed this MODIS LST code.  Assistances with integrating
   PGS, M-API toolkits, and HDF was provided by Xiaoning Duan under helps of
   Gang Ye, and Paul Fisher of SDST/STIG.

   The part of writing metadata was implemented by Gang Ye of SDST.

   R. Cember modified the code to solve the memory leak problem, to replace
   some prohibited functions, to add function prototypes, and to reduce the
   level of nesting.

   The adapted subset of the L2G library functions developed by
   Robert E. Wolfe, MODIS Land Team Support Group in Christopher O. Justice's
   MODIS Land Science Team.

!Design Notes
   It is the phase 1 of the LST V2 code.  MODIS products Atmospheric profiles
   and Snow-cover will be used as additional inputs to the MOD11 code.

   The cumulative comments have been cleared for clarity.  To see old versions
   for the detailed modifications in different stages.

!END
******************************************************************************/ 

void Granule_tile ( int32 i_tile_proc )
{

  PGSt_integer    Vers13 = 1;
  PGSt_integer    Vers53 = 1;

  size_t   siz = 0;

  int32   i_case, n_cases = 12;
  int32   i_day_v, i_nit_v;
  int32   i_day_e_s, i_day_e_l, i_nit_e_s, i_nit_e_l;
  int32   i_day_n_s, i_day_n_l, i_nit_n_s, i_nit_n_l;
  float64 lat_6km_grid;

  int32	   n_cld_grid, n_sea_grid;
  int32	   kn, i_grid_land_type;
  float64  n_pix_land_type[N_LANDCOV_TYPES+1];
  float64  n_pix_land_type_sum = 0.0;
  float32  pct_sea_pix = 0.0;

  int32	   i_1st_loop = 1;
  int32	   i_scan_loop = 0;
  int32	   f_id = 0;
  int32    sds_id = 0;
  int32    istat = 0;
  int32    ier = 0, ier_unc = 0;
  int32	   i = 0;
  int32    j = 0;
  int32    k = 0;
  int32	   k1_data[7] = { 0, 2, 3, 8, 10, 11, 12 };
  int32	   k1_data_subset[7] = { 0, 1, 2, 3, 4, 5, 6 };
  int32    k2_cali[7] = { 0, 2, 3, 8, 10, 11, 12 };
  int32	   k1_split = 4;	/* the relative band no. for MODIS band 31 */
  int32	   k2_split = 5;	/* the relative band no. for MODIS band 32 */
  int32    k_band = 0;
  
  /* functiona and variables used in data production */

  int32	   nn = 0;
  int32	   id_coef_tbl = 0;
  int32    id_tair = 0;
  int32    id_cwv = 0;
  int32    id_ts_tair = 0;
  int32    j_pix = 0;
  int32    k_emis = 0;
  int32	   n_iterate = 3;
  int32    i_iterate = 0;
  float64  bt_to_radiance (int32, float64, int32);
  float64  f_mod_rad7_inv (int32, float32, int32);
  float64  mu_pix;
  float64  srf_elv = 0.0;
  float64  mu_mesh[9] = {
                0.415059, 0.445869, 0.475084, 0.529560, 0.626080,
                0.713005, 0.781367, 0.966438, 0.998631 };
  
  /* tair_srf=288.2 and cwv=1.42 in US standard */
  /* surface air temperature and column water vapor in mid-latitude summer */
  /* standard atmospheric condition */

  float64  tair_srf = 294.2;	      
  float64  tair_s = 294.2;	      
  float64  cwv = 2.92, cwv1;

  int32	   i_class = 0;
  int32	   i_class_state = 0; 
  uint8	   u8_1, u8_2;  
  uint8	   u8_3, u8_4;
  uint16   u16, u16r;
  float32  cov_em_6km = 0;
  float32  cov_em_6km_2 = 0;
  float32  temp_d = 0.0;
  float32  temp_d_2 = 0.0;
  float32  cov_sum_6km = 0;
  float32  temp_m_t = 0.0;
  float32  temp_m_a = 0.0;
  float32  temp_m = 0.0;
  float32  cov_sum_6km_2 = 0;
  float32  temp_m_t_2 = 0.0;
  float32  temp_m_a_2 = 0.0;
  float32  temp_m_2 = 0.0;
  float32  lst_ret = 0.0;
  static float64  e31 = 0.0;
  static float64  e32 = 0.0;
  static float64  rms_m_em = 0.0;
  static float64  rms_d_em = 0.0;
  float64  emis_m = 0.0;
  float64  emis_d = 0.0;
  float64  tmp = 0.0;
  float64  tmp_d = 0.0;
  float64  tmp_em31 = 0.0;
  float64  tmp_em32 = 0.0;
  float64  tmp_em31_2 = 0.0;
  float64  tmp_em32_2 = 0.0;
  float64  d1 = 0.0;
  float64  d2 = 0.0;
  float64  d3 = 0.0;
  float64  d4 = 0.0;
  float64  d5 = 0.0;
  float64  d6 = 0.0;
  float64  dt = 0.0;
  float64  dt_inst = 0.0;
  float64  dt_quant = 0.0;
  float64  quantization_temp[6] = { 230, 220, 210, 200, 190, 180 };
  float64  quantization_extra_err_on_lst[2][6] = {
		0.04, 0.10, 0.18, 0.29, 0.43, 0.65,
		0.00, 0.00, 0.00, 0.07, 0.16, 0.29 };

  /* variables for output */

  /* long     int dims_o[2] = {100, 1354}; 12/12/99 Z. Wan */
  long     int dims_1d_o[1] = { 1200 };
  long     int sta_1d_o[1] = { 0 };

  float64  Scale_factor_offset[2][3] = {
                50, 25, 500,
		0,  0, -245
	      };

  int32		i_water_pix = 0;
  int32		n_1km_l3_old_file;
  long int	sta_1km_o[2], dims_1km_o[2];
  float32	cell_cov, cell_snow_sum, cell_ice_sum;
  float32       obs_cov, obs_cov_sum;
  int32		i_snow_input;
  float32	upd_cov_d_max[N2_VIEW_ANGS], upd_cov_n_max[N2_VIEW_ANGS];

  int32		n_6km_l3_old_file;
  long int	sta_6km_o[2], dims_6km_o[2];

  /* variables for setting up tile */

  float64 cen[2];	/* geodetic coordinates of the center of the current 
  			 * observation */

  float32 **lat_p, **lon_p;	/* pointers to lat & lon */

  int16 np;		/* number of parameters ini observation pointer file */
  char msgString[PGS_SMF_MAX_MSG_SIZE]; /* msg string buffer for SMF */

#define LAU 3		/* Lau's version 1 code with zooming */

#if LAU > 0
  int32 i1, i2, i3, i4;
  int32 j1, j2, j3, j4;
  /* sinusoidal grid x; for ref_lon=-PI  */  
#endif

  int32 ii1;
  int32 lkk;
  dgeo_t p_cor[4];

  /* parameters (orginally in GenSamp) used for calling GenObs */
  obs_t obs_1km[MAX_NOBS];  /* observation pointer * data structures at 1km */
  int32 nobs_1km;               /* number of observations at 1km resolution */
  int32 iobs_1km;		/* variable number of an observation */
  int32	i_1km, j_1km;		/* row and column numbers of 1km L3 cells */
  int32	i_6km, j_6km;		/* row and column numbers of 6km L3 cells */
  uint8 indx_6km_6d;

  /* variables used in day/night LST algorithm */
  uint8 vtime_d_in = 0;
  uint8 vtime_n_in = 0;
  int32 i_pair_type = 0;
  int32 i_pair_exist = 0;
  int32	n_pair_new = 0;
  double chisq = 0.;
 
  /* solar irradiance correction factor for the Sun's elliptic orbit */
  int32   i_day_yr, i_day_yr_sv = -1;
  int32   nday[13] = {
		1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 };

  double  sol_rat[13] = {
		1.034, 1.030, 1.019, 1.001, 0.985, 0.972, 0.967,
		0.971, 0.982, 0.998, 1.015, 1.029, 1.034 };

  float32 *t_prof_S_p, *w_prof_S_p;
  int16   *w_prof_p, ts_prof_v;
  uint16  band_data;
  int32	  tmp_int = 0;
  int32	  ipx_bad_split_window = 0;
  int32	  ipx_bad_day_nit = 0;
  int32   ipx_other_qual_sw = 0;
  int32   ipx_other_qual_dn = 0;
  int32	  n_old_1km = 0;
  int32	  i_ch3 = 0;
  int32	  i_upd_ang, n_upd_angs = N_VIEW_ANGS;
  int32	  i_upd_a_o, n2_upd_angs = N2_VIEW_ANGS;
  int32	  i_day_nit = 0, i_clean_upd = 0;
  int16   i_view_6km_from_east = 0;
  int16   view_ang_6km_upd = 0;
  int32	  i_view = 0, i_view_p1 = 1;
  uint8	  band_uncert, band_uncert1;
  uint8	  lc_land_sea = 0;
  uint8	  u8_qa_1km_man, u8_qa_1km_dat, u8_qa_1km_emer, u8_qa_1km_lster;
  uint8	  u8_qa_day, u8_qa_nit;

  /* for Grid structure of 250m grid for all L3 MODLAND products */
  int32	  irows_250m, irowe_250m, icols_250m, icole_250m;
  int32	  nColAtCen_250m, Nrow_250m;
  float64 clat_250m, colDist_250m;
  float64 ulCornerGrid[2];
  float64 lrCornerGrid[2];
  float64 projParam[13] = { 6371007.181, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
			    86400.0, 0.0, 1.0, 0.0, 0.0 };
  int32	  projType;

  /* Initialize the binning and tiling functions */
  tile.itile_horiz = uparam.itile_horiz;
  tile.itile_vert = uparam.itile_vert;
  tile.itile_size = uparam.itile_size;

  InitBin(&tile, uparam.itile_size);

if ( i_tile_proc == 0 ) {

  j = TILE_XDIM;		/* always for a whole scan line */

  /* allocate data array for input landcov */

  siz = sizeof (uint8);
  land_type = (uint8 **) alloc_2d (tile.nrow, j, siz);
  landcov_qc = (uint8 ***) alloc_3d (1, tile.nrow, j, siz);
  lc_qc = landcov_qc[0];

  /* allocate data arrays for output 1km L3 LST */

  siz = sizeof (uint16);
  lst_1km_Sv = (uint16 **) alloc_2d (1, j, siz);
  lst_1km_Sp = lst_1km_Sv[0];
  clear_day_cov = (uint16 **) alloc_2d (1, j, siz);
  clear_nit_cov = (uint16 **) alloc_2d (1, j, siz);
  clear_day_cov_p = clear_day_cov[0];
  clear_nit_cov_p = clear_nit_cov[0];
  siz = sizeof (uint8);
  QC_1km_l3 = (uint8 **) alloc_2d (1, j, siz);
  QC_1km_l3_p = QC_1km_l3[0];
  v_time_1km_l3_Sv = (uint8 **) alloc_2d (1, j, siz);
  v_angl_1km_l3_Sv = (uint8 **) alloc_2d (1, j, siz);
  v_time_1km_l3_Sp = v_time_1km_l3_Sv[0];
  v_angl_1km_l3_Sp = v_angl_1km_l3_Sv[0];
  em31_1km_l3_Sv = (uint8 **) alloc_2d (1, j, siz);
  em32_1km_l3_Sv = (uint8 **) alloc_2d (1, j, siz);
  em31_1km_l3_Sp = em31_1km_l3_Sv[0];
  em32_1km_l3_Sp = em32_1km_l3_Sv[0];

  siz = sizeof (float32);
  i1 = TILE_YDIM;
  cov_1km_day_S = (float32 **) alloc_2d (i1, j, siz);
  cov_1km_nit_S = (float32 **) alloc_2d (i1, j, siz);
  lst_1km_S = (float32 **) alloc_2d (i1, j, siz);
  siz = sizeof (uint8);
  qc_1km_l3d = (uint8 **) alloc_2d (i1, j, siz);
  qc_1km_l3n = (uint8 **) alloc_2d (i1, j, siz);
  siz = sizeof (float32);
  em31_1km_l3_S = (float32 **) alloc_2d (i1, j, siz);
  em32_1km_l3_S = (float32 **) alloc_2d (i1, j, siz);
  vtime_1km_l3_S = (float32 **) alloc_2d (i1, j, siz);
  vangl_1km_l3_S = (float32 **) alloc_2d (i1, j, siz);

  /* allocate data arrays for update_for_lst */

  siz = SZ_UPD_ANG_LST;
  upd_ang_lst_W =(_UPD_ANG_LST **) alloc_2d (nrow_6km, ncol_6km, siz);

  siz = SZ_UPD_ANG_LST_SV;
  upd_dims[1] = ncol_6km;
  upd_dims[2] = siz / sizeof (uint16);
  if ( upd_dims[2] != TILE_ZDIM_UPD )
	SMFLOG("STOP--check update_for_lst.h and related parameters");
  upd_ang_lst_Sv =(_UPD_ANG_LST_SV **) alloc_2d (nrow_6km, ncol_6km, siz);

  siz = sizeof (float32);
  cov_prof_d =(float32 ***) alloc_3d (nrow_6km, ncol_6km, N2_VIEW_ANGS, siz);
  cov_prof_n =(float32 ***) alloc_3d (nrow_6km, ncol_6km, N2_VIEW_ANGS, siz);

  /* allocate data arrays for output 6km L3 LST */

  i = 1;
  j = ncol_6km;
  siz = sizeof (uint16);
  lst_6km_S_from_1km = (uint16 **) alloc_2d (i, j, siz);
  lst_6km_Sp_from_1km = lst_6km_S_from_1km[0];
  lst_6km_Sd = (uint16 **) alloc_2d (i, j, siz);

  siz = sizeof (float32);
  sf_elev_6km_d = (float32 **) alloc_2d (nrow_6km, j, siz);
  sf_elev_6km_n = (float32 **) alloc_2d (nrow_6km, j, siz);
  em31_6km_d = (float32 **) alloc_2d (nrow_6km, j, siz);
  em31_6km_n = (float32 **) alloc_2d (nrow_6km, j, siz);
  upd_cov_d = (float32 ***) alloc_3d (nrow_6km, j, N2_VIEW_ANGS, siz);
  upd_cov_n = (float32 ***) alloc_3d (nrow_6km, j, N2_VIEW_ANGS, siz);

  siz = sizeof (int16);
  d_azm_6km = (int16 **) alloc_2d (nrow_6km, j, siz);

  siz = sizeof (uint8);
  QC_6km_day = (uint8 **) alloc_2d (i, j, siz);
  v_time_6km_l3_Sd = (uint8 **) alloc_2d (nrow_6km, j, siz);
  siz = sizeof (uint16);
  lst_6km_Sn = (uint16 **) alloc_2d (i, j, siz);
  siz = sizeof (int16);
  v_angl_6km_l3_Sd = (int16 **) alloc_2d (nrow_6km, j, siz);
  v_angl_6km_l3_Sn = (int16 **) alloc_2d (nrow_6km, j, siz);
  siz = sizeof (uint8);
  QC_6km_nit = (uint8 **) alloc_2d (i, j, siz);
  v_time_6km_l3_Sn = (uint8 **) alloc_2d (nrow_6km, j, siz);
  v_time_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  v_angl_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em20_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em22_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em23_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em29_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em31_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em32_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  QC_emis = (uint8 **) alloc_2d (i, j, siz);
  pct_land = (uint8 **) alloc_2d (i, j, siz);

  siz = sizeof (uint8);
  snowcv_old = (uint8 **) alloc_2d (6*nrow_6km, j, siz);
  lakeice_old = (uint8 **) alloc_2d (6*nrow_6km, j, siz);

  slope_6km = (uint8 ***) alloc_3d(N_LAND_TILE, nrow_6km, ncol_6km, siz);

 /* read the slope file if available */
 Vers_tmp = 1;
 Vers = Vers_tmp;
 returnStatus = PGS_PC_GetReference(PCLogslopelut,&Vers_tmp,slope_fname);
 if ( returnStatus != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_GET_GEN_FILE, "slope_landonly_6km.hdf");

   if ( Hishdf ( slope_fname ) == TRUE ) {

     if ((slope_handle = openMODISfile (slope_fname, "r")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, slope_fname);

     {
       long int	sta_slope[3] = { 0, 0, 0 };
       long int	dims_slope[3] = { N_LAND_TILE, TILE_YDIM_6KM, TILE_XDIM_6KM };

       getMODISarray(slope_handle, "SlopeLandOnlyCoarse", "\0",
	sta_slope, dims_slope, &slope_6km[0][0][0]);
     }
     closeMODISfile(&slope_handle);
  }
  else {
   for ( k = 0; k < N_LAND_TILE; k++ ) {
     for ( i_6km = 0; i_6km < TILE_YDIM_6KM; i_6km++ ) {
	for ( j_6km = 0; j_6km < TILE_XDIM_6KM; j_6km++ )
		slope_6km[k][i_6km][j_6km] = 0;
     }
   }	
  }
} /* i_tile_proc == 0 */

  /* R. Cember: initialize snowcv_old. */
  zeroout_general(&snowcv_old[0][0], 6 * nrow_6km * ncol_6km, 1);
  zeroout_general(&lakeice_old[0][0], 6 * nrow_6km * ncol_6km, 1);

  if ( i_tile_proc == 0 && uparam.itile_size > 1 ) {
	for (i = 0; i < NSD_LST_1KM_L3; i++) {
		lst_1km_l3[i].dims[0] /= uparam.itile_size;
		lst_1km_l3[i].dims[1] /= uparam.itile_size;
	}
	for (i = 0; i < NSD_LST_6KM_L3; i++) {
		lst_6km_l3[i].dims[0] /= uparam.itile_size;
		lst_6km_l3[i].dims[1] /= uparam.itile_size;
	}
  }

  /* determine the tile boundary */

  if ( tile.itile_vert > 8 ) {

   for ( i1 = 0; i1 < set->nrow_tile; i1++ )
	if ( set->row[i1].ncol_tile )
		break;
   j1 = 0;
   i2 = i1;
   j2 = set->row[i2].ncol_tile - 1;
   if ( tile.itile_horiz > 17 ) {
    j3 = set->row[i2].ncol_tile;
    for ( i3 = i2 + 1; i3 < set->nrow_tile; i3++ ) {
	if ( set->row[i3].ncol_tile == j3 )	continue;
	else		break;
    }
    if ( i3 == set->nrow_tile )	i3--;
    j3 -= 1;
    for ( i4 = set->nrow_tile - 1; i4 > 0; i4-- ) {
	if ( ! set->row[i4].ncol_tile )	continue;
	if ( set->row[i4].ncol_tile )
		break;
    }
    j4 = 0;
   }
   else {
    j4 = set->row[i2].ncol_tile;
    for ( i4 = i2 + 1; i4 < set->nrow_tile; i4++ ) {
	if ( set->row[i4].ncol_tile == j4 )	continue;
	else		break;
    }
    if ( i4 == set->nrow_tile )	i4--;
    j4 = 0;
    for ( i3 = set->nrow_tile - 1; i3 > 0; i3-- ) {
	if ( ! set->row[i3].ncol_tile )	continue;
	if ( set->row[i3].ncol_tile )
		break;
    }
    j3 = set->row[i3].ncol_tile - 1;
   }
  }
  else {
   if ( tile.itile_horiz > 17 ) {
    for ( i1 = 0; i1 < set->nrow_tile; i1++ )
	if ( set->row[i1].ncol_tile )
		break;
    j1 = 0;
    i2 = set->nrow_tile - 1;
    j2 = set->row[i2].ncol_tile;
    for ( i2 = i2 - 1; i2 > 0; i2-- ) {
	    if ( set->row[i2].ncol_tile == j2 )     continue;
	    else            break;
    }
    j2 -= 1;
   }
   else {
    for ( i2 = 0; i2 < set->nrow_tile; i2++ )
	if ( set->row[i2].ncol_tile )
		break;
    j2 = set->row[i2].ncol_tile - 1;
    i1 = set->nrow_tile - 1;
    j1 = set->row[i1].ncol_tile;
    for ( i1 = i1 - 1; i1 > 0; i1-- ) {
	if ( set->row[i1].ncol_tile == j1 )	continue;
	else		break;
    }
    j1 = 0;
   }
   for ( i3 = set->nrow_tile - 1; i3 > 0; i3-- )
	if ( set->row[i3].ncol_tile )
		break;
   j3 = set->row[i3].ncol_tile - 1;
   i4 = i3;
   j4 = 0;
  }

  (*pLbin_Inv)(i1, j1, 0., -0.499, &tile_LatLons[1][0], &tile_LatLons[0][0]);
  if ( i1 == i4 && j1 == j4 )
   (*pLbin_Inv)(i4, j4, 0.499,-0.499, &tile_LatLons[1][3], &tile_LatLons[0][3]);
  else
   (*pLbin_Inv)(i4, j4, 0., -0.499, &tile_LatLons[1][3], &tile_LatLons[0][3]);
  (*pLbin_Inv)(i2, j2, 0.,  0.499, &tile_LatLons[1][1], &tile_LatLons[0][1]);
  if ( i2 == i3 && j2 == j3 )
   (*pLbin_Inv)(i3, j3, 0.499, 0.499, &tile_LatLons[1][2], &tile_LatLons[0][2]);
  else
   (*pLbin_Inv)(i3, j3, 0.,  0.499, &tile_LatLons[1][2], &tile_LatLons[0][2]);

  for ( i1 = 0; i1 < 2; i1++)
     for ( i2 = 0; i2 < 4; i2++) {
	tile_LatLons[i1][i2] = RTD( tile_LatLons[i1][i2] );
	tile_LatLons_edge[i1][i2] = tile_LatLons[i1][i2];
     }
  /* ####
  tile_LatLons_edge[0][0] = 90.0 - tile_pntr[i_tile_proc].iv * 10.0;
  tile_LatLons_edge[0][1] = tile_LatLons_edge[0][0];
  tile_LatLons_edge[0][2] = tile_LatLons_edge[0][0] - 10.0;
  tile_LatLons_edge[0][3] = tile_LatLons_edge[0][2];	*/

  lat_6km_1row = tile_LatLons[0][0] - 2.0 * DLAT_6KM_ROW;

  tile_bound[2] = -180;
  for ( i1 = 0; i1 < 4; i1++)
	if ( tile_LatLons[1][i1] > tile_bound[2] )
		tile_bound[2] = tile_LatLons[1][i1];

  tile_bound[3] = 180;
  for ( i1 = 0; i1 < 4; i1++)
	if ( tile_LatLons[1][i1] < tile_bound[3] )
		tile_bound[3] = tile_LatLons[1][i1];

  tile_bound[0] = -90;
  for ( i1 = 0; i1 < 4; i1++)
	if ( tile_LatLons[0][i1] > tile_bound[0] )
		tile_bound[0] = tile_LatLons[0][i1];

  tile_bound[1] = 90;
  for ( i1 = 0; i1 < 4; i1++)
	if ( tile_LatLons[0][i1] < tile_bound[1] )
		tile_bound[1] = tile_LatLons[0][i1];

  /* set tile corners for metadata output later.  Note that the corners
      are defined by cell corners, not cell centers.
             0 ----- 1     +---> column
             |       |     |
             |   *   |     |
             |       |     V
             3 ----- 2    row
  */

  /* Set up parameter list */
  np = NSD_LST_1KM_L3;

  /* get gd_1km */
  gd = Obs_CreateGrid(tile.nrow, tile.npix, np);

  if (gd == NULL)
    Error("create observation grid data structure", "mod11_lst");

  /* open the landcover input file */

  /* Vers_lc = tile_pntr[i_tile_proc].s_num + 1; */
  id2 = tile_pntr[i_tile_proc].s_num;
  Vers_lc = ver_landcv_to_lut[id2];
  Vers_tmp = Vers_lc;

  if (PGS_PC_GetReference(PCLogLC, &Vers_tmp, fname_LC) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogLC, Vers_lc);

  if ((modinflc = openMODISfile (fname_LC, "r")) == NULL)
     HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_LC);

  /* input landcover */
  if((ier = getMODISarray(modinflc, land_cv[0].name, land_cv[0].gname, sta_lc,
	   land_cv[0].dims, &land_type[0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray Land_Cover_Type_1");

  if((ier = getMODISarray(modinflc, land_cv[1].name, land_cv[1].gname,
	   lc_qc_sta, lc_qc_dims, &landcov_qc[0][0][0])) == -1) {

    /* Land_Cover_Type_QC not found, try to read Land_Cover_QC */
    if((ier = getMODISarray(modinflc, "Land_Cover_QC", land_cv[1].gname,
	   lc_qc_sta, lc_qc_dims, &landcov_qc[0][0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray Land_Cover_QC");
  }

  for ( i = 0; i < TILE_YDIM; i++ ) {
	for ( j = 0; j < TILE_XDIM; j++ ) {
		if ( land_type[i][j] == 255 )
				land_type[i][j] = 20;	/* filled grid */
		else if (land_type[i][j] == 254)
				land_type[i][j] = 17;	/* unclassified */
		     else {
				lc_land_sea = lc_qc[i][j] >> 4 & ~(~0 << 4);
				if ( lc_land_sea == 2 )
					land_type[i][j] = 18;	/* coastline */
				if ( lc_land_sea == 0 || lc_land_sea > 5 )
					land_type[i][j] = 19;	/* ocean */
		     }
	}
  }

  /* close the landcover file */
  returnStatus = closeMODISfile (&modinflc);
  if (returnStatus == MFAIL)
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "landcover file failed");

n_stops[1] += 1;

  /* open a HDF file for 6km L3 LST output */

  Vers53 = tile_pntr[i_tile_proc].s_num + 1;
  Vers_tmp = Vers53;

  if(PGS_PC_GetReference(PCLogL3_6km,&Vers_tmp,fname_6km_L3) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL3_6km, Vers53);

  rm_path(fname_6km_L3, esdt_tmp);
  sscanf ( esdt_tmp, "%7s", &esdt_6km_L3[0] );

  if ( Hishdf (fname_6km_L3) == TRUE ) {

	if ((lst_6km_l3_outf = openMODISfile (fname_6km_L3, "a")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE,fname_6km_L3);

  	n_6km_l3_old_file = 1;
	tile_pntr[i_tile_proc].l3_6km_exist = 1;

  }
  else {

    n_6km_l3_old_file = 0;
    tile_pntr[i_tile_proc].l3_6km_exist = 0;

  }

  /* Setup map projection parameters */

  for (i = 0; i < 13; i++) projParam[i] = 0.0;

  if (set->proj == LBIN_ISIN) {
    projType = GCTP_ISINUS;
    projParam[0] = LBIN_RE_ISIN;

    /* For the ISIN grid the map projection parameter 'projParam[8]' is
     * set to 86400.0 to be compatible with the other MODIS land programs.
     * In reality, the value used by this program
     * is 'set->nrow' (== 21600 or 180 * 60 * 2).
     * Robert Wolfe, 2002/09/12.
     */
    projParam[8] = 86400.0;
    projParam[10] = (double)1.0;
  } else {
    projType = GCTP_SNSOID;
    projParam[0] = LBIN_RE_SIN;
    projParam[8] = 86400.0;
  }

/* create SDS for 6km LST L3 files */

if ( !n_6km_l3_old_file ) {

 {
  intn		hdfReturn;
  int32		gfid, gid;
  int32		nn, ntilev;

  ntilev = 18;
  Nrow_250m = projParam[8];
  nn = Nrow_250m / ntilev;
  clat_250m = 0.5 * PI / (double) Nrow_250m;
  nColAtCen_250m = floor( 2.0 * cos(clat_250m) * Nrow_250m + 0.5 );
  colDist_250m = (2.0 * PI * projParam[0]) / (double) nColAtCen_250m;
  irows_250m = nn * tile_pntr[i_tile_proc].iv;
  irowe_250m = irows_250m + nn;
  icols_250m = nn * tile_pntr[i_tile_proc].ih;
  icole_250m = icols_250m + nn;

  ulCornerGrid[0] = (icols_250m - nColAtCen_250m / 2.) * colDist_250m;
  ulCornerGrid[1] = ( Nrow_250m / 2. - irows_250m) * colDist_250m;
  lrCornerGrid[0] = (icole_250m - nColAtCen_250m / 2.) * colDist_250m;
  lrCornerGrid[1] = ( Nrow_250m / 2. - irowe_250m ) * colDist_250m;

  if( (gfid = GDopen(fname_6km_L3, DFACC_CREATE)) == FAIL )
	HALT("GDopen failed for 6km L3 LST file");

  if( (gid = GDcreate(gfid, "MODIS_Grid_Daily_6km_LST", TILE_XDIM_6KM,
		TILE_YDIM_6KM, ulCornerGrid, lrCornerGrid)) == FAIL )
	HALT("GDcreate failed for 6km L3 LST file");

  if ( projType == GCTP_ISINUS) {
     if( GDdefproj(gid, GCTP_ISINUS, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for 6km L3 LST file");
  } else {
     if( GDdefproj(gid, GCTP_SNSOID, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for 6km L3 LST file");
  }

  if( GDdeforigin(gid, HDFE_GD_UL) == FAIL )
	HALT("GDdeforigin failed for 6km L3 LST file");

  for (i = 0; i < NSD_LST_6KM_L3; i++) {

   hdfReturn = GDdeffield(gid, lst_6km_l3[i].name, "YDim,XDim",
		lparam_6km_l3[i].type, HDFE_NOMERGE );
   if ( hdfReturn == FAIL ) {
	   sprintf (msgString, "GDdeffield failed on %s", lst_6km_l3[i].name);
	   HALT(msgString);
   }

  }

  if( GDattach(gfid, "MODIS_Grid_Daily_6km_LST") == FAIL )
	HALT("GDattach failed for 6km L3 LST file");

  if( GDdetach(gid) == FAIL )
	HALT("GDdetach failed for 6km L3 LST file");

  if( GDclose(gfid) == FAIL )
	HALT("GDclose failed for 6km L3 LST file");

 }

 if ( (lst_6km_l3_outf = openMODISfile (fname_6km_L3, "a")) == NULL )
	HALT("openMODISfile failed on 6km L3 LST file");

}

  /* initialization */

  n_update_visit = 0;

  ii1 = TILE_YDIM;	lkk = TILE_XDIM;

  for (i = 0; i < ii1; i++) {
      for (j = 0; j < lkk; j++) {

	if ( land_type[i][j] == 19 ) {
		qc_1km_l3d[i][j] = SEA_BIT;
		qc_1km_l3n[i][j] = SEA_BIT;
	}
	else {
		qc_1km_l3d[i][j] = CLD_BIT;
		qc_1km_l3n[i][j] = CLD_BIT;
	}
      }
  }
  z_siz = TILE_YDIM * TILE_XDIM;
  zeroout_general ( &cov_1km_day_S[0][0], z_siz, 4);
  zeroout_general ( &cov_1km_nit_S[0][0], z_siz, 4);
  zeroout_general ( &em31_1km_l3_S[0][0], z_siz, 4);
  zeroout_general ( &em32_1km_l3_S[0][0], z_siz, 4);

  z_siz = ncol_6km;
  zeroout_general ( &sf_elev_6km_d[0][0], z_siz * z_siz, 4);
  zeroout_general ( &sf_elev_6km_n[0][0], z_siz * z_siz, 4);
  zeroout_general ( &em31_6km_d[0][0], z_siz * z_siz, 4);
  zeroout_general ( &em31_6km_n[0][0], z_siz * z_siz, 4);
  zeroout_general ( &upd_cov_d[0][0][0], N2_VIEW_ANGS * z_siz * z_siz, 4);
  zeroout_general ( &upd_cov_n[0][0][0], N2_VIEW_ANGS * z_siz * z_siz, 4);
  zeroout_general ( &v_time_6km_l3_Sd[0][0], nrow_6km * z_siz, 1);
  zeroout_general ( &v_time_6km_l3_Sn[0][0], nrow_6km * z_siz, 1);
  zeroout_general ( &v_angl_6km_l3_Sd[0][0], nrow_6km * z_siz, 2);
  zeroout_general ( &v_angl_6km_l3_Sn[0][0], nrow_6km * z_siz, 2);
  zeroout_general ( &d_azm_6km[0][0], nrow_6km * z_siz, 2);

  z_siz = N2_VIEW_ANGS * nrow_6km * ncol_6km;
  zeroout_general ( &cov_prof_d[0][0][0], z_siz, 4);
  zeroout_general ( &cov_prof_n[0][0][0], z_siz, 4);

  z_siz = nrow_6km * ncol_6km;
  for ( k = 0; k < N2_VIEW_ANGS; k++ ) {
    zeroout_general ( &t_prof_sum_d[k][0][0][0], z_siz, 4);
    zeroout_general ( &t_prof_sum_n[k][0][0][0], z_siz, 4);
  }

  z_siz = nrow_6km * ncol_6km * SZ_UPD_ANG_LST_SV / sizeof(uint16);
  zeroout_general ( &upd_ang_lst_Sv[0][0], z_siz, 2);

  z_siz = nrow_6km * ncol_6km * SZ_UPD_ANG_LST / sizeof(int32);
  zeroout_general ( &upd_ang_lst_W[0][0], z_siz, 4);

  for ( k = 0; k < n2_upd_angs; k++ )
	upd_cov_d_max[k] = upd_cov_n_max[k] = 0;

  /* OPEN the intermediate update_for_lst file */

  Vers_tmp = tile_pntr[i_tile_proc].s_num + 1;
  Vers = Vers_tmp;
  returnStatus = PGS_PC_GetReference(PCLogupdate[i_days_in_cycle_update-1],
		&Vers_tmp, upd_fname);
  if ( returnStatus != PGS_S_SUCCESS )
	   HALTsmf(MODIS_F_GET_GEN_FILE, "update_for_lst");

  rm_path(upd_fname, esdt_tmp);
  sscanf ( esdt_tmp, "%8s", &esdt_lst_upd[0] );

  if ( i_do_multi_days ) {

	sscanf ( &esdt_tmp[10], "%1s", &trash_char[0] );
	n_days_per_cycle_update_pcf = atoi (trash_char);
	sscanf ( &esdt_tmp[12], "%1s", &trash_char[0] );
	i_days_in_cycle_update_pcf = atoi (trash_char);

	if ( esdt_tmp[9] != id_char_update[0] ||
	     n_days_per_cycle_update_pcf != n_days_per_cycle_update ||
	     i_days_in_cycle_update_pcf != i_days_in_cycle_update ) {
	  sprintf(msgString,"the UPD file name does not start with %s.%c%d-%d.",
		esdt_lst_upd, id_char_update[0], n_days_per_cycle_update,
		i_days_in_cycle_update);
	  HALT(msgString);
	}
  }
  else {
	if ( esdt_tmp[9] != '1' ) {
	   sprintf (msgString, "the UPD file name does not start with %s.1.",
		esdt_lst_upd );
	   HALT(msgString);
	}
  }

  if ( Hishdf ( upd_fname ) == TRUE ) {

    if ((update_handle = openMODISfile (upd_fname, "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname);

    n_old_update = 1;
    tile_pntr[i_tile_proc].update_exist = 1;

    closeMODISfile (&update_handle);
  }
  else {

  /* create Grid structure for update_for_lst files */
  {
  intn		hdfReturn;
  int32		gfid, gid;

  if( (gfid = GDopen(upd_fname, DFACC_CREATE)) == FAIL )
	HALT("GDopen failed for update_for_lst file");

  if( (gid = GDcreate(gfid, "MODIS_Grid_Daily_UPD_LST", TILE_XDIM_6KM,
		TILE_YDIM_6KM, ulCornerGrid, lrCornerGrid)) == FAIL )
	HALT("GDcreate failed for update_for_lst file");

  if ( projType == GCTP_ISINUS ) {
     if( GDdefproj(gid, GCTP_ISINUS, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for update_for_lst file");
  } else {
     if( GDdefproj(gid, GCTP_SNSOID, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for update_for_lst file");
  }

  if( GDdeforigin(gid, HDFE_GD_UL) == FAIL )
	HALT("GDdeforigin failed for update_for_lst file");

  if( GDdefdim(gid, "YDim", TILE_YDIM_6KM) == FAIL )
	HALT("GDdefdim failed YDim for update_for_lst file");

  if( GDdefdim(gid, "XDim", TILE_XDIM_6KM) == FAIL )
	HALT("GDdefdim failed XDim for update_for_lst file");

  if( GDdefdim(gid, "ZDim", TILE_ZDIM_UPD) == FAIL )
	HALT("GDdefdim failed ZDim for update_for_lst file");

  for (i = 0; i < 1; i++) {

	hdfReturn = GDdeffield(gid, upd_lst_6km[i].name, "YDim,XDim,ZDim",
		lparam_upd_lst[i].type, HDFE_NOMERGE );
   if ( hdfReturn == FAIL ) {
	   sprintf (msgString, "GDdeffield failed on %s", upd_lst_6km[i].name);
	   HALT(msgString);
   }

  }

  if( GDattach(gfid, "MODIS_Grid_Daily_UPD_LST") == FAIL )
	HALT("GDattach failed for update_for_lst file");

  if( GDdetach(gid) == FAIL )
	HALT("GDdetach failed for update_for_lst file");

  if( GDclose(gfid) == FAIL )
	HALT("GDclose failed for update_for_lst file");

  }

  if ((update_handle = openMODISfile (upd_fname, "a")) == NULL)
	   HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname);

  n_old_update = 0;
  tile_pntr[i_tile_proc].update_exist = 0;

  /* initialize slope_6km in snow_bits of layer 8 for each grids */
  {
	  int16 i_seq;

	  i_seq = tile_pntr[i_tile_proc].s_num;

	  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {
	    for (j_6km = 0; j_6km < ncol_6km; j_6km++) {
		upd_a_lst_S_svp = &upd_ang_lst_Sv[i_6km][j_6km];
		upd_lst_S_svp = &upd_a_lst_S_svp->_upd_for_lst_sv[8];
		upd_lst_S_svp->snow_bits =
					slope_6km[i_seq][i_6km][j_6km];
		upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];
		upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[8];
		upd_lst_w_p->snow_bits = upd_lst_S_svp->snow_bits;
	    }
	  }
  }

	/* write initial and fill values */

	upd_sta[0] = 0;		upd_dims[0] = nrow_6km;
	ier = putMODISarray(update_handle,upd_lst_6km[0].name,
		upd_lst_6km[0].gname, upd_sta, upd_dims, &upd_ang_lst_Sv[0][0]);
	if (ier == -1)
		HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray to upd_ang_lst_sv");
	closeMODISfile (&update_handle);
 }

 /* this loop is designed for dealing with mixed scan mode */
 for ( i_scan_loop = 0; i_scan_loop < 2; i_scan_loop++ ) {

  if (i_scan_loop == 0) {
	if ( !(tile_pntr[i_tile_proc].scan_mode == 1 ||
	     tile_pntr[i_tile_proc].scan_mode == 2) )	continue;

   	nit_mode = 0;		/* day scan mode */
	qc_1km_l3 = qc_1km_l3d;
	cov_1km_Sp = cov_1km_day_S;
  }
  else	{
	if ( !(tile_pntr[i_tile_proc].scan_mode == 0 ||
	     tile_pntr[i_tile_proc].scan_mode == 2) )	continue;

   	nit_mode = 1;		/* nit scan mode */
	qc_1km_l3 = qc_1km_l3n;
	cov_1km_Sp = cov_1km_nit_S;
  }

 /* #### need BRDF only for daytime */
 i_brdf = 0;
 if ( !nit_mode ) {

  /*next piece of logic added by C. Collins on 10/20/04 so that code determines
if using the MOD43B1 or MOD43B1C products first and then knows which LUN to
rence and which number of files to use, modified by Z. Wan in V5.0.2 */
  if ( numfiles_brdf ) {
		brdf_num_files = numfiles_brdf;
		brdf_lun = PCLogBRDF;
  }
  else if ( numfiles_brdf1km ) {
		brdf_num_files = numfiles_brdf1km;
		brdf_lun = PCLogBRDF1km;
  }
  /*C. Collins changes numfiles_brdf in next line to brdf_num_files*/

  if ( Vers_lc <= brdf_num_files ) {

     Vers_tmp = Vers_lc;
/*C. Collins changed PCLogBRDF in next line to brdf_lun*/
     if (PGS_PC_GetReference(brdf_lun, &Vers_tmp, fname_brdf) == PGS_S_SUCCESS) {

     if ( Hishdf (fname_brdf) == TRUE) {

	if ( (modinfbrdf = openMODISfile (fname_brdf, "r")) != NULL)
		i_brdf = 1;

	if (getMODISardims(modinfbrdf, brdf_param, brdf_grpnm, data_type,
		&rank_brdf, dim_brdf_sizes) == FAIL) {
	  sprintf (modstring, "getMODISardims of BRDF_Albedo_Parameters");
	  HALT(modstring);
	}
	if ( i_brdf ) {
	 if ( brdf_3p_6km == NULL ) {
	   siz = sizeof (int16);
	   brdf_3p_6km = (int16 ***) alloc_3d (200*200, 1, 3, siz);
	   brdf_p_6km = (int16 ****) malloc (200 * sizeof(int16 *));
	   for ( i = 0; i < TILE_YDIM_6KM; i++ )
		   brdf_p_6km[i] = brdf_3p_6km + i * 200;
	 }
	 if ( dim_brdf_sizes[0] == brdf_p_dims[0] ) { /* from 1km MOD43B1 */
	   if ( brdf_p == NULL ) {
		siz = sizeof (int16);
		brdf_3p = (int16 ***) alloc_3d (TILE_YDIM*TILE_XDIM, 1, 3, siz);
		brdf_p = (int16 ****) malloc (TILE_YDIM * sizeof(int16 *));
		for ( i = 0; i < TILE_YDIM; i++ )
			brdf_p[i] = brdf_3p + i * TILE_XDIM;
	   }
	   sta_brdf_p[0] = 0;
	   sta_brdf_p[1] = 0;
	   sta_brdf_p[2] = 6;		/* input band 7 only */
	   if ( (ier = getMODISarray(modinfbrdf, brdf_param, brdf_grpnm,
		sta_brdf_p, brdf_p_dims, &brdf_p[0][0][0][0] )) == -1)
	    HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray 1km BRDF_Albedo_Parameters");
	   /* calculate BRDF parameters at 6km resolution */
	   for (i_6km = 0; i_6km < nrow_6km; i_6km++) {
	     for (j_6km = 0; j_6km < ncol_6km; j_6km++) {
	       {
		int32 i, j, k, ii6f, ii6l, jj6f, jj6l;
		int16 *brdf_p_p, n_pts_brdf;

		ii6f = 6 * i_6km;
		ii6l = ii6f + 6;
		jj6f = 6 * j_6km;
		jj6l = jj6f + 6;

		n_pts_brdf = 0;
		brdf_6km[0] = brdf_6km[1] = brdf_6km[2] = 0.0;

		for (i = ii6f; i < ii6l; i++) {
		 for (j = jj6f; j < jj6l; j++) {
			brdf_p_p = brdf_p[i][j][0];
			for ( k = 0; k < 3; k++ ) {
				if ( brdf_p_p[k] <= 32766 ) {
					brdf_6km[k] += brdf_p_p[k];
					if ( k == 0 )   n_pts_brdf++;
				}
			}
		 }
		}
		if ( n_pts_brdf ) {
			for ( k = 0; k < 3; k++ )
				brdf_6km[k] /= n_pts_brdf;
		}
	       }
	       brdf_p_p = brdf_p_6km[i_6km][j_6km][0];
	       brdf_p_p[0] = brdf_6km[0];
	       brdf_p_p[1] = brdf_6km[1];
	       brdf_p_p[2] = brdf_6km[2];

	     } /* the j_6km loop */
	   }
	 }
	 else if ( dim_brdf_sizes[0] == brdf_p_dims_6km[0] ) {
	   /* input BRDF data from 6km MOD43B1C file */
	   brdf_p_dims_6km[2] = 1;
	   brdf_p_dims_6km[3] = 3;
	   sta_brdf_p_6km[2] = 6;	/* only band 7 */
	   sta_brdf_p_6km[3] = 0;
	   if ( (ier = getMODISarray(modinfbrdf, brdf_param, brdf_grpnm,
		sta_brdf_p_6km, brdf_p_dims_6km, brdf_p_6km[0][0][0] )) == -1)
	    HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray 6km BRDF_Albedo_Parameters");
	 }
	 else {	/* may be 5km MOD43B1C file which is not valid here */
		i_brdf = 0;
	 }
	 if ( modinfbrdf )    closeMODISfile (&modinfbrdf);
	}
     }
    }
  }

  if ( !i_brdf && brdf_3p_6km )	/* no input MOD43B1 file for this tile */
	zeroout_general ( &brdf_3p_6km[0][0][0], 200*200*3, 2);

 }

  /* open a HDF file for 1km L3 LST output */

  Vers13 = tile_pntr[i_tile_proc].s_num + 1;
  Vers_tmp = Vers13;

  if(PGS_PC_GetReference(PCLogL3_1km,&Vers_tmp,fname_1km_L3) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL3_1km, Vers13);

  rm_path(fname_1km_L3, esdt_tmp);
  sscanf ( esdt_tmp, "%7s", &esdt_1km_L3[0] );

  if ( Hishdf (fname_1km_L3) == TRUE) {

	if ((lst_1km_l3_outf = openMODISfile (fname_1km_L3, "a")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE,fname_1km_L3);

  	n_1km_l3_old_file = 1;
	if ( i_1st_loop || !i_1st_loop && n_old_1km ) {
		n_old_1km = 1;
		tile_pntr[i_tile_proc].l3_1km_exist = 1;
	}

	/* check itile_vert itile_horiz and itile_size */
  }
  else {

    n_1km_l3_old_file = 0;	n_old_1km = 0;
    tile_pntr[i_tile_proc].l3_1km_exist = 0;
  }

/* create SDS for 1km LST L3 files */

if (!n_1km_l3_old_file) {

 {
  intn		hdfReturn;
  int32		gfid, gid;

  if( (gfid = GDopen(fname_1km_L3, DFACC_CREATE)) == FAIL )
	HALT("GDopen failed for 1km L3 LST file");

  if( (gid = GDcreate(gfid, "MODIS_Grid_Daily_1km_LST", TILE_XDIM, TILE_YDIM,
		ulCornerGrid, lrCornerGrid)) == FAIL )
	HALT("GDcreate failed for 1km L3 LST file");

  if ( projType == GCTP_ISINUS ) {
     if( GDdefproj(gid, GCTP_ISINUS, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for 1km L3 LST file");
  } else {
     if( GDdefproj(gid, GCTP_SNSOID, (int32) NULL, SPHERE_CODE, projParam) )
	HALT("GDdefproj fialed for 1km L3 LST file");
  }

  if( GDdeforigin(gid, HDFE_GD_UL) == FAIL )
	HALT("GDdeforigin failed for 1km L3 LST file");

  for (i = 0; i < NSD_LST_1KM_L3; i++) {

   hdfReturn = GDdeffield(gid, lst_1km_l3[i].name, "YDim,XDim",
		lparam_1km_l3[i].type, HDFE_NOMERGE );
   if ( hdfReturn == FAIL ) {
	   sprintf (msgString, "GDdeffield failed on %s", lst_1km_l3[i].name);
	   HALT(msgString);
   }

  }

  if( GDattach(gfid, "MODIS_Grid_Daily_1km_LST") == FAIL )
	HALT("GDattach failed for 1km L3 LST file");

  if( GDdetach(gid) == FAIL )
	HALT("GDdetach failed for 1km L3 LST file");

  if( GDclose(gfid) == FAIL )
	HALT("GDclose failed for 1km L3 LST file");

 }

 if ( (lst_1km_l3_outf = openMODISfile (fname_1km_L3, "a")) == NULL )
	HALT("openMODISfile failed on 1km L3 LST file");
}

  /* initialize 1km L3 working space */

  z_siz = TILE_YDIM * TILE_XDIM;
  zeroout_general ( &em31_1km_l3_S[0][0], z_siz, 4);
  zeroout_general ( &em32_1km_l3_S[0][0], z_siz, 4);

  ii1 = TILE_YDIM;
  lkk = TILE_XDIM;

  z_siz = TILE_XDIM;
  zeroout_general ( &lst_1km_Sp[0], z_siz, 2);
  zeroout_general ( &clear_day_cov_p[0], z_siz, 2);
  zeroout_general ( &clear_nit_cov_p[0], z_siz, 2);
  zeroout_general ( &em31_1km_l3_Sp[0], z_siz, 1);
  zeroout_general ( &em32_1km_l3_Sp[0], z_siz, 1);

  for (j = 0; j < lkk; j++) {
	QC_1km_l3_p[j] = SEA_BIT;
	v_angl_1km_l3_Sp[j] = (uint8) 255;
	v_time_1km_l3_Sp[j] = (uint8) 255;
  }

  z_siz = TILE_YDIM * TILE_XDIM;
  zeroout_general ( &lst_1km_S[0][0], z_siz, 4);
  zeroout_general ( &vtime_1km_l3_S[0][0], z_siz, 4);
  zeroout_general ( &vangl_1km_l3_S[0][0], z_siz, 4);

  /* initialize 6km L3 working space */

  z_siz = ncol_6km;
  zeroout_general ( &lst_6km_S_from_1km[0][0], z_siz, 2);
  zeroout_general ( &lst_6km_Sd[0][0], z_siz, 2);
  zeroout_general ( &lst_6km_Sn[0][0], z_siz, 2);
  
  for (j = 0; j < ncol_6km; j++) {
	v_angl_6km_l3_Sv[0][j] = (uint8) 255;
	v_time_6km_l3_Sv[0][j] = (uint8) 255;
  }

  zeroout_general ( &em20_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &em22_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &em23_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &em29_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &em31_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &em32_6km_l3_Sv[0][0], z_siz, 1);
  zeroout_general ( &QC_emis[0][0], z_siz, 1);
  zeroout_general ( &pct_land[0][0], z_siz, 1);

  i = 0;
  for (j = 0; j < ncol_6km; j++) {

	QC_6km_day[i][j] = u8_3cons;
	QC_6km_nit[i][j] = u8_3cons;
  }

  if (!n_6km_l3_old_file) {

    sta_6km_o[1] = 0;
    dims_6km_o[0] = 1;
    dims_6km_o[1] = ncol_6km;

    for (j = 0; j < ncol_6km; j++) {
	v_angl_6km_l3_Sv[0][j] = (uint8) 255;
	v_time_6km_l3_Sv[0][j] = (uint8) 255;
    }

    for (i = 0; i < nrow_6km; i++) {

	{
	  int32	ii, jj, n_cld_grids, n_sea_grids;
	  int32	ii6f, ii6l, jj6f, jj6l;
	  float32 pct_land_grid;

	  ii6f = 6 * i;
	  ii6l = ii6f + 6;

	  for (j = 0; j < ncol_6km; j++) {

	    jj6f = 6 * j;
	    jj6l = jj6f + 6;
	    n_cld_grids = n_sea_grids = 0;
	    pct_land_grid = 0.0;

	    for ( ii = ii6f; ii < ii6l; ii++ ) {
		for ( jj = jj6f; jj < jj6l; jj++ ) {
			if ( land_type[ii][jj] == 19 )
				n_sea_grids += 1;
		}
	    }

	    if ( n_sea_grids > 25 ) {
		QC_6km_day[0][j] = u8_3cons;
		QC_6km_nit[0][j] = u8_3cons;
	    }
	    else {
		QC_6km_day[0][j] = u8_2cons;
		QC_6km_nit[0][j] = u8_2cons;
	    }
	    pct_land_grid = 100.0 * (36 - n_sea_grids) / 36;
	    pct_land[0][j] = pct_land_grid + 0.5;
	  }
	}

        sta_6km_o[0] = i;

	putMODISarray(lst_6km_l3_outf, lst_6km_l3[0].name, lst_6km_l3[0].gname,
		sta_6km_o, dims_6km_o, &lst_6km_Sd[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name, lst_6km_l3[1].gname,
		sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[2].name,lst_6km_l3[2].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sd[i][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[3].name,lst_6km_l3[3].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[4].name, lst_6km_l3[4].gname,
		sta_6km_o, dims_6km_o, &lst_6km_Sn[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name, lst_6km_l3[5].gname,
		sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[6].name,lst_6km_l3[6].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sn[i][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[7].name,lst_6km_l3[7].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[8].name,lst_6km_l3[8].gname,
		sta_6km_o, dims_6km_o, &em20_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[9].name,lst_6km_l3[9].gname,
		sta_6km_o, dims_6km_o, &em22_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[10].name,lst_6km_l3[10].gname,
		sta_6km_o, dims_6km_o, &em23_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[11].name,lst_6km_l3[11].gname,
		sta_6km_o, dims_6km_o, &em29_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[12].name,lst_6km_l3[12].gname,
		sta_6km_o, dims_6km_o, &em31_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[13].name,lst_6km_l3[13].gname,
		sta_6km_o, dims_6km_o, &em32_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[14].name,lst_6km_l3[14].gname,
		sta_6km_o, dims_6km_o, &lst_6km_S_from_1km[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[15].name,lst_6km_l3[15].gname,
		sta_6km_o, dims_6km_o, &lst_6km_S_from_1km[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[16].name,lst_6km_l3[16].gname,
		sta_6km_o, dims_6km_o, &QC_emis[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[17].name,lst_6km_l3[17].gname,
		sta_6km_o, dims_6km_o, &pct_land[0][0]);
    }
  }

  if ( n_old_update && !n_update_visit ) {

    if ((update_handle = openMODISfile (upd_fname, "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname);

    /* set snowcv_old with snow_bits in the update file */

     upd_sta[0] = 0;	upd_dims[0] = nrow_6km;
     ier = getMODISarray(update_handle,upd_lst_6km[0].name,
	   upd_lst_6km[0].gname, upd_sta, upd_dims, &upd_ang_lst_Sv[0][0]);
     if (ier == -1) {
	  /* HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray from upd_ang_lst"); */
  sprintf(add_message,"%s failed to getMODISarray from upd_ang_lst %2d%2d",
		gran_link.local_gran_id, tile_pntr[i_tile_proc].ih,
		tile_pntr[i_tile_proc].iv);
	  HALTsmf(MODIS_F_MAPI_FAIL, add_message);
     }
     closeMODISfile (&update_handle);

     /* set slope_6km in snow_bits
	for each grids in upd_ang_lst_W */
     for (i_6km = 0; i_6km < nrow_6km; i_6km++) {
	for (j_6km = 0; j_6km < ncol_6km; j_6km++) {
		upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];
		upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[8];
		upd_a_lst_S_svp = &upd_ang_lst_Sv[i_6km][j_6km];
		upd_lst_S_svp = &upd_a_lst_S_svp->_upd_for_lst_sv[8];
		upd_lst_w_p->snow_bits = upd_lst_S_svp->snow_bits;
	}
     }

     for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

n_stops[2] += 1;

	for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	  upd_a_lst_S_svp = &upd_ang_lst_Sv[i_6km][j_6km];

	  /* n_upd_angs is 8 in this version */
	  for ( i_upd_ang = 0; i_upd_ang < 6; i_upd_ang++ ) {

	   upd_lst_S_svp_snow = &upd_a_lst_S_svp->_upd_for_lst_sv[i_upd_ang];
	   upd_lst_S_svp_ice = &upd_a_lst_S_svp->_upd_for_lst_sv[i_upd_ang];

	   /* get the previous snowcv state for 1km grids as default */
	   /* note that the snow-cover product cut off at scan angle 45 deg */
	   snowcv_old[6*i_6km+i_upd_ang][j_6km] = upd_lst_S_svp_snow->snow_bits;

	   /* get the previous lakeice state for 1km grids as default */
	   lakeice_old[6*i_6km+i_upd_ang][j_6km] = upd_lst_S_svp_ice->lakeice_bits;
          }
	}
     }
     n_update_visit = 1;
  }

  dims[1] = n_read_lines;

  /* write 1km LST file */

  if ( !n_1km_l3_old_file ) {

    ii1 = TILE_YDIM;
    sta_1km_o[1] = 0;
    dims_1km_o[0] = 1;
    dims_1km_o[1] = TILE_XDIM;

    for (j = 0; j < TILE_XDIM; j++) {
	v_angl_1km_l3_Sp[j] = (uint8) 255;
	v_time_1km_l3_Sp[j] = (uint8) 255;
    }

    for ( i = 0; i < ii1; i++) {
      sta_1km_o[0] = i;
      if ( nit_mode ) {
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[0].name, lst_1km_l3[0].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[1].name, lst_1km_l3[1].gname,
		sta_1km_o, dims_1km_o, &qc_1km_l3d[i][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[2].name,lst_1km_l3[2].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[3].name,lst_1km_l3[3].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[8].name,lst_1km_l3[8].gname,
		sta_1km_o, dims_1km_o, &em31_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[9].name,lst_1km_l3[9].gname,
		sta_1km_o, dims_1km_o, &em32_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[10].name,lst_1km_l3[10].gname,
		sta_1km_o, dims_1km_o, &clear_day_cov[0][0]);
      }
      else {
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[4].name, lst_1km_l3[4].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[5].name, lst_1km_l3[5].gname,
		sta_1km_o, dims_1km_o, &qc_1km_l3n[i][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[6].name,lst_1km_l3[6].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[7].name,lst_1km_l3[7].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[11].name,lst_1km_l3[11].gname,
		sta_1km_o, dims_1km_o, &clear_nit_cov[0][0]);
      }
    }
  }

  /* ## modified on 5/2/98 for the DayNight BOTH mode */

  /* open a HDF file for the L2 LST output */

  Vers2 = vers_gran;
  Vers_tmp = Vers2;
  if (PGS_PC_GetReference(PCLogL2, &Vers_tmp, fname_L2) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL2, Vers2);

  if ( Hishdf (fname_L2) == TRUE ) {

     if ((lst_l2_outf = openMODISfile (fname_L2, "a")) == NULL)
	  HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_L2);

     n_l2_old_file = 1;

  }
  else {

     n_l2_old_file = 0;

     for ( i1 = 0; i1 < 7; i1++)
	lst_l2[i1].dims[0] = dim_sizes[1];

     /*** Get swath handle ***/
     if( (SWfidout = SWopen( fname_L2, DFACC_CREATE )) == FAIL )
	HALT("SWopen failed for L2 LST output file");

     /*** Link the swath handle to to toolkit. ***/
     if( EHidinfo(SWfidout, &hdfID, &LstHDFhandle) == FAIL )
	HALT("EHindinfo failure");

     /*** Create the swath ***/
     if( (SWidLST=SWcreate( SWfidout, "MOD_Swath_LST" )) == FAIL )
	HALT("SWcreate failed for the LST swath");

     /*** Define the swath dimensions. ***/
     if( SWdefdim( SWidLST, "Along_swath_lines_1km", dim_sizes[1] ) == FAIL )
	HALT("SWdefdim failed 'Along track' for LST swath");

     if( SWdefdim( SWidLST, "Cross_swath_pixels_1km", n_pix_per_line ) == FAIL )
	HALT("SWdefdim failed 'Cross track' for LST swath");

     for ( i = 0; i < (NSD_LST_L2 - 2); i++) {
	if( SWdefdatafield( SWidLST, lst_l2[i].name,
		"Along_swath_lines_1km,Cross_swath_pixels_1km",
			lparam_lst_l2[i].type, HDFE_NOMERGE ) == FAIL ) {
	   sprintf (msgString, "SWdefdatafield failed on %s", lst_l2[i].name);
	   HALT(msgString);
	}
     }

     if( SWdefdim( SWidLST, "Coarse_swath_lines_5km", lst_l2[7].dims[0]) == FAIL )
	HALT("SWdefdim failed 'Coarse_swath_lines_5km' for LST swath");

     if( SWdefdim( SWidLST, "Coarse_swath_pixels_5km", lst_l2[7].dims[1]) == FAIL )
	HALT("SWdefdim failed 'Coarse_swath_pixels_5km' for LST swath");

     /*** Define geolocation fields. ***/
     /*     added by G. Fireman 2005-09-15 */
     if (SWdefgeofield( SWidLST, lst_l2[7].name, 
			"Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			DFNT_FLOAT32, HDFE_NOMERGE) == FAIL)
       HALT("SWdefgeofield failed Latitude for LST swath");
     if (SWdefgeofield( SWidLST, lst_l2[8].name, 
			"Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			DFNT_FLOAT32, HDFE_NOMERGE) == FAIL)
       HALT("SWdefgeofield failed Longitude for LST swath");
     /* end GeoField defs */

     /*** No need to re-define latitude and longitude as data fields ***/
     /*     commented out by G. Fireman 2005-09-15 */
/*      for ( i = NSD_LST_L2 - 2; i < NSD_LST_L2; i++) { */
/* 	if( SWdefdatafield(SWidLST, lst_l2[i].name, */
/* 		"Coarse_swath_lines_5km,Coarse_swath_pixels_5km", */
/* 			lparam_lst_l2[i].type, HDFE_NOMERGE ) == FAIL ) { */
/* 	   sprintf (msgString, "SWdefdatafield failed on %s", lst_l2[i].name); */
/* 	   HALT(msgString); */
/* 	} */
/*      } */

     if( SWdetach( SWidLST ) == FAIL )
	HALT("SWdetach failed after calling SWdefgeofield");

     if( (SWidLST=SWattach( SWfidout, "MOD_Swath_LST" )) == FAIL )
	HALT("SWattach failed to re-attach swath after calling SWdefgeofield");

     if( SWdefdimmap( SWidLST, "Coarse_swath_lines_5km",
		"Along_swath_lines_1km", 2, 5) == FAIL )
	HALT("SWdefdimmap failed for Coarse_swath_lines_5km");

     if( SWdefdimmap(SWidLST,"Coarse_swath_pixels_5km",
		"Cross_swath_pixels_1km", 2, 5)== FAIL)
	HALT("SWdefdimmap failed for Coarse_swath_pixels_5km");

     if( SWdetach( SWidLST ) == FAIL )
	HALT("SWdetach failed for LST swath");

     if( SWclose( SWfidout ) == FAIL )
	HALT("SWclose failed for LST swath");

     if ((lst_l2_outf = openMODISfile (fname_L2, "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, fname_L2);

     ier = putMODISarray(lst_l2_outf, lst_l2[7].name, lst_l2[7].gname,
                sta_o_lat, lst_l2[7].dims, &lat_5pix[0][0]);
     if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Latitude to L2");

     ier = putMODISarray(lst_l2_outf, lst_l2[8].name, lst_l2[8].gname,
		sta_o_lat, lst_l2[8].dims, &lon_5pix[0][0]);
     if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Longitude to L2");

     free (lat_5pix[0]); free (lat_5pix);	lat_5pix = NULL;
     free (lon_5pix[0]); free (lon_5pix);	lon_5pix = NULL;

     /* write L2 LST output file with filled values */
     dims_o[0] = n_read_lines;

     for (i = 0; i < nn_lines; i++) {
	lst_o_p = lst_out[i];
	err_lst_p = err_lst_out[i];
	e31_o_p = emis31_out[i];
	e32_o_p = emis32_out[i];
	vangl_o_p = view_angl_out[i];
	vtime_o_p = view_time_out[i];

	for (j = 0; j < n_pix_per_line; j++) {
		lst_o_p[j] = 0;
		err_lst_p[j] = 0;
		e31_o_p[j] = 0;
		e32_o_p[j] = 0;
		vangl_o_p[j] = (uint8) 255;
		vtime_o_p[j] = (uint8) 255;
	}
     }

     for (kk = 0; kk < n_read_times; kk++) {

      sta_o[0] = kk * n_read_lines;

      for (i = 0; i < n_read_lines; i++) {

	QC_o_p = QC_out[i];
	i_scan_line = i + sta_o[0];
	i_ch3 = 0;
	if ( (i_scan_line % 10) == 3 )      i_ch3 = 1;

	for (j = 0; j < n_pix_per_line; j++) {

	   QC_o_p[j] = u16_3cons;

	   i_cloudy_pix = cldmask[i_scan_line][j] & 01;

	   if ( i_cloudy_pix )
			QC_o_p[j] = u16_2cons;
	}
      }

      ier = putMODISarray(lst_l2_outf, lst_l2[0].name, lst_l2[0].gname,
		sta_o,dims_o, &lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		sta_o,dims_o, &QC_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray QC to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[2].name, lst_l2[2].gname,
		sta_o,dims_o, &err_lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray Error_LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[3].name, lst_l2[3].gname,
		sta_o,dims_o, &emis31_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray Emis_31 to L2");
    
      ier = putMODISarray(lst_l2_outf, lst_l2[4].name, lst_l2[4].gname,
		sta_o,dims_o, &emis32_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray Emis_32 to L2");
    
      ier = putMODISarray(lst_l2_outf, lst_l2[5].name, lst_l2[5].gname,
		sta_o,dims_o, &view_angl_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray View_angle to L2");
    
      ier = putMODISarray(lst_l2_outf, lst_l2[6].name, lst_l2[6].gname,
		sta_o,dims_o, &view_time_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray View_time to L2");
     }
  }

  for (kk = tile_pntr[i_tile_proc].k_scan_start;
		kk <= tile_pntr[i_tile_proc].k_scan_end; kk++)
  {

    if ( i_cld_scan[kk] )	continue;

    if ( nit_mode && i_nit_scan[kk] != 1 )	continue;
    if ( !nit_mode && i_day_scan[kk] != 1 )	continue;

    sta[1] = kk * n_read_lines;

    sta_o[0] = kk * n_read_lines;
    dims_o[0] = n_read_lines;

    /* input L1A data */

    sta_geo2[0] = kk * n_read_lines;

    for (i = 0; i < 3; i++) {
      if((ier = getMODISarray(modinf1a,geo_1a[i].name,geo_1a[i].gname,sta_geo2,
  		geo_1a[i].dims, &dat1_1a[i][0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray height sensor/solarZenith");
    }
    if ( !nit_mode ) {
     for (i = 7; i < 9; i++) {
      if((ier = getMODISarray(modinf1a,geo_1a[i].name,geo_1a[i].gname,sta_geo2,
		geo_1a[i].dims, &dat1_1a[i-4][0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray Sensor/SolarAzimuth");
     }
    }

  /* input L1B data */

  for (i = 0; i < 7; i++) {

    if ( !i_subset_1b ) {
	sta[0] = k1_data[i];
        ier = getMODISarray (modinf1b, arrnm, grpnm, sta, dims, &data[i][0][0]);
        ier_unc = getMODISarray (modinf1b, arrnm_unc, grpnm, sta, dims,
				&emissive_uncert_indexes[i][0][0]);
    }
    else {
	sta[0] = k1_data_subset[i];
        ier =getMODISarray(modinf1b,arrnm_subset,grpnm,sta,dims,&data[i][0][0]);
        ier_unc = getMODISarray(modinf1b,arrnm_subset_unc,grpnm,sta,dims,
				&emissive_uncert_indexes[i][0][0]);
    }
    if (ier == -1)
      HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray EV_1KM_Emissive");

    if (ier_unc == -1)
      HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray EV_1KM_Emissive_Uncert_Indexes");

  }

  /* input snow-cover data */

  if ( modinfsnow ) {
    sta_snow[0] = 2 * kk * n_read_lines;
/*next block of code modified by c. collins on 11/11/04 so either old snow
cover or new snow cover could be processed*/
/*next block modified by Z. Wan on 5/11/05
   if ( snow_cv_use == 1 ) {
    if((ier = getMODISarray (modinfsnow, snow_cv1[0].name, snow_cv1[0].gname,
		sta_snow, snow_cv1[0].dims, &snowcv[0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray snowcv");
   }
   if ( snow_cv_use == 2 ) {
    if((ier = getMODISarray (modinfsnow, snow_cv2[0].name, snow_cv2[0].gname,
		sta_snow, snow_cv2[0].dims, &snowcv[0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray snowcv");
   }
*/
   if((ier = getMODISarray (modinfsnow, arrnm_snow, snow_cv[0].gname,
		sta_snow, snow_cv[0].dims, &snowcv[0][0])) == -1)
	HALTsmf( MODIS_F_MAPI_FAIL, "getMODISarray snowcv");

    /* averaging for 1km pixels in the granule */
    {
	int32	i_snowcv, i_lakeice;
	int32	ii, jj;

	for (i = 0; i < n_read_lines; i++) {
	    ii = 2 * i;
	    for (j = 0; j < n_pix_per_line; j++) {
	      if ( sensr_zen[i][j] <= 4500 ) {
		jj = 2 * j;
		i_snowcv = 0;	i_lakeice = 0;
		if ( snowcv[ii][jj] == 200 )
			i_snowcv ++;	/* snow */
		if ( snowcv[ii][jj+1] == 200 )
			i_snowcv ++;
		if ( snowcv[ii+1][jj] == 200 )
			i_snowcv ++;
		if ( snowcv[ii+1][jj+1] == 200 )
			i_snowcv ++;
		if ( snowcv[ii][jj] == 100 )
			i_lakeice ++;	/* lake ice */
		if ( snowcv[ii][jj+1] == 100 )
			i_lakeice ++;
		if ( snowcv[ii+1][jj] == 100 )
			i_lakeice ++;
		if ( snowcv[ii+1][jj+1] == 100 )
			i_lakeice ++;
		if ( i_snowcv >= 3 )	snowcv[i][j] = 1;
		else			snowcv[i][j] = 0;
		if ( i_lakeice >= 3 )	lakeice[i][j] = 1;
		else			lakeice[i][j] = 0;
	      }
	      else {
		snowcv[i][j] = 0;	/* by Z. Wan, 13 March 1998 */
		lakeice[i][j] = 0;
	      }
	    }
	}
    }
  }

  /* read old L2 file */

  if ( n_l2_old_file ) {

      ier = getMODISarray(lst_l2_outf, lst_l2[0].name, lst_l2[0].gname,
		sta_o,dims_o, &lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray LST from L2");

      ier = getMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		sta_o,dims_o, &QC_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray QC from L2");

      ier = getMODISarray(lst_l2_outf, lst_l2[2].name, lst_l2[2].gname,
		sta_o,dims_o, &err_lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray Error_LST from L2");

      ier = getMODISarray(lst_l2_outf, lst_l2[3].name, lst_l2[3].gname,
		sta_o,dims_o, &emis31_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray Emis_31 from L2");
    
      ier = getMODISarray(lst_l2_outf, lst_l2[4].name, lst_l2[4].gname,
		sta_o,dims_o, &emis32_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray Emis_32 from L2");
    
      ier = getMODISarray(lst_l2_outf, lst_l2[5].name, lst_l2[5].gname,
		sta_o,dims_o, &view_angl_out[0][0]);

      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray View_angle from L2");
    
      ier = getMODISarray(lst_l2_outf, lst_l2[6].name, lst_l2[6].gname,
		sta_o,dims_o, &view_time_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL,"getMODISarray View_time from L2");
  }

  /* reinitialization */

  if ( !n_l2_old_file && multi_write_out ) {

      for (i = 0; i < n_read_lines; i++) {

	i_scan_line = i + sta[1];
	i_ch3 = 0;
	if ( (i_scan_line % 10) == 3 )      i_ch3 = 1;
	lst_o_p = lst_out[i];
	QC_o_p = QC_out[i];
	err_lst_p = err_lst_out[i];
	e31_o_p = emis31_out[i];
	e32_o_p = emis32_out[i];
	vangl_o_p = view_angl_out[i];
	vtime_o_p = view_time_out[i];

	for (j = 0; j < n_pix_per_line; j++) {

	  lst_o_p[j] = 0;
	  QC_o_p[j] = u16_3cons;

	  i_cloudy_pix = cldmask[i_scan_line][j] & 01;
	  if ( i_cloudy_pix )		QC_o_p[j] = u16_2cons;

	  err_lst_p[j] = 0;
	  e31_o_p[j] = 0;
	  e32_o_p[j] = 0;
	  vangl_o_p[j] = (uint8) 255;
	  vtime_o_p[j] = (uint8) 255;
	}
      }
  }

  for (i = 0; i < n_read_lines; i++) {
      
    i_scan_line = i + sta[1];

    i_ch3 = 0;
    if ( (i_scan_line % 10) == 3 )	i_ch3 = 1;

    ii_scan_b = i_scan_line / 10;
    i_scan_prof = i_scan_line / 5;

    nn = multi_write_out ? i : sta_o[0] + i;
    lst_o_p = lst_out[nn];
    QC_o_p = QC_out[nn];
    err_lst_p = err_lst_out[nn];
    e31_o_p = emis31_out[nn];
    e32_o_p = emis32_out[nn];
    vangl_o_p = view_angl_out[nn];
    vtime_o_p = view_time_out[nn];

    for (j = 0; j < n_pix_per_line; j++) {

	/* skip the pixels affected in lunar calibration or deep space maneuver */
	if ( sensr_zen[i][j] > 7500 )	continue;

	view_sign = 1.0;	/* from west */

	if ( j < 677 ) {
		if ( i_fm1 && nit_mode || !i_fm1 && !nit_mode )
			view_sign = -1.0;
	}
	else {
		if ( i_fm1 && !nit_mode || !i_fm1 && nit_mode )
			view_sign = -1.0;
	}
	u8_1 = land_sea_mask_1a[i_scan_line][j];

	/* skip ocean pixels first */
	if ( u8_1 == u8_0cons || u8_1 >= u8_6cons ) {
		QC_o_p[j] = u16_3cons;
		continue;
	}

	i_lake_pix = 0;
	if ( u8_1 > u8_1cons && u8_1 < u8_6cons &&
	     sf_height[i][j] > 5 )	i_lake_pix = 1;

	seconds_tmp = seconds_start + EV_TIME( ev_start_time[ii_scan_b], j)
		      - pix_zero_time;

	pix_time = seconds_tmp / 3600.;

	/* Z. Wan, lon has been converted to radian in main()
	   15 degrees equal to 0.26179939 radian */

	pix_time += lon[i_scan_line][j] / 0.26179939;
	if ( pix_time > 24. )	pix_time -= 24.0;
	if ( pix_time < 0. )	pix_time += 24.0;
	continu_time = pix_time;
	if ( pix_time >= 6. && pix_time < 18.0 ) {
		if ( nit_mode )		continue;
	}
	else {
		if ( !nit_mode)		continue;
		if ( pix_time < 6.0 )	continu_time += 24.0;
	}

	i_cloudy_pix = cldmask[i_scan_line][j] & 01;

	/* skip cloudy pixels */
	if ( i_cloudy_pix ) {
		QC_o_p[j] = u16_2cons;
		continue;
	}

	/* convert to radiance and btemp for bands 20, 22, 23, 29, 31-33 */

	ipx_bad_split_window = 0;
	ipx_bad_day_nit = 0;
	ipx_other_qual_sw = 0;
	ipx_other_qual_dn = 0;

	for (k = 0; k < 7; k++) {
	    k_band = k2_cali[k];
	    data_p = data[k];
	    uncert_p = emissive_uncert_indexes[k];
	    band_data = data_p[i][j];

	    if ( band_data > 32767 ) {
		/* met_cnt_L2.QA_MissingData++;		Wan 7/18/2000 */
		ipx_bad_day_nit = 1;
		npx_bad_day_nit++;
		if ( k == 4 || k == 5 ) {

		     if ( band_data == 65535 )
			l1b_qa[i_scan_line][j] = u8_1cons; /* MissingData */
		     else
			l1b_qa[i_scan_line][j] = u8_2cons; /* OutofBounds */

		     ipx_bad_split_window = 1;
		     npx_bad_split_window++;
		     break;
		}
	    }
	    if ( !i_ignore_uncert ) {

	      band_uncert = uncert_p[i][j];

	      /* in uncert range of 0.82-1.74% */
	      if ( band_uncert >= 2 && band_uncert < 6 ) {
		ipx_other_qual_dn = 1;
		if ( k == 4 || k == 5 ) {
		  if ( band_uncert > ipx_other_qual_sw )
			ipx_other_qual_sw = band_uncert;
		}
	      }
	      if ( band_uncert >= 6 ) {		/* uncertainty >= 2.24% */
		ipx_bad_day_nit = 1;
		npx_bad_day_nit++;
	      }
	      if ( band_uncert >= 6 && ( k == 4 || k == 5 ) ) {
			ipx_bad_split_window = 1;
			npx_bad_split_window++;
			if ( band_uncert > max_EV_1KM_Emissive_Uncert_Indexes )
			   max_EV_1KM_Emissive_Uncert_Indexes = band_uncert;
			break;
	      }
	    }

	    if ( i_new_l1b )
		radiance[k] = scales[k_band] * (band_data - offsets[k_band]);
	    else
		radiance[k] = scales[k_band] * band_data - offsets[k_band];

	    if ( radiance[k] <= 0.0 ) {
		ipx_bad_day_nit = 1;
		npx_bad_day_nit++;
		if ( k == 4 || k == 5 ) {
			ipx_bad_split_window = 1;
			npx_bad_split_window++;
			break;
		}
	    }
	    else if ( k == 4 || k == 5 )
			btemp[k] = f_mod_rad7_inv(k, radiance[k], i_fm1);
	}

	if ( ipx_bad_split_window ) {
		QC_o_p[j] = u16_3cons;
		u16setbits ( &QC_o_p[j], 3, 2, u16_3cons );
		continue;
	}
	/* Z. Wan, 06/01/2001
	if ( ipx_other_qual_sw && u16getbits(QC_o_p[j],1,2) != u16_0cons )
	*/
	/* Z. Wan, 11/26/2002
	if ( ipx_other_qual_sw )
		QC_o_p[j] = u16_1cons; */

	for (k = 0; k <= N_LANDCOV_TYPES; k++)
		n_pix_land_type[k] = 0.0;

	n_pix_land_type_sum = 0.0;
	obs_cov_sum = 0.0;

	/* mapping from pixel's geolocation to grid coordinates */

	lat_pix = lat[i_scan_line][j];	lon_pix = lon[i_scan_line][j];

	/* skip those pixels with Fill value for lat or lon - in radians here */
	if ( lat_pix < - PIO2 || lon_pix < - PI )	continue;

	near_tile_pix = 0;

	returnStatus = (*pLbin_FwdW)(lon_pix, lat_pix, &irow_pix_o, &icol_pix_o,
		&irow_pix, &icol_pix, &rrr_pix, &ccc_pix);

	if ( returnStatus == LBIN_NORMAL )
	   near_tile_pix = 1;

	if ( !near_tile_pix )	continue;

n_stops[4] += 1;

	j_scan_prof = j / 5;
	if ( j_scan_prof >= n_cross_prof ) j_scan_prof = n_cross_prof - 1;
	ts_prof_v = ts_prof[i_scan_prof][j_scan_prof];

	lon_p = &lon[sta[1]];
	lat_p = &lat[sta[1]];

	GeoCor_lst(i, j, n_lines_per_scub, n_pix_per_line, lon_p, lat_p, p_cor);
	cen[0] = lon_pix;
	cen[1] = lat_pix;

	(*pGenObs)( cen, p_cor, &nobs_1km, obs_1km );

	if ( !nobs_1km ) {
		if ( lst_o_p[j] )	continue;
		QC_o_p[j] = u16_3cons;
		continue;
	}

	u16_qa_l2_man = u16_0cons;	u16_qa_l2_dat = u16_0cons;
	u16_qa_l2_cld = u16_0cons;	u16_qa_l2_emer = u16_0cons;

	nobs_1km_tot += nobs_1km;

	n_pix_near_tile++;

	in_tile_pix = (irow_pix_o >= 0 && irow_pix_o < set->nrow_tile &&
	  icol_pix_o >= 0 && icol_pix_o < set->row[irow_pix].ncol_tile) ? 1 : 0;

	if ( in_tile_pix )	n_pix_in_tile++;

        /* to determine the landcover type for the pixel */

	cell_snow_sum = 0.0;
	cell_ice_sum = 0.0;

	i_snow_input = ( !nit_mode && modinfsnow &&
	      sensr_zen[i][j] <= 4500 && solar_zen[i][j] < 9000 ) ? 1 : 0;

	/* i_water_pix = ( u8_1 >= u8_2cons && u8_1 <= u8_5cons ) ? 1 : 0;*/

	for (iobs_1km = 0;  iobs_1km < nobs_1km; iobs_1km++) {

	  i_1km = obs_1km[iobs_1km].irow;
	  j_1km = obs_1km[iobs_1km].ipix;
	  cell_cov = obs_1km[iobs_1km].cellcov;
	  obs_cov = obs_1km[iobs_1km].obscov;

	  i_grid_land_type = (int32) land_type[i_1km][j_1km];

	  /* if ( i_water_pix )	i_grid_land_type = 1;	*/

	  /* update the snowcv state in the lst_update file */
	  if ( cell_cov > 0.66 ) {
	    i_6km = i_1km / 6;
	    j_6km = j_1km / 6;
	    i_1km -= 6 * i_6km;
	    j_1km -= 6 * j_6km;

	    if ( i_snow_input ) {
	      upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];
	      upd_lst_w_p_snow = &upd_a_lst_w_p->_upd_for_lst[i_1km];
	      upd_lst_w_p_ice = &upd_a_lst_w_p->_upd_for_lst[i_1km];
	      if ( snowcv[i][j] ) {
		SET_BIT(&upd_lst_w_p_snow->snow_bits, j_1km);
		i_grid_land_type = 15;
		cell_snow_sum += cell_cov;
	      }
	      else
		OFF_BIT(&upd_lst_w_p_snow->snow_bits, j_1km);

	      if ( lakeice[i][j] ) {
		SET_BIT(&upd_lst_w_p_ice->lakeice_bits, j_1km);
		i_grid_land_type = 15;
		cell_ice_sum += cell_cov;
	      }
	      else
		OFF_BIT(&upd_lst_w_p_ice->lakeice_bits, j_1km);
	    }
	    else {	/* use snowcv_old */
	      u8_1 = snowcv_old[6*i_6km + i_1km][j_6km];
	      if ( CHK_BIT(u8_1, j_1km) ) {
		i_grid_land_type = 15;
		cell_snow_sum += cell_cov;
	      }
	      u8_1 = lakeice_old[6*i_6km + i_1km][j_6km];
	      if ( CHK_BIT(u8_1, j_1km) ) {
		i_grid_land_type = 15;
		cell_ice_sum += cell_cov;
	      }
	    }
	  }

	  if ( i_grid_land_type <= N_LANDCOV_TYPES ) {
	  	n_pix_land_type[i_grid_land_type] += cell_cov;
	  	n_pix_land_type_sum += cell_cov;
	  }
	  obs_cov_sum += obs_cov;

	}

	if (n_pix_land_type_sum < SMALL_VALUE) {  /* by the tile edge */

	   /* skip without change QC if LST is already retrieved.  Z. Wan */
	   if ( lst_o_p[j] )	continue;
	   QC_o_p[j] = u16_3cons;
	   continue;
	}

n_stops[5] += 1;

	if ( !i_snow_input ) {
	     if ( (cell_snow_sum / n_pix_land_type_sum) > 0.66 )
		snowcv[i][j] = 1;
	     if ( (cell_ice_sum / n_pix_land_type_sum) > 0.66 )
		lakeice[i][j] = 1;
	}

	pct_sea_pix = n_pix_land_type[19] / n_pix_land_type_sum;

	if ( pct_sea_pix > 0.8 ) {
		QC_o_p[j] = u16_3cons;
		continue;
	}

n_stops[6] += 1;

	/* to determine id_coef_tbl according to
	   tair_srf and cwv values of the pixel */

	tair_srf = tair_zones[i_scan_line/100];
	cwv = cwv_zones[i_scan_line/100];

	if ( !(tair_srf >= 150.) )	tair_srf = tair_zones[n_zones];
	if ( !(cwv > 0.05) )		cwv = cwv_zones[n_zones];

	/* use the values in mod07 profile for tair_srf & cwv */
	if ( ts_prof_v >= 0 && ts_prof_v <= 20000 )
	     tair_srf = 0.01 * (ts_prof_v + 15000);

	/* use the IR retrieved Column Precipitable Water Vapor */
	if ( cwv_prof[i_scan_prof][j_scan_prof] > 0 )
	 cwv = 0.001 * cwv_prof[i_scan_prof][j_scan_prof];

	tair_s = tair_srf;
	cwv1 = cwv;

	if ( tair_srf > _tair_tbl[6] )  tair_srf = _tair_tbl[6];
	if ( cwv > _wv_tbl[10] )	cwv = _wv_tbl[10];

	id_tair = 0;

	for (k = 0; k < 7; k++) {

	  if (tair_srf > _tair_tbl[k]) {
		id_tair = k;

		if (k <= 5) {
			tmp = tair_srf - _tair_tbl[k];
			dt = _tair_tbl[k+1] - tair_srf;
			if (dt < tmp) id_tair++;
		}

	  }
	}
	
	id_cwv = 0;

	for (k = 0; k < 11; k++) {

	  if (cwv > _wv_tbl[k]) {

	    id_cwv = k;
	  
	    if (k <= 9) {
	      tmp = cwv - _wv_tbl[k];
	      dt = _wv_tbl[k+1] - cwv;
	    
	      if (dt < tmp)	
		id_cwv++;
	    }
	  }
	}

	id_coef_tbl = pointer_coef_tbl[id_tair][id_cwv];
	
	if (id_coef_tbl < 0)
	  id_coef_tbl = 2;
	
	/* the main loop for the generalized split-window LST algorithm */
	
	for (i_iterate = 0; i_iterate < n_iterate; i_iterate++) {
	  
	  if (i_iterate == 0)	
	    id_ts_tair = 0;
	  else if ( i_iterate == 2 && lst_ret < btemp[k1_split] )
			break;
	  
	  if (i_iterate == 1) {
	    tmp = lst_ret - tair_srf;
	    if (tmp > 0.)	
	      id_ts_tair = 1;
	    else
	      id_ts_tair = 2;
	  }

	  if (i_iterate == 2) {
	    tmp = lst_ret - tair_srf;

	    id_ts_tair = 6;
	    if (tmp > -7.) id_ts_tair = 5;
	    if (tmp > 0.)  id_ts_tair = 4;
            if (tmp > 7.)  id_ts_tair = 3;	

	  }

	  for (k = 0; k < 9; k++) {
	    gsw_lst_coef_p = &gsw_lst_coef_tbl[id_coef_tbl][k][id_ts_tair];
	
	    coef_mesh[0][k] = gsw_lst_coef_p->c;
	    coef_mesh[1][k] = gsw_lst_coef_p->a1;
	    coef_mesh[2][k] = gsw_lst_coef_p->a2;
	    coef_mesh[3][k] = gsw_lst_coef_p->a3;
	    coef_mesh[4][k] = gsw_lst_coef_p->b1;
	    coef_mesh[5][k] = gsw_lst_coef_p->b2;
	    coef_mesh[6][k] = gsw_lst_coef_p->b3;
	    coef_mesh[7][k] = gsw_lst_coef_p->rms_algorithm;
	  }

	  k_emis = 0;

	  /* k_emis and e31 & e32 updated by land-cover & snow cover data */
	  i_class_state = 0;
	  if ( lakeice[i][j] )	i_class_state = 1;

	  if (i_iterate == 0) {

	    e31 = e32 = 0.0;
	    rms_m_em = rms_d_em = 0.0;
	    temp_d = 0.01 * sensr_zen[i][j];
	    tmp = DTR(temp_d) - DTR(42.3);

	    /* for snow pixels */
	    if ( snowcv[i][j] || lakeice[i][j] ) {
		k_emis = _landcov_to_emisclass[15][i_class_state];
		tmp_em31 = mod_band_emis_db[k_emis].e31;
		tmp_em32 = mod_band_emis_db[k_emis].e32;
		if (tmp > 0.0) {
			tmp_em31 += mod_band_emis_db[k_emis].ang_e31 * tmp;
			tmp_em32 += mod_band_emis_db[k_emis].ang_e32 * tmp;
		}
		e31 = tmp_em31;
		e32 = tmp_em32;
		rms_m_em = mod_band_emis_db[k_emis].rms_m_em;
		rms_d_em = mod_band_emis_db[k_emis].rms_d_em;
	    }
	    else {

	      for ( i_class = 0; i_class <= N_LANDCOV_TYPES; i_class++) {
		k_emis = _landcov_to_emisclass[i_class][i_class_state];
		tmp_em31 = mod_band_emis_db[k_emis].e31;
		tmp_em32 = mod_band_emis_db[k_emis].e32;
		if (tmp > 0.0) {
			tmp_em31 += mod_band_emis_db[k_emis].ang_e31 * tmp;
			tmp_em32 += mod_band_emis_db[k_emis].ang_e32 * tmp;
		}
		e31 += n_pix_land_type[i_class] * tmp_em31;
		e32 += n_pix_land_type[i_class] * tmp_em32;
		rms_m_em += n_pix_land_type[i_class] *
				mod_band_emis_db[k_emis].rms_m_em;
		rms_d_em += n_pix_land_type[i_class] *
				mod_band_emis_db[k_emis].rms_d_em;
	      }
	      e31 /= n_pix_land_type_sum;
	      e32 /= n_pix_land_type_sum;
	      rms_m_em /= n_pix_land_type_sum;
	      rms_d_em /= n_pix_land_type_sum;
	    }

	    emis_m = 0.5 * (e31 + e32);
	    emis_d = (e31 - e32) / (emis_m * emis_m);
	    tmp = (1. - emis_m) / emis_m;
	    emis_m = tmp;
	    tmp = DTR(temp_d);
	    mu_pix = cos(tmp);
	    j_pix = 0;

	    if (mu_pix >= mu_mesh[0]) j_pix = 0;
	    if (mu_pix >= mu_mesh[1]) j_pix = 1;
	    if (mu_pix >= mu_mesh[2]) j_pix = 2;
	    if (mu_pix >= mu_mesh[3]) j_pix = 3;
	    if (mu_pix >= mu_mesh[4]) j_pix = 4;
	    if (mu_pix >= mu_mesh[5]) j_pix = 5;
	    if (mu_pix >= mu_mesh[6]) j_pix = 6;
	  }
	  
	  poly_intp_vector (mu_mesh,j_pix, mu_pix, 8, coef_mesh, coeff_p);

	  temp_m = 0.5 * (btemp[k1_split] + btemp[k2_split]);
	  temp_d = 0.5 * (btemp[k1_split] - btemp[k2_split]);
	  d1 = coeff_p[2] * temp_m;
	  d2 = coeff_p[3] * temp_m;
	  d3 = coeff_p[5] * temp_d;
	  d4 = coeff_p[6] * temp_d;
	  lst_ret = coeff_p[1] * temp_m + d1 * emis_m +
	            d2 * emis_d + coeff_p[4] * temp_d +
		    d3 * emis_m + d4 * emis_d + coeff_p[0];

	  /* to correct the effect of calibration errors in bands 31 and 32 */
	  {
		float32	temp_d_dt, d3_dt, d4_dt, lst_ret_dt_d;
		float32	temp_a_dt, d1_dt, d2_dt, cos_i, tmp1, tmp2;

		if ( i_calib_correction ) {

			/* updated by Z. Wan 2001/01/08
			   temp_d_dt = 0.5 * (b31_dt - b32_dt); 2001/01/08 */
			tmp1 = 0.01 * sensr_zen[i][j];
			tmp2 = DTR(tmp1);
			tmp1 = cosf(tmp2);
			cos_i = 1.0 / tmp1;
			temp_a_dt = dt_avg_b31_b32_a + cos_i * dt_avg_b31_b32_b;
			temp_d_dt = dt_dif_b31_b32_a + cos_i * dt_dif_b31_b32_b;
			d1_dt = coeff_p[2] * temp_a_dt;
			d2_dt = coeff_p[3] * temp_a_dt;
			d3_dt = coeff_p[5] * temp_d_dt;
			d4_dt = coeff_p[6] * temp_d_dt;
			lst_ret_dt_d = coeff_p[4] * temp_d_dt
				+ d3_dt * emis_m + d4_dt * emis_d;

			lst_ret_dt_d += coeff_p[1] * temp_a_dt + d1_dt * emis_m
				+ d2_dt * emis_d;

			lst_ret -= lst_ret_dt_d;
		}
	  }
	}

	d5 = (d1 + d3) * rms_m_em;
	d6 = (d2 + d4) * rms_d_em;

	dt_inst = ipx_other_qual_sw * 0.6;
	dt = coeff_p[7] * coeff_p[7] + d5 * d5 + d6 * d6 + dt_inst * dt_inst;

	dt_quant = 0.0;

	for ( k = 0; k < 6; k++ )
	    if ( temp_m < quantization_temp[k] )
		dt_quant = quantization_extra_err_on_lst[i_fm1][k];

	dt += dt_quant * dt_quant;

	near_cld = (cldmask[i_scan_line][j] & NEAR_CLD) ? 01 : 0;
	far_cld = (cldmask[i_scan_line][j] & FAR_CLD) ? 01 : 0;

	if ( near_cld || j < j_scan_start_pix )	dt += 2.25;

	if ( far_cld )	dt += 0.5625;

	if ( ipx_other_qual_sw ) {
		u16_qa_l2_dat = u16_2cons;
		u16setbits ( &QC_o_p[j], 3, 2, u16_2cons );
		dt += 0.7;
	}

	dt = sqrt (dt);

	if ( dt > 1.0 )       u16_qa_l2_man = u16_1cons;

	lst_o_p[j] = lst_ret * Scale_factor_offset[0][0] + 0.5;
	err_lst_p[j] = dt * Scale_factor_offset[0][1] + 0.5;
	e31_o_p[j] = e31 * Scale_factor_offset[0][2] +
			   Scale_factor_offset[1][2] + 0.5;
	e32_o_p[j] = e32 * Scale_factor_offset[0][2] +
			   Scale_factor_offset[1][2] + 0.5;
	pix_zen_deg = 0.01 * sensr_zen[i][j];
	vangl_o_p[j] = pix_zen_deg / lst_l2[5].scale + 0.5; /* in unit 0.5D */
	pix_sol_deg = 0.01 * solar_zen[i][j] / lst_l2[5].scale + 0.5;
	if ( pix_sol_deg > 255 )	pix_sol_deg = 255;

	if ( rms_m_em <= 0.01 )		u16 = 0;
	else if ( rms_m_em <= 0.02)	u16 = 1;
	else if ( rms_m_em <= 0.04 )	u16 = 2;
	else				u16 = 3;
	u8_1 = u16;
	u16_qa_l2_emer = u16;
	u16setbits ( &QC_o_p[j], 15, 2, u16_qa_l2_emer );

	if (  near_cld || far_cld ) {
		u16_qa_l2_cld = u16_3cons;
		u16setbits ( &QC_o_p[j], 5, 2, u16_3cons );
	}
	/*if ( (u16_qa_l2_dat > u16_1cons) || (u16_qa_l2_cld > u16_1cons) ||
	     (u16_qa_l2_emer > u16_0cons) )	u16_qa_l2_man = u16_1cons;*/

	u16setbits ( &QC_o_p[j], 1, 2, u16_qa_l2_man);

	u8_2 = 0;
	if ( dt > 1.0 && dt <= 2.0 )		u8_2 = 1;
	else if ( dt > 2.0 && dt <= 3.0 )	u8_2 = 2;
	     else if ( dt > 3.0 )		u8_2 = 3;

	vtime_o_p[j] = pix_time / lst_l2[6].scale + 0.5; /* unit 0.1 hr */
	i_time_pix = 0.5 * vtime_o_p[j] + 0.5;		 /* unit 0.2 hr */

	n_pix_lst_done += 1;

	/* processing for 1km L3 and 6km upd_for_lst cells */

	for (iobs_1km = 0;  iobs_1km < nobs_1km; iobs_1km++) {

	    i_1km = obs_1km[iobs_1km].irow;
	    j_1km = obs_1km[iobs_1km].ipix;
	    cell_cov = obs_1km[iobs_1km].cellcov;

	    /* skip ocean grids, Z. Wan, 23 Jan 2002 */
	    if ( land_type[i_1km][j_1km] == 19 ) continue;

	  if ( !ipx_bad_day_nit &&
		( !is_a_side_data || is_a_side_data && !i_ch3 ) ) {

	    i_6km = i_1km / 6;
	    j_6km = j_1km / 6;
	    upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];

	    upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[8];
	    indx_6km_6d = upd_lst_w_p->snow_bits;

	    view_ang_6km_upd = 2.0 * pix_zen_deg + 0.5;	/* unit of 0.5 deg */

	    if ( view_ang_6km_upd > _upd_ang_range_u2[N_VIEW_ANGS-1] ) {
		view_ang_6km_upd = _upd_ang_range_u2[N_VIEW_ANGS-1];
		i_upd_a_o = N_VIEW_ANGS - 1;
	    }
	    else {
		for ( k = 0; k < n_upd_angs; k++ ) {
			if ( view_ang_6km_upd <= _upd_ang_range_u2[k] ) {
				i_upd_a_o = k;
				break;
			}
		}
	    }
	    if ( view_sign < 0 || ! view_ang_6km_upd )
	    		i_upd_ang = abs (i_upd_a_o - 7);
	    else	i_upd_ang = i_upd_a_o + 8;

	    upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[i_upd_ang];
	    upd_cov_d_p = &upd_cov_d[i_6km][j_6km][i_upd_ang];
	    upd_cov_n_p = &upd_cov_n[i_6km][j_6km][i_upd_ang];
	    sf_elev_6km_d_p = &sf_elev_6km_d[i_6km][j_6km];
	    sf_elev_6km_n_p = &sf_elev_6km_n[i_6km][j_6km];

	    if ( nit_mode ) {

	      u8setbits(&upd_lst_w_p->i_data_yr_dn, 3, 4, i_data_yr);

	      if ( pix_sol_deg > 180 && btemp[4] >= 195.0 ) {

		*upd_cov_n_p += cell_cov;
		upd_lst_w_p->view_zenith_nit = (uint8) view_ang_6km_upd;
		v_angl_6km_l3_Sn[i_6km][j_6km] = view_ang_6km_upd;
		v_time_6km_l3_Sn[i_6km][j_6km] = i_time_pix;
		if ( ipx_other_qual_dn )
			u8setbits(&upd_lst_w_p->QA_bits, 3, 4, u8_1cons);
		for (k = 0; k < 7; k++)
		    upd_lst_w_p->tb_nit[k] += radiance[k] * cell_cov;
		upd_lst_w_p->tb_nit[7] += e31*bt_to_radiance(4,lst_ret,i_fm1) * cell_cov;
		em31_6km_n[i_6km][j_6km] += e31 * cell_cov;
		upd_lst_w_p->cwv_nit += cwv1 * cell_cov;
		*sf_elev_6km_n_p += sf_height[i][j] * cell_cov;

		if ( near_cld || j < j_scan_start_pix )
			upd_lst_w_p->nit_status |= NEAR_CLD;
		if ( far_cld )	upd_lst_w_p->nit_status |= FAR_CLD;
		if ( view_sign < 0 )
			v_angl_6km_l3_Sn[i_6km][j_6km] *= -1;

		tmp = 0.5 * (sensr_azm[i][j] - solar_azm[i][j]);
		d_azm_6km[i_6km][j_6km] = (int16) tmp;

		/* #### */
		if ( tair_s > 0.0 ) {
		  cov_prof_p = &cov_prof_n[i_6km][j_6km][i_upd_ang];
		  *cov_prof_p += cell_cov;
		  t_prof_S_p = &t_prof_sum_n[i_upd_ang][i_6km][j_6km][0];
		  for (k = 0; k < 1; k++) {
			t_prof_S_p[k] += cell_cov * tair_s;
		  }
		}
	      }
	    }
	    else if ( pix_sol_deg <= 150 && btemp[4] >= 198.0 ) {

		u8setbits(&upd_lst_w_p->i_data_yr_dn, 7, 4, i_data_yr);
		*upd_cov_d_p += cell_cov;
		upd_lst_w_p->view_zenith_day = view_ang_6km_upd;
		v_angl_6km_l3_Sd[i_6km][j_6km] = view_ang_6km_upd;
		v_time_6km_l3_Sd[i_6km][j_6km] = i_time_pix;
		upd_lst_w_p->sola_zenith = pix_sol_deg;
		if ( ipx_other_qual_dn )
			u8setbits(&upd_lst_w_p->QA_bits, 7, 4, u8_1cons);
		for (k = 0; k < 7; k++)
		    upd_lst_w_p->tb_day[k] += radiance[k] * cell_cov;
		upd_lst_w_p->tb_day[7] += e31*bt_to_radiance(4,lst_ret,i_fm1) * cell_cov;
		em31_6km_d[i_6km][j_6km] += e31 * cell_cov;
		upd_lst_w_p->cwv_day += cwv1 * cell_cov;
		*sf_elev_6km_d_p += sf_height[i][j] * cell_cov;
		/* #### */
		if ( tair_s > 0.0 ) {
		  cov_prof_p = &cov_prof_d[i_6km][j_6km][i_upd_ang];
		  *cov_prof_p += cell_cov;
		  t_prof_S_p = &t_prof_sum_d[i_upd_ang][i_6km][j_6km][0];
		  for (k = 0; k < 1; k++) {
			t_prof_S_p[k] += cell_cov * tair_s;
		  }
		}
		if ( i_lake_pix && !(cldmask[i_scan_line][j] & 020) )
			upd_lst_w_p->landcov = 0200;	/* in sunglint */

		if ( near_cld || j < j_scan_start_pix )
			upd_lst_w_p->day_status |= NEAR_CLD;
		if ( far_cld )	upd_lst_w_p->day_status |= FAR_CLD;
		if ( view_sign < 0 )
			v_angl_6km_l3_Sd[i_6km][j_6km] *= -1;
	    }
	  }

	    cov_1km_Sp[i_1km][j_1km] += cell_cov;

	    u8_3 = (lst_1km_S[i_1km][j_1km] > 0.0) ? 1 : 0;
	    if ( !u8_3 ) {
			u8setbits ( &qc_1km_l3[i_1km][j_1km], 5, 2, u8_1 );
			u8setbits ( &qc_1km_l3[i_1km][j_1km], 7, 2, u8_2 );
	    }
	    else {
	        if (cell_cov > 0.66) {
		     u8_1r = u8getbits (qc_1km_l3[i_1km][j_1km], 5, 2);
		     u8_2r = u8getbits (qc_1km_l3[i_1km][j_1km], 7, 2);
		     if ( u8_1 > u8_1r )
			u8setbits ( &qc_1km_l3[i_1km][j_1km], 5, 2, u8_1 );
		     if ( u8_2 > u8_2r )
			u8setbits ( &qc_1km_l3[i_1km][j_1km], 7, 2, u8_2 );
		}
	    }
	    /* #### Z. Wan, 06/01/2001 */
	    if ( ipx_other_qual_sw )
			u8setbits ( &qc_1km_l3[i_1km][j_1km], 3, 2, u8_1cons );

	    lst_1km_S[i_1km][j_1km] += e31 * bt_to_radiance(4,lst_ret,i_fm1) * cell_cov;
	    vtime_1km_l3_S[i_1km][j_1km] += continu_time * cell_cov;
	    view_ang_sign = 65.0 + view_sign * pix_zen_deg;
	    if ( view_ang_sign < 0 )	view_ang_sign = 0;
	    if ( view_ang_sign > 130 )	view_ang_sign = 130;
	    vangl_1km_l3_S[i_1km][j_1km] += cell_cov * view_ang_sign;
	    em31_1km_l3_S[i_1km][j_1km] += e31 * cell_cov;
	    em32_1km_l3_S[i_1km][j_1km] += e32 * cell_cov;
	}

    } /* j pixel loop */
  } /* i lines */

    /* write L2 LST output file */

    if (multi_write_out) {
      ier = putMODISarray(lst_l2_outf, lst_l2[0].name, lst_l2[0].gname,
		sta_o,dims_o, &lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		sta_o,dims_o, &QC_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray QC to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[2].name, lst_l2[2].gname,
		sta_o,dims_o, &err_lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Error_LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[3].name, lst_l2[3].gname,
		sta_o,dims_o, &emis31_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Emis_31 to L2");
    
      ier = putMODISarray(lst_l2_outf, lst_l2[4].name, lst_l2[4].gname,
		sta_o,dims_o, &emis32_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Emis_32 to L2");
    
      ier = putMODISarray(lst_l2_outf, lst_l2[5].name, lst_l2[5].gname,
		sta_o,dims_o, &view_angl_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray View_angle to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[6].name, lst_l2[6].gname,
		sta_o,dims_o, &view_time_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray View_time to L2");
    }
  }	/* end of the L2 processing for the i_scan_loop */

  if (!multi_write_out) {		/* write L2 output data arrays once */

      sta_o[0] = 0;

      ier = putMODISarray(lst_l2_outf, lst_l2[0].name, lst_l2[0].gname,
		sta_o,dims_o, &lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[1].name, lst_l2[1].gname,
		sta_o,dims_o, &QC_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray QC to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[2].name, lst_l2[2].gname,
		sta_o,dims_o, &err_lst_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Error_LST to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[3].name, lst_l2[3].gname,
		sta_o,dims_o, &emis31_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Emis_31 to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[4].name, lst_l2[4].gname,
		sta_o,dims_o, &emis32_out[0][0]);
      if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray Emis_32 to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[5].name, lst_l2[5].gname,
		sta_o,dims_o, &view_angl_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray View_angle to L2");

      ier = putMODISarray(lst_l2_outf, lst_l2[6].name, lst_l2[6].gname,
		sta_o,dims_o, &view_time_out[0][0]);
      if (ier == -1)
        HALTsmf(MODIS_F_MAPI_FAIL, "putMODISarray View_time to L2");
  }

 /* processing and writing 1km L3 output data */

 if (lst_1km_l3_outf) {

  ii1 = TILE_YDIM;
  lkk = TILE_XDIM;

  sta_1km_o[1] = 0;
  dims_1km_o[0] = 1;
  dims_1km_o[1] = lkk;

  for (i = 0; i < ii1; i++) {

    sta_1km_o[0] = i;

    if ( n_1km_l3_old_file ) {
      if (nit_mode) {
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[4].name, lst_1km_l3[4].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[5].name, lst_1km_l3[5].gname,
		sta_1km_o, dims_1km_o, &QC_1km_l3[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[6].name, lst_1km_l3[6].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[7].name, lst_1km_l3[7].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
      }
      else {
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[0].name, lst_1km_l3[0].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[1].name, lst_1km_l3[1].gname,
		sta_1km_o, dims_1km_o, &QC_1km_l3[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[2].name, lst_1km_l3[2].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	getMODISarray(lst_1km_l3_outf, lst_1km_l3[3].name, lst_1km_l3[3].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
      }
      getMODISarray(lst_1km_l3_outf, lst_1km_l3[8].name,lst_1km_l3[8].gname,
	sta_1km_o, dims_1km_o, &em31_1km_l3_Sv[0][0]);
      getMODISarray(lst_1km_l3_outf, lst_1km_l3[9].name,lst_1km_l3[9].gname,
	sta_1km_o, dims_1km_o, &em32_1km_l3_Sv[0][0]);
      getMODISarray(lst_1km_l3_outf, lst_1km_l3[10].name,lst_1km_l3[10].gname,
	sta_1km_o, dims_1km_o, &clear_day_cov[0][0]);
      getMODISarray(lst_1km_l3_outf, lst_1km_l3[11].name,lst_1km_l3[11].gname,
	sta_1km_o, dims_1km_o, &clear_nit_cov[0][0]);
    }
    else {
	z_siz = TILE_XDIM;
	zeroout_general ( &clear_day_cov_p[0], z_siz, 2);
	zeroout_general ( &clear_nit_cov_p[0], z_siz, 2);
	zeroout_general ( &em31_1km_l3_Sp[0], z_siz, 1);
	zeroout_general ( &em32_1km_l3_Sp[0], z_siz, 1);
	zeroout_general ( &lst_1km_Sp[0], z_siz, 2);
	zeroout_general ( &QC_1km_l3_p[0], z_siz, 1);
	for ( j = 0; j < lkk; j++ ) {
		v_time_1km_l3_Sp[j] = (uint8) 255;
		v_angl_1km_l3_Sp[j] = (uint8) 255;
	}
    }

    for (j = 0; j < lkk; j++) {
	if ( land_type[i][j] == 19 )
		u8setbits ( &qc_1km_l3[i][j], 1, 2, (uint8) SEA_BIT);
    }

    if (nit_mode) {
	for (j = 0; j < lkk; j++) {
	  if (cov_1km_nit_S[i][j] > 0.0) {
	     continu_time = vtime_1km_l3_S[i][j] / cov_1km_nit_S[i][j];
	     continu_time_sv = v_time_1km_l3_Sp[j] * lst_1km_l3[6].scale;
	     if ( continu_time_sv < 6.0 )	continu_time_sv += 24.0;
	     v_time_1km_l3_Sp[j] = continu_time_sv / lst_1km_l3[6].scale + 0.5;
	     vtime_1km_l3_S[i][j] = continu_time / lst_1km_l3[6].scale;
	     vangl_1km_l3_S[i][j] /= cov_1km_nit_S[i][j];
	     if ( clear_nit_cov_p[j] > 0 ) {
		if (fabs((double) vtime_1km_l3_S[i][j]-v_time_1km_l3_Sp[j]) < 1 ) {
			/* combine if within 6 minutes */
		   tmp = clear_nit_cov_p[j] * lst_1km_l3[11].scale;
		   cov_1km_nit_S[i][j] += tmp;
		   tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		   tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		   tmp_d = 0.02 * lst_1km_Sp[j];
		   lst_1km_S[i][j] += tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		   em31_1km_l3_S[i][j] += tmp_em31 * tmp;
		   em32_1km_l3_S[i][j] += tmp_em32 * tmp;
		}
		else if ((fabs((double) v_angl_1km_l3_Sp[j] - 65.) <
			  fabs((double) vangl_1km_l3_S[i][j] - 65.) &&
			  (lst_1km_Sp[j] > (uint16)
			  (50*lst_1km_S[i][j]/cov_1km_nit_S[i][j]))) ||
			(lst_1km_Sp[j] > (uint16)
			(50*lst_1km_S[i][j]/cov_1km_nit_S[i][j] + 50)) ) {

		   /* use the old one */

		   tmp = clear_nit_cov_p[j] * lst_1km_l3[11].scale;
		   cov_1km_nit_S[i][j] = tmp;
		   qc_1km_l3[i][j] = QC_1km_l3_p[j];
		   vtime_1km_l3_S[i][j] = v_time_1km_l3_Sp[j];
		   vangl_1km_l3_S[i][j] = (float32) v_angl_1km_l3_Sp[j];
		   tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		   tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		   tmp_d = lst_1km_Sp[j] * lst_1km_l3[4].scale;
		   lst_1km_S[i][j] = tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		   em31_1km_l3_S[i][j] = tmp_em31 * tmp;
		   em32_1km_l3_S[i][j] = tmp_em32 * tmp;
		}
	     }
	    }
	    else if ( clear_nit_cov_p[j] > 0 ) {	/* use the old one */
		   tmp = clear_nit_cov_p[j] * lst_1km_l3[11].scale;
		   cov_1km_nit_S[i][j] = tmp;
		   qc_1km_l3[i][j] = QC_1km_l3_p[j];
		   vtime_1km_l3_S[i][j] = v_time_1km_l3_Sp[j];
		   vangl_1km_l3_S[i][j] = (float32) v_angl_1km_l3_Sp[j];
		   tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		   tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		   tmp_d = lst_1km_Sp[j] * lst_1km_l3[4].scale;
		   lst_1km_S[i][j] = tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		   em31_1km_l3_S[i][j] = tmp_em31 * tmp;
		   em32_1km_l3_S[i][j] = tmp_em32 * tmp;
	    }
	    if ( cov_1km_nit_S[i][j] > 0.0 ) {
		rad_v = lst_1km_S[i][j] / em31_1km_l3_S[i][j];
		lst_1km_Sp[j] = f_mod_rad7_inv(4, rad_v, i_fm1) * 50.0 + 0.5;
		u8_qa_1km_dat = u8getbits (qc_1km_l3[i][j], 3, 2);
		u8_qa_1km_emer = u8getbits (qc_1km_l3[i][j], 5, 2);
		u8_qa_1km_lster = u8getbits (qc_1km_l3[i][j], 7, 2);
		if ( u8_qa_1km_dat > u8_0cons || u8_qa_1km_emer > u8_0cons ||
		     u8_qa_1km_lster > u8_0cons )
			u8setbits ( &qc_1km_l3[i][j], 1, 2, u8_1cons);
		else
			u8setbits ( &qc_1km_l3[i][j], 1, 2, u8_0cons);
	    }
	  }
	}
	else {				/* daytime */
	  for (j = 0; j < lkk; j++) {
	    if (cov_1km_day_S[i][j] > 0.0) {
		continu_time = vtime_1km_l3_S[i][j] / cov_1km_day_S[i][j];
		vtime_1km_l3_S[i][j] = continu_time / lst_1km_l3[2].scale;
		vangl_1km_l3_S[i][j] /= cov_1km_day_S[i][j];
		if ( clear_day_cov_p[j] > 0 ) {
		   if (fabs((double) vtime_1km_l3_S[i][j]-v_time_1km_l3_Sp[j]) < 1 ) {
			/* combine if within 6 minutes */
		      tmp = clear_day_cov_p[j] * lst_1km_l3[10].scale;
		      cov_1km_day_S[i][j] += tmp;
		      tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		      tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		      tmp_d = 0.02 * lst_1km_Sp[j];
		      lst_1km_S[i][j] += tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		      em31_1km_l3_S[i][j] += tmp_em31 * tmp;
		      em32_1km_l3_S[i][j] += tmp_em32 * tmp;
		   }
		   else if ((fabs((double) v_angl_1km_l3_Sp[j] - 65.) <
			     fabs((double) vangl_1km_l3_S[i][j] - 65.) &&
			     (lst_1km_Sp[j] > (uint16)
			     (50*lst_1km_S[i][j]/cov_1km_day_S[i][j]))) ||
			   (lst_1km_Sp[j] > (uint16)
			   (50*lst_1km_S[i][j]/cov_1km_day_S[i][j] + 50)) ) {

		      /* use the old one */

		      tmp = clear_day_cov_p[j] * lst_1km_l3[10].scale;
		      cov_1km_day_S[i][j] = tmp;
		      qc_1km_l3[i][j] = QC_1km_l3_p[j];
		      vtime_1km_l3_S[i][j] = v_time_1km_l3_Sp[j];
		      vangl_1km_l3_S[i][j] = (float32) v_angl_1km_l3_Sp[j];
		      tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		      tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		      tmp_d = lst_1km_Sp[j]* lst_1km_l3[0].scale;
		      lst_1km_S[i][j] = tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		      em31_1km_l3_S[i][j] = tmp_em31 * tmp;
		      em32_1km_l3_S[i][j] = tmp_em32 * tmp;
		   }
		}
	    }
	    else if ( clear_day_cov_p[j] > 0 ) {	/* use the old one */
		      tmp = clear_day_cov_p[j] * lst_1km_l3[10].scale;
		      cov_1km_day_S[i][j] = tmp;
		      qc_1km_l3[i][j] = QC_1km_l3_p[j];
		      vtime_1km_l3_S[i][j] = v_time_1km_l3_Sp[j];
		      vangl_1km_l3_S[i][j] = (float32) v_angl_1km_l3_Sp[j];
		      tmp_em31 = em31_1km_l3_Sp[j] * lst_1km_l3[8].scale + 0.49;
		      tmp_em32 = em32_1km_l3_Sp[j] * lst_1km_l3[9].scale + 0.49;
		      tmp_d = lst_1km_Sp[j] * lst_1km_l3[0].scale;
		      lst_1km_S[i][j] = tmp_em31 * bt_to_radiance(4,tmp_d,i_fm1) * tmp;
		      em31_1km_l3_S[i][j] = tmp_em31 * tmp;
		      em32_1km_l3_S[i][j] = tmp_em32 * tmp;
	    }
	    if ( cov_1km_day_S[i][j] > 0.0 ) {
		rad_v = lst_1km_S[i][j] / em31_1km_l3_S[i][j];
		lst_1km_Sp[j] = f_mod_rad7_inv(4,rad_v,i_fm1) * 50.0 + 0.5;
		u8_qa_1km_dat = u8getbits (qc_1km_l3[i][j], 3, 2);
		u8_qa_1km_emer = u8getbits (qc_1km_l3[i][j], 5, 2);
		u8_qa_1km_lster = u8getbits (qc_1km_l3[i][j], 7, 2);
		if ( u8_qa_1km_dat > u8_0cons || u8_qa_1km_emer > u8_0cons ||
		     u8_qa_1km_lster > u8_0cons )
			u8setbits ( &qc_1km_l3[i][j], 1, 2, u8_1cons);
		else
			u8setbits ( &qc_1km_l3[i][j], 1, 2, u8_0cons);
	    }
	  }
	}

	for (j = 0; j < lkk; j++) {

	    if (em31_1km_l3_S[i][j] > 0.0) {
	      tmp = em31_1km_l3_S[i][j] / cov_1km_Sp[i][j];
	      em31_1km_l3_Sp[j] = (tmp - lst_1km_l3[8].offset)
	      				/ lst_1km_l3[8].scale + 0.5;
	      tmp = em32_1km_l3_S[i][j] / cov_1km_Sp[i][j];
	      em32_1km_l3_Sp[j] = (tmp - lst_1km_l3[9].offset)
					/ lst_1km_l3[9].scale + 0.5;
	    } /* keep the old one
	    else {
	    	em31_1km_l3_Sp[j] = 0;
	        em32_1km_l3_Sp[j] = 0;
	    } */

	    clear_day_cov_p[j] = cov_1km_day_S[i][j] / lst_1km_l3[10].scale+0.5;
	    clear_nit_cov_p[j] = cov_1km_nit_S[i][j] / lst_1km_l3[11].scale+0.5;
	    if ( lst_1km_Sp[j] ) {
	      continu_time = vtime_1km_l3_S[i][j];
	      if ( nit_mode && continu_time >= 240 ) {
			continu_time -= 240;
			vtime_1km_l3_S[i][j] -= 240.0;
	      }
	      v_time_1km_l3_Sp[j] = continu_time + 0.5;	/* unit of 0.1 hrs */
	      view_ang_sign = (int32) (vangl_1km_l3_S[i][j] + 0.5);
	      if ( view_ang_sign < 0 )		view_ang_sign = 0;
	      if ( view_ang_sign > 130 )	view_ang_sign = 130;
	      v_angl_1km_l3_Sp[j] = (uint8) view_ang_sign;
	    }
	}

    sta_1km_o[0] = i;
    if (nit_mode) {
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[4].name, lst_1km_l3[4].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[5].name, lst_1km_l3[5].gname,
		sta_1km_o, dims_1km_o, &qc_1km_l3[i][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[6].name,lst_1km_l3[6].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[7].name,lst_1km_l3[7].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[11].name,lst_1km_l3[11].gname,
		sta_1km_o, dims_1km_o, &clear_nit_cov[0][0]);
    }
    else {
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[0].name, lst_1km_l3[0].gname,
		sta_1km_o, dims_1km_o, &lst_1km_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[1].name, lst_1km_l3[1].gname,
		sta_1km_o, dims_1km_o, &qc_1km_l3[i][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[2].name,lst_1km_l3[2].gname,
		sta_1km_o, dims_1km_o, &v_time_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[3].name,lst_1km_l3[3].gname,
		sta_1km_o, dims_1km_o, &v_angl_1km_l3_Sv[0][0]);
	putMODISarray(lst_1km_l3_outf, lst_1km_l3[10].name,lst_1km_l3[10].gname,
		sta_1km_o, dims_1km_o, &clear_day_cov[0][0]);
    }

    putMODISarray(lst_1km_l3_outf, lst_1km_l3[8].name,lst_1km_l3[8].gname,
		sta_1km_o, dims_1km_o, &em31_1km_l3_Sv[0][0]);
    putMODISarray(lst_1km_l3_outf, lst_1km_l3[9].name,lst_1km_l3[9].gname,
		sta_1km_o, dims_1km_o, &em32_1km_l3_Sv[0][0]);
  }

  /* ### calculate and write lst_6km_S_from_1km */
  sta_6km_o[1] = 0;
  dims_6km_o[0] = 1;
  dims_6km_o[1] = ncol_6km;
  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

    sta_6km_o[0] = i_6km;

    if (nit_mode) {
	QC_6km = QC_6km_nit;
	getMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name,lst_6km_l3[5].gname,
		sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);
    }
    else {
	QC_6km = QC_6km_day;
	getMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name,lst_6km_l3[1].gname,
		sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
    }
    getMODISarray(lst_6km_l3_outf, lst_6km_l3[12].name,lst_6km_l3[12].gname,
	sta_6km_o, dims_6km_o, &em31_6km_l3_Sv[0][0]);
    getMODISarray(lst_6km_l3_outf, lst_6km_l3[13].name,lst_6km_l3[13].gname,
	sta_6km_o, dims_6km_o, &em32_6km_l3_Sv[0][0]);

    for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

      cov_sum_6km = 0;	temp_m = 0.0;
      temp_m_a = 0.0;	temp_m_t = 0.0;
      cov_sum_6km_2 = 0;	temp_m_2 = 0.0;
      temp_m_a_2 = 0.0;		temp_m_t_2 = 0.0;
      tmp_em31 = 0.0;		cov_em_6km = 0.0;
      tmp_em32 = 0.0;
      tmp_em31_2 = 0.0;		cov_em_6km_2 = 0.0;
      tmp_em32_2 = 0.0;

      {
	int32 ix, jx, ii6f, ii6l, jj6f, jj6l;
	int32 n_qc_qua[4], n_qc_em[4], n_qc_lst[4];
	int32 n_qc_qua2[4], n_qc_em2[4], n_qc_lst2[4];
	int32 m_qc_man, m_qc_qua, m_qc_em, m_qc_lst;
	int32 i_qc_man, i_qc_qua, i_qc_em, i_qc_lst;
	uint8 k_qc_man, k_qc_qua, k_qc_em, k_qc_lst;

	ii6f = 6 * i_6km;
	ii6l = ii6f + 6;
	jj6f = 6 * j_6km;
	jj6l = jj6f + 6;

	for ( ix = 0; ix < 4; ix++ ) {
		n_qc_qua[ix] = 0;	n_qc_qua2[ix] = 0;
		n_qc_em[ix] = 0;	n_qc_em2[ix] = 0;
		n_qc_lst[ix] = 0;	n_qc_lst2[ix] = 0;
	}
	for ( ix = ii6f; ix < ii6l; ix++ ) {
	  for ( jx = jj6f; jx < jj6l; jx++ ) {

	    if ( lst_1km_S[ix][jx] > 0.0 ) {	/* sum of radiance in b31 */

	      if ( !nit_mode || nit_mode && vtime_1km_l3_S[ix][jx] <= 60.0 ) {

		temp_m += lst_1km_S[ix][jx];

		if ( cov_1km_Sp[ix][jx] ) {
		  cov_em_6km += cov_1km_Sp[ix][jx];
		  tmp_em31 += em31_1km_l3_S[ix][jx];
		  tmp_em32 += em32_1km_l3_S[ix][jx];
		}

		temp_d = vangl_1km_l3_S[ix][jx];	/* in range of 0-130 */
	    	temp_d_2 = vtime_1km_l3_S[ix][jx];	/* in unit of 0.1 hrs */

		cov_sum_6km += cov_1km_Sp[ix][jx];
		temp_m_a += temp_d * cov_1km_Sp[ix][jx];
		temp_m_t += temp_d_2 * cov_1km_Sp[ix][jx];

		i_qc_qua = (int32) u8getbits (qc_1km_l3[ix][jx], 3, 2);
		i_qc_em = (int32) u8getbits (qc_1km_l3[ix][jx], 5, 2);
		i_qc_lst = (int32) u8getbits (qc_1km_l3[ix][jx], 7, 2);

		n_qc_qua[i_qc_qua] += 1;
		n_qc_em[i_qc_em] += 1;
		n_qc_lst[i_qc_lst] += 1;
	      }
	      if ( nit_mode && vtime_1km_l3_S[ix][jx] > 180.0 ) {

		temp_m_2 += lst_1km_S[ix][jx];

		if ( cov_1km_Sp[ix][jx] ) {
		  cov_em_6km_2 += cov_1km_Sp[ix][jx];
		  tmp_em31_2 += em31_1km_l3_S[ix][jx];
		  tmp_em32_2 += em32_1km_l3_S[ix][jx];
		}

		cov_sum_6km_2 += cov_1km_nit_S[ix][jx];
		temp_m_a_2 += vangl_1km_l3_S[ix][jx] * cov_1km_nit_S[ix][jx];
		temp_m_t_2 += vtime_1km_l3_S[ix][jx] * cov_1km_nit_S[ix][jx];

		i_qc_qua = (int32) u8getbits (qc_1km_l3[ix][jx], 3, 2);
		i_qc_em = (int32) u8getbits (qc_1km_l3[ix][jx], 5, 2);
		i_qc_lst = (int32) u8getbits (qc_1km_l3[ix][jx], 7, 2);

		n_qc_qua2[i_qc_qua] += 1;
		n_qc_em2[i_qc_em] += 1;
		n_qc_lst2[i_qc_lst] += 1;
	      }
	    }
	  }
      }

      if ( cov_sum_6km > 0.0 || cov_sum_6km_2 > 0.0 ) {

	    m_qc_man = 0;	m_qc_qua = 0;	m_qc_em = 0;	m_qc_lst = 0;
	    k_qc_man = 0;	k_qc_qua = 0;	k_qc_em = 0;	k_qc_lst = 0;

	    if ( !nit_mode && cov_sum_6km || nit_mode && cov_sum_6km
		  && (cov_sum_6km > 0.5 * cov_sum_6km_2) ) {

		rad_v = temp_m / tmp_em31;
		temp_m = f_mod_rad7_inv(4, rad_v, i_fm1);

	       if ( !nit_mode || (nit_mode && em31_6km_l3_Sv[0][j_6km] == u8_0cons) ) {
		tmp_em31 = tmp_em31 / cov_em_6km - 0.49;
		tmp_em32 = tmp_em32 / cov_em_6km - 0.49;
		em31_6km_l3_Sv[0][j_6km] = (uint8) (500.0 * tmp_em31 + 0.5);
		em32_6km_l3_Sv[0][j_6km] = (uint8) (500.0 * tmp_em32 + 0.5);
	       }
		continu_time = temp_m_t / cov_sum_6km;
		view_ang_sign = temp_m_a / cov_sum_6km + 0.5;

		for ( ix = 0; ix < 4; ix++ ) {
		  if ( ix < 3 && (n_qc_qua[ix] > m_qc_qua) ) {
			m_qc_qua = n_qc_qua[ix];
			k_qc_qua = ix;
		  }
		  if ( n_qc_em[ix] > m_qc_em ) {
			m_qc_em = n_qc_em[ix];
			k_qc_em = ix;
		  }
		  if ( n_qc_lst[ix] > m_qc_lst ) {
			m_qc_lst = n_qc_lst[ix];
			k_qc_lst = ix;
		  }
		}
	    }
	    else if ( cov_sum_6km_2 > 0.0 ) {

		rad_v = temp_m_2 / tmp_em31_2;
		temp_m = f_mod_rad7_inv(4, rad_v, i_fm1);

	       if ( nit_mode && em31_6km_l3_Sv[0][j_6km] == u8_0cons ) {
		tmp_em31_2 = tmp_em31_2 / cov_em_6km_2 - 0.49;
		tmp_em32_2 = tmp_em32_2 / cov_em_6km_2 - 0.49;
		em31_6km_l3_Sv[0][j_6km] = (uint8) (500.0 * tmp_em31_2 + 0.5);
		em32_6km_l3_Sv[0][j_6km] = (uint8) (500.0 * tmp_em32_2 + 0.5);
	       }
		continu_time = temp_m_t_2 / cov_sum_6km_2;
		view_ang_sign = temp_m_a_2 / cov_sum_6km_2 + 0.5;

		for ( ix = 0; ix < 4; ix++ ) {
		  if ( ix < 3 && (n_qc_qua2[ix] > m_qc_qua) ) {
			m_qc_qua = n_qc_qua2[ix];
			k_qc_qua = ix;
		  }
		  if ( n_qc_em2[ix] > m_qc_em ) {
			m_qc_em = n_qc_em2[ix];
			k_qc_em = ix;
		  }
		  if ( n_qc_lst2[ix] > m_qc_lst ) {
			m_qc_lst = n_qc_lst2[ix];
			k_qc_lst = ix;
		  }
		}
	    }

	    lst_6km_Sp_from_1km[j_6km] = temp_m / lst_6km_l3[15].scale + 0.5;

	    if ( view_ang_sign < 0 )	view_ang_sign = 0;
	    if ( view_ang_sign > 130 )	view_ang_sign = 130;
	    v_angl_6km_l3_Sv[0][j_6km] = (uint8) view_ang_sign;

	    if ( continu_time >= 240.0 )
		continu_time -= 240.0;
							/* in unit of 0.2 hrs */
	    v_time_6km_l3_Sv[0][j_6km] = (uint8) (0.5 * continu_time + 0.5);


	    if ( k_qc_qua > u8_1cons || k_qc_em > u8_0cons ||
		 k_qc_lst > u8_0cons )         k_qc_man = u8_1cons;

	    u8setbits (&QC_6km[0][j_6km], 1, 2, k_qc_man);
	    u8setbits (&QC_6km[0][j_6km], 3, 2, k_qc_qua);
	    u8setbits (&QC_6km[0][j_6km], 5, 2, k_qc_em);
	    u8setbits (&QC_6km[0][j_6km], 7, 2, k_qc_lst);
      }
      else {
	   lst_6km_Sp_from_1km[j_6km] = (uint16) 0;
	   v_angl_6km_l3_Sv[0][j_6km] = (uint8) 255;
	   v_time_6km_l3_Sv[0][j_6km] = (uint8) 255;
      }
     }
    }
    if (nit_mode) {
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name,lst_6km_l3[5].gname,
		sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[6].name,lst_6km_l3[6].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[7].name,lst_6km_l3[7].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[15].name,lst_6km_l3[15].gname,
		sta_6km_o, dims_6km_o, &lst_6km_S_from_1km[0][0]);
    }
    else {
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name,lst_6km_l3[1].gname,
		sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[2].name,lst_6km_l3[2].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[3].name,lst_6km_l3[3].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sv[0][0]);
	putMODISarray(lst_6km_l3_outf, lst_6km_l3[14].name,lst_6km_l3[14].gname,
		sta_6km_o, dims_6km_o, &lst_6km_S_from_1km[0][0]);
    }
    putMODISarray(lst_6km_l3_outf, lst_6km_l3[12].name,lst_6km_l3[12].gname,
	sta_6km_o, dims_6km_o, &em31_6km_l3_Sv[0][0]);
    putMODISarray(lst_6km_l3_outf, lst_6km_l3[13].name,lst_6km_l3[13].gname,
	sta_6km_o, dims_6km_o, &em32_6km_l3_Sv[0][0]);
  }
 }

/* move to outside of the i_scan_loop
  DoMetaData( n_1km_l3_old_file, (int32) 3, (PGSt_PC_Logical) PCLogL3_1km, 
	  (PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, lst_1km_l3_outf);
*/
  /* Close the output MODIS-HDF files */

  if ( closeMODISfile(&lst_l2_outf) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "L2 output file failed");

  if ( closeMODISfile(&lst_1km_l3_outf) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "1km L3 output file failed");

  /* average data in upd_ang_lst */

  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

    for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	/* to determine cld or sea grid here */
	/* ##### also to calculate BRDF parameters at 6km resolution */
	{
		int32 i, j, ii6f, ii6l, jj6f, jj6l;
		int16 *brdf_p_p;
		int32 n_lake_pix = 0;

		n_cld_grid = n_sea_grid = 0;

		ii6f = 6 * i_6km;
		ii6l = ii6f + 6;
		jj6f = 6 * j_6km;
		jj6l = jj6f + 6;

		for (k = 0; k <= N_LANDCOV_TYPES; k++)
			n_pix_land_type[k] = 0.0;

		i_lake_pix = 0;
		for (i = ii6f; i < ii6l; i++) {
			for (j = jj6f; j < jj6l; j++) {
				u8_2 = (uint8) qc_1km_l3[i][j] & 03;
				if ( u8_2 == u8_2cons ) {
					n_cld_grid++;
					continue;
				}
				if ( u8_2 == u8_3cons ) {
					n_sea_grid++;
					continue;
				}
				i_grid_land_type = (int32) land_type[i][j];
				n_pix_land_type[i_grid_land_type] += 1;
				lc_land_sea = lc_qc[i][j] >> 4 & ~(~0 << 4);
				if ( lc_land_sea > u8_2cons && lc_land_sea < u8_6cons )
					n_lake_pix += 1;
			}
		}
		n_pix_land_type_sum = 0;
		for ( k = 0; k <= N_LANDCOV_TYPES; k++ ) {
		   if ( n_pix_land_type[k] > n_pix_land_type_sum ) {
			n_pix_land_type_sum = n_pix_land_type[k];
			i_grid_land_type = k;
		   }
		}
		if ( n_pix_land_type_sum > 1 &&
		     ( n_lake_pix / n_pix_land_type_sum) > 0.80 )
			i_lake_pix = 1;
	}

	i_clean_upd = 0;

	if ( n_cld_grid > 24 || n_sea_grid > 24 )
		i_clean_upd = 1;

	upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];
	upd_a_lst_w_p_snow = upd_a_lst_w_p;
	upd_a_lst_w_p_ice  = upd_a_lst_w_p;

	upd_cov_d_p = &upd_cov_d[i_6km][j_6km][0];
	upd_cov_n_p = &upd_cov_n[i_6km][j_6km][0];

	if ( i_clean_upd ) {

	   for ( i_view = 0; i_view < n2_upd_angs; i_view++ ) {

		upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[i_view];

		if ( nit_mode ) {
		   if ( upd_cov_n_p[i_view] > 0.0 ) {
			upd_cov_n_p[i_view] = 0.0;
			upd_lst_w_p->nit_status = 0;
			u8setbits(&upd_lst_w_p->QA_bits, 3, 4, u8_0cons);
			upd_lst_w_p->upd_cov_n = 0;
			if ( n_cld_grid > 24 )
			     u8setbits(&upd_lst_w_p->QA_bits, 3, 4, u8_2cons);
			if ( n_sea_grid > 24 )
			     u8setbits(&upd_lst_w_p->QA_bits, 3, 4, SEA_BIT);
			for (k = 0; k < 8; k++)
				upd_lst_w_p->tb_nit[k] = 0.0;
			upd_lst_w_p->cwv_nit = 0.0;
		   }
		}
		else {
			upd_cov_d_p[i_view] = 0.0;
			upd_lst_w_p->day_status = 0;
			u8setbits(&upd_lst_w_p->QA_bits, 7, 4, u8_0cons);
			upd_lst_w_p->upd_cov_d = 0;
			upd_lst_w_p->landcov = 0;
			if ( n_cld_grid > 24 )
			     u8setbits(&upd_lst_w_p->QA_bits, 7, 4, u8_2cons);
			if ( n_sea_grid > 24 )
			     u8setbits(&upd_lst_w_p->QA_bits, 7, 4, SEA_BIT);
			for (k = 0; k < 8; k++)
				upd_lst_w_p->tb_day[k] = 0.0;
			upd_lst_w_p->cwv_day = 0.0;
		}
	   }
	}		/* i_clean_upd done */
	else {

	 i_view_6km_from_east = 0;

	 if ( nit_mode )			/* in unit of 0.5 deg */
		view_ang_6km_upd = v_angl_6km_l3_Sn[i_6km][j_6km];
	 else
		view_ang_6km_upd = v_angl_6km_l3_Sd[i_6km][j_6km];

	 if ( view_ang_6km_upd < 0 ) {
		view_ang_6km_upd *= -1;
		i_view_6km_from_east = 1;
	 }

	 if ( view_ang_6km_upd > _upd_ang_range_u2[N_VIEW_ANGS-1] )
		i_upd_a_o = N_VIEW_ANGS - 1;
	 else {
		for ( k = 0; k < n_upd_angs; k++ ) {
			if ( view_ang_6km_upd <= _upd_ang_range_u2[k] ) {
				i_upd_a_o = k;
				break;
			}
		}
	 }
	 if ( i_view_6km_from_east || ! view_ang_6km_upd )
		i_upd_ang = abs(i_upd_a_o - 7);
	 else	i_upd_ang = i_upd_a_o + 8;

	 i_view = i_upd_ang;
	 upd_a_lst_w_p = &upd_ang_lst_W[i_6km][j_6km];
	 upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[i_view];

	 /* merging to the i_upd_ang component at both edges */

	 i_view_p1 = i_view - 1;

	 if ( i_view_p1 >= 0 ) {
	
	   upd_lst_w_p1 = &upd_a_lst_w_p->_upd_for_lst[i_view_p1];

	   if ( nit_mode ) {

	      if( upd_cov_n_p[i_view] > 0.0 && upd_cov_n_p[i_view_p1] > 0.0 ) {

		for ( k = 0; k < 8; k++ ) {
			upd_lst_w_p->tb_nit[k] += upd_lst_w_p1->tb_nit[k];
			upd_lst_w_p1->tb_nit[k] = 0.0;
		}
		upd_lst_w_p->cwv_nit += upd_lst_w_p1->cwv_nit;
		upd_lst_w_p->view_zenith_nit = view_ang_6km_upd; /* 2d */
		upd_lst_w_p1->cwv_nit = 0.0;
		upd_lst_w_p1->nit_status = 0;
		u8setbits(&upd_lst_w_p1->QA_bits, 3, 4, SEA_BIT);
		upd_lst_w_p1->upd_cov_n = 0;
		upd_cov_n_p[i_view] += upd_cov_n_p[i_view_p1];
		upd_cov_n_p[i_view_p1] = 0.0;
		cov_prof_n[i_6km][j_6km][i_view] +=
			cov_prof_n[i_6km][j_6km][i_view_p1];
		cov_prof_n[i_6km][j_6km][i_view_p1] = 0;
		t_prof_sum_n[i_view][i_6km][j_6km][0] +=
			t_prof_sum_n[i_view_p1][i_6km][j_6km][0];
		t_prof_sum_n[i_view_p1][i_6km][j_6km][0] = 0;
	      }
	   }
	   else {

	      if( upd_cov_d_p[i_view] > 0.0 && upd_cov_d_p[i_view_p1] > 0.0 ) {

		for ( k = 0; k < 8; k++ ) {
			upd_lst_w_p->tb_day[k] += upd_lst_w_p1->tb_day[k];
			upd_lst_w_p1->tb_day[k] = 0.0;
		}
		upd_lst_w_p->cwv_day += upd_lst_w_p1->cwv_day;
		upd_lst_w_p->view_zenith_day = view_ang_6km_upd; /* 2d */
		upd_lst_w_p1->cwv_day = 0.0;
		upd_lst_w_p1->day_status = 0;
		upd_lst_w_p1->landcov = 0;
		u8setbits(&upd_lst_w_p1->QA_bits, 7, 4, SEA_BIT);
		upd_lst_w_p1->upd_cov_d = 0;
		upd_cov_d_p[i_view] += upd_cov_d_p[i_view_p1];
		upd_cov_d_p[i_view_p1] = 0.0;
		cov_prof_d[i_6km][j_6km][i_view] +=
			cov_prof_d[i_6km][j_6km][i_view_p1];
		cov_prof_d[i_6km][j_6km][i_view_p1] = 0;
		t_prof_sum_d[i_view][i_6km][j_6km][0] +=
			t_prof_sum_d[i_view_p1][i_6km][j_6km][0];
		t_prof_sum_d[i_view_p1][i_6km][j_6km][0] = 0;
	      }
	   }
	 }

	 i_view_p1 = i_view + 1;

	 if ( i_view_p1 < n2_upd_angs ) {
	
	   upd_lst_w_p1 = &upd_a_lst_w_p->_upd_for_lst[i_view_p1];

	   if ( nit_mode ) {

	      if( upd_cov_n_p[i_view] > 0.0 && upd_cov_n_p[i_view_p1] > 0.0 ) {

		for ( k = 0; k < 8; k++ ) {
			upd_lst_w_p->tb_nit[k] += upd_lst_w_p1->tb_nit[k];
			upd_lst_w_p1->tb_nit[k] = 0.0;
		}
		upd_lst_w_p->cwv_nit += upd_lst_w_p1->cwv_nit;
		upd_lst_w_p->view_zenith_nit = view_ang_6km_upd; /* 2d */
		upd_lst_w_p1->cwv_nit = 0.0;
		upd_lst_w_p1->nit_status = 0;
		u8setbits(&upd_lst_w_p1->QA_bits, 3, 4, SEA_BIT);
		upd_lst_w_p1->upd_cov_n = 0;
		upd_cov_n_p[i_view] += upd_cov_n_p[i_view_p1];
		upd_cov_n_p[i_view_p1] = 0.0;
		cov_prof_n[i_6km][j_6km][i_view] +=
			cov_prof_n[i_6km][j_6km][i_view_p1];
		cov_prof_n[i_6km][j_6km][i_view_p1] = 0;
		t_prof_sum_n[i_view][i_6km][j_6km][0] +=
			t_prof_sum_n[i_view_p1][i_6km][j_6km][0];
		t_prof_sum_n[i_view_p1][i_6km][j_6km][0] = 0;
	      }
	   }
	   else {
	      if( upd_cov_d_p[i_view] > 0.0 && upd_cov_d_p[i_view_p1] > 0.0 ) {

		for ( k = 0; k < 8; k++ ) {
			upd_lst_w_p->tb_day[k] += upd_lst_w_p1->tb_day[k];
			upd_lst_w_p1->tb_day[k] = 0.0;
		}
		upd_lst_w_p->cwv_day += upd_lst_w_p1->cwv_day;
		upd_lst_w_p->view_zenith_day = view_ang_6km_upd; /* 2d */
		upd_lst_w_p1->cwv_day = 0.0;
		upd_lst_w_p1->day_status = 0;
		upd_lst_w_p1->landcov = 0;
		u8setbits(&upd_lst_w_p1->QA_bits, 7, 4, SEA_BIT);
		upd_lst_w_p1->upd_cov_d = 0;
		upd_cov_d_p[i_view] += upd_cov_d_p[i_view_p1];
		upd_cov_d_p[i_view_p1] = 0.0;
		cov_prof_d[i_6km][j_6km][i_view] +=
			cov_prof_d[i_6km][j_6km][i_view_p1];
		cov_prof_d[i_6km][j_6km][i_view_p1] = 0;
		t_prof_sum_d[i_view][i_6km][j_6km][0] +=
			t_prof_sum_d[i_view_p1][i_6km][j_6km][0];
		t_prof_sum_d[i_view_p1][i_6km][j_6km][0] = 0;
	      }
	    }
	 } /* merging at both edges done */

	 upd_lst_w_p = &upd_a_lst_w_p->_upd_for_lst[i_upd_ang];

	 if ( nit_mode ) {

	   cov_v_prof = cov_prof_n[i_6km][j_6km][i_upd_ang];
	   t_prof_S_p = &t_prof_sum_n[i_upd_ang][i_6km][j_6km][0];
	   cov_v = upd_cov_n_p[i_upd_ang];
	   cov_s_u8 = (uint8) cov_v;
	   i_time_pix = v_time_6km_l3_Sn[i_6km][j_6km];

	   if ( cov_v > 0.0 ) {

if ( cov_v > upd_cov_n_max[i_upd_ang] )  upd_cov_n_max[i_upd_ang] = cov_v;

	      for (k = 0; k < 7; k++) {
		rad_v = upd_lst_w_p->tb_nit[k] / cov_v;
		tmp = f_mod_rad7_inv (k, rad_v, i_fm1);
		upd_lst_w_p->tb_nit[k] = tmp;
	      }
	      if ( upd_lst_w_p->tb_nit[4] < 195.0 ) {
		upd_cov_n_p[i_upd_ang] = 0.0;
		continue;
	      }
	   
	      upd_lst_w_p->nit_status |= NEW_EXIST_BITS;
	      upd_lst_w_p->upd_cov_n = cov_s_u8;
	      upd_lst_w_p->nit_date_and_view_time =
		CB_DAY_HRS(i_day_of_yr_curr, i_time_pix);
	      if ( n_cld_grid > 18 || n_sea_grid > 18 )
		u8setbits(&upd_lst_w_p->QA_bits, 3, 4, u8_1cons);
	      rad_v = upd_lst_w_p->tb_nit[7] / em31_6km_n[i_6km][j_6km];
	      tmp = f_mod_rad7_inv (4, rad_v, i_fm1);
	      upd_lst_w_p->tb_nit[7] = tmp;
	      upd_lst_w_p->cwv_nit /= cov_v;
	      upd_lst_w_p->sf_elev = (sf_elev_6km_n[i_6km][j_6km]/cov_v) + 0.5;
	      /*upd_lst_w_p->view_zenith_nit = view_ang_6km_upd; done */
	      
	      /* #### */
	      if ( cov_v_prof ) {
			tmp_int = (t_prof_S_p[0]/cov_v_prof - 185.) / 0.6 + 0.5;
			if (tmp_int < 0)	tmp_int = 0;
			if (tmp_int > 255)	tmp_int = 255;
			upd_lst_w_p->ta_nit = tmp_int;
	      }
	   }
	   else		upd_cov_n_p[i_upd_ang] = 0.0;
	 }
	 else {

	   cov_v_prof = cov_prof_d[i_6km][j_6km][i_upd_ang];
	   t_prof_S_p = &t_prof_sum_d[i_upd_ang][i_6km][j_6km][0];
	   cov_v = upd_cov_d_p[i_upd_ang];
	   cov_s_u8 = (uint8) cov_v;
	   i_time_pix = v_time_6km_l3_Sd[i_6km][j_6km];

	   if ( cov_v > 0.0 && upd_lst_w_p->sola_zenith <= 150 ) {

if ( cov_v > upd_cov_d_max[i_upd_ang] )  upd_cov_d_max[i_upd_ang] = cov_v;

	      for (k = 0; k < 7; k++) {
		rad_v = upd_lst_w_p->tb_day[k] / cov_v;
		tmp = f_mod_rad7_inv (k, rad_v, i_fm1);
		upd_lst_w_p->tb_day[k] = tmp;
	      }
	      if ( upd_lst_w_p->tb_day[4] < 198.0 ) {
		upd_cov_d_p[i_upd_ang] = 0.0;
	      	continue;
	      }
	      upd_lst_w_p->day_status |= NEW_EXIST_BITS;
	      upd_lst_w_p->upd_cov_d = cov_s_u8;
	      upd_lst_w_p->day_date_and_view_time =
			CB_DAY_HRS(i_day_of_yr_curr, i_time_pix);
	      if ( n_cld_grid > 18 || n_sea_grid > 18 )
		u8setbits(&upd_lst_w_p->QA_bits, 7, 4, u8_1cons);
	      rad_v = upd_lst_w_p->tb_day[7] / em31_6km_d[i_6km][j_6km];
	      tmp = f_mod_rad7_inv (4, rad_v, i_fm1);
	      upd_lst_w_p->tb_day[7] = tmp;
	      upd_lst_w_p->reserved[0] = (int) (500.0 *
			(em31_6km_d[i_6km][j_6km]/cov_v - 0.49) + 0.5);
	      upd_lst_w_p->cwv_day /= cov_v;
	      upd_lst_w_p->sf_elev = (sf_elev_6km_d[i_6km][j_6km]/cov_v) + 0.5;
	      /*upd_lst_w_p->view_zenith_day = view_ang_6km_upd; done */

	      /* #### */
	      if ( cov_v_prof ) {
		for (k = 0; k < 1; k++) {
			tmp_int = (t_prof_S_p[0]/cov_v_prof - 185.) / 0.6 + 0.5;
			if (tmp_int < 0)	tmp_int = 0;
			if (tmp_int > 255)	tmp_int = 255;
			upd_lst_w_p->ta_day = tmp_int;
		}
	      }

	      {
		int32	n_snow = 0;
		int32	n_ice = 0;
		int32	ii, jj;

		for ( ii = 0; ii < 6; ii++ ) {
		    upd_lst_w_p_snow = &upd_a_lst_w_p_snow->_upd_for_lst[ii];
		    upd_lst_w_p_ice = &upd_a_lst_w_p_ice->_upd_for_lst[ii];
		    for ( jj = 0; jj < 6; jj++ ) {
			if ( CHK_BIT (upd_lst_w_p_snow->snow_bits, jj) )
				n_snow += 1;
			if ( CHK_BIT (upd_lst_w_p_ice->lakeice_bits, jj) )
				n_ice += 1;
		    }
		}
		if ( ( n_snow / n_pix_land_type_sum ) > 0.8 )
			i_grid_land_type = 15;
		if ( ( n_ice / n_pix_land_type_sum ) > 0.8 )
			i_grid_land_type = 21;
	      }

	      upd_lst_w_p->landcov |= (uint8) i_grid_land_type;

	      if ( i_lake_pix || i_grid_land_type == 21 ) {
					/* in-land water or lake ice */

			upd_lst_w_p->landcov |= 0100;   /* lake grid */
	      }
	      else if ( upd_lst_w_p->landcov & 0200 )
			upd_lst_w_p->landcov &= ~0200;

	      /* ##### to set b7_alpha */
	      if ( i_brdf ) {
			brdf_p_p = brdf_p_6km[i_6km][j_6km][0];
			brdf_6km[0] = brdf_p_p[0];
			brdf_6km[1] = brdf_p_p[1];
			brdf_6km[2] = brdf_p_p[2];
	      }

	      upd_lst_w_p->b7_alpha = 0;

	      if ( i_brdf || upd_lst_w_p->landcov > 63 ) {

	       {
		double  theta_v, theta_s, r_azm, scat_a;
		double  theta_vp, theta_sp, scat_ap, cos_dazm, sin_dazm;
		double  cos_v, cos_s, sin_v, sin_s, cos_a, sin_a;
		double  cos_vp, cos_sp, sin_vp, sin_sp, cos_ap, sin_ap;
		double  ang_t, cos_t, sin_t;
		double  tan_vp, tan_sp;
		double  k_vol, k_geo;
		double  ratio_h_to_b = 2.0;
		double  ratio_b_to_r = 1.0;
		double  d_square, b7_alpha;
		double  theta_v2, theta_v3;
		double  black_sky_albedo;

		r_azm = DTR(0.02 * d_azm_6km[i_6km][j_6km]);
		cos_dazm = cos(r_azm);  sin_dazm = sin(r_azm);
		theta_v = DTR(0.5 * upd_lst_w_p->view_zenith_day);
		theta_v2 = theta_v * theta_v;
		theta_v3 = theta_v * theta_v2;
		theta_s = DTR(0.5 * upd_lst_w_p->sola_zenith);

	if ( upd_lst_w_p->landcov < 64 ) {

	     b7_alpha = 0.0;

	     if ( brdf_6km[0] && brdf_6km[0] < 32767 ) { /* in valid range */

		black_sky_albedo = 0.001 * brdf_6km[0] + 0.001 * brdf_6km[1] *
		   (0.307588 * theta_v3 - 0.070987 * theta_v2 - 0.007574) +
		   0.001*brdf_6km[2] * (0.04184 * theta_v3 - 0.166314 * theta_v2
		    - 1.284090);
		black_sky_albedo /= 0.005;
		if ( black_sky_albedo < 0.0 )   black_sky_albedo = 0.0;
		if ( black_sky_albedo > 255.0 ) black_sky_albedo = 255.0;
		/* upd_lst_w_p->b7_black_sky_albedo = black_sky_albedo; */

		if ( fabs(black_sky_albedo) > SMALL_VALUE ) {
		 
		 cos_v = cos(theta_v);	 cos_s = cos(theta_s);
		 sin_v = sin(theta_v);   sin_s = sin(theta_s);
		 cos_a = cos_v * cos_s + sin_v * sin_s * cos_dazm;
		 scat_a = acos(cos_a);
		 sin_a = sin(scat_a);

		 theta_vp = atan(ratio_b_to_r * tan(theta_v));
		 theta_sp = atan(ratio_b_to_r * tan(theta_s));
		 cos_vp = cos(theta_vp); cos_sp = cos(theta_sp);
		 sin_vp = sin(theta_vp); sin_sp = sin(theta_sp);
		 tan_vp = tan(theta_vp); tan_sp = tan(theta_sp);
		 cos_ap = cos_vp * cos_sp + sin_vp * sin_sp * cos_dazm;
		 scat_ap = acos(cos_ap);
		 sin_ap = sin(scat_ap);
		 d_square = tan_sp * tan_sp + tan_vp * tan_vp -
			2.0 * tan_sp * tan_vp * cos_dazm;
		 tmp = tan_sp * tan_vp * sin_dazm;
		 tmp = d_square + tmp * tmp;
		 cos_t = ratio_h_to_b * sqrt(tmp) / (1.0/cos_sp + 1.0/cos_vp);
		 ang_t = acos(cos_t);
		 sin_t = sin(ang_t);
		 k_geo = (ang_t - sin_t * cos_t)*(1.0/cos_sp + 1.0/cos_vp) / PI
			- 1.0/cos_sp - 1.0/cos_vp
			+ 0.5 * (1 + cos_ap) / (cos_sp * cos_vp);
		 k_vol = ((0.5 * PI - scat_a) * cos_a + sin_a) / (cos_s + cos_v)
			- 0.25 * PI;

		 tmp = 0.001 * (brdf_6km[0] + brdf_6km[1] * k_vol
			+ brdf_6km[2] * k_geo);
		 b7_alpha = tmp / (0.005 * black_sky_albedo);
		 if ( b7_alpha < 0.16 )	b7_alpha = 0.16;
		}
		n_b7_alpha += 1;
	     }
	}
	else {  /* over lake pixels */
		if ( upd_lst_w_p->landcov >= 128 ) { /* in sunglint */
			k_geo = cos (theta_v - theta_s);
			tmp = cos_dazm * cos_dazm * cos_dazm * cos_dazm;
			tmp *= k_geo * k_geo * k_geo * k_geo;
			b7_alpha = 6.0 * tmp;
			if ( b7_alpha < 0.3 )   b7_alpha = 0.3;
			n_6km_water_sunglint += 1;
		}
		else {
			b7_alpha = 0.3;
			n_6km_water += 1;
		}
	}
		 b7_alpha /= 0.0002;
		 if ( b7_alpha < 0.0 )   b7_alpha = 0.0;
		 if ( b7_alpha > 65535 ) b7_alpha = 65535;

		 upd_lst_w_p->b7_alpha = (uint16) (b7_alpha + 0.5);
	       } /* internal loop */
	      }
	   }
	   else		upd_cov_d_p[i_upd_ang] = 0.0;
	 }
	} /* there is something */
    } /* j_6km loop */
  } /* i_6km loop */

  i_1st_loop = 0;

n_stops[7] += 1;

 } /* end of the i_scan_loop */

  /* update and write update_for_lst */

  if ((update_handle = openMODISfile (upd_fname, "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname);

  /* update upd according to data quality and solar/view angle */

  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

     for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	upd_a_lst_S_svp = &upd_ang_lst_Sv[i_6km][j_6km]; /* saved previously */

	upd_a_lst_w_p1 = &upd_ang_lst_W[i_6km][j_6km];

	for ( i_day_nit = 0; i_day_nit < 2; i_day_nit++ ) {

	 for ( i_upd_ang = 0; i_upd_ang < n2_upd_angs; i_upd_ang++ ) {

	  upd_lst_w_p1 = &upd_a_lst_w_p1->_upd_for_lst[i_upd_ang];
	  upd_lst_S_svp = &upd_a_lst_S_svp->_upd_for_lst_sv[i_upd_ang];

	  if ( ! i_day_nit && !(upd_lst_w_p1->nit_status & NEW_EXIST_BITS) )
		continue;

	  if ( i_day_nit && !(upd_lst_w_p1->day_status & NEW_EXIST_BITS) )
		continue;

	  i_day_of_yr_d = GET_DAY(upd_lst_S_svp->day_date_and_view_time);
	  i_day_of_yr_n = GET_DAY(upd_lst_S_svp->nit_date_and_view_time);

	 if ( i_day_nit ) {

	  cov_previous = (float32) upd_lst_S_svp->upd_cov_d;
	  cov_current =  (float32) upd_lst_w_p1->upd_cov_d;

	  if ( (upd_lst_w_p1->day_status & EXIST_BIT) &&
	     ( i_day_of_yr_d != i_day_of_yr_curr ||
	     !(upd_lst_S_svp->day_status & NEW_BIT) ||
	     i_day_of_yr_d == i_day_of_yr_curr &&
	     (upd_lst_S_svp->day_status & NEW_BIT) &&
  ((50.0 * upd_lst_w_p1->tb_day[7] > upd_lst_S_svp->tb_day[7] &&
  ((cov_current > 1.25 * cov_previous) || ((cov_current > 0.75 * cov_previous)
  && upd_lst_w_p1->sola_zenith < upd_lst_S_svp->sola_zenith))) ||
   ((cov_current > 0.75 * cov_previous) &&
     50.0 * upd_lst_w_p1->tb_day[7] > (upd_lst_S_svp->tb_day[7] + 100)) )) ) {

     upd_lst_S_svp->day_date_and_view_time = upd_lst_w_p1->day_date_and_view_time;

	   upd_lst_S_svp->day_status = upd_lst_w_p1->day_status;

	   u8_qa_day = u8getbits(upd_lst_w_p1->QA_bits, 7, 4);
	   u8setbits(&upd_lst_S_svp->QA_bits, 7, 4, u8_qa_day);
	   upd_lst_S_svp->upd_cov_d = upd_lst_w_p1->upd_cov_d;
	   upd_lst_S_svp->view_zenith_day = upd_lst_w_p1->view_zenith_day;
	   upd_lst_S_svp->sola_zenith = upd_lst_w_p1->sola_zenith;
	   for (k = 0; k < 8; k++)
	     upd_lst_S_svp->tb_day[k] = 50.0 * upd_lst_w_p1->tb_day[k] + 0.5;
	   upd_lst_S_svp->averaged_em[6] = upd_lst_w_p1->reserved[0];
	   upd_lst_S_svp->cwv_day = 20.0 * upd_lst_w_p1->cwv_day + 0.5;
	   upd_lst_S_svp->sf_elev = upd_lst_w_p1->sf_elev;
	   upd_lst_S_svp->ta_day = upd_lst_w_p1->ta_day;
	   upd_lst_S_svp->b7_alpha = upd_lst_w_p1->b7_alpha;
	   upd_lst_S_svp->landcov = upd_lst_w_p1->landcov;
	   i_data_yr_d = u8getbits(upd_lst_w_p1->i_data_yr_dn, 7, 4);
	   u8setbits(&upd_lst_S_svp->i_data_yr_dn, 7, 4, i_data_yr_d);

	   /* snow and ice_bits are always kept in the regular grids */
	   upd_a_lst_w_p1 = &upd_ang_lst_W[i_6km][j_6km];
	   upd_a_lst_S_svp = &upd_ang_lst_Sv[i_6km][j_6km];
	   for ( kn = 0; kn < 6; kn++ ) {
	     upd_lst_S_svp_snow = &upd_a_lst_S_svp->_upd_for_lst_sv[kn];
	     upd_lst_S_svp_ice = &upd_a_lst_S_svp->_upd_for_lst_sv[kn];
	     upd_lst_w_p1_snow = &upd_a_lst_w_p1->_upd_for_lst[kn];
	     upd_lst_w_p1_ice = &upd_a_lst_w_p1->_upd_for_lst[kn];
	     upd_lst_S_svp_snow->snow_bits = upd_lst_w_p1_snow->snow_bits;
	     upd_lst_S_svp_ice->lakeice_bits = upd_lst_w_p1_ice->lakeice_bits;
	   }
	  }
	 }
	 else {

	  cov_previous = upd_lst_S_svp->upd_cov_n;
	  cov_current =  upd_lst_w_p1->upd_cov_n;

	  if ( (upd_lst_w_p1->nit_status & EXIST_BIT) &&
	     ( i_day_of_yr_n != i_day_of_yr_curr ||
	     !(upd_lst_S_svp->nit_status & NEW_BIT) ||
	     i_day_of_yr_n == i_day_of_yr_curr &&
	     (upd_lst_S_svp->nit_status & NEW_BIT) &&
  ((50.0 * upd_lst_w_p1->tb_nit[7] > upd_lst_S_svp->tb_nit[7]) &&
  ((cov_current > 1.5 * cov_previous) || ((cov_current > 0.75 *cov_previous)
  && (upd_lst_w_p1->view_zenith_nit < upd_lst_S_svp->view_zenith_nit))) ||
  ((cov_current > 0.75 *cov_previous) &&
    50.0 * upd_lst_w_p1->tb_nit[7] > (upd_lst_S_svp->tb_nit[7] + 100)) )) ) {

     upd_lst_S_svp->nit_date_and_view_time = upd_lst_w_p1->nit_date_and_view_time;

	   upd_lst_S_svp->nit_status = upd_lst_w_p1->nit_status;

	   u8_qa_nit = u8getbits(upd_lst_w_p1->QA_bits, 3, 4);
	   u8setbits(&upd_lst_S_svp->QA_bits, 3, 4, u8_qa_nit);
	   upd_lst_S_svp->upd_cov_n = upd_lst_w_p1->upd_cov_n;
	   upd_lst_S_svp->view_zenith_nit = upd_lst_w_p1->view_zenith_nit;
	   for (k = 0; k < 8; k++)
	     upd_lst_S_svp->tb_nit[k] = 50.0 * upd_lst_w_p1->tb_nit[k] + 0.5;
	   upd_lst_S_svp->cwv_nit = 20.0 * upd_lst_w_p1->cwv_nit + 0.5;
	   upd_lst_S_svp->sf_elev = upd_lst_w_p1->sf_elev;
	   upd_lst_S_svp->ta_nit = upd_lst_w_p1->ta_nit;
	   i_data_yr_n = u8getbits(upd_lst_w_p1->i_data_yr_dn, 3, 4);
	   u8setbits(&upd_lst_S_svp->i_data_yr_dn, 3, 4, i_data_yr_n);
	  }
	 }
	} /* i_upd_ang loop */
       } /* i_day_nit loop */
     } /* end of j_6km loop */

n_stops[8] += 1;

  } /* end of i_6km loop */

  upd_sta[0] = 0;        upd_dims[0] = nrow_6km;

  ier = putMODISarray(update_handle,upd_lst_6km[0].name,upd_lst_6km[0].gname,
	upd_sta, upd_dims, &upd_ang_lst_Sv[0][0]);
  if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray to upd_ang_lst");

  /* write metadata to the update_for_lst file */
  if ( !n_old_update )
      DoMetaData( n_old_update, (int32) 3,
        (PGSt_PC_Logical) PCLogupdate[i_days_in_cycle_update-1],
	(PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, update_handle);

  if ( closeMODISfile(&update_handle) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "update_for_lst file failed");

  /* write metadata to the 1km L3 output file */
  if ((lst_1km_l3_outf = openMODISfile (fname_1km_L3, "a")) == NULL)
        HALTsmf(MODIS_F_OPEN_HDF_FILE,fname_1km_L3);

  sta_1km_o[0] = 0;	dims_1km_o[0] = ii1;
  getMODISarray(lst_1km_l3_outf, lst_1km_l3[1].name, lst_1km_l3[1].gname,
                sta_1km_o, dims_1km_o, &qc_1km_l3d[0][0]);
  getMODISarray(lst_1km_l3_outf, lst_1km_l3[5].name, lst_1km_l3[5].gname,
                sta_1km_o, dims_1km_o, &qc_1km_l3n[0][0]);

  for (i = 0; i < ii1; i++) {
      for (j = 0; j < lkk; j++) {

	    u8_qa_1km_man = u8getbits (qc_1km_l3d[i][j], 1, 2);
	    total_l3_1km[u8_qa_1km_man] += 1;

	    u8_qa_1km_man = u8getbits (qc_1km_l3n[i][j], 1, 2);
	    total_l3_1km[u8_qa_1km_man] += 1;

      }
  }
  met_cnt_1km.QA_GoodQuality = total_l3_1km[0];
  met_cnt_1km.QA_OtherQuality = total_l3_1km[1];
  met_cnt_1km.QA_NotProducedCloud = total_l3_1km[2];
  met_cnt_1km.QA_NotProducedOther = total_l3_1km[3];
  /* It is necessary to use the following formula because the tile has not
     been completely covered by input granules */
  met_cnt_1km.QA_NotProducedOther = (met_cnt_1km.AllPixelsProcessed -
			total_l3_1km[0] - total_l3_1km[1] - total_l3_1km[2]);
  if ( met_cnt_1km.QA_NotProducedOther < 0 )
	met_cnt_1km.QA_NotProducedOther = 0;

  sec_to_hrs( &tile_begin_seconds, &tile_hours, &tile_minutes);
  sprintf (&time_begin[0][0], "%02i:%02i:%09.6f",
	tile_hours, tile_minutes, tile_begin_seconds);
  sec_to_hrs( &tile_end_seconds, &tile_hours, &tile_minutes);
  sprintf (&time_end[0][0], "%02i:%02i:%09.6f",
	tile_hours, tile_minutes, tile_end_seconds);

 DoMetaData( n_old_1km, (int32) 3, (PGSt_PC_Logical) PCLogL3_1km,
	(PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, lst_1km_l3_outf);
 if ( closeMODISfile(&lst_1km_l3_outf) == -1 )
		HALTsmf(MODIS_E_CLOSE_HDF_FILE, "1km L3 output file failed");

  /* write metadata to the 6km L3 output file */

  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

    sta_6km_o[0] = i_6km;
    getMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name, lst_6km_l3[1].gname,
		sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
    getMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name, lst_6km_l3[5].gname,
		sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);

    for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	u8_1 = QC_6km_day[0][j_6km] & u8_3cons;
	u8_2 = QC_6km_nit[0][j_6km] & u8_3cons;

	total_l3_6km_dn[u8_1] += 1;
	total_l3_6km_dn[u8_2] += 1;
    }
  }

  met_cnt_6km.QA_GoodQuality = total_l3_6km_dn[0];
  met_cnt_6km.QA_OtherQuality = total_l3_6km_dn[1];
  met_cnt_6km.QA_NotProducedCloud = total_l3_6km_dn[2];
  met_cnt_6km.QA_NotProducedOther = total_l3_6km_dn[3];
  /* It is necessary to use the following formula because the tile has not
     been completely covered by input granules */
  met_cnt_6km.QA_NotProducedOther = (met_cnt_6km.AllPixelsProcessed -
	total_l3_6km_dn[0] - total_l3_6km_dn[1] - total_l3_6km_dn[2]);
  if ( met_cnt_6km.QA_NotProducedOther < 0 )
	met_cnt_6km.QA_NotProducedOther = 0;

  DoMetaData( n_6km_l3_old_file, (int32) 3, (PGSt_PC_Logical) PCLogL3_6km, 
	(PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, lst_6km_l3_outf);

  if ( closeMODISfile(&lst_6km_l3_outf) == -1 )
		HALTsmf(MODIS_E_CLOSE_HDF_FILE, "6km L3 output file failed");

 if ( !n_old_1km ) {

  /* Open the file and initiate the SD interface */

  f_id = SDstart(fname_1km_L3, DFACC_WRITE);
  if ( f_id == FAIL )
    HALTsmf(MODIS_E_HDF_FAIL, "SDstart(fname_1km_L3, DFACC_WRITE)");

  /* ---- Set attributes for LST_1KM SDSs ----- */

  {
    int32       i_offset, n_chars, x = 5;
    double	dummy;

    for ( i = 0; i < NSD_LST_1KM_L3; i++) {

      SDsetdim_LST (f_id, lst_1km_l3[i], &sds_id);

/* skip the next step for V2
      SDsetattr_LST (sds_id, "Description", DFNT_CHAR8,
		     strlen(descrip_1km_l3[i]), 
		     (VOIDP) descrip_1km_l3[i]);
*/
      if (strcmp (lparam_1km_l3[i].units, "none"))
	istat = SDsetdatastrs (sds_id, lparam_1km_l3[i].long_name,
			(VOIDP) lparam_1km_l3[i].units, NULL, NULL);
      else
	istat = SDsetdatastrs (sds_id, lparam_1km_l3[i].long_name, NULL,
			     NULL, NULL);

      if ( istat == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDsetdatastrs, LST_Day");

      SDsetattr_LST (sds_id, "Number Type", DFNT_CHAR8, 
		     strlen(lst_1km_l3[i].type), 
		     (VOIDP) lst_1km_l3[i].type);

/* skip the next two steps for V2
      SDsetattr_LST (sds_id, "Rank", DFNT_INT32, 1, &lst_1km_l3[i].rank);
      SDsetattr_LST (sds_id, "Occurrence", DFNT_CHAR8, 6, (VOIDP) "always");
*/
      SDsetattr_LST_V (sds_id, "valid_range", lparam_1km_l3[i].type, 2, 
		     lparam_1km_l3[i].vrange); 
      if ( lparam_1km_l3[i].fillv > -1.0 || lparam_1km_l3[i].fillv < -2.0 )
         SDsetattr_LST_V (sds_id, "_FillValue", lparam_1km_l3[i].type, 1,
		     &lparam_1km_l3[i].fillv); 


      if ( cal_1km_l3[i].valid ) {

	i_offset = ( fabs (cal_1km_l3[i].offset) > SMALL_VALUE ) ? 1 : 0;
	n_chars = strlen (dataconvt_1km_l3[2*i+1]);
	if ( ! i_offset ) n_chars -= 13;
	SDsetattr_LST (sds_id, dataconvt_1km_l3[2*i], DFNT_CHAR8, 
		       n_chars, (VOIDP) dataconvt_1km_l3[2*i+1] );
  
       if ( cal_1km_l3[i].sd_type == 1 ) {
	SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_1km_l3[i].scale);
	if ( i_offset )
		SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_1km_l3[i].offset);
       }
       else if ( cal_1km_l3[i].sd_type == 2 ) {
	  SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
			 (VOIDP) &cal_1km_l3[i].scale);
	  if ( i_offset )
		SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1, 
			 (VOIDP) &cal_1km_l3[i].offset);
       }
       if ( cal_1km_l3[i].sd_type == 1  ||  cal_1km_l3[i].sd_type == 2 ) {
	dummy = 0.;
	SDsetattr_LST (sds_id,"scale_factor_err",DFNT_FLOAT64,1,(VOIDP) &dummy);
	SDsetattr_LST (sds_id,"add_offset_err", DFNT_FLOAT64,1, (VOIDP) &dummy);
	SDsetattr_LST (sds_id, "calibrated_nt", DFNT_INT32,1, (VOIDP) &x);
       }
      }

      /* Terminate access the the array */

      if ( SDendaccess (sds_id) == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDendaccess(sds_id)");
    }
  }

  /* Terminate access to the SD interface and close the file */

  if ( SDend (f_id) == -1 )
    HALTsmf(MODIS_E_HDF_FAIL, "SDend (f_id)");
 }

 if (!n_6km_l3_old_file) {

  /* Open the file and initiate the SD interface */

  f_id = SDstart(fname_6km_L3, DFACC_WRITE);
  if ( f_id == FAIL )
     HALTsmf(MODIS_E_HDF_FAIL, "SDstart(fname_6km_L3, DFACC_WRITE)");

  /* ---- Set attributes for LST_6KM SDSs ----- */

  {
    int32	i_offset, n_chars, x = 5;
    double	dummy;

    for ( i = 0; i < NSD_LST_6KM_L3; i++) {

      SDsetdim_LST (f_id, lst_6km_l3[i], &sds_id);

/* skip the next step for V2
      SDsetattr_LST (sds_id, "Description", DFNT_CHAR8,
		     strlen(descrip_6km_l3[i]), 
		     (VOIDP) descrip_6km_l3[i]);
*/
      if (strcmp (lparam_6km_l3[i].units, "none"))
	istat = SDsetdatastrs (sds_id, lparam_6km_l3[i].long_name,
			(VOIDP) lparam_6km_l3[i].units, NULL, NULL);
      else
	istat = SDsetdatastrs (sds_id, lparam_6km_l3[i].long_name, NULL,
			     NULL, NULL);

      if ( istat == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDsetdatastrs, LST_6KM_L3");

      SDsetattr_LST (sds_id, "Number Type", DFNT_CHAR8, 
		     strlen(lst_6km_l3[i].type), 
		     (VOIDP) lst_6km_l3[i].type);

/* skip the next two steps for V2
      SDsetattr_LST (sds_id, "Rank", DFNT_INT32, 1, &lst_6km_l3[i].rank);
      SDsetattr_LST (sds_id, "Occurrence", DFNT_CHAR8, 6, (VOIDP) "always");
*/
      SDsetattr_LST_V (sds_id, "valid_range", lparam_6km_l3[i].type, 2, 
		     lparam_6km_l3[i].vrange); 
      if ( lparam_6km_l3[i].fillv > -1.0 || lparam_6km_l3[i].fillv < -2.0 )
         SDsetattr_LST_V (sds_id, "_FillValue", lparam_6km_l3[i].type, 1,
		     &lparam_6km_l3[i].fillv);

      if ( cal_6km_l3[i].valid ) {

	i_offset = ( fabs (cal_6km_l3[i].offset) > SMALL_VALUE ) ? 1 : 0;
	n_chars = strlen (dataconvt_6km_l3[2*i+1]);
	if ( ! i_offset ) n_chars -= 13;
	SDsetattr_LST (sds_id, dataconvt_6km_l3[2*i], DFNT_CHAR8, 
		       n_chars, (VOIDP) dataconvt_6km_l3[2*i+1] );
  
        if ( cal_6km_l3[i].sd_type == 1 ) {
	   SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_6km_l3[i].scale);
	   SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1,
		       (VOIDP) &cal_6km_l3[i].offset);
        }
        else if ( cal_6km_l3[i].sd_type == 2 ) {
	    SDsetattr_LST (sds_id, "scale_factor", DFNT_FLOAT64, 1,
			 (VOIDP) &cal_6km_l3[i].scale);
	    SDsetattr_LST (sds_id, "add_offset", DFNT_FLOAT64, 1, 
			 (VOIDP) &cal_6km_l3[i].offset);
        }
       if ( cal_6km_l3[i].sd_type == 1  ||  cal_6km_l3[i].sd_type == 2 ) {
	dummy = 0.;
	SDsetattr_LST (sds_id,"scale_factor_err",DFNT_FLOAT64,1,(VOIDP) &dummy);
	SDsetattr_LST (sds_id,"add_offset_err", DFNT_FLOAT64,1, (VOIDP) &dummy);
	SDsetattr_LST (sds_id, "calibrated_nt", DFNT_INT32,1, (VOIDP) &x);
       }
     }
     /* Terminate access the the array */
     if ( SDendaccess (sds_id) == -1 )
	HALTsmf(MODIS_E_HDF_FAIL, "SDendaccess(sds_id)");
    }
  }

  /* Terminate access to the SD interface and close the file */

  if ( SDend (f_id) == -1 )
    HALTsmf(MODIS_E_HDF_FAIL, "SDend (f_id)");

 }

  return;

} /* End of the Granule_tile function */

float64 f_mod_rad7_inv (int32 iband, float32 rad1, int32 i_mod)
{
/*
!C***************************************************************************

!Description:
   This function calculates brightness temperature of MODIS band radiance

!Input Parameters:	iband -	MODIS band index (0-6 for bands 20, 22, 23, 29,
				31-33)
			rad1 -  band radiance

			i_mod - 1 for Aqua FM1 MODIS data

!Output Parameters:	none

!Revision History:

Returns:

	tb - brightness temperature.

References and Credits:

   Version V2.1  2002/09/08  11:40     wan

   Version V1.1  2000/05/15  02:02:41  wan

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes
	look-up table method and quick bineary search are used in
	this function.

!END***********************************************************************/

	int32   i, j, k, m;
	float64	tb;

	i = 0;
	j = NCELM1;

	if ( i_mod ) {
	   while ((m = j - i) != 1) {
		k = (i + j) / 2;
		if (rad1 >= _mod_fm1_rad7[iband][k])	i = k;
		else 					j = k;
	   }
	   tb = TMIN + DT_LUT * (i + (rad1 - _mod_fm1_rad7[iband][i]) /
			(_mod_fm1_rad7[iband][j] - _mod_fm1_rad7[iband][i]));
	}
	else {
	   while ((m = j - i) != 1) {
		k = (i + j) / 2;
		if (rad1 >= _mod_rad7[iband][k])	i = k;
		else 					j = k;
	   }
	   tb = TMIN + DT_LUT * (i + (rad1 - _mod_rad7[iband][i]) /
			(_mod_rad7[iband][j] - _mod_rad7[iband][i]));
	}

	return (tb);
}

void
poly_intp_vector(float64 *x, int32 i, float64 xx, int32 k, float64 **y, float64 *z)
{
/*
!C***************************************************************************

!Description:
This function makes three-point polyminal interpolation for a vector variables

!Input Parameters:	x 	pointer to X axis
			i 	number of x elements
			k 	number of elements for output
			xx 	value at which interpolation will be made
			y 	2-d array for Y values at x's
		     
!Output Parameters:	z 	pointer to interpolated vector values


!Revision History:

Returns:

   none

References and Credits:

   Version B3.2 09-11-95

   Written by:

        Zhengming Wan
        UCSB/ICESS
        wan@icess.ucsb.edu

!Team-unique Header:
         This software is developed by MODIS Science Team member
	 Zhengming Wan for the National Aeronautics and Space Administration,
         Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  int32		        i1, i2, n;
  static float64	x_sv[3] = { 0., 0., 0. };
  static float64	a1, a2, a3;
  static float64	f1, f2, f3;
  float64		x0, x1, x2;
  float64		*y_p;

  i1 = i + 1;	i2 = i + 2;

  if (  fabs (x[i] - x_sv[0]) > SMALL_VALUE ||
	fabs (x[i1] - x_sv[1]) > SMALL_VALUE ||
	fabs (x[i2] - x_sv[2]) > SMALL_VALUE ) {
    x_sv[0] = x[i];
    x_sv[1] = x[i1];
    x_sv[2] = x[i2];
    a1 = 1. / (x[i] - x[i2]);
    a2 = 1. / (x[i1] - x[i]);
    a3 = 1. / (x[i1] - x[i2]);
    f1 = - a1 * a2;
    f2 = a2 * a3;
    f3 = a1 * a3;
  }
  
  x0 = xx - x[i];
  x1 = xx - x[i1];
  x2 = xx - x[i2];
  
  for (n = 0; n < k; n++) {
    y_p = &y[n][0];
    z[n] = y_p[i] * f1 * x1 * x2 + y_p[i1] * f2 * x0 * x2 +
      y_p[i2] * f3 * x0 * x1;
  }
}

char **
alloc_2d (int32 rows, int32 cols, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 2-d array.

!Input Parameters:	rows 	the first dimension
			cols 	the second dimension
			elsize 	element size in bytes
		     
!Output Parameters:	xh 	pointer to the array


!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

        Zhengming Wan
        UCSB/ICESS
        wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **xh = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	sp = sizeof(char *);
  int32	j = 0;
  size_t  siz;
  
  cols *= elsize; 	/*  cols is now row-size in bytes	*/
  siz = rows * cols;
  
  /*  buf is pointer to data area itself	*/
  /*  xh is pointer to row addresses	*/
  if ((buf = (char *) malloc (siz)) == NULL) {
    sprintf (modstring, "alloc_2d: can't allocate buffer of size %lu", siz);
    HALTsmf(MODIS_E_CALLOC, modstring);
  }

total_space += siz;

  /*  make xh for pointer vector */
  if ((xh = (char **) malloc(rows * sp)) == NULL) {
    free (buf);
    HALTsmf(MODIS_E_CALLOC, "alloc_2d: can't allocate pointer vector");
  }

total_space += rows * sp;

  /* set pointers to the beginning of each row */
  for (j = 0; j < rows; j++)
    xh[j] = buf + j*cols;
  
  return(xh);
}

char ***
alloc_3d (int32 dim1, int32 dim2, int32 dim3, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 3-d array.

!Input Parameters:	dim1, dim2, dim3 	dimensions
			elsize 			element size in bytes
		     
!Output Parameters:	x1d 	pointer to the array


!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.


!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **x2d = NULL;
  char  ***x1d = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	j=0;
  int32	sp = sizeof(char *);
  size_t	siz;
  
  dim3 *= elsize; 	/*  dim3 is now row-size in bytes */

  siz = dim1 * dim2 * dim3;

  /*  buf is pointer to data area itself	*/
  if ((buf = (char *) malloc(siz)) == NULL) {
     sprintf (modstring, "alloc_3d: can't allocate buffer of size %lu", siz);
     HALTsmf(MODIS_E_CALLOC, modstring);
  }

total_space += siz;

  /*  make x2d for pointer vector for the 2nd dimension */
  if ((x2d = (char **) malloc(dim1 * dim2 * sp)) == NULL) {
     free(buf);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 2nd dim");
  }

total_space += dim1 * dim2 * sp;

  /*  make x1d for pointer vector for the 1st dimemsion */
  if ((x1d = (char ***) malloc(dim1 * sp)) == NULL) {
     free(buf);	free(x2d[0]);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 1st dim");
  }

total_space += dim1 * sp;

  /* set the 2nd dimension pointers */
  for (j = 0; j < dim1 * dim2; j++)
    x2d[j] = buf + j*dim3;

  /* set the 1st dimension pointers */
  for (j = 0; j < dim1; j++)
    x1d[j] = x2d + j * dim2;
  
  return(x1d);
}

void free_lst_space(void) 
{
/*
!C***************************************************************************

!Description:
This function frees all allocated dynamic memories normally or abnormally 

!Input Parameters:	none
		     
!Output Parameters:	none


!Revision History:

   Version B3.2 09-11-95

Returns:                none

References and Credits:

   Version V1.1 07-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  int32    k, k1 = 0;

  if ( gran_order ) { free(gran_order[0]); free(gran_order); gran_order = NULL; }

  if ( i_day_scan ) {	free (i_day_scan); i_day_scan = NULL; }

  if ( i_nit_scan ) {	free (i_nit_scan); i_nit_scan = NULL; }

  if ( i_scan_mode ) {	free (i_scan_mode); i_scan_mode = NULL; }

  if ( i_cld_scan ) {	free (i_cld_scan); i_cld_scan = NULL; }

  if ( snowcv )	    { free (snowcv[0]); free (snowcv); snowcv = NULL; }
  if ( snowcv_old ) { free (snowcv_old[0]); free (snowcv_old); snowcv_old = NULL; }

  if ( lakeice )    { free (lakeice[0]); free (lakeice); lakeice = NULL; }
  if ( lakeice_old ) { free (lakeice_old[0]); free (lakeice_old); lakeice_old = NULL; }

  if ( cmask ) { free (cmask[0][0]); free (cmask[0]); free (cmask); cmask = NULL; }

  if ( cmask_prof ) { free (cmask_prof[0]); free (cmask_prof); cmask_prof  = NULL; }

  if ( ts_prof ) { free (ts_prof[0]); free (ts_prof); ts_prof = NULL; }

  if ( cwv_prof ) { free (cwv_prof[0]); free (cwv_prof); cwv_prof = NULL; }

  if ( v_time_6km_l3_Sv ) { free (v_time_6km_l3_Sv[0]);
	free (v_time_6km_l3_Sv); v_time_6km_l3_Sv = NULL; }

  if ( v_angl_6km_l3_Sv ) { free (v_angl_6km_l3_Sv[0]);
	free (v_angl_6km_l3_Sv); v_angl_6km_l3_Sv = NULL; }

  if ( upd_cov_d ) { free (upd_cov_d[0][0]);
	free (upd_cov_d[0]); free (upd_cov_d); upd_cov_d = NULL; }

  if ( upd_cov_n ) { free (upd_cov_n[0][0]);
	free (upd_cov_n[0]); free (upd_cov_n); upd_cov_n = NULL; }

  for ( k = 0; k < N2_VIEW_ANGS; k++ ) {
    if( t_prof_sum_d[k] ) { free(t_prof_sum_d[k][0][0]); free(t_prof_sum_d[k][0]);
	free (t_prof_sum_d[k]); t_prof_sum_d[k] = NULL; }

    if( t_prof_sum_n[k] ) { free(t_prof_sum_n[k][0][0]); free(t_prof_sum_n[k][0]);
	free (t_prof_sum_n[k]); t_prof_sum_n[k] = NULL; }
  }

  if ( data ) { free (data[0][0]); free (data[0]); free (data); data = NULL; }

  if ( emissive_uncert_indexes ) {
	free (emissive_uncert_indexes[0][0]); free (emissive_uncert_indexes[0]);
	free (emissive_uncert_indexes); emissive_uncert_indexes = NULL; }

  if ( l1b_qa ) { free (l1b_qa[0]); free (l1b_qa); l1b_qa = NULL; }

  if ( coef_mesh ) { free (coef_mesh[0]); free (coef_mesh); coef_mesh = NULL; }

  if ( land_type ) { free (land_type[0]); free (land_type); land_type = NULL; }

  if ( landcov_qc ) { free (landcov_qc[0][0]); free (landcov_qc[0]);
			free (landcov_qc); landcov_qc = NULL; }

  if ( coeff_p ) { free (coeff_p); coeff_p = NULL; }

  if ( lst_out ) { free (lst_out[0]); free (lst_out); lst_out = NULL; }

  if ( QC_out ) { free (QC_out[0]); free (QC_out); QC_out = NULL; }
  
  if ( err_lst_out ) { free (err_lst_out[0]); free (err_lst_out); err_lst_out = NULL; }
  
  if ( emis31_out ) { free (emis31_out[0]); free (emis31_out); emis31_out = NULL; }

  if ( emis32_out ) { free (emis32_out[0]); free (emis32_out); emis32_out = NULL; }

  if ( view_angl_out ) {
    free (view_angl_out[0]);	free (view_angl_out); view_angl_out = NULL; }

  if ( view_time_out ) {
    free (view_time_out[0]);	free (view_time_out); view_time_out = NULL; }

  if ( ev_start_time ) {	free (ev_start_time); ev_start_time = NULL; }
  if ( dat1_1a ) { free (dat1_1a[0][0]); free (dat1_1a[0]);
		   free (dat1_1a); dat1_1a = NULL; }
  if ( brdf_p ) { free (brdf_p[0][0][0]); free (brdf_p[0][0]);
		free (brdf_p[0]); free (brdf_p); brdf_p = NULL; }
  if ( brdf_p_6km ) { free (brdf_p_6km[0][0][0]); free (brdf_p_6km[0][0]);
		free (brdf_p_6km[0]); free (brdf_p_6km); brdf_p_6km = NULL; }
  if ( brdf_q_6km ) { free (brdf_q_6km[0][0]);
		free (brdf_q_6km[0]); free (brdf_q_6km); brdf_q_6km = NULL; }
  if ( d_azm_6km ) { free (d_azm_6km[0]); free (d_azm_6km); d_azm_6km = NULL; }

  if ( lat )	{ free (lat[0]); free (lat); lat = NULL; }
  if ( lon )	{ free (lon[0]); free (lon); lon = NULL; }
  if ( land_sea_mask_1a ) { free (land_sea_mask_1a[0]); free (land_sea_mask_1a);
			    land_sea_mask_1a = NULL; }
  if ( lat_5pix ) { free (lat_5pix[0]); free (lat_5pix); lat_5pix = NULL; }
  if ( lon_5pix ) { free (lon_5pix[0]); free (lon_5pix); lon_5pix = NULL; }

  if ( set )	{ Lbin_Free(); set = NULL; }

  if ( lst_1km_Sv) { free(lst_1km_Sv[0]); free(lst_1km_Sv); lst_1km_Sv = NULL; }
  if ( QC_1km_l3 ) { free (QC_1km_l3[0]); free (QC_1km_l3); QC_1km_l3 = NULL; }
  if (v_time_1km_l3_Sv) { free (v_time_1km_l3_Sv[0]); free (v_time_1km_l3_Sv);
			  v_time_1km_l3_Sv = NULL; }
  if (v_angl_1km_l3_Sv) { free (v_angl_1km_l3_Sv[0]); free (v_angl_1km_l3_Sv);
			  v_angl_1km_l3_Sv = NULL; }
  if (em31_1km_l3_Sv) { free (em31_1km_l3_Sv[0]); free (em31_1km_l3_Sv);
			em31_1km_l3_Sv = NULL; }
  if (em32_1km_l3_Sv) { free (em32_1km_l3_Sv[0]); free (em32_1km_l3_Sv);
			em32_1km_l3_Sv = NULL; }

  if ( clear_day_cov ) { free (clear_day_cov[0]); free (clear_day_cov);
			 clear_day_cov = NULL; }
  if ( clear_nit_cov ) { free (clear_nit_cov[0]); free (clear_nit_cov);
			 clear_nit_cov = NULL; }

  if ( cov_1km_day_S ) { free (cov_1km_day_S[0]); free (cov_1km_day_S);
			 cov_1km_day_S = NULL; }
  if ( cov_1km_nit_S ) { free (cov_1km_nit_S[0]); free (cov_1km_nit_S);
			 cov_1km_nit_S = NULL; }
  if ( lst_1km_S ) { free(lst_1km_S[0]); free(lst_1km_S); lst_1km_S = NULL; }
  if ( qc_1km_l3d ) { free(qc_1km_l3d[0]); free(qc_1km_l3d); qc_1km_l3d = NULL;}
  if ( qc_1km_l3n ) { free(qc_1km_l3n[0]); free(qc_1km_l3n); qc_1km_l3n = NULL;}
  if (vtime_1km_l3_S) { free (vtime_1km_l3_S[0]); free (vtime_1km_l3_S);
			vtime_1km_l3_S = NULL; }
  if (vangl_1km_l3_S) { free (vangl_1km_l3_S[0]); free (vangl_1km_l3_S);
			vangl_1km_l3_S = NULL; }
  if (em31_1km_l3_S) {  free (em31_1km_l3_S[0]); free (em31_1km_l3_S);
			em31_1km_l3_S = NULL; }
  if (em32_1km_l3_S) {  free (em32_1km_l3_S[0]); free (em32_1km_l3_S);
			em32_1km_l3_S = NULL; }

  if (lst_6km_S_from_1km) { free(lst_6km_S_from_1km[0]);
		free (lst_6km_S_from_1km); lst_6km_S_from_1km = NULL; }

  if (lst_6km_Sd) { free(lst_6km_Sd[0]); free (lst_6km_Sd); lst_6km_Sd = NULL; }
  if ( p_time ) { free(p_time[0]); free (p_time); p_time = NULL; }
  if (QC_6km_day) { free(QC_6km_day[0]); free(QC_6km_day); QC_6km_day = NULL; }
  if (v_time_6km_l3_Sd) { free (v_time_6km_l3_Sd[0]); free (v_time_6km_l3_Sd);
			v_time_6km_l3_Sd = NULL; }
  if (v_angl_6km_l3_Sd) { free (v_angl_6km_l3_Sd[0]); free (v_angl_6km_l3_Sd);
			v_angl_6km_l3_Sd = NULL; }
  if (lst_6km_Sn) { free(lst_6km_Sn[0]); free (lst_6km_Sn); lst_6km_Sn = NULL; }
  if (QC_6km_nit) { free(QC_6km_nit[0]); free (QC_6km_nit); QC_6km_nit = NULL; }
  if (v_time_6km_l3_Sn) { free (v_time_6km_l3_Sn[0]); free (v_time_6km_l3_Sn);
			v_time_6km_l3_Sn = NULL; }
  if (v_angl_6km_l3_Sn) { free (v_angl_6km_l3_Sn[0]); free (v_angl_6km_l3_Sn);
			v_angl_6km_l3_Sn = NULL; }
  if (em20_6km_l3_Sv) { free (em20_6km_l3_Sv[0]); free (em20_6km_l3_Sv);
			em20_6km_l3_Sv = NULL; }
  if (em22_6km_l3_Sv) { free (em22_6km_l3_Sv[0]); free (em22_6km_l3_Sv);
			em22_6km_l3_Sv = NULL; }
  if (em23_6km_l3_Sv) { free (em23_6km_l3_Sv[0]); free (em23_6km_l3_Sv);
			em23_6km_l3_Sv = NULL; }
  if (em29_6km_l3_Sv) { free (em29_6km_l3_Sv[0]); free (em29_6km_l3_Sv);
			em29_6km_l3_Sv = NULL; }
  if (em31_6km_l3_Sv) { free (em31_6km_l3_Sv[0]); free (em31_6km_l3_Sv);
			em31_6km_l3_Sv = NULL; }
  if (em32_6km_l3_Sv) { free (em32_6km_l3_Sv[0]); free (em32_6km_l3_Sv);
			em32_6km_l3_Sv = NULL; }
  if (QC_emis) { free (QC_emis[0]); free (QC_emis); QC_emis = NULL; }
  if (pct_land) { free (pct_land[0]); free (pct_land); pct_land = NULL; }
  if (em31_6km_d) { free(em31_6km_d[0]); free(em31_6km_d); em31_6km_d = NULL; }
  if (em31_6km_n) { free(em31_6km_n[0]); free(em31_6km_n); em31_6km_n = NULL; }
  
  if (cov_prof_d) { free(cov_prof_d[0]); free(cov_prof_d); cov_prof_d = NULL; }

  if (cov_prof_n) { free(cov_prof_n[0]); free(cov_prof_n); cov_prof_n = NULL; }

  if ( upd_ang_lst_W ) { free (upd_ang_lst_W[0]); free (upd_ang_lst_W);
			upd_ang_lst_W = NULL; }
  if ( upd_ang_lst_Sv ) { free (upd_ang_lst_Sv[0]); free (upd_ang_lst_Sv);
			upd_ang_lst_Sv = NULL; }

  if ( slope_6km ) { free(slope_6km[0][0]);free(slope_6km[0]);
	free (slope_6km); slope_6km = NULL; }

  if ( gd ) { free (gd); gd = NULL; }

}

void SDsetdim_LST (int32 f_id, sds_t sds_struct, int32 *sds_id) 
{
/*
!C***************************************************************************

!Description:
This function sets dimension names and returns SD id for LST products

!Input Parameters:	f_id 		file id
			sds_struct 	attribute structure
		     
!Output Parameters:	sds_id 		pointer to attributes id


!Revision History:

   $Log: SDsetdim_LST.c,v $
 * Revision 2.0  1997/09/15  23:55     Z. Wan
 *
 * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 10-21-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  int32	arr_index, dim_id;
  
  arr_index = SDnametoindex (f_id, sds_struct.name);
  *sds_id = SDselect (f_id, arr_index);

  if ( sds_struct.rank > 2 )
    HALTsmf(MODIS_E_ATTR_FAIL, sds_struct.name);

}

void SDsetattr_LST (int32 sd_id, char *name, int32 nt, int32 count, VOIDP data)
/*
!C***************************************************************************

!Description:
   This function sets SDS attribute

!Input Parameters:	sd_id 	file or SDS id number
			name 	pointer to attribute name
			nt 	data type
			count 	number of attributes to be set
			data 	attributes to be set
		     
!Output Parameters:	none


!Revision History:

   * Revision V2.01 1997/11/24	16:14:06 Z.Wan
   * make vrange & fillv to be compatible with field data type.
   *
   * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 09-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{

  if ( SDsetattr (sd_id, name, nt, count, data) == FAIL)
    HALTsmf(MODIS_E_ATTR_FAIL, name);
}

void InitBin(tile_t *tile, int32 itile_size)
/******************************************************************************
!C

!Description: 
  'InitBin' initializes the grid and computes information 
  about the tile.

!Input Parameters:
  itile_size	Tile size; 1 = full, 2 = quarter, 4 = 16th, etc.

!Output Parameters:
  tile		tile information data structure

!Revision History:

!Team-unique Header:

  This software was developed by:
    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05.

!References and Credits:

    Revision 00.03  1995/10/23

    MODIS Science Team Member:
	Christopher O. Justice
	MODIS Land Science Team           University of Maryland
	justice@kratmos.gsfc.nasa.gov     NASA/GSFC Code 923
	phone: 301-286-7372               Greenbelt, MD 20771

    Developers:
	Robert E. Wolfe
	MODIS Land Team Support Group     Hughes STX
	robert.wolfe@gsfc.nasa.gov        7701 Greenbelt Road, #400

  Usage Notes:
    1. All of the information in the tile data structure is written.

!END
*****************************************************************************/
{

  int32 irow, ncol;            /* row index, number of columns per row */
  int32 nest, i;
  double ref_lon = -PI;       /* ref longitude for initialization */

  /* Set up gridtype dependent stuffs.  Also set full gridtype string 
     consistent with meta data dictionary. */

  if (gridtype[0]=='i' && gridtype[1]=='s') {  /* integerized sinusoidal */

    /* strcpy(gridtype,"integerized sinusoidal"); */
    pLbin_Init = &Lbin_Init_isin;
    pLbin_FwdW = &Lbin_FwdW_isin;
    pLbin_Inv = &Lbin_Inv_isin;
    pGenObs = &GenObs;

  } else if (gridtype[0]=='s' && gridtype[1]=='i') {  /* sinusoidal */
    
    /* strcpy(gridtype,"sinusoidal"); */
    pLbin_Init = &Lbin_Init_sin;
    pLbin_FwdW = &Lbin_FwdW_sin;
    pLbin_Inv = &Lbin_Inv_sin;
    pGenObs = &GenObs;

  } else
     Error("invalid grid type", "InitBin");

  nest = 1;

  (*pLbin_Init)(0, ref_lon, nest, (int16) (NTILE_HORIZ * itile_size),
		(int16) (NTILE_VERT *  itile_size),
		tile->itile_horiz, tile->itile_vert);
  
  /* Gather information about the tile: start row, number of rows, 
   * end row and number of pixels per row */

  tile->irow_start = set->irow_start;
  tile->nrow = set->nrow_tile;
  tile->irow_end = tile->irow_start + tile->nrow - 1;
  tile->npix = set->npix;

  for (irow = 0; irow < tile->nrow; irow++) {

    /* Calculate for the tile: start column, end column and start pixel.
       If ncol==0, icol_start = icol_end -1 */

    ncol = set->row[irow].ncol_tile;
    set->row[irow].icol_end = set->row[irow].icol_start -
    		set->row[irow].ipix_start
      		+ ( set->row[irow].ipix_start + ncol - 1 ) % set->npix;
  }

  return;
}

void Error(const char *message, const char *module)
/******************************************************************************
!C

!Description: 

  'Error' handles the error status for the 'pntr' functions.
  The error messages are written to LogStatus using SDP toolkit SMF
  functionality, and execution terminates through the HALTsmf micro.
 
!Input Parameters:
  message	error message string
  module	module in which the error occurred

!Output Parameters: (none)

!Revision History:

 * revised 05/20/1999 17:43	Z. Wan

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

  Usage Notes: (none)

!END
*******************************************************************************/
{

  char msg[PGS_SMF_MAX_MSG_SIZE];

  sprintf(msg, "%s in %s", message, module);
  HALTsmf(MODIS_E_BUF_ERROR, msg);

}

void GenObs(const double cen[2], const dgeo_t *cor, int32 *nobs, obs_t *obs)
/******************************************************************************
!C
Note:  This new subroutine replaces the old one completely.

!Description: 
  'GenObs_isin' finds the grid cells intersecting with a single observation.
  For those cells meeting the minimum coverage criterion a pointer to the 
  observation is stored.

  The intersection computation is done by a cakecut procedure, which uses a 
  cake-cutting approach to compute the intersection.

  Flow description of the cakecut procedure:
   1. horizontal 'cakecuth' process: returns polygons q in horizontal rows
   2. rescale and shift for each row
   3. vertical 'cakecutv': returns areas in each column within a row

!Input Parameters:
  cen		center longitude and latitude of the observation (radians)
  cor		observation corner's longitudes and latitudes relative to 
 		the observation center 'cen' (radians)

!Output Parameters:
  nobs	    number of observations satisfying min coverage criterion
  obs	    observation data structure 
	     obscov -  observation coverage; area of intersection between
		       observation and cell divided by observation area 
	     cellcov - cell coverage; area of intersection between
		       observation and cell divided by cell area
	     dline -   delta-line location of center of cell relative 
		       to observation center
	     dsample - delta-sample location of center of cell relative 
		       to observation center

!Revision History:

!Team-unique Header:

 Usage Notes: 
    1. The global parameters 'coverage_min' and 'size' are used 
       by this routine.

    2. The order of the four corners of the observation is:  
         p0 - top left     (-line, -sample)
         p1 - top right    (-line, +sample)
         p2 - bottom right (+line, +sample)
         p3 - bottom left  (+line, -sample)
         
             0 ----- 1     +---> samples
             |       |     |
             |   *   |     |
             |       |     V
             3 ----- 2   lines

    3. The dsample=d0 and dline=d1 are defined by

         w = d0 * u + d1 * v = cell_center - obs_center

       where the vectors in the sample (u) and line (v) are:

         u = 0.5* ( p1 + p2 - p0 - p3 )   [vector form]
         v = 0.5* ( p3 + p2 - p0 - p1 )

       The solutions are:  d0 = ( v x w) / ( v x u)
                           d1 = ( w x u) / ( v x u)

       One can show that v x u = area in the p_i polygon.  Thus no
       degenerate case ever occurs if the observation p_i span a
       nonzero area.

       All u,v,w are defined for the unscaled coordinates, so (v x u)
       is the same for all rows and columns.

 Author: Y-T Lau,  U of Maryland, NASA/GSFC/920.2   Last update: 4-25-96

 Version 1.1, 1997/01/13, Z. Wan corrected two bugs (incomplete range check
    	of irow_max and icol_min) that were found by Richard Cember, SDST.

!END
******************************************************************************/
{
  double lon, lat;	   /* latitude and longitude */
  int32 irow, icol, jrow, ipix; /* row, column and pixel numbers */
  float32 drow, dcol;	   	/* delta-row and delta-column values */
  int32 irow_min, irow_max; /* minimum and maximum row numbers */
  int32 icol_min, icol_max; /* minimum and maximum column numbers */
  double lon_min; 	/* minimum and maximum longitude */
  double lat_min; 	/* minimum and maximum latitude */
  double start[2];	   /* location of center of first column in row_max */
  double start0, start1;   /* location of center of first column in row */
  Vertices p;		/* Z. Wan, replace double p[4][2] in order to be
			   consistent to the parameter in cakecuth() */
  /* double p[4][2]; */    /* longitude and latitude of each of 
  			    * the corners relative to the observation center */
  int32 ncol = 0, nrow;    /* number of columns and rows */
  int32 icol_o, irow_o;    /* column and row numbers before adjustment */
  double cend[2];  	   /* observation center relative to start
  			    * (adjusted for dateline +/- 'pi' discontinuity) */

  Polygon q[IJMAX];        /* polygon in each row */
  double A2[IJMAX];        /* A=area2 of intersection */
  int32 i,j,n;

  /* R. Cember: use three distinct variables rather than a single "junk"
  variable that will be passed as pointers into Lbin_FwdW() in order to avoid
  the side effect: to make the index on the set->row[].ncol_tile variable out
  of bounds. */
  int32 junk1, junk2, junk3;

  double A, Aobs_in, Aobs_scale, dstart0, dw0, u[2], v[2];

  ipix = 0;

  /* Start with zero observations */
  *nobs = 0;

  /* find min lon/lat relative to cen.
     Gave same result as original code.  Tested 4/27/96 */
  lon_min = cor[0].dlon;
  lat_min = cor[0].dlat;
  for (i = 1; i < 4; i++) {
    if (cor[i].dlon < lon_min) lon_min = cor[i].dlon;
    if (cor[i].dlat < lat_min) lat_min = cor[i].dlat;
  } 
  /* Then find irow min/max, OK since lat is the same in tile space.
     Note that lat_max <-> irow_min */

  lon = lon_min + cen[0];  lat = lat_min + cen[1];

  /* take care of the pixel by the tile edge */
  returnStatus = (*pLbin_FwdW)(lon, lat, &irow_o, &icol_o, &irow_max,
		&icol_min, &drow, &dcol); 

  if (irow_max < 0 || irow_max >= set->nrow_tile )	return;

  irow = irow_max;

  /* cell center location of the first column in the row; unadjusted */

  returnStatus = (*pLbin_Inv)(irow, icol_min, 0.0, 0.0, &start[0], &start[1]);

  /* center relative to start; adjusted for discontinuity at date line */
  cend[0] = cen[0] - start[0];
  if (cend[0] >= PI) cend[0] -= TWOPI;
  else if (cend[0] < -PI) cend[0] += TWOPI;
  cend[1] = cen[1] - start[1];

  /* observation corner locations in local cell coordinates */
  /* scale p_y to local grid cell size and shift to lower cell boundary */

  for (i = 0; i < 4; i++) {
    p[i][0] = (cor[i].dlon + cend[0]);
    p[i][1] = (cor[i].dlat + cend[1]) * set->row[irow].scale[1] + 0.5;  
  }

  /* horizontal cut into rows */
  cakecuth(p, 4, q, &nrow);
  irow_min = irow_max - nrow + 1;

  /* u,v for computing dsample and dline (unscaled) */
  u[0] = 0.5* ( cor[1].dlon + cor[2].dlon - cor[0].dlon - cor[3].dlon );
  u[1] = 0.5* ( cor[1].dlat + cor[2].dlat - cor[0].dlat - cor[3].dlat );
  v[0] = 0.5* ( cor[3].dlon + cor[2].dlon - cor[0].dlon - cor[1].dlon );
  v[1] = 0.5* ( cor[3].dlat + cor[2].dlat - cor[0].dlat - cor[1].dlat );

  /* 1/area of polygon spanned by the four corners (unscaled) */
  /* do not need this: Aobs_in = (Aobs_in==0.0) ? 0.0 : 1./Aobs_in; */

  Aobs_in = 1./(v[0] * u[1] - v[1] * u[0]);

  /* Do for each row */

  for (irow = irow_min; irow <= irow_max; irow++) {

    n = irow_max - irow;

    if (irow < 0 || irow >= set->nrow_tile) continue; 

    /* Row number relative to start of tile */
    jrow = irow;

    /* Find minimum column in the row */
    returnStatus = (*pLbin_Inv)(irow, 0, 0.0, 0.0, &lon, &lat);

    returnStatus = (*pLbin_FwdW)(lon_min+cen[0],lat,&junk1,&junk2,&junk3,
				  &icol_min,&drow,&dcol); 
    
    /* skip if out of range */

    if (icol_min < icol_left || icol_min > set->row[jrow].ncol_tile) continue;

    /* cell center location of the first column in the row */
    returnStatus = (*pLbin_Inv)(irow, icol_min, 0.0, 0.0, &start0, &start1); 

    dstart0 = (start0 - start[0]);

    /* shift x component to start of current row, also scale it so q is 
       now fully scaled to local cell coordinates.  No need to adjust for
       dateline discontinuity since cend and thus q.v are already adjusted. */

    for (j=0; j<q[n].n; j++) {
      q[n].v[j][0] = ( q[n].v[j][0] - dstart0 ) * set->row[jrow].scale[0] +0.5;
    }

    /* next do vertical cutting of current row, A2 = scaled area2 */
    cakecutv(q[n].v, q[n].n, A2, &ncol);

    if (ncol > MAX_NOBS) Error("too many columns", "GenObs_isin");
    
    icol_max = icol_min + ncol - 1;

    /* center of cell relative to center of obs, unscaled del_w_0 */

    dw0 = 1. / set->row[jrow].scale[0];

    Aobs_scale = Aobs_in * dw0 / set->row[jrow].scale[1];

    for (i = 0, icol = icol_min; i < ncol; i++, icol++) {

      /* skip out of range columns */
      if (icol < 0 || icol > (set->row[jrow].ncol_tile - 1)) continue;

      ipix = icol + set->row[jrow].ipix_start;
      A = 0.5*A2[i];

      /* Save it if the cell coverage meets the minimum coverage */
      if ( A > coverage_min ) {

        if (*nobs >= MAX_NOBS) 
          Error("too many observations", "GenObs_isin");

	/* adjustment for pixels by the tile edge */
	if ( A > 1.0 )	A = 1.0;

	obs[*nobs].cellcov = A;
	obs[*nobs].obscov = A * Aobs_scale;
        obs[*nobs].ipix = ipix;
        obs[*nobs].irow = jrow;

        (*nobs)++;
      }
    } /* for (icol... */

  } /* for (irow... */

  return;
}

/* === Integerized Sinusoidal Grid routines extacted into seperate file === */
/* === Robert Wolfe, 2002/09/12 === */

/* the following portion was adapted from obsgrid.c */

#ifdef TRUE
#undef TRUE
#endif 
#define TRUE 1
#define FALSE 0

/* Private HDF I/O Structure */

typedef struct {
  unsigned char *buf;	/* Buffer for hdf i/o */
  unsigned char *pnt;	/* Pointer to current observation parameter in buffer */
  int32 size;		/* Size of the observation parameter */
} Obs_hdfio_t;

/* Private Error Structure */

typedef struct {
  int32 num;             /* Error number */
  char *str;           /* Error message */
} Obs_error_t;

static Obs_error_t OBS_BADSIZE    = { -5, "Bad grid size parameter"};
static Obs_error_t OBS_NOMEMORY   = { -6, "Out of memory"};

int32 Obs_Error(const Obs_error_t *, const char *);  
 
Obs_gd_p Obs_CreateGrid(int32 nl, int32 ns, int16 np)
/*
!C*****************************************************************************

!Description: Obs_CreateGrid (Create Grid) - creates an observation grid 
 data structure.

!Input Parameters:
 nl		Number of lines (rows) in the grid.
 ns		Number of samples (columns) per line in the grid.
 obssize	Size of the observation.

!Output Parameters:
 (returns)	Pointer to the memory structure of the new observation grid. 
                NULL is returned if there is an error.

!Revision History:

!Team-unique Header:

 ! Usage Notes:
   1. The observation grid is 'nl' lines by 'ns' samples.
   2. Each observation consists of 'np' parameters which are specified by
      name and HDF data type in 'parameters'.

!END***************************************************************************
*/
{

  int32 ip, stat; 

  /* Check Inputs */
  if (nl < 0  ||  ns < 0)
    {stat = Obs_Error(&OBS_BADSIZE, "CreateGrid");  return NULL;}

  if (gd == NULL) {

     if ((gd = (Obs_gd_p) malloc(sizeof(Obs_gd_t))) == NULL)
       { stat = Obs_Error(&OBS_NOMEMORY, "CreateGrid");  return NULL; }

  }
  gd->nl = nl;
  gd->ns = ns;
  gd->np = np;

  return(gd);
}

int32 Obs_Error(const Obs_error_t *err, const char *module)  

/*
!C*****************************************************************************

!Description: Obs_Error (Error) handles errors for the observation 
 grid functions.

!Input Parameters:
 err		Error structure.
 module		Module name (minus 'Obs_').

!Output Parameters:
 (returns)	Status: Always OBS_ERROR.

!Revision History:
 * revised 05/20/1999 17:45	Z. Wan

!Team-unique Header:

!END***************************************************************************
*/
{
  char msg[PGS_SMF_MAX_MSG_SIZE];

  sprintf(msg, "%s %s obsgrid.c", err->str, module);

  HALTsmf(MODIS_E_OBS_ERROR, msg);

  return (OBS_ERROR);
}

/******************************************************************************
!C

File: geocor_lst.c


!Description: 
  Geolocation Corners - Compute the corners of an observation 
  from the geolocation information.  An observation is the four sided polygon 
  projection a single spatial element.  It is able to work on night mode data.

  In addition a stand-alone test program is included to test the correct 
  functioning of this routine.  The compilation of this test program is 
  controlled by the 'STAND_ALONE' variable.

!Input Parameters:

!Output Parameters:
 
!Revision History:

  Revision 01.01  1996/09/28
  Zhengming Wan/MODIS Science Team member

  Revision 00.01  1995/09/15
  Robert E. Wolfe/MODIS Land Team Support
  Original version.

!Team-unique Header:
  This software was developed by the MODIS Land Science Team Support 
  Group for the National Aeronautics and Space Administration, Goddard 
  Space Flight Center, under NASA Task 92-012-05.

  References and Credits:
    Written by Robert E. Wolfe
               Hughes STX
               robert.wolfe@gsfc.nasa.gov
               phone: 301-441-4585
               
               Hughes STX
               7701 Greenbelt Road, #400
               Greenbelt, MD 20770  

!END***************************************************************************
*/

/* macro to adjust an angle for the +/- 'pi' boundary */
/*
#define ADJ(x)  (((x) >= PI) ? (x) - TWOPI : (((x) < -PI) ? (x) + TWOPI : (x)))
*/
#define ADJ(x , y) ( (((x)-(y)) >= PI) ? (x)-(y) -TWOPI : (((x)-(y)) < -PI) ? \
	(x) - (y) + TWOPI : (x) - (y) )

/* Start of 'GeoCor_lst' Function */

void GeoCor_lst(int32 il, int32 is, int32 nl_scub, int32 ns, 
            float32 **lon, float32 **lat, dgeo_t *cor)
/*
!C*****************************************************************************

!Description: 
  'GenCor' computes the corners of an observation 
  from the geolocation information.  An observation is the four sided polygon 
  projection a single spatial element.  For observations not on the edges of 
  a scan the corners are linearly interpolated from the four center locations 
  of the adjacent observations, otherwise they are linearly extrapolated.
  It is modified by Zhengming Wan, 10/28/1996, for its use in night mode
  as well.
 
!Input Parameters:
  il 		line number of observation 
  is		sample number of observation
  nl_scub	number of lines in the scan
  ns		number of samples per line in the scan
  lon		array of longitudes of observation centers (radians)
  lat		array of latitudes of observation centers (radians)

!Output Parameters:
  cor		observation's corners longitude and latitude relative to 
                the observation center 'cen' (radians); see order below

!Revision History:

 *Revision 2.0  1999/09/25  09:54  Z. Wan
 * change night mode into ascending mode (in the exact sense)
 *
 *Revision 1.1  1996/10/28	   Z. Wan
 * for night mode

!Team-unique Header:

  Usage Notes: 
    1. The order of the four corners is:  

        0 - top left     (-line, -sample)
        1 - top right    (-line, +sample)
        2 - bottom right (+line, +sample)
        3 - bottom left  (+line, -sample)
        
     As a figure:
     
             0 ----- 1     +---> samples
             |       |     |
             |   *   |     |
             |       |     V
             3 ----- 2   lines

!END***************************************************************************
*/
{
  int32	i_ascend;
  int32 isa, isb, ila, ilb; /* sample and line indexes for left and right 
  			 * observations (sample) and top and bottom 
  			 * observations (line); adjusted for extrapolation over 
  			 * the scan edges */
  int32 nscub, iil, iila, iilb;
  float dsa, dsa1, dsb, dsb1; /* delta-sample and one minus the delta-sample
  			 * for left and right observations */
  float dla, dla1, dlb, dlb1; /* delta-line and one minus the delta-line
  			 * for top and bottom observations */
  double clon, clat;	/* center longitude and latitude */
  float d0, d1, d2;	/* intermediate values */
  float dlon[3][3];	/* delta-longitude of the 9 points to interpolate */

  /* determine if it is ascending */
  dlb = SIZE;
  ila = il + 1;
  ilb = il - 1;
  if (ilb == -1) { ilb = 1;  dlb = -dlb; }
  i_ascend = (dlb * (lat[il][is] - lat[ilb][is]) > 0.0) ? 1 : 0;

  /* Calculate the sample indexes and delta-samples for the left and right 
   * observations; adjust for extrapolation over scan edges */
  dsa = SIZE;
  dsb = SIZE;
  if ( i_ascend ) {
	isa = is + 1;
	isb = is - 1;
  	if (isb == -1) { isb = 1;  dsb = -dsb; }
  	else if (isa == ns) { isa = isb;  dsa = -dsa; }
  }
  else {
	isa = is - 1;
	isb = is + 1;
  	if (isa == -1) { isa = 1;  dsa = -dsa; }
  	else if (isb == ns) { isb = isa;  dsb = -dsb; }
  }
  dsa1 = 1.0 - dsa;
  dsb1 = 1.0 - dsb;

  /* Calculate the line indexes and delta-lines for the top and bottom 
   * observations; adjust for extrapolation over scan edges  */

  nscub = il / nl_scub;
  iil = il - nscub * nl_scub;
  dla = SIZE;
  dlb = SIZE;
  if ( i_ascend ) {
	ila = il + 1;
	iila = iil + 1;
	ilb = il - 1;
	iilb = iil - 1;
  	if (iilb == -1) { ilb = ila;  dlb = -dlb; }
  	else if (iila == nl_scub) { ila = ilb;  dla = -dla; }
  }
  else {
	ila = il - 1;
	iila = iil - 1;
	ilb = il + 1;
	iilb = iil + 1;
  	if (iila == -1) {ila = ilb;  dla = -dla;}
  	else if (iilb == nl_scub) {ilb = ila;  dlb = -dlb;}
  }
  dla1 = 1.0 - dla;
  dlb1 = 1.0 - dlb;

  /* Get the intermediate delta-longitudes adjusting for the +/- pi 
   * discontinuity */
  clon = lon[il][is];

  dlon[0][0] = ADJ(lon[ila][isa] , clon);
  dlon[1][0] = ADJ(lon[il][isa]  , clon);
  dlon[2][0] = ADJ(lon[ilb][isa] , clon);

  dlon[0][1] = ADJ(lon[ila][is] , clon);
  dlon[1][1] = 0.0;
  dlon[2][1] = ADJ(lon[ilb][is] , clon);

  dlon[0][2] = ADJ(lon[ila][isb] , clon);
  dlon[1][2] = ADJ(lon[il][isb]  , clon);
  dlon[2][2] = ADJ(lon[ilb][isb] , clon);

  /* Generate the three intermediate longitude points for the left edge */
  d0 = dsa1 * dlon[0][1] + dsa * dlon[0][0];
  d1 =                     dsa * dlon[1][0];
  d2 = dsa1 * dlon[2][1] + dsa * dlon[2][0];

  /* Calculate the top and bottom longitude corners on the left edge */
  cor[0].dlon = dla1 * d1 + dla * d0;
  cor[3].dlon = dlb1 * d1 + dlb * d2;

  /* Generate the three intermediate longitude points for the right edge */
  d0 = dsb1 * dlon[0][1] + dsb * dlon[0][2];
  d1 =                     dsb * dlon[1][2];
  d2 = dsb1 * dlon[2][1] + dsb * dlon[2][2];

  /* Calculate the top and bottom longitude corners on the right edge */
  cor[1].dlon = dla1 * d1 + dla * d0;
  cor[2].dlon = dlb1 * d1 + dlb * d2;

  /* Observation center latitude */
  clat = lat[il][is];

  /* Generate the three intermediate latitude points for the left edge */
  d0 = dsa1 * lat[ila][is] + dsa * lat[ila][isa];
  d1 = dsa1 * lat[il][is]  + dsa * lat[il][isa];
  d2 = dsa1 * lat[ilb][is] + dsa * lat[ilb][isa];

  /* Calculate the top and bottom latitude corners on the left edge */
  cor[0].dlat = dla1 * d1 + dla * d0 - clat;
  cor[3].dlat = dlb1 * d1 + dlb * d2 - clat;

  /* Generate the three intermediate latitude points for the right edge */
  d0 = dsb1 * lat[ila][is] + dsb * lat[ila][isb];
  d1 = dsb1 * lat[il][is]  + dsb * lat[il][isb];
  d2 = dsb1 * lat[ilb][is] + dsb * lat[ilb][isb];

  /* Calculate the top and bottom latitude corners on the right edge */
  cor[1].dlat = dla1 * d1 + dla * d0 - clat;
  cor[2].dlat = dlb1 * d1 + dlb * d2 - clat;

  return;
}

void zeroout_general(void *buf, int32 nelems, int32 type_size)
/*
!C*****************************************************************************

!Description:
   This function sets contiguous memory locations to zero.

!Input Parameters:	buf	the starting address of the memory location
			nelems	number of elements 
		        type_size  number of bytes in the data type
!Output Parameters:	none


!Revision History:


!References and Credits:

   Version v1.1 09-15-96

   Written by:

	R. Cember, MODIS Science Data Support Team, based on function 
          "zeroout" by ...

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/
{  /* begin function */

uint8 *buf8;
uint16 *buf16;
uint32 *buf32;

if (type_size == 1)  {
  buf8 = buf;
  do {
    *buf8 = 0;
    buf8++;
  }
  while (--nelems);
}
else if (type_size == 2) {
  buf16 = buf;
  do {
    *buf16 = 0;
    buf16++;
  }
  while (--nelems);
}
else if (type_size == 4) {
  buf32 = buf;
  do {
    *buf32 = 0;
    buf32++;
  }
  while (--nelems);
}
else {
  HALT("STOP -- wrong data type in zeroout_general -- use 1, 2 or 4.\n");
  exit (1);
}

}  /* end function */


int32 iday_of_year(int32 iyr, int32 imnth, int32 iday)
{
/*
!C*****************************************************************************

!Description:
   This function calculates the day-of-year for given iyr, imnth, and iday

!Input Parameters:	iyr	- year
			imnth	- month
			iday	- date
		     
!Output Parameters:	iday	- day-of-year

  Returns:

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	daytab[2][13] = {
		0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
		0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
	};
	int32	i, leap;

	leap = iyr % 4 == 0 && iyr % 100 != 0 || iyr % 400 == 0;

	for (i = 1; i < imnth; i++)
		iday += daytab[leap][i];

	return (iday);
}

/*
!C*****************************************************************************

!Description:

This function, SMFLOG, is called by the DoMetaData function.  It replaces 
a macro named SMFLOG, the behavior of which depends on syntactical subtleties
that seem to be compiler-dependent.

!Input Parameters:  Message string

!Output Parameters: None		

!Revision History:

	Modified by Z. Wan for v2.1.10 on 23 September 1999.

	Created by Richard P. Cember, MODIS SDST, from macro SMFLOG in
	original V1 code delivery. 3 January 1997.

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
        
!Design Notes:
  
!END***************************************************************************
*/
void SMFLOG(char *a)
{

  char		systemTag[PGSd_SMF_TAG_LENGTH_MAX] = "MOD11 LST Version 5" ;
  char		LogMsg[1024] = { NULL };
  char		a1[128] = { NULL };

  strcpy (&a1[0], a);
  a1[18] = '\0';

  sprintf(systemTag, "MOD11 LST Version %d", vers_id_num);
  sprintf(LogMsg,"%s: %s", systemTag, a);
  PGS_SMF_GenerateStatusReport(LogMsg);
  if ( strcmp(a1, "Completed MOD11_L2") != 0)
/*These next lines were added by C. Collins on 6/8/01 so comparison to Aqua*/
/*output name or processing an ocean granule does not cause an exit value of 1 */
     {
     if ( strcmp(a1, "Completed MYD11_L2") != 0)
	{
	if ( strcmp(a1, "Completed MOD_PR11") != 0)
	  {
	     if (strcmp(a1, "STOP - L2 file alr") != 0)
		exit(1);
	     else exit(0);
	  }
	else exit(0);
	}
      else exit(0);
      }
  else	exit(0);

}

/*
!C*****************************************************************************

!Description:

This function, DoMetaData, is called by the main function for writing L2
Metadata, and by function Granule_tile for writing 1km & 6km L3 MetaData.
It is responsible for writing all "core" and "archive" meta data on the
L2 and 1km & 6km L3 LST HDF files.
The meta data that it processes comes from the L1A and/or L1B input files, 
the MOD12 Landcover input files, the PCF file, the MCF file, and some 
generated internally by this function itself.

This void function has no return code.  The program aborts if there are errors.

!Input Parameters:

        Int32 id_old            id_old=1 for the existed output file
        Int32 Level             Output product Level
        PCF Logical number      Output file logical number in PCF file
	PGSt_integer            Number of tiles involved in this run
	PGSt_integer            The tile number for L3 files
	MODFILE         	Handle of the output file

!Output Parameters:		None

!Revision History:

  $Log: DoMetaData() $
 * Revision 2.2.3 2000/03/07  21:00     Z. Wan
 *	comment out the change made on 1999/12/29 so that MCF files will
 *	not modified during run time.
 * Revision 2.2.1 1999/12/29  11:04     Z. Wan
 *	to set a true value to INPUTPOINTER's NUM_VAL
 * Revision 2.03	 1998/10-05   ff       Z. Wan
 * add MOD11UPD file
 * Revision 2.02	 1998/01-03   ff       R. Cember
 * also modified MCF files
 * Revision 2.01	 1997/11/20  23:55     Z. Wan
 * working with two seperate MCF files
 *
 * DoMetaData()
 * Revision 2.00	 1997/09/30  23:55     Z. Wan
 * for L2, 1km & 6km L3 output files

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
   Originally developed by 
        Gang Ye,
	MODIS/SDST, SAIC/GSC
	301-352-2109
	gang.ye@gsfc.nasa.gov

   Major modifications made by
	Richard Cember
	MODIS/SDST, SAIC/GSC
	301-352-2148
	rcember@ltpmail.gsfc.nasa.gov

!Design Notes:

  Richard Cember made the following modifications:
  1. used a separate MCF for each output file.
  2. checked metadata in each MCF file.
  3. made N_BYTES_CORE_L3 and N_BYTES_ARCH_L3 big enough.
  4. all calls to PGS_PC_GetReference() have been changed to calls to
     PGS_PC_GetUniversalRef().

!END***************************************************************************
*/

#define	   ESDT_L2		"MOD11_L2"
#define    SATELLITE		"A"
#define    VERSION		"005"

#define    N_BYTES_CORE_L3	128000

#define    N_BYTES_ARCH_L3	256000	

char	   core_L3[N_BYTES_CORE_L3];
char	   arch_L3[N_BYTES_ARCH_L3];

#define MAX_CODE_STG_VALS	600

char	         *FloatStg = "SIZEMBECSDATAGRANULE";
PGSt_double      FloatVal;

char	         *DoubleStg1[] = {  "NORTHBOUNDINGCOORDINATE", 
				    "SOUTHBOUNDINGCOORDINATE",
				    "EASTBOUNDINGCOORDINATE",  
				    "WESTBOUNDINGCOORDINATE" };

char	CodeStgVals[MAX_CODE_STG_VALS][PGSd_PC_VALUE_LENGTH_MAX]; 

char	*MoreCodeStg[4] = {
			   "SHORTNAME",
		 	   "HORIZONTALTILENUMBER",
			   "VERTICALTILENUMBER",
			   "GRIDTYPE" };

char	MoreCodeStgVals[4][PGSd_PC_VALUE_LENGTH_MAX];

char	*DoubleStg2[] = {
	 "GRINGPOINTLATITUDE.1",
	 "GRINGPOINTLONGITUDE.1",
	 "EQUATORCROSSINGLONGITUDE.1" };
/* R. Cember: in the current ECS metadata model, EquatorCrossingLongitude is
   a float, not a double, but I see no reason to change the code, since the
   conversion should be OK except for a loss of unnecessary precision. */

char	*IntegerStg2[] = {
	 "GRINGPOINTSEQUENCENO.1",
	 "ORBITNUMBER.1" };

void DoMetaData( int32 id_old, int32 level, PGSt_PC_Logical PCLOG,
	PGSt_integer tiles, PGSt_integer i_tile, MODFILE *outfile)
{

  PGSt_integer		j, version;
  char	*local2;
  char	s_tmp[128] = {NULL};
  char	time2[128] = {NULL};
  char	timeb[128] = {NULL};
  char	LocalGranuleID[128] = {NULL};
  char	augmented_string[MAX_NC_NAME] = {NULL};
  char	str_num[32] = {NULL};
  char  CharSt[256] = {NULL};
  char  GranuleNumber[32] = {NULL};
  int32 i, ii = 0, jj = 2;
  int32	n_gran_pnts_old = 0, n_gran_pnts = 0;
  int32 n_val;
  PGSt_integer	i4[4];
  PGSt_double   d4[4];
  PGSt_double	DoubleBounds[4] = {2.F};

  /* R. Cember: for LocalGranuleID proc date & time upgrade. */
  char proc_year[PGSd_PC_VALUE_LENGTH_MAX], proc_day[PGSd_PC_VALUE_LENGTH_MAX];
  char proc_hour[PGSd_PC_VALUE_LENGTH_MAX], proc_minute[PGSd_PC_VALUE_LENGTH_MAX];  
  char proc_second[PGSd_PC_VALUE_LENGTH_MAX];
  char pdt_string[PGSd_PC_VALUE_LENGTH_MAX], *pdt_string_ptr = pdt_string;

/* R. Cember: for InputPointer metadatum upgrade. */
#define MAX_INPUT_PTRS 2000
  char InputPointer[MAX_INPUT_PTRS][PGSd_PC_VALUE_LENGTH_MAX];
  char *InputPointerPtr[MAX_INPUT_PTRS];
  int32 NextInputPointer = 0;
  int32 accum_input_ptrs (char *, const char *); 
/* Z. Wan comments the following line out
  char    mcf_name[128], mcf_kp_name[128], shcmd[256];	*/
  
  /* get core and archiv metadata from the existing L3 output file */
  if ( level == 3 && id_old && PCLOG != PCLogupdate[i_days_in_cycle_update-1]) {

    if ( PCLOG == PCLogL3_1km ) {

      sprintf(sd_attr_name, ARCHIVES_IN);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_ARCH_L3;
      if(getMODISfileinfo(lst_1km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		arch_L3) != MAPIOK)
	Error("Re: lst_1km_l3_outf: failure on call to ", "getMODISfileinfo");

      sprintf(sd_attr_name, CORES);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_CORE_L3;

      if(getMODISfileinfo(lst_1km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		core_L3) != MAPIOK)
	 Error("Re: lst_1km_l3_outf: failure on call to ", "getMODISfileinfo");

    }
    if ( PCLOG == PCLogL3_6km ) {

      sprintf(sd_attr_name, ARCHIVES_IN);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_ARCH_L3;
      if(getMODISfileinfo(lst_6km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		arch_L3) != MAPIOK)
	Error("Re: lst_6km_l3_outf: failure on call to ", "getMODISfileinfo");

      sprintf(sd_attr_name, CORES);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_CORE_L3;
      if(getMODISfileinfo(lst_6km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		core_L3) != MAPIOK)
	 Error("Re: lst_6km_l3_outf: failure on call to ", "getMODISfileinfo");
    }
    date_begin[1][0] = '\0';
    local2 = date_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[0], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_begin from core_L3");
    time_begin[1][0] = '\0';
    local2 = time_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[1], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_begin from core_L3");
    date_end[1][0] = '\0';
    local2 = date_end[1];
    if ( get_meta_attr(core_L3, CharStg2[2], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_end from core_L3");
    time_end[1][0] = '\0';
    local2 = time_end[1];
    if ( get_meta_attr(core_L3, CharStg2[3], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_end from core_L3");

  /* get INPUTGRANULEPOINTERs from the existing output file */

    local = s_tmp;
    if (get_meta_attr(core_L3,"PARAMETERVALUE.5", 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_L3");
    sscanf (local, "%d", &n_gran_pnts_old);

    version = tile_pntr[i_tile].s_num + 1;

    for (j = 1; j <= n_gran_pnts_old; j++) {

	local = CodeStgVals[jj];
	CodeStgVals[jj][0] = '\0';

	strcpy(augmented_string, "INPUTGRANULEPOINTER");  
	sprintf(str_num, ".%d", j);
	strcat(augmented_string, str_num);

	/* R. Cember: move InputGranulePointer from core to archive */
	if (get_meta_attr(arch_L3,augmented_string,1, 1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3"); 
 
	jj++;

    }
  }

  /* set input granule pointers from PCF */

  /* R. Cember: all calls to PGS_PC_GetReference in this routine have been
     changed to calls to PGS_PC_GetUniversalRef. */

    for (j = 2; j < 7; j++) {

      switch (j) {

	case 2:
	 version = gran_link.geo1A_ver_n;	Vers_tmp = version;
	 if(PGS_PC_GetUniversalRef(PCLog1A,&Vers_tmp,CodeStgVals[jj])
		!=PGS_S_SUCCESS)
	    SMFLOG("PGS_PC_GetUniversalRef--Error getting L1A input UR name"); 
         jj++;
		break;
	case 3:
	 version = gran_link.gran1B_ver_n;	Vers_tmp = version;
	 if(PGS_PC_GetUniversalRef(PCLog1B,&Vers_tmp,CodeStgVals[jj])
		!=PGS_S_SUCCESS)
	    SMFLOG("PGS_PC_GetUniversalRef--Error getting L1B input UR name");
         jj++;
		break;
	case 4:
	 version = gran_link.cmask_ver_n;	Vers_tmp = version;
	 if(PGS_PC_GetUniversalRef(PCLogCmask, &Vers_tmp, CodeStgVals[jj])
			!=PGS_S_SUCCESS)
	 SMFLOG("PGS_PC_GetUniversalRef--Error getting CloudMask input UR name");
	 jj++;
		break;
	case 5:
	 version = gran_link.snow_ver_n;
	 if ( !version )	break;
	 Vers_tmp = version;
	 if(PGS_PC_GetUniversalRef(PCLogSnow, &Vers_tmp, CodeStgVals[jj])
			!=PGS_S_SUCCESS)
	   SMFLOG("PGS_PC_GetUniversalRef--Error getting Snow input UR name");
         jj++;
		break;
	case 6:
	 version = gran_link.aprof_ver_n;
	 if ( !version )	break;
	 Vers_tmp = version;
	 if(PGS_PC_GetUniversalRef(PCLogProfile,&Vers_tmp,CodeStgVals[jj])
			!=PGS_S_SUCCESS)
	 SMFLOG("PGS_PC_GetUniversalRef--Error getting Profile input UR name");
	 jj++;
		break; 

      }		/* end switch on j */
    }  /* end for-loop on j */

    n_gran_pnts = jj - 2;

if (PCLOG == PCLogL2) { 
 if ( PGS_MET_Init( MCF_FILE_L2, mdhandles) != PGS_S_SUCCESS )
	HALT("Failure in PGS_MET_Init");
}
else if (PCLOG == PCLogL3_1km) { 
 if ( PGS_MET_Init( MCF_FILE_L3_1KM, mdhandles) != PGS_S_SUCCESS ) 
	HALT("Failure in PGS_MET_Init");
}
else if (PCLOG == PCLogL3_6km) {
 if ( PGS_MET_Init( MCF_FILE_L3_6KM, mdhandles) != PGS_S_SUCCESS )
	HALT("Failure in PGS_MET_Init");
}
else if (PCLOG == PCLogupdate[i_days_in_cycle_update-1]) {
 if ( PGS_MET_Init( MCF_FILE_L3_UPD, mdhandles) != PGS_S_SUCCESS )
	HALT("Failure in PGS_MET_Init");
}
else {
 HALT("Unknown PCLOG value in DoMetaData"); exit(1); 
}

  /*** get VersionID from MCF file**04/06/01**/

  if ( PGS_MET_GetSetAttr(mdhandles[1],"VersionID",&vers_id_num) != PGS_S_SUCCESS )
	SMFLOG("Failure in PGS_MET_GetSetAttr for VersionID");

  if ( level == 2 )
	met_cnt = &met_cnt_L2;
  else {
	if ( PCLOG == PCLogL3_1km )	met_cnt = &met_cnt_1km;
	if ( PCLOG == PCLogL3_6km )	met_cnt = &met_cnt_6km;
  }

 if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {

   /* Process Char meta data. */
   for ( j = 0; j < 9; j++ ) {

    /* R. Cember: there is no more OperationMode metadatum, which is j=7, the
    way this code is set.  Skip this case. */
    if (j==7) continue;

    /* Comment added by R. Cember: This next "if" bypasses the 
    EquatorCrossingDate and EquatorCrossingTime metadata for the L3 products */
    if ( level == 3 && j > 4 && j < 7 ) continue;

    CharVals[j][0] = '\0';
    local2 = CharVals[j];

    if ( get_meta_attr(core_1B, CharStg2[j], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails on core_1B");

    if ( level == 3 && j < 4 ) {
	if (id_old) {
		switch ( j ) {
		case 0: local2 = ( strcmp (CharVals[0], date_begin[1]) < 0) ?
				CharVals[0] : date_begin[1];	break;
		case 1: local2 = ( strcmp (time_begin[0], time_begin[1]) < 0) ?
				time_begin[0] : time_begin[1];	break;
		case 2: local2 = ( strcmp (CharVals[2], date_end[1]) > 0) ?
				CharVals[2] : date_end[1];	break;
		case 3: local2 = ( strcmp (time_end[0], time_end[1]) > 0) ?
				time_end[0] : time_end[1];	break;
		}
	}
	else {
		switch ( j ) {
		case 0: local2 = CharVals[0];	break;
		case 1: local2 = time_begin[0];	break;
		case 2: local2 = CharVals[2];	break;
		case 3: local2 = time_end[0];	break;
		}
	}
    }

    if ( j == 8 )
	strcpy ( &local2[0], pgeversion );
    local = local2; 
 
     if(PGS_MET_SetAttr(mdhandles[1], CharStg2[j], &local ) != PGS_S_SUCCESS ) 
      HALT("FAILURE putting a CORE string value");
   }

   /* Process DOUBLE meta data */

   for ( j = 0; j < 3; j++ ) {

    if ( j == 2 && level == 3)	break;

    if ( level == 2 ) {
	n_val = ( j < 2 ) ? 4 : 1;
	if(get_meta_attr(core_1B,DoubleStg2[j],3,n_val,(char *) &d4[0])==FAIL)
		HALT("get_meta_attr fails on core_1B");
    }
    else {
	for (i = 0; i < 4; i++)
		d4[i] = tile_LatLons_edge[j][i];
    }

    if(PGS_MET_SetAttr(mdhandles[1],DoubleStg2[j], d4) != PGS_S_SUCCESS )
      HALT("FAILURE putting d4");
   }

   /* Process INTEGER meta data */
   for ( j = 0; j < 2; j++ ) {

    if ( level == 3 && j == 1 )	break;
    n_val = ( j ) ? 1 : 4;

    if ( level == 2 ) {
      if (get_meta_attr(core_1B,IntegerStg2[j],2,n_val,(char *) &i4[0]) == FAIL)
	 HALT("get_meta_attr fails on core_1B");
    }
    else {

      /* set GRINGPOINTSEQUENCENO */
      i4[0] = 1; i4[1] = 2; i4[2] = 3; i4[3] = 4;
    }

    if(PGS_MET_SetAttr(mdhandles[1], IntegerStg2[j], &i4 ) != PGS_S_SUCCESS )
      HALT("FAILURE putting a CORE integer value");
   }

 }

   /* R. Cember: moved ProcessingDateTime up here so we can use its info to
   construct the date/time suffix for LocalGranuleID. */
   {
    /* Construct ProcessingDateTime from UNIX time. */
    char	s[256], *p = s;
    time_t	clock;
    struct tm	*gm_p;

    clock = time(NULL);
    gm_p = gmtime( &clock );

    sprintf(pdt_string, "%i-%02i-%02iT%02i:%02i:%02i.000Z",
	    gm_p->tm_year+1900, gm_p->tm_mon+1, gm_p->tm_mday,
	    gm_p->tm_hour, gm_p->tm_min, gm_p->tm_sec );

  if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {
    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingDateTime",&pdt_string_ptr) == FAIL )
      HALT("FAILURE putting ProcessingDateTime");

    strcpy( s, process_env );
    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingEnvironment",&p ) == FAIL )
      HALT("FAILURE putting ProcessingEnvironment");

  }

    /* R. Cember: for LocalGranuleID. */
    sprintf( proc_year, "%.4d", 1900+(gm_p->tm_year) );
    sprintf( proc_day, "%.3d", (gm_p->tm_yday)+1 );
    sprintf( proc_hour, "%.2d", gm_p->tm_hour );
    sprintf( proc_minute, "%.2d", gm_p->tm_min );
    sprintf( proc_second, "%.2d", gm_p->tm_sec );
  }

  if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {

    /* Make the LocalGranuleID from RangeBeginningDateTime */
    sprintf( time2, "%sT%sZ", CharVals[0], CharVals[1] );

    if ( PGS_TD_ASCIItime_AtoB( time2, timeb ) != PGS_S_SUCCESS )
	HALT("PGS_TD_ASCIItime_AtoB failed");
    timeb[4] = timeb[8] = timeb[11] = timeb[14] = '\0';

    /* R. Cember: for suffixing processing date and time to LocalGranuleID. */
    if ( level == 2 )
      sprintf( &LocalGranuleID[0], "%s.%s%s%s.%s%s.%03d.%s%s%s%s%s.hdf", esdt_L2, 
             SATELLITE, timeb, &timeb[5], &timeb[9], &timeb[12], vers_id_num, 
             proc_year, proc_day, proc_hour, proc_minute, proc_second);

    else 
      sprintf( &LocalGranuleID[0], "%s.%s%s%s.h%.2dv%.2d.%03d.%s%s%s%s%s.hdf", 
	(PCLOG == PCLogL3_1km) ? esdt_1km_L3 : esdt_6km_L3, SATELLITE, timeb,
	&timeb[5], tile_pntr[i_tile].ih, tile_pntr[i_tile].iv, vers_id_num,
	proc_year, proc_day, proc_hour, proc_minute, proc_second);

  }
  else {
    if ( i_do_multi_days )
	sprintf( &LocalGranuleID[0], "%s.%c%d-%d.%s.h%.2dv%.2d",esdt_lst_upd,
	    id_char_update[0], n_days_per_cycle_update, i_days_in_cycle_update,
	    "update_for_lst", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
    else
	sprintf( &LocalGranuleID[0], "%s.1.%s.h%.2dv%.2d",esdt_lst_upd,
		"update_for_lst", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
  }

 {
   char	s[128], *p = s;
   strcpy ( s, LocalGranuleID );
   if (PGS_MET_SetAttr(mdhandles[1],"LocalGranuleID", &p ) != PGS_S_SUCCESS)
	HALT("FAILURE putting LocalGranuleID in CORE");
 }

 if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {

  /* Process BoundingCoordinate double meta data */
  for ( j = 0; j < 4; j++ ) {

    if ( level == 2 ) {
     if(get_meta_attr(arch_1A,DoubleStg1[j],3,1,(char *)&DoubleBounds[j])==FAIL)
	HALT("get_meta_attr failure getting a ARCHIVE double value");
    }
    else
	DoubleBounds[j] = tile_bound[j];

    if(PGS_MET_SetAttr(mdhandles[2], DoubleStg1[j], 
		&DoubleBounds[j] ) != PGS_S_SUCCESS )
      HALT("FAILURE putting an GEO input ARCHIVE double value");
  }

  if ( level == 2 ) {
    /* Get the GranuleNumber */
    CharSt[0] = '\0';
    local = &CharSt[0];
    if ( get_meta_attr(core_1A, CharStg2[9], 1, 1, (char *) local) == FAIL )
     HALT("get_meta_attr failure getting a AdditionalAttributeName for GranuleNumber value from GEO file");

    if( strcmp( CharSt, "GRANULENUMBER" ) != 0 )
     HALT("AdditionalAttributeName for GranuleNumber from GEO file is incorrect.");
    local = GranuleNumber;
    GranuleNumber[0] = '\0';
    if ( get_meta_attr(core_1A,"PARAMETERVALUE.1",1,1,(char *) local) == FAIL )
      HALT("get_meta_attr failure getting a GranuleNumber value");

  }

    for (j = 2; j < jj; j++) {

      /* R. Cember: move InputGranulePointer to archive from core metadata */
      sprintf (&s_tmp[0], "INPUTGRANULEPOINTER.%d", j - 1); 
      rm_path(CodeStgVals[j], local);
      if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
	   SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");

      /* R. Cember: to collect all the input pointers, in this case the
         InputGranulePointers.*/
      NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                CodeStgVals[j]);

    }  /* end for-loop on j */

    /* set input tile pointers */

    for ( i = 0; i < tiles; i++ ) {

	if ( level == 3 && i != i_tile )  continue;

	version = tile_pntr[i].s_num + 1;	Vers_tmp = version;
	if(PGS_PC_GetUniversalRef(PCLogLC, &Vers_tmp, CodeStgVals[jj + ii])
			!=PGS_S_SUCCESS)
	SMFLOG("PGS_PC_GetUniversalRef--Error getting landcover input UR name");
	sprintf (&s_tmp[0], "INPUTTILEPOINTER.%d", ii + 1);
	rm_path(CodeStgVals[jj+ii], local);
	/* R. Cember: move InputTilePointer from core to archive */
	if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS) 
		SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");
/* It is not a granule file, 1999/12/29, Z. Wan
        NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                local);
*/
	ii++;

    if ( i_brdf ) {
	version = tile_pntr[i].s_num + 1;	Vers_tmp = version;
	if(PGS_PC_GetUniversalRef(brdf_lun, &Vers_tmp, CodeStgVals[jj + ii])
			!=PGS_S_SUCCESS)
	SMFLOG("PGS_PC_GetUniversalRef--Error getting MOD43B1C input UR name");
	sprintf (&s_tmp[0], "INPUTTILEPOINTER.%d", ii + 1);
	rm_path(CodeStgVals[jj+ii], local);
	if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS) 
		SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");
	ii++;
    }
	if ( level == 2 )	continue;

	if ( tile_pntr[i].l3_1km_exist ) {
	   version = tile_pntr[i].s_num + 1;	Vers_tmp = version;
	   if(PGS_PC_GetUniversalRef(PCLogL3_1km, &Vers_tmp,
			CodeStgVals[jj+ii]) != PGS_S_SUCCESS)
		SMFLOG("PGS_PC_GetUniversalRef -- Error getting 1km L3 input UR name");
	   sprintf (&s_tmp[0], "INPUTTILEPOINTER.%d", ii + 1);
	   rm_path(CodeStgVals[jj+ii], local);
	   /* R. Cember: move InputTilePointer from core to archive */
	   if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
		SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");
/* It is not a granule file, 1999/12/29, Z. Wan
           NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                local);
*/
	   ii++;
	}
	if ( tile_pntr[i].l3_6km_exist ) {
	   version = tile_pntr[i].s_num + 1;	Vers_tmp = version;
	   if(PGS_PC_GetUniversalRef(PCLogL3_6km, &Vers_tmp,
			CodeStgVals[jj+ii]) != PGS_S_SUCCESS)
		SMFLOG("PGS_PC_GetUniversalRef -- Error getting 6km L3 input UR name");
	   sprintf (&s_tmp[0], "INPUTTILEPOINTER.%d", ii + 1);
	   rm_path(CodeStgVals[jj+ii], local);
	  /* R. Cember: move InputTilePointer from core to archive */
	   if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
		SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");
/* It is not a granule file, 1999/12/29, Z. Wan
           NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                local);
*/
	   ii++;
	}
	if ( tile_pntr[i].update_exist ) {
	   version = tile_pntr[i].s_num + 1;	Vers_tmp = version;
	   if(PGS_PC_GetUniversalRef(PCLogupdate[i_days_in_cycle_update-1],
	      &Vers_tmp, CodeStgVals[jj+ii]) != PGS_S_SUCCESS)
		SMFLOG("PGS_PC_GetUniversalRef -- Error getting MOD11 Update input UR name");
	   sprintf (&s_tmp[0], "INPUTTILEPOINTER.%d", ii + 1);
	   rm_path(CodeStgVals[jj+ii], local);
	   /* R. Cember: move InputTilePointer from core to archive */
	   if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
		SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");
/* It is not a granule file, 1999/12/29, Z. Wan
           NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                local);
*/
	   ii++;
	}
    }
 }

{  /* Comment added by R. Cember: Begin local block. */

  char		s[256], *p = s, CharString[256], *p1;
  int32		x;
  void		*loc2 = CharString, *loc3 = GranuleNumber;
  char		s_name[64], s_val[64], s_msg[128];	/* added by Z. Wan */

 if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {

  strcpy( s, rep_actual );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingActual",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingActual");

  strcpy( s, rep_planned );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingPlanned",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingPlanned");

  if ( level == 2 )
	strcpy( s, DayNightFlag );
  else
	strcpy( s, BOTH );
  if(PGS_MET_SetAttr(mdhandles[1],"DayNightFlag",&p ) == FAIL )
    HALT("FAILURE putting DayNightFlag");

  strcpy( s, "SCF V5.2.2A" );
  if(PGS_MET_SetAttr(mdhandles[1],"LocalVersionID",&p ) == FAIL )
    HALT("FAILURE putting LocalVersionID");

  strcpy( s, "Land-cover based band emissivity database" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.1",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.1");
  strcpy( s, "band_emis.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.1",&p ) == FAIL )
    HALT("FAILURE putting band_emis.h in LookupTablePointer");

  strcpy( s, "the generalized split-window LST coefficients" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.2",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.2");
  strcpy( s, "lst_coef.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.2",&p ) == FAIL )
    HALT("FAILURE putting lst_coef.h in LookupTablePointer");

/* R. Cember: End conversion of AncillaryInputType and AncillaryInputPointer
to LookupTableType and LookupTable Pointer. */

  /* Set up for placement in the metadata as a string array. */
  for ( i = 0; i < NextInputPointer; i++ )
	InputPointerPtr[i] = InputPointer[i];

  /* Close out the InputPointer list. */

  InputPointerPtr[NextInputPointer] = NULL;

  /* Reset the InputPointer count for next call to DoMetaData. */
  s[0] = '\0';
  NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], s);

  /* Set the InputPointer metadatum. */
  if(PGS_MET_SetAttr(mdhandles[1],"InputPointer", &InputPointerPtr ) == FAIL )
    HALT("FAILURE setting InputPointer");

  if ( level == 2 )
	strcpy( s, "MODIS LST" );
  else {
    if ( PCLOG == PCLogL3_1km )
	strcpy( s, "MOD 1KM L3 LST" );
    else
	strcpy( s, "MOD 6KM L3 LST" );
  }
  if(PGS_MET_SetAttr(mdhandles[1],"ParameterName.1",&p ) == FAIL )
    HALT("FAILURE putting ParameterName");

  switch(AutomaticQualityFlag) {
  case PASSED:      strcpy( s, "Passed" );      break;
  case SUSPECT:     strcpy( s, "Suspect");      break;
  case FAILED:      strcpy( s, "Failed" );
  }
  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlag.1",&p) == FAIL)
    HALT("FAILURE putting AutomaticQualityFlag");

  strcpy(s,"Passed if algorithm ran within bounds of execution constraints.  ");
  strcat(s,"Suspect if bounds of execution constraints violated.  ");
  strcat(s,"Failed if PGE failed.");

  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlagExplanation.1",&p ) == FAIL )
    HALT("FAILURE putting AutomaticQualityFlagExplanation");

  strcpy( s, sci_qua_exp );
  if(PGS_MET_SetAttr(mdhandles[1],"ScienceQualityFlagExplanation.1",&p ) == FAIL )
    HALT("FAILURE putting ScienceQualityFlagExplanation");

/* comment out the OperationalQualityFlag & its Explanation
  per David Roy's e-mail of 05/18/00 and Mile Linda's e-mail of 05/19/00

  if ( i_ignore_uncert )
	strcpy(s,"ignored EV_1KM_Emissive_Uncert_Indexes.");
  else
	strcpy(s,"skipped pixels with EV_1KM_Emissive_Uncert_Indexes >= 6.");
  if(PGS_MET_SetAttr(mdhandles[1],"OperationalQualityFlag.1",&p ) == FAIL )
    HALT("FAILURE putting OperationalQualityFlag");

  if ( i_ignore_uncert )
	strcpy(s,"ignore EV_1KM_Emissive_Uncert_Indexes if runtime parameter IGNORE_UNCERT set to 1.");
  else {
	strcpy(s,"skip pixels with EV_1KM_Emissive_Uncert_Indexes >= 6 ");
	strcat(s,"if runtime parameter IGNORE_UNCERT is not set to 1.");
  }
  if(PGS_MET_SetAttr(mdhandles[1],"OperationalQualityFlagExplanation.1",&p ) == FAIL )
    HALT("FAILURE putting OperationalQualityFlagExplanation");
*/

  x = 0;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentInterpolatedData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentInterpolatedData");

  x =  met_cnt->QA_MissingData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],
		     "QAPercentMissingData.1", &x ) == FAIL )
    HALT("FAILURE putting QAPercentMissingData");

  x =  met_cnt->QA_OutOfBoundsData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentOutOfBoundsData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentOutOfBoundsData");

  x = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if ( level == 2 )		/* including cloudy ocean pixels */
     x = tot_cld_pixels * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentCloudCover.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentCloudCover");

  if ( strcmp ( esdt_L1B, "MOD021KM" ) )
	strcpy( s, "PM-1" );
  else
	strcpy( s, "AM-1" );
  /* R. Cember: PlatformShortName is now AssociatedPlatformShortName. */
/* it is definded in MCF now
  if(PGS_MET_SetAttr(mdhandles[1],"AssociatedPlatformShortName.1",&p ) == FAIL )
    HALT("FAILURE putting AssociatedPlatformShortName");
*/
  /* R. Cember: InstrumentShortName is now AssociatedInstrumentShortName. */
/* it is definded in MCF now
  strcpy( s, "MODIS" );
  if(PGS_MET_SetAttr(mdhandles[1],"AssociatedInstrumentShortName.1",&p) == FAIL )
    HALT("FAILURE putting AssociatedInstrumentShortName");
  strcpy( s, "CCD" );
  if(PGS_MET_SetAttr(mdhandles[1],"AssociatedSensorShortName.1",&p) == FAIL )
    HALT("FAILURE putting AssociatedSensorShortName");
*/
/* Do the AdditionalAttributes.n. The ParamterValue.n must be a char string. */

  FloatVal = met_cnt->QA_GoodQuality * 100.0 / met_cnt->AllPixelsProcessed +0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTGOODQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.1", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.1");
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.1", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.1");

  FloatVal = met_cnt->QA_OtherQuality * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTOTHERQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.2", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.2");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.2", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.2");

  FloatVal = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDCLOUD");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.3", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.3");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.3", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.3");

  FloatVal = met_cnt->QA_NotProducedOther * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDOTHER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.4", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.4");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.4", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.4");

  if ( level == 2 ) {
	strcpy (s, "GRANULENUMBER");
	local2 = loc3;
  }
  else {
	strcpy (s, "N_GRAN_POINTERS");
	sprintf (s_tmp, "%d", n_gran_pnts);
	local2 = s_tmp;
  }

  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.5", &p )
		== FAIL )
	HALT("FAILURE putting AdditionalAttributeName.5");

  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.5", &local2 ) == FAIL )
	HALT("FAILURE putting ParameterValue.5");

/* R. Cember: Re: DDTS 728.  Adding two more PSAs here. */
if (level ==3) {
  strcpy(p, "HORIZONTALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.6", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.6");
  sprintf(p, "%.2d", tile_pntr[i_tile].ih);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.6", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.6");

  strcpy(p, "VERTICALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.7", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.7");
  sprintf(p, "%.2d", tile_pntr[i_tile].iv);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.7", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.7");

/* C. Collins, 11/4/02, change TILEID to TileID and its leading character 
   from 1 to 5 for SIN processing */
  strcpy(p, "TileID");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.8", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.8");
  if ( set->proj == LBIN_ISIN )
	sprintf(p, "11%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
  else
	sprintf(p, "51%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.8", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.8");
}

  FloatVal = met_cnt->QA_GoodQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONGOODQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 6 : 9);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 6 : 9);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_OtherQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONOTHERQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 7 : 10);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 7 : 10);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedCloud * 1.0  / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDCLOUD");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 8 : 11);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 8 : 11);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedOther * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDOTHER");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 9 : 12);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 9 : 12);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  sprintf(CharString, "%s", "NO");
/* next line changed by C. Collins on 10/25/04 so name uses underscores rather
than hyphens */
  strcpy (s, "CLOUD_CONTAMINATED_LST_SCREENED");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 10 : 13);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 10 : 13);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  strcpy( s, "102004" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageAcceptanceDate",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageAcceptanceDate");
  strcpy( s, "Normal" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageMaturityCode",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageMaturityCode");
  strcpy( s, "MOD_PR11A" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageName",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageName");
/* next lines edited by C. Collins on 11/11/04 so AlgorithmPackageVersion would
 be string*/
  strcpy (s, "5");
/* x = 5; - line commented out by ccollins on 11/11/04*/
/* x changed to p in next line by ccollins on 11/11/04*/
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageVersion",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageVersion");

if ( level == 2 )
    strcpy( s, "MODIS Level 2 Land Surface Temperature and Emissivity" );
else {
  if ( PCLOG == PCLogL3_1km )
    strcpy( s, "MODIS Level-3 1km Land Surface Temperature and Emissivity" );
  else
    strcpy( s, "MODIS Level-3 6km Land Surface Temperature and Emissivity" );
}

  strcpy( s, "GSFC" );
  if(PGS_MET_SetAttr(mdhandles[2],"ProcessingCenter",&p ) == FAIL )
    HALT("FAILURE putting ProcessingCenter");

  strcpy( s, "2484 and 3323" );
  if(PGS_MET_SetAttr(mdhandles[2],"SPSOParameters",&p ) == FAIL )
    HALT("FAILURE putting SPSOParameters");

  if ( id_old ) {
	local = MoreCodeStgVals[0];
	MoreCodeStgVals[0][0] = '\0';
	if (get_meta_attr(arch_L3,"LOCALINPUTGRANULEID",1,1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3");

	/* Z. Wan, Dec 29, 1998, use short local_gran_id for L3 files
	rm_path(fname_L1b, local2);
	strcat( local, ", ");
	strcat( local, local2); */
/* to short the local_gran_id, Z. Wan, May 20, 2000 */
	strcat( local, ",");
	strcat( local, &( gran_link.local_gran_id[9] ) );
	p1 = local;
  }
  else if ( level == 2 )
		p1 = UR_L1b;
       else
	        /* Z. Wan, Dec 29, 1998, use short local_gran_id for L3 files */
        	p1 = &( gran_link.local_gran_id[9] );	/* Z. Wan, 05/20/00 */

  if(PGS_MET_SetAttr( mdhandles[2], "LocalInputGranuleID", &p1 ) == FAIL )
    HALT("FAILURE putting LocalInputGranuleID");

  /* R. Cember: add AncillaryInputType and AncillaryInputPointer with use
     according to current ECS metadata model. */
 
  if (PCLOG == PCLogL2) { 
    p = UR_geo;
    if(PGS_MET_SetAttr( mdhandles[1], "AncillaryInputPointer.1", &p ) == FAIL )
        HALT("FAILURE putting AncillaryInputPointer");
    strcpy(p, "Geolocation");
    if(PGS_MET_SetAttr( mdhandles[1], "AncillaryInputType.1", &p ) == FAIL )
        HALT("FAILURE putting AncillaryInputType");
  }  

  if ( level == 3 ) {

  /* set the shortnamd and tile information */

  if ( PCLOG == PCLogL3_1km )
	sprintf( MoreCodeStgVals[0], "%s", "MOD11_1km_L3");
  else
	sprintf( MoreCodeStgVals[0], "%s", "MOD11_6km_L3");
  local = MoreCodeStgVals[0];

  /* R. Cember: shortname & gridtype is now set by the MCF, per current ECS
  metadata model. 
  if(PGS_MET_SetAttr(mdhandles[1], MoreCodeStg[0], &local)!=PGS_S_SUCCESS)
	SMFLOG("PGS_MET_SetAttr -- Error adding shortname to arch meta"); */

  sprintf( MoreCodeStgVals[1], "%.2d", tile_pntr[i_tile].ih);
  sprintf( MoreCodeStgVals[2], "%.2d", tile_pntr[i_tile].iv);
  /*strcpy( MoreCodeStgVals[3], "INTEGERIZED SINUSOIDAL");*/

   for ( j = 1; j < 4; j++ ) {

    local = MoreCodeStgVals[j];

    /* R. Cember: shortname & gridtype is now set by the MCF, skip it here */
    if (j==3) continue;

    if(PGS_MET_SetAttr(mdhandles[2], MoreCodeStg[j], &local)!=PGS_S_SUCCESS)
	SMFLOG("PGS_MET_SetAttr -- Error adding arch meta data");
   } 
  }
 }
 else {
/* C. Collins, 11/4/02, change TILEID to TileID and its leading character 
   from 1 to 5 for SIN processing */
   strcpy(p, "TileID");
   if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.1", &p ) == FAIL)
	HALT("FAILURE putting AdditionalAttributeName.1");
   if ( set->proj == LBIN_ISIN )
	sprintf(p, "11%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
   else
	sprintf(p, "51%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);
   if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.1", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.1");
 }
}

 sd_id = outfile->sd_id;

 if(PGS_MET_Write(mdhandles[1], CORES, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing CORE");
 if ( PCLOG != PCLogupdate[i_days_in_cycle_update-1] ) {
    if(PGS_MET_Write(mdhandles[2], ARCHIVES_OUT, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing ARCHIVES");
 }

 PGS_MET_Remove();

}

int32 accum_input_ptrs (char *InputPtrElem, const char *InputString)
/*
!C*****************************************************************************

!Description:

This is a service function for DoMetaData.  It accumulates the 
InputGranulePointers, InputtilePointers and LookupTablePointers into one
array so they can be conveniently put into the InputPointer metadatum.

!Input Parameters:
  char *InputPtrArray   
  char *InputPtrElement

!Output Parameters:  
  None

!Returns:
  The number of elements so far accumulated.

!Revision History:

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

   This function written by Richard P. Cember, SDST, 29 January 1998

!References and Credits:

!Design Notes:

This is basically a kludge to adapt PGE16/MOD_PR11 to ECS metadata requirements
which were not well known when the code was written.

It is expected that the last function call within each call to DoMetaData
will contain a NULL InputPtrElement per TK User's Guide p. 6-60.  The 
current number of elements, including the final null, is returned on each call.
When the final null is passed in, the PtrIndex is reset for the next time 
DoMetaData itself is called.

!END***************************************************************************
*/
{

  static int32	PtrIndex = 0;
  int32		NumPtrs;
  char		test_string[1] = {'\0'};

  strcpy(InputPtrElem, InputString);

  PtrIndex++;
  NumPtrs = PtrIndex; 

  /* Get set for the next pass through DoMetaData. */
  if ( strcmp(InputString, test_string) == 0) PtrIndex = 0;

  if ( NumPtrs == MAX_INPUT_PTRS ) {
    free_lst_space();
    SMFLOG("STOP--NumPtrs reachs the limit");
  }

  return (NumPtrs);

}

/*
!C*****************************************************************************

!Description:

This function, get_meta_attr, is used to get an attribute of metadata
which is stored in main memory.  It is more efficient than getting a
metadata attribute from the HDF file.

!Input Parameters:

        s			pointer to a metadata string
        p			pointer to the attribute name string
        type			type of the attribute (1 for char,
				2 for int32, 3 for float64)
	n			number of the values of the attribute

!Output Parameters:

	d			pointer to the returned attribute values

!Revision History:

  $Log: get_meta_attr() $
 * Revision 2.00	 1997/11/20  21:55     Z. Wan
 *

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:

!Design Notes:

!END***************************************************************************
*/

#define OK	0

int32 get_meta_attr (char *s, char *p, int32 type, int32 n, char *d)
{
	char	*s1, *s2, *s3;
	char	pt[64];
	char	*s_o;
	int32	class, i_class;
	int32	i_c_match;
	int32	i, j, k;

	int32	*i_o;
	double	*d_o;

	k = strlen (p);

	i_class = 0;

	for (i = 0; i < k; i++) {
		if (p[i] == '.') {
			sscanf (&p[i+1], "%d", &i_class);
			if ( i_class == 0 )	return ( FAIL );
			break;
		}
		pt[i] = p[i];
	}
	pt[i] = '\0';

	if ( (s1 = strstr (s, pt)) == NULL)
		return ( FAIL );

	i_c_match = 1;

	if ( i_class ) {
		i_c_match = 0;
		for ( i = 0; i < i_class; i++) {
			s2 = strstr (s1, "CLASS");
			s3 = strstr (s2, "= ");
			sscanf (s3+3, "%d", &class);
			if ( i_class == class ) {
				i_c_match = 1;
				s1 = s3;
				break;
			}
			else {
				s1 = s3+3;
				s2 = strstr (s1, "END_OBJECT");
				s3 = strstr (s2, pt);
				s1 = s3 + 1;
				if ( (s2 = strstr (s1, pt)) == NULL)
					return ( FAIL );
				s1 = s2;
			}
		}
	}

	if ( ! i_c_match ) return ( FAIL );

	s2 = strstr (s1, "VALUE");
	s3 = strstr (s2, "= ");

   if ( type == 1 ) {

	s_o = (char *) &d[0];
	s3 += 3;
	for ( i = 0; i < n; i++ ) {
		for ( j = 0; ; j++) {
			if (s3[j] == '"') {
				s_o[j] = '\0';
				break;
			}
			s_o[j] = s3[j];
		}
	}
   }
   else if ( type == 2 ) {
	  
	  i_o = (int32 *) &d[0];
	  s3++;

	  if ( n == 1 ) {
		sscanf (s3, "%d", &i_o[0]);
	  }
	  else {
	    for ( i = 0; i < n; i++ ) {

		if ( i )
			s2 = strstr (s3, ",");
		else
			s2 = strstr (s3, "(");
		s3 = s2;

		sscanf (s3+1, "%d", &i_o[i]);
		s3++;
	    }
	  }
	}
	else if ( type == 3 ) {

		d_o = (double *) &d[0];
		s3++;

		if ( n == 1 ) {
			sscanf (s3, "%le", &d_o[0]);
		}
		else {
		   for ( i = 0; i < n; i++ ) {
			if ( i )
				s2 = strstr (s3, ",");
			else
				s2 = strstr (s3, "(");
			s3 = s2;

			sscanf (s3+1, "%le", &d_o[i]);
			s3++;
		   }
		}
	     }
	     else	return ( FAIL );

	return ( OK );
}

void
SDsetattr_LST_V (int32 sd_id, char *name, int32 nt, int32 count, float32 *data)
/*
!C***************************************************************************

!Description:
   This function sets SDS attribute

!Input Parameters:	sd_id 	file or SDS id number
			name 	pointer to attribute name
			nt 	data type
			count 	number of attributes to be set
			data 	attributes to be set
		     
!Output Parameters:	none


!Revision History:

   * Revised  05/20/1999	17:50	 Z. Wan
   * Revision V2.01 1997/11/24	16:14:06 Z. Wan
   * make vrange & fillv to be compatible with field data type.
   *
   * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 09-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{

  PGSt_SMF_status ret = 1;
  uint8		u8_vrange[2], u8_fillv;
  uint16	u16_vrange[2], u16_fillv;
  int32		i32_vrange[2], i32_fillv;
  float32	f32_vrange[2], f32_fillv;

  if ( ! strcmp(name, "valid_range") ) {

    switch ( nt ) {

    case DFNT_UINT8: u8_vrange[0] = (uint8) data[0];
    		     u8_vrange[1] = (uint8) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) u8_vrange);
		     break;

    case DFNT_UINT16: u16_vrange[0] = (uint16) data[0];
   		     u16_vrange[1] = (uint16) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) u16_vrange);
		     break;

    case DFNT_INT32: i32_vrange[0] = (int32) data[0];
		     i32_vrange[1] = (int32) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) i32_vrange);
		     break;

    case DFNT_FLOAT32: f32_vrange[0] = data[0]; f32_vrange[1] = data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) f32_vrange);
		     break;
    }
  }
  else if ( ! strcmp(name, "_FillValue") && *data != -1) {

    switch ( nt ) {

    case DFNT_UINT8: u8_fillv = (uint8) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &u8_fillv);
		     break;

    case DFNT_UINT16: u16_fillv = (uint16) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &u16_fillv);
		     break;

    case DFNT_INT32: i32_fillv = (int32) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &i32_fillv);
		     break;

    case DFNT_FLOAT32: f32_fillv = (float32) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &f32_fillv);
		     break;
    }
  }

  if ( ret == FAIL)
	HALTsmf(MODIS_E_ATTR_FAIL, name);
}

void sec_to_hrs (float64 *s, int32 *h, int32 *m)
/*
!C***************************************************************************

!Description:
   convert total seconds to hours, minutes, and seconds

!Input Parameters:	s 	pointer to variable total seconds
			h 	pointer to variable hours
			m 	pointer to variable minutes
		     
!Output Parameters:	s, h, m		the same pointers


!Revision History:

   * Version V2.02 1999/03/01   K.S Tai
   * Vision V2.01 1997/12/15	14:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-15-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	*h = *s / 3600;
	*s -= *h * 3600;
	*m = *s / 60;
	*s -= *m * 60;
        if ( *h >= 24. ) *h -=24.; /* get proper date/time range, KSTai, 3/1/99 */
}

uint8 u8getbits (uint8 x, int32 p, int32 n)
/*
!C***************************************************************************

!Description:
    This function gets the (right adjusted) n-bit field of x that begins at
    position p.  The bit position 0 is at the right end.

!Input Parameters:	x	uint8
			p	the beginning bit posision
			n	number of bits to be checked
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	uint8	x1;

	x1 = x >> (p+1-n) & ~(~0 << n);

	return (x1);
}

void u8setbits (uint8 *x, int32 p, int32 n, uint8 y)
/*
!C***************************************************************************

!Description:
    This function sets value y to the (right adjusted) n-bit field of *x that
    begins at position p.  The bit position 0 is at the right end.

!Input Parameters:	x	pointer to uint8
			p	the beginning bit posision
			n	number of bits to be set
			y	the value to be set
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	int32	pn;
	uint8	y1, z;

	pn = p + 1 - n;
	y1 = (y & ~(~0 << n)) << pn;
	z = ~((~0 >> pn) & ~(~0 << n) << pn);
	*x &= z;
	*x |= y1;
}

uint16 u16getbits (uint16 x, int32 p, int32 n)
/*
!C***************************************************************************

!Description:
    This function gets the (right adjusted) n-bit field of x that begins at
    position p.  The bit position 0 is at the right end.

!Input Parameters:	x	uint16
			p	the beginning bit posision
			n	number of bits to be checked
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	uint16	x1;

	x1 = x >> (p+1-n) & ~(~0 << n);

	return (x1);
}

void u16setbits (uint16 *x, int32 p, int32 n, uint16 y)
/*
!C***************************************************************************

!Description:
    This function sets value y to the (right adjusted) n-bit field of *x that
    begins at position p.  The bit position 0 is at the right end.

!Input Parameters:	x	pointer to uint16
			p	the beginning bit posision
			n	number of bits to be set
			y	the value to be set
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	int32	pn;
	uint16	y1, z;

	pn = p + 1 - n;
	y1 = (y & ~(~0 << n)) << pn;
	z = ~((~0 >> pn) & ~(~0 << n) << pn);
	*x &= z;
	*x |= y1;
}

#include "cakecut.h"    /* <== IJMAX is defined in here */

/* Mod by R. Cember, SDST, 13 January 1997.  Preprocessor directives
   added to avoid equality testing of floating point numbers.  From the book 
   "The Elements of C Programming Style", by Jay Ranade and Alan Nash, 
   McGraw-Hill, 1993, p. 124.
*/

#define TOL_FACTOR 2.0
#define EPS2 (DBL_EPSILON * TOL_FACTOR)
#define EQ(x,y)  (fabs((x)-(y)) <= fabs((x)+(y))*EPS2)

/* DBL_EPSILON is essentially the fractional precision inherent in a double
   for the particular machine on which you will be compiling.  It is defined
   in the header float.h.  The code itself is machine-independent.  You
   decide how many times the absolute fractional precision you want to have
   as the interval around zero to be considered equality.  I've chosen 
   twice the smallest representable fractional precision (TOL_FACTOR of 2.0).
*/

/* Mod by R. Cember, SDST, 27 January 1997; add function prototype. */
/* void Error(const char *, const char *); */

void cakecuth(Vertices p, const int32 nm, Polygon q[], int32 *im)
/******************************************************************************
!C

!Description: 
  'cakecuth' cuts out *im horizontal strips (rows) from an observation (with
  the corners defined by p).  The row boundaries are in the [0,*im] range.
  The row indices range from 0 to *im-1.

  It first finds the starting vertex with ymin.  Then marches along the p's 
  clockwise until it hits the end vertex (with ymax) in the last row.  Then it
  marches counter-clockwise to the same end point.  The intersecting polygons
  in the rows are recorded in the polygon structure q (q.v are in the same
  sense of rotation as the input p's).

  q.n is the number of vertices in a polygon.  It is < 3 if there is no
  intersection in the row.

  Any area below y=0 is included to the first row so all input p's should 
  lie above y=0.
 
!Input Parameters:
  p[nm][2]  locations of the corners of the observation in local cell
            coordinates, where the cell vertex (0,0) being the origin.
	    They should be scaled already.
  nm        number of corners in p (typically 4)

!Output Parameters:
  q[*im]     polygons
  *im	    number of rows or strips.  q[*im] and beyond contain no polygons.
            The row index: [0 to *im-1].  p is contained in a subset of this
	    range.

!Revision History:

  Revision 3.0  Aug 31, 2001
  Z. Wan, UCSB, a bug was fixed in cakecutv() for pixels in strang shapes.

  Revision 2.1  December 1998
  Z. Wan, UCSB, for using it in the MODIS LST code.

  Revision 2.0  January 1997
  R. Cember, SDST, for use of preprocessor directive EQ() and function error()

  Original version:  4/25/1996
  Y-T Lau,  U of Maryland, NASA/GSFC/920.2

!Team-unique Header:

  This software was adapted by:
  Zhengming Wan for the National Aeronautics and Space Administration,
  Goddard Space Flight Center, under NASA contract NAS5-31370.
 
!END
******************************************************************************/
{
  int32 k, k1, na, nb;
  int32 n;        /* index of current p */
  int32 n1;       /* either = n+1 or n-1 */
  int32 j;        /* index of current row */
  int32 ja;       /* index of starting row */
  int32 j1;       /* index of next row = j+1 */
  int32 jm;       /* total number of output rows (= *im here) */
  int32 m;        /* counter for vertices within each row or strip.
		 Index assignment for the two marching:
		 m = 0 <- 1 <- 2 <- 3 -- 4 -> 5 -> 6 -> 7
		 clockwise: starting from m=4
		 counter-clockwise: starting m=3 */
  double s, ymin, ymax;

  /* first find ymin, ymax */
  ymin = ymax = p[0][1];
  na = nb = 0;
  for (n = 1; n < nm; n++) {
    if (p[n][1] < ymin) { ymin = p[n][1];  na = n; }
    if (p[n][1] > ymax) { ymax = p[n][1];  nb = n; }
  }

  /* set bounds for j */

  ja = floor( ymin );
  *im = jm = ceil( ymax );

  ja = ja < 0 ? 0 : ja;           /* cap by 0 */

  if (jm > IJMAX)	Error("Number of row over IJMAX","cakecuth");

  for (j = 0; j < jm; j++)	q[j].n=0;	/* initialization */

  if (jm == ja+1) {		/* set q.v = p  and return */
    for (n = 0; n < nm; n++) {
      q[ja].v[n][0] = p[n][0];
      q[ja].v[n][1] = p[n][1];
    }
    q[ja].n = nm;
    return;
  }

  m = 4;			/* starting m index going clockwise */
  q[ja].v[m][0] = p[na][0];	/* save to vertex */
  q[ja].v[m][1] = p[na][1];

  /* Marching clockwise from na to nb */
  for (j = ja, n = na ; n != nb && j < jm; ) {
    n1 = (n+1+nm) % nm;
    j1 = j+1;
    m++;			/* increment vertex number in this row */
    if ( (p[n][1]-j1)*(p[n1][1]-j1) <= 0. ) {
      /* intersecting y=j1, set vertex in current row 
	 and next row */
      s = (p[n1][1]-p[n][1]);
      if ( EQ(s, 0.) )	q[j1].v[4][0] = q[j].v[m][0] = p[n1][0]; 
      else		q[j1].v[4][0] = q[j].v[m][0] = 
			  p[n][0] + (p[n1][0]-p[n][0])*(j1-p[n][1])/s ;
      q[j1].v[4][1] = q[j].v[m][1] = j1;
      q[j].n = m-3;		/* set current vertex number in this row */
      j = j1;			/* increment row number */
      m = 4;			/* reset second m for next row*/
      /* next n */
      if ( EQ(p[n1][1], (double)j1) )	n = n1;
    }
    else {			/* not intersecting y=j1 */
      n = n1;			/* next n */
      q[j].v[m][0] = p[n][0];	/* set vertex in current row */ 
      q[j].v[m][1] = p[n][1];

    }
  }
  q[j].n = m-3;			/* # in last row */

  /* Marching counter-clockwise from na to nb */
  n = na;	m = 4;

  for (j = ja; n != nb && j < jm; ) {
    n1 = (n-1+nm) % nm;
    j1 = j+1;
    if ( (p[n][1]-j1)*(p[n1][1]-j1) <= 0. ) {
      /* intersecting y=j1, set vertex in current row 
	 and next row */
      m--;			/* decrement vertex number in this row */
      s = (p[n1][1]-p[n][1]);
      if ( EQ(s, 0.) )	q[j1].v[3][0] = q[j].v[m][0] = p[n1][0];
      else		q[j1].v[3][0] = q[j].v[m][0] = 
			  p[n][0] + (p[n1][0]-p[n][0])*(j1-p[n][1])/s ;
      q[j1].v[3][1] = q[j].v[m][1] = j1;

      if ( EQ(p[n1][1], (double)j1) ) { 
	n = n1;			/* p on j1 line, next n */
	if (n1 == nb)	m++;	/* take away last p for this march */
      }

      q[j].n += 4-m;		/* set current vertex number in this row */
      if (m > 0) {		/* shift the starting vertex if needed */
	for (k = 0; k < q[j].n; k++) {
	  k1 = k+m;
	  q[j].v[k][0] = q[j].v[k1][0];
	  q[j].v[k][1] = q[j].v[k1][1];
	}
      }
      j = j1;		/* increment row number */
      m = 3;		/* reset second m for next row*/

    }
    else {			/* not intersecting y=j1 */
      if (n1 == nb)	break;	/* Break out from loop at last p. 
			         Do not set last p for this march
				 since it is already set before */
      n = n1;			/* next n */
      m--;			/* decrement vertex number in this row */
      q[j].v[m][0] = p[n][0];	/* set vertex in current row */
      q[j].v[m][1] = p[n][1];
    }
  }

  /* fix the last row */
  q[j].n += 4-m;		/* # in last row */
  if (m > 0) {		/* shift the starting vertices for the last row */
    for (n = 0; n < q[j].n; n++) {
      n1 = n+m;
      q[j].v[n][0] = q[j].v[n1][0];
      q[j].v[n][1] = q[j].v[n1][1];
    }
  }

}

void cakecutv(Vertices p, const int32 nm, double a2[], int32 *im)
/******************************************************************************
!C

!Description: 
  'cakecutv' is similar to 'cakecuth' except it does vertical cutting within
  a row.  It returns the intersecting area2 in each cell along the row (with
  indices from 0 to im-1).

  Any area in x<0 is included to the first column so all input p's should 
  lie above x=0.

!Input Parameters:
  p[nm][2]  vertices of the polygon found by 'cakecuth' in the current row.
            Should be ordered clockwise or counter-clockwise.
  nm        number of vertices

!Output Parameters:
  a2[im]    area2 (two times the actual area) for grid cells in the row 
            [column index: 0 to im-1]. (a[im] and beyond would be zero.)
  *im	    number of cells

!Revision History:

  Revision 2.1  December 1998
  Z. Wan, UCSB, for using it in the MODIS LST code.

  Original version:  4/25/1996
  Y-T Lau,  U of Maryland, NASA/GSFC/920.2

!Team-unique Header:

  This software was adapted by:
  Zhengming Wan for the National Aeronautics and Space Administration,
  Goddard Space Flight Center, under NASA contract NAS5-31370.

!END
******************************************************************************/
{
  Polygon qv[IJMAX];
  int32 k, k1, na, nb;
  int32 nn;
  int32 n;        /* index of current p */
  int32 n1;       /* either = n+1 or n-1 */
  int32 j;        /* index of current column */
  int32 j1;       /* index of next column = j+1 */
  int32 ja;       /* index of starting column  */
  int32 jm;       /* total number of output columns (= *im here) */
  int32 m;        /* counter for vertices within each column or strip.
		 Index assignment for the two marching:
		 m = 0 <- 1 <- 2 <- 3 -- 4 -> 5 -> 6 -> 7
		 clockwise: starting from m=4
		 counter-clockwise: starting m=3 */
  double s, xmin, xmax;

  if ( nm < 3 ) { *im = 0; return; }

  /* zero input area */
  if ( EQ(area2(p,nm), 0.) ) { *im=0; return; } 

  /* first find xmin, xmax */
  xmin = xmax = p[0][0];
  na = nb = 0;
  for (n = 1; n < nm; n++) {
    if (p[n][0] < xmin) { xmin = p[n][0];	na = n; }
    if (p[n][0] > xmax) { xmax = p[n][0];	nb = n; }
  }

  /* set bounds for j */

  ja = floor( xmin );
  *im = jm = ceil( xmax );

  ja = ja < 0 ? 0 : ja;		/* cap by 0 */

  if (jm > IJMAX)	Error("Number of columns over IJMAX","cakecutv");

  for (j = 0; j < ja; j++)	a2[j]=0.;	/* zero areas before ja */

  if (jm == ja+1) {		/* not crossing vertical lines */
	a2[ja] = area2(p,nm);
	return;
  }

  for (j = ja; j < jm; j++ )	qv[j].n = 0;	/* initialization */

  m = 4;			/* starting m index going clockwise */
  qv[ja].v[m][0] = p[na][0];	/* save to vertex */
  qv[ja].v[m][1] = p[na][1];

  /* Marching clockwise from na to nb */
  for (j = ja, n = na ; n != nb && j < jm; ) {
    n1 = (n+1+nm) % nm;
    j1 = j+1;
    m++;                     /* increment vertex number in this column */
    if ( (p[n][0]-j1)*(p[n1][0]-j1) <= 0. ) {
      /* intersecting y=j1, set vertex in current column 
	 and next column */
      s = (p[n1][0]-p[n][0]);
      if ( EQ(s, 0.) )		qv[j1].v[4][1] = qv[j].v[m][1] = p[n1][1];
      else			qv[j1].v[4][1] = qv[j].v[m][1] = 
				  p[n][1] + (p[n1][1]-p[n][1])*(j1-p[n][0])/s ;
      qv[j1].v[4][0] = qv[j].v[m][0] = j1;
      qv[j].n = m-3;		/* set current vertex number in this column */

      j = j1;			/* increment column number */
      m = 4;			/* reset second m for next column*/
      /* next n */
      if ( EQ(p[n1][0], (double)j1) ) n = n1;

    }
    else {
      n = n1;			/* next n */
      qv[j].v[m][0] = p[n][0];	/* set vertex in current column */
      qv[j].v[m][1] = p[n][1];
    }
  }
  qv[j].n = m-3;			/* # in last column */

  /* Marching counter-clockwise from na to nb */
  m = 4;
  for (j = ja, n = na ; n != nb && j < jm; ) {
    n1 = (n-1+nm) % nm;
    j1 = j+1;
    if ( (p[n][0]-j1)*(p[n1][0]-j1) <= 0. ) {
      /* intersecting y=j1, set vertex in current column 
	 and next column */
      m--;			/* decrement vertex number in this column */

      if ( m >= 0 ) {		/* add this by Z. Wan, Aug. 31, 2001 */

        s = (p[n1][0]-p[n][0]);
        if ( EQ(s, 0.) )	qv[j1].v[3][1] = qv[j].v[m][1] = p[n1][1]; 
        else		qv[j1].v[3][1] = qv[j].v[m][1] = 
			   p[n][1] + (p[n1][1]-p[n][1])*(j1-p[n][0])/s ;
        qv[j1].v[3][0] = qv[j].v[m][0] = j1;

      }

      if ( EQ(p[n1][0], (double)j1) ) { 
	n = n1;			/* p on j1 line, next n */
	if (n1 == nb)	m++;	/* take away last p for this march */
      }

      qv[j].n += 4-m;		/* set current vertex number in this column */

      if (m > 0) {		/* shift the starting vertex if needed */
	nn = qv[j].n;
	for (k = 0; k < nn; k++) {
	  k1 = k+m;
	  qv[j].v[k][0] = qv[j].v[k1][0];
	  qv[j].v[k][1] = qv[j].v[k1][1];
	}
      }
      j = j1;			/* increment column number */
      m = 3;			/* reset second m for next column*/

    }
    else {
      if (n1 == nb)	break;	/* Break out from loop at last p. 
			         Do not set last p for this march
				 since it is already set before */
      n = n1;			/* next n */
      m--;			/* decrement vertex number in this column */

      if ( m < 0 ) {	/* add this by Z. Wan, Aug. 31, 2001 */
	 m = 0;
	 break;
      }

      qv[j].v[m][0] = p[n][0];	/* set vertex in current column */
      qv[j].v[m][1] = p[n][1];
    }
  }

  /* fix the last column */
  qv[j].n += 4-m;		/* # in last column */

  if (m > 0) {		/* shift the starting vertices for the last column */
    nn = qv[j].n;
    for (n = 0; n < nn; n++) {
      n1 = n+m;
      qv[j].v[n][0] = qv[j].v[n1][0];
      qv[j].v[n][1] = qv[j].v[n1][1];
    }
  }

  /* compute area2 */
  for (j = ja; j < jm; j++) {
    if ( qv[j].n <= VMAX )
	a2[j] = area2(qv[j].v, qv[j].n );
  }

}

/******************************************************************************
!C

!Description: returns 2 times the polygon area.

!Input Parameters:
	p - vertices of the input polygon
	n - number of corners of the polygon

!Output Parameters:
 
!Revision History:

  Revision 2.1  December 1998
  Z. Wan, UCSB, for the use of x0 and y0

  Original version:  4/25/1996
  Y-T Lau,  U of Maryland, NASA/GSFC/920.2

!Team-unique Header:

  This function was adapted by:
  Zhengming Wan for the National Aeronautics and Space Administration,
  Goddard Space Flight Center, under NASA contract NAS5-31370.
 
!END
******************************************************************************/
double area2(Vertices p, int32 n)
{
  int32	 i, i1, n1, n2;
  double x0, y0;
  double a2 = 0.;

  if (n < 3) return a2;

  x0 = p[0][0];		y0 = p[0][1];

  n1 = n - 1;

  for (i = 1; i < n1; i = i1) {
    i1 = i+1;
    a2 += +(p[i1][0] - x0) * (p[i][1] - y0)
          -(p[i1][1] - y0) * (p[i][0] - x0) ;
  }
  return a2;
}

float64 bt_to_radiance (int32 ic, float64 tb, int32 i_mod)
{
/*
!C*****************************************************************************

!Description:
   This function gives MODIS band radiance at given brightness temperature
   based on the MODIS PFM relative spectral response, version C.

!Input Parameters:	ic - index (0-6) for MODIS bands 20, 22, 23, 29, 31-33
			tb - band brightness temperature (K).
			i_mod - 1 for Aqua FM1 MODIS data

!Output Parameters:	none

 Returns:

	rr - band radiance in W/m^2/mu/sr.

!Revision History:

!References and Credits:

   Version v2.1 2002/10/12 22:15    Z. Wan

   Version v1.2 2001/04/21 21:29:30 Z. Wan
    also renamed from planck()

   Version v1.1 2000/05/15 02:02:41 Z. Wan

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	ii, ik;
	double	t2, rr;

	t2 = (tb - TMIN) / DT_LUT;

	ii = t2;
	if (ii < 0)		ii = 0;
	if (ii > NCELM2)	ii = NCELM2;
	ik = ii + 1;

	if ( i_mod )
	  rr = _mod_fm1_rad7[ic][ii] +
		(_mod_fm1_rad7[ic][ik] - _mod_fm1_rad7[ic][ii]) * (t2 - ii);
	else
	  rr = _mod_rad7[ic][ii] +
		(_mod_rad7[ic][ik] - _mod_rad7[ic][ii]) * (t2 - ii);

	return (rr);
}

/* a short program to setup __ctype_b, __ctype_toupper and __ctype_tolower
   written by Scott Burns, Mirrabooka Systems.
   It is downloaded from http://www.dbforums.com/t964847.html
   by
   Zhengming Wan, ICESS, University of California, Santa Barbara, CA 93106
   It is used only #ifdef _LINUX_ct
*/

#ifdef _LINUX_ct

#include <ctype.h>

__const unsigned short int *__ctype_b;
__const __int32_t *__ctype_tolower;
__const __int32_t *__ctype_toupper;

void ctSetup()
{
	__ctype_b = *(__ctype_b_loc());
	__ctype_toupper = *(__ctype_toupper_loc());
	__ctype_tolower = *(__ctype_tolower_loc());
}

#endif
