/*
!C*****************************************************************************

!Description:
   Main program of MODIS Land-Surface Temperature (LST) Version 5.2.1, part 2

   It reads interim results in UPD files and retrieves surface emissivities
   and LSTs with the day/night LST algorithms.
   It writes metadata for the MOD11B1 output file and free all allocated space
   before completion of the PGE.

!Input Parameters:
   PCLogupdate & upd_fname	for the UPD file from MOD_PR11.pcf
   PCLogL3_6km & fname_6km_L3	for the MOD11B1 file from MOD_PR11.pcf
   PCLogsollut & sol_fname	for the sol_term_tbls from MOD_PR11.pcf
   PCLogtirlut & tir_fname	for the tir_term_tbls from MOD_PR11.pcf

!Output Parameters:


!Revision History:

   $Log: mod11_lstB1.c,v $
 * Revision 5.2.2 2005/11/23  22:42     Z. Wan
 *	Nothing changed
 *	Set "SCF V5.2.2B"
 * Revision 5.2.1 2005/09/19  22:42     Z. Wan
 *	Nothing changed
 *	Set "SCF V5.2.1B"
 * Revision 5.2.0 2005/06/19  22:42     Z. Wan
 *	Nothing changed
 *	Set "SCF V5.2.0B"
 * Revision 5.1.0 2005/05/11  10:21     Z. Wan
 *	Update QA flag assignments
 *	Set "SCF V5.1.0B"
 * Revision 5.0.4 2005/04/04  11:36     Z. Wan
 *	Use V5.0.4 update_for_lst.h
 *	Set "SCF V5.0.4B"
 * Revision 5.0.2 2004/10/21  22:07     Z. Wan
 *	Use V5.0.2 update_for_lst.h (N2_VIEW_ANGS = 16) and lst_1dy_data.h
 *	Fully coupling the split-window method to the day/night method
 *	Set "SCF V5.0.2B"
 * Revision 5.0.1 2004/11/11		C. Collins
 *	Set value of AlgorithmPackageVersion as string
 * Revision 5.0.0 2004/09/20  16:40     Z. Wan
 *	Add a default option for completely combined use of Terra & Aqua data
 *	 specify tile version # in argv[1] [ and n_upd_sets in argv[2] if 2 ]
 *	Use new update_for_lst.h for 6km grids, set N_VIEW_ANGS to 6
 *	Streamline the procedure searching for day/night pairs
 *	Set "SCF V5.0.0B"
 * Revision 4.1.7 2002/12/24  15:58     Z. Wan
 *	Fix a bug that overestimates the effect of nearby clouds
 *	Consider the dominant quantization error in band 20 in cold conditions
 *	Set "SCF V4.1.7B"
 * Revision 4.1.5 2002/12/16  22:11     Z. Wan
 *	Update the bit 00-01 value in QC_5km_day and QC_5km_nit
 *	Set "SCF V4.1.5B"
 * Revision 4.1.3 2002/11/28  11:50     Z. Wan
 *	Use updated lst_1dy_data.h and MOD11B1.fs (v4.1.3)
 *	Set "SCF V4.1.3B"
 * Revision 4.1.1 2002/11/10  23:41     Z. Wan
 *	Change Day_view_angl and Night_view_angl in MOD11B1 and its file spec
 *	 so that they are negative if viewing from east
 *	Set "SCF V4.1.1B"
 * Revision 4.1.0 2002/10/12  21:50     Z. Wan
 *	Use new lst_1dy_data.h and update_for_lst.h (V4.1.0)
 *	Use Terra and Aqua combined sol_term & tir_term LUTs
 *	#include "mod.fm1.rad7.h" for Aqua MODIS data
 *	Store info of viewing angles and data source to QC_Emis
 *	Set "SCF V4.1.0B"
 * Revision 4.0.1 2002/06/08  00:57     Z. Wan
 *	Set "SCF V4.0.1B"
 * Revision 4.0.0 2002/04/29  21:40     Z. Wan
 *	Use modified update_for_lst.h and lst_1dy_data.h (V4.0.0)
 *	Set initial value of the anisotropic factor of the solar beam BRDF
 *	 at the surface with the BRDF Albedo Parameters for band 7 in MOD43B1
 *	Set "SCF V4.0.0B"
 * Revision 3.1.3 2002/04/27  18:00     Z. Wan
 *	Set "SCF V3.1.3B"
 * Revision 3.1.0 2002/01/26  11:26     Z. Wan
 *	Set "SCF V3.1.0B"
 * Revision 3.0.3 2001/08/31  16:52     Z. Wan
 *	Set "SCF V3.0.3B"
 * Revision 3.0.2 2001/08/19  16:52     Z. Wan
 *	To get esdt_5km_L3 (ESDT for the 5km LST product: MOD11B1 for Terra
 *	 or MYD11B1 for Aqua) from its file in PCF.
 *	Deleted line 2564.
 *	Set met_cnt_5km.QA_NotProducedOther to 0 if it has a negative value.
 *	Added the modifications in SMFLOG() made by C. Collins on 6/8/01.
 * Revision 3.0.1 2001/06/01  18:14     Z. Wan
 *	Updated day_nit_lut.h (v3.0.1)
 *	Updated LUTs sol_term_4tbls_v3.0.0 & tir_term_4tbls_v3.0.0 to
 *	 sol_term_4tbls_v3.0.1 & tir_term_4tbls_v3.0.1
 *	Modified MOD11B1.fs, QC_Day & QC_Night flags in MOD11B1
 * Revision 3.0.0 2001/05/04  09:21     Z. Wan
 *	The original mod11_lst2.c was separated into two parts.  This is the
 *	 second part for the production of MOD11B1 product with the
 *	 the day/night LST algorithm.
 * Revision 2.5.1 2001/04/29  21:30     Z. Wan
 *	LUT mod_rad7_lut.h was replaced by mod.pfm-vC.rad7.h with a wider
 *	 temperature range from 180K to 400K.
 *	LUT day_nit_lut.h was replaced by day_nit_lut21x15.h for wider
 *	 ranges of atmospheric temperature and water vapor conditions.
 *	LUT f_stat_lut.h was removed because a modified TISI (temperature
 *	 independent spectral index method was used to replace the
 *	 statistical regression method for providing initial guess for
 *	 day/night LST algorithm.
 * Revision 2.4.1 2001/03/24  10:44     Z. Wan
 *	To make this code Terra and Aqua compatible.  It is assumed that
 *	 the names of all input and output files are started with "MOD" in
 *	 the processing of Terra data and with "MYD" in the Aqua processing.
 *	 "PM-1" is used for Aqua's ASSOCIATEDPLATFORMSHORTNAME.
 * Revision 2.3.6 2001/03/13  16:29     Z. Wan
 *	To change the scale from 0.0001 to 0.0004 in cal_1km_l3[10] & [11],
 *	 for Clear_day_cov & Clear_night_cov in the 1km L3 LST product.
 *	To do u8setbits for an initial qc_1km_l3 even if cell_cov <= 0.66.
 * Revision 2.3.4 2001/01/08  15:39     Z. Wan
 *	To update the option to correct the effect of calibration errors in
 *	 bands 31 and 32 (viewing angle dependent dt_avg_b31_b32 and
 *	 dt_dif_b31_b32 based on V2.5.4 L1B data and the vicarious calibration
 *	 field campaign over Lake Titicaca, June 2000) on the MOD11_L2 product.
 *	 This option can be turned off by setting a run-time parameter in PCF
 *	 as 211315|TURN_OFF_CALIBRATION_CORRECTION|1.
 * Revision 2.3.2 2000/12/09  09:57     Z. Wan
 *	To fix the Anomolously high LST value problem posted in JB_MOD11_00340
 *	 on the LDOPE website (this problem would be avoid if turning
 *	 IGNORE_UNCERT off with a line 211314|IGNORE_UNCERT|0 in PCF, or
 *	 if SDS EV_1KM_Emissive values are given as _FillValue 65535 or -1
 *	 rather than 32765 in the valid range in
 *	 MOD021KM.A2000252.1540.002.2000259114748.hdf, not a normal L1B file,
 *	 in which all "%Valid EV Observations" are zero), ignore granules with
 *	 "%Valid EV Observations" < 0.0001 in bands 31 and 32, and print a
 *	 message to LogReport file.
 *	To change the lower boundary of the valid range to 1 from 0 for
 *	 Columns_per_Global_Grid_Row in MOD11A1 and MOD11B1 (in response to
 *	 JB_MOD11_00335).
 * Revision 2.3.1 2000/12/02  17:50     Z. Wan
 *	To make the code default for B-side data, add runtime parameter line
 *	 211316|IS A-SIDE DATA|1 in PCF for processing A-side data.
 *	To fix the Buffer error problem when working on partly bad granules or
 *	 titled granules with Sector Rotation EV Data (such as in A2000291.1500,
 *	 A2000291.1640, and A2000291.1645), IJMAX was increased to 192 from 24
 *	 in cakecut.h and MAX_NOBS was increased to 1020 from 255 in
 *	 obsgrid_lst.h.
 * Revision 2.2.21 2000/09/30  13:19     Z. Wan
 *	To add an option to correct the effect of calibration errors in bands
 *	 31 and 32 (b31_dt = 0.18, b32_dt = -0.15 as results of the vicarious
 *	 calibration field campaign over Lake Titicaca, June 2000) on the
 *	 MOD11_L2 product.  This option can be turned off by setting a run-time
 *	 parameter in PCF as 211315|TURN_OFF_CALIBRATION_CORRECTION|1.
 * Revision 2.2.20 2000/09/14  22:44     Z. Wan
 *	To add two SDSs to the MOD11B1 output (see MOD11B1.fs and lst_1dy_data.h),
 *	 one is LST_Day_5km_Aggregated_from_1km, another is
 *	 LST_Night_5km_Aggregated_from_1km.  They are based on MOD11A1 and
 *	 can be used in the global LST browse image processing.
 *	To replace dimensions 10*nscans, Max_EV_frames, 2*nscans, and
 *	 Max_EV_frames/5 with Along_swath_lines_1km, Cross_swath_pixels_1km,
 *	 Coarse_swath_lines_5km, and Coarse_swath_pixels_5km, respectively.
 * Revision 2.2.16 2000/08/10  10:01     Z. Wan
 *	To fix the "Fatal error in ... getMODISarray from upd_ang_lst" problem
 * Revision 2.2.14 2000/07/18  23:11     Z. Wan
 *	To fix the MOD11_L2 QAPERCENTMISSINGDATA problem per Mike Linda's
 *	e-mail and LDOPE case number SD_MOD11_060100A
 * Revision 2.2.11 2000/05/20  2:10     Z. Wan
 *	To rectify the OperationalQualityFlag problem per David Roy and Mike
 *	Linda's e-mails (refer DR_MOD11_051800B)
 *	The format of input granule IDs in archive metadata LOCALINPUTGRANULEID
 *	was simplified to avoid failure in the high latitude zones.
 * Revision 2.2.9 2000/04/28  12:17     Z. Wan
 *	To fix the GRING problem (refer DR_MOD11_040500)
 *	To add the new PSA QAFRACTION's after the original ones
 *	 (refer DR_MOD11_033000)
 *	To reduce the stripping caused by the cloudyness due to the noisy 4th
 *	channel of band 22 (refer DR_MOD11_022800)
 *	To set bits 04-05 of QC in MOD11_L2 appropriately (CC_MOD11_032700)
 *	The micro ADJ() was modified
 *	clarification of most possibly confusing points:
 *	a) The value of AllPixelsProcessed is the fixed maximum number of
 *	  pixels or grids possibly in a granule or tile.
 *	b) In MOD11_L2, the value of QAPERCENTCLOUDCOVER is defined by the
 *	  number of all pixels not VeryHighConfidentClear in MOD35, but in
 *	  MOD11A1 and MOD11B1, the value of QAPERCENTCLOUDCOVER is defined
 *	  by the number of land grids not covering any clear-sky LST pixels.
 *	c) The value of QAPERCENTNOTPRODUCEDOTHER is defined by the number
 *	  of pixels or grids skipped by all reasons but clouds.  Ocean pixels
 *	  are skipped before skipping cloudy land pixels.
 *	d) The value of QAPERCENTNOTPRODUCEDCLOUD is defined by the number
 *	  of land pixels or grids skipped due to clouds.
 *	e) The value of QAPERCENTGOODQUALITY is defined by the number of
 *	  pixels or grids with good quality LST.
 *	f) The value of QAPERCENTOTHERQUALITY is defined by the number of
 *	  pixels or grids with LST not in good quality.
 * Revision 2.2.8 2000/04/11  10:29     Z. Wan
 *	four QAFRACTION* of floating point numbers are moved to core metadata
 *	from archive metadata
 *	MOD11_L2.mcf, MOD11A1.mcf, and MOD11B1.mcf modified accordingly
 *	skip those pixels with Fill value for latitude or longitude.
 * Revision 2.2.7 2000/04/07  00:31     Z. Wan
 *	DoMetaData( n_5km_l3_old_file, ...) after DoMetaData( n_old_1km, ...)
 *	to fix the RANGEBEGINNINGTIME or/and RANGEENDINGTIME problem informed
 *	by Nazmi Z El Saleous on March 29, 2000
 * Revision 2.2.5 2000/03/18  15:47     Z. Wan
 *	mod11_lst2.c and lst_1dy_data.h modified to use the SDS Water_Vapor
 *	in MOD07
 * Revision 2.2.4 2000/03/13  11:20     Z. Wan
 *	remove the QC=3 seam in L2 LST output by the edge of tiles (to correct
 *	the problem in QA Case # DR_MOD11_030900)
 * Revision 2.2.3 2000/03/07  21:00     Z. Wan
 *	Per SDST's suggestion in Mike Linda's e-mail, to make things easy
 *	INPUTPOINTER's NUM_VAL will be kept to a fixed value rather than
 *	the true value of the number of inputpointers used in the metadata
 *	so that the MCF files will not be modified during run time.
 * Revision 2.2.2 2000/02/20  22:34     Z. Wan
 *	change the QAPERCENT* values to integers,
 *	add four QAFRACTION* of floating point numbers in archive metadata.
 * Revision 2.2.1 1999/12/29  11:04     Z. Wan
 *	change "SCF V2.1.9" to "SCF V2.2.1"
 *	land_tile_tbl.h updated for the use of the UM at-launch land-cover
 *	 product (version 3).  From the total 328 tiles, only 317 tiles were
 *	 selected as input to the PGE16 processing.  There are land and/or
 *	 coastline grids in these 317 tiles.
 *	band_emis.h was modified correspondingly
 *	lst_1dy_data.h updated for adding SDS Land_Cover_Type_QC
 *	move the calculation of total_l2_pix[] to the location just after
 *	 the Granule_tile loop
 *	DoMetaData() was modified to set a true value to INPUTPOINTER's NUM_VAL
 *	values of N_BYTES_CORE_L3, N_BYTES_ARCH_L3, and MAX_CODE_STG_VALS
 *	 were increased to solve the problem of processing in zones N1 & S1
 *	 (latitude ranges from 60 deg. to 90 deg. and from -90 deg. to -60 deg.)
 *	take DoMetaData() out from the i_scan_loop to avoid the duplicate
 *	 input pointers in metadata
 * Revision 2.1.11 1999/10/28  10:25     Z. Wan
 *      print more message to Logreport
 *        in case where the input granule is an ocean granule
 * Revision 2.1.10 1999/09/25  21:40     Z. Wan
 *	print more message to LogReport
 *	EV_1KM_Emissive_Uncert_Indexes will be ignored if a run-time parameter
 *	  is defined in PCF with 211314|IGNORE_UNCERT|1
 *	change the type of QA_GoodQuality, QA_OtherQuality,
 *	  QA_NotProducedCloud, and QA_NotProducedOther back to FloatVal
 *	  in order to provide useful information for ocean granules that
 *	  may cover only a few land pixels over a small island.
 * Revision 2.1.9 1999/09/17  09:18     Z. Wan
 *	use Day, Night or Both in DayNightFlag instead DAY, NIGHT or BOTH
 *      read PGEVersion (PGE16_Version with LUN 800500) from PCF
 *	add begin_seconds & end_seconds into struct _tile_pointer in
 *	  land_tile_tbl.h
 *	change the type of QA_GoodQuality, QA_OtherQuality,
 *	  QA_NotProducedCloud, and QA_NotProducedOther to Integer from FloatVal
 * Revision 2.1.8 1999/08/11  20:42     Z. Wan
 *	fixed a bug in lines 1391 & 1392 ("core_1B" was changed to "core_Snow")
 * Revision 2.1.7 1999/07/11  13:15     Z. Wan
 *     type of tile_begin_seconds, tile_end_seconds, and seconds_tmp was
 *	changed to float64 from float32.
 *     time format "%02i:%02i:%f" was changed to "%02i:%02i:%09.6f".
 * Revision 2.1.6 1999/05/20  17:50     Z. Wan
 *     use micro HALTsmf() to write error message to LogReport
 *     set VERSIONID to 1 for the at-launch version of code in MCF files
 *     use L1B's PGEVERSION to determine its format
 * Revision 2.1.5 1999/05/17  20:20     Z. Wan
 *     some modifications made in Granule_tile()
 * Revision 2.1.4 1999/05/07  16:10     Z. Wan
 *     It is able to input new and old format L1B data.
 * Revision 2.1.1 1998/04/02   ff.      Z. Wan
 * Revision 2.0  1998/01/24   ff.      R. Cember, SDST
 * Revision 2.03 1998/01/03  02:00     Z. Wan
 * Revision 2.02 1997/12/01  01:56     Z. Wan
 * Revision 2.01 1997/09/15  23:55     Z. Wan
 * Revision 1.3  1997/09/08  23:05     Z. Wan
 * Revision 1.2  1997/08/25  13:13     R. Cember and G. Ye, SDST
 * Revision 1.1  1996/11/20  08:56:56  Wan
 * Revision 1.0  1996/09/25  23:18:29  Wan
 *
 * 1. Output L2 & L3
 * 2. Changed *modoutf1 to *lst_l2_outf
 * 3. Added intermediate binary data file update_for_lst
 * 4. Added day/night LST algorithm
 *
 * Revision B3.2  1995/09/11  11:26:20  wan
 *
 * Zhengming Wan: use generic variable types such as uint16, int32, float32,
 * float64, and etc. except long int required by M-API2.2.1
 * All allocated spaces are freed in normal and abnormal exits.
 * Write SD attributes according to the MOD11 product HDF formats.
 *
 * Revision B3.1  1995/09/01  11:07:01  wan
 * Initial version
 *
   $Id: mod11_lstB1.c, v 3.0.0 2001/05/04 09:21 wan Exp wan $

!Team-unique Header:
   
   This software was developed by:
   Zhengming Wan for the National Aeronautics and Space Administration,
   Goddard Space Flight Center, under NASA contract NAS5-31370.

   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541
   Programmer/Analyst, Xiaoning Duan
   	duan@icess.ucsb.edu
   	805-893-7353
   ICESS
   University of California
   Santa Barbara, CA 93106-3060

   HDF Portions developed at the National Center for Supercomputing
   Applications at the Univ. of Illinois at Urbana-Champaign.

!References and Credits:

   Zhengming Wan and Jeff Dozier, "A generalized split-window algorithm for
     retrieving land-surface temperature from space", IEEE Trans. Geoscience
     and Remote Sensing, vol. 34, no. 4, pp. 892-905, 1996.

   Zhengming Wan and Zhao-Liang Li, "A Physics-based algorithm for retrieving
     land-surface emissivity and temperature from EOS/MODIS data", IEEE
     Trans. Geoscience and Remote Sensing, vol. 35, no. 4, pp. 980-996, 1997.

   Zhengming Wan, MODIS LST ATBD (ATBD-MOD-11), v3.3, April, 1999.

   Zhengming Wan developed this MODIS LST code.  Assistances with integrating
   PGS, M-API toolkits, and HDF was provided by Xiaoning Duan under helps of
   Gang Ye, and Paul Fisher of SDST/STIG.

   The code of writing metadata was implemented by Gang Ye of SDST in V1 and
   was modified by Zhengming Wan for Version 2.  

   R. Cember modified the code to solve the memory leak problem, to replace
   ECS-prohibited functions (such as lseek, read, and write with fseek, fread,
   and fwrite, respectively, on 1/29/97), to fuly prototype all functions, and
   to reduce the level of nesting.

   The adapted subset of the L2G library functions developed by Robert E.
   Wolfe, MODIS Land Team Support Group in Christopher O. Justice's MODIS
   Land Science Team.

!Design Notes:

   This is the phase 1 of the daily LST V2.1 code. MODIS product snow-cover and
   Atmospheric profiles have been used as additional inputs to the MOD11 code.

   Because the snow-cover product is only available for daytime granules and
   day/night pairs on the same day will be used as the first choice for the
   day/night LST method, granules need to be processed in sequence of files
   with DAYNIGHTFLAG as "DAY" in the first place, then "BOTH", and finally
   "NIGHT".

   The cumulative comments have been cleared for clarity.  To see old versions
   for the detailed modifications in different stages.

   1. The version number of the input granule file is given in argv[1] that
      is assigned by the script file for MODIS PGE16.
   2. The reg_id number(s) of selected land-tiles are given in argv[2].  These
      numbers are separated by comma (,) without blank space between them.
      If no such id # is given, all land-tiles are selected.
      See land_tile_tbl.h for the reg_id # of each land tiles.
   3. In order to provide LST values twice a day for all land grids in the
      1km L3 product, daytime is defined as local standard time 6:00 to 18:00.

This source file includes calls to functions of the HDF (Hierarchical Data
Format) applications programming interface libraries, developed and distributed
by the National Center for Supercomputing Applications (NCSA), University of 
Illinois, Urbana-Champaign.

!END**************************************************************************
*/

#define SMF_SOURCE	"main mod11_lstB1.c"

#define USE_ECS		/* ECS meta data handling */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#include "mapi.h"	/* M-API 2.3.1 */

#include "PGS_SMF.h"
#include "PGS_MODIS_37111.h"

#include "smfio.h"

#include "mod.fm1.rad7.h"	/* radiance look-up table for bands 20, 22, 23,
				   29, and 31-33 of Aqua MODIS */
#include "mod.pfm-vC.rad7.h"	/* radiance look-up table for bands 20, 22, 23,
			           29, and 31-33 */
#include "lst_coef.h"	   /* coefficients of the generalized split-window
			      LST algorithm */
#include "band_emis.h"	   /* prototype band emissivity knowledge base */
#include "bit_flag.h"	   /* bit flags for QC */
#include "HdfEosDef.h"
#include "lst_1dy_data.h"
#include "land_tile_tbl.h"

/* for writing MOD_Swath_LST */
#include "PGS_MEM.h"
#include "hntdefs.h"
int32   SWfidout, SWidLST;              /* Swath handles */
int32   hdfID;
int32   LstHDFhandle;

#define	SMALL_VALUE	1.0e-07

/* functions for space allocation and allocated variables */
/* declared as global variables so that they can be freed */
/* both in normal and anormal exits */

char 	   **alloc_2d (int32, int32, size_t);
char 	   ***alloc_3d (int32, int32, int32, size_t);
int32	   z_siz;
int32	   siz1 = 0, siz2 = 0, siz3 = 0;

/* for daily 6km L3 LST */

uint16	**lst_6km_Sd = NULL;
uint8	**QC_6km_day = NULL;
uint8	**v_time_6km_l3_Sd = NULL;
uint8	**v_angl_6km_l3_Sd = NULL;
uint16	**lst_6km_Sn = NULL;
uint8	**QC_6km_nit = NULL;
uint8	**v_time_6km_l3_Sn = NULL;
uint8	**v_angl_6km_l3_Sn = NULL;
uint8	**em20_6km_l3_Sv = NULL;
uint8	**em22_6km_l3_Sv = NULL;
uint8	**em23_6km_l3_Sv = NULL;
uint8	**em29_6km_l3_Sv = NULL;
uint8	**em31_6km_l3_Sv = NULL;
uint8	**em32_6km_l3_Sv = NULL;
uint8	**QC_emis = NULL;
uint16  **lst_6km_from_1km_d = NULL;
uint16  **lst_6km_from_1km_n = NULL;

float64 **coef_mesh = NULL;

/* for the intermediate update file */

#include "update_for_lst.h"

int32	n_old_update = 1;
size_t	upd_siz = SZ_UPD_ANG_LST_SV;

int32	nrow_6km;
int32	ncol_6km;

_UPD_ANG_LST_SV	**upd_ang_lst_Sv[4] = { NULL, NULL, NULL, NULL };
_UPD_ANG_LST_SV	**upd_ang_lst_Sp;
_UPD_ANG_LST_SV	**upd_ang_lst_Dp;
_UPD_ANG_LST_SV	**upd_ang_lst_Np;
_UPD_ANG_LST_SV	*upd_a_lst_S_p = NULL;	/* pointer to upd_ang_lst_S */
_UPD_ANG_LST_SV	*upd_a_lst_D_p = NULL;	/* pointer to upd_ang_lst_S */
_UPD_ANG_LST_SV	*upd_a_lst_N_p = NULL;	/* pointer to upd_ang_lst_S */
_UPD_FOR_LST_SV	*upd_lst_S_p = NULL;	/* pointer to upd_for_lst_S */
_UPD_FOR_LST_SV	*upd_lst_S_p1 = NULL;	/* pointer to upd_for_lst_S */
_UPD_FOR_LST_SV	*upd_lst_S_d = NULL;	/* pointer selected for day */
_UPD_FOR_LST_SV	*upd_lst_S_n = NULL;	/* pointer selected for nit */

_tile_pointer tile_pntr[N_TILE_PNTRS];

int32	n_pair_tot[2] = { 0, 0 };

char msgString[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for use with modsmf */
char LogMsg[PGS_SMF_MAX_MSG_SIZE]={NULL};    /* string for smf status report */
char systemTag[PGS_SMF_MAX_MSG_SIZE]={NULL}; /* string for systemTag */

#include "PGS_PC.h"

int32	PCLogupdate[4] = { 211000, 211001, 211104, 211105 };
int32	numfiles_update[4];
char	id_char_update[8] = "A";
int32	n_days_per_cycle_update_pcf;
int32	i_days_in_cycle_update_pcf;
int32	n_days_per_cycle_update = 2;
int32	i_days_in_cycle_update = 0;
int32	i_other_in_cycle_update = 2;
int32	i_do_multi_days = 1;
int32	i_use_2nd_upd = 0;
int32	n_day_diff = 0;
int32	n_day_diff_2nd = 0;

#define PCLogsollut     211002   /* the binary file for sol_term_tbls */
#define PCLogtirlut     211003   /* the binary file for tir_term_tbls */
#define PCLogL3_6km     211035   /* MOD11 L3 LST 6km input/output */

/* R. Cember: a separate MCF for each output file. */
#define MCF_FILE_L3_6KM     211254       /* MCF for the L3 6km LST file */
#define MCF_FILE_L3_UPD     211259       /* MCF for the update_for_lst file */

/* R. Cember: setting the DAYNIGHTFLAG valids through macros. */
#define DAY "Day"
#define NIGHT "Night"
#define BOTH "Both"

/* R. Cember: for use in metadata setting. */
char UR_sollut[PGSd_PC_VALUE_LENGTH_MAX];
char UR_tirlut[PGSd_PC_VALUE_LENGTH_MAX];

/* the following 122 lines come from pntr.c only with minor changes */

#ifdef USE_ECS          /* ECS meta data handling (Paul Fisher/YTLau) */

#include "PGS_MET.h"
PGSt_MET_all_handles            mdhandles;      /* Meta data handles */

/* ECS metadata string and variable names */
#define CORES		"CoreMetadata.0"
#define ARCHIVES_IN	"ArchiveMetadata.0"
#define ARCHIVES_OUT	"ArchiveMetadata.0"

/* the following three defines must be defined in relation to one another */
#define METAHANDLES mdhandles
#define COREHANDLES mdhandles[1]
#define ARCHIVEHANDLES mdhandles[2]

/* default value for QAPERCENTINTERPOLATEDDATA */
#define QAINTDATA 0
/* default value for QAPERCENTMISSINGDATA */
#define QAMISDATA 0
/* default value for QAPERCENTOUTOFBOUNDSDATA */
#define QAOBDATA 0

#endif  /* end USE_ECS */

/* Fixed parameters */

#define AQFLAG	"passed"

#define NTILE_HORIZ 36	/* number of horizontal tiles in the grid; nominally 
			 * this value (36) will generate 10 degree (1100 km)
			 * tiles; this should always be either 1 or two 
			 * times 'NTILE_VERT' */
#define NTILE_VERT 18	/* number of vertical tiles in the grid; nominally 
			 * this value (18) will generate 10 degree (1100 km)
			 * tiles */
#define FILL_GRANULE_PT 255 /* Fill value for granule pointer */

#define DTR(x)	((x) * 0.01745329)	/* degree to radians by * PI / 180. */
#define RTD(x)	((x) * 57.2957795)	/* radians to degree by * 180. / PI */
#define DLAT_6KM_ROW 0.05	        /* the latitude increament (deg) for
					   one 6km row */

/* Resolution type */

/*typedef enum {RES_1KM, RES_6KM, NRES} res_t;*/

/* Macros for testing and setting bits */

#define BIT_TEST(x, b) (((x) & BIT[(b)]) != 0)
#define BIT_SET(x, b) x = x | BIT[(b)]

/* User entered parameters data structure */

typedef struct {
  
  int16 itile_horiz, itile_vert; /* vertical and horizontal tile numbers */
  float32 coverage_min;	/* minimum coverage */
  char gridtype[8];    /* grid type string */
  int32 itile_size;	/* tile size */
  int32 ngran;          /* number of granules */
} uparam_t;

/* Tile and binning data structure */

typedef struct {
  int16 itile_horiz, itile_vert; /* selected horizontal and vertical tile */
  int32 nrow, npix;		/* number of rows and pixels per row */
  int32 irow_start, irow_end; /* start and end row of the tile in entire grid */
  int32 itile_size;
} tile_t;

/* G. Ye added function DoMetaData that was modified by R. Cember. and Z. Wan */

void DoMetaData( int32, int32, PGSt_PC_Logical, PGSt_integer, PGSt_integer, MODFILE *);

void Error(const char *, const char *); 
void poly_intp_vector (float64 *, int32, float64, int32, float64 **, float64 *);

/* the following 86 lines are used for day/night LST algorithm */

#include "day_nit_lut.h"

#define NX	14
#define MA	14
#define NPT	16
#define NC	 7
#define NC2	14
#define NRHM	N_CWV
#define NTSA	N_TSA

_SOL_TERM_TBL   ***sol_term_tbls;
_TIR_TERM_TBL   **tir_term_tbld;
_TIR_TERM_TBL   **tir_term_tbln;

_SOL_TERM_TBL	*sol_term_pntr;
_TIR_TERM_TBL	*dtir_term_pntr;
_TIR_TERM_TBL	*ntir_term_pntr;

off_t   off_set = 0;

float64	sf_elev_pair = 0.0;
int32	id_atm_day, id_atm_nit;
int32	i_sf_prt = 0;

double	Day_Nit_Lst (double, double, double);
int32	i_day_nit_lst_status_d = 0;
int32	i_day_nit_lst_status_n = 0;

double  split_window_lst (int32, double *);
float32 dt_split_window_bad = 2.5;

int32	ixx, ixy;

double	dt_ref = 3.0, t_ref_d = 300., t_ref_n = 290.;
int32	i_out_range_lst = 0, i_out_range_em = 0;
static int32 n_out_range_case[20] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
static int32 n_out_range_upd[2] = { 0, 0 };
static int32 n_out_range_brdf[2] = { 0, 0 };
static int32 n_out_range_wvd[2] = { 0, 0 };
static int32 n_out_range_wvn[2] = { 0, 0 };
static int32 n_out_range_tad[2] = { 0, 0 };
static int32 n_out_range_tan[2] = { 0, 0 };
static float32	dtdn_max_x, dtdn_max_a;
static double	tadd, tann, wda, wna, tsd, tsn;
float32	dtdn_x, dtdn_a;
float32 d_n_dt_d_min = -3.0, d_n_dt_d_max = 4.0;
float32 d_n_dt_n_min = -3.0, d_n_dt_n_max = 4.0;
float32 dtdn_x_limit = 65.0, dtdn_max_a_limit = 65.0;
double  em_est0[5][NC] = {
	0.874, 0.880, 0.870, 0.932, 0.965, 0.975, 0.988,  /* soil and rocks */
	0.894, 0.900, 0.890, 0.952, 0.975, 0.975, 0.988,  /* others */
	0.914, 0.920, 0.910, 0.957, 0.990, 0.990, 0.988,  /* evergreen veg */
	0.970, 0.970, 0.970, 0.980, 0.993, 0.983, 0.970,  /* water, snow */
	0.965, 0.970, 0.972, 0.975, 0.965, 0.943, 0.935 }; /* ice */
double	em_min[6][NC] = { 0.52, 0.52, 0.52, 0.60, 0.76, 0.80, 0.86,
			  0.70, 0.70, 0.70, 0.77, 0.84, 0.85, 0.88,
			  0.89, 0.89, 0.89, 0.90, 0.90, 0.90, 0.90,
			  0.91, 0.91, 0.91, 0.92, 0.93, 0.92, 0.92,
			  0.90, 0.90, 0.90, 0.92, 0.92, 0.90, 0.90,
			  0.955, 0.96, 0.96, 0.97, 0.98, 0.97, 0.96 };
double	em_max = 0.998;
double	sol_irrad_f = 1.0;
double	sol_irrad_f_sv = 1.0;
double	wda_min = 1.8, wda_max = 3.5;
double	wna_min = 1.8, wna_max = 3.5;

double rups[NTSA][NRHM][NC];
double rgfd[NTSA][NRHM][NC];
double rds[NTSA][NRHM][NC];
double pad[5][NC];

double wd[NRHM], tadit[NTSA];
double rupt[NTSA][NRHM][NC];
double rgft[NTSA][NRHM][NC];
double transd[NTSA][NRHM][NC];

double wn[NRHM], tanit[NTSA];
double rupn[NTSA][NRHM][NC];
double rgfn[NTSA][NRHM][NC];
double transn[NTSA][NRHM][NC];
double pan[3][NC];

double dt_neq[NC] = { 0.05, 0.07, 0.07, 0.05, 0.05, 0.05, 0.20 };
static double tbd[NC], tbn[NC];
static double rtbd[NC], rtbn[NC];

int32	ntsa = N_TSA;
int32	nrhu = N_CWV;
int32	ia[MA], mfit;
int32	i_land_k, i_land_catalog = 0;
int32	ia_in[MA] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0 };
int32	ix1[NPT] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15 };
double	a[MA], a_sv[MA], atry[MA];
double	x[NX], z[NX], *a_x_p;
double	ymod[NPT], dyda[NPT][MA];
double	amin[MA], amax[MA];
double	rad_ref_d[NC], rad_ref_n[NC];
double	brdf, brdf_ref = 10.0, wv_ref = 12.5;
double	y[NPT], sig[NPT];
double	alpha[MA][MA], beta[MA];    /* used in 1st call of mrqcof in mrqmin */
double	covar[MA][MA], da[MA];	    /* used in 2nd call of mrqcof in mrqmin */
double	a_sv3 = 0, a_sv4 = 0, a_sv5 = 0, a_sv6 = 0;
double	ochisq_main, ochisq_min;
float64	bt_to_radiance (int32, float64, int32);
float64 f_mod_rad7_inv (int32, float32, int32);
void	covsrt(int32), gaussj(int32); 
void	interp_atm_d (double, double), interp_atm_n(double, double);
int32	locate (double *, int32, double);
int32	ntot_mrqcof = 0;

uint8   u8_dbt, u8_pdb;
uint8	u8_qa_0 = 0, u8_qa_1 = 1, u8_qa_2 = 2, u8_qa_3 = 3, u8_cov = 0;
uint8	u8_quant = 0;
static uint8 u8_qa_man = 0, u8_qa_data = 0, u8_qa_emis = 0, u8_qa_lst = 0;
int32	indx_view_day_select, indx_view_nit_select;

double  mrqmin(double *);

double mrqcof (double *, double [][MA], double *);

/* the end of the global variables used in the day/night LST algorithm */

/* mod11 specific functions */

void	free_lst_space(void);
void 	SDsetdim_LST (int32, sds_t, int32 *);
void 	SDsetattr_LST (int32, char *, int32, int32, VOIDP);
void 	SDsetattr_LST_V (int32, char *, int32, int32, float32 *);

/* R. Cember: add function "zeroout_general". */
void zeroout_general(void *buf, int32 nelems, int32 type_size);

#define PGE16_Version	   800500  	/* run-time defined PGE16 Version  */

#define ReprocessingPlanned 800503	/* it's a common LUN number */
#define ReprocessingActual  800504	/* it's a common LUN number */
#define ScienceQualityFlagExplanation  800506   /* it's a LUN number */
#define ProcessingEnvironment  800550	/* it's a common LUN number */

char    rep_planned[PGSd_PC_VALUE_LENGTH_MAX];
char    rep_actual[PGSd_PC_VALUE_LENGTH_MAX];
char    sci_qua_exp[PGSd_PC_VALUE_LENGTH_MAX];
char	process_env[PGSd_PC_VALUE_LENGTH_MAX];
char    localgranuleId_vers_num[PGSd_PC_VALUE_LENGTH_MAX];

char    data_id[16], data_day[16];
int32	i_data_day;
int32   vers_id_num = 5;
 
char	pgeversion[64];

char	pval[PGSd_PC_VALUE_LENGTH_MAX];
int32	i_tile_proc = 0;

/* R. Cember: Add declaration of file handles for conversion from
   ECS-prohibited C functions to equivalent SDP toolkit calls. */ 

  PGSt_IO_Gen_FileHandle *sol_lut_handle=NULL;
  PGSt_IO_Gen_FileHandle *tir_lut_handle=NULL;

  PGSt_integer Vers;

/* R. Cember: making this declaration global. */
  PGSt_SMF_status returnStatus = MODIS_S_SUCCESS;  /* Default for success */

  MODFILE	  *update_handle[4] = { NULL, NULL, NULL, NULL };
  PGSt_integer    Vers_tmp = 1;
  PGSt_integer    n_tiles = 0;

  /* variables for reading input file */

  char	modstring[MAX_NC_NAME] = {0};	/* used in complex messages */
  char	sol_fname[MAX_NC_NAME] = {NULL};
  char	tir_fname[MAX_NC_NAME] = {NULL};
  char	upd_fname[4][MAX_NC_NAME] = {NULL};

  char	HaltMsg[2048];
  char	smf_message[PGS_SMF_MAX_MSGBUF_SIZE]={0};
  char	add_message[PGS_SMF_MAX_MSGBUF_SIZE]={0};

#define HALT(a)       { sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, a);\
			PGS_SMF_GenerateStatusReport(HaltMsg); \
			free_lst_space(); exit(1);   }

#define WARN(a)       { sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, a);\
			PGS_SMF_GenerateStatusReport(HaltMsg); }

#define HALTsmf(mnemon, a)      { \
	PGS_SMF_GetMsgByCode(mnemon, smf_message); \
	sprintf(HaltMsg,"%s[%d]: %s %s", __FILE__, __LINE__, smf_message, a);\
	PGS_SMF_GenerateStatusReport(HaltMsg); \
	free_lst_space(); exit(1);   }

#define modsmf_id_vers(i1, i2)	{ \
	sprintf (modstring, "Logical Id %d Version %d", i1, i2); \
	sprintf(HaltMsg,"%s[%d]: %s", __FILE__, __LINE__, modstring);\
	PGS_SMF_GenerateStatusReport(HaltMsg); \
	free_lst_space();	exit (1); }

int32 get_meta_attr(char *s, char *p, int32 type, int32 num, char *d);

#define rm_path(s, p) { p = ( s[0] == '/' || s[0] == '.') ? strrchr ( s, '/' ) + 1 : s; }

int32      iday_of_year(int32, int32, int32);

char	   datetime[MAX_NC_NAME];
char	   *pattern_date = "RANGEBEGINNINGDATE";
char	   *pattern_time = "RANGEBEGINNINGTIME";
char	   *s_start;
int32	   iyr, imonth, iday;
int32	   ihrs, iminutes;
float32    seconds;
float64    seconds_start;
int32	   i_day_of_yr_d = 0;
int32	   i_day_of_yr_n = 0;
int32	   i_day_of_yr_curr = 0;
int32	   i_wrt_out_day = 0, i_wrt_out_nit = 0;
int32	   i_time_pix = 0;

uparam_t uparam;      		/* user parameters data structure */
uparam_t *uparam_p = &uparam;

/* arrary dim. for update_for_lst's I/O */
long int upd_dims[3] = {TILE_YDIM_6KM, TILE_XDIM_6KM, TILE_ZDIM_UPD};
long int upd_sta[3] = { 0, 0, 0};

void SMFLOG(char *a);

/* R. Cember: for calls to MAPI function getMODISfileinfo() */
char sd_attr_type[MAX_NC_ATTRS]; 
char sd_attr_name[MAX_NC_NAME];
long nelements;

/* R. Cember: add declaration. */
char trash_char[5];

PGSt_SMF_status      current_time_a(char *);

void          *local = NULL;

PGSt_double	tile_bound[4] = {2.F};

PGSt_double	tile_LatLons[2][4];

PGSt_integer   	rel_vers_tile = 1;
PGSt_integer   	vers_tile = 1;
PGSt_integer   	vers_tmp = 1;
PGSt_integer   	Vers3 = 1;

MODFILE	 *lst_6km_l3_outf = NULL;

char     fname_6km_L3[MAX_NC_NAME] = {NULL};

/* processed good and other quality pixels,
   not processed due to cloud and other reasons, respectively */

int32	total_l3_6km_dn[4] = { 0, 0, 0, 0 };

char			DayNightFlag[32];
char			PGEVERSION_L1B[32];
enum AutoQualFlag	AutomaticQualityFlag = PASSED;
MET_counts		met_cnt_6km;
MET_counts		*met_cnt;

char    CharVals[10][PGSd_PC_VALUE_LENGTH_MAX];

char	*CharStg2[] = {
	 "RANGEBEGINNINGDATE",
	 "RANGEBEGINNINGTIME",
	 "RANGEENDINGDATE",
	 "RANGEENDINGTIME",
	 "EXCLUSIONGRINGFLAG.1",
	 "EQUATORCROSSINGDATE.1",
	 "EQUATORCROSSINGTIME.1",
	 "OPERATIONMODE",
	 "PGEVERSION",
	 "ADDITIONALATTRIBUTENAME.1" };

char    date_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char    time_begin[2][PGSd_PC_VALUE_LENGTH_MAX];
char    date_end[2][PGSd_PC_VALUE_LENGTH_MAX];
char    time_end[2][PGSd_PC_VALUE_LENGTH_MAX];
char	ch_tileid[128] = {NULL};

int32	tile_hours, tile_minutes;
float64 tile_begin_seconds;	
float64 tile_end_seconds, seconds_tmp;
int32	i_new_l1b = 1;

void sec_to_hrs (float64 *, int32 *, int32 *);

/* R. Cember: making this main void to quiet the fullwarn flag since exit
   from main is through SMFLOG(). */

uint8  u8_0cons = 0, u8_1cons = 1, u8_2cons = 2, u8_3cons = 3;
uint8  u8_4cons = 4, u8_5cons = 5, u8_6cons = 6, u8_7cons = 7;
uint16 u16_0cons = 0, u16_1cons = 1, u16_2cons = 2, u16_3cons = 3;

char    esdt_6km_L3[8] = "MOD11B1";
char    esdt_lst_upd[16] = "MOD11UPD";
char    esdt_lst_upd_a[16] = "MYD11UPD";
char    *esdt_tmp, *esdt_tmp_a;

#define    SATELLITE		"A"
#define    VERSION		"005"

#define    N_BYTES_CORE_L3	128000

#define    N_BYTES_ARCH_L3	256000	

char	   core_L3[N_BYTES_CORE_L3];
char	   arch_L3[N_BYTES_ARCH_L3];

#define	MAXTILES 317
#define	DEM_SLOPE_CONSIDERED 0200

int32	numfiles_6km = 0;
int32   i_param = 0;

int32	n_pairs_day[N2_VIEW_ANGS][N2_VIEW_ANGS];
int32	n_pairs_nit[N2_VIEW_ANGS][N2_VIEW_ANGS];
double	dt_pairs_day[N2_VIEW_ANGS][N2_VIEW_ANGS];
double	dt_pairs_nit[N2_VIEW_ANGS][N2_VIEW_ANGS];
int32	i_view_day_ewg[3], i_view_nit_ewg[3];
int32	i_ang_day_ewg[3], i_ang_nit_ewg[3];
int32	i_view_day_select, i_view_nit_select;
int32	i_ang_day[4][N2_VIEW_ANGS], i_ang_nit[4][N2_VIEW_ANGS];
uint16	n_bt4_day[4][N2_VIEW_ANGS], n_bt4_nit[4][N2_VIEW_ANGS];
int32	indx_v_zone[4][4] = {  6,  7,  8,  9,
			       4,  5, 10, 11,
			       2,  3, 12, 13,
			       0,  1, 14, 15 };
int32	i_v_zone_v, n_v_zone_v = 4;
int32	i_v_zone_z, n_v_zone_z = 4;
uint16	bt4_max_day_v_zone[4], bt4_max_nit_v_zone[4];
uint16	bt4_day_ewg[3], bt4_day_upd[4], bt4_day_f;
uint16	bt4_nit_ewg[3], bt4_nit_upd[4], bt4_nit_f;
float64 vzd_pix_in = 0.0, vzn_pix_in = 0.0;

int32	i_aqua_d = 0, i_aqua_n = 0;

int32	n_upd_sets = 4, i_complete_combined = 1;
int32	ii_upd, nt_upd = 0;
int32	ii_day, mx_days = 33;

int32   i_bad_spec, n_bad_spec = 0;
int32	total_space = 0;

main(int32 argc, char **argv) 
{

   char    a_ih_upd[4], a_iv_upd[4];
   char    a_ih_6km[PGSd_PC_VALUE_LENGTH_MAX];
   char	   a_iv_6km[PGSd_PC_VALUE_LENGTH_MAX];
   char	   *local2;
   uint16  ih_6km, iv_6km;
   uint16  ih_upd, iv_upd;

   size_t     siz = 0;

   char  *InputStg[4][2] = {
	"ORBITNUMBER.1", "ORBITNUMBER",
	"LOCALGRANULEID", "LOCALGRANULEID",
	"PARAMETERVALUE.6", "PARAMETERVALUE.6", /* tile horizontal num */
	"PARAMETERVALUE.7", "PARAMETERVALUE.7"  /* tile vertical num */
	};

   char  CharV[2][PGSd_PC_VALUE_LENGTH_MAX];
   char  *pattern_id = ".A";

   /* R. Cember: use a character string rather than a pointor in order to avoid
   memory leaks by stepping on memory locations just a little higher than the
   location of void pointer "local". */
   char  str_start[MAX_NC_NAME];

   char  *str_sub1, *str_sub2;

   int32 returnStatus;
   int32 sl;

   /* to hold the PROCESSINGDATETIME metadata field */
   char protime[PGSd_PC_VALUE_LENGTH_MAX]="Error in PROCESSINGDATETIME";

  int32   i_day_n_s[N2_VIEW_ANGS], i_nit_n_s[N2_VIEW_ANGS];	/* NEW */
  int32   n_day_n_s[N2_VIEW_ANGS], n_nit_n_s[N2_VIEW_ANGS];
  int32   i_day_e_s[4][N2_VIEW_ANGS], i_nit_e_s[4][N2_VIEW_ANGS]; /* EXIST */
  int32   n_day_e_s[4][N2_VIEW_ANGS], n_nit_e_s[4][N2_VIEW_ANGS];
  int32	  iii, i_d_n_m_s[4];

  int32    ier = 0;
  int32	   i = 0, j = 0, k = 0;
  
  uint8	   u8_1, u8_2;  
  uint8	   u8_data_yr_d, u8_data_yr_n;

  int32		n_6km_l3_old_file;
  long int	sta_6km_o[2], dims_6km_o[2];

  int32	i_6km, j_6km;		/* row and column numbers of 6km L3 cells */

  /* variables used in day/night LST algorithm */
  uint8 vtime_d_in = 0;
  uint8 vtime_n_in = 0;
  int32 i_d_n_m;
  int32 i_pair_type = 0;
  static int32 i_pair_exist = 0;
  int32 n_new_pairs = 0;
  int32	n_pair_new[2] = { 0, 0};
  int32	n_view_new[2] = { 0, 0};
  int32	n_view_exist[4][2];
  double chisq = 0.;
  float64 szn_pix_in = 0.;
 
  /* solar irradiance correction factor for the Sun's elliptic orbit */
  int32   i_day_yr, i_day_yr_sv = -1;
  int32   nday[13] = {
		1, 32, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 };

  double  sol_rat[13] = {
		1.034, 1.030, 1.019, 1.001, 0.985, 0.972, 0.967,
		0.971, 0.982, 0.998, 1.015, 1.029, 1.034 };

  double  tmp, tmp1, tmp2;

  int16   *t_prof_p, *w_prof_p;
  float32 *t_prof_S_p, *w_prof_S_p;

  float32     frac1, frac2;
  float32     frac3, frac4;

  int32	  i_aqua[4] = { 0, 0, 0, 0 };
  int32	  i_view, kn;
  int32	  n_view_angs = N_VIEW_ANGS;
  int32	  n2_view_angs = N2_VIEW_ANGS;
  int32	  ii_view, i_d_n_done_once = 0;
  int32	  i_day_nit_loop, n_day_nit_loop = 2;
  int32	  n_view_day, n_view_nit;
  int32	  n_view_day_e, n_view_nit_e;
  int32	  n_view_day_w, n_view_nit_w;
  int32	  i_ang_dif[4][N2_VIEW_ANGS];
  int32	  min_ang_dif_upd[4];
  int32	  min_dif_view_upd[4];
  int32	  min_ang_dif_f, min_dif_view_f;
  int32   i_case, n_cases = 3;
  int32	  n_new_view_day, n_new_view_nit;
  int32	  n_new_view_day_e, n_new_view_day_w;
  int32   n_new_view_nit_e, n_new_view_nit_w;
  int32   min_ang_dif_ewg[3], min_dif_view_ewg[3];
  int32   i_use_2nd_upd_ewg[3];
  int32	  n_times_2nd_upd[2][N2_VIEW_ANGS];
  int32	  dn_day_d_n[4];
  uint8   em_6km_sv[8];
  uint8	  retrieved_em[6];
  uint8	  i_near_cld_d, i_far_cld_d, i_near_cld_n, i_far_cld_n;
  int32	  i_break1 = 0, i_break2 = 0;
  int32	  i_pair_type_sv, i_view_day_sv, i_view_nit_sv;
  int32	  view_ang_sign = 0;
  int16	  int16_sf_elev;
  int16   i_split_dt;
  uint8   u8_grid_slope;

#ifdef _LINUX_ct
  void ctSetup();
#endif

   if (argc < 2) {
     sprintf (modstring, "no tile version provided");
     HALT(modstring);
   }

   /* set callerID first to get rid of the error message to LogStatus */
   PGS_SMF_SetCallerID(PGSd_CALLERID_SMF);

   /* Initialize the granule ids */

   uparam.ngran = 1;
   uparam.coverage_min = 0.02;
   uparam.itile_size = 1;
   strcpy(uparam_p->gridtype, "s");

   if (PGS_PC_GetConfigData(PGE16_Version, pgeversion) != PGS_S_SUCCESS) {
  	sprintf(modstring,"no PGE16_Version (LUN 800500) in PCF");
	HALT(modstring);
   }

   if (PGS_PC_GetConfigData(ReprocessingPlanned, rep_planned) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingPlanned info from pcf");

   if (PGS_PC_GetConfigData(ReprocessingActual, rep_actual) != PGS_S_SUCCESS)
	SMFLOG("Error getting ReprocessingActual info from pcf");

   if (PGS_PC_GetConfigData(ScienceQualityFlagExplanation, sci_qua_exp) != PGS_S_SUCCESS)
	SMFLOG("Error getting ScienceQualityFlagExplanation info from pcf");

   if (PGS_PC_GetConfigData(ProcessingEnvironment, process_env) != PGS_S_SUCCESS)
	SMFLOG("Error getting ProcessingEnvironment info from pcf");

   rel_vers_tile = atoi (argv[1]);

   if ( argc >= 3 ) {
      n_upd_sets = atoi (argv[2]);
      if ( n_upd_sets < 4 )	i_complete_combined = 0;
   }

   if ( (returnStatus = PGS_PC_GetNumberOfFiles(PCLogL3_6km, &numfiles_6km)) !=
                PGS_S_SUCCESS )
	SMFLOG("Error getting number of 6km L3 files");

   if ( numfiles_6km > MAXTILES )
        SMFLOG("STOP--too many 6km L3 tiles in PCF");

   if ( rel_vers_tile > numfiles_6km ) {
	sprintf (LogMsg,
	"Completed MOD_PR11 successfully (%d -all 6km tiles already processed)",
	  rel_vers_tile );
	SMFLOG(LogMsg);
   }

   vers_tmp = rel_vers_tile;
   if (PGS_PC_GetReference(PCLogL3_6km,&vers_tmp,fname_6km_L3) != PGS_S_SUCCESS)
	modsmf_id_vers(PCLogL3_6km, rel_vers_tile);

   rm_path(fname_6km_L3, esdt_tmp);
   sscanf ( esdt_tmp, "%7s", &esdt_6km_L3[0] );

   if ( Hishdf ( fname_6km_L3 ) == TRUE ) {

	if ((lst_6km_l3_outf = openMODISfile (fname_6km_L3, "a")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE,fname_6km_L3);

	n_6km_l3_old_file = 1;
	tile_pntr[0].l3_6km_exist = 1;

   }
   else {
	exit(0);
   }

   sprintf(sd_attr_name, CORES);
   sprintf(sd_attr_type, "char *");
   nelements = (long) N_BYTES_CORE_L3;
   memset ( &core_L3[0], '\0', nelements );
   if (getMODISfileinfo(lst_6km_l3_outf, sd_attr_name, sd_attr_type,
		&nelements, core_L3) != MAPIOK)
	Error("Re: MOD11B1: failure on call to get core", "getMODISfileinfo");

   local = CharV[0];
   if (get_meta_attr (core_L3, InputStg[1][0], 1, 1, (char *) local ) == FAIL){
	sprintf(msgString, "get_meta_attr fails on core_L3");
	HALT(msgString);
   }

   sscanf (local, "%3s", data_id);
   strcpy(str_start, strstr (local, pattern_id) );
   sscanf (str_start+1, "%8s", data_day);
   i_data_day = atoi (&data_day[5]);

   if ( !i_days_in_cycle_update ) {
	i_days_in_cycle_update = i_data_day % 2;
	if ( !i_days_in_cycle_update ) i_days_in_cycle_update = 2;
	i_other_in_cycle_update = ( i_days_in_cycle_update == 1 ) ? 2 : 1;
   }

/*fprintf(stderr, "i_days_in_cycle_update %d\n", i_days_in_cycle_update);*/

   s_start = strstr (core_L3, pattern_date);
   if (s_start == NULL)
	Error ("String search error", "main, mod11_lstB1.c");

   sscanf(s_start+80, "%10s", datetime);

   sscanf (datetime, "%4d%c%2d%c%2d",
	&iyr, &trash_char[0], &imonth, &trash_char[1], &iday);

   i_day_of_yr_curr = iday_of_year (iyr, imonth, iday);

   if ( i_do_multi_days ) {
	if ( ( i_days_in_cycle_update == 1 &&
		( (i_day_of_yr_curr % n_days_per_cycle_update ) != 1 ) ) ||
	     ( i_days_in_cycle_update == 2 &&
		( (i_day_of_yr_curr % n_days_per_cycle_update ) != 0 ) ) ) {

		sprintf (msgString, "wrong UPD file used");
		HALT(msgString);
	}
   }

   a_ih_6km[0] = '\0';
   local2 = a_ih_6km;
   if (get_meta_attr(core_L3, InputStg[2][0], 1, 1, (char *) local2 ) == FAIL )
	HALT("get_meta_attr for HORIZONTALTILENUMBER fails on core_L3");
   ih_6km = (uint16) atoi (a_ih_6km);

   a_iv_6km[0] = '\0';
   local2 = a_iv_6km;
   if (get_meta_attr(core_L3, InputStg[3][0], 1, 1, (char *) local2 ) == FAIL )
	HALT("get_meta_attr for VERTICALTILENUMBER fails on core_L3");
   iv_6km = (uint16) atoi (a_iv_6km);

   tile_pntr[i_tile_proc].ih = ih_6km;
   tile_pntr[i_tile_proc].iv = iv_6km;

   vers_tmp = rel_vers_tile;
   returnStatus = PGS_PC_GetReference(PCLogupdate[i_days_in_cycle_update-1],
		  &vers_tmp, upd_fname[0]);
   if ( returnStatus != PGS_S_SUCCESS )
	HALTsmf(MODIS_F_GET_GEN_FILE, "update_for_lst");

   if ((update_handle[0] = openMODISfile (upd_fname[0], "a")) == NULL)
	HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname[0]);

   nt_upd++;

   rm_path(upd_fname[0], esdt_tmp);
   sscanf ( esdt_tmp, "%8s", &esdt_lst_upd[0] );

   i_aqua[0] = 0;
   if ( strcmp (esdt_lst_upd, "MOD11UPD") )	i_aqua[0] = 1;

/*fprintf(stderr, "i_aqua[0] %d %s\n", i_aqua[0], upd_fname[0]);*/

   if ( i_do_multi_days ) {

	sscanf ( &esdt_tmp[10], "%1s", &trash_char[0] );
	n_days_per_cycle_update_pcf = atoi (trash_char);
	sscanf ( &esdt_tmp[12], "%1s", &trash_char[0] );
	i_days_in_cycle_update_pcf = atoi (trash_char);

	if ( esdt_tmp[9] != id_char_update[0] ||
	     n_days_per_cycle_update_pcf != n_days_per_cycle_update ||
	     i_days_in_cycle_update_pcf != i_days_in_cycle_update ) {
	  sprintf(msgString,"the UPD file name does not start with %s.%c%d-%d.",
		esdt_lst_upd, id_char_update[0], n_days_per_cycle_update,
		i_days_in_cycle_update);
	  HALT(msgString);
	}
/* #### */
     if ( i_complete_combined ) {

	vers_tmp = rel_vers_tile;
	returnStatus=PGS_PC_GetReference(PCLogupdate[2],
		  &vers_tmp, upd_fname[nt_upd]);

	if ( Hishdf ( upd_fname[nt_upd] ) ) {
	    if((update_handle[nt_upd] = openMODISfile(upd_fname[nt_upd],"r")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname[nt_upd]);

	    rm_path(upd_fname[nt_upd], esdt_tmp_a);
	    sscanf ( esdt_tmp_a, "%8s", &esdt_lst_upd_a[0] );
	    i_aqua[nt_upd] = 0;
	    if ( strcmp (esdt_lst_upd_a, "MOD11UPD") )      i_aqua[nt_upd] = 1;
/*fprintf(stderr,"i_aqua[%d] %d %s\n",nt_upd,i_aqua[nt_upd],upd_fname[nt_upd]);*/

	    nt_upd++;
	}
	else	i_complete_combined = 0;

	vers_tmp = rel_vers_tile;
	returnStatus=PGS_PC_GetReference(PCLogupdate[3],
		  &vers_tmp, upd_fname[nt_upd]);

	if ( Hishdf ( upd_fname[nt_upd] ) ) {
	    if((update_handle[nt_upd] = openMODISfile(upd_fname[nt_upd],"r")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname[nt_upd]);

	    rm_path(upd_fname[nt_upd], esdt_tmp_a);
	    sscanf ( esdt_tmp_a, "%8s", esdt_lst_upd_a );
	    i_aqua[nt_upd] = 0;
	    if ( strcmp (esdt_lst_upd_a, "MOD11UPD") )      i_aqua[nt_upd] = 1;
/*fprintf(stderr,"i_aqua[%d] %d %s\n",nt_upd,i_aqua[nt_upd],upd_fname[nt_upd]);*/

	    nt_upd++;
	}
	else	i_complete_combined = 0;

	vers_tmp = rel_vers_tile;
	returnStatus=PGS_PC_GetReference(PCLogupdate[i_other_in_cycle_update-1],
		  &vers_tmp, upd_fname[nt_upd]);

	if ( Hishdf ( upd_fname[nt_upd] ) ) {
	    if((update_handle[nt_upd] = openMODISfile(upd_fname[nt_upd],"r")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname[nt_upd]);

	    rm_path(upd_fname[nt_upd], esdt_tmp_a);
	    sscanf ( esdt_tmp_a, "%8s", &esdt_lst_upd_a[0] );
	    i_aqua[nt_upd] = 0;
	    if ( strcmp (esdt_lst_upd_a, "MOD11UPD") )      i_aqua[nt_upd] = 1;
/*fprintf(stderr,"i_aqua[%d] %d %s\n",nt_upd,i_aqua[nt_upd],upd_fname[nt_upd]);*/

	    nt_upd++;
	}
	else	i_do_multi_days = 0;
     }
     else {
	vers_tmp = rel_vers_tile;
	returnStatus=PGS_PC_GetReference(PCLogupdate[i_other_in_cycle_update-1],
		  &vers_tmp, upd_fname[nt_upd]);

	if ( Hishdf ( upd_fname[nt_upd] ) ) {
	    if((update_handle[nt_upd] = openMODISfile(upd_fname[nt_upd],"r")) == NULL)
		HALTsmf(MODIS_F_OPEN_HDF_FILE, upd_fname[nt_upd]);

	    rm_path(upd_fname[nt_upd], esdt_tmp_a);
	    sscanf ( esdt_tmp_a, "%8s", &esdt_lst_upd_a[0] );
	    i_aqua[nt_upd] = 0;
	    if ( strcmp (esdt_lst_upd_a, "MOD11UPD") )      i_aqua[nt_upd] = 1;
/*fprintf(stderr,"i_aqua[%d] %d %s\n",nt_upd,i_aqua[nt_upd],upd_fname[nt_upd]);*/

	    nt_upd++;
	}
	else	i_do_multi_days = 0;
     }
   }

   s_start = strstr (esdt_tmp, ".h" );
   sscanf ( s_start+2, "%2s", a_ih_upd );
   sscanf ( s_start+5, "%2s", a_iv_upd );

   ih_upd = (uint16) atoi (a_ih_upd);
   iv_upd = (uint16) atoi (a_iv_upd);

   if ( ih_6km != ih_upd || iv_6km != iv_upd )
	SMFLOG("STOP--check the names and orders of MOD11B1 and MOD11UPD files in PCF");

   /* allocate data arrays for update_for_lst */

   nrow_6km = TILE_YDIM_6KM;
   ncol_6km = TILE_XDIM_6KM;

   siz = SZ_UPD_ANG_LST_SV;
   upd_dims[1] = ncol_6km;
   upd_dims[2] = siz / sizeof (uint16);
   if ( upd_dims[2] != TILE_ZDIM_UPD )
	SMFLOG("STOP--check update_for_lst.h and related parameters");

 for ( ii_upd = 0; ii_upd < 4; ii_upd++ )
   upd_ang_lst_Sv[ii_upd] = NULL;

 for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ )
   upd_ang_lst_Sv[ii_upd] =(_UPD_ANG_LST_SV **) alloc_2d (nrow_6km, ncol_6km, siz);

   /* read update_for_lst */

   upd_siz = ncol_6km * SZ_UPD_ANG_LST_SV;

   sta_6km_o[0] = 0;
   sta_6km_o[1] = 0;
   dims_6km_o[0] = nrow_6km;
   dims_6km_o[1] = ncol_6km;

   upd_sta[0] = 0;
   upd_dims[0] = nrow_6km;

   z_siz = ncol_6km * SZ_UPD_ANG_LST_SV / 2;

   dtdn_max_x = 0.0;	dtdn_max_a = 0.0;

 for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

   upd_ang_lst_Sp = upd_ang_lst_Sv[ii_upd];

   ier = getMODISarray(update_handle[ii_upd], upd_lst_6km[0].name,
	 upd_lst_6km[0].gname, upd_sta,upd_dims, &upd_ang_lst_Sp[0][0]);
   if (ier == -1) {
    	  /* HALTsmf(MODIS_F_MAPI_FAIL, "getMODISarray from upd_ang_lst"); */
      sprintf(add_message,"failed to getMODISarray from upd_ang_lst %2d%2d",
		tile_pntr[i_tile_proc].ih, tile_pntr[i_tile_proc].iv);
	  HALTsmf(MODIS_F_MAPI_FAIL, add_message);
   }
 }

  /* #### check if there is any new pair of day/night data */

   n_new_pairs = 0;

   for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

     for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	i_pair_exist = 0;
	n_view_new[0] = n_view_new[1] = 0;

	/* check availability of the pair of day/night data */

	for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	  n_view_exist[ii_upd][0] = n_view_exist[ii_upd][1] = 0;

	  upd_ang_lst_Sp = upd_ang_lst_Sv[ii_upd];
	  upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	  for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

		upd_lst_S_p = &upd_a_lst_S_p->_upd_for_lst_sv[i_view];
		i_day_of_yr_d = GET_DAY(upd_lst_S_p->day_date_and_view_time);
		i_day_of_yr_n = GET_DAY(upd_lst_S_p->nit_date_and_view_time);

		if ( ii_upd == 0 ) {
		   if ( i_day_of_yr_d == i_day_of_yr_curr &&
		     upd_lst_S_p->day_status & NEW_BIT )
			n_view_new[0] += 1;
		   if ( i_day_of_yr_n == i_day_of_yr_curr &&
		     upd_lst_S_p->nit_status & NEW_BIT )
			n_view_new[1] += 1;
		}

		if ( upd_lst_S_p->day_status & EXIST_BIT )
			n_view_exist[ii_upd][0] += 1;
		if ( upd_lst_S_p->nit_status & EXIST_BIT )
			n_view_exist[ii_upd][1] += 1;
	  }
	}

	i_pair_exist = 0;

	if ( n_view_new[0] ) {
	  for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ )
		if ( n_view_exist[ii_upd][1] )
			i_pair_exist = 1;
	}
	if ( n_view_new[1] ) {
	  for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ )
		if ( n_view_exist[ii_upd][0] )
			i_pair_exist = 1;
	}
	if ( i_pair_exist )	n_new_pairs += 1;
     }
  }

/*fprintf(stderr, "n_new_pairs %d\n", n_new_pairs);*/

  /* untouch the MOD11B1 file if there is no new pair of day/night data */

  if ( n_new_pairs == 0 ) {

	for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ )
		closeMODISfile(&update_handle[ii_upd]);

	closeMODISfile(&lst_6km_l3_outf);

	free_lst_space ();

	exit(0);
  }

  for ( i = 0; i < N2_VIEW_ANGS; i++ ) {
      n_times_2nd_upd[0][i] = 0;
      n_times_2nd_upd[1][i] = 0;
      for ( j = 0; j < N2_VIEW_ANGS; j++ ) {
	n_pairs_day[i][j] = 0;
	n_pairs_nit[i][j] = 0;
	dt_pairs_day[i][j] = 0.0;
	dt_pairs_nit[i][j] = 0.0;
      }
  }

  /* get PROCESSINGDATETIME */
  if(current_time_a(protime) != MODIS_S_SUCCESS) {
    sprintf(modstring,
	"the PROCESSINGDATETIME metadata field will not be written correctly.");
    HALT(modstring);
  }

  /* to open binary files that are used by the day/night LST algorithm */

  Vers_tmp = 1;
  Vers = Vers_tmp;
  if (PGS_PC_GetReference(PCLogsollut, &Vers_tmp, sol_fname) != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_GET_GEN_FILE, "sollut");

  /* R. Cember: get UR for metadata use. */
  Vers_tmp = 1;
  if(PGS_PC_GetUniversalRef(PCLogsollut, &Vers_tmp, UR_sollut) != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_GET_GEN_FILE, "sollut -- UR");

  returnStatus = PGS_IO_Gen_Open( PCLogsollut, PGSd_IO_Gen_Read, 
		&sol_lut_handle, Vers );
  if (returnStatus != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_OPEN_GEN_FILE, sol_fname);

  Vers_tmp = 1;
  Vers = Vers_tmp;
  if (PGS_PC_GetReference(PCLogtirlut, &Vers_tmp, tir_fname) != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_GET_GEN_FILE, "tirlut");

  /* R. Cember: get UR for metadata use. */
  Vers_tmp = 1;
  if(PGS_PC_GetUniversalRef(PCLogtirlut, &Vers_tmp, UR_tirlut) != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_GET_GEN_FILE, "tirlut -- UR");

  returnStatus = PGS_IO_Gen_Open( PCLogtirlut, PGSd_IO_Gen_Read, 
		&tir_lut_handle, Vers );
  if (returnStatus != PGS_S_SUCCESS)
	HALTsmf(MODIS_F_OPEN_GEN_FILE, tir_fname);

  /* allocate space for look-up tables used in the day/night LST algorithm */

  siz1 = N_VZN;
  siz = SZ_TIR_TERM_TBL;
  tir_term_tbld = (_TIR_TERM_TBL **) alloc_2d (4*N_ABASES, siz1, siz);
  tir_term_tbln = (_TIR_TERM_TBL **) alloc_2d (4*N_ABASES, siz1, siz);
  siz1 = N_SZN;
  siz2 = N_VZN;
  siz = SZ_SOL_TERM_TBL;
  sol_term_tbls = (_SOL_TERM_TBL ***) alloc_3d (4*N_ABASES, siz1, siz2, siz);

  off_set = 0;
  returnStatus = (int32) fseek (sol_lut_handle, off_set, SEEK_SET);
  if (returnStatus != 0)
	Error("Re: solar look-up-table", "fseek");
  sol_term_pntr = &sol_term_tbls[0][0][0];
  returnStatus = (int32) fread ((void *)&sol_term_pntr->sol_terms[0],
	(size_t)(SZ_SOL_TERM_TBL), (size_t)(4*N_ABASES * N_SZN * N_VZN),
	sol_lut_handle);
  if (returnStatus != (4*N_ABASES * N_SZN * N_VZN) )
	Error("Re: solar look-up-table", "fread");

  off_set = 0;
  returnStatus = (int32) fseek (tir_lut_handle, off_set, SEEK_SET);
  if (returnStatus != 0) Error("Thermal IR LUT, ", "fseek");
  dtir_term_pntr = &tir_term_tbld[0][0];
  returnStatus = (int32) fread ((void *)&dtir_term_pntr->tir_terms[0],
	(size_t)SZ_TIR_TERM_TBL, (size_t)(4*N_ABASES * N_VZN),
	tir_lut_handle);
  if (returnStatus != (4*N_ABASES * N_VZN) )
	Error("Thermal IR LUT, ", "fread");

  dtir_term_pntr = &tir_term_tbld[0][0];
  ntir_term_pntr = &tir_term_tbln[0][0];
  memcpy(&ntir_term_pntr->tir_terms[0], &dtir_term_pntr->tir_terms[0],
	(size_t) (4*N_ABASES * N_VZN * SZ_TIR_TERM_TBL) );

  returnStatus = PGS_IO_Gen_Close(sol_lut_handle);
  if (returnStatus != PGS_S_SUCCESS)
	HALTsmf(MODIS_E_CLOSE_GEN_FILE, sol_fname);
  returnStatus = PGS_IO_Gen_Close(tir_lut_handle);
  if (returnStatus != PGS_S_SUCCESS)
	HALTsmf(MODIS_E_CLOSE_GEN_FILE, tir_fname);

  mfit = 0;
  for (sl = 0; sl < MA; sl++) {
	ia[sl] = ia_in[sl];
	if (ia[sl])	mfit += 1;
  }

  uparam.itile_horiz = tile_pntr[i_tile_proc].ih;
  uparam.itile_vert = tile_pntr[i_tile_proc].iv;

  memset( &met_cnt_6km.FirstEntryInStruct, 0, sizeof( MET_counts ) );
  met_cnt_6km.AllPixelsProcessed =  2 * 200 *  200;	/* day and night */

  for (i = 0; i < 4; i++)
	total_l3_6km_dn[i] = 0;

  /* allocate data arrays for output 6km L3 LST */

  i = nrow_6km;
  j = ncol_6km;
  siz = sizeof (uint16);
  lst_6km_Sd = (uint16 **) alloc_2d (i, j, siz);
  siz = sizeof (uint8);
  QC_6km_day = (uint8 **) alloc_2d (i, j, siz);
  v_time_6km_l3_Sd = (uint8 **) alloc_2d (i, j, siz);
  v_angl_6km_l3_Sd = (uint8 **) alloc_2d (i, j, siz);
  siz = sizeof (uint16);
  lst_6km_Sn = (uint16 **) alloc_2d (i, j, siz);
  siz = sizeof (uint8);
  QC_6km_nit = (uint8 **) alloc_2d (i, j, siz);
  v_time_6km_l3_Sn = (uint8 **) alloc_2d (i, j, siz);
  v_angl_6km_l3_Sn = (uint8 **) alloc_2d (i, j, siz);
  em20_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em22_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em23_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em29_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em31_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  em32_6km_l3_Sv = (uint8 **) alloc_2d (i, j, siz);
  QC_emis = (uint8 **) alloc_2d (i, j, siz);
  siz = sizeof (uint16);
  lst_6km_from_1km_d = (uint16 **) alloc_2d (i, j, siz);
  lst_6km_from_1km_n = (uint16 **) alloc_2d (i, j, siz);
  siz = sizeof (float64);
  coef_mesh = (float64 **) alloc_2d (8, 9, siz);

   for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

     for (j_6km = 0; j_6km < ncol_6km; j_6km++) {
		lst_6km_Sd[i_6km][j_6km] = 0;
		lst_6km_Sn[i_6km][j_6km] = 0;
		em20_6km_l3_Sv[i_6km][j_6km] = 0;
		em22_6km_l3_Sv[i_6km][j_6km] = 0;
		em23_6km_l3_Sv[i_6km][j_6km] = 0;
		em29_6km_l3_Sv[i_6km][j_6km] = 0;
		QC_emis[i_6km][j_6km] = 0;
     }
   }

   getMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name,lst_6km_l3[1].gname,
	sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[2].name,lst_6km_l3[2].gname,
	sta_6km_o, dims_6km_o, &v_time_6km_l3_Sd[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[3].name,lst_6km_l3[3].gname,
	sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sd[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name,lst_6km_l3[5].gname,
	sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[6].name,lst_6km_l3[6].gname,
	sta_6km_o, dims_6km_o, &v_time_6km_l3_Sn[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[7].name,lst_6km_l3[7].gname,
	sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sn[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[12].name,lst_6km_l3[12].gname,
	sta_6km_o, dims_6km_o, &em31_6km_l3_Sv[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[13].name,lst_6km_l3[13].gname,
	sta_6km_o, dims_6km_o, &em32_6km_l3_Sv[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[14].name,lst_6km_l3[14].gname,
	sta_6km_o, dims_6km_o, &lst_6km_from_1km_d[0][0]);
   getMODISarray(lst_6km_l3_outf, lst_6km_l3[15].name,lst_6km_l3[15].gname,
	sta_6km_o, dims_6km_o, &lst_6km_from_1km_n[0][0]);

   /* set the searching sequence for day/night pairs */

   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ )
	i_d_n_m_s[ii_upd] = ii_upd;

   if ( nt_upd == 4 ) {
	i_d_n_m_s[0] = 1;
	i_d_n_m_s[1] = 2;
	i_d_n_m_s[2] = 0;
   }

   for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

     n_pair_new[0] = 0;	n_pair_new[1] = 0;

     for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

      i_view_day_sv = i_view_nit_sv = -1;
      i_pair_type_sv = -1;
      i_d_n_done_once = 0;

      n_view_day = 0;		n_view_nit = 0;
      n_view_day_e = 0;		n_view_nit_e = 0;
      n_view_day_w = 0;		n_view_nit_w = 0;
      n_new_view_day = 0;	n_new_view_nit = 0;
      n_new_view_day_e = 0;	n_new_view_nit_e = 0;
      n_new_view_day_w = 0;	n_new_view_nit_w = 0;

      for ( i_v_zone_z = 0; i_v_zone_z < n_v_zone_z; i_v_zone_z++ )
	bt4_max_day_v_zone[i_v_zone_z] = bt4_max_nit_v_zone[i_v_zone_z] = 0;

      /* check availability of day and night data */

      for ( i_view = 0; i_view < n2_view_angs; i_view++ )
      	   n_day_n_s[i_view] = n_nit_n_s[i_view] = -1;

      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	   iii = i_d_n_m_s[ii_upd];
	   upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	   upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	   for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

		i_ang_dif[iii][i_view] = 255;
		i_ang_day[iii][i_view] = 255;
		i_ang_nit[iii][i_view] = 255;

		n_bt4_day[iii][i_view] = 0;
		n_bt4_nit[iii][i_view] = 0;
		
		i_day_e_s[iii][i_view] = 0;
		i_nit_e_s[iii][i_view] = 0;

		n_day_e_s[iii][i_view] = 999;
		n_nit_e_s[iii][i_view] = 999;
	   }

	   for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

		upd_lst_S_p = &upd_a_lst_S_p->_upd_for_lst_sv[i_view];

		i_day_e_s[iii][i_view] = upd_lst_S_p->day_status & EXIST_BIT ;
		i_nit_e_s[iii][i_view] = upd_lst_S_p->nit_status & EXIST_BIT ;

		if ( i_day_e_s[iii][i_view] ) {

		   n_day_e_s[iii][i_view] = GET_DAY(upd_lst_S_p->day_date_and_view_time);
		   if ( n_day_e_s[iii][i_view] && upd_lst_S_p->tb_day[4] ) {
		     n_view_day++;
		     if ( i_view < 8 )	n_view_day_e++;
		     else			n_view_day_w++;
		     i_ang_day[iii][i_view] = upd_lst_S_p->view_zenith_day;

		     n_bt4_day[iii][i_view] = upd_lst_S_p->tb_day[4];

		     upd_lst_S_p1 = &upd_a_lst_S_p->_upd_for_lst_sv[8];
		     u8_grid_slope = upd_lst_S_p1->snow_bits;
		   }
		   else {
			n_day_e_s[iii][i_view] = 999;
			i_day_e_s[iii][i_view] = 0;
		   }
		}
		if ( i_nit_e_s[iii][i_view] ) {

		   n_nit_e_s[iii][i_view] = GET_DAY(upd_lst_S_p->nit_date_and_view_time);
		   if ( n_nit_e_s[iii][i_view] && upd_lst_S_p->tb_nit[4] ) {
		     n_view_nit++;
		     if ( i_view < 8 )	n_view_nit_e++;
		     else			n_view_nit_w++;
		     i_ang_nit[iii][i_view] = upd_lst_S_p->view_zenith_nit;

		     n_bt4_nit[iii][i_view] = upd_lst_S_p->tb_nit[4];

		     upd_lst_S_p1 = &upd_a_lst_S_p->_upd_for_lst_sv[8];
		     u8_grid_slope = upd_lst_S_p1->snow_bits;
		   }
		   else {
		     n_nit_e_s[iii][i_view] = 999;
		     i_nit_e_s[iii][i_view] = 0;
		   }
		}

	      if ( iii == 0 ) {
		i_day_n_s[i_view] = (n_day_e_s[iii][i_view] == i_day_of_yr_curr
				     && ( upd_lst_S_p->day_status & NEW_BIT ));
		i_nit_n_s[i_view] = (n_nit_e_s[iii][i_view] == i_day_of_yr_curr
				     && ( upd_lst_S_p->nit_status & NEW_BIT ));

		if ( i_day_n_s[i_view] && upd_lst_S_p->tb_day[4] ) {
			n_day_n_s[i_view] = n_day_e_s[iii][i_view];
			n_new_view_day++;
			if ( i_view < 8 )	n_new_view_day_e++;
			else			n_new_view_day_w++;
		}
		if ( i_nit_n_s[i_view] && upd_lst_S_p->tb_nit[4] ) {
			n_nit_n_s[i_view] = n_nit_e_s[iii][i_view];
			n_new_view_nit++;
			if ( i_view < 8 )	n_new_view_nit_e++;
			else			n_new_view_nit_w++;
		}
	      }
	    }
      }

      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

       iii = i_d_n_m_s[ii_upd];
       upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
       upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

       for ( i_v_zone_z = 0; i_v_zone_z < n_v_zone_z; i_v_zone_z++ ) {
	for ( i_v_zone_v = 0; i_v_zone_v < n_v_zone_v; i_v_zone_v++ ) {
		i_view = indx_v_zone[i_v_zone_z][i_v_zone_v];
		upd_lst_S_p = &upd_a_lst_S_p->_upd_for_lst_sv[i_view];
		if ( i_day_e_s[iii][i_view] &&
		     upd_lst_S_p->tb_day[4] > bt4_max_day_v_zone[i_v_zone_z] )
		   bt4_max_day_v_zone[i_v_zone_z] = upd_lst_S_p->tb_day[4];
		if ( i_nit_e_s[iii][i_view] &&
		     upd_lst_S_p->tb_nit[4] > bt4_max_nit_v_zone[i_v_zone_z] )
		   bt4_max_nit_v_zone[i_v_zone_z] = upd_lst_S_p->tb_nit[4];
	}
       }
      }
      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

       iii = i_d_n_m_s[ii_upd];
       upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
       upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

       for ( i_v_zone_z = 0; i_v_zone_z < n_v_zone_z; i_v_zone_z++ ) {
	for ( i_v_zone_v = 0; i_v_zone_v < n_v_zone_v; i_v_zone_v++ ) {
		i_view = indx_v_zone[i_v_zone_z][i_v_zone_v];
		upd_lst_S_p = &upd_a_lst_S_p->_upd_for_lst_sv[i_view];
		if ( i_day_e_s[iii][i_view] && (upd_lst_S_p->tb_day[4] <
			(bt4_max_day_v_zone[i_v_zone_z] - 450) ) ) {
		   i_day_e_s[iii][i_view] = 0;
		   n_view_day--;
		}
		if ( i_nit_e_s[iii][i_view] && (upd_lst_S_p->tb_nit[4] <
			(bt4_max_nit_v_zone[i_v_zone_z] - 300) ) ) {
		   i_nit_e_s[iii][i_view] = 0;
		   n_view_nit--;
		}
	}
       }
      }

      for(i_day_nit_loop = 0; i_day_nit_loop<n_day_nit_loop;i_day_nit_loop++) {

	i_wrt_out_day = i_wrt_out_nit = 0;

	/* check availability of the pair of day/night data */

	i_pair_type = 0;
	i_pair_exist = 0;
	min_ang_dif_f = 255;
	min_dif_view_f = N2_VIEW_ANGS;
	i_use_2nd_upd = -1;

	i_view_day_select = -N2_VIEW_ANGS;
	i_view_nit_select = -N2_VIEW_ANGS;
	bt4_day_f = bt4_nit_f = 0;

	for ( i_case = 0; i_case < n_cases; i_case++ ) {
		i_view_day_ewg[i_case] = i_view_nit_ewg[i_case] = -1;
		i_ang_day_ewg[i_case] = i_ang_nit_ewg[i_case] = 255;
		bt4_day_ewg[i_case] = bt4_nit_ewg[i_case] = 0;
		min_ang_dif_ewg[i_case] = 255;
		min_dif_view_ewg[i_case] = N2_VIEW_ANGS;
		i_use_2nd_upd_ewg[i_case] = -1;
	}

	upd_ang_lst_Dp = upd_ang_lst_Sv[0];
	upd_a_lst_D_p = &upd_ang_lst_Dp[i_6km][j_6km];

	upd_ang_lst_Np = upd_ang_lst_Sv[0];
	upd_a_lst_N_p = &upd_ang_lst_Np[i_6km][j_6km];

	if ( i_day_nit_loop == 0 ) {	/* day */

	 if ( n_new_view_day && n_view_nit ) {

	  if ( n_new_view_day_w && n_view_nit_w ) {

	   for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {

	     upd_lst_S_d = &upd_a_lst_D_p->_upd_for_lst_sv[i_view];

	     if ( i_day_n_s[i_view] && n_day_n_s[i_view] == i_day_of_yr_curr
	          && n_bt4_day[0][i_view] > (150 + bt4_day_ewg[0]) ) {
		i_view_day_ewg[0] = i_view;
		i_ang_day_ewg[0] = upd_lst_S_d->view_zenith_day;
		bt4_day_ewg[0] = n_bt4_day[0][i_view];
	     }
	   }

	   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_nit_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {
		if ( i_nit_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_nit[iii][i_view] - i_ang_day_ewg[0]);
		   dn_day_d_n[iii] = abs(n_day_n_s[i_view_day_ewg[0]] - n_nit_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	   }
	   for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_nit_e_s[iii][i_view] &&
		     i_ang_dif[iii][i_view] < (2.0 * min_ang_dif_upd[iii]) &&
		     n_bt4_nit[iii][i_view] > (100 + bt4_nit_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_nit_upd[iii] = n_bt4_nit[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0*min_ang_dif_ewg[0]) &&
		     bt4_nit_upd[iii] > (100 + bt4_nit_ewg[0]) ) {
			min_ang_dif_ewg[0] = min_ang_dif_upd[iii];
			min_dif_view_ewg[0] = min_dif_view_upd[iii];
			bt4_nit_ewg[0] = bt4_nit_upd[iii];
			i_use_2nd_upd_ewg[0] = iii;
		}
	     }
	   }
	  } /* case 1 */

	  if ( n_new_view_day_e && n_view_nit_e ) {

	   for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {

	     upd_lst_S_d = &upd_a_lst_D_p->_upd_for_lst_sv[i_view];

	     if ( i_day_n_s[i_view] && n_day_n_s[i_view] == i_day_of_yr_curr
		  && n_bt4_day[0][i_view] > (150 + bt4_day_ewg[1]) ) {
		i_view_day_ewg[1] = i_view;
		i_ang_day_ewg[1] = upd_lst_S_d->view_zenith_day;
		bt4_day_ewg[1] = n_bt4_day[0][i_view];
	     }
	   }

	   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = 0; i_view < n_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_nit_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {
		if ( i_nit_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_nit[iii][i_view] - i_ang_day_ewg[1]);
		   dn_day_d_n[iii] = abs(n_day_n_s[i_view_day_ewg[1]] - n_nit_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	   }
	   for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_nit_e_s[iii][i_view] &&
		     i_ang_dif[iii][i_view] < (2.0*min_ang_dif_upd[iii]) &&
		     n_bt4_nit[iii][i_view] > (100 + bt4_nit_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_nit_upd[iii] = n_bt4_nit[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0*min_ang_dif_ewg[1]) &&
		     bt4_nit_upd[iii] > (100 + bt4_nit_ewg[1]) ) {
			min_ang_dif_ewg[1] = min_ang_dif_upd[iii];
			min_dif_view_ewg[1] = min_dif_view_upd[iii];
			bt4_nit_ewg[1] = bt4_nit_upd[iii];
			i_use_2nd_upd_ewg[1] = iii;
		}
	     }
	   }
	  } /* case 2 */

	  if ( !(n_new_view_day_e && n_view_nit_e ||
	         n_new_view_day_w && n_view_nit_w) || u8_grid_slope < 5 ) {

	   for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

	     upd_lst_S_d = &upd_a_lst_D_p->_upd_for_lst_sv[i_view];

	     if ( i_day_n_s[i_view] && n_day_n_s[i_view] == i_day_of_yr_curr
		  && n_bt4_day[0][i_view] > (150 + bt4_day_ewg[2]) ) {
		i_view_day_ewg[2] = i_view;
		i_ang_day_ewg[2] = upd_lst_S_d->view_zenith_day;
		bt4_day_ewg[2] = n_bt4_day[0][i_view];
	     }
	   }

	   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = 0; i_view < n2_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_nit_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {
		if ( i_nit_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_nit[iii][i_view] - i_ang_day_ewg[2]);
		   dn_day_d_n[iii] = abs(n_day_n_s[i_view_day_ewg[2]] - n_nit_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	   }
	   for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_nit_e_s[iii][i_view] &&
		     i_ang_dif[iii][i_view] < (2.0*min_ang_dif_upd[iii]) &&
		     n_bt4_nit[iii][i_view] > (100+ bt4_nit_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_nit_upd[iii] = n_bt4_nit[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0*min_ang_dif_ewg[2]) &&
		      bt4_nit_upd[iii] > (100 + bt4_nit_ewg[2]) ) {
			min_ang_dif_ewg[2] = min_ang_dif_upd[iii];
			min_dif_view_ewg[2] = min_dif_view_upd[iii];
			bt4_nit_ewg[2] = bt4_nit_upd[iii];
			i_use_2nd_upd_ewg[2] = iii;
		}
	     }
	   }
	  } /* case 3 */

	  for ( i_case = 0; i_case < n_cases; i_case++ ) {
		if ( min_ang_dif_ewg[i_case] < (2.0 * min_ang_dif_f) &&
		     bt4_day_ewg[i_case] > (150 + bt4_day_f) &&
		     bt4_nit_ewg[i_case] > (100 + bt4_nit_f) ) {
			min_ang_dif_f = min_ang_dif_ewg[i_case];
			min_dif_view_f = min_dif_view_ewg[i_case];
			i_view_day_select = i_view_day_ewg[i_case];
			i_use_2nd_upd = i_use_2nd_upd_ewg[i_case];
			bt4_day_f = bt4_day_ewg[i_case];
			bt4_nit_f = bt4_nit_ewg[i_case];
			i_pair_exist = 1;
		}
	  }

	  if ( i_pair_exist && min_ang_dif_f < 131 )
		i_view_nit_select = min_dif_view_f;

	  if ( i_view_nit_select < 0 || i_view_day_select < 0 || i_use_2nd_upd < 0 )
		i_pair_exist = 0;

	  if ( ! i_pair_exist && i_view_day_select >= 0 ) {
		upd_ang_lst_Sp = upd_ang_lst_Sv[0];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_d = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_day_select];
		upd_lst_S_d->day_status &= ~NEW_BIT;
	  }
	 }
	}
	else {				/* night */

	 if ( n_new_view_nit && n_view_day ) {

	  if ( n_new_view_nit_e && n_view_day_e ) {

	  for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {

	     upd_lst_S_n = &upd_a_lst_N_p->_upd_for_lst_sv[i_view];

	     if ( i_nit_n_s[i_view] && n_nit_n_s[i_view] == i_day_of_yr_curr
		  && n_bt4_nit[0][i_view] > (100+ bt4_nit_ewg[0]) ) {
			i_view_nit_ewg[0] = i_view;
			i_ang_nit_ewg[0] = upd_lst_S_n->view_zenith_nit;
			bt4_nit_ewg[0] = n_bt4_nit[0][i_view];
	     }
	  }

	  for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = 0; i_view < n_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_day_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {
		if ( i_day_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_day[iii][i_view] - i_ang_nit_ewg[0]);
		   dn_day_d_n[iii] = abs(n_nit_n_s[i_view_nit_ewg[0]] - n_day_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	  }
	  for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = n_view_angs - 1; i_view >= 0; i_view-- ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_day_e_s[iii][i_view] &&
		      i_ang_dif[iii][i_view] < (2.0 * min_ang_dif_upd[iii]) &&
		      n_bt4_day[iii][i_view] > (150 + bt4_day_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_day_upd[iii] = n_bt4_day[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0 * min_ang_dif_ewg[0]) &&
		     bt4_day_upd[iii] > (150 + bt4_day_ewg[0]) ) {
			min_ang_dif_ewg[0] = min_ang_dif_upd[iii];
			min_dif_view_ewg[0] = min_dif_view_upd[iii];
			bt4_day_ewg[0] = bt4_day_upd[iii];
			i_use_2nd_upd_ewg[0] = iii;
		}
	     }
	   }
	  } /* case 1 */

	  if ( n_new_view_nit_w && n_view_day_w ) {

	   for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {

	     upd_lst_S_n = &upd_a_lst_N_p->_upd_for_lst_sv[i_view];

	     if ( i_nit_n_s[i_view] && n_nit_n_s[i_view] == i_day_of_yr_curr
		  && n_bt4_nit[0][i_view] > (100 + bt4_nit_ewg[1]) ) {
			i_view_nit_ewg[1] = i_view;
			i_ang_nit_ewg[1] = upd_lst_S_n->view_zenith_nit;
			bt4_nit_ewg[1] = n_bt4_nit[0][i_view];
	     }
	   }

	   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_day_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {
		if ( i_day_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_day[iii][i_view] - i_ang_nit_ewg[1]);
		   dn_day_d_n[iii] = abs(n_nit_n_s[i_view_nit_ewg[1]] - n_day_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	   }
	   for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = n_view_angs; i_view < n2_view_angs; i_view++ ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_day_e_s[iii][i_view] &&
		     i_ang_dif[iii][i_view] < (2.0 * min_ang_dif_upd[iii]) &&
		     n_bt4_day[iii][i_view] > (150 + bt4_day_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_day_upd[iii] = n_bt4_day[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0 * min_ang_dif_ewg[1]) &&
		     bt4_day_upd[iii] > (150 + bt4_day_ewg[1]) ) {
			min_ang_dif_ewg[1] = min_ang_dif_upd[iii];
			min_dif_view_ewg[1] = min_dif_view_upd[iii];
			bt4_day_ewg[1] = bt4_day_upd[iii];
			i_use_2nd_upd_ewg[1] = iii;
		}
	     }
	   }
	  } /* case 2 */

	  if ( !(n_new_view_nit_e && n_view_day_e ||
	         n_new_view_nit_w && n_view_day_w) || u8_grid_slope < 5 ) {

	   for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

	     upd_lst_S_n = &upd_a_lst_N_p->_upd_for_lst_sv[i_view];

	     if ( i_nit_n_s[i_view] && n_nit_n_s[i_view] == i_day_of_yr_curr
		  && n_bt4_nit[0][i_view] > (100 + bt4_nit_ewg[2]) ) {
			i_view_nit_ewg[2] = i_view;
			i_ang_nit_ewg[2] = upd_lst_S_n->view_zenith_nit;
			bt4_nit_ewg[2] = n_bt4_nit[0][i_view];
	     }
	   }

	   for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

	      iii = i_d_n_m_s[ii_upd];
	      upd_ang_lst_Sp = upd_ang_lst_Sv[iii];
	      upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	      for ( i_view = 0; i_view < n2_view_angs; i_view++ )
			i_ang_dif[iii][i_view] = 255;

	      dn_day_d_n[iii] = 33;
	      min_ang_dif_upd[iii] = 255;
	      bt4_day_upd[iii] = 0;
	      min_dif_view_upd[iii] = -1;

	      for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {
		if ( i_day_e_s[iii][i_view] ) {
		   i_ang_dif[iii][i_view] = abs(i_ang_day[iii][i_view] - i_ang_nit_ewg[2]);
		   dn_day_d_n[iii] = abs(n_nit_n_s[i_view_nit_ewg[2]] - n_day_e_s[iii][i_view]);
		   if ( dn_day_d_n[iii] > 333 )
			dn_day_d_n[iii] = 365 - dn_day_d_n[iii];
			/* one day of difference for leap year is ok */
	        }
	      }
	   }
	   for ( ii_day = 0; ii_day < mx_days; ii_day += 2 ) {

	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		for ( i_view = 0; i_view < n2_view_angs; i_view++ ) {

		  if ( dn_day_d_n[iii] <= ii_day && i_day_e_s[iii][i_view] &&
		     i_ang_dif[iii][i_view] < (2.0 * min_ang_dif_upd[iii]) &&
		     n_bt4_day[iii][i_view] > (150 + bt4_day_upd[iii]) ) {
			min_ang_dif_upd[iii] = i_ang_dif[iii][i_view];
			min_dif_view_upd[iii] = i_view;
			bt4_day_upd[iii] = n_bt4_day[iii][i_view];
		  }
		}
	      }
	      for ( ii_upd = 0; ii_upd < nt_upd; ii_upd++ ) {

		iii = i_d_n_m_s[ii_upd];

		if ( min_ang_dif_upd[iii] < (2.0 * min_ang_dif_ewg[2]) &&
		     bt4_day_upd[iii] > (150 + bt4_day_ewg[2]) ) {
			min_ang_dif_ewg[2] = min_ang_dif_upd[iii];
			min_dif_view_ewg[2] = min_dif_view_upd[iii];
			bt4_day_ewg[2] = bt4_day_upd[iii];
			i_use_2nd_upd_ewg[2] = iii;
		}
	     }
	   }
	  } /* case 3 */

	  for ( i_case = 0; i_case < n_cases; i_case++ ) {
		if ( min_ang_dif_ewg[i_case] < (2.0 * min_ang_dif_f) &&
		     bt4_day_ewg[i_case] > (150 + bt4_day_f) &&
		     bt4_nit_ewg[i_case] > (100 + bt4_nit_f ) ) {
			min_ang_dif_f = min_ang_dif_ewg[i_case];
			min_dif_view_f = min_dif_view_ewg[i_case];
			i_view_nit_select = i_view_nit_ewg[i_case];
			i_use_2nd_upd = i_use_2nd_upd_ewg[i_case];
			bt4_day_f = bt4_day_ewg[i_case];
			bt4_nit_f = bt4_nit_ewg[i_case];
			i_pair_exist = 1;
		}
	  }

	  if ( i_pair_exist && min_ang_dif_f < 131 )
		i_view_day_select = min_dif_view_f;

	  if ( i_view_day_select < 0 || i_view_nit_select < 0 || i_use_2nd_upd < 0 )
		i_pair_exist = 0;

	  if ( ! i_pair_exist && i_view_nit_select >= 0 ) {
		upd_ang_lst_Sp = upd_ang_lst_Sv[0];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_n = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_nit_select];
		upd_lst_S_n->nit_status &= ~NEW_BIT;
	   }
	 }
	}

	if ( ! i_pair_exist )	continue;

	if ( u8_grid_slope > u8_6cons ) {
	    indx_view_day_select = i_view_day_select;
	    indx_view_nit_select = i_view_nit_select;
	    u8_qa_emis = abs(indx_view_day_select - indx_view_nit_select);
	    if ( i_aqua_d && i_aqua_n || !i_aqua_d && !i_aqua_n )
	    	u8_qa_emis += 1;
	    u8_qa_emis /= 2;
	}
	else {
	    if ( i_view_day_select <= 7 )
		indx_view_day_select = abs(i_view_day_select - 7);
	    else
		indx_view_day_select = i_view_day_select - 8;
	    if ( i_view_nit_select <= 7 )
		indx_view_nit_select = abs(i_view_nit_select - 7);
	    else
		indx_view_nit_select = i_view_nit_select - 8;
	    u8_qa_emis = abs(indx_view_day_select - indx_view_nit_select) / 2;
	}
	u8_qa_lst = u8_qa_emis / 2;
	if ( u8_qa_emis > u8_qa_3 )	u8_qa_emis = u8_qa_3;
	if ( u8_qa_lst > u8_qa_3 )	u8_qa_lst = u8_qa_3;

	i_pair_type = 1;

	if ( i_pair_exist ) {

	  if ( i_day_nit_loop == 0 ) {		/* day */
		upd_ang_lst_Sp = upd_ang_lst_Sv[0];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_d = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_day_select];
		upd_ang_lst_Sp = upd_ang_lst_Sv[i_use_2nd_upd];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_n = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_nit_select];
		i_aqua_d = i_aqua[0];
		i_aqua_n = i_aqua[i_use_2nd_upd];
	  }
	  else {					/* night */
		upd_ang_lst_Sp = upd_ang_lst_Sv[0];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_n = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_nit_select];
		upd_ang_lst_Sp = upd_ang_lst_Sv[i_use_2nd_upd];
		upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];
		upd_lst_S_d = &upd_a_lst_S_p->_upd_for_lst_sv[i_view_day_select];
		i_aqua_d = i_aqua[i_use_2nd_upd];
		i_aqua_n = i_aqua[0];
	  }
	  i_day_of_yr_d = GET_DAY(upd_lst_S_d->day_date_and_view_time);
	  i_day_of_yr_n = GET_DAY(upd_lst_S_n->nit_date_and_view_time);

	  i_wrt_out_day = i_day_of_yr_d == i_day_of_yr_curr &&
			(upd_lst_S_d->day_status & NEW_BIT) && !i_day_nit_loop;
	  i_wrt_out_nit = i_day_of_yr_n == i_day_of_yr_curr &&
			(upd_lst_S_n->nit_status & NEW_BIT) &&
			(!i_day_nit_loop && !i_use_2nd_upd || i_day_nit_loop);

	  tsd = 0.02 * upd_lst_S_d->tb_day[7];
	  tsn = 0.02 * upd_lst_S_n->tb_nit[7];
	  dtdn_x = tsd - tsn;
	  if ( dtdn_x > dtdn_x_limit ) {
		if ( i_wrt_out_day )
			upd_lst_S_d->day_status &= ~NEW_BIT;
		if ( i_wrt_out_nit )
			upd_lst_S_n->nit_status &= ~NEW_BIT;

		i_pair_exist = 0;
	  }
	  wda = 0.05 * upd_lst_S_d->cwv_day;
	  wna = 0.05 * upd_lst_S_n->cwv_nit;
	}

	/* make selection based on date and quality */

	/* apply day/night LST algorithm and update lst_6km_l3_outf */

	if ( i_pair_exist ) {

	     /* the profile product will override the default values */

	     id_atm_day = N_ABASES - 1;	id_atm_nit = N_ABASES - 1;

	     int16_sf_elev = upd_lst_S_d->sf_elev;
	     sf_elev_pair = int16_sf_elev;

	     tadd = upd_lst_S_d->ta_day * 0.6 + 185;
	     tann = upd_lst_S_n->ta_nit * 0.6 + 185;

	     u8_data_yr_d = u8getbits(upd_lst_S_d->i_data_yr_dn, 7, 4);
	     u8_data_yr_n = u8getbits(upd_lst_S_n->i_data_yr_dn, 3, 4);

	     for ( k = N_ABASES - 2; k >= 0; k-- ) {

		if ( tadd > _atm_base_temp[k] )	id_atm_day = k;
		else if ( wda > _cwv_lut[id_atm_day][N_CWVm1-1] )
				id_atm_day -= 1;

		if ( tann > _atm_base_temp[k] )	id_atm_nit = k;
		else if ( wna > _cwv_lut[id_atm_nit][N_CWVm1-1] )
				id_atm_nit -= 1;

	     }

	     /* set id_atm_nit = id_atm_day
	     id_atm_nit = id_atm_day; */

	     for (k = 0; k < ntsa; k++) {
		tadit[k] = _tsa_lut[id_atm_day][k];
		tanit[k] = _tsa_lut[id_atm_nit][k];
	     }
	     for (k = 0; k < nrhu; k++) {
		wd[k] = _cwv_lut[id_atm_day][k];
		wn[k] = _cwv_lut[id_atm_nit][k];
	     }
	     
	     /* #### set constrains */

	     if ( wda > 8.0 )	wda = 8.0;
	     if ( wna > 8.0 )	wna = 8.0;

	     if ( wda < 0.10 )	wda = 0.10;
	     if ( wna < 0.10 )	wna = 0.10;

	     wda_min = 0.70 * wda - 0.2;
	     if ( wda_min < 0.01 )   wda_min = 0.01;
	     wda_max = 1.30 * wda + 0.2;

	     wna_min = 0.70 * wna - 0.2;
	     if ( wna_min < 0.01 )   wna_min = 0.01;
	     wna_max = 1.30 * wna + 0.2;

	     if ( wda_max > wna_max )	wv_ref = wda_max;
	     else			wv_ref = wna_max;

/* R. Cember: The PR:QA code-checker gives a violation here, asserting bitwise
   operations on negative numbers.  This is bull.  The arguments passed here to
   macro GET_HRS are unsigned integer 16. */

	     vtime_d_in = GET_HRS(upd_lst_S_d->day_date_and_view_time);
	     vtime_n_in = GET_HRS(upd_lst_S_n->nit_date_and_view_time);

	     szn_pix_in = cos (DTR(0.5*upd_lst_S_d->sola_zenith));
	     vzd_pix_in = cos (DTR(0.5*upd_lst_S_d->view_zenith_day));
	     vzn_pix_in = cos (DTR(0.5*upd_lst_S_n->view_zenith_nit));

	     for (k = 0; k < 7; k++) {
		tbd[k] = 0.02 * upd_lst_S_d->tb_day[k];
		rtbd[k] = bt_to_radiance (k, tbd[k], i_aqua_d);
		tbn[k] = 0.02 * upd_lst_S_n->tb_nit[k];
		rtbn[k] = bt_to_radiance (k, tbn[k], i_aqua_n);
	     }

	     /* calculate rad_ref_d and rad_ref_n */
	     t_ref_d = t_ref_n = tbd[0] + 25.0;

	     for (k = 0; k < 7; k++) {
		rad_ref_d[k] = bt_to_radiance (k, t_ref_d, i_aqua_d);
		rad_ref_n[k] = bt_to_radiance (k, t_ref_n, i_aqua_n);
	     }

	     i_day_yr = GET_DAY (upd_lst_S_d->day_date_and_view_time);

	/* calculate the Sun's elliptic orbit factor */

	if (i_day_yr != i_day_yr_sv) {
	   if (i_day_yr < nday[0] || i_day_yr > nday[12]) sol_irrad_f = 1.0;
	   else {
	     j = 0;
 	     i = 1;
	     while ( (i_day_yr > nday[i]) && (i < 12) ) {
		j = i; 
		i++;
	     } 
	     sol_irrad_f = sol_rat[j] + (i_day_yr - nday[j]) *
			(sol_rat[i] - sol_rat[j]) / (nday[i] - nday[j]);

	   }
	   i_day_yr_sv = i_day_yr;
	   sol_irrad_f_sv = sol_irrad_f;
	}
	else	sol_irrad_f = sol_irrad_f_sv;

	/* set the initial alpha value with the one based on MOD43B1 */

	if ( upd_lst_S_d->b7_alpha ) {
		x[2] = 0.0002 * upd_lst_S_d->b7_alpha;
		if ( x[2] > 6.0 )	x[2] = 6.0;
	}
	else	x[2] = 1.1;

	brdf_ref = 1.5 * x[2];
	amin[2] = 0.5 * x[2] / brdf_ref;
	amax[2] = 1.5 * x[2] / brdf_ref;

	i_land_k = upd_lst_S_d->landcov & 037;

	i_land_catalog = 1;	/* others */

	if ( i_land_k == 21 )	i_land_catalog = 4;	/* ice */

	if ( i_land_k == 16 || i_land_k == 17 || i_land_k == 20 )   /* soil */
					i_land_catalog = 0;
	else if ( i_land_k && i_land_k < 3 || i_land_k > 10 && i_land_k < 13 )
					i_land_catalog = 2;  /* evergreen veg */
	     else if ( i_land_k == 0 || i_land_k == 15 )  /* water or snow */
					i_land_catalog = 3;

	x[NC+4] = 0.0;     /* em31 not defined */

	if ( upd_lst_S_d->n_retrieve_times ) {
	     for (k = 0; k < 6; k++)
		x[NC+k] = 0.002 * upd_lst_S_d->averaged_em[k] + 0.49;

	     x[NC+4] = em_est0[i_land_catalog][4];
	     x[NC+5] = em_est0[i_land_catalog][5];
	}

	/* apply the day/night LST algorithm */

	chisq = Day_Nit_Lst (szn_pix_in, vzd_pix_in, vzn_pix_in);

	a[0] *= t_ref_d;
	a[1] *= t_ref_n;

	tmp1 = split_window_lst (14, &a[0]) - a[0];
	if ( fabs (tmp1) > dt_split_window_bad ) {
		n_out_range_case[16] += 1;
		i_day_nit_lst_status_d = 1;
	}

	tmp2 = split_window_lst (15, &a[0]) - a[1];
	if ( fabs (tmp2) > dt_split_window_bad ) {
		n_out_range_case[17] += 1;
		i_day_nit_lst_status_n = 1;
	}

	i_out_range_lst = 0;	i_bad_spec = 0;
	i_break1 = i_break2 = 0;

	upd_ang_lst_Sp = upd_ang_lst_Sv[0];
	upd_a_lst_S_p = &upd_ang_lst_Sp[i_6km][j_6km];

	if ( a[NC] > (a[NC+4] + 0.04) && a[NC] > (a[NC+5] + 0.04) ) {
		i_bad_spec = 1;
		n_bad_spec++;
	}

	dtdn_a = a[0] - a[1];

	if ( i_wrt_out_day ) {

	   i_near_cld_d = upd_lst_S_d->day_status & NEAR_CLD;
	   i_far_cld_d = upd_lst_S_d->day_status & FAR_CLD;

	   tmp = a[0] - x[0];
	   if ( tmp < d_n_dt_d_min || tmp > d_n_dt_d_max )
	   	n_out_range_case[18] += 1;
	   if ( i_bad_spec || i_day_nit_lst_status_d ||
	       /* dtdn_a > dtdn_max_a_limit ||	fires may be skipped */
	       (tmp < d_n_dt_d_min || tmp > d_n_dt_d_max ) ) {
		upd_lst_S_d->day_status &= ~NEW_BIT;
		i_out_range_lst = 1;
		i_break1 = 1;
	   }
	}
	if ( i_wrt_out_nit ) {

	   i_near_cld_n = upd_lst_S_n->nit_status & NEAR_CLD;
	   i_far_cld_n = upd_lst_S_n->nit_status & FAR_CLD;

	   tmp = a[1] - x[1];
	   if ( tmp < d_n_dt_n_min || tmp > d_n_dt_n_max )
	   	n_out_range_case[19] += 1;
	   if ( i_bad_spec || i_day_nit_lst_status_n ||
	       /* dtdn_a > dtdn_max_a_limit ||	fires may be skipped */
	       (tmp < d_n_dt_n_min || tmp > d_n_dt_n_max ) ) {
		upd_lst_S_n->nit_status &= ~NEW_BIT;
		i_out_range_lst = 1;
		i_break2 = 1;
	   }
	}

	if ( i_out_range_em || i_out_range_lst )
		n_out_range_upd[i_day_nit_loop] += 1;

	if ( i_break1 || i_break2 )
		break;

	u8_quant = u8_qa_0;
	if ( tbn[0] < 216.0 )	u8_quant = u8_qa_1;
	if ( tbn[0] < 206.0 )	u8_quant = u8_qa_2;

	if ( i_out_range_em )
		u8_qa_emis = u8_qa_3;
	if ( u8_quant == u8_qa_1 ) {
	   if ( u8_qa_lst < u8_qa_1 )	u8_qa_lst = u8_qa_1;
	   if ( u8_qa_emis < u8_qa_1 )	u8_qa_emis = u8_qa_1;
	}
	if ( u8_quant == u8_qa_2 ) {
	   if ( u8_qa_lst < u8_qa_2 )	u8_qa_lst = u8_qa_2;
	   if ( u8_qa_emis < u8_qa_2 )	u8_qa_emis = u8_qa_2;
	}
	if ( i_near_cld_d && i_near_cld_n ) {
	   if ( u8_qa_lst < u8_qa_3 )	u8_qa_lst = u8_qa_3;
	   if ( u8_qa_emis < u8_qa_3 )	u8_qa_emis = u8_qa_3;
	}

	/* write out results with corresponding QA bit values */
	{	

	     if ( (x[0] - x[1]) > dtdn_max_x )	dtdn_max_x = x[0] - x[1];
	     dtdn_a = a[0] - a[1];

	     if ( dtdn_a > dtdn_max_a )	dtdn_max_a = dtdn_a;

	     n_pair_new[i_day_nit_loop] += 1;

	     a[2] *= brdf_ref;
	     a[3] *= wv_ref;
	     a[4] *= wv_ref;
	     a[5] *= t_ref_d;
	     a[6] *= t_ref_n;

	     a_x_p = &a[0];

	     if ( !i_day_nit_loop	/* from daytime data */
		|| (i_day_nit_loop && (i_pair_type_sv == -1)) ) {/* no daytime data */
		if ( ! i_day_nit_loop ) {	/* current daytime data used */

		 u8setbits(&QC_emis[i_6km][j_6km], 3, 4,
			(uint8) i_view_nit_select);

		 iii = abs(i_day_of_yr_d - i_day_of_yr_n) / 2;
		 if ( iii > 7 )		iii = 7;
		 u8setbits(&QC_emis[i_6km][j_6km], 6, 3, (uint8) iii);

		 /* if DEM slope considered in day/night method */
			QC_emis[i_6km][j_6km] |= DEM_SLOPE_CONSIDERED;
		}
		else if ( ! em29_6km_l3_Sv[i_6km][j_6km] ) {

		/* current nighttime data used and QC_emis not defined */

		 u8setbits(&QC_emis[i_6km][j_6km], 3, 4,
		 	(uint8) i_view_day_select);

		 iii = abs(i_day_of_yr_d - i_day_of_yr_n) / 2;
		 if ( iii > 7 )		iii = 7;
		 u8setbits(&QC_emis[i_6km][j_6km], 6, 3, (uint8) iii);

		 /* if DEM slope considered in day/night method */
			QC_emis[i_6km][j_6km] |= DEM_SLOPE_CONSIDERED;
		}

		em20_6km_l3_Sv[i_6km][j_6km] = (a_x_p[7] - lst_6km_l3[8].offset)
						/ lst_6km_l3[8].scale;
		em22_6km_l3_Sv[i_6km][j_6km] = (a_x_p[8] - lst_6km_l3[9].offset)
						/ lst_6km_l3[9].scale;
		em23_6km_l3_Sv[i_6km][j_6km] = (a_x_p[9] - lst_6km_l3[10].offset)
						/ lst_6km_l3[10].scale;
		em29_6km_l3_Sv[i_6km][j_6km] = (a_x_p[10] - lst_6km_l3[11].offset)
						/ lst_6km_l3[11].scale;
		em31_6km_l3_Sv[i_6km][j_6km] = (a_x_p[11] - lst_6km_l3[12].offset)
						/ lst_6km_l3[12].scale;
		em32_6km_l3_Sv[i_6km][j_6km] = (a_x_p[12] - lst_6km_l3[13].offset)
						/ lst_6km_l3[13].scale;
	     }

	     if ( i_day_nit_loop && em29_6km_l3_Sv[i_6km][j_6km] &&
		  i_view_day_select == i_view_day_sv &&
		  i_view_nit_select == i_view_nit_sv ) {

		em_6km_sv[0] = (a_x_p[7] - lst_6km_l3[8].offset)
						/ lst_6km_l3[8].scale;
		em_6km_sv[1] = (a_x_p[8] - lst_6km_l3[9].offset)
						/ lst_6km_l3[9].scale;
		em_6km_sv[2] = (a_x_p[9] - lst_6km_l3[10].offset)
						/ lst_6km_l3[10].scale;
		em_6km_sv[3] = (a_x_p[10] - lst_6km_l3[11].offset)
						/ lst_6km_l3[11].scale;
		em_6km_sv[4] = (a_x_p[11] - lst_6km_l3[12].offset)
						/ lst_6km_l3[12].scale;
		em_6km_sv[5] = (a_x_p[12] - lst_6km_l3[13].offset)
						/ lst_6km_l3[13].scale;

		em20_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[0] +
					em20_6km_l3_Sv[i_6km][j_6km] );
		em22_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[1] +
					em22_6km_l3_Sv[i_6km][j_6km] );
		em23_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[2] +
					em23_6km_l3_Sv[i_6km][j_6km] );
		em29_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[3] +
					em29_6km_l3_Sv[i_6km][j_6km] );
		em31_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[4] +
					em31_6km_l3_Sv[i_6km][j_6km] );
		em32_6km_l3_Sv[i_6km][j_6km] = 0.5 * ( em_6km_sv[5] +
					em32_6km_l3_Sv[i_6km][j_6km] );
	     }

	if ( i_day_nit_loop &&
	    (i_view_nit_select >= 4 && i_view_nit_select <= 11 &&
	     min_ang_dif_f < 45 ||
	     (i_view_nit_select < 4 || i_view_nit_select > 11) &&
	     min_ang_dif_f < 30 ) ) {

	     for ( k = 0; k < 6; k++ ) {
		retrieved_em[k] = (int) (500.0 * (a[NC+k] - 0.49) + 0.5);
		upd_lst_S_n->averaged_em[k] =
			( (float32) upd_lst_S_n->n_retrieve_times *
			  upd_lst_S_n->averaged_em[k] +
			  retrieved_em[k] ) /
			( upd_lst_S_n->n_retrieve_times + 1.0 );
	     }
	     if ( upd_lst_S_n->n_retrieve_times < 255 )
			upd_lst_S_n->n_retrieve_times += 1;
	}
	if ( !i_day_nit_loop &&
	     (i_view_day_select >= 4 && i_view_day_select <= 11 &&
	      min_ang_dif_f < 45 ||
	      (i_view_day_select < 4 || i_view_day_select > 11) &&
	      min_ang_dif_f < 30 ) ) {

	     for ( k = 0; k < 6; k++ ) {
		retrieved_em[k] = (int) (500.0 * (a[NC+k] - 0.49) + 0.5);
		upd_lst_S_d->averaged_em[k] =
			( (float32) upd_lst_S_d->n_retrieve_times *
			  upd_lst_S_d->averaged_em[k] +
			  retrieved_em[k] ) /
			( upd_lst_S_d->n_retrieve_times + 1.0 );
	     }
	     if ( upd_lst_S_d->n_retrieve_times < 255 )
			upd_lst_S_d->n_retrieve_times += 1;
	}

	if ( i_wrt_out_day ) {

	 if ( i_use_2nd_upd )	n_times_2nd_upd[0][i_view_day_select] += 1;
	 n_pairs_day[i_view_day_select][i_view_nit_select] += 1;

	 dt_pairs_day[i_view_day_select][i_view_nit_select] += a_x_p[0] - x[0];

		lst_6km_Sd[i_6km][j_6km] = a_x_p[0] / lst_6km_l3[0].scale;

		lst_6km_from_1km_d[i_6km][j_6km] = x[0] / lst_6km_l3[0].scale;
		
		v_time_6km_l3_Sd[i_6km][j_6km] = vtime_d_in;
		if ( i_view_day_select < n_view_angs ) {
		  view_ang_sign = 65.0 - 0.5 * upd_lst_S_d->view_zenith_day;
		  if ( view_ang_sign < 0 )    view_ang_sign = 0;
		  v_angl_6km_l3_Sd[i_6km][j_6km] = (uint8) view_ang_sign;
		}
		else {
		  view_ang_sign = 65.0 + 0.5 * upd_lst_S_d->view_zenith_day;
		  if ( view_ang_sign > 130 )  view_ang_sign = 130;
		  v_angl_6km_l3_Sd[i_6km][j_6km] = (uint8) view_ang_sign;
		}

		u8_qa_man = u8getbits(upd_lst_S_d->QA_bits, 7, 4);
		/* u8_qa_man == u8_1cons if ipx_other_qual_dn or
		   n_cld_grid > 12 or n_sea_grid > 12 in mod11_lst2A1.c */
		if ( u8_qa_man > u8_qa_1 )	u8_qa_man = u8_qa_1;
		u8_cov = upd_lst_S_d->upd_cov_d;
		u8_qa_data = u8_qa_0;
		if ( u8_qa_man == u8_qa_1 )	u8_qa_data = u8_qa_1;
		if ( u8_cov < 27 ) 		u8_qa_data = u8_qa_1;
		u8setbits ( &QC_6km_day[i_6km][j_6km], 2, 1, u8_qa_data );
		if ( i_aqua_d && !i_aqua_n || !i_aqua_d && i_aqua_n )
		   u8setbits ( &QC_6km_day[i_6km][j_6km], 3, 1, u8_qa_1 );
		if ( u8_qa_data >= u8_qa_1 ) {
			if ( u8_qa_lst < u8_qa_1 )	u8_qa_lst = u8_qa_1;
			if ( u8_qa_emis < u8_qa_1 )	u8_qa_emis = u8_qa_1;
		}
		if ( u8_cov < 18 ) {
			u8_qa_emis = u8_qa_2;
			u8_qa_lst = u8_qa_2;
		}
		if ( u8_cov < 9 ) {
			u8_qa_emis = u8_qa_3;
			u8_qa_lst = u8_qa_3;
		}
		u8setbits ( &QC_6km_day[i_6km][j_6km], 5, 2, u8_qa_emis );
		u8setbits ( &QC_6km_day[i_6km][j_6km], 7, 2, u8_qa_lst );
		if ( u8_qa_man == u8_qa_0 &&
		    (u8_qa_data || u8_qa_emis || u8_qa_lst) )
			u8_qa_man = u8_qa_1;
		u8setbits ( &QC_6km_day[i_6km][j_6km], 1, 2, u8_qa_man );

		upd_lst_S_d->day_status &= ~NEW_BIT;
		/* upd_lst_S_d->tb_day[7] = a[0]; */
	     }

	if ( i_wrt_out_nit ) {

	 if ( !i_day_nit_loop )	i_d_n_done_once = 1;

	 if ( i_use_2nd_upd )    n_times_2nd_upd[1][i_view_nit_select] += 1;
	 n_pairs_nit[i_view_nit_select][i_view_day_select] += 1;
	 dt_pairs_nit[i_view_nit_select][i_view_day_select] += a_x_p[1] - x[1];

		lst_6km_Sn[i_6km][j_6km] = a_x_p[1] / lst_6km_l3[4].scale;

		lst_6km_from_1km_n[i_6km][j_6km] = x[1] / lst_6km_l3[4].scale;
		
		v_time_6km_l3_Sn[i_6km][j_6km] = vtime_n_in;
		if ( i_view_nit_select < n_view_angs ) {
		 view_ang_sign = 65.0 - 0.5 * upd_lst_S_n->view_zenith_nit;
		 if ( view_ang_sign < 0 )    view_ang_sign = 0;
		 v_angl_6km_l3_Sn[i_6km][j_6km] = (uint8) view_ang_sign;
		}
		else {
		 view_ang_sign = 65.0 + 0.5 * upd_lst_S_n->view_zenith_nit;
		 if ( view_ang_sign > 130 )  view_ang_sign = 130;
		 v_angl_6km_l3_Sn[i_6km][j_6km] = (uint8) view_ang_sign;
		}

		u8_qa_man = u8getbits(upd_lst_S_n->QA_bits, 3, 4);
		/* u8_qa_man == u8_1cons if ipx_other_qual_dn or
		   n_cld_grid > 12 or n_sea_grid > 12 in mod11_lst2A1.c */
		if ( u8_qa_man > u8_qa_1 )	u8_qa_man = u8_qa_1;
		u8_cov = upd_lst_S_n->upd_cov_n;
		u8_qa_data = u8_qa_0;
		if ( u8_qa_man == u8_qa_1 )	u8_qa_data = u8_qa_1;
		if ( u8_cov < 27 )		u8_qa_data = u8_qa_1;
		u8setbits ( &QC_6km_nit[i_6km][j_6km], 2, 1, u8_qa_data );
		if ( i_aqua_d && !i_aqua_n || !i_aqua_d && i_aqua_n )
		   u8setbits ( &QC_6km_nit[i_6km][j_6km], 3, 1, u8_qa_1 );
		if ( u8_qa_data >= u8_qa_1 ) {
			if ( u8_qa_lst < u8_qa_1 )	u8_qa_lst = u8_qa_1;
			if ( u8_qa_emis < u8_qa_1 )	u8_qa_emis = u8_qa_1;
		}
		if ( u8_cov < 18 ) {
			u8_qa_emis = u8_qa_2;
			u8_qa_lst = u8_qa_2;
		}
		if ( u8_cov < 9 ) {
			u8_qa_emis = u8_qa_3;
			u8_qa_lst = u8_qa_3;
		}
		u8setbits ( &QC_6km_nit[i_6km][j_6km], 5, 2, u8_qa_emis );
		u8setbits ( &QC_6km_nit[i_6km][j_6km], 7, 2, u8_qa_lst );
		if ( u8_qa_man == u8_qa_0 &&
		    (u8_qa_data || u8_qa_emis || u8_qa_lst) )
			u8_qa_man = u8_qa_1;
		u8setbits ( &QC_6km_nit[i_6km][j_6km], 1, 2, u8_qa_man );

		upd_lst_S_n->nit_status &= ~NEW_BIT;
		/* upd_lst_S_n->tb_nit[7] = a[1]; */
	    }
	  }
	}
	if ( !i_day_nit_loop ) {
		i_view_day_sv = i_view_day_select;
		i_view_nit_sv = i_view_nit_select;
		i_pair_type_sv = i_pair_type;
	}
	if ( i_d_n_done_once )	break;

      } /* end of i_day_nit_loop */
     } /* end of j_6km loop */

     if (n_pair_new[0])	n_pair_tot[0] += n_pair_new[0];
     if (n_pair_new[1])	n_pair_tot[1] += n_pair_new[1];

   } /* end of i_6km loop */

   /* update update_handle */

   upd_ang_lst_Sp = upd_ang_lst_Sv[0];
   ier = putMODISarray(update_handle[0],upd_lst_6km[0].name,upd_lst_6km[0].gname,
	upd_sta, upd_dims, &upd_ang_lst_Sp[0][0]);
   if (ier == -1)
	HALTsmf(MODIS_F_MAPI_FAIL,"putMODISarray to upd_ang_lst");

   /* write lst_6km_l3_outf */

   putMODISarray(lst_6km_l3_outf, lst_6km_l3[0].name, lst_6km_l3[0].gname,
		sta_6km_o, dims_6km_o, &lst_6km_Sd[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[1].name, lst_6km_l3[1].gname,
		sta_6km_o, dims_6km_o, &QC_6km_day[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[2].name,lst_6km_l3[2].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sd[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[3].name,lst_6km_l3[3].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sd[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[4].name, lst_6km_l3[4].gname,
		sta_6km_o, dims_6km_o, &lst_6km_Sn[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[5].name, lst_6km_l3[5].gname,
		sta_6km_o, dims_6km_o, &QC_6km_nit[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[6].name,lst_6km_l3[6].gname,
		sta_6km_o, dims_6km_o, &v_time_6km_l3_Sn[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[7].name,lst_6km_l3[7].gname,
		sta_6km_o, dims_6km_o, &v_angl_6km_l3_Sn[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[8].name,lst_6km_l3[8].gname,
		sta_6km_o, dims_6km_o, &em20_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[9].name,lst_6km_l3[9].gname,
		sta_6km_o, dims_6km_o, &em22_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[10].name,lst_6km_l3[10].gname,
		sta_6km_o, dims_6km_o, &em23_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[11].name,lst_6km_l3[11].gname,
		sta_6km_o, dims_6km_o, &em29_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[12].name,lst_6km_l3[12].gname,
		sta_6km_o, dims_6km_o, &em31_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[13].name,lst_6km_l3[13].gname,
		sta_6km_o, dims_6km_o, &em32_6km_l3_Sv[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[14].name,lst_6km_l3[14].gname,
		sta_6km_o, dims_6km_o, &lst_6km_from_1km_d[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[15].name,lst_6km_l3[15].gname,
		sta_6km_o, dims_6km_o, &lst_6km_from_1km_n[0][0]);
   putMODISarray(lst_6km_l3_outf, lst_6km_l3[16].name,lst_6km_l3[16].gname,
		sta_6km_o, dims_6km_o, &QC_emis[0][0]);

   for ( ii_upd = 1; ii_upd < nt_upd; ii_upd++ )
     if ( update_handle[ii_upd] )
	closeMODISfile(&update_handle[ii_upd]);

  /* calculate global statistics in the 6km L3 output */

  for (i_6km = 0; i_6km < nrow_6km; i_6km++) {

    for (j_6km = 0; j_6km < ncol_6km; j_6km++) {

	u8_1 = (uint8) QC_6km_day[i_6km][j_6km] & u8_3cons;
	u8_2 = (uint8) QC_6km_nit[i_6km][j_6km] & u8_3cons;
	if ( u8_1 == u8_0cons )	total_l3_6km_dn[0]++;
	if ( u8_1 == u8_1cons ) 	total_l3_6km_dn[1]++;
	if ( u8_1 == u8_2cons ) 	total_l3_6km_dn[2]++;
	if ( u8_1 == u8_3cons ) 	total_l3_6km_dn[3]++;
	if ( u8_2 == u8_0cons )		total_l3_6km_dn[0]++;
	if ( u8_2 == u8_1cons ) 	total_l3_6km_dn[1]++;
	if ( u8_2 == u8_2cons ) 	total_l3_6km_dn[2]++;
	if ( u8_2 == u8_3cons ) 	total_l3_6km_dn[3]++;
    }
  }

  met_cnt_6km.QA_GoodQuality = total_l3_6km_dn[0];
  met_cnt_6km.QA_OtherQuality = total_l3_6km_dn[1];
  met_cnt_6km.QA_NotProducedCloud = total_l3_6km_dn[2];
  met_cnt_6km.QA_NotProducedOther = total_l3_6km_dn[3];
  frac1 = (float32) met_cnt_6km.QA_GoodQuality / met_cnt_6km.AllPixelsProcessed;
  frac2 = (float32) met_cnt_6km.QA_OtherQuality / met_cnt_6km.AllPixelsProcessed;
  frac3 = (float32) met_cnt_6km.QA_NotProducedCloud / met_cnt_6km.AllPixelsProcessed;
  frac4 = (float32) met_cnt_6km.QA_NotProducedOther / met_cnt_6km.AllPixelsProcessed;

  /* It is necessary to use the following formula because the tile has not
     been completely covered by input granules */
  met_cnt_6km.QA_NotProducedOther = (met_cnt_6km.AllPixelsProcessed -
	total_l3_6km_dn[0] - total_l3_6km_dn[1] - total_l3_6km_dn[2]);

  if (met_cnt_6km.QA_NotProducedOther < 0) met_cnt_6km.QA_NotProducedOther = 0;

  n_tiles = 0;

  DoMetaData( n_6km_l3_old_file, (int32) 3, (PGSt_PC_Logical) PCLogL3_6km,
	(PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, lst_6km_l3_outf);
/*
  DoMetaData( n_old_update, (int32) 3,
	(PGSt_PC_Logical) PCLogupdate[i_days_in_cycle_update-1],
	(PGSt_integer) n_tiles, (PGSt_integer) i_tile_proc, update_handle[0]);
*/
  if ( closeMODISfile(&update_handle[0]) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "update_for_lst file failed");

  if ( closeMODISfile(&lst_6km_l3_outf) == -1 )
	HALTsmf(MODIS_E_CLOSE_HDF_FILE, "6km L3 output file failed");

/* #### */
  for ( i = 0; i < N2_VIEW_ANGS; i++ ) {
     for ( j = 0; j < N2_VIEW_ANGS; j++ ) {
	if ( n_pairs_day[i][j] )  dt_pairs_day[i][j] /= n_pairs_day[i][j];
	if ( n_pairs_nit[i][j] )  dt_pairs_nit[i][j] /= n_pairs_nit[i][j];
     }
  }

  /* free all allocated spaces */

  free_lst_space ();

  /* R. Cember: exit thru SMFLOG() */

if ( (frac1 + frac2) > 0 )
sprintf(LogMsg,"Completed %s successfully (%d for h%02dv%02d) QA Fractions: %.5f %.5f",
		esdt_6km_L3, rel_vers_tile, tile_pntr[i_tile_proc].ih,
		tile_pntr[i_tile_proc].iv, frac1, frac2 );
else
sprintf(LogMsg,"Completed %s (%d for h%02dv%02d) QA Fractions: %.5f %.5f %.5f %.5f",
		esdt_6km_L3, rel_vers_tile, tile_pntr[i_tile_proc].ih,
		tile_pntr[i_tile_proc].iv, frac1, frac2, frac3, frac4 );

  SMFLOG(LogMsg);

} /* end main function */


float64 f_mod_rad7_inv (int32 iband, float32 rad1, int32 i_mod)
{
/*
!C***************************************************************************

!Description:
   This function calculates brightness temperature of MODIS band radiance

!Input Parameters:	iband -	MODIS band index (0-6 for bands 20, 22, 23, 29,
				31-33)
			rad1 -  band radiance

			i_mod - 1 for Aqua FM1 MODIS data
		     
!Output Parameters:	none

!Revision History:

Returns:

	tb - brightness temperature.

References and Credits:

   Version V2.1  2002/09/08  11:42     wan

   Version V1.1  2000/05/15  02:02:41  wan

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes
	look-up table method and quick bineary search are used in
	this function.

!END***********************************************************************/

	int32   i, j, k, m;
	float64	tb;

	i = 0;
	j = NCELM1;

	if ( i_mod ) {
	  while ((m = j - i) != 1) {
		k = (i + j) / 2;
		if (rad1 >= _mod_fm1_rad7[iband][k])	i = k;
		else 					j = k;
	  }
	  tb = TMIN + DT_LUT * (i + (rad1 - _mod_fm1_rad7[iband][i]) /
			(_mod_fm1_rad7[iband][j] - _mod_fm1_rad7[iband][i]));
	}
	else {
	  while ((m = j - i) != 1) {
		k = (i + j) / 2;
		if (rad1 >= _mod_rad7[iband][k])	i = k;
		else 					j = k;
	  }
	  tb = TMIN + DT_LUT * (i + (rad1 - _mod_rad7[iband][i]) /
			(_mod_rad7[iband][j] - _mod_rad7[iband][i]));
	}

	return (tb);
}

void
poly_intp_vector(float64 *x, int32 i, float64 xx, int32 k, float64 **y, float64 *z)
{
/*
!C***************************************************************************

!Description:
This function makes three-point polyminal interpolation for a vector variables

!Input Parameters:	x 	pointer to X axis
			i 	number of x elements
			k 	number of elements for output
			xx 	value at which interpolation will be made
			y 	2-d array for Y values at x's
		     
!Output Parameters:	z 	pointer to interpolated vector values


!Revision History:

Returns:

   none

References and Credits:

   Version B3.2 09-11-95

   Written by:

        Zhengming Wan
        UCSB/ICESS
        wan@icess.ucsb.edu

!Team-unique Header:
         This software is developed by MODIS Science Team member
	 Zhengming Wan for the National Aeronautics and Space Administration,
         Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  int32		        i1, i2, n;
  static float64	x_sv[3] = { 0., 0., 0. };
  static float64	a1, a2, a3;
  static float64	f1, f2, f3;
  float64		x0, x1, x2;
  float64		*y_p;

  i1 = i + 1;	i2 = i + 2;

  if (  fabs (x[i] - x_sv[0]) > SMALL_VALUE ||
	fabs (x[i1] - x_sv[1]) > SMALL_VALUE ||
	fabs (x[i2] - x_sv[2]) > SMALL_VALUE ) {
    x_sv[0] = x[i];
    x_sv[1] = x[i1];
    x_sv[2] = x[i2];
    a1 = 1. / (x[i] - x[i2]);
    a2 = 1. / (x[i1] - x[i]);
    a3 = 1. / (x[i1] - x[i2]);
    f1 = - a1 * a2;
    f2 = a2 * a3;
    f3 = a1 * a3;
  }
  
  x0 = xx - x[i];
  x1 = xx - x[i1];
  x2 = xx - x[i2];
  
  for (n = 0; n < k; n++) {
    y_p = &y[n][0];
    z[n] = y_p[i] * f1 * x1 * x2 + y_p[i1] * f2 * x0 * x2 +
      y_p[i2] * f3 * x0 * x1;
  }
}

char **
alloc_2d (int32 rows, int32 cols, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 2-d array.

!Input Parameters:	rows 	the first dimension
			cols 	the second dimension
			elsize 	element size in bytes
		     
!Output Parameters:	xh 	pointer to the array


!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

        Zhengming Wan
        UCSB/ICESS
        wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **xh = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	sp = sizeof(char *);
  int32	j = 0;
  size_t  siz;
  
  cols *= elsize; 	/*  cols is now row-size in bytes	*/
  siz = rows * cols;
  
  /*  buf is pointer to data area itself	*/
  /*  xh is pointer to row addresses	*/
  if ((buf = (char *) malloc (siz)) == NULL) {
    sprintf (modstring, "alloc_2d: can't allocate buffer of size %lu", siz);
    HALTsmf(MODIS_E_CALLOC, modstring);
  }

total_space += siz;

  /*  make xh for pointer vector */
  if ((xh = (char **) malloc(rows * sp)) == NULL) {
    free (buf);
    HALTsmf(MODIS_E_CALLOC, "alloc_2d: can't allocate pointer vector");
  }

total_space += rows * sp;

  /* set pointers to the beginning of each row */
  for (j = 0; j < rows; j++)
    xh[j] = buf + j*cols;
  
  return(xh);
}

char ***
alloc_3d (int32 dim1, int32 dim2, int32 dim3, size_t elsize)
{
/*
!C***************************************************************************

!Description:
This function allocates dynamic memories for a 3-d array.

!Input Parameters:	dim1, dim2, dim3 	dimensions
			elsize 			element size in bytes
		     
!Output Parameters:	x1d 	pointer to the array


!Revision History:

Returns:

   xh upon success.
   exit upon failure.

References and Credits:

   Version B3.1 08-11-95

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.


!Design Notes

!END***********************************************************************/

  char	*buf = NULL;
  char  **x2d = NULL;
  char  ***x1d = NULL;
  char  modstring[PGS_SMF_MAX_MSG_SIZE] = {0};
  int32	j=0;
  int32	sp = sizeof(char *);
  size_t	siz;
  
  dim3 *= elsize; 	/*  dim3 is now row-size in bytes */

  siz = dim1 * dim2 * dim3;

  /*  buf is pointer to data area itself	*/
  if ((buf = (char *) malloc(siz)) == NULL) {
     sprintf (modstring, "alloc_3d: can't allocate buffer of size %lu", siz);
     HALTsmf(MODIS_E_CALLOC, modstring);
  }

total_space += siz;

  /*  make x2d for pointer vector for the 2nd dimension */
  if ((x2d = (char **) malloc(dim1 * dim2 * sp)) == NULL) {
     free(buf);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 2nd dim");
  }

total_space += dim1 * dim2 * sp;

  /*  make x1d for pointer vector for the 1st dimemsion */
  if ((x1d = (char ***) malloc(dim1 * sp)) == NULL) {
     free(buf);	free(x2d[0]);
     HALTsmf(MODIS_E_CALLOC, 
	   "alloc_3d: can't allocate pointer vector for 1st dim");
  }

total_space += dim1 * sp;

  /* set the 2nd dimension pointers */
  for (j = 0; j < dim1 * dim2; j++)
    x2d[j] = buf + j*dim3;

  /* set the 1st dimension pointers */
  for (j = 0; j < dim1; j++)
    x1d[j] = x2d + j * dim2;
  
  return(x1d);
}

void free_lst_space(void) 
{
/*
!C***************************************************************************

!Description:
This function frees all allocated dynamic memories normally or abnormally 

!Input Parameters:	none
		     
!Output Parameters:	none


!Revision History:

   Version B3.2 09-11-95

Returns:                none

References and Credits:

   Version V1.1 07-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  if (lst_6km_Sd) { free(lst_6km_Sd[0]); free (lst_6km_Sd); lst_6km_Sd = NULL; }
  if (QC_6km_day) { free(QC_6km_day[0]); free(QC_6km_day); QC_6km_day = NULL; }
  if (v_time_6km_l3_Sd) { free (v_time_6km_l3_Sd[0]); free (v_time_6km_l3_Sd);
			v_time_6km_l3_Sd = NULL; }
  if (v_angl_6km_l3_Sd) { free (v_angl_6km_l3_Sd[0]); free (v_angl_6km_l3_Sd);
			v_angl_6km_l3_Sd = NULL; }
  if (lst_6km_Sn) { free(lst_6km_Sn[0]); free (lst_6km_Sn); lst_6km_Sn = NULL; }
  if (QC_6km_nit) { free(QC_6km_nit[0]); free (QC_6km_nit); QC_6km_nit = NULL; }
  if (v_time_6km_l3_Sn) { free (v_time_6km_l3_Sn[0]); free (v_time_6km_l3_Sn);
			v_time_6km_l3_Sn = NULL; }
  if (v_angl_6km_l3_Sn) { free (v_angl_6km_l3_Sn[0]); free (v_angl_6km_l3_Sn);
			v_angl_6km_l3_Sn = NULL; }
  if (em20_6km_l3_Sv) { free (em20_6km_l3_Sv[0]); free (em20_6km_l3_Sv);
			em20_6km_l3_Sv = NULL; }
  if (em22_6km_l3_Sv) { free (em22_6km_l3_Sv[0]); free (em22_6km_l3_Sv);
			em22_6km_l3_Sv = NULL; }
  if (em23_6km_l3_Sv) { free (em23_6km_l3_Sv[0]); free (em23_6km_l3_Sv);
			em23_6km_l3_Sv = NULL; }
  if (em29_6km_l3_Sv) { free (em29_6km_l3_Sv[0]); free (em29_6km_l3_Sv);
			em29_6km_l3_Sv = NULL; }
  if (em31_6km_l3_Sv) { free (em31_6km_l3_Sv[0]); free (em31_6km_l3_Sv);
			em31_6km_l3_Sv = NULL; }
  if (em32_6km_l3_Sv) { free (em32_6km_l3_Sv[0]); free (em32_6km_l3_Sv);
			em32_6km_l3_Sv = NULL; }
  if (QC_emis) { free (QC_emis[0]); free (QC_emis); QC_emis = NULL; }
  if (lst_6km_from_1km_d) { free (lst_6km_from_1km_d[0]);
	free (lst_6km_from_1km_d); lst_6km_from_1km_d = NULL; }
  if (lst_6km_from_1km_n) { free (lst_6km_from_1km_n[0]);
	free (lst_6km_from_1km_n); lst_6km_from_1km_n = NULL; }

  if ( upd_ang_lst_Sv[0] ) { free (upd_ang_lst_Sv[0][0]);
	free (upd_ang_lst_Sv[0]); upd_ang_lst_Sv[0] = NULL; }
  if ( upd_ang_lst_Sv[1] ) { free (upd_ang_lst_Sv[1][0]);
	free (upd_ang_lst_Sv[1]); upd_ang_lst_Sv[1] = NULL; }
  if ( upd_ang_lst_Sv[2] ) { free (upd_ang_lst_Sv[2][0]);
	free (upd_ang_lst_Sv[2]); upd_ang_lst_Sv[2] = NULL; }
  if ( upd_ang_lst_Sv[3] ) { free (upd_ang_lst_Sv[3][0]);
	free (upd_ang_lst_Sv[3]); upd_ang_lst_Sv[3] = NULL; }

  if ( tir_term_tbld ) { free (tir_term_tbld[0]); free (tir_term_tbld);
		tir_term_tbld = NULL; }
  if ( tir_term_tbln ) { free (tir_term_tbln[0]); free (tir_term_tbln);
		tir_term_tbln = NULL; }
  if ( sol_term_tbls ) { free (sol_term_tbls[0][0]); free (sol_term_tbls[0]);
		free (sol_term_tbls); sol_term_tbls = NULL; }

}

void SDsetdim_LST (int32 f_id, sds_t sds_struct, int32 *sds_id) 
{
/*
!C***************************************************************************

!Description:
This function sets dimension names and returns SD id for LST products

!Input Parameters:	f_id 		file id
			sds_struct 	attribute structure
		     
!Output Parameters:	sds_id 		pointer to attributes id


!Revision History:

   $Log: SDsetdim_LST.c,v $
 * Revision 2.0  1997/09/15  23:55     Z. Wan
 *
 * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 10-21-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/

  int32	arr_index, dim_id;
  
  arr_index = SDnametoindex (f_id, sds_struct.name);
  *sds_id = SDselect (f_id, arr_index);

  if ( sds_struct.rank > 2 )
    HALTsmf(MODIS_E_ATTR_FAIL, sds_struct.name);

}

void SDsetattr_LST (int32 sd_id, char *name, int32 nt, int32 count, VOIDP data)
/*
!C***************************************************************************

!Description:
   This function sets SDS attribute

!Input Parameters:	sd_id 	file or SDS id number
			name 	pointer to attribute name
			nt 	data type
			count 	number of attributes to be set
			data 	attributes to be set
		     
!Output Parameters:	none


!Revision History:

   * Revision V2.01 1997/11/24	16:14:06 Z.Wan
   * make vrange & fillv to be compatible with field data type.
   *
   * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 09-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{

  if ( SDsetattr (sd_id, name, nt, count, data) == FAIL)
    HALTsmf(MODIS_E_ATTR_FAIL, name);
}

void Error(const char *message, const char *module)
/******************************************************************************
!C

!Description: 

  'Error' handles the error status for the 'pntr' functions.
  The error messages are written to LogStatus using SDP toolkit SMF
  functionality, and execution terminates through the HALTsmf micro.
 
!Input Parameters:
  message	error message string
  module	module in which the error occurred

!Output Parameters: (none)

!Revision History:

 * revised 05/20/1999 17:43	Z. Wan

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

  Usage Notes: (none)

!END
*******************************************************************************/
{

  char msg[PGS_SMF_MAX_MSG_SIZE];

  sprintf(msg, "%s in %s", message, module);
  HALTsmf(MODIS_E_BUF_ERROR, msg);

}


void zeroout_general(void *buf, int32 nelems, int32 type_size)
/*
!C*****************************************************************************

!Description:
   This function sets contiguous memory locations to zero.

!Input Parameters:	buf	the starting address of the memory location
			nelems	number of elements 
		        type_size  number of bytes in the data type
!Output Parameters:	none


!Revision History:


!References and Credits:

   Version v1.1 09-15-96

   Written by:

	R. Cember, MODIS Science Data Support Team, based on function 
          "zeroout" by ...

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/
{  /* begin function */

uint8 *buf8;
uint16 *buf16;
uint32 *buf32;

if (type_size == 1)  {
  buf8 = buf;
  do {
    *buf8 = 0;
    buf8++;
  }
  while (--nelems);
}
else if (type_size == 2) {
  buf16 = buf;
  do {
    *buf16 = 0;
    buf16++;
  }
  while (--nelems);
}
else if (type_size == 4) {
  buf32 = buf;
  do {
    *buf32 = 0;
    buf32++;
  }
  while (--nelems);
}
else {
  HALT("STOP -- wrong data type in zeroout_general -- use 1, 2 or 4.\n");
  exit (1);
}

}  /* end function */


int32 iday_of_year(int32 iyr, int32 imnth, int32 iday)
{
/*
!C*****************************************************************************

!Description:
   This function calculates the day-of-year for given iyr, imnth, and iday

!Input Parameters:	iyr	- year
			imnth	- month
			iday	- date
		     
!Output Parameters:	iday	- day-of-year

  Returns:

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	daytab[2][13] = {
		0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
		0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
	};
	int32	i, leap;

	leap = iyr % 4 == 0 && iyr % 100 != 0 || iyr % 400 == 0;

	for (i = 1; i < imnth; i++)
		iday += daytab[leap][i];

	return (iday);
}

double
Day_Nit_Lst ( double szn_pix, double vzd_pix, double vzn_pix )
{
/*
!C****************************************************************************

!Description:
   This function is the MODIS day/night LST algorithm

!Input Parameters: szn_pix - cosine of solar zenith angle at the pixel (grid)
		   vzd_pix - cosine of day viewing zenith angle
		   vzn_pix - cosine of night viewing zenith angle

	and in global variables:
		   tbd[], rtbd[] - daytime brightness temperatures and
		   		radiances in MODIS bands 20, 22-23, 29, 31-33.
		   tbn[], rtbn[] - nighttime brightness temperatures and
		   		radiances in MODIS bands 20, 22-23, 29, 31-33.
		     
!Output Parameters:	a[14]	solution for Ts-day, Ts-night, alpha,
				cwv-day, cwv-night, Ta-day, Ta-night, and
				emissivity values in the above seven bands.

  Returns:

   chisq - the least squares of the differences in given radiances and
	   the calculated radiances based on the retrieved values.

!Revision History:

!References and Credits:

   Zhengming Wan and Zhao-Liang Li, "A Physics-based algorithm for retrieving
	land-surface emissivity and temperature from EOS/MODIS data", IEEE
	Trans. Geoscience and Remote Sensing, vol. 35, no. 4, pp. 980-996, 1997.

   Zhengming Wan, MODIS LST ATBD (ATBD-MOD-11), v3.3, April, 1999.


   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	ipt;
	int32	j, k;
	int32	ib, ic, ij;
	int32	iyy, izz;
	int32	nsv, i_save_all;
	int32	iout, nout;
	int32	isv[MA];
	double	a_bk[MA];

	double	chisq_sml;
	double	chisq, alamda, eps_m = 2.0e-5;

	double	exn[MA] = {
		12.91, 12.25, 11.98, 6.00, 4.70, 4.11, 3.74,
		12.91, 12.25, 11.98, 6.00, 4.70, 4.11, 3.74 };

	/* variables used in bilinear interpolation for solar terms */
	int32	i_szn_pix, i_vzd_pix, i_vzn_pix;
	int32	i_szn_pix1, i_vzd_pix1, i_vzn_pix1;
	int32	N_SZNm1, N_VZNm1;
	int32	n_D, n_N;
	double	tmp, tmp1, tmp2;
	/* the 2st number for i_sun, the 2nd for i_vzn */
	float32	*fpt00, *fpt01, *fpt10, *fpt11;
	float32	*fpt00h, *fpt01h, *fpt10h, *fpt11h;
	float64 tmp_0km, tmp_4km, h_scale = 0.0;

	char modsmf_buffer[128];

	N_SZNm1 = N_SZN - 1;
	N_VZNm1 = N_VZN - 1;

	i_szn_pix = locate (_szn_lut, N_SZN, szn_pix);
	i_vzd_pix = locate (_vzn_lut, N_VZN, vzd_pix);
	i_vzn_pix = locate (_vzn_lut, N_VZN, vzn_pix);

	if (i_szn_pix < 0)		i_szn_pix = 0;
	if (i_szn_pix == N_SZNm1)	i_szn_pix = N_SZNm1 - 1;
	if (i_vzd_pix < 0)		i_vzd_pix = 0;
	if (i_vzd_pix == N_VZNm1)	i_vzd_pix = N_VZNm1 - 1;
	if (i_vzn_pix < 0)		i_vzn_pix = 0;
	if (i_vzn_pix == N_VZNm1)	i_vzn_pix = N_VZNm1 - 1;

	i_szn_pix1 = i_szn_pix + 1;
	i_vzd_pix1 = i_vzd_pix + 1;
	i_vzn_pix1 = i_vzn_pix + 1;

	tmp1 = (szn_pix - _szn_lut[i_szn_pix]) /
		(_szn_lut[i_szn_pix1] - _szn_lut[i_szn_pix]);
	tmp2 = (vzd_pix - _vzn_lut[i_vzd_pix]) /
		(_vzn_lut[i_vzd_pix1] - _vzn_lut[i_vzd_pix]);

    n_D = i_aqua_d * 2 * N_ABASES;
    n_N = i_aqua_n * 2 * N_ABASES;

    fpt00 = &sol_term_tbls[n_D+id_atm_day][i_szn_pix][i_vzd_pix].sol_terms[0];
    fpt01 = &sol_term_tbls[n_D+id_atm_day][i_szn_pix][i_vzd_pix1].sol_terms[0];
    fpt10 = &sol_term_tbls[n_D+id_atm_day][i_szn_pix1][i_vzd_pix].sol_terms[0];
    fpt11 = &sol_term_tbls[n_D+id_atm_day][i_szn_pix1][i_vzd_pix1].sol_terms[0];

    fpt00h = &sol_term_tbls[n_D+id_atm_day+N_ABASES][i_szn_pix][i_vzd_pix].sol_terms[0];
    fpt01h = &sol_term_tbls[n_D+id_atm_day+N_ABASES][i_szn_pix][i_vzd_pix1].sol_terms[0];
    fpt10h = &sol_term_tbls[n_D+id_atm_day+N_ABASES][i_szn_pix1][i_vzd_pix].sol_terms[0];
    fpt11h = &sol_term_tbls[n_D+id_atm_day+N_ABASES][i_szn_pix1][i_vzd_pix1].sol_terms[0];

    for (j = 0; j < ntsa; j++) {
      for (k = 0; k < nrhu; k++) {

	for (ic = 0; ic < 3; ic++) {

	  tmp = *fpt01 - *fpt00;
	  tmp_0km = ((tmp2 * (*fpt11 - *fpt10 - tmp) + *fpt10 - *fpt00) *
			   tmp1 + tmp2 * tmp + *fpt00) * sol_irrad_f;
	  tmp = *fpt01h - *fpt00h;
	  tmp_4km = ((tmp2 * (*fpt11h - *fpt10h - tmp) + *fpt10h - *fpt00h) *
			   tmp1 + tmp2 * tmp + *fpt00h) * sol_irrad_f;
	  h_scale = (tmp_4km - tmp_0km) / 4000.;
	  rups[j][k][ic] = tmp_0km + sf_elev_pair * h_scale;

	  fpt00++;	fpt01++;	fpt10++;	fpt11++;
	  fpt00h++;	fpt01h++;	fpt10h++;	fpt11h++;
	}
	for (ic = 0; ic < 3; ic++) {

	  tmp = *fpt01 - *fpt00;
	  tmp_0km = ((tmp2 * (*fpt11 - *fpt10 - tmp) + *fpt10 - *fpt00) *
			   tmp1 + tmp2 * tmp + *fpt00) * sol_irrad_f;
	  tmp = *fpt01h - *fpt00h;
	  tmp_4km = ((tmp2 * (*fpt11h - *fpt10h - tmp) + *fpt10h - *fpt00h) *
			   tmp1 + tmp2 * tmp + *fpt00h) * sol_irrad_f;
	  h_scale = (tmp_4km - tmp_0km) / 4000.;
	  rgfd[j][k][ic] = tmp_0km + sf_elev_pair * h_scale;

	  fpt00++;	fpt01++;	fpt10++;	fpt11++;
	  fpt00h++;	fpt01h++;	fpt10h++;	fpt11h++;
	}
	for (ic = 0; ic < 3; ic++) {

	  tmp = *fpt01 - *fpt00;
	  tmp_0km = ((tmp2 * (*fpt11 - *fpt10 - tmp) + *fpt10 - *fpt00) *
			   tmp1 + tmp2 * tmp + *fpt00) * sol_irrad_f;
	  tmp = *fpt01h - *fpt00h;
	  tmp_4km = ((tmp2 * (*fpt11h - *fpt10h - tmp) + *fpt10h - *fpt00h) *
			   tmp1 + tmp2 * tmp + *fpt00h) * sol_irrad_f;
	  h_scale = (tmp_4km - tmp_0km) / 4000.;
	  rds[j][k][ic] = tmp_0km + sf_elev_pair * h_scale;

	  fpt00++;	fpt01++;	fpt10++;	fpt11++;
	  fpt00h++;	fpt01h++;	fpt10h++;	fpt11h++;
	}
      }
    }

	/* daytime thermal terms */

	tmp2 = (vzd_pix - _vzn_lut[i_vzd_pix]) /
		(_vzn_lut[i_vzd_pix1] - _vzn_lut[i_vzd_pix]);

	fpt00 = &tir_term_tbld[n_D+id_atm_day][i_vzd_pix].tir_terms[0];
	fpt01 = &tir_term_tbld[n_D+id_atm_day][i_vzd_pix1].tir_terms[0];
	fpt00h = &tir_term_tbld[n_D+id_atm_day+N_ABASES][i_vzd_pix].tir_terms[0];
	fpt01h = &tir_term_tbld[n_D+id_atm_day+N_ABASES][i_vzd_pix1].tir_terms[0];

	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

			tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
			tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
			transd[j][k][ic] = tmp_0km + sf_elev_pair * h_scale;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}
	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

			tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
			tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
			rupt[j][k][ic] = tmp_0km + h_scale * sf_elev_pair;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}
	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

			tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
			tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
			rgft[j][k][ic] = tmp_0km + h_scale * sf_elev_pair;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}

	/* nighttime terms */

	tmp2 = (vzn_pix - _vzn_lut[i_vzn_pix]) /
		(_vzn_lut[i_vzn_pix1] - _vzn_lut[i_vzn_pix]);

	fpt00 = &tir_term_tbln[n_N+id_atm_nit][i_vzn_pix].tir_terms[0];
	fpt01 = &tir_term_tbln[n_N+id_atm_nit][i_vzn_pix1].tir_terms[0];
	fpt00h = &tir_term_tbln[n_N+id_atm_nit+N_ABASES][i_vzn_pix].tir_terms[0];
	fpt01h = &tir_term_tbln[n_N+id_atm_nit+N_ABASES][i_vzn_pix1].tir_terms[0];

	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

	  		tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
	  		tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
	  		transn[j][k][ic] = tmp_0km + h_scale * sf_elev_pair;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}
	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

			tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
			tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
			rupn[j][k][ic] = tmp_0km + h_scale * sf_elev_pair;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}
	for (j = 0; j < ntsa; j++) {
	    for (k = 0; k < nrhu; k++) {
		for (ic = 0; ic < NC; ic++) {

			tmp_0km = tmp2 * (*fpt01 - *fpt00) + *fpt00;
			tmp_4km = tmp2 * (*fpt01h - *fpt00h) + *fpt00h;
			h_scale = (tmp_4km - tmp_0km) / 4000.;
			rgfn[j][k][ic] = tmp_0km + h_scale * sf_elev_pair;

			fpt00++;	fpt01++;
			fpt00h++;	fpt01h++;
		}
	    }
	}

/* 2001/04/29 Z. Wan replaced the statisticl regression method
   with initial guess of tsd and tsn from the split-window method, and
   initial emissivities from mean and/or previously retrieved values */

	x[0] = tsd;
	x[1] = tsn;
	/* set x[2] before calling Day_Nit_Lst */
	x[3] = wda;
	x[4] = wna;
	x[5] = tadd;
	x[6] = tann;

	if ( x[NC+4] < 0.001 ) {
	   for ( ib = 0; ib < 6; ib++ )
	      x[NC+ib] = em_est0[i_land_catalog][ib]; /* use the mean value */
	}
	x[NC+6] = em_est0[i_land_catalog][6];

		chisq_sml = 1.0e10;

		for (ipt = 0; ipt < NC2; ipt++) {
		    if (ipt < NC) {
			y[ipt] = rtbd[ipt] / rad_ref_d[ipt];
			sig[ipt] = y[ipt] * exn[ipt] *
					dt_neq[ipt] / tbd[ipt];
		    }
		    else {
			y[ipt] = rtbn[ipt-NC] / rad_ref_n[ipt-NC];
			sig[ipt] = y[ipt] * exn[ipt] *
					dt_neq[ipt-NC] / tbn[ipt-NC];
		    }
		}
		
		tmp = sig[NC];

		if ( tmp > 0. ) {
			for (ipt = 0; ipt < NC; ipt++) {
			    sig[ipt] /= tmp;
			    sig[ipt+NC] /= tmp;
			    sig[ipt] = 1. / (sig[ipt] * sig[ipt]);
			    sig[ipt+NC] = 1. / (sig[ipt+NC] * sig[ipt+NC]);
			}
		}
		y[14] = y[15] = 0.0;
		sig[14] = 1.5;
		sig[15] = 0.7;

			/* make some adjustments */

			for (ipt = 0; ipt < NC2; ipt++) {

			    if (ipt == 0)	sig[ipt] *= 6.0;
			    if (ipt == 1)	sig[ipt] *= 6.0;
			    if (ipt == 2)	sig[ipt] *= 8.0;
			    if (ipt == 3)	sig[ipt] *= 4.0;
			    if (ipt == 4)	sig[ipt] *= 8.0;
			    if (ipt == 5)	sig[ipt] *= 8.0;

			    if (ipt == 7)	sig[ipt] *= 3.0;
			    if (ipt == 8)	sig[ipt] *= 1.8;
			    if (ipt == 9)	sig[ipt] *= 4.0;
			    if (ipt == 10)	sig[ipt] *= 4.0;
			    if (ipt == 11)	sig[ipt] *= 4.0;
			    if (ipt == 12)	sig[ipt] *= 4.0;
			}

		/* set A's boundaries */

		amin[0] = (x[0] + d_n_dt_d_min) / t_ref_d;
		amax[0] = (x[0] + 15.0 + d_n_dt_d_max) / t_ref_d;

		amin[1] = (x[1] + d_n_dt_n_min) / t_ref_n;
		amax[1] = (x[1] + d_n_dt_n_max) / t_ref_n;

		/* set amin[2] and amax[2] before calling Day_Nit_Lst */

		amin[3] = wda_min / wv_ref;
		amax[3] = wda_max / wv_ref;

		amin[4] = wna_min / wv_ref;
		amax[4] = wna_max / wv_ref;

		amin[5] = (x[5] - 20.0) / t_ref_d;  /* tadd may be not Tair */
		amax[5] = (x[5] + 10.0) / t_ref_d;

		amin[6] = (x[6] - 7.0) / t_ref_n;
		amax[6] = (x[6] + 7.0) / t_ref_n;

		for (ic = NC; ic < NC2; ic++) {
			if (x[ic] > em_max)	x[ic] = em_max;
			amin[ic] = em_min[i_land_catalog][ic-NC];
			if ( i_land_catalog == 3 && i_vzd_pix < 7 &&
			     i_vzn_pix < 7 )	amin[ic] = em_min[5][ic-NC];
			amax[ic] = em_max;
		}

		for (ib = 0; ib < MA; ib++)
			a[ib] = x[ib];

		a[0] /= t_ref_d;
		a[1] /= t_ref_n;
		a[2] /= brdf_ref;
		a[3] /= wv_ref;
		a[4] /= wv_ref;
		a[5] /= t_ref_d;
		a[6] /= t_ref_n;

		ixx = ixy = 0;
		nsv = 0;
		alamda = -1.0;
		for ( ib = 0; ib < MA; ib++ )
			isv[ib] = 0;

	for (nout = 0; nout < 2; nout++) {

		for (ib = 0; ib < MA; ib++) {
			if (a[ib] < amin[ib] || a[ib] > amax[ib]) {
			   tmp = sin (a[ib]);
			   a[ib] = amin[ib] + (amax[ib] - amin[ib]) * tmp * tmp;
			   alamda = -1.0;
			}
		}

		chisq = mrqmin (&alamda);

		iyy = 0;

		for (izz = 0; izz < 3; izz++) {

		    ochisq_main = chisq;
		    ixx += 1;
		    chisq = mrqmin (&alamda);
		    if ((chisq - ochisq_main) > eps_m)	iyy = 0;
		    else {
			tmp = fabs(ochisq_main - chisq) / chisq;
			if (tmp < 5.e-2)	iyy += 1;
		    }

		    iout = 1;
		    i_save_all = 1;

		    for (ib = 0; ib < MA; ib++) {
			if (a[ib] < amin[ib] || a[ib] > amax[ib]) {
			   if ( ib == 2 ) {
				if( a[ib] < amin[ib] ) n_out_range_brdf[0] += 1;
				if( a[ib] > amax[ib] ) n_out_range_brdf[1] += 1;
			   }
			   if ( ib == 3 ) {
				if( a[ib] < amin[ib] ) n_out_range_wvd[0] += 1;
				if( a[ib] > amax[ib] ) n_out_range_wvd[1] += 1;
			   }
			   if ( ib == 4 ) {
				if( a[ib] < amin[ib] ) n_out_range_wvn[0] += 1;
				if( a[ib] > amax[ib] ) n_out_range_wvn[1] += 1;
			   }
			   if ( ib == 5 ) {
				if( a[ib] < amin[ib] ) n_out_range_tad[0] += 1;
				if( a[ib] > amax[ib] ) n_out_range_tad[1] += 1;
			   }
			   if ( ib == 6 ) {
				if( a[ib] < amin[ib] ) n_out_range_tan[0] += 1;
				if( a[ib] > amax[ib] ) n_out_range_tan[1] += 1;
			   }
			   tmp = sin (a[ib]);
			   a[ib] = amin[ib] + (amax[ib]-amin[ib]) * tmp * tmp;
			   ixy += 1;
			   i_save_all = 0;
			}
			else {
				isv[ib] = 1;
				a_bk[ib] = a[ib];
			}
		    }

		    if ( i_save_all ) {

			nsv = 1;

			if (chisq < chisq_sml) {
			    chisq_sml = chisq;
				for (ib = 0; ib < MA; ib++)
					a_sv[ib] = a[ib];
			}
		    }
		    else	iout = 0;

		    if (iyy >= 2)	break;
		}
		if (iout == 0 || ixy >= 2)	break;
	}

	i_day_nit_lst_status_d = 0;
	i_day_nit_lst_status_n = 0;
	i_out_range_em = 0;

	if (nsv) {

	    for (ib = 0; ib < MA; ib++)
		a[ib] = a_sv[ib];
	}
	else {
	    for (ib = 0; ib < MA; ib++) {
		if ( isv[ib] )
			a[ib] = a_bk[ib];
		else {
			if ( ib == 0 )	i_day_nit_lst_status_d = 1;
			if ( ib == 1 )	i_day_nit_lst_status_n = 1;
			if ( ib >= NC && ib < NC2 )	i_out_range_em = 1;

			n_out_range_case[ib] += 1;

			a[ib] = x[ib];
			switch (ib) {
				case 0: a[0] /= t_ref_d;	break;
				case 1: a[1] /= t_ref_n;	break;
				case 2: a[2] /= brdf_ref;	break;
				case 3: a[3] /= wv_ref;		break;
				case 4: a[4] /= wv_ref;		break;
				case 5: a[5] /= t_ref_d;	break;
				case 6: a[6] /= t_ref_n;	break;
				default: break;
			}
		}
	    }
	    chisq = chisq_sml;
	}

	return (chisq);
}

float64 bt_to_radiance (int32 ic, float64 tb, int32 i_mod)
{
/*
!C*****************************************************************************

!Description:
   This function gives MODIS band radiance at given brightness temperature
   based on the MODIS PFM relative spectral response, version C.

!Input Parameters:	ic - index (0-6) for MODIS bands 20, 22, 23, 29, 31-33
			tb - band brightness temperature (K).
			i_mod - 1 for Aqua FM1 MODIS data

!Output Parameters:	none

 Returns:

	rr - band radiance in W/m^2/mu/sr.

!Revision History:

!References and Credits:

   Version v2.1 2002/10/12 22:15    Z. Wan

   Version v1.2 2001/04/21 21:29:30 Z. Wan
    also renamed from planck()

   Version v1.1 2000/05/15 02:02:41 Z. Wan

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	ii, ik;
	double	t2, rr;

	t2 = (tb - TMIN) / DT_LUT;

	ii = t2;
	if (ii < 0)		ii = 0;
	if (ii > NCELM2)	ii = NCELM2;
	ik = ii + 1;

	if ( i_mod )
	  rr = _mod_fm1_rad7[ic][ii] +
		(_mod_fm1_rad7[ic][ik] - _mod_fm1_rad7[ic][ii]) * (t2 - ii);
	else
	  rr = _mod_rad7[ic][ii] +
		(_mod_rad7[ic][ik] - _mod_rad7[ic][ii]) * (t2 - ii);

	return (rr);
}

double mrqmin (double *alamda)
{
/*
!C*****************************************************************************

!Description:
   This function is Marquartdt's method for the least-squares estimation of
   nonlinear parameters.

!Input Parameters:	alamda - the address of a parameter.
		     
!Output Parameters:	a[14]   solution for Ts-day, Ts-night, alpha,
				cwv-day, cwv-night, Ta-day, Ta-night, and
				emissivity values in the above seven bands.

 Returns:

   chisq - the least squares of the differences in given radiances and
	      the calculated radiances based on the retrieved values.

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	j, k, l, m;
	int32	jm1, km1;
	double	chisq = 10.0;

	if (*alamda < 0.) {
		*alamda = 0.0008;
		chisq = mrqcof (a,alpha,beta);
		ochisq_min = chisq;
		for (j = 0; j < MA; j++)
			atry[j] = a[j];
	}
	j = 0;
	for (l = 0; l < MA; l++) {
		if (ia[l]) {
			j += 1;
			jm1 = j - 1;
			k = 0;
			for (m = 0; m < MA; m++) {
				if (ia[m]) {
					k += 1;
					km1 = k - 1;
					covar[jm1][km1] = alpha[jm1][km1];
				}
			}
			covar[jm1][jm1] = alpha[jm1][jm1] * (1.0 + *alamda);
			da[jm1] = beta[jm1];
		}
	}

	gaussj (mfit);

	if ( fabs(*alamda) < SMALL_VALUE ) {
		covsrt (MA);
		return (chisq);
	}
	j = 0;
	for (l = 0; l < MA; l++) {
		if (ia[l]) {
			j += 1;
			jm1 = j - 1;
			atry[l] = a[l] + da[jm1];
		}
	}
	for (l = NC; l < MA; l++)
		if (atry[l] > 1.0)	atry[l] = 1.0;

	chisq = mrqcof (atry,covar,da);

	if (chisq <= ochisq_min) {
		*alamda *= 0.1;
		ochisq_min = chisq;
		j = 0;
		for (l = 0; l < MA; l++) {
			if (ia[l]) {
				j += 1;
				jm1 = j - 1;
				k = 0;
				for (m = 0; m < MA; m++) {
					if (ia[m]) {
					   k += 1;
					   km1 = k - 1;
					   alpha[jm1][km1] = covar[jm1][km1];
					}
				}
				beta[jm1] = da[jm1];
				a[l] = atry[l];
			}
		}
	}
	else {
		*alamda *= 10.0;
		chisq = ochisq_min;
	}
	return (chisq);
}

double mrqcof (double *ab, double b[][MA], double *c)
{
/*
!C*****************************************************************************

!Description:
   This function is the subroutine called by mrqmin()

!Input Parameters:	ab - address of the initial values of parameters.
		     
!Output Parameters:	b - the matrix in the least-squares fitting.
			*c - derivatives of the fitting functions.

 Returns:

   chisq - least-squares

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	ma = MA, ndata = NPT;
	int32	i, j, k, l, m, ipt;
	int32	jm1, km1;
	int32	iptpnc, iptmnc;
	double	dy, sig2i, wt;
	double	chisq;
	double	y1[NPT][MA], y2[NPT][MA], h[MA];
	double	tbdx, tbnx, rp1, rp2, tmp, rst;
	double	rw, taa;
	double	eps = 1.0e-3;

	ntot_mrqcof += 1;

	for (j = 0; j < mfit; j++) {
		for (k = 0; k <= j; k++)
			b[j][k] = 0.0;
		c[j] = 0.0;
	}

	chisq = 0.0;

	for (j = 0; j < ma; j++) {
		if (ia[j] == 0)	continue;
		h[j] = eps * ab[j];
	}
	
    for (j = 0; j < ma; j++) {

	tmp = ab[j];

	if (ia[j] == 0)	continue;

	for (k = 0; k < 2; k++) {

	    if (k == 0)	ab[j] = tmp + h[j];
	    else	ab[j] = tmp - h[j];

	    for (i = 0; i < ndata; i++) {

		ipt = ix1[i];

		if (ipt < NC) {

		    if ( fabs(a_sv3 - ab[3]) > SMALL_VALUE ||
		         fabs(a_sv5 - ab[5]) > SMALL_VALUE ) {
			a_sv3 = ab[3];
			a_sv5 = ab[5];
			rw = ab[3] * wv_ref;
			taa = ab[5] * t_ref_d;

			interp_atm_d (taa, rw);
		    }
		    iptpnc = ipt + NC;
		    tbdx = ab[0] * t_ref_d;
		    rp1 = bt_to_radiance (ipt, tbdx, i_aqua_d);
		    if (ipt < 3)
			rst = (ab[iptpnc] * rp1 * pad[4][ipt]
			    + pad[0][ipt] + pad[1][ipt]
			    + (1.0 - ab[iptpnc]) * (pad[2][ipt] +
			       brdf_ref * ab[2] * pad[3][ipt])) / rad_ref_d[ipt];
		    else
			rst = (ab[iptpnc] * rp1 * pad[4][ipt]
			    + pad[0][ipt]
			    + (1.0 - ab[iptpnc]) * pad[2][ipt]) / rad_ref_d[ipt];
		}
		else if ( ipt < NC2 ) {

		    if ( fabs (a_sv4 - ab[4]) > SMALL_VALUE ||
			 fabs (a_sv6 - ab[6]) > SMALL_VALUE ) {
			a_sv4 = ab[4];
			a_sv6 = ab[6];
			rw = ab[4] * wv_ref;
			taa = ab[6] * t_ref_n;

			interp_atm_n (taa, rw);
		    }
		    iptmnc = ipt - NC;
		    tbnx = ab[1] * t_ref_n;
		    rp2 = bt_to_radiance (iptmnc, tbnx, i_aqua_n);
		    rst = (ab[ipt] * rp2 * pan[2][iptmnc]
			    + pan[0][iptmnc]
			    + (1.0 - ab[ipt])*pan[1][iptmnc])/rad_ref_n[iptmnc];
		}
		else {
		    if ( ipt == 14 )
		     rst = (split_window_lst(14, &ab[0]) - ab[0]*t_ref_d)/dt_ref;
		    else
		     rst = (split_window_lst(15, &ab[0]) - ab[1]*t_ref_n)/dt_ref;
		}

		if (k == 0)	y1[i][j] = rst;
		else		y2[i][j] = rst;
	    }
	}
	ab[j] = tmp;
    }


    for (i = 0; i < ndata; i++) {
	ymod[i] = 0.5 * (y1[i][0] + y2[i][0]);
	for (j = 0; j < ma; j++) {
		if (ia[j] == 0)	continue;
		dyda[i][j] = (y1[i][j] - y2[i][j]) / (2.0*h[j]);
	}
    }

	for (i = 0; i < ndata; i++) {

	    sig2i = 1.0;
	    if (sig[i] > 0.0)	sig2i *= sig[i];
	    dy = y[i] - ymod[i];
	    j = 0;
	    for (l = 0; l < ma; l++) {
		if (ia[l]) {
			j += 1;
			jm1 = j - 1;
			wt = dyda[i][l] * sig2i;
			k = 0;
			for (m = 0; m <= l; m++) {
			    if (ia[m]) {
				k += 1;
				km1 = k - 1;
				b[jm1][km1] += wt * dyda[i][m];
			    }
			}
			c[jm1] += dy * wt;
		}
	    }
	    chisq += dy * dy * sig2i;
	}
	for (j = 1; j < mfit; j++) {
		for (k = 0; k <= j - 1; k++)
			b[k][j] = b[j][k];
	}

	return (chisq);
}

void	covsrt (int32 n) 
{
/*
!C*****************************************************************************

!Description:
   This function fills the curvature matrix

!Input Parameters:	n - dimension of the matrix
		     
!Output Parameters:	complete the curvature matrix covar (global variable)

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	i, j, k, l;
	double	swap;

	for (i = mfit; i < n; i++) {
		for (j = 0; j <= i; j++) {
			covar[i][j] = 0.;
			covar[j][i] = 0.;
		}
	}
	k = mfit - 1;
	for (j = n - 1; j >= 0; j--) {
	    if (ia[j]) {
		for (i = 0; i < n; i++) {
		    swap = covar[i][k];
		    covar[i][k] = covar[i][j];
		    covar[i][j] = swap;
		}
		for (i = 0; i < n; i++) {
		    swap = covar[k][i];
		    covar[k][i] = covar[j][i];
		    covar[j][i] = swap;
		}
		k -= 1;
	    }
	}

	return;
}

void gaussj (int32 n)
{
/*
!C*****************************************************************************

!Description:
   This function solves the curvature matrix equation for the parameter
   increments (da) by using the Gauss-Jordan method of elimination.

!Input Parameters:	n - dimension of the curvature matrix.
		     
!Output Parameters:	da - the parameter increments.

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	icol, irow, i, j, k, l, ll, m;
	int32	indxc[MA], indxr[MA], ipiv[MA];
	double	big, dum, pivinv, tmp;

	for (j = 0; j < n; j++)
		ipiv[j] = 0;

	for (i = 0; i < n; i++) {
	    big = 0.0;
	    for (j = 0; j < n; j++) {
		if (ipiv[j] != 1) {
		    for (k = 0; k < n; k++) {
			if (ipiv[k] == 0) {
			   tmp = fabs (covar[j][k]);
			   if (tmp >= big) {
				big = tmp;
				irow = j;
				icol = k;
			   }
			}
		    }
		}
	    }
	    ipiv[icol] += 1;
	    if (irow != icol) {
		for (l = 0; l < n; l++) {
		    dum = covar[irow][l];
		    covar[irow][l] = covar[icol][l];
		    covar[icol][l] = dum;
		}
		dum = da[irow];
		da[irow] = da[icol];
		da[icol] = dum;
	    }
	    indxr[i] = irow;
	    indxc[i] = icol;
	    if ( fabs (covar[icol][icol]) >= SMALL_VALUE )
	    	pivinv = 1. / covar[icol][icol];
	    else
	    	return;
	    covar[icol][icol] = 1.0;
	    for (l = 0; l < n; l++)
		covar[icol][l] *= pivinv;
	    da[icol] *= pivinv;
	    for (ll = 0; ll < n; ll++) {
		if (ll != icol) {
		   dum = covar[ll][icol];
		   covar[ll][icol] = 0.0;
		   for (l = 0; l < n; l++)
			covar[ll][l] -= covar[icol][l] * dum;
		   da[ll] -= da[icol] * dum;
		}
	    }
	}
	for (l = n - 1; l >= 0; l--) {
	    if (indxr[l] != indxc[l]) {
		for (k = 0; k < n; k++) {
		    dum = covar[k][indxr[l]];
		    covar[k][indxr[l]] = covar[k][indxc[l]];
		    covar[k][indxc[l]] = dum;
		}
	    }
	}

	return;
}

int32	locate (double *xx, int32 nn, double x)
{
/*
!C*****************************************************************************

!Description:
   This function gives the nearest location of x in vector xx.

!Input Parameters:	*xx - the vector of
			nn  - dimension
		     
!Output Parameters:	none

 Returns:

   jl - the nearest location

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	nnm1, iorder;
	int32	jl, jm, ju, k;

	nnm1 = nn - 1;
	iorder = (xx[nnm1] > xx[0]) ?  1 : 0;

	jl = 0;
	ju = nnm1;

	if (iorder) {
		while ((ju - jl) > 1) {

			jm = (ju + jl) / 2;

			if (x < xx[jm])
				ju = jm;
			else
				jl = jm;
		}
	}
	else {
		while ((ju - jl) > 1) {

			jm = (ju + jl) / 2;

			if (x > xx[jm])
				ju = jm;
			else
				jl = jm;
		}
	}

	return (jl);
}

void interp_atm_d (double taa, double rw) 
{
/*
!C*****************************************************************************

!Description:
   This function gives the daytime atmospheric and solar terms at given
   air surface temperature and column water vapor conditions.

!Input Parameters:	taa - air surface temperature
			rw  - column water vapor
		     
!Output Parameters:	pad[][] - atmospheric and solar terms

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	i, j, iwd, itad, iwd1, itad1;
	int32	nrhum1, ntsam1;
	double	tmp, tmp1, tmp2, dya;

	nrhum1 = nrhu - 1;
	ntsam1 = ntsa - 1;

	iwd = locate (wd, NRHM, rw);
	itad = locate (tadit, NTSA, taa);

	if (iwd < 0)		iwd = 0;
	if (iwd >= nrhum1)	iwd = nrhum1 - 1;
	if (itad < 0)		itad = 0;
	if (itad >= ntsam1)	itad = ntsam1 - 1;

	itad1 = itad + 1;
	iwd1 = iwd + 1;

	tmp1 = (taa - tadit[itad]) / (tadit[itad1] - tadit[itad]);
	tmp2 = (rw - wd[iwd]) / (wd[iwd1] - wd[iwd]);

    for (i = 0; i < 3; i++) {

	 dya = rupt[itad][iwd1][i] - rupt[itad][iwd][i];
	 pad[0][i] = (tmp2 * (rupt[itad1][iwd1][i] - rupt[itad1][iwd][i] - dya)
		    + rupt[itad1][iwd][i] - rupt[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rupt[itad][iwd][i];

	 dya = rups[itad][iwd1][i] - rups[itad][iwd][i];
	 pad[1][i] = (tmp2 * (rups[itad1][iwd1][i] - rups[itad1][iwd][i] - dya)
		    + rups[itad1][iwd][i] - rups[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rups[itad][iwd][i];

	 dya = rgft[itad][iwd1][i] - rgft[itad][iwd][i];
	 tmp = (tmp2 * (rgft[itad1][iwd1][i] - rgft[itad1][iwd][i] - dya)
		    + rgft[itad1][iwd][i] - rgft[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rgft[itad][iwd][i];

	 dya = rgfd[itad][iwd1][i] - rgfd[itad][iwd][i];
	 pad[2][i] = (tmp2 * (rgfd[itad1][iwd1][i] - rgfd[itad1][iwd][i] - dya)
		    + rgfd[itad1][iwd][i] - rgfd[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rgfd[itad][iwd][i];

	 pad[2][i] += tmp;	/* adding the therm term */

	 dya = rds[itad][iwd1][i] - rds[itad][iwd][i];
	 pad[3][i] = (tmp2 * (rds[itad1][iwd1][i] - rds[itad1][iwd][i] - dya)
		    + rds[itad1][iwd][i] - rds[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rds[itad][iwd][i];

	 dya = transd[itad][iwd1][i] - transd[itad][iwd][i];
	 pad[4][i] = (tmp2 * (transd[itad1][iwd1][i]-transd[itad1][iwd][i]-dya)
		    + transd[itad1][iwd][i] - transd[itad][iwd][i]) * tmp1
		    + tmp2 * dya + transd[itad][iwd][i];

    }

    for (i = 3; i < NC; i++) {

	 dya = rupt[itad][iwd1][i] - rupt[itad][iwd][i];
	 pad[0][i] = (tmp2 * (rupt[itad1][iwd1][i] - rupt[itad1][iwd][i] - dya)
		    + rupt[itad1][iwd][i] - rupt[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rupt[itad][iwd][i];

	 pad[1][i] = 0.0;

	 dya = rgft[itad][iwd1][i] - rgft[itad][iwd][i];
	 pad[2][i] = (tmp2 * (rgft[itad1][iwd1][i] - rgft[itad1][iwd][i] - dya)
		    + rgft[itad1][iwd][i] - rgft[itad][iwd][i]) * tmp1
		    + tmp2 * dya + rgft[itad][iwd][i];

	 pad[3][i] = 0.0;

	 dya = transd[itad][iwd1][i] - transd[itad][iwd][i];
	 pad[4][i] = (tmp2 * (transd[itad1][iwd1][i]-transd[itad1][iwd][i]-dya)
		    + transd[itad1][iwd][i] - transd[itad][iwd][i]) * tmp1
		    + tmp2 * dya + transd[itad][iwd][i];

    }

    return;

}

void interp_atm_n (double taa, double rw)
{
/*
!C*****************************************************************************

!Description:
   This function gives the nighttime atmospheric and solar terms at given
   air surface temperature and column water vapor conditions.

!Input Parameters:	taa - air surface temperature
			rw  - column water vapor
		     
!Output Parameters:	pan[][] - atmospheric terms

!Revision History:

!References and Credits:

   Version v1.1 09-15-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
        This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***************************************************************************
*/

	int32	i, j, iwn, itan, iwn1, itan1;
	int32	nrhum1, ntsam1;
	double	tmp1, tmp2, dya;

	nrhum1 = nrhu - 1;
	ntsam1 = ntsa - 1;

	iwn = locate (wn, NRHM, rw);
	itan = locate (tanit, NTSA, taa);

	if (iwn < 0)		iwn = 0;
	if (iwn >= nrhum1)	iwn = nrhum1 - 1;
	if (itan < 0)		itan = 0;
	if (itan >= ntsam1)	itan = ntsam1 - 1;

	itan1 = itan + 1;
	iwn1 = iwn + 1;

	tmp1 = (taa - tanit[itan]) / (tanit[itan1] - tanit[itan]);
	tmp2 = (rw - wn[iwn]) / (wn[iwn1] - wn[iwn]);

    for (i = 0; i < NC; i++) {

	 dya = rupn[itan][iwn1][i] - rupn[itan][iwn][i];
	 pan[0][i] = (tmp2 * (rupn[itan1][iwn1][i] - rupn[itan1][iwn][i] - dya)
		    + rupn[itan1][iwn][i] - rupn[itan][iwn][i]) * tmp1
		    + tmp2 * dya + rupn[itan][iwn][i];

	 dya = rgfn[itan][iwn1][i] - rgfn[itan][iwn][i];
	 pan[1][i] = (tmp2 * (rgfn[itan1][iwn1][i] - rgfn[itan1][iwn][i] - dya)
		    + rgfn[itan1][iwn][i] - rgfn[itan][iwn][i]) * tmp1
		    + tmp2 * dya + rgfn[itan][iwn][i];

	 dya = transn[itan][iwn1][i] - transn[itan][iwn][i];
	 pan[2][i] = (tmp2 * (transn[itan1][iwn1][i]-transn[itan1][iwn][i]-dya)
		    + transn[itan1][iwn][i] - transn[itan][iwn][i]) * tmp1
		    + tmp2 * dya + transn[itan][iwn][i];

    }

    return; 
}

/*
!C*****************************************************************************

!Description:

This function, SMFLOG, is called by the DoMetaData function.  It replaces 
a macro named SMFLOG, the behavior of which depends on syntactical subtleties
that seem to be compiler-dependent.

!Input Parameters:  Message string

!Output Parameters: None		

!Revision History:

	Modified by Z. Wan for v2.1.10 on 23 September 1999.

	Created by Richard P. Cember, MODIS SDST, from macro SMFLOG in
	original V1 code delivery. 3 January 1997.

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
        
!Design Notes:
  
!END***************************************************************************
*/
void SMFLOG(char *a)
{

  char		systemTag[PGSd_SMF_TAG_LENGTH_MAX] = "MOD11B1 Version 5" ;
  char		LogMsg[1024] = { NULL };
  char		a1[128] = { NULL };

  strcpy (&a1[0], a);
  a1[18] = '\0';

  sprintf(systemTag, "MOD11B1 Version %d", vers_id_num);
  sprintf(LogMsg,"%s: %s", systemTag, a);
  PGS_SMF_GenerateStatusReport(LogMsg);
    if ( strcmp(a1, "Completed MOD11B1 ") != 0)
  /*These next lines were added by C. Collins on 6/8/01 so comparison to Aqua*/
  /*output name or processing an ocean granule does not cause an exit value of 1 */
        {
        if ( strcmp(a1, "Completed MYD11B1 ") != 0)
             {
             if ( strcmp(a1, "Completed MOD_PR11") != 0) 
                exit(1);
             else exit(0);
             }
        else exit(0);
        } 
   else  exit(0);
}

/*
!C*****************************************************************************

!Description:

This function, DoMetaData, is called by the main function for writing L2
Metadata, and by function Granule_tile for writing 1km & 6km L3 MetaData.
It is responsible for writing all "core" and "archive" meta data on the
L2 and 1km & 6km L3 LST HDF files.
The meta data that it processes comes from the L1A and/or L1B input files, 
the MOD12 Landcover input files, the PCF file, the MCF file, and some 
generated internally by this function itself.

This void function has no return code.  The program aborts if there are errors.

!Input Parameters:

        Int32 id_old            id_old=1 for the existed output file
        Int32 Level             Output product Level
        PCF Logical number      Output file logical number in PCF file
	PGSt_integer            Number of tiles involved in this run
	PGSt_integer            The tile number for L3 files
	MODFILE         	Handle of the output file

!Output Parameters:		None

!Revision History:

  $Log: DoMetaData() $
 * Revision 2.2.3 2000/03/07  21:00     Z. Wan
 *	comment out the change made on 1999/12/29 so that MCF files will
 *	not modified during run time.
 * Revision 2.2.1 1999/12/29  11:04     Z. Wan
 *	to set a true value to INPUTPOINTER's NUM_VAL
 * Revision 2.03	 1998/10-05   ff       Z. Wan
 * add MOD11UPD file
 * Revision 2.02	 1998/01-03   ff       R. Cember
 * also modified MCF files
 * Revision 2.01	 1997/11/20  23:55     Z. Wan
 * working with two seperate MCF files
 *
 * DoMetaData()
 * Revision 2.00	 1997/09/30  23:55     Z. Wan
 * for L2, 1km & 6km L3 output files

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:
   Originally developed by 
        Gang Ye,
	MODIS/SDST, SAIC/GSC
	301-352-2109
	gang.ye@gsfc.nasa.gov

   Major modifications made by
	Richard Cember
	MODIS/SDST, SAIC/GSC
	301-352-2148
	rcember@ltpmail.gsfc.nasa.gov

!Design Notes:

  Richard Cember made the following modifications:
  1. used a separate MCF for each output file.
  2. checked metadata in each MCF file.
  3. made N_BYTES_CORE_L3 and N_BYTES_ARCH_L3 big enough.
  4. all calls to PGS_PC_GetReference() have been changed to calls to
     PGS_PC_GetUniversalRef().

!END***************************************************************************
*/

#define MAX_CODE_STG_VALS	600

char	         *FloatStg = "SIZEMBECSDATAGRANULE";
PGSt_double      FloatVal;

char	         *DoubleStg1[] = {  "NORTHBOUNDINGCOORDINATE", 
				    "SOUTHBOUNDINGCOORDINATE",
				    "EASTBOUNDINGCOORDINATE",  
				    "WESTBOUNDINGCOORDINATE" };

char	CodeStgVals[MAX_CODE_STG_VALS][PGSd_PC_VALUE_LENGTH_MAX]; 

char	*MoreCodeStg[4] = {
			   "SHORTNAME",
		 	   "HORIZONTALTILENUMBER",
			   "VERTICALTILENUMBER",
			   "GRIDTYPE" };

char	MoreCodeStgVals[4][PGSd_PC_VALUE_LENGTH_MAX];

char	*DoubleStg2[] = {
	 "GRINGPOINTLATITUDE.1",
	 "GRINGPOINTLONGITUDE.1",
	 "EQUATORCROSSINGLONGITUDE.1" };
/* R. Cember: in the current ECS metadata model, EquatorCrossingLongitude is
   a float, not a double, but I see no reason to change the code, since the
   conversion should be OK except for a loss of unnecessary precision. */

char	*IntegerStg2[] = {
	 "GRINGPOINTSEQUENCENO.1",
	 "ORBITNUMBER.1" };

void DoMetaData( int32 id_old, int32 level, PGSt_PC_Logical PCLOG,
	PGSt_integer tiles, PGSt_integer i_tile, MODFILE *outfile)
{

  PGSt_integer		j, version;
  char	*local2;
  char	s_tmp[128] = {NULL};
  char	time2[128] = {NULL};
  char	timeb[128] = {NULL};
  char	LocalGranuleID[128] = {NULL};
  char	augmented_string[MAX_NC_NAME] = {NULL};
  char	str_num[32] = {NULL};
  char  CharSt[256] = {NULL};
  char  GranuleNumber[32] = {NULL};
  int32 i, ii = 0, jj = 2;
  int32	n_gran_pnts_old = 0, n_gran_pnts = 0;
  int32 n_val, sd_id;
  PGSt_integer	i4[4];
  PGSt_double   d4[4];
  PGSt_double	DoubleBounds[4] = {2.F};

  /* R. Cember: for LocalGranuleID proc date & time upgrade. */
  char proc_year[PGSd_PC_VALUE_LENGTH_MAX], proc_day[PGSd_PC_VALUE_LENGTH_MAX];
  char proc_hour[PGSd_PC_VALUE_LENGTH_MAX], proc_minute[PGSd_PC_VALUE_LENGTH_MAX];  
  char proc_second[PGSd_PC_VALUE_LENGTH_MAX];
  char pdt_string[PGSd_PC_VALUE_LENGTH_MAX], *pdt_string_ptr = pdt_string;

/* R. Cember: for InputPointer metadatum upgrade. */
#define MAX_INPUT_PTRS 2000
  char InputPointer[MAX_INPUT_PTRS][PGSd_PC_VALUE_LENGTH_MAX];
  char *InputPointerPtr[MAX_INPUT_PTRS];
  int32 NextInputPointer = 0;
  int32 accum_input_ptrs (char *, const char *); 
/* Z. Wan comments the following line out
  char    mcf_name[128], mcf_kp_name[128], shcmd[256];	*/
  
  /* get core and archiv metadata from the existing L3 output file */

  if ( level == 3 && id_old && PCLOG != PCLogupdate[i_days_in_cycle_update-1]) {

    if ( PCLOG == PCLogL3_6km ) {

      sprintf(sd_attr_name, ARCHIVES_IN);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_ARCH_L3;
      if(getMODISfileinfo(lst_6km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		arch_L3) != MAPIOK)
	Error("Re: lst_6km_l3_outf: failure on call to ", "getMODISfileinfo");

      sprintf(sd_attr_name, CORES);
      sprintf(sd_attr_type, "char *");
      nelements = (long) N_BYTES_CORE_L3;
      if(getMODISfileinfo(lst_6km_l3_outf,sd_attr_name,sd_attr_type,&nelements,
		core_L3) != MAPIOK)
	 Error("Re: lst_6km_l3_outf: failure on call to ", "getMODISfileinfo");
    }
    date_begin[1][0] = '\0';
    local2 = date_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[0], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_begin from core_L3");
    time_begin[1][0] = '\0';
    local2 = time_begin[1];
    if ( get_meta_attr(core_L3, CharStg2[1], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_begin from core_L3");
    date_end[1][0] = '\0';
    local2 = date_end[1];
    if ( get_meta_attr(core_L3, CharStg2[2], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get date_end from core_L3");
    time_end[1][0] = '\0';
    local2 = time_end[1];
    if ( get_meta_attr(core_L3, CharStg2[3], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails to get time_end from core_L3");

  /* get INPUTGRANULEPOINTERs from the existing output file */

    local = s_tmp;
    if (get_meta_attr(core_L3,"PARAMETERVALUE.5", 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_L3");
    sscanf (local, "%d", &n_gran_pnts_old);

    local = ch_tileid;
    if (get_meta_attr(core_L3,"PARAMETERVALUE.8", 1, 1, (char *) local) == FAIL)
	HALT("get_meta_attr fails on core_L3");

    version = tile_pntr[i_tile].s_num + 1;

    for (j = 1; j <= n_gran_pnts_old; j++) {

	local = CodeStgVals[jj];
	CodeStgVals[jj][0] = '\0';

	strcpy(augmented_string, "INPUTGRANULEPOINTER");  
	sprintf(str_num, ".%d", j);
	strcat(augmented_string, str_num);

	/* R. Cember: move InputGranulePointer from core to archive */
	if (get_meta_attr(arch_L3,augmented_string,1, 1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3"); 
 
	jj++;

    }
  }

  n_gran_pnts = jj - 2;

  if ( PGS_MET_Init(MCF_FILE_L3_6KM, mdhandles) != PGS_S_SUCCESS )
	HALT("Failure in PGS_MET_Init");

  /*** get VersionID from MCF file**04/06/01**/

  if(PGS_MET_GetSetAttr(mdhandles[1],"VersionID",&vers_id_num) != PGS_S_SUCCESS)
	SMFLOG("Failure in PGS_MET_GetSetAttr for VersionID");

  met_cnt = &met_cnt_6km;

  /* Process Char meta data. */
  for ( j = 0; j < 9; j++ ) {

    /* R. Cember: there is no more OperationMode metadatum, which is j=7, the
    way this code is set.  Skip this case. */
    if (j==7) continue;

    /* Comment added by R. Cember: This next "if" bypasses the 
    EquatorCrossingDate and EquatorCrossingTime metadata for the L3 products */
    if ( level == 3 && j > 4 && j < 7 ) continue;

    CharVals[j][0] = '\0';
    local2 = CharVals[j];

    if ( get_meta_attr(core_L3, CharStg2[j], 1, 1, (char *) local2) == FAIL)
	HALT("get_meta_attr fails on core_L3");

    if ( j == 8 )
	strcpy ( &local2[0], pgeversion );
    local = local2; 
 
     if(PGS_MET_SetAttr(mdhandles[1], CharStg2[j], &local ) != PGS_S_SUCCESS ) 
      HALT("FAILURE putting a CORE string value");
   }

   /* Process DOUBLE meta data */

   for ( j = 0; j < 3; j++ ) {

    if ( j == 2 && level == 3)	break;

    n_val = 4;

    if(get_meta_attr(core_L3,DoubleStg2[j],3,n_val,(char *) &d4[0])==FAIL)
		HALT("get_meta_attr fails on core_L3");

    if(PGS_MET_SetAttr(mdhandles[1],DoubleStg2[j], d4) != PGS_S_SUCCESS )
      HALT("FAILURE putting d4");
   }

   /* Process INTEGER meta data */
   for ( j = 0; j < 2; j++ ) {

    if ( level == 3 && j == 1 )	break;
    n_val = ( j ) ? 1 : 4;

    /* set GRINGPOINTSEQUENCENO */
    i4[0] = 1; i4[1] = 2; i4[2] = 3; i4[3] = 4;

    if(PGS_MET_SetAttr(mdhandles[1], IntegerStg2[j], &i4 ) != PGS_S_SUCCESS )
      HALT("FAILURE putting a CORE integer value");
   }

   /* R. Cember: moved ProcessingDateTime up here so we can use its info to
   construct the date/time suffix for LocalGranuleID. */
   {
    /* Construct ProcessingDateTime from UNIX time. */
    char	s[256], *p = s;
    time_t	clock;
    struct tm	*gm_p;

    clock = time(NULL);
    gm_p = gmtime( &clock );

    sprintf(pdt_string, "%i-%02i-%02iT%02i:%02i:%02i.000Z",
	    gm_p->tm_year+1900, gm_p->tm_mon+1, gm_p->tm_mday,
	    gm_p->tm_hour, gm_p->tm_min, gm_p->tm_sec );

    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingDateTime",&pdt_string_ptr) == FAIL )
      HALT("FAILURE putting ProcessingDateTime");

    strcpy( s, process_env );
    if(PGS_MET_SetAttr(mdhandles[2],"ProcessingEnvironment",&p ) == FAIL )
	HALT("FAILURE putting ProcessingEnvironment");

    /* R. Cember: for LocalGranuleID. */
    sprintf( proc_year, "%.4d", 1900+(gm_p->tm_year) );
    sprintf( proc_day, "%.3d", (gm_p->tm_yday)+1 );
    sprintf( proc_hour, "%.2d", gm_p->tm_hour );
    sprintf( proc_minute, "%.2d", gm_p->tm_min );
    sprintf( proc_second, "%.2d", gm_p->tm_sec );
  }

    /* Make the LocalGranuleID from RangeBeginningDateTime */
    sprintf( time2, "%sT%sZ", CharVals[0], CharVals[1] );

    if ( PGS_TD_ASCIItime_AtoB( time2, timeb ) != PGS_S_SUCCESS )
	HALT("PGS_TD_ASCIItime_AtoB failed");
    timeb[4] = timeb[8] = timeb[11] = timeb[14] = '\0';

    sprintf( LocalGranuleID, "%s.%s%s%s.h%.2dv%.2d.%03d.%s%s%s%s%s.hdf", 
	esdt_6km_L3, SATELLITE, timeb,
	&timeb[5], tile_pntr[i_tile].ih, tile_pntr[i_tile].iv, vers_id_num,
	proc_year, proc_day, proc_hour, proc_minute, proc_second);

  local = LocalGranuleID;
  if (PGS_MET_SetAttr(mdhandles[1],"LocalGranuleID", &local ) != PGS_S_SUCCESS)
	HALT("FAILURE putting LocalGranuleID in CORE");

  /* Process BoundingCoordinate double meta data */
  for ( j = 0; j < 4; j++ ) {

    if(get_meta_attr(arch_L3,DoubleStg1[j],3,1,(char *)&DoubleBounds[j])==FAIL)
	HALT("get_meta_attr failure getting a ARCHIVE double value");

    if(PGS_MET_SetAttr(mdhandles[2], DoubleStg1[j], 
		&DoubleBounds[j] ) != PGS_S_SUCCESS )
      HALT("FAILURE putting an GEO input ARCHIVE double value");
  }

    for (j = 2; j < jj; j++) {

      /* R. Cember: move InputGranulePointer to archive from core metadata */
      sprintf (&s_tmp[0], "INPUTGRANULEPOINTER.%d", j - 1); 
      rm_path(CodeStgVals[j], local);
      if(PGS_MET_SetAttr(mdhandles[2], s_tmp, &local) != PGS_S_SUCCESS)
	   SMFLOG("PGS_MET_SetAttr -- Error adding a code meta data");

      /* R. Cember: to collect all the input pointers, in this case the
         InputGranulePointers.*/
      NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], 
                CodeStgVals[j]);

    }  /* end for-loop on j */

{  /* Comment added by R. Cember: Begin local block. */

  char		s[256], *p = s, CharString[256], *p1;
  int32		x;
  void		*loc2 = CharString, *loc3 = GranuleNumber;
  char		s_name[64], s_val[64], s_msg[128];	/* added by Z. Wan */

  strcpy( s, rep_actual );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingActual",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingActual");

  strcpy( s, rep_planned );
  if(PGS_MET_SetAttr(mdhandles[1],"ReprocessingPlanned",&p ) == FAIL )
    HALT("FAILURE putting ReprocessingPlanned");

  if ( level == 2 )
	strcpy( s, DayNightFlag );
  else
	strcpy( s, BOTH );
  if(PGS_MET_SetAttr(mdhandles[1],"DayNightFlag",&p ) == FAIL )
    HALT("FAILURE putting DayNightFlag");

  strcpy( s, "SCF V5.2.2B" );
  if(PGS_MET_SetAttr(mdhandles[1],"LocalVersionID",&p ) == FAIL )
    HALT("FAILURE putting LocalVersionID");

  strcpy( s, "Land-cover based band emissivity database" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.1",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.1");
  strcpy( s, "band_emis.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.1",&p ) == FAIL )
    HALT("FAILURE putting band_emis.h in LookupTablePointer");

  strcpy( s, "the generalized split-window LST coefficients" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTableType.2",&p ) == FAIL )
    HALT("FAILURE putting LookupTableType.2");
  strcpy( s, "lst_coef.h" );
  if(PGS_MET_SetAttr(mdhandles[2],"LookupTablePointer.2",&p ) == FAIL )
    HALT("FAILURE putting lst_coef.h in LookupTablePointer");

/* R. Cember: End conversion of AncillaryInputType and AncillaryInputPointer
to LookupTableType and LookupTable Pointer. */

  /* Set up for placement in the metadata as a string array. */
  for ( i = 0; i < NextInputPointer; i++ )
	InputPointerPtr[i] = InputPointer[i];

  /* Close out the InputPointer list. */

  InputPointerPtr[NextInputPointer] = NULL;

  /* Reset the InputPointer count for next call to DoMetaData. */
  s[0] = '\0';
  NextInputPointer = accum_input_ptrs(InputPointer[NextInputPointer], s);

  /* Set the InputPointer metadatum. */
  if(PGS_MET_SetAttr(mdhandles[1],"InputPointer", &InputPointerPtr ) == FAIL )
    HALT("FAILURE setting InputPointer");

  strcpy( s, "MOD 6KM L3 LST" );
  if(PGS_MET_SetAttr(mdhandles[1],"ParameterName.1",&p ) == FAIL )
    HALT("FAILURE putting ParameterName");

  switch(AutomaticQualityFlag) {
  case PASSED:      strcpy( s, "Passed" );      break;
  case SUSPECT:     strcpy( s, "Suspect");      break;
  case FAILED:      strcpy( s, "Failed" );
  }
  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlag.1",&p) == FAIL)
    HALT("FAILURE putting AutomaticQualityFlag");

  strcpy(s,"Passed if algorithm ran within bounds of execution constraints.  ");
  strcat(s,"Suspect if bounds of execution constraints violated.  ");
  strcat(s,"Failed if PGE failed.");

  if(PGS_MET_SetAttr(mdhandles[1],"AutomaticQualityFlagExplanation.1",&p ) == FAIL )
    HALT("FAILURE putting AutomaticQualityFlagExplanation");

  strcpy( s, sci_qua_exp );
  if(PGS_MET_SetAttr(mdhandles[1],"ScienceQualityFlagExplanation.1",&p ) == FAIL)
    HALT("FAILURE putting ScienceQualityFlagExplanation");

  x = 0;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentInterpolatedData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentInterpolatedData");

  x =  met_cnt->QA_MissingData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],
		     "QAPercentMissingData.1", &x ) == FAIL )
    HALT("FAILURE putting QAPercentMissingData");

  x =  met_cnt->QA_OutOfBoundsData * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentOutOfBoundsData.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentOutOfBoundsData");

  x = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  if(PGS_MET_SetAttr(mdhandles[1],"QAPercentCloudCover.1",&x ) == FAIL )
    HALT("FAILURE putting QAPercentCloudCover");

/* Do the AdditionalAttributes.n. The ParamterValue.n must be a char string. */

  FloatVal = met_cnt->QA_GoodQuality * 100.0 / met_cnt->AllPixelsProcessed +0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTGOODQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.1", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.1");
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.1", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.1");

  FloatVal = met_cnt->QA_OtherQuality * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTOTHERQUALITY");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.2", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.2");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.2", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.2");

  FloatVal = met_cnt->QA_NotProducedCloud * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDCLOUD");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.3", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.3");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.3", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.3");

  FloatVal = met_cnt->QA_NotProducedOther * 100.0 / met_cnt->AllPixelsProcessed + 0.5;
  sprintf(CharString,"%d", (int32) FloatVal);
  strcpy (s, "QAPERCENTNOTPRODUCEDOTHER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.4", &p ) == FAIL )
    HALT("FAILURE putting AdditionalAttributeName.4");
  if(PGS_MET_SetAttr( mdhandles[1], "ParameterValue.4", &loc2 ) == FAIL )
    HALT("FAILURE putting ParameterValue.4");

  if ( level == 2 ) {
	strcpy (s, "GRANULENUMBER");
	local2 = loc3;
  }
  else {
	strcpy (s, "N_GRAN_POINTERS");
	sprintf (s_tmp, "%d", n_gran_pnts);
	local2 = s_tmp;
  }

  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.5", &p )
		== FAIL )
	HALT("FAILURE putting AdditionalAttributeName.5");

  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.5", &local2 ) == FAIL )
	HALT("FAILURE putting ParameterValue.5");

/* R. Cember: Re: DDTS 728.  Adding two more PSAs here. */
if (level ==3) {
  strcpy(p, "HORIZONTALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.6", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.6");
  sprintf(p, "%.2d", tile_pntr[i_tile].ih);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.6", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.6");

  strcpy(p, "VERTICALTILENUMBER");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.7", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.7");
  sprintf(p, "%.2d", tile_pntr[i_tile].iv);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.7", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.7");

/* C. Collins, 11/4/02, change TILEID to TileID */
  strcpy(p, "TileID");
  if(PGS_MET_SetAttr( mdhandles[1], "AdditionalAttributeName.8", &p ) == FAIL )
	HALT("FAILURE putting AdditionalAttributeName.8");
  /*sprintf(p, "11%.3d%.3d", tile_pntr[i_tile].ih, tile_pntr[i_tile].iv);*/
  sprintf(p, "%s", ch_tileid);
  if(PGS_MET_SetAttr( mdhandles[1],"ParameterValue.8", &p ) == FAIL )
	HALT("FAILURE putting ParameterValue.8");
}

  FloatVal = met_cnt->QA_GoodQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONGOODQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 6 : 9);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 6 : 9);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_OtherQuality * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONOTHERQUALITY");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 7 : 10);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 7 : 10);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedCloud * 1.0  / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDCLOUD");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 8 : 11);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 8 : 11);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  FloatVal = met_cnt->QA_NotProducedOther * 1.0 / met_cnt->AllPixelsProcessed;
  sprintf(CharString,"%9.7f", FloatVal);
  strcpy (s, "QAFRACTIONNOTPRODUCEDOTHER");
  sprintf(s_name, "AdditionalAttributeName.%d", (level == 2) ? 9 : 12);
  sprintf(s_val, "ParameterValue.%d", (level == 2) ? 9 : 12);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  sprintf(CharString, "%s", "NO");
  strcpy (s, "CLOUD_CONTAMINATED_LST_SCREENED");
  sprintf(s_name, "AdditionalAttributeName.%d", 13);
  sprintf(s_val, "ParameterValue.%d", 13);
  sprintf(s_msg, "FAILURE putting %s", s_name);
  if(PGS_MET_SetAttr( mdhandles[1], s_name, &p ) == FAIL )
    HALT(s_msg);
  sprintf(s_msg, "FAILURE putting %s", s_val);
  if(PGS_MET_SetAttr( mdhandles[1], s_val, &loc2 ) == FAIL )
    HALT(s_msg);

  strcpy( s, "102004" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageAcceptanceDate",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageAcceptanceDate");
  strcpy( s, "Normal" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageMaturityCode",&p ) == FAIL)
    HALT("FAILURE putting AlgorithmPackageMaturityCode");
  strcpy( s, "MOD_PR11B" );
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageName",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageName");
/*next lines edited by C. Collins on 11/11/04 so AlgorithmPackageVersion would
be string*/
  strcpy (s, "5");
/*  x = 5; - line commented out by ccollins on 11/11/04*/
/*x changed to p in next line by ccollins on 11/11/04*/
  if(PGS_MET_SetAttr(mdhandles[2],"AlgorithmPackageVersion",&p ) == FAIL )
    HALT("FAILURE putting AlgorithmPackageVersion");

  strcpy( s, "GSFC" );
  if(PGS_MET_SetAttr(mdhandles[2],"ProcessingCenter",&p ) == FAIL )
    HALT("FAILURE putting ProcessingCenter");

  strcpy( s, "2484 and 3323" );
  if(PGS_MET_SetAttr(mdhandles[2],"SPSOParameters",&p ) == FAIL )
    HALT("FAILURE putting SPSOParameters");

  local = MoreCodeStgVals[0];
  MoreCodeStgVals[0][0] = '\0';
  if (get_meta_attr(arch_L3,"LOCALINPUTGRANULEID",1,1,(char *) local) == FAIL)
		HALT("get_meta_attr fails on arch_L3");

  if(PGS_MET_SetAttr( mdhandles[2], "LocalInputGranuleID", &local ) == FAIL )
    HALT("FAILURE putting LocalInputGranuleID");

  /* set tile information */

  sprintf( MoreCodeStgVals[1], "%.2d", tile_pntr[i_tile].ih);
  sprintf( MoreCodeStgVals[2], "%.2d", tile_pntr[i_tile].iv);
  strcpy( MoreCodeStgVals[3], "INTEGERIZED SINUSOIDAL");

  for ( j = 1; j < 3; j++ ) {

    local = MoreCodeStgVals[j];

    if(PGS_MET_SetAttr(mdhandles[2], MoreCodeStg[j], &local)!=PGS_S_SUCCESS)
	SMFLOG("PGS_MET_SetAttr -- Error adding arch meta data");
  } 
}

 sd_id = outfile->sd_id;

 if(PGS_MET_Write(mdhandles[1], CORES, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing CORE");
/*
 if(PGS_MET_Write(mdhandles[2], ARCHIVES_OUT, sd_id) != PGS_S_SUCCESS)
	HALT("PGS_MET_Write -- Error writing ARCHIVES");
*/

 PGS_MET_Remove();

}

int32 accum_input_ptrs (char *InputPtrElem, const char *InputString)
/*
!C*****************************************************************************

!Description:

This is a service function for DoMetaData.  It accumulates the 
InputGranulePointers, InputtilePointers and LookupTablePointers into one
array so they can be conveniently put into the InputPointer metadatum.

!Input Parameters:
  char *InputPtrArray   
  char *InputPtrElement

!Output Parameters:  
  None

!Returns:
  The number of elements so far accumulated.

!Revision History:

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

   This function written by Richard P. Cember, SDST, 29 January 1998

!References and Credits:

!Design Notes:

This is basically a kludge to adapt PGE16/MOD_PR11 to ECS metadata requirements
which were not well known when the code was written.

It is expected that the last function call within each call to DoMetaData
will contain a NULL InputPtrElement per TK User's Guide p. 6-60.  The 
current number of elements, including the final null, is returned on each call.
When the final null is passed in, the PtrIndex is reset for the next time 
DoMetaData itself is called.

!END***************************************************************************
*/
{

  static int32	PtrIndex = 0;
  int32		NumPtrs;
  char		test_string[1] = {'\0'};

  strcpy(InputPtrElem, InputString);

  PtrIndex++;
  NumPtrs = PtrIndex; 

  /* Get set for the next pass through DoMetaData. */
  if ( strcmp(InputString, test_string) == 0) PtrIndex = 0;

  if ( NumPtrs == MAX_INPUT_PTRS ) {
    free_lst_space();
    SMFLOG("STOP--NumPtrs reachs the limit");
  }

  return (NumPtrs);

}

/*
!C*****************************************************************************

!Description:

This function, get_meta_attr, is used to get an attribute of metadata
which is stored in main memory.  It is more efficient than getting a
metadata attribute from the HDF file.

!Input Parameters:

        s			pointer to a metadata string
        p			pointer to the attribute name string
        type			type of the attribute (1 for char,
				2 for int32, 3 for float64)
	n			number of the values of the attribute

!Output Parameters:

	d			pointer to the returned attribute values

!Revision History:

  $Log: get_meta_attr() $
 * Revision 2.00	 1997/11/20  21:55     Z. Wan
 *

!Team-unique Header:
   Principal Investigator, Zhengming Wan
   	wan@icess.ucsb.edu
   	805-893-4541

!References and Credits:

!Design Notes:

!END***************************************************************************
*/

#define OK	0

int32 get_meta_attr (char *s, char *p, int32 type, int32 n, char *d)
{
	char	*s1, *s2, *s3;
	char	pt[64];
	char	*s_o;
	int32	class, i_class;
	int32	i_c_match;
	int32	i, j, k;

	int32	*i_o;
	double	*d_o;

	k = strlen (p);

	i_class = 0;

	for (i = 0; i < k; i++) {
		if (p[i] == '.') {
			sscanf (&p[i+1], "%d", &i_class);
			if ( i_class == 0 )	return ( FAIL );
			break;
		}
		pt[i] = p[i];
	}
	pt[i] = '\0';

	if ( (s1 = strstr (s, pt)) == NULL)
		return ( FAIL );

	i_c_match = 1;

	if ( i_class ) {
		i_c_match = 0;
		for ( i = 0; i < i_class; i++) {
			s2 = strstr (s1, "CLASS");
			s3 = strstr (s2, "= ");
			sscanf (s3+3, "%d", &class);
			if ( i_class == class ) {
				i_c_match = 1;
				s1 = s3;
				break;
			}
			else {
				s1 = s3+3;
				s2 = strstr (s1, "END_OBJECT");
				s3 = strstr (s2, pt);
				s1 = s3 + 1;
				if ( (s2 = strstr (s1, pt)) == NULL)
					return ( FAIL );
				s1 = s2;
			}
		}
	}

	if ( ! i_c_match ) return ( FAIL );

	s2 = strstr (s1, "VALUE");
	s3 = strstr (s2, "= ");

   if ( type == 1 ) {

	s_o = (char *) &d[0];
	s3 += 3;
	for ( i = 0; i < n; i++ ) {
		for ( j = 0; ; j++) {
			if (s3[j] == '"') {
				s_o[j] = '\0';
				break;
			}
			s_o[j] = s3[j];
		}
	}
   }
   else if ( type == 2 ) {
	  
	  i_o = (int32 *) &d[0];
	  s3++;

	  if ( n == 1 ) {
		sscanf (s3, "%d", &i_o[0]);
	  }
	  else {
	    for ( i = 0; i < n; i++ ) {

		if ( i )
			s2 = strstr (s3, ",");
		else
			s2 = strstr (s3, "(");
		s3 = s2;

		sscanf (s3+1, "%d", &i_o[i]);
		s3++;
	    }
	  }
	}
	else if ( type == 3 ) {

		d_o = (double *) &d[0];
		s3++;

		if ( n == 1 ) {
			sscanf (s3, "%le", &d_o[0]);
		}
		else {
		   for ( i = 0; i < n; i++ ) {
			if ( i )
				s2 = strstr (s3, ",");
			else
				s2 = strstr (s3, "(");
			s3 = s2;

			sscanf (s3+1, "%le", &d_o[i]);
			s3++;
		   }
		}
	     }
	     else	return ( FAIL );

	return ( OK );
}

void
SDsetattr_LST_V (int32 sd_id, char *name, int32 nt, int32 count, float32 *data)
/*
!C***************************************************************************

!Description:
   This function sets SDS attribute

!Input Parameters:	sd_id 	file or SDS id number
			name 	pointer to attribute name
			nt 	data type
			count 	number of attributes to be set
			data 	attributes to be set
		     
!Output Parameters:	none


!Revision History:

   * Revised  05/20/1999	17:50	 Z. Wan
   * Revision V2.01 1997/11/24	16:14:06 Z. Wan
   * make vrange & fillv to be compatible with field data type.
   *
   * Revision B3.2 09-11-95

!References and Credits:

   Version V1.1 09-11-96

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{

  PGSt_SMF_status ret;
  uint8		u8_vrange[2], u8_fillv;
  uint16	u16_vrange[2], u16_fillv;
  int32		i32_vrange[2], i32_fillv;
  float32	f32_vrange[2], f32_fillv;

  if ( ! strcmp(name, "valid_range") ) {

    switch ( nt ) {

    case DFNT_UINT8: u8_vrange[0] = (uint8) data[0];
    		     u8_vrange[1] = (uint8) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) u8_vrange);
		     break;

    case DFNT_UINT16: u16_vrange[0] = (uint16) data[0];
   		     u16_vrange[1] = (uint16) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) u16_vrange);
		     break;

    case DFNT_INT32: i32_vrange[0] = (int32) data[0];
		     i32_vrange[1] = (int32) data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) i32_vrange);
		     break;

    case DFNT_FLOAT32: f32_vrange[0] = data[0]; f32_vrange[1] = data[1];
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) f32_vrange);
		     break;
    }
  }
  else if ( ! strcmp(name, "_FillValue") ) {

    switch ( nt ) {

    case DFNT_UINT8: u8_fillv = (uint8) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &u8_fillv);
		     break;

    case DFNT_UINT16: u16_fillv = (uint16) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &u16_fillv);
		     break;

    case DFNT_INT32: i32_fillv = (int32) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &i32_fillv);
		     break;

    case DFNT_FLOAT32: f32_fillv = (float32) *data;
		     ret = SDsetattr(sd_id, name, nt,count, (VOIDP) &f32_fillv);
		     break;
    }
  }

  if ( ret == FAIL)
	HALTsmf(MODIS_E_ATTR_FAIL, name);
}

void sec_to_hrs (float64 *s, int32 *h, int32 *m)
/*
!C***************************************************************************

!Description:
   convert total seconds to hours, minutes, and seconds

!Input Parameters:	s 	pointer to variable total seconds
			h 	pointer to variable hours
			m 	pointer to variable minutes
		     
!Output Parameters:	s, h, m		the same pointers


!Revision History:

   * Version V2.02 1999/03/01   K.S Tai
   * Vision V2.01 1997/12/15	14:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-15-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	*h = *s / 3600;
	*s -= *h * 3600;
	*m = *s / 60;
	*s -= *m * 60;
        if ( *h >= 24. ) *h -=24.; /* get proper date/time range, KSTai, 3/1/99 */
}

uint8 u8getbits (uint8 x, int32 p, int32 n)
/*
!C***************************************************************************

!Description:
    This function gets the (right adjusted) n-bit field of x that begins at
    position p.  The bit position 0 is at the right end.

!Input Parameters:	x	uint8
			p	the beginning bit posision
			n	number of bits to be checked
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	uint8	x1;

	x1 = x >> (p+1-n) & ~(~0 << n);

	return (x1);
}

void u8setbits (uint8 *x, int32 p, int32 n, uint8 y)
/*
!C***************************************************************************

!Description:
    This function sets value y to the (right adjusted) n-bit field of *x that
    begins at position p.  The bit position 0 is at the right end.

!Input Parameters:	x	pointer to uint8
			p	the beginning bit posision
			n	number of bits to be set
			y	the value to be set
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	int32	pn;
	uint8	y1, z;

	pn = p + 1 - n;
	y1 = (y & ~(~0 << n)) << pn;
	z = ~((~0 >> pn) & ~(~0 << n) << pn);
	*x &= z;
	*x |= y1;
}

uint16 u16getbits (uint16 x, int32 p, int32 n)
/*
!C***************************************************************************

!Description:
    This function gets the (right adjusted) n-bit field of x that begins at
    position p.  The bit position 0 is at the right end.

!Input Parameters:	x	uint16
			p	the beginning bit posision
			n	number of bits to be checked
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	uint16	x1;

	x1 = x >> (p+1-n) & ~(~0 << n);

	return (x1);
}

void u16setbits (uint16 *x, int32 p, int32 n, uint16 y)
/*
!C***************************************************************************

!Description:
    This function sets value y to the (right adjusted) n-bit field of *x that
    begins at position p.  The bit position 0 is at the right end.

!Input Parameters:	x	pointer to uint16
			p	the beginning bit posision
			n	number of bits to be set
			y	the value to be set
!Output Parameters:
			the value in the n-bit field

!Revision History:

   * Vision V2.01 1997/12/23	10:12:05 Z.Wan
   *

!References and Credits:

   Version V2.01 12-23-97

   Written by:

	Zhengming Wan
	UCSB/ICESS
	wan@icess.ucsb.edu

!Team-unique Header:
	This software is developed by MODIS Science Team member
	Zhengming Wan for the National Aeronautics and Space Administration,
	Goddard Space Flight Center, under contract NAS5-31370.

!Design Notes

!END***********************************************************************/
{
	int32	pn;
	uint16	y1, z;

	pn = p + 1 - n;
	y1 = (y & ~(~0 << n)) << pn;
	z = ~((~0 >> pn) & ~(~0 << n) << pn);
	*x &= z;
	*x |= y1;
}

double split_window_lst ( int32 id_num, double *ac )
{
	double	tair_a, cwv_a, e31, e32, bt31, bt32, mu_view;

	int32	i_iterate, n_iterate = 3;
	int32	id_coef_tbl, id_tair, id_cwv, id_ts_tair;
	int32	j_pix, k;
	float64 tmp, dt, lst_ret;
	float64	emis_m, emis_d;
	float64 temp_m, temp_d;
	float64	coeff_p[8];
	float64 d1, d2, d3, d4;
	float64	lst_ret_dt_d;
	float64  mu_mesh[9] = { 0.415059, 0.445869, 0.475084, 0.529560,
			0.626080, 0.713005, 0.781367, 0.966438, 0.998631 };

	_GSW_LST_COEF_TBL	*gsw_lst_coef_p;

	if ( id_num == 14 ) {	/* daytime */
		tair_a = ac[5] * t_ref_d;
		cwv_a = ac[3] * wv_ref;
		e31 = ac[NC+4];
		e32 = ac[NC+5];
		bt31 = tbd[4];
		bt32 = tbd[5];
		mu_view = vzd_pix_in;
	}
	else {			/* nighttime */
		tair_a = ac[6] * t_ref_n;
		cwv_a = ac[4] * wv_ref;
		e31 = ac[NC+4];
		e32 = ac[NC+5];
		bt31 = tbn[4];
		bt32 = tbn[5];
		mu_view = vzn_pix_in;
	}

	if ( tair_a > _tair_tbl[6] )	tair_a = _tair_tbl[6];
	if ( cwv_a > _wv_tbl[10] )	cwv_a = _wv_tbl[10];

	id_tair = 0;

	for (k = 0; k < 7; k++) {

	  if (tair_a > _tair_tbl[k]) {
		id_tair = k;

		if (k <= 5) {
			tmp = tair_a - _tair_tbl[k];
			dt = _tair_tbl[k+1] - tair_a;
			if (dt < tmp) id_tair++;
		}

	  }
	}
	
	id_cwv = 0;

	for (k = 0; k < 11; k++) {

	  if (cwv_a > _wv_tbl[k]) {

	    id_cwv = k;
	  
	    if (k <= 9) {
	      tmp = cwv_a - _wv_tbl[k];
	      dt = _wv_tbl[k+1] - cwv_a;
	    
	      if (dt < tmp)	
		id_cwv++;
	    }
	  }
	}

	id_coef_tbl = pointer_coef_tbl[id_tair][id_cwv];
	
	if (id_coef_tbl < 0)
	  id_coef_tbl = 2;
	
	/* the main loop for the generalized split-window LST algorithm */
	
	for (i_iterate = 0; i_iterate < n_iterate; i_iterate++) {
	  
	  if (i_iterate == 0)	
	    id_ts_tair = 0;
	  else if ( i_iterate == 2 && lst_ret < bt31 )
			break;
	  
	  if (i_iterate == 1) {
	    tmp = lst_ret - tair_a;
	    if (tmp > 0.)	
	      id_ts_tair = 1;
	    else
	      id_ts_tair = 2;
	  }

	  if (i_iterate == 2) {
	    tmp = lst_ret - tair_a;

	    id_ts_tair = 6;
	    if (tmp > -7.) id_ts_tair = 5;
	    if (tmp > 0.)  id_ts_tair = 4;
            if (tmp > 7.)  id_ts_tair = 3;	

	  }

	  for (k = 0; k < 9; k++) {
	    gsw_lst_coef_p = &gsw_lst_coef_tbl[id_coef_tbl][k][id_ts_tair];
	
	    coef_mesh[0][k] = gsw_lst_coef_p->c;
	    coef_mesh[1][k] = gsw_lst_coef_p->a1;
	    coef_mesh[2][k] = gsw_lst_coef_p->a2;
	    coef_mesh[3][k] = gsw_lst_coef_p->a3;
	    coef_mesh[4][k] = gsw_lst_coef_p->b1;
	    coef_mesh[5][k] = gsw_lst_coef_p->b2;
	    coef_mesh[6][k] = gsw_lst_coef_p->b3;
	    coef_mesh[7][k] = gsw_lst_coef_p->rms_algorithm;
	  }

	  if (i_iterate == 0) {

	    emis_m = 0.5 * (e31 + e32);
	    emis_d = (e31 - e32) / (emis_m * emis_m);
	    tmp = (1. - emis_m) / emis_m;
	    emis_m = tmp;

	    j_pix = 0;

	    if (mu_view >= mu_mesh[0]) j_pix = 0;
	    if (mu_view >= mu_mesh[1]) j_pix = 1;
	    if (mu_view >= mu_mesh[2]) j_pix = 2;
	    if (mu_view >= mu_mesh[3]) j_pix = 3;
	    if (mu_view >= mu_mesh[4]) j_pix = 4;
	    if (mu_view >= mu_mesh[5]) j_pix = 5;
	    if (mu_view >= mu_mesh[6]) j_pix = 6;
	  }
	  
	  poly_intp_vector (mu_mesh,j_pix, mu_view, 8, coef_mesh, &coeff_p[0]);

	  temp_m = 0.5 * (bt31 + bt32);
	  temp_d = 0.5 * (bt31 - bt32);
	  d1 = coeff_p[2] * temp_m;
	  d2 = coeff_p[3] * temp_m;
	  d3 = coeff_p[5] * temp_d;
	  d4 = coeff_p[6] * temp_d;
	  lst_ret = coeff_p[1] * temp_m + d1 * emis_m +
	            d2 * emis_d + coeff_p[4] * temp_d +
		    d3 * emis_m + d4 * emis_d + coeff_p[0];
	}

	return (lst_ret);
}
/* a short program to setup __ctype_b, __ctype_toupper and __ctype_tolower
   written by Scott Burns, Mirrabooka Systems.
   It is downloaded from http://www.dbforums.com/t964847.html
   by
   Zhengming Wan, ICESS, University of California, Santa Barbara, CA 93106
   It is used only #ifdef _LINUX_ct

*/

#ifdef _LINUX_ct

#include <ctype.h>

__const unsigned short int *__ctype_b;
__const __int32_t *__ctype_tolower;
__const __int32_t *__ctype_toupper;

void ctSetup()
{
	__ctype_b = *(__ctype_b_loc());
	__ctype_toupper = *(__ctype_toupper_loc());
	__ctype_tolower = *(__ctype_tolower_loc());
}

#endif
